#!/usr/bin/env python
########
# Get Shannon radii, calculate goldschmidt tolerance factor
# 2015-01-30 Tam Mayeshiba
# 2015-11-04 TTM use correct coordination numbers
# 2015-12-18 TTM separate from radii and pv_combo, and separate from list making
########
import numpy as np
from descriptors.radii import shannon_entry

def calculate_goldschmidt_pure(arad="",brad="",orad=""):
    """Calculate Goldschmidt tolerance factor
    """
    arad=float(arad)
    brad=float(brad)
    orad=float(orad)
    gold=(arad+orad)/(np.sqrt(2)*(brad+orad))
    return gold

def calculate_goldschmidt_from_pv_combo(my_pv_combo, verbose=1):
    """Calculate Goldschmidt tolerance factor from a pv_combo object
    """
    a1rad = my_pv_combo.aentry.crystalrad
    b1rad = my_pv_combo.bentry.crystalrad
    a2rad=0.0
    b2rad=0.0
    if not (my_pv_combo.aprimeentry == ""):
        a2rad = my_pv_combo.aprimeentry.crystalrad
        a1frac = my_pv_combo.afrac
        a2frac = my_pv_combo.aprimefrac
        arad = (a1frac * a1rad) + (a2frac * a2rad)
    else:
        arad = a1rad
    if not (my_pv_combo.bprimeentry == ""):
        b2rad = my_pv_combo.bprimeentry.crystalrad
        b1frac = my_pv_combo.bfrac
        b2frac = my_pv_combo.bprimefrac
        brad = (b1frac * b1rad) + (b2frac * b2rad)
    else:
        brad = b1rad
    crad = my_pv_combo.centry.crystalrad
    if (verbose == 1):
        print "a1rad, aprimerad, arad: %3.3f, %3.3f, %3.3f" % (a1rad, a2rad, arad)
        print "b1rad, bprimerad, brad: %3.3f, %3.3f, %3.3f" % (b1rad, b2rad, brad)
        print "crad, %3.3f" % crad
    return calculate_goldschmidt_pure(arad, brad, crad)

def get_goldschmidt_category(gold):
    """Get category based on goldschmidt tolerance factor
        Args:
            gold <float>: Goldschmidt tolerance factor
        based on http://www.ccp14.ac.uk/ccp/web-mirrors/pki/uni/pki/members/schinzer/stru_chem/perov/di_gold.html
    """
    rgold = round(gold, 2)
    if rgold < 0.71:
        return "not perovskite"
    elif rgold < 0.90:
        return "orthorhombic or rhombohedral perovskite"
    elif rgold <= 1.00:
        return "cubic perovskite"
    else:
        return "hexagonal perovskite"
    return "evaluation error"
    
