#!/usr/bin/env python
# Tam Mayeshiba 2015-11-06
# cleanup from strain data
# S12b bulk modulus
######

import os
import subprocess
import Vgetvolume
import hocking_lstsq
topdir="//home/tam/strain_emig_clean/S12_elastic_model/bulkset"
strdir="//home/tam/strain_emig_clean/S3_lattice_fitting"
import sys
skiplist=list()
fitdict=dict()
#blist=list(sys.argv[1])
blist=['c','f','g','m','n','r','s','t','v']

# Get pressure-versus-volumes and bulk moduli
for bsite in blist:
    fitdict[bsite] = dict()
    fitdict[bsite]['vol'] = list()
    fitdict[bsite]['press'] = list()
    bdir = "%s/%sbm" % (topdir, bsite)
    dir1s = os.listdir(bdir)
    vlist=list()
    plist=list()
    for dir1 in dir1s:
        dir1path = os.path.join(bdir, dir1)
        if os.path.isdir(dir1path):
            myvol = Vgetvolume.main(os.path.join(dir1path, "perfect_stat"))
            greppress = subprocess.Popen("grep pressure %s/perfect_stat/OUTCAR" % dir1path, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            pressstring = greppress.communicate()[0]
            mypress = pressstring.split()[3]
            fitdict[bsite]['press'].append(float(mypress))
            fitdict[bsite]['vol'].append(float(myvol))
import elastic_constants
import simpleplot
import numpy as np
import time
bmgpalist=list()
bmelist=list()
dbmlist=list()
for bsite in blist:
    print "BSITE: ", bsite
    plist = fitdict[bsite]['press']
    pvec = np.array(plist, 'float')
    vlist = fitdict[bsite]['vol']
    vvec = np.array(vlist, 'float')
    [bret,bmat,bssq,berr,bulkmod,bulkmoderr,derivbm] = elastic_constants.bulkmod(pvec,vvec,"B-site %s" % bsite)
    fitdict[bsite]['bulkmod'] = bulkmod
    fitdict[bsite]['bulkmoderr'] = bulkmoderr
    fitdict[bsite]['derivbm'] = derivbm

# Get strains used and Poisson's ratios
strainfile=open(os.path.join(strdir, "S3_strains_used_for_poisson_fit"), 'rb')
#For this file, only a single point at each strain value is used.
#The actual document has multiple lines because some grids were done over,
# so the extra lines were skipped using "skip" at the end.
strainlines = strainfile.readlines()
for strainline in strainlines:
    strainsplit = strainline.split()
    dirpc = strainsplit[0]
    apc = strainsplit[1]
    cpc = strainsplit[3]
    dirsplit = dirpc.split("/")
    bsite = dirsplit[6][0]
    if not bsite in fitdict.keys():
        fitdict[bsite] = dict()
    if not 'amult' in fitdict[bsite].keys():
        fitdict[bsite]['amult'] = list()
        fitdict[bsite]['cmult'] = list()
    if strainsplit[-1] == "skip":
        pass
    else:
        fitdict[bsite]['amult'].append(apc)
        fitdict[bsite]['cmult'].append(cpc)
for bsite in blist:
    [poisson, poissonerr] = elastic_constants.poisson_from_biaxial(fitdict[bsite]['amult'], fitdict[bsite]['cmult'], "B-site %s" % bsite)
    fitdict[bsite]['poisson'] = poisson
    fitdict[bsite]['poissonerr'] = poissonerr
    [youngsmod, youngsmoderr] = elastic_constants.youngsmod(fitdict[bsite]['bulkmod'], fitdict[bsite]['bulkmoderr'],fitdict[bsite]['poisson'],fitdict[bsite]['poissonerr'])
    fitdict[bsite]['youngsmod'] = youngsmod
    fitdict[bsite]['youngsmoderr'] = youngsmoderr
    print "Young's modulus, kbar, eV/A^3", youngsmod, youngsmod/10.0/160.2
    print "Error in E, kbar, eV/A^3", youngsmoderr, youngsmoderr/10.0/160.2
    [prefactor, prefactorerr] = elastic_constants.strain_model_prefactor(fitdict[bsite]['poisson'],fitdict[bsite]['poissonerr'],fitdict[bsite]['youngsmod'],fitdict[bsite]['youngsmoderr'])
    fitdict[bsite]['prefactor']=prefactor
    fitdict[bsite]['prefactorerr']=prefactorerr
#Print output
print "B-site, Poisson's ratio, error, Bulk modulus (GPa), error, B0', Young's modulus (GPa), error, Prefactor (eV/A^3), error"
for bsite in blist:
    print "%s,%3.3f, %3.3f, %3.3f,%3.3f, %3.3f, %3.3f, %3.3f, %3.3f, %3.3f" % (bsite,
            fitdict[bsite]['poisson'],
            fitdict[bsite]['poissonerr'],
            fitdict[bsite]['bulkmod']/10.0,
            fitdict[bsite]['bulkmoderr']/10.0,
            fitdict[bsite]['derivbm']/10.0,
            fitdict[bsite]['youngsmod']/10.0,
            fitdict[bsite]['youngsmoderr']/10.0,
            fitdict[bsite]['prefactor'],
            fitdict[bsite]['prefactorerr'])
