#!/usr/bin/env python
# Tam Mayeshiba 2015-12-02
# cleanup from strain data
# S12b bulk modulus for endpoints and nebs
# (defected bulk moduli, Table 12.4)
######

import os
import subprocess
import Vgetvolume
import hocking_lstsq
topdir="//home/tam/strain_emig_clean/S12_elastic_model"
strdir="//home/tam/strain_emig_clean/S3_lattice_fitting"
import sys
import elastic_constants
import simpleplot
import numpy as np
import time
skiplist=list()
fitdict=dict()
#blist=list(sys.argv[1])
blist=['m','r']
subfolderlist=['bulkset',
    'epneb_bulkmod/endpoint_comp',
    'epneb_bulkmod/neb_comp',
    'epneb_bulkmod/ip_neb_comp',
    'epneb_bulkmod/endpoint_uncomp',
    'epneb_bulkmod/neb_uncomp']
fulldict=dict()

# Get pressure-versus-volumes and bulk moduli
for subfolder in subfolderlist:
    fullpath = topdir + "/" + subfolder
    fulldict[subfolder]=dict()
    for bsite in blist:
        fitdict[bsite] = dict()
        fitdict[bsite]['vol'] = list()
        fitdict[bsite]['press'] = list()
        bdir = "%s/%s/%sbm" % (topdir, subfolder, bsite)
        dir1s = os.listdir(bdir)
        if (len(dir1s) == 0):
            continue
        vlist=list()
        plist=list()
        for dir1 in dir1s:
            dir1path = os.path.join(bdir, dir1)
            if os.path.isdir(dir1path):
                myvol = Vgetvolume.main(os.path.join(dir1path, "perfect_stat"))
                greppress = subprocess.Popen("grep pressure %s/perfect_stat/OUTCAR" % dir1path, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                pressstring = greppress.communicate()[0]
                mypress = pressstring.split()[3]
                fitdict[bsite]['press'].append(float(mypress))
                fitdict[bsite]['vol'].append(float(myvol))
        print "BSITE: ", bsite
        plist = fitdict[bsite]['press']
        pvec = np.array(plist, 'float')
        vlist = fitdict[bsite]['vol']
        vvec = np.array(vlist, 'float')
        [bret,bmat,bssq,berr,bulkmod,bulkmoderr,derivbm] = elastic_constants.bulkmod(pvec,vvec,"B-site %s" % bsite)
        fitdict[bsite]['bulkmod'] = bulkmod
        fitdict[bsite]['bulkmoderr'] = bulkmoderr
        fitdict[bsite]['derivbm'] = derivbm
        fulldict[subfolder][bsite]=dict(fitdict[bsite])

# Get strains used and Poisson's ratios
strainfile=open(os.path.join(strdir, "S3_strains_used_for_poisson_fit"), 'rb')
#For this file, only a single point at each strain value is used.
#The actual document has multiple lines because some grids were done over,
# so the extra lines were skipped using "skip" at the end.
strainlines = strainfile.readlines()
for strainline in strainlines:
    strainsplit = strainline.split()
    dirpc = strainsplit[0]
    apc = strainsplit[1]
    cpc = strainsplit[3]
    dirsplit = dirpc.split("/")
    bsite = dirsplit[6][0]
    if not bsite in fitdict.keys():
        fitdict[bsite] = dict()
    if not 'amult' in fitdict[bsite].keys():
        fitdict[bsite]['amult'] = list()
        fitdict[bsite]['cmult'] = list()
    if strainsplit[-1] == "skip":
        pass
    else:
        fitdict[bsite]['amult'].append(apc)
        fitdict[bsite]['cmult'].append(cpc)
for subfolder in subfolderlist:
    for bsite in blist:
        if not (bsite in fulldict[subfolder].keys()):
            continue
        [poisson, poissonerr] = elastic_constants.poisson_from_biaxial(fitdict[bsite]['amult'], fitdict[bsite]['cmult'], "B-site %s" % bsite)
        fitdict[bsite]['poisson'] = poisson
        fitdict[bsite]['poissonerr'] = poissonerr
        [youngsmod, youngsmoderr] = elastic_constants.youngsmod(fulldict[subfolder][bsite]['bulkmod'], fulldict[subfolder][bsite]['bulkmoderr'],fitdict[bsite]['poisson'],fitdict[bsite]['poissonerr'])
        fulldict[subfolder][bsite]['youngsmod'] = youngsmod
        fulldict[subfolder][bsite]['youngsmoderr'] = youngsmoderr
        print "Young's modulus, kbar, eV/A^3", youngsmod, youngsmod/10.0/160.2
        print "Error in E, kbar, eV/A^3", youngsmoderr, youngsmoderr/10.0/160.2
        [prefactor, prefactorerr] = elastic_constants.strain_model_prefactor(fitdict[bsite]['poisson'],fitdict[bsite]['poissonerr'],fulldict[subfolder][bsite]['youngsmod'],fulldict[subfolder][bsite]['youngsmoderr'])
        fulldict[subfolder][bsite]['prefactor']=prefactor
        fulldict[subfolder][bsite]['prefactorerr']=prefactorerr
#Print output
print fulldict
print "B-site, Poisson's ratio, error, Bulk modulus (GPa), error, B0', Young's modulus (GPa), error, Prefactor (eV/A^3), error, subfolder"
for bsite in blist:
    for subfolder in subfolderlist:
        if not (bsite in fulldict[subfolder].keys()):
            continue
        print "%s,%3.3f, %3.3f, %3.3f,%3.3f, %3.3f, %3.3f, %3.3f, %3.3f, %3.3f, %s" % (bsite,
            fitdict[bsite]['poisson'],
            fitdict[bsite]['poissonerr'],
            fulldict[subfolder][bsite]['bulkmod']/10.0,
            fulldict[subfolder][bsite]['bulkmoderr']/10.0,
            fulldict[subfolder][bsite]['derivbm']/10.0,
            fulldict[subfolder][bsite]['youngsmod']/10.0,
            fulldict[subfolder][bsite]['youngsmoderr']/10.0,
            fulldict[subfolder][bsite]['prefactor'],
            fulldict[subfolder][bsite]['prefactorerr'],
            subfolder)


#Print output
print "B-site, Bulk modulus (eV/A^3), Young's modulus (eV/A^3), Prefactor (eV/A^3), subfolder"
for bsite in blist:
    for subfolder in subfolderlist:
        if not (bsite in fulldict[subfolder].keys()):
            continue
        print "%s,%3.3f, %3.3f, %3.3f, %s" % (bsite,
            fulldict[subfolder][bsite]['bulkmod']/10.0/160.2,
            fulldict[subfolder][bsite]['youngsmod']/10.0/160.2,
            fulldict[subfolder][bsite]['prefactor'],
            subfolder)
