#!/usr/bin/env python
# Tam Mayeshiba 2015-11-09
# cleanup from strain data
# S12d migration volumes from Birch-Murnaghan fit
#   Note that the strain paper Vmig had an error in the original fit,
#   where the numpy powers were given as 7/3 instead of 7./3. etc.
#   and therefore evaluated as (V0/V)^2 instead of (V0/V)^(7/3) and so on.
######

import os
import simplegrep
import subprocess
import Vgetvolume
import hocking_lstsq
import numpy as np
topdir="//home/tam/strain_emig_clean/S4_migration_barriers"
import sys
skiplist=list()
fitdict=dict()
#blist=list(sys.argv[1])
blist=['c','f','g','m','n','r','s','t','v']

#Get zero-strain fixed volume, defected initial pressure,
# out-of-plane migration image pressure, and
# in-plane migration image pressure
for bsite in blist:
    #print "BSITE: ", bsite
    fitdict[bsite] = dict()
    bdir = "%s/%sgrid" % (topdir, bsite)
    subdirs = os.listdir(bdir)
    nostrainsubdirs=list()
    for subdir in subdirs:
        if subdir[0:4] == "1000":
            nostrainsubdirs.append(subdir)
    for nostrainsubdir in nostrainsubdirs:
        subpath = os.path.join(bdir, nostrainsubdir)
        for hdir in ['ip','oop']:
            nebpath = os.path.join(subpath, "neb_vac1-%spv_q=p2_stat" % hdir[0])
            if not ("SKIP_FOR_PROCESSING" in os.listdir(nebpath)):
                fitdict[bsite][hdir]=dict()
                print "Using %s" % nebpath
                defpath = os.path.join(subpath, "defect_vac1_q=p2_stat")
                fixedvol = simplegrep.get_vasp_volume_of_cell("%s/OUTCAR" % defpath)
                press_init = simplegrep.get_vasp_pressure("%s/OUTCAR" % defpath)
                press_im = simplegrep.get_vasp_pressure("%s/02/OUTCAR" % nebpath)
                volrelax_init_vol = simplegrep.get_vasp_volume_of_cell("%s/volrelax_vac1_volonly/OUTCAR" % subpath)
                volrelax_im_vol = simplegrep.get_vasp_volume_of_cell("%s/volrelax_%spv02_volonly/OUTCAR" % (subpath, hdir[0]))
                fitdict[bsite][hdir]["fixedvol"] = float(fixedvol)
                fitdict[bsite][hdir]["press_init"] = float(press_init)
                fitdict[bsite][hdir]["press_im"] = float(press_im)
                fitdict[bsite][hdir]["volrelax_init_vol"] = float(volrelax_init_vol)
                fitdict[bsite][hdir]["volrelax_im_vol"] = float(volrelax_im_vol)
                fitdict[bsite][hdir]["dVmig_volrelax"] = fitdict[bsite][hdir]["volrelax_im_vol"] - fitdict[bsite][hdir]["volrelax_init_vol"]
#Get bulk moduli previously calculated
bulkmodtailproc = subprocess.Popen("tail //home/tam/strain_emig_clean/S12_elastic_model/S12_elastic_model_output", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
bulkmodtail = bulkmodtailproc.communicate()[0]
bulkmodlines = bulkmodtail.split("\n")
for bulkmodline in bulkmodlines:
    bulkmodsplit = bulkmodline.split(",")
    bsite = bulkmodsplit[0]
    if len(bsite) == 1:
        if not bsite in fitdict.keys():
            fitdict[bsite] = dict()
        bulkmod = float(bulkmodsplit[3])*10.0 #in KBAR
        derivbm = float(bulkmodsplit[5])*10.0 #in KBAR
        fitdict[bsite]['bulkmod'] = bulkmod
        fitdict[bsite]['derivbm'] = derivbm

import elastic_constants
for bsite in blist:
    print "BSITE: ", bsite
    bulkmod = fitdict[bsite]['bulkmod'] # in kbar
    derivbm = fitdict[bsite]['derivbm'] # in kbar
    print "Bulk mod (kbar)", bulkmod
    print "B0'", derivbm
    for hdir in ['ip','oop']:
        fitdict[bsite][hdir]['v_init_BM'] = elastic_constants.get_volume_from_birch_murnaghan(bulkmod, derivbm,
            fitdict[bsite][hdir]['press_init'],
            fitdict[bsite][hdir]['fixedvol'])
        fitdict[bsite][hdir]['v_im_BM'] = elastic_constants.get_volume_from_birch_murnaghan(bulkmod, derivbm,
            fitdict[bsite][hdir]['press_im'],
            fitdict[bsite][hdir]['fixedvol'])
        fitdict[bsite][hdir]['dVmig_BM'] = fitdict[bsite][hdir]['v_im_BM'] - fitdict[bsite][hdir]['v_init_BM']
for bsite in blist:
    for hdir in ['ip','oop']:
        print "BSITE: %s %s" % (bsite, hdir)
        print " %20s: %3.2f" % ("fixed vol", fitdict[bsite][hdir]["fixedvol"])
        print " %20s: %3.2f" % ("init press", fitdict[bsite][hdir]["press_init"])
        print " %20s: %3.2f" % ("im press", fitdict[bsite][hdir]["press_im"])
        print " %20s: %3.1f" % ("v_init_BM", fitdict[bsite][hdir]["v_init_BM"])
        print " %20s: %3.1f" % ("v_im_BM", fitdict[bsite][hdir]["v_im_BM"])
        print " %20s: %3.2f" % ("volrelax_init", fitdict[bsite][hdir]["volrelax_init_vol"])
        print " %20s: %3.2f" % ("volrelax_im", fitdict[bsite][hdir]["volrelax_im_vol"])

print "B-site, BM dVmig IP, BM dVmig OOP, volrelax dVmig IP, volrelax dVmig OOP"

for bsite in blist:
    print "%s,%3.1f,%3.1f,%3.2f,%3.2f" % (bsite,
        fitdict[bsite]['ip']['dVmig_BM'],
        fitdict[bsite]['oop']['dVmig_BM'],
        fitdict[bsite]['ip']['dVmig_volrelax'],
        fitdict[bsite]['oop']['dVmig_volrelax'])
