#!/usr/bin/env python
# Tam Mayeshiba 2015-11-09, mod 2015-12-10
# cleanup from strain data
# S4 migration barriers, DMEPS
# Use lowest barrier calculation if there are multiple
#    calculations for each point
######

import os
import subprocess
import numpy as np
import sys
import nebenergy
import time
topdir="//home/tam/strain_emig_clean/S4_migration_barriers"

fitdict=dict()
#
#blist=list(sys.argv[1])
blist=['c','f','g','m','n','r','s','t','v']
strainlow=-2.0
strainhigh=2.0
#For ip and oop and each strain, get all neb reports
for bsite in blist:
    fitdict[bsite]=dict()
    fitdict[bsite]['ip'] = dict()
    fitdict[bsite]['oop'] = dict()
    trydirs=list()
    btopdir = os.path.join(topdir, "%sgrid" % bsite)
    bpaths = os.listdir(btopdir)
    for bpath in bpaths:
        trydir = os.path.join(btopdir, bpath)
        if os.path.isdir(trydir):
            trydirs.append(trydir)
    for trydir in trydirs:
        print "Trying %s" % trydir
        try:
            straininit = float(os.path.basename(trydir).split("_")[0])
        except (ValueError):
            print "Error in directory %s" % trydir
            sys.exit()
        if straininit > 2000: #extra digits
            straininit = straininit/10.0
        straininit = straininit - 1000 #normalize to difference
        strain = (straininit / 10.0) # get percent strain
        if (strain < strainlow) or (strain > strainhigh):
            print "Skipping due to strain not in range: %s" % trydir
            continue
        for hdir in ['ip','oop']:
            nebdir = os.path.join(trydir, "neb_vac1-%spv_q=p2_stat" % hdir[0]) 
            if "SKIP_FOR_PROCESSING" in os.listdir(nebdir):
                print "Skipping processing due to SKIP_FOR_PROCESSING FILE: %s" % nebdir
            else:
                print "Strain: ", strain
                myneb = nebenergy.report(nebdir)
                if not strain in fitdict[bsite][hdir].keys():
                    fitdict[bsite][hdir][strain]=dict()
                    fitdict[bsite][hdir][strain]['nebs']=list()
                    fitdict[bsite][hdir][strain]['labels']=list()
                fitdict[bsite][hdir][strain]['nebs'].append(myneb)
                fitdict[bsite][hdir][strain]['labels'].append(trydir)

#Get barrier for each strain by using lowest barrier
for bsite in blist:
    for hdir in ['ip','oop']:
        strains = fitdict[bsite][hdir].keys()
        strains.sort()
        for strain in strains:
            print "%s %s LENGTH OF NEB LIST for strain %3.3f: %i" % (bsite, hdir, strain, len(fitdict[bsite][hdir][strain]['nebs']))
            [barrmin, barridx] = nebenergy.get_minimum_from_list(fitdict[bsite][hdir][strain]['nebs'], "barrfrominit")
            dirused = fitdict[bsite][hdir][strain]['labels'][barridx]
            fitdict[bsite][hdir][strain]["barrfrominit"] = barrmin
            fitdict[bsite][hdir][strain]["dirused"] = dirused
            print "Min barr: %3.3f" % barrmin
            print "Dir used: %s" % dirused
            for labelitem in fitdict[bsite][hdir][strain]['labels']:
                if not (labelitem == dirused):
                    pathitem = os.path.join(labelitem, "neb_vac1-%spv_q=p2_stat" % hdir[0])
                    print "Not used: %s " % pathitem
                    myfile = open(os.path.join(pathitem, "SKIP_FOR_PROCESSING"), 'ab')
                    myfile.write(time.asctime())
                    myfile.write("  Not lowest barrier %3.3f and should be skipped: %s\n" % (barrmin, pathitem))
                    myfile.close()
#Get DMEPS values
for bsite in blist:
    for hdir in ['ip','oop']:
        slist=list()
        barrlist=list()
        strains = fitdict[bsite][hdir].keys()
        strains.sort()
        print "BSITE: ", bsite
        for strain in strains:
            migbarr = fitdict[bsite][hdir][strain]["barrfrominit"]
            dirused = fitdict[bsite][hdir][strain]["dirused"]
            print "%3.3f, %3.3f, %s" % (strain, migbarr, dirused)
            slist.append(strain)
            barrlist.append(migbarr)
        fitdict[bsite][hdir]['strains'] = list(slist)
        fitdict[bsite][hdir]['barriers'] = list(barrlist)
        [dmeps, dmepserr] = nebenergy.get_dmeps(slist, barrlist, 1, bsite + "," + hdir)
        fitdict[bsite][hdir]['dmeps'] = dmeps
        fitdict[bsite][hdir]['dmepserr'] = dmepserr
#Print output tables
for hdir in ['ip','oop']:
    print "Direction", hdir
    allstrains=list()
    for bsite in blist:
        slist = fitdict[bsite][hdir]['strains']
        for strain in slist:
            if not strain in allstrains:
                allstrains.append(strain)
    allstrains.sort()
    dispstr = "% strain"
    for bsite in blist:
        dispstr = dispstr + "," + bsite
    print dispstr
    for thisstrain in allstrains: #sort of inefficient.
        strainstr = "%3.3f" % thisstrain
        for bsite in blist:
            strainstr = strainstr + ","
            slist = fitdict[bsite][hdir]['strains']
            barrlist = fitdict[bsite][hdir]['barriers']
            for sidx in range(0, len(slist)):
                mystrain = slist[sidx]
                if mystrain == thisstrain:
                    strainstr = strainstr + "%3.3f" % barrlist[sidx]
        print strainstr
print "Strain bounds: %3.3f to %3.3f" % (strainlow, strainhigh)
print "B-site, DMEPS IP, error, DMEPS OOP, error"
for bsite in blist:
    print "%s,%3.3f,%3.3f,%3.3f,%3.3f" % (bsite, fitdict[bsite]['ip']['dmeps'],
        fitdict[bsite]['ip']['dmepserr'],
        fitdict[bsite]['oop']['dmeps'],
        fitdict[bsite]['oop']['dmepserr'])

