#!/usr/bin/env python
# Tam Mayeshiba 2015-11-17
# cleanup from strain data
# S6 magnetic moment antiferromagnetic data for Table S6.2
######

import os
import subprocess
import numpy as np
import sys
import nebenergy
import Vgetenergy
topdir="//home/tam/strain_emig_clean/S6_magnetic_moment"
addldir1="//home/tam/strain_emig_clean/S5_GGAU_barrier_comparison/nonu"
addldir2="//home/tam/strain_emig_clean/S6_literature_barriers/nonu_uncompensated"

fitdict=dict()

#
#blist=list(sys.argv[1])
blist=['f','m','r','t','v']

for bsite in blist:
    fitdict[bsite]=dict()
    bulk_fm = float(Vgetenergy.main("%s/a%s/a%s/a%s_bulk_static" % (addldir2, bsite, bsite, bsite)))
    uncomp_neb = float(nebenergy.report("%s/a%s/a%s/a%s_neb_static" % (addldir2, bsite, bsite, bsite))["barrfrominit"])
    comp_neb = float(nebenergy.report("%s/a%sc/a%sc_neb_static" % (addldir1, bsite, bsite))["barrfrominit"])
    types = os.listdir("%s/%s_mag" % (topdir, bsite))
    fitdict[bsite]['_fm']=dict()
    fitdict[bsite]['_fm']['bulk']=bulk_fm
    fitdict[bsite]['_fm']['uncomp_neb']=uncomp_neb
    fitdict[bsite]['_fm']['comp_neb']=comp_neb
    for type in types:
        fitdict[bsite][type]=dict()
        mydir = "%s/%s_mag/%s" % (topdir, bsite, type)
        uncompdir = "%s/uncomp" % mydir
        subdirs = os.listdir(uncompdir)
        for subdir in subdirs:
            if "bulk_static" in subdir:
                bulk_afm = float(Vgetenergy.main("%s/%s" % (uncompdir, subdir)))
                fitdict[bsite][type]['bulk'] = bulk_afm
            if "neb_static" in subdir:
                uncomp_neb_afm = float(nebenergy.report("%s/%s" % (uncompdir, subdir))['barrfrominit'])
                fitdict[bsite][type]['uncomp_neb'] = uncomp_neb_afm
        compdir = "%s/comp" % mydir
        if os.path.isdir(compdir):
            subdirs = os.listdir(compdir)
            for subdir in subdirs:
                if "neb_static" in subdir:
                    comp_neb_afm = float(nebenergy.report("%s/%s" % (compdir, subdir))['barrfrominit'])
                    fitdict[bsite][type]['comp_neb'] = comp_neb_afm

print "B-site, magnetic arrangement, bulk energy, compensated Emig, uncompensated Emig"
difflist=list()
for bsite in blist:
    types = fitdict[bsite].keys()
    types.sort()
    for type in types:
        diffstr=""
        keys = fitdict[bsite][type].keys()
        if not (type == "_fm"):
            bulkdiff = fitdict[bsite][type]['bulk'] - fitdict[bsite]['_fm']['bulk']
            diffstr = "%s,%s,%3.3f" % (bsite, type, bulkdiff)
        if "comp_neb" in keys:
            comp_neb_str = "%3.3f" % fitdict[bsite][type]['comp_neb']
            if not (type == "_fm"):
                compdiff = fitdict[bsite][type]['comp_neb'] - fitdict[bsite]['_fm']['comp_neb']
                diffstr = diffstr + ",%3.3f" % compdiff
        else:
            comp_neb_str = ""
        if "uncomp_neb" in keys:
            uncomp_neb_str = "%3.3f" % fitdict[bsite][type]['uncomp_neb']
            if not (type == "_fm"):
                uncompdiff = fitdict[bsite][type]['uncomp_neb'] - fitdict[bsite]['_fm']['uncomp_neb']
                diffstr = diffstr + ",%3.3f" % uncompdiff
        else:
            uncomp_neb_str = ""
        print "%s,%s,%3.3f,%s,%s" % (bsite, type,fitdict[bsite][type]['bulk'],comp_neb_str, uncomp_neb_str)
        if not (diffstr == ""):
            difflist.append(diffstr)

print ""
print "B-site, magnetic arrangement, bulk difference from FM, compensated Emig difference from FM, uncompensated Emig difference from FM"
for diffitem in difflist:
    print diffitem

