#!/usr/bin/env python
# Tam Mayeshiba 2015-11-11
# cleanup from strain data
# S7 doped barriers, DMEPS
######

import os
import subprocess
import numpy as np
import sys
import nebenergy
topdir="//home/tam/strain_emig_clean/S7_doped_barriers/doped_barriers/%s" % sys.argv[1]
#call with in_line, face_diagonal, and cross_body_diagonal as argument

fitdict=dict()
#
#blist=list(sys.argv[1])
blist=['m','r','s']

#For ip and oop and each strain, get all neb reports
for bsite in blist:
    fitdict[bsite]=dict()
    fitdict[bsite]['ip'] = dict()
    fitdict[bsite]['oop'] = dict()
    trydirs=list()
    btopdir = os.path.join(topdir, "%sdoped" % bsite)
    bpaths = os.listdir(btopdir)
    for bpath in bpaths:
        trydir = os.path.join(btopdir, bpath)
        if os.path.isdir(trydir):
            if "skip" in trydir:
                print "Skipping %s" % os.listdir(trydir)
            else:
                trydirs.append(trydir)
    for trydir in trydirs:
        print "Using %s" % trydir
        try:
            straininit = float(os.path.basename(trydir).split("_")[1])
        except (ValueError):
            print "Error in directory %s" % trydir
            sys.exit()
        if straininit > 2000: #extra digits
            straininit = straininit/10.0
        straininit = straininit - 1000 #normalize to difference
        strain = (straininit / 10.0) # get percent strain
        ipnebdir = os.path.join(trydir, "neb_vac1-ipv_q=p0_stat") 
        oopnebdir = os.path.join(trydir, "neb_vac1-opv_q=p0_stat") 
        print "Strain: ", strain
        ipneb = nebenergy.report(ipnebdir)
        oopneb = nebenergy.report(oopnebdir)
        if not strain in fitdict[bsite]['ip'].keys():
            fitdict[bsite]['ip'][strain]=dict()
            fitdict[bsite]['ip'][strain]['nebs']=list()
            fitdict[bsite]['oop'][strain]=dict()
            fitdict[bsite]['oop'][strain]['nebs']=list()
        fitdict[bsite]['ip'][strain]['nebs'].append(ipneb)
        fitdict[bsite]['oop'][strain]['nebs'].append(oopneb)

#Get barrier for each strain by using lowest saddle at each strain and
# the lowest initial endpoint
for bsite in blist:
    for hdir in ['ip','oop']:
        strains = fitdict[bsite][hdir].keys()
        strains.sort()
        for strain in strains:
            print "LENGTH OF NEB LIST:" , len(fitdict[bsite][hdir][strain]['nebs'])
            emax_for_strain = nebenergy.get_minimum_saddle(fitdict[bsite][hdir][strain]['nebs'])
            einit_for_strain = nebenergy.get_minimum_initial_endpoint(fitdict[bsite][hdir][strain]['nebs'])
            fitdict[bsite][hdir][strain]["barrfrominit"] = emax_for_strain - einit_for_strain
#Get DMEPS values
for bsite in blist:
    for hdir in ['ip','oop']:
        slist=list()
        barrlist=list()
        strains = fitdict[bsite][hdir].keys()
        strains.sort()
        print "BSITE: ", bsite
        for strain in strains:
            migbarr = fitdict[bsite][hdir][strain]["barrfrominit"]
            print "%3.3f, %3.3f" % (strain, migbarr)
            slist.append(strain)
            barrlist.append(migbarr)
        fitdict[bsite][hdir]['strains'] = list(slist)
        fitdict[bsite][hdir]['barriers'] = list(barrlist)
        [dmeps, dmepserr] = nebenergy.get_dmeps(slist, barrlist, 1, bsite + "," + hdir)
        fitdict[bsite][hdir]['dmeps'] = dmeps
        fitdict[bsite][hdir]['dmepserr'] = dmepserr
#Print output tables
for hdir in ['ip','oop']:
    print "Direction", hdir
    allstrains=list()
    for bsite in blist:
        slist = fitdict[bsite][hdir]['strains']
        for strain in slist:
            if not strain in allstrains:
                allstrains.append(strain)
    allstrains.sort()
    dispstr = "% strain"
    for bsite in blist:
        dispstr = dispstr + "," + bsite
    print dispstr
    for thisstrain in allstrains: #sort of inefficient.
        strainstr = "%3.3f" % thisstrain
        for bsite in blist:
            strainstr = strainstr + ","
            slist = fitdict[bsite][hdir]['strains']
            barrlist = fitdict[bsite][hdir]['barriers']
            for sidx in range(0, len(slist)):
                mystrain = slist[sidx]
                if mystrain == thisstrain:
                    strainstr = strainstr + "%3.3f" % barrlist[sidx]
        print strainstr
print "B-site, DMEPS IP, error, DMEPS OOP, error"
for bsite in blist:
    print "%s,%3.3f,%3.3f,%3.3f,%3.3f" % (bsite, fitdict[bsite]['ip']['dmeps'],
        fitdict[bsite]['ip']['dmepserr'],
        fitdict[bsite]['oop']['dmeps'],
        fitdict[bsite]['oop']['dmepserr'])

