#!/usr/bin/env python
# Tam Mayeshiba 2015-11-12
# cleanup from strain data
# S7 lattice fitting for doped supercells
######
import sys
import os
import subprocess
import Vgetenergy
import hocking_lstsq
topdir="//home/tam/strain_emig_clean/S7_doped_barriers/doped_fitting/%s" % sys.argv[1]
#run with argument "cross_body_diagonal", "face_diagonal", and "in_line"

import sys
skiplist=list()
fitdict=dict()
#for bsite in ['c','f','g','m','n','r','s','t','v']:
elemdirs = os.listdir(topdir)
labeldict=dict()
labeldict['_n2']=[0.98,[0.99,1.00,1.01,1.02,1.03,1.04,1.05]]
labeldict['_n1']=[0.99,[0.98,0.99,1.00,1.01,1.02,1.03,1.04]]
labeldict['_p0']=[1.00,[0.97,0.98,0.99,1.00,1.01,1.02,1.03]]
labeldict['_p1']=[1.01,[0.96,0.97,0.98,0.99,1.00,1.01,1.02,1.03]]
labeldict['_p2']=[1.02,[0.96,0.97,0.98,0.99,1.00,1.01,1.02]]
for elemdir in elemdirs:
    dirpath = os.path.join(topdir, elemdir)
    bsite = elemdir.split("_")[2]
    if not bsite in fitdict.keys():
        fitdict[bsite]=dict()
    if os.path.isdir(dirpath):
        statdirs = os.listdir(dirpath)
        for statdir in statdirs:
            astrain=""
            cstrain=""
            energy=""
            statpath = os.path.join(dirpath, statdir)
            if "perfstat" in statdir:
                suffix = statdir[-4:-1]
                desig = int(statdir[-1])
                astrain=labeldict[suffix][0]
                cstrain=labeldict[suffix][1][desig - 1]
                odir = os.path.join(statpath, "OSZICAR")
                if os.path.isfile(odir):
                    energy = Vgetenergy.main(statpath)
                else:
                    skiplist.append("%s: does not exist" % odir)
                    continue
                if not astrain in fitdict[bsite].keys():
                    fitdict[bsite][astrain]=dict()
                    fitdict[bsite][astrain]['cstrains']=list()
                    fitdict[bsite][astrain]['energies']=list()
                fitdict[bsite][astrain]['cstrains'].append(float(cstrain))
                fitdict[bsite][astrain]['energies'].append(float(energy))

import simpleplot
import numpy as np
import time
bsites = fitdict.keys()
bsites.sort()
for bsite in bsites:
    astrains = fitdict[bsite].keys()
    astrains.sort()
    for astrain in astrains:
        print "BSITE: ", bsite
        print "ASTRAIN: ", astrain
        clist = fitdict[bsite][astrain]['cstrains']
        cvec = np.array(clist, 'float')
        elist = fitdict[bsite][astrain]['energies']
        evec = np.array(elist, 'float')
        print "Cvec, Evec: ", cvec, evec
        simpleplot.plot(cvec, evec, "Bsite %s: a-strain %s" % (bsite, astrain))
        [betamat, ssq, betaerr] = hocking_lstsq.least_squares_regression(cvec, evec, 3)
        fitdict[bsite][astrain]['fit_s'] = np.power(ssq, 0.5)
        coeffa=float(betamat[3])
        coeffb=float(betamat[2])
        coeffc=float(betamat[1])
        deriva = 3.0 * coeffa #times x^2
        derivb = 2.0 * coeffb #times x
        derivc = coeffc # intercept of derivative
        #let's solve the quadratic equation!
        #[-b +/- sqrt(b^2 - 4ac)]/ 2a
        insqrtterm = np.power(derivb, 2) - 4.0*deriva*derivc
        #print "In sqrt: ", insqrtterm
        sqrtterm = np.power(insqrtterm, 0.5)
        topterm1 = -1.0*derivb + sqrtterm
        topterm2 = -1.0*derivb - sqrtterm
        solve1 = topterm1 / (2.0 * deriva)
        solve2 = topterm2 / (2.0 * deriva)
        lowerbound=0.95
        upperbound=1.05
        results="RESULTS: "
        if (solve1 <= upperbound) and (solve1 >= lowerbound): 
            results = results + str(solve1)
        if (solve2 <= upperbound) and (solve2 >= lowerbound):
            results = results + ":" + str(solve2)
        print results
        fitdict[bsite][astrain]['results'] = results
print "B-site:a strain: fit S:results"
for bsite in bsites:
    for astrain in astrains:
        print "%s:%s:%3.3f:%s" % (bsite, astrain, 
            fitdict[bsite][astrain]['fit_s'],
            fitdict[bsite][astrain]['results'])


            





