#!/usr/bin/env python
# Tam Mayeshiba 2015-11-19
# cleanup from strain data
# S8 full hop direction barriers, DMEPS, migration volumes, and
#   elastic model DMEPS
######

import os
import subprocess
import numpy as np
import sys
import nebenergy
import simplegrep
import elastic_constants
topdir="//home/tam/strain_emig_clean/S8_hop_directions/fullgrid"
addldir="//home/tam/strain_emig_clean/S12_elastic_model"
fitdict=dict()
#
#blist=list(sys.argv[1])
#blist=['c','f','g','m','n','r','s','t','v']

astraindict=dict()
astraindict["1000"]=0
astraindict["1010"]=1
astraindict["1020"]=2
astraindict["980"]=-2
astraindict["990"]=-1

r_elastic = simplegrep.grep_return_last("r,", "%s/%s" % (addldir, "S12_elastic_model_output"))
m_elastic = simplegrep.grep_return_last("m,", "%s/%s" % (addldir, "S12_elastic_model_output"))

bulkmod=dict()
bulkmod["CrLaO"] = float(r_elastic.split(",")[3]) * 10.0 #GPa to kbar
bulkmod["OLaMn"] = float(m_elastic.split(",")[3]) * 10.0 #GPa to kbar
derivbm=dict()
derivbm["CrLaO"] = float(r_elastic.split(",")[5]) * 10.0 #GPa to kbar
derivbm["OLaMn"] = float(m_elastic.split(",")[5]) * 10.0 #GPa to kbar

prefactor=dict()
prefactor["CrLaO"] = float(r_elastic.split(",")[8])
prefactor["OLaMn"] = float(m_elastic.split(",")[8])

prefactorerr=dict()
prefactorerr["CrLaO"] = float(r_elastic.split(",")[9])
prefactorerr["OLaMn"] = float(m_elastic.split(",")[9])

vmigerr = 0.1 * np.sqrt(2.0)
volatnostrain=dict()

sysdirs = os.listdir(topdir)
for sysdir in sysdirs:
    bsite=sysdir.split("_")[1]
    straininit=sysdir.split("_")[0]
    strain = astraindict[straininit]
    if strain == 0:
        volatnostrain[bsite] = simplegrep.get_vasp_volume_of_cell(os.path.join(topdir, sysdir, "perfect_stat"))

#For ip and oop and each strain, get all neb reports

ofile = open(os.path.join(os.path.dirname(topdir),"S8_oxygen_key"),'rb')
olines = ofile.readlines()
ofile.close()
odict=dict()
for oline in olines:
    olabel=oline.split()[0]
    olevel=oline.split()[-1]
    odict[olabel] = olevel

bfile = open(os.path.join(os.path.dirname(topdir),"S8_center_b_key"),'rb')
blines = bfile.readlines()
bfile.close()
bdict=dict()
for bline in blines:
    blabel=bline.split()[0]
    boxygens=bline.split()[1]
    bdict[blabel]=boxygens

nebkey=0
sysdirs = os.listdir(topdir)
for sysdir in sysdirs:
    bsite=sysdir.split("_")[1]
    straininit=sysdir.split("_")[0]
    strain = astraindict[straininit]
    syspath = os.path.join(topdir, sysdir)
    nebdirs = os.listdir(syspath)
    for nebdir in nebdirs:
        if ("neb" in nebdir) and ("stat" in nebdir):
            nebkey = nebkey + 1
            fitdict[nebkey]=dict()
            opc = nebdir.split("_")[1]
            oinit = opc.split("-")[0]
            ofin = opc.split("-")[1]
            if (odict[oinit] == odict[ofin]): # same level
                hopdir="ip"
            else:
                hopdir="oop"
            nebpath = os.path.join(syspath, nebdir)
            nebrpt = nebenergy.report(nebpath)
            centerb="???"
            for btries in bdict.keys():
                if (oinit in bdict[btries]) and (ofin in bdict[btries]):
                    centerb = btries
            fitdict[nebkey]['bsite']=bsite
            fitdict[nebkey]['strain']=strain
            fitdict[nebkey]['nebrpt']=dict(nebrpt)
            fitdict[nebkey]['opc']=opc
            fitdict[nebkey]['centerb']=centerb
            fitdict[nebkey]['hopdir']=hopdir
            if strain == 0:
                saddle_press = simplegrep.get_vasp_pressure(os.path.join(nebpath,"01"))
                if saddle_press == "error":
                    print nebpath
                saddle_v0 = elastic_constants.get_volume_from_birch_murnaghan(bulkmod[bsite],derivbm[bsite],saddle_press,volatnostrain[bsite])
                initpath = os.path.join(os.path.dirname(nebpath),"defect_%s_q=p2_stat" % oinit)
                init_press = simplegrep.get_vasp_pressure(initpath)
                if init_press == "error":
                    print "init press ", nebpath, initpath
                init_v0 = elastic_constants.get_volume_from_birch_murnaghan(bulkmod[bsite],derivbm[bsite],init_press,volatnostrain[bsite])
                vmig = saddle_v0 - init_v0
                fitdict[nebkey]['vmig_nostrain']=vmig
                [slope, slopeerr] = elastic_constants.strain_model_slope(prefactor[bsite],prefactorerr[bsite],vmig,vmigerr)
                fitdict[nebkey]['model_slope'] = slope
                fitdict[nebkey]['model_slope_err'] = slopeerr

#print barrier values
allbsites=list()
allcenterbs=list()
allstrains=list()
allopcs=list()
allhopdirs=list()
for nebkey in fitdict.keys():
    if not fitdict[nebkey]['bsite'] in allbsites:
        allbsites.append(fitdict[nebkey]['bsite'])
    if not fitdict[nebkey]['hopdir'] in allhopdirs:
        allhopdirs.append(fitdict[nebkey]['hopdir'])
    if not fitdict[nebkey]['centerb'] in allcenterbs:
        allcenterbs.append(fitdict[nebkey]['centerb'])
    if not fitdict[nebkey]['strain'] in allstrains:
        allstrains.append(fitdict[nebkey]['strain'])
    if not fitdict[nebkey]['opc'] in allopcs:
        allopcs.append(fitdict[nebkey]['opc'])
allbsites.sort()
allcenterbs.sort()
allstrains.sort()
allopcs.sort()
allhopdirs.sort()

dmepsdict=dict()
for bsite in allbsites:
    dmepsdict[bsite]=dict()
    for centerb in allcenterbs:
        dmepsdict[bsite][centerb]=dict()
        for hopdir in allhopdirs:
            dmepsdict[bsite][centerb][hopdir]=dict()
for bsite in allbsites:
    print "B-site: %s" % bsite
    for centerb in allcenterbs:
        print "Center B: %s" % centerb
        for hopdir in allhopdirs:
            print "Hop direction: %s" % hopdir
            print "Barriers vs. strain for %s,%s,%s" % (bsite,centerb,hopdir)
            dispstr="% strain"
            strainstrlist=list()
            for strain in allstrains:
                strainstr=""
                strainstr = strainstr + str(strain)
                for opc in allopcs:
                    for nebkey in fitdict.keys():
                        okay=0
                        if fitdict[nebkey]['bsite'] == bsite:
                            if fitdict[nebkey]['centerb'] == centerb:
                                if fitdict[nebkey]['strain'] == strain:
                                    if fitdict[nebkey]['opc'] == opc:
                                        if fitdict[nebkey]['hopdir'] == hopdir:
                                            okay=1
                        if (okay == 1):
                            if not opc in dispstr:
                                dispstr = dispstr + ",%s" % opc
                            mybarr=fitdict[nebkey]['nebrpt']['barrfrominit']
                            strainstr = strainstr + ",%3.3f" % mybarr
                            if not (opc in dmepsdict[bsite][centerb][hopdir].keys()):
                                dmepsdict[bsite][centerb][hopdir][opc]=dict()
                                dmepsdict[bsite][centerb][hopdir][opc]['strains']=list()
                                dmepsdict[bsite][centerb][hopdir][opc]['barriers']=list()
                            dmepsdict[bsite][centerb][hopdir][opc]['strains'].append(strain)
                            dmepsdict[bsite][centerb][hopdir][opc]['barriers'].append(mybarr)
                            if "vmig_nostrain" in fitdict[nebkey].keys():
                                for modelkey in ["vmig_nostrain","model_slope","model_slope_err"]:
                                    dmepsdict[bsite][centerb][hopdir][opc][modelkey] = fitdict[nebkey][modelkey]
                strainstrlist.append(strainstr)
            print dispstr
            for strainstritem in strainstrlist:
                print strainstritem
                                    

#Get DMEPS values
for bsite in allbsites:
    for centerb in allcenterbs:
        for hopdir in allhopdirs:
            for opc in allopcs:
                if opc in dmepsdict[bsite][centerb][hopdir].keys():
                    [dmeps, dmepserr] = nebenergy.get_dmeps(dmepsdict[bsite][centerb][hopdir][opc]['strains'],dmepsdict[bsite][centerb][hopdir][opc]['barriers'], 1, "%s,%s,%s,%s" % (bsite, centerb, hopdir, opc))
                    dmepsdict[bsite][centerb][hopdir][opc]['dmeps'] = dmeps
                    dmepsdict[bsite][centerb][hopdir][opc]['dmepserr'] = dmepserr





#print DMEPS table
for bsite in allbsites:
    for centerb in allcenterbs:
        for hopdir in allhopdirs:
            print "DMEPS for %s, %s, %s" % (bsite, centerb, hopdir)
            print "hop, DMEPS, error, Vmig, model DMEPS, model DMEPS error"
            opcs = dmepsdict[bsite][centerb][hopdir].keys()
            opcs.sort()
            for opc in opcs:
                print "%s, %3.3f, %3.3f, %3.1f, %3.3f, %3.3f" % (opc, 
                    dmepsdict[bsite][centerb][hopdir][opc]['dmeps'], 
                    dmepsdict[bsite][centerb][hopdir][opc]['dmepserr'], 
                    dmepsdict[bsite][centerb][hopdir][opc]['vmig_nostrain'],
                    dmepsdict[bsite][centerb][hopdir][opc]['model_slope'],
                    dmepsdict[bsite][centerb][hopdir][opc]['model_slope_err'])

#print hop tables for zero strain
for bsite in allbsites:
    for hopdir in allhopdirs:
        print "B-site %s, %s" % (bsite, hopdir)
        print "Center-b hops at zero strain"
        datastrlist=list()
        for centerb in allcenterbs:
            datastr = centerb
            for nebkey in fitdict.keys():
                if fitdict[nebkey]['bsite'] == bsite:
                    if fitdict[nebkey]['hopdir'] == hopdir:
                        if fitdict[nebkey]['centerb'] == centerb:
                            if fitdict[nebkey]['strain'] == 0:
                                datastr = datastr + ",%3.3f" % fitdict[nebkey]['nebrpt']['barrfrominit']
            datastrlist.append(datastr)
        for datastritem in datastrlist:
            print datastritem

#print DMEPS for center B's
for bsite in allbsites:
    for hopdir in allhopdirs:
        print "DMEPS for %s, %s" % (bsite, hopdir)
        for centerb in allcenterbs:
            dispstr = centerb
            opcs = dmepsdict[bsite][centerb][hopdir].keys()
            opcs.sort()
            for opc in opcs:
                dispstr = dispstr + ",%3.3f" % dmepsdict[bsite][centerb][hopdir][opc]['dmeps']
            print dispstr


