#!/usr/bin/env python
##
#@package   Pgetline 
#@ingroup   vasp
#@ingroup   python
#@brief     gets a line from a specified file
#@author    Tam Mayeshiba
#@date      11/9/10 created
#@date      12/22/10 splitting up input into directory and file,
#              in order to facilitate looping
#@date      8/3/11 add line number           
###
def main(existdir, existname, findstr, findln='X', getlast='0', lineno=0):
    """
    Purpose: gets a line from a specified output file
        inputs:  
            [$1] path for file to search in
                 example: /home/tam/newNaCl
            [$2] file name to search for
                 example: POSCAR
            [$3] string to search for, in quotes
	             example: "[VASP_POTCAR_FILE]"
		         enter 0 if searching by line number instead
	        [$4] line number to search for
	             enter X if searching by string instead
            [$5] first or last occurrence of string to search for
	             0 - first occurrence (default)
		         1 - last occurrence
                 this parameter is ignored if searching by line number
            [$6] append the line number
                 0 - no (default)
                 1 - yes (will append :# to the return line)
                        only works if FIRST occurrence is found
        outputs: found line, or None if not found 
    Note: right now this only finds the first occurrence in the file.
          also, the file should never have any blank lines in it
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
#
    tgfile=-1
    readln=""
    readf=""
    linect=1
#   validate inputs
#   #   validate directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None    
#   #   validate file existence
    existf=""
    existf = existdir + '/' + existname
    if (os.path.isfile(existf)) == False:
        print existf, 'does not exist.'
        print '(Exiting.)'
        return None
#   #   convert string to find
    findstr = str(findstr)
#   #   #   validate line number
    findln = str(findln)
    if (findln in ['X', 'x']):
        findln = str.upper(findln)
        if (findstr == '0'):
            print '0 is not a good string for which to search.'
            print 'Please try again with a different string'
            print 'or use a line number in the third parameter instead.'
            print '(Exiting.)'
            return None
    else:
        try:
            findln = int(findln)
        except (TypeError, ValueError):
            print findln, 'should be a line number or X.'
            print '(Exiting.)'
            return None
        if (findstr != "0"):
            print 'Line number will be used instead of search string.'
#   #   #   validate first or last for searchstring (default to 0)
    if (findln == 'X'):
        try:
            getlast = str(getlast)
        except (TypeError, ValueError):
            print getlast, 'should be 1 or 0.'
            print '(Exiting.)'
            return None
        if not getlast in ['1', '0']:
            getlast = '0'
            print 'Warning: first or last string occurrence parameter should'
            print 'only be 1 or 0. Defaulting to 0 (first occurrence).'
    else:
        getlast = '0'
#   #   #   lineno
    lineno = Xvalidinteger.main(lineno, 0, 1)
    if (lineno == None):
        print '(Exiting.)'
        return None
#   #   open target file
    readf = open(existf, 'rb')
#   #   read lines until target line is found
    pasttg=-1
    pastln=""
    firsttg=-1
    firstln=""
    if (findln == 'X'):
        while (tgfile == -1):
            readln = readf.readline()
            if not readln:
                break                      # file should not have blank lines
            linect = linect + 1
            tgfile = readln.find(findstr)
        if (getlast == '0'):
            pass
        else:
            firsttg = tgfile
            firstln = readln
            while (readln):
                pasttg = tgfile
                pastln = readln
                tgfile = -1
                while (tgfile == -1):
                    readln = readf.readline()
                    if not readln:
                        break
                    linect = linect + 1
                    tgfile = readln.find(findstr)
            if (pasttg == -1 and firsttg != -1):
                tgfile = firsttg
                readln = firstln
            else:
                tgfile = pasttg
                readln = pastln
#   #   TTM debug: what if the string is never found?
        if (tgfile == -1):
            print findstr, 'was not found.'
            print '(Exiting.)'
            readf.close()
            return None
    else:
        while (linect <= findln):
            readln = readf.readline()
            if not readln:
                break
            linect = linect + 1
#   #   readln is the search line or found string line
    readf.close()
    if (lineno == 1):
        readln = readln + ':' + str(linect-1)  # linect overshoots by 1
    return (readln)
#
#
def loop(topdir, fname, findstr, findln='X', getlast='0'):
    """
    Purpose: gets lines from files through a directory
        inputs:  
            [$1] top path for file to search in
                 example: /home/tam/newNaCl
            [$2] file name (should be the same in each folder)
                example: POSCAR
            [$2] string to search for, in quotes
	             example: "[VASP_POTCAR_FILE]"
		         enter 0 if searching by line number instead
	        [$3] line number to search for
	             enter X if searching by string instead
            [$4] first or last occurrence of string to search for
	             0 - first occurrence (default)
		         1 - last occurrence
                 this parameter is ignored if searching by line number
        outputs: found line, or None if not found 
    Note: right now this only finds the first occurrence in the file.
          also, the file should never have any blank lines in it
    """
#   import
    import os
    import sys
    import Xgetdirs
    import Xvaliddirectory
    import Pgetline
#   #   validate directory
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   other validation will be handled by Pgetline.main
    dirlist=[]
    dirlist = Xgetdirs.main(topdir, 1, 5)
    mydir=""
    resultlist=[]
    myresult=""
    for mydir in dirlist:
        myresult = Pgetline.main(mydir, fname, findstr, 
                                    findln, getlast)
        if (myresult != None):
            resultlist.append(mydir + ': ' + str(myresult))
    for myresult in resultlist:
        print myresult
    return True



