#!/usr/bin/env python
##
#@package   Pgetseg
#@ingroup   vasp
#@ingroup   python
#@brief     returns a segment of a line
#@author    Tam Mayeshiba
#@date      12/3/10 created
###
def main(myline, firstch=1, lastch=0):
    """
    Purpose: returns a segment of a line 
         inputs:  
            [$1] line (string)
            [$2] EITHER: 
                positive integer to start at this character position
                default 1 to start at beginning of line
                (when in a text file, starts counting at 1; use this.
                In python, strings start at 0, so 1 will be subtracted)
                OR:
                string to search for, after which the segment will start
            [$3] 
                EITHER:
                positive integer to end at this character position (inclusive)
                example: 'apple', 1, 3 will return 'app'
                OR:
                string to search for, before which the segment will end
                example: 'I like cheese curds.', 'like', 'curd'
                        will return ' cheese '
                OR:
                0, to get the whole line after the starting position (default)
        returns:
            segment if it exists
            '' if there is actually no string
            None if there was an error
        notes:
            - if the ending criterion falls before the beginning criterion,
                returns ''
            -'33' is a string, while 33 is an integer
    """
#   Modules:
    import os
    import sys
#   
#   validate inputs
#   #   validate string
    try:
        myline = str(myline)
    except (ValueError, TypeError):
        print myline, 'should be a string.'
        print '(Exiting.)'
        return None
    if type(myline) != str:
        print myline, 'should be a string.'
        print '(Exiting.)'
        return None
#   #   validate firstch and lastch
    linelen=0
    linelen = len(myline)
    if type(firstch) != str:        
        try:
            firstch = int(firstch)
        except (ValueError, TypeError):
            print firstch, 'should be a string or a number.'
            print '(Exiting.)'
            return None
    if (type(firstch) == int) and (firstch > linelen):
        print firstch, 'is greater than the line length.'
        print '(Exiting.)'
        return None
    if type(lastch) != str:        
        try:
            lastch = int(lastch)
        except (ValueError, TypeError):
            print lastch, 'should be a string or a number.'
            print '(Exiting.)'
            return None
#   #   get positions
    startpos=0
    addstart=0
    endpos=0
    if (type(firstch) == int):
        startpos = firstch - 1 # subtract 1 because characters in python
                                # strings start at 0
    elif (type(firstch) == str):
        startpos = myline.find(firstch)
        if (startpos == -1):
            print firstch, 'not found.'
            print '(Exiting.)'
            return None
        addstart = len(firstch)
        startpos = startpos + addstart # want to get after the string
    else:
        print 'Error calculating starting position.'
        print '(Exiting.)'
        return None
    if (type(lastch) == int):
        endpos = lastch
    elif (type(lastch) == str):
        endpos = myline.find(lastch, startpos)
        if (endpos == -1):
            print lastch, 'not found.'
            print '(Exiting.)'
            return None
    else:
        print 'Error calculating ending position.'
        print '(Exiting.)'
        return None
#   #
    #print startpos #debug
    #print endpos #debug
    if (endpos == 0):
        return myline[startpos:]
    else:
        return myline[startpos:endpos]
