#!/usr/bin/env python
##
#@package   Vanisotropy
#@ingroup   vasp
#@ingroup   python
#@brief     calculates some crystal structure parameters
#@author    Tam Mayeshiba
#@date      
#@date      7/14/11 added looping function
###
def main(existdir, fname='CONTCAR'):
    """
    Purpose: Calculates degrees between lattice vectors and
             magnitude ratios
        inputs:  
            [$1] directory of run 
            example: /home/tam/NaCl
            [$2] file name of CONTCAR or POSCAR type file
                 default - CONTCAR
        outputs:
            None if error
    """
#   Modules:
    import os
    import sys
    import math
    import Xvaliddirectory
#   Validation
#   #  existdir
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   fname
    if type(fname) != str:
        print fname, 'should be a string.'
        print '(Exiting.)'
        return None
#   get CONTCAR
    fullname=""
    fullname = existdir + '/' + fname
    if not os.path.isfile(fullname):
        print fullname, 'not found.'
        print '(Exiting.)'
        return None
#   read lattice vectors
    readme=""
    readln=""
    vec1list=[]
    vec2list=[]
    vec3list=[]
    readme = open(fullname, 'rb')
    readln = readme.readline() # get first line: comment or header
    readln = readme.readline() # get second line: scaling factor
    readln = readme.readline() # get first vector
    readln = readln.strip()
    vec1list = readln.split()
    readln = readme.readline() # get second vector
    readln = readln.strip()
    vec2list = readln.split()
    readln = readme.readline() # get third vector
    readln = readln.strip()
    vec3list = readln.split()
    readme.close()
#   parse vectors
    if (vec1list == None) or (vec2list == None) or (vec3list == None):
        print 'Error retrieving one or more lattice vectors.'
        print '(Exiting.)'
        return None
    if (len(vec1list) != 3) or (len(vec2list) != 3) or (len(vec3list) != 3):
        print 'One or more lattice vectors does not have three coordinates.'
        print '(Exiting.)'
        return None
    vec1flt=[]
    vec2flt=[]
    vec3flt=[]
    flttry=0
    for item in vec1list:
        try:
            flttry = float(item)
        except (ValueError, TypeError):
            print item, 'in vector 1 could not be converted to a float.'
            print '(Exiting.)'
            return None
        vec1flt.append(flttry)
    for item in vec2list:
        try:
            flttry = float(item)
        except (ValueError, TypeError):
            print item, 'in vector 2 could not be converted to a float.'
            print '(Exiting.)'
            return None
        vec2flt.append(flttry)
    for item in vec3list:
        try:
            flttry = float(item)
        except (ValueError, TypeError):
            print item, 'in vector 3 could not be converted to a float.'
            print '(Exiting.)'
            return None
        vec3flt.append(flttry)
#   Calculate values of interest
    dot1to2=0
    dot1to3=0
    dot2to3=0
    len1=0
    len2=0
    len3=0
    len1to2=0
    len1to3=0
    len2to3=0
    cos1to2=0
    cos1to3=0
    cos2to3=0
    theta1to2=0
    theta1to3=0
    theta2to3=0
    deg1to2=0
    deg1to3=0
    deg2to3=0
#   dot products
    dot1to2 = vec1flt[0]*vec2flt[0] + vec1flt[1]*vec2flt[1]
    dot1to2 = dot1to2 + vec1flt[2]*vec2flt[2]
    dot1to3 = vec1flt[0]*vec3flt[0] + vec1flt[1]*vec3flt[1]
    dot1to3 = dot1to3 + vec1flt[2]*vec3flt[2]
    dot2to3 = vec2flt[0]*vec3flt[0] + vec2flt[1]*vec3flt[1]
    dot2to3 = dot2to3 + vec2flt[2]*vec3flt[2]
#   magnitudes
    len1 = math.sqrt(vec1flt[0]**2 + vec1flt[1]**2 + vec1flt[2]**2)
    len2 = math.sqrt(vec2flt[0]**2 + vec2flt[1]**2 + vec2flt[2]**2)
    len3 = math.sqrt(vec3flt[0]**2 + vec3flt[1]**2 + vec3flt[2]**2)
#   cosines
    cos1to2 = dot1to2 / (len1 * len2)
    cos1to3 = dot1to3 / (len1 * len3)
    cos2to3 = dot2to3 / (len2 * len3)
#   angles
    theta1to2 = math.acos(cos1to2)
    theta1to3 = math.acos(cos1to3)
    theta2to3 = math.acos(cos2to3)
    deg1to2 = theta1to2 * 180 / math.pi
    deg1to3 = theta1to3 * 180 / math.pi
    deg2to3 = theta2to3 * 180 / math.pi
#   lengths
    len1to2 = len1 / len2
    len1to3 = len1 / len3
    len2to3 = len2 / len3
#   return
    returnstr=""
    returnstr = 'magnitudes and angles: a to b, a to c, b to c: '
    returnstr = returnstr + existdir + ': '
    returnstr = returnstr + str(len1to2) + ', ' + str(len1to3) + ', ' 
    returnstr = returnstr + str(len2to3) + ', '
    returnstr = returnstr + str(deg1to2) + ', ' + str(deg1to3) + ', ' 
    returnstr = returnstr + str(deg2to3)
    return returnstr
#
#
def loop(topdir="", fname='CONTCAR'):
    """
    Purpose: Calculates degrees between lattice vectors and
             magnitude ratios
        inputs:  
            [$1] top directory in which to search
            example: /home/tam/NaCl
            [$2] file name of CONTCAR or POSCAR type file
                 default - CONTCAR
        outputs:
            None if error
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Vanisotropy
    import Xgetdirs
    import Pmakemonfile
#   Validation
#   #  existdir
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   fname
    if type(fname) != str:
        print fname, 'should be a string.'
        print '(Exiting.)'
        return None
#   get directories
    dirlist=[]
    onedir=""
    onedirres=""
    dirlist = Xgetdirs.main(topdir, 1, 5)
    if (dirlist == None):
        print 'Error getting subdirectories for', topdir
        print '(Exiting.)'
        return None
    if len(dirlist) == 0:
        print 'No subdirectories found for', topdir
        print '(Exiting.)'
        return None
    myfilename=""
    myfilename = Pmakemonfile.main(topdir)
    if (myfilename == None):
        print 'Could not create unique file name for writing.'
        print '(Exiting.)'
        return None
    myfile=""
    myfile = open(myfilename, 'ab')
    for onedir in dirlist:
        onedirres = Vanisotropy.main(onedir, fname)
        myfile.write(str(onedirres) + '\n')
    myfile.close()
    return True
