#!/usr/bin/env python
##
#@package   Vgetenergy
#@ingroup   vasp
#@ingroup   python
#@brief     gets last E0 from OSZICAR
#@author    Tam Mayeshiba
#@date      12/20/10 created 
#@date      12/22/10 change Pgetline call to reflect new inputs
#@date      12/28/10 add loop tag
###
def main(mydir):
    """
    Purpose: gets last E0 from OSZICAR
        inputs:  
            [$1] directory 
            example: '/home/tam/NaCl1'
        outputs:
            None - error
            last E0 from OSZICAR
    """
#   Modules:
    import os
    import sys
    import Pgetline
    import Pgetseg
#   validation handled by Pgetline
#   #
#   get energy
    myline=""
    myseg=""
    myline = Pgetline.main(mydir, 'OSZICAR', 'E0', 'X', 1)
    if (myline == None):
        print 'Error reading OSZICAR for', mydir
        print '(Exiting.)'
        return None
    myseg = Pgetseg.main(myline, 'E0=', 'd E =')
    if (myseg == None):
        print 'Error isolating energy for', mydir
        print '(Exiting.)'
        return None
    return myseg
#
#
def loop(topdir):
    """
    Purpose: gets last E0 from OSZICAR files in directories under directory
        inputs:  
            [$1] top directory 
            example: '/home/tam/NaClcollection'
        outputs:
            None - error
            last E0 from OSZICAR
    """
#   Modules
    import os
    import sys
    import Vgetenergy
    import Xvaliddirectory
    import Xgetdirs
#   Validate directory
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   other validation will be handled by Vgetenergy
    dirlist=[]
    dirlist = Xgetdirs.main(topdir, 1, 5)
    mydir=""
    resultlist=[]
    for mydir in dirlist:
        myresult = Vgetenergy.main(mydir)
        if (myresult != None):
            resultlist.append(mydir + ': ' + str(myresult))
    for myresult in resultlist:
        print myresult
    return True
