#!/usr/bin/env python
##
#@package   Vgetvolume
#@ingroup   vasp
#@ingroup   python
#@brief     gets cell volume
#@author    Tam Mayeshiba
#@date      1/4/11 
#@date      2/25/11 initialize myresult in loop
#@date      8/19/11 change grep volume to volume of cell for use with RWIGS
###
def main(mydir):
    """
    Purpose: gets cell volume
        inputs:  
            [$1] directory 
            example: '/home/tam/NaCl1'
        outputs:
            None - error
            total CPU time
    """
#   Modules:
    import os
    import sys
    import Pgetline
    import Pgetseg
#   validation handled by Pgetline
#   #
#   get total CPU time
    myline=""
    myseg=""
    myline = Pgetline.main(mydir, 'OUTCAR', 'volume of cell', 'X', 1)
    if (myline == None):
        print 'Error reading OUTCAR for', mydir
        print '(Exiting.)'
        return None
    myseg = Pgetseg.main(myline, 'cell :')
    if (myseg == None):
        print 'Error isolating volume for', mydir
        print '(Exiting.)'
        return None
    return myseg
#
#
def loop(topdir):
    """
    Purpose: gets total CPU time from directories under directory
        inputs:  
            [$1] top directory 
            example: '/home/tam/NaClcollection'
        outputs:
            None - error
            last E0 from OSZICAR
    """
#   Modules
    import os
    import sys
    import Vgetvolume
    import Xvaliddirectory
    import Xgetdirs
#   Validate directory
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   other validation will be handled by Vgetenergy
    dirlist=[]
    dirlist = Xgetdirs.main(topdir, 1, 5)
    mydir=""
    myresult=""
    resultlist=[]
    noresultlist=[]
    for mydir in dirlist:
        myresult = Vgetvolume.main(mydir)
        if (myresult != None):
            resultlist.append(mydir + ': ' + str(myresult))
        else:
            noresultlist.append(mydir)
    print 'Total volume in cubic Angstroms: '
    for myresult in resultlist:
        print myresult,
    if len(noresultlist) > 0:
        print 'Runs without a total volume: '
        for myresult in noresultlist:
            print myresult
    return True
