#!/usr/bin/env python
##
#@package   Xvalidinteger
#@ingroup   other
#@ingroup   python
#@brief     checks to see if an input is a valid integer
#           within an optional range
#@author    Tam Mayeshiba
#@date      11/30/10 
###
def main(checkint, lbound=None, ubound=None):
    """
    Purpose:
        inputs:  
            [$1] input to check
                example: '-20'
            [$2] lower bound (integer, inclusive) 
                - enter None for no lower bound
            [$3] upper bound (integer, inclusive) 
                - enter None for no upper bound
            e.g. Xvalidinteger.main(-20,0,3) checks to see if -20 is
                either 0, 1, 2, or 3 (and returns None)
        outputs:
            returns the integer in integer format if it is good
            None - input failed a check
        notes:
            will not handle decimals
            
    """
#   Modules:
#    import os
#    import sys
#   
#   #   check input for commas or periods
    mystr=""
    try:
        mystr = str(checkint) # any valid integer can also be a string
    except (ValueError, TypeError):
        print checkint, 'should be an integer.'
        print '(Exiting.)'
        return None
    if ('.' in mystr):
        print checkint, 'should be an integer.'
        print '(Exiting.)'
        return None
    if (',' in mystr):
        print 'Please enter', checkint, 'without any commas.'
        print '(Exiting.)'
        return None
#   #  check to see if input is an integer
    myint=""
    try:
        myint = int(checkint)
    except (ValueError, TypeError):
        print checkint, 'should be an integer.'
        print '(Exiting.)'
        return None
#   # check lower bound
    if lbound != None:
        try:
            lbound = int(lbound)
        except (ValueError, TypeError):
            print lbound, 'should be an integer'
            print 'for the lower bound, inclusive.'
            print '(Exiting.)'
            return None
#   # validate upper bound
    if ubound != None:
        try:
            ubound = int(ubound)
        except (ValueError, TypeError):
            print ubound, 'should be an integer'
            print 'for the upper bound, inclusive.'
            print '(Exiting.)'
            return None
    if (lbound != None) and (ubound == None):
        if myint < lbound:
            print checkint, 'should be greater than or equal to', lbound
            print '(Exiting.)'
            return None
    elif (ubound != None) and (lbound == None):
        if myint > ubound:
            print myint, 'should be less than or equal to', ubound
            print '(Exiting.)'
            return None
    elif (lbound != None) and (ubound != None):
        if ubound < lbound:
            print 'Upper bound', ubound, 'should be greater than or'
            print 'equal to the lower bound,', lbound
            print '(Exiting.)'
            return None
        if (myint < lbound) or (myint > ubound):
            print myint, 'should be between', lbound, 'and', ubound,
            print '(inclusive).'
            print '(Exiting.)'
            return None
#   #   check for too large integer
    if (myint > 999999999999999999) or (myint < -999999999999999999):
        print myint, 'is outside of the reasonable integer range.'
        print 'If it should be accepted, please do not use Xvalidinteger.py.'
        print '(Exiting.)'
        return None
    return myint





