#!/usr/bin/env python
#####################
# Tam Mayeshiba 2015-11-09
# migration energy for strain cleanup
#####################
import os
import sys
import Vgetenergy
import simpleplot
import numpy as np
import hocking_lstsq
def report(mypath):
    """Report a migration barrier.
        Args:
            mypath <str>: Path for NEB subfolders. Take static energies.
    """
    mysubs = os.listdir(mypath)
    #if not '00' in mysubs:
    #    print "Does not appear to be an NEB path."
    #    return
    imlist=list()
    for imdir in mysubs:
        trypath = os.path.join(mypath, imdir)
        if os.path.isdir(trypath):
            #if imdir.isdigit():
            #    imlist.append(trypath)
            imlist.append(trypath)
    imlist.sort()
    energylist=list()
    actualenergies=list()
    for imdir in imlist:
        imenergy = Vgetenergy.main(imdir)
        if imenergy == None:
            energylist.append("?")
        else:
            energylist.append(float(imenergy))
            actualenergies.append(float(imenergy))
        #print imenergy
    einit = energylist[0]
    efin = energylist[-1]
    efinmininit = efin - einit
    emax = max(actualenergies)
    emin = min(actualenergies)
    barrfrominit = emax - einit
    barrfromfin = emax - efin
    maxminmin = emax - emin
    #print imlist
    #print energylist
    pattern=""
    for eidx in range(0, len(energylist)-1):
        currenergy = energylist[eidx]
        nextenergy = energylist[eidx + 1]
        if currenergy == "?":
            pattern = pattern + "N"
            currenergy = lastavailenergy
        else:
            pattern = pattern + "x"
            lastavailenergy = currenergy
        if nextenergy == "?":
            prelation="-?-"
        elif nextenergy > currenergy:
            prelation="-/-"
        elif nextenergy < currenergy:
            prelation="-\-"
        else:
            prelation="-=-"
        pattern = pattern + prelation
    if energylist[-1] == "?":
        pattern = pattern + "?"
    else:
        pattern = pattern + "x"
    retdict=dict()
    retdict['einit'] = einit
    retdict['efin'] = efin
    retdict['emax'] = emax
    retdict['efinmininit'] = efinmininit
    retdict['maxminmin'] = maxminmin
    retdict['barrfrominit'] = barrfrominit
    retdict['barrfromfin'] = barrfromfin
    retdict['pattern'] = pattern
    retdict['energies'] = energylist
    retdesc="init,fin,max,efinmininit; maxminmin; barrfrominit; barrfromfin; pattern; energies"
    retstr="einit: %3.3f,efin: %3.3f,emax: %3.3f,efinmininit: %3.3f,maxminmin: %3.3f,barrfrominit: %3.3f,barrfromfin: %3.3f,pattern: %s,energies" % (einit,
        efin, emax,
        efinmininit,maxminmin,barrfrominit,barrfromfin,pattern)
    for energy in energylist:
        if energy == "?":
            retstr = retstr + ",no_calc"
        else:
            retstr = retstr + ",%3.3f" % energy
    print retstr
    #print retdict
    return retdict

def get_dmeps(svec, barrvec, usemeV=1, label="", units="meV/percent strain"):
    """Get delta in migration energy per percent strain.
        Args:
            svec <list or np.ndarray of float>: percent strains
            barrvec <list or np.ndarray of float>: barrier energies in eV
            usemeV <int>: 1 to multiply by 1000 to use meV, 0 to leave alone.
            label <str>: optional label
    """
    if type(svec) == list:
        svec = np.array(svec, 'float')
    if type(barrvec) == list:
        barrvec = np.array(barrvec, 'float')    
    print "strainvec, barrvec: ", svec, barrvec
    simpleplot.plot(svec, barrvec, "%s Energy(Strain)" % label)
    [betamat, ssq, betaerr] = hocking_lstsq.least_squares_regression(svec, barrvec, 1)
    dmeps = betamat[1]
    dmepserr = betaerr[1]
    if usemeV == 1:
        dmeps = 1000*dmeps
        dmepserr = 1000*dmepserr
    print "DMEPS (%s), error: %3.3f, %3.3f" % (units,dmeps,dmepserr)
    return [dmeps, dmepserr]

def get_minimum_from_list(list_of_neb_report_dicts, keystring):
    """Get minimum value from a list of dictionaries
        Args:
            list_of_neb_report_dicts <list of dict>: list of neb report dictionaries
            keystring <str>: key name for the value to minimize, e.g. "emax"
                             or "barrfrominit"
        Returns:
            [minimum value, index in the list]
    """
    minval=""
    minidx=""
    ict=0
    for nebreport in list_of_neb_report_dicts:
        if not (keystring in nebreport.keys()):
            print "key %s not found in index %i" % (keystring, ict)
        else:
            myval = nebreport[keystring]
            if ((minval == "") or (myval < minval)):
                minval = myval
                minidx = ict
        ict=ict + 1
    return [minval, minidx]


def get_minimum_saddle(list_of_neb_report_dicts):
    """Get minimum saddle energy.
        Args:
            list_of_neb_report_dicts <list of dict>: list of neb report dictionaries
    """
    emaxes=list()
    for nebreport in list_of_neb_report_dicts:
        emaxes.append(nebreport['emax'])
    return min(emaxes)

def get_minimum_barrfrominit(list_of_neb_report_dicts):
    """Get minimum barrier from initial endpoint.
        Args:
            list_of_neb_report_dicts <list of dict>: list of neb report dictionaries
    """
    barrfrominits=list()
    for nebreport in list_of_neb_report_dicts:
        barrfrominits.append(nebreport['barrfrominit'])
    return min(barrfrominits)

def get_minimum_initial_endpoint(list_of_neb_report_dicts):
    """Get minimum initial endpoint energy.
        Args:
            list_of_neb_report_dicts <list of dict>: list of neb report dictionaries
    """
    einits=list()
    for nebreport in list_of_neb_report_dicts:
        einits.append(nebreport['einit'])
    return min(einits)


    

if __name__=="__main__":
    #report("/home/tam/strain_emig_clean/S4_migration_barriers/tgrid/1000_OLaTi_strnebpvperc_20131015T171344/neb_vac1-opv_q=p2_stat")
    report(sys.argv[1])
