#!/usr/bin/env python
####################
# Tam Mayeshiba 2015-11-19
# simple grep utility and some examples
#######################
import numpy as np
import subprocess
import os
def simplesubprocess(commandstr=""):
    """Simple subprocess utility
    """
    myproc = subprocess.Popen(commandstr,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
    myprocstr = myproc.communicate()[0]
    myproc.wait()
    return myprocstr

def grep_me(grepstr="",filename=""):
    """grep process
    """
    grepstr = simplesubprocess("grep %s %s" % (grepstr, filename))
    if grepstr == "":
        print "Empty grep return."
    return grepstr

def grep_return_last(grepstr="", filename=""):
    """grep and return last line
    """
    alllines = grep_me(grepstr, filename)
    allsplit = alllines.split("\n")
    if len(allsplit) == 0:
        print "No last line."
        return ""
    skipblank=1
    while (skipblank <= len(allsplit)):
        myline = allsplit[-1*skipblank]
        if myline == "":
            skipblank = skipblank + 1
        else:
            return myline
    print "All returns blank."
    return ""

def get_vasp_oszicar_energy(filename):
    """Get last E0 from OSZICAR
    """
    if os.path.isdir(filename):
        estring = grep_return_last("E0", os.path.join(filename, "OSZICAR"))
    else:
        estring = grep_return_last("E0",filename)
    if estring == "":
        print "Empty grep return."
        return "error"
    try:
        myenergy = float(estring.strip().split()[4])
    except (ValueError, IndexError):
        print estring
        return "error"
    return myenergy

def get_vasp_volume_of_cell(filename):
    """Get volume of cell from OUTCAR
    """
    if os.path.isdir(filename):
        estring = grep_return_last('"volume of cell"',os.path.join(filename, "OUTCAR"))
    else:
        estring = grep_return_last('"volume of cell"',filename)
    if estring == "":
        print "Empty grep return."
        return "error"
    try:
        myvol = float(estring.strip().split()[4])
    except (ValueError, IndexError):
        print estring
        return "error"
    return myvol

def get_vasp_pressure(filename):
    """Get pressure from OUTCAR in kbar
    """
    if os.path.isdir(filename):
        estring = grep_return_last('"pressure"',os.path.join(filename, "OUTCAR"))
    else:
        estring = grep_return_last('"pressure"',filename)
    if estring == "":
        print "Empty grep return."
        return "error"
    try:
        mypress = float(estring.strip().split()[3])
    except (ValueError, IndexError):
        print estring
        return "error"
    return mypress
