#!/usr/bin/env python
###############
# unit test helper scripts
# Tam Mayeshiba 2015-12-15
###############
import subprocess
import os
import sys
import time
import difflib

def generic_run(scriptrun="", scriptoutput="", overwrite=1):
    """Open a script using subprocess and output its results
        Args:
            scriptrun <str>: full script running command, including "python"
                                if necessary, e.g. "python ~/myscript.py"
            scriptoutput <str>: filename to which to output the results
            overwrite <int>: 1 - overwrite (default)
                             0 - append
    """
    mycmd=""
    if (int(overwrite) == 1):
        mycmd = "%s > %s" % (scriptrun, scriptoutput)
    else:
        mycmd = "%s >> %s" % (scriptrun, scriptoutput)
    myrun = subprocess.Popen(mycmd,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE)
    myrun.wait()
    myresults = myrun.communicate()
    return myresults

def last_X_lines(oldfilename="", newfilename="", xnum=10):
    """Get the last X lines of an old file into a new file
        Args:
            oldfilename <str>: Old file name
            newfilename <str>: New file name
            xnum <int>: Number of lines
    """
    myrunline = "tail -n %i %s" % (int(xnum), oldfilename)
    generic_run(myrunline, newfilename, 1)
    return

def is_same_as_standard(newfile="", standardfile=""):
    """Compare a new file to a standard file.
        Uses readlines(), so should not be used with
        very long files.
        Args:
            newfile <str>: Newly generated file's name
            standardfile <str>: Standard file's name
        Returns:
            True if identical, False otherwise
    """
    mydiff=difflib.Differ()
    with open(standardfile, 'rb') as sfile:
        s_data = sfile.readlines()
    with open(newfile, 'rb') as nfile:
        n_data = nfile.readlines()
    mycomp = list(mydiff.compare(s_data, n_data))
    mydiffs=0
    for myitem in mycomp:
        if myitem[0] in ['-','+','?']:
            print myitem
            mydiffs = mydiffs + 1
    if mydiffs > 0:
        return False
    return True
