"""Tests for strain scripts"""

import unittest
from unittest import SkipTest
import os
import time
import shutil
import subprocess
import testhelper
import numpy as np
import elastic_constants
testname="tscripts/tests"
print "************************CURDIR:", os.getcwd()

class TestElasticConstants(unittest.TestCase):
    def setUp(self):
        return
    
    def tearDown(self):
        return

    def test_bulkmod(self):
        pvec_kbar=[-30,-20,-10,0,10,20,30]
        pvec_kbar = np.array(pvec_kbar, 'float')
        vvec=[200,250,300,400,500,600,700]
        vvec = np.array(vvec, 'float')
        my_ec=elastic_constants.bulkmod(pvec_kbar, vvec, "test")
        bulkmod=my_ec[-3]
        derivbm=my_ec[-1]
        self.assertEqual("%3.3f" % bulkmod,"-41.663")
        self.assertEqual("%3.3f" % derivbm,"-0.480")

    def test_poisson_from_biaxial(self):
        amults=[0.98,0.99,1.00,1.01,1.02]
        cmults=[1.015,1.007,1.001,0.996,0.994]
        avec=np.array(amults,'float')
        cvec=np.array(cmults,'float')
        my_v=elastic_constants.poisson_from_biaxial(avec, cvec, "test")[0]
        self.assertEqual("%3.3f" % my_v, "0.209")
    def test_youngsmod(self):
        my_Y=elastic_constants.youngsmod(34,0,0.222,0)[0]
        self.assertEqual("%3.3f" % my_Y, "56.712")
    def test_strain_model_prefactor(self):
        my_prefactor=elastic_constants.strain_model_prefactor(0.321,0,85,0)[0]
        self.assertEqual("%3.3f" % my_prefactor, "-0.052")
    def test_strain_model_slope(self):
        my_slope=elastic_constants.strain_model_slope(-0.051,0,12.2,0)[0]
        self.assertEqual("%3.3f" % my_slope, "-6.222")
    def test_eval_bm_guess(self):
        my_eval=elastic_constants.evaluate_birch_murnaghan_volume_guess(73,
            0.25,25,200,125)
        self.assertEqual("%3.3f" % my_eval, "48.638")
    def test_get_vol_from_bm(self):
        my_vol=elastic_constants.get_volume_from_birch_murnaghan(73,
            0.25,25,400)
        self.assertEqual("%3.1f" % my_vol, "532.0")
