#!/usr/bin/env python
####################
# TTM 2015-09-011
# Cut-and-paste hocking least squares fit
#####################

import numpy as np
import sys

def get_vectors(totlist):
    xlist=list()
    ylist=list()
    for myitem in totlist:
        mysplit=myitem.split()
        try:
            xlist.append(mysplit[0].strip())
            ylist.append(mysplit[1].strip())
        except:
            print "Value ignored: %s" % myitem
    xvec=np.array(xlist,'float')
    yvec=np.array(ylist,'float')
    return [xvec,yvec]

def do_fit(xvec,yvec,fitorder):
    import hocking_lstsq
    myfit=hocking_lstsq.least_squares_regression(xvec,yvec,fitorder)
    return myfit

def get_min(xvec, myfit, step=0.01):
    xmin=min(xvec)
    xmax=max(xvec)
    myval=xmin
    intercept=0.0
    ax3=0.0
    bx2=0.0
    cx=0.0
    coeffs = myfit[0]
    if len(coeffs) > 0:
        intercept=coeffs[0]
    if len(coeffs) > 1:
        cx = coeffs[1]
    if len(coeffs) > 2:
        bx2 = coeffs[2]
    if len(coeffs) > 3:
        ax3 = coeffs[3]
    if len(coeffs) > 4:
        print "FIT OVER ORDER 3 NOT ALLOWED."
        return
    minevaluated=10000000000
    myvalformin=""
    while myval<=xmax:
        evaluated = ax3 * np.power(myval, 3) + bx2 * np.power(myval, 2) + cx * np.power(myval, 1) + intercept
        if evaluated < minevaluated:
            myvalformin = myval
            minevaluated = evaluated
        myval = myval + step
    print "Minimum evaluated value: %3.3f" % minevaluated
    print "Value for minimum evaluation: %3.3f" % myvalformin
    return myvalformin

def main(totlist=list(),fitorder=3):
    [xvec,yvec]=get_vectors(totlist)
    myfit=do_fit(xvec,yvec,int(fitorder))
    minval = get_min(xvec, myfit)
    print minval
    return

if __name__=="__main__":
    if len(sys.argv) > 1:
        fitorder=sys.argv[1]
    else:
        fitorder=3
    print "Cut and paste the x, y values for fitting here."
    totlist=list()
    mylist="initialize"
    while (not mylist==""):
        mylist=raw_input("prompt: ")
        if not mylist=="":
            totlist.append(mylist)
    main(totlist, fitorder)
