#!/usr/bin/env python
##
#@package   Ldirautorelaxsilent
#@ingroup   vasp
#@ingroup   python
#@brief     loop over top directory to run Pdirautorelax in the background
#@author    Tam Mayeshiba
#@date      1/5/11 copied from Lqautorelaxsilent
#@date      2/16/11 add email address parameter
#@date      5/4/11 update header to use autodoc
###
def main(toppath, numatoms=1, maxout=5, checksec=1800, checklag=1800, 
            encomp=0.001, entype='T', volcomp=1, voltype='P', emaddr=''):
    """
    Purpose: loop over top directory to run Pqautorelax silently in the background
       inputs:  
	        [$1] top path of folder of job to submit
                example: '/home/tam/rocksalt_rel1/'
            [$2] number of atoms in system - default 1
                between 1 and 10000, inclusive
            [$3] maximum number of relaxations to do - default 5
                between 1 and 100, inclusive
            [$4] second between checks - default 1800
            [$5] acceptable number of seconds before job considered hung
                    - default 1800
            [$4] energy comparison number - default 0.001
            [$5] energy comparison type - default 'T' for threshhold
            [$6] volume comparison number - default 1
            [$7] volume comparison type - default 'P' for percent
            [$8] email address for error notification
       outputs:
            True - job has finished
            None - error
    """
#   Modules:
    import os
    import sys
    import subprocess
    import Xtimestring
    import Xuniquename
    import Xvaliddirectory
    import Pdirautorelaxsilent
#   validation
#   #   top directory
    toppath = Xvaliddirectory.main(toppath)
    if (toppath == None):
        print '(Exiting.)'
        return None
#   #   other validation will be handled by Pdirautorelax
#   walk through
    walkme=""
    walkdir=""
    walkct=0
    returnval=""
    walkme = os.walk(toppath)
    if walkme == ():
        print 'No folders found.'
    for walkentry in walkme:
        if walkct == 0:
            print 'Ignoring top directory, ', walkentry[0]
        else:
            walkdir = walkentry[0]
            returnval = Pdirautorelaxsilent.main(walkdir, numatoms, maxout, 
                        checksec, checklag, encomp, entype, volcomp, 
                        voltype, emaddr)
            print returnval
        walkct = walkct + 1
    print 'Done.'
