#!/usr/bin/env python
##
#@package   Pavgbondang
#@ingroup   vasp
#@ingroup   python
#@brief     gets average bond angles for certain type of bond
#@author    Tam Mayeshiba
#@date      4/19/11 created
#@date      7/14/11 added looping function
###
def main(dirname, whichset1, toset, whichset2, fname='CONTCAR', dist=3,
            expected=160, plusmin=30):
    """
    Purpose: gets average bond angles for certain type of bond
        inputs:  
            [$1] directory
            example: /home/tam/NaCl
            [$2] period delimited string of which atoms to start with
            [$3] period delimited string of which atoms can be in the middle
            [$4] period delimited string of which number atoms to end with
            [$5] file name - default CONTCAR
            [$6] distance in Angstroms - default 3
            So for a perovskite , it would look like
            Uwrapper.py Pavgbondang /home/tam/myperov 9.10.11.12.13.14.15.16
              17.18.19.20.21.22...etc... 9.10.11.12.13.14.15.16
            [$7] expected angle - default 160
            [$8] tolerance (positive number for range on either side) 
                 - default 30
                 angles that fall outside of the expected angle and tolerance
                 range will not be counted in the average
        outputs:
        notes: should probably update this to use regular expressions
                module re to pattern match
    """
#   Modules:
    import os
    import sys
    import subprocess
    import Xvaliddirectory
    import Xvalidinteger
    import Xuniquename
#   Validation
#   #   dirname
    dirname = Xvaliddirectory.main(dirname)
    if (dirname == None):
        print '(Exiting.)'
        return None
#   #   start
    if type(whichset1) != str:
        print whichset1, 'should be a string.'
        print '(Exiting.)'
        return None
#   #   middle
    if type(toset) != str:
        print toset, 'should be a string.'
        print '(Exiting.)'
        return None
#   #   end
    if type(whichset2) != str:
        print whichset2, 'should be a string.'
        print '(Exiting.)'
        return None
#   #   fname
    if not os.path.exists(dirname + '/' + fname):
        print fname, 'does not exist in', dirname
        print '(Exiting.)'
        return None
#   #   dist
    distflt=0
    try:
        distflt = float(dist)
    except (ValueError, TypeError):
        print dist, 'should be able to be converted into a float.'
        print '(Exiting.)'
        return None
#   #   expected
    expected = Xvalidinteger.main(expected, 0, 360)
    if (expected == None):
        print '(Exiting.)'
        return None
#   #   plusmin
    plusmin = Xvalidinteger.main(plusmin, 0, 360)
    if (plusmin == None):
        print '(Exiting.)'
        return None
#   get unique filename
    writeto=""
    writeto = Xuniquename.main(dirname, fname + '_dist_', 'f')
    if (writeto == None):
        print 'Error creating unique file for distances.'
        print '(Exiting.)'
        return None
#   run convasp
    oldwd=""
    oldwd = os.getcwd()
    os.chdir(dirname)
    doconvangle=""
    convstr=""
    convstr = 'nice -n 19 convasp -angle '
    convstr = convstr + str(distflt) + ' < ' + fname + ' >> '
    convstr = convstr + writeto
    openwrite = open(writeto, 'ab')
    openwrite.write(convstr)
    openwrite.close()
    doconvangle = subprocess.Popen([convstr], shell=True)
    doconvangle.wait()
    os.chdir(oldwd)
#   parse strings
    startlist=[]
    midlist=[]
    endlist=[]
    startlist = whichset1.split('.')
    midlist = toset.split('.')
    endlist = whichset2.split('.')
    listitem=""
    for listitem in startlist:
        if not listitem.isdigit():
            print listitem, 'in starting list is not a number.'
            print '(Exiting.)'
            return None
    for listitem in midlist:
        if not listitem.isdigit():
            print listitem, 'in middle list is not a number.'
            print '(Exiting.)'
            return None
    for listitem in endlist:
        if not listitem.isdigit():
            print listitem, 'in ending list is not a number.'
            print '(Exiting.)'
            return None
#   parse data
    fullpath = dirname + '/' + writeto
    openread = open(fullpath, 'rb')
    readline=""
    readlist=[]
    startatom=""
    midatom=""
    endatom=""
    angle=""
    anglelist=[]
    readmid=""
    readend=""
    readmlist=[]
    readelist=[]
    elen=0
    readline = openread.readline()  #gets the header line we put in
    bondlist=[]
    bondstr=""
    #pdb.set_trace()
    while (readline):
        #pdb.set_trace()
        print readline
        readlist = readline.split() #split by spaces
        if (readlist == None):
            readline = openread.readline() #  problem here. Go to next line.
        elif len(readlist) < 1:
            readline = openread.readline() #  empty list for some reason
        else:
            if (readline[0].isdigit()):
                startatom = readlist[0]
                midatom = 0 #reset mid atom
                endatom = 0 #reset end atom
            elif (readline[5].isdigit()):
                midatom = readlist[0]
                endatom = 0 #reset end atom
            else:
                endatom = readlist[0]
            if (startatom in startlist):
                if (midatom in midlist):
                    if (endatom in endlist) and (endatom != startatom):
                        bondstr = startatom + ' ' + midatom + ' ' + endatom
                        bondlist.append(bondstr)
                        angle = readlist[len(readlist) - 1]
                        anglelist.append(angle)
        readline = openread.readline()
    openread.close()
#   #   calculate average
    myangle=""
    anglflt=0
    anglfltsum=0
    anglbad=0
    anglmin = expected - plusmin
    anglmax = expected + plusmin
    anglct=0
    anglavg=0
    badanglct=0
    for myangle in anglelist:
        anglbad = 0
        try:
            anglflt = float(myangle)
        except (ValueError, TypeError):
            print 'Error converting value', myangle, 'to a float. Skipping.'
            anglbad = 1
            badanglct = badanglct + 1
        if (anglbad == 0):
            if (anglflt <= anglmax) and (anglflt >= anglmin):
                anglfltsum = anglfltsum + anglflt
                anglct = anglct + 1
            else:
                badanglct = badanglct + 1
    if (anglct == 0):
        print 'No appropriate angles found.'
        return None
    anglavg = anglfltsum / anglct
#   format return list
    returnlist=[]
    lenlist=0
    lenlist = len(anglelist)
    listct=0
    returnstr=""
    returnstr = str(anglct) + ' angles counted, ' 
    returnstr = returnstr + str(badanglct) + ' skipped. Average: '
    returnstr = returnstr + str(anglavg)
    returnlist.append(returnstr)
    returnstr = 'start: ' + whichset1 + ' middle: ' + toset 
    returnstr = returnstr + ' end: ' + whichset2
    returnlist.append(returnstr)
    returnstr = 'Angles within ' + str(expected) + ' +/- ' + str(plusmin)
    returnstr = returnstr + ' degrees counted.'
    returnlist.append(returnstr)
    while (listct < lenlist):
        returnstr = bondlist[listct] + ': ' + anglelist[listct]
        returnlist.append(returnstr)
        listct = listct + 1
    return returnlist
#
#
def loop(topdir, whichset1, toset, whichset2, fname='CONTCAR', dist=3,
            expected=160, plusmin=30):
    """
    Purpose: gets average bond angles for certain type of bond
        inputs:  
            [$1] top directory
            example: /home/tam/rocksalts
            [$2] period delimited string of which atoms to start with
            [$3] period delimited string of which atoms can be in the middle
            [$4] period delimited string of which number atoms to end with
            [$5] file name - default CONTCAR
            [$6] distance in Angstroms - default 3
            So for a perovskite , it would look like
            Uwrapper.py Pavgbondang /home/tam/myperov 9.10.11.12.13.14.15.16
              17.18.19.20.21.22...etc... 9.10.11.12.13.14.15.16
            [$7] expected angle - default 160
            [$8] tolerance (positive number for range on either side) 
                 - default 30
                 angles that fall outside of the expected angle and tolerance
                 range will not be counted in the average
        outputs:
        notes: should probably update this to use regular expressions
                module re to pattern match
    """
#   Modules:
    import os
    import sys
    import subprocess
    import Pavgbondang
    import Xgetdirs
    import Pmakemonfile
    import Xvaliddirectory
    import Xvalidinteger
    import Xuniquename
#   Validation
#   #   dirname
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   start
    if type(whichset1) != str:
        print whichset1, 'should be a string.'
        print '(Exiting.)'
        return None
#   #   middle
    if type(toset) != str:
        print toset, 'should be a string.'
        print '(Exiting.)'
        return None
#   #   end
    if type(whichset2) != str:
        print whichset2, 'should be a string.'
        print '(Exiting.)'
        return None
#   #   dist
    distflt=0
    try:
        distflt = float(dist)
    except (ValueError, TypeError):
        print dist, 'should be able to be converted into a float.'
        print '(Exiting.)'
        return None
#   #   expected
    expected = Xvalidinteger.main(expected, 0, 360)
    if (expected == None):
        print '(Exiting.)'
        return None
#   #   plusmin
    plusmin = Xvalidinteger.main(plusmin, 0, 360)
    if (plusmin == None):
        print '(Exiting.)'
        return None
#   get directories
    dirlist=[]
    onedir=""
    onedirres=""
    dirlist = Xgetdirs.main(topdir, 1, 5)
    if (dirlist == None):
        print 'Error getting subdirectories for', topdir
        print '(Exiting.)'
        return None
    if len(dirlist) == 0:
        print 'No subdirectories found for', topdir
        print '(Exiting.)'
        return None
    myfilename=""
    myfilename = Pmakemonfile.main(topdir)
    if (myfilename == None):
        print 'Could not create unique file name for writing.'
        print '(Exiting.)'
        return None
    myfile=""
    myfile = open(myfilename, 'ab')
    for onedir in dirlist:
        onedirres = Pavgbondang.main(onedir, whichset1, toset,
                    whichset2, fname, dist, expected, plusmin)
        myfile.write(onedir + ': ' + str(onedirres) + '\n')
    myfile.close()
    return True
