#!/usr/bin/env python
##
#@package   Pcompare
#@ingroup   vasp
#@ingroup   python
#@brief     compares directories (not useful any more?)
#@author    Tam Mayeshiba
#@date      11/12/2010 created
#@date      12/22/10 change Pgetline call to reflect new inputs
###
def main(existdir1, existdir2, fname, psearch, pline='X', pgetlast='0',
        firstch=1, lastch=0, thresh=0, divisor=1):
    """
    Purpose: evaluates two directories for comparability
             if they are comparable, executes Pgetline for each.
             compares identical character ranges on a line according
             to user input for the range, an optional divisor, 
             and a threshhold value.
             Comparison is numeric.
        inputs:  
            [$1] first directory containing the file
                 example: /home/tam/NaCl/
            [$2] second directory containing the file
                 example: /home/tam/NaClr/
            [$3] file name (just the name); should be identical in 
                 each folder
                 example: myoutputfile
            [$4] Pgetline search string (or 0 to use line number)
            [$5] Pgetline line number (or X to use search string)
            [$6] Pgetline search first or last occurrence of string (1 or 0)
            [$7] character position (integer) of first character in range 
                 (if you vi an output file  beforehand, you can get the 
                 character range from the bottom right-hand of the screen) 
                 for comparison (line starts at 1)
            [$8] character position (integer) of last character in range 
                 for comparison
                 Scientific notation will be handled.
            [$9] threshhold (optional) - number within which numerical
                 values should match (ex: 0.01)
                 Please don't use commas.
                 use 0 for exact match (default)
                 if line segments are non-numerical, exact match used
            [$10] divisor (optional) - default 1
                 if line segments are non-numerical, ignored
        returns:
            True - if comparison is within threshhold
            False - if comparison is not within threshhold
            None - if some error has occurred
    """
#   Modules:
    import os
    import sys
    import shutil
    import Pgetline
#   
    fullf1=""
    fullf2=""
#   validate inputs
#   #   validate existing directory path 1
    try:
        existdir1 = str(existdir1)
    except (ValueError, TypeError):
        print existdir1, 'should be a directory like "//home/flast/mydir"'
        print '(Exiting.)'
        return None
    if (existdir1 == ""):
        print 'Directory was blank. If this is intentional, please try again'
        print 'using //home/<username>.'
        print '(Exiting.)'
        return None
    if (os.path.exists(existdir1))==False:
        print existdir1, 'is not a valid path.'
        print '(Exiting.)'
        return None
    if (os.path.isfile(existdir1)):
        print existdir1, 'is a file, not a directory.'
        print '(Exiting.)'
        return None
    else:
        existdir1 = os.path.normpath(existdir1)  # normpath may break in Windows
#   #   validate existing directory path 2
    try:
        existdir2 = str(existdir2)
    except (ValueError, TypeError):
        print existdir2, 'should be a directory like "//home/flast/mydir"'
        print '(Exiting.)'
        return None
    if (existdir2 == ""):
        print 'Directory was blank. If this is intentional, please try again'
        print 'using //home/<username>.'
        print '(Exiting.)'
        return None
    if (os.path.exists(existdir2))==False:
        print existdir2, 'is not a valid path.'
        print '(Exiting.)'
        return None
    if (os.path.isfile(existdir2)):
        print existdir2, 'is a file, not a directory.'
        print '(Exiting.)'
        return None
    else:
        existdir2 = os.path.normpath(existdir2) # normpath may break in Windows
#   #   validation of Psearch, Pline, Pgetlast will be handled by Pgetline
#   #   validate firstch
    try:
        firstch = int(firstch)
    except (ValueError, TypeError):
        print firstch, ' should be a number for the starting comparison positon.'
        print '(Exiting.)'
        return None
    firstch = firstch - 1 
#   #   # characters in vi start at position 1, but
#   #   # python strings start numbering at 0
    if firstch < 0:
        print firstch + 1, 'should be positive.'
        print '(Exiting.)'
        return None
#   #   validate lastch
    try:
        lastch = int(lastch)
    except (ValueError, TypeError):
        print lastch, 'should be a number for the ending comparison positon.'
        print '(Exiting.)'
        return None
#   #   # not subtracting here, because 1 to 5 should get 5 chars.
    if lastch < 0:
        print lastch, 'should be positive.'
        print '(Exiting.)'
        return None
#   #   validate thresh
    try:
        thresh = float(thresh)
    except (ValueError, TypeError):
        print thresh, 'should be a number for the comparison threshhold,'
        print 'e.g. 0.01 for 0.01 Angstrom^3 threshhold.'
        print 'Use 0 for an exact match.'
        print '(Exiting.)'
        return None
    if thresh < 0:
        print thresh, 'entered. Making threshhold value positive.'
        thresh = -1 * thresh
#   #   validate divisor
    try:
        divisor = float(divisor)
    except (ValueError, TypeError):
        print divisor, '  should be a number for dividing before comparing,'
        print 'e.g. 40 to divide the volume by 40 atoms.'
        print 'Use 1 to keep the number as is.'
        print '(Exiting.)'
        return None
    if divisor == 0:
        print 'Divisor cannot be zero.'
        print '(Exiting.)'
        return None
#   #   run script for each directory
    fullf1 = existdir1 + '/' + fname
    fullf2 = existdir2 + '/' + fname
    findln1=""
    findln2=""
    findln1 = Pgetline.main(existdir1, fname, psearch, pline, pgetlast)
    findln2 = Pgetline.main(existdir2, fname, psearch, pline, pgetlast)
#   #   one line or another not found
    if (findln1 == None) or (findln2 == None):
        print 'One of the lines was not found.'
        print '(Exiting.)'
        return None
#   get line segement chunks
    seg1=""
    seg2=""
#   #   comparison range improperly defined, take all after firstch
    if lastch <= firstch:
        seg1 = findln1[firstch:]
        seg2 = findln2[firstch:]
    else:
        seg1 = findln1[firstch:lastch]
        seg2 = findln2[firstch:lastch]
#   #   check line segments for numericity
    seg1float=""
    seg1floatok=1
    seg2float=""
    seg2floatok=1
    numcompare=1
    try:
        seg1float = float(seg1)
    except (ValueError, TypeError):
        seg1floatok=0
    try:
        seg2float = float(seg2)
    except (ValueError, TypeError):
        seg2floatok=0
#   #   #  if one can be a float and the other can't, can't be matching        
    if (seg1floatok != seg2floatok):
        return False
    if (seg1floatok == 0) and (seg2floatok == 0):
        print fullf1,': ',seg1
        print fullf2,': ',seg2
        return (seg1 == seg2)
#   #   #   otherwise compare floats
    seg1float = seg1float / divisor
    seg2float = seg2float / divisor
    print fullf1,': ',seg1float
    print fullf2,': ',seg2float
    if abs(seg1float - seg2float) <= thresh:
        return True
    else:
        return False
