#!/usr/bin/env python
##
#@package   Pcompseg   
#@ingroup   other
#@ingroup   python
#@brief     compares line segments according to range, divisor, threshhold
#@note      additional code notes
#@author    Tam Mayeshiba
#@date      12/3/10 created
#@date      12/22/10 removed unncessary Pgetline tag
###
def main(seg1, seg2, howcomp='t', thresh=0, divisor=1, percent=10):
    """
    Purpose: compares line segments according
             to user input for the range, an optional divisor, 
             and a threshhold value.
             Comparison is numeric where possible, and scientific notation
             is handled thanks to python.
        inputs:  
            [$1] first line segment
            [$2] second line segment
                 Scientific notation will be handled.
            [$3] T - compare using threshhold (default)
                 P - compare using percentage
            [$4] threshhold (optional) - number within which numerical
                 values should match (ex: 0.01)
                 Please don't use commas.
                 use 0 for exact match (default)
                 if line segments are non-numerical, exact match used
            [$5] divisor (optional) - default 1
                 if line segments are non-numerical, ignored
            [$6] percent (optional) - default 10
                 absolute value of percent change to tolerate
                 this is 100*|x2-x1|/x1
                 if line segments are non-numerical, exact match used
        returns:
            True - if comparison is within threshhold or percentage (abs val)
            False - if comparison is not within threshhold or percentage
            None - if some error has occurred
    """
#   Modules:
    import os
    import sys
    import Pgetseg
    import Xvalidletter
#   
#   validate inputs
#   #   validate strings
    if (type(seg1) != str):
        print seg1, 'should be a string.'
        print '(Exiting.)'
        return None
    if (type(seg2) != str):
        print seg2, 'should be a string.'
        print '(Exiting.)'
        return None
#   #   validate toggle
    howcomp = Xvalidletter.main(howcomp, ['t','p'], 1)
    if (howcomp == None):
        print 'Use T for threshold compare or P for percent',
        print 'change compare.'
        print '(Exiting.)'
        return None
#   #   validate thresh
    if (howcomp == 't'):
        try:
            thresh = float(thresh)
        except (ValueError, TypeError):
            print thresh, 'should be a number for the comparison threshhold,'
            print 'e.g. 0.01 for 0.01 Angstrom^3 threshhold.'
            print 'Use 0 for an exact match.'
            print '(Exiting.)'
            return None
        if thresh < 0:
            print thresh, 'entered. Making threshhold value positive.'
            thresh = -1 * thresh
#   #   validate divisor
    try:
        divisor = float(divisor)
    except (ValueError, TypeError):
        print divisor, '  should be a number for dividing before comparing,'
        print 'e.g. 40 to divide the volume by 40 atoms.'
        print 'Use 1 to keep the number as is.'
        print '(Exiting.)'
        return None
    if divisor == 0:
        print 'Divisor cannot be zero.'
        print '(Exiting.)'
        return None
#   #   validate percentage
    if (howcomp == 'p'):
        try:
            percent = float(percent)
        except (ValueError, TypeError):
            print percent, '  should be a number for percentage change.'
            print '(Exiting.)'
            return None
        if percent < 0:
            print percent, 'entered. Absolute percentage change will be used.'
            percent = -1 * percent
#   #   check line segments for numericity
    seg1float=""
    seg1floatok=1
    seg2float=""
    seg2floatok=1
    numcompare=1
    try:
        seg1float = float(seg1)
    except (ValueError, TypeError):
        seg1floatok=0
    try:
        seg2float = float(seg2)
    except (ValueError, TypeError):
        seg2floatok=0
#   #   #  if one can be a float and the other can't, can't be matching        
    if (seg1floatok != seg2floatok):
        return False
    if (seg1floatok == 0) and (seg2floatok == 0):
        return (seg1 == seg2)
#   #   #   otherwise compare floats
    seg1float = seg1float / divisor
    seg2float = seg2float / divisor
    percentchange = 0
    if (howcomp == 't'):
        if abs(seg1float - seg2float) <= thresh:
            return True
        else:
            return False
    elif (howcomp == 'p'):
        percentchange = 100 * (seg2float - seg1float) / seg1float
        percentchange = abs(percentchange)
        if (percentchange <= percent):
            return True
        else:
            return False
    else:
        return None
