#!/usr/bin/env python
##
#@package   Pcopyout 
#@ingroup   other
#@ingroup   python
#@brief     copies one file to multiple directories
#@author    Tam Mayeshiba
#@date      8/2/11
###
def main(existdir, filetocopy, minlev=1, maxlev=1, overwrite=0):
    """
    Purpose:
        inputs:  
            [$1] directory housing directories within which to copy file
            example: /home/tam/rocksalts
            [$2] file name to copy (full path or in current path)
            [$3] minimum levels to look for folders - default 1
            [$4] maximum levels deep to look for folders - default 1
            [$5] overwrite if file already exists
                    No - 0 (default)
                    Yes - 1
        outputs:
            None if error
            True if successful
    """
#   Modules:
    import os
    import shutil
    import sys
    import Xgetdirs
    import Xvaliddirectory
    import Xvalidinteger
#   Validation
#   #  directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   file
    try:
        filetocopy = str(filetocopy)
    except (ValueError, TypeError):
        print filetocopy, 'must be a string.'
        print '(Exiting.)'
        return None
    cwdfile=""
    homefile=""
    if not os.path.isfile(filetocopy):
        cwdfile = os.path.join(os.getcwd(), filetocopy)
        if os.path.isfile(cwdfile):
            filetocopy = cwdfile
        else:
            homefile = os.path.join(os.environ["HOME"], filetocopy)
            if os.path.isfile(homefile):
                filetocopy = homefile
            else:
                print filetocopy, 'does not exist as is or'
                print 'in home or current directory.'
                print '(Exiting.)'
                return None
#   #   min levels
#   #   #    will be handled by Xgetdirs
#   #   max levels
#   #   #   will be handled by Xgetdirs
#   #   overwrite
    overwrite = Xvalidinteger.main(overwrite, 0, 1)
    if (overwrite == None):
        print '(Exiting.)'
        return None
#   
    dirlist=[]
    mydir=""
    dirlist = Xgetdirs.main(existdir, minlev, maxlev)
    if (dirlist == None):
        print 'Error retrieving directories.'
        print '(Exiting.)'
        return None
    if len(dirlist) == 0:
        print 'No directories found.'
        print '(Exiting.)'
        return None
    #print dirlist
#   copy
    fileshort=""
    filecheck=""
    fileshort = os.path.basename(filetocopy)
    for mydir in dirlist:
        if (overwrite == 0):
            filecheck = os.path.join(mydir, fileshort)
            if os.path.isfile(filecheck):
                print filecheck, 'exists. Skipping copy.'
            else:
                shutil.copy(filetocopy, mydir)
                print 'Copied to', mydir
        else:
            shutil.copy(filetocopy, mydir)
            print 'Copied to', mydir
    return True

