#!/usr/bin/env python
##
#@package   Pdirautorelaxwait   
#@ingroup   vasp
#@ingroup   python
#@brief     run Pdirautorelax in the background, but wait for it
#@details   used within other programs - not useful from command line
#@author    Tam Mayeshiba
#@date      3/23/11 copied from Pdirautorelaxsilent
#@date      5/18/11 flush buffer occasionally to prevent jobs whose
#                   monfile and OUTCAR update at the same time from
#                   being considered hung.
###
def main(fullpath, numatoms=1, maxout=5, checksec=1800, checklag=1800, 
            encomp=0.001, entype='T', volcomp=1, voltype='P', emaddr=''):
    """
    Purpose: run Pdirautorelax silently in the background
       inputs:  
	        [$1] full path of folder of job to submit
                example: '/home/tam/rocksalt_rel1/NaClep1'
            [$2] number of atoms in system - default 1
                between 1 and 10000, inclusive
            [$3] maximum number of relaxations to do - default 5
                between 1 and 100, inclusive
            [$4] second between checks - default 1800
            [$5] acceptable number of seconds before job considered hung
                    - default 1800
            [$4] energy comparison number - default 0.001
            [$5] energy comparison type - default 'T' for threshhold
            [$6] volume comparison number - default 1
            [$7] volume comparison type - default 'P' for percent
            [$8] email address
       outputs:
            True - job has finished
            None - error
    """
#   Modules:
    import os
    import sys
    import subprocess
    import time
    import Xtimestring
    import Xuniquename
    import Pmakemonfile
#   validation will be handled by Pqautorelax
#   convert inputs into strings
    try:
        fullpath = str(fullpath)
    except (TypeError, ValueError):
        print fullpath, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        numatoms = str(numatoms)
    except (TypeError, ValueError):
        print numatoms, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        maxout = str(maxout)
    except (TypeError, ValueError):
        print maxout, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        checksec = str(checksec)
    except (TypeError, ValueError):
        print checksec, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        checklag = str(checklag)
    except (TypeError, ValueError):
        print checklag, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        encomp = str(encomp)
    except (TypeError, ValueError):
        print encomp, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        entype = str(entype)
    except (TypeError, ValueError):
        print entype, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        volcomp = str(volcomp)
    except (TypeError, ValueError):
        print volcomp, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        voltype = str(voltype)
    except (TypeError, ValueError):
        print voltype, 'not valid.'
        print '(Exiting.)'
        return None
#   set up output file
    filestr=""
    fileme=""
    filestr = Pmakemonfile.main(fullpath)
    if (filestr == None):
        print 'Could not create monitor file.'
        print '(Exiting.)'
        return None
    fileme = open(filestr, 'ab')
#   run process
    hidemon=""
    hidemon = subprocess.Popen(['Uwrapper.py', 'Pdirautorelax',
            fullpath, numatoms, maxout, checksec, checklag, 
            encomp, entype, volcomp, voltype, emaddr],
            stdout=fileme, stderr=fileme)
    print hidemon
#   wait for process, and update files per checksec
    hidemon.wait()
    #procdone=""
    #checkflt=0.0
    #try:
    #    checkflt = float(checksec)
    #except (ValueError, TypeError):
    #    print checksec, 'could not be converted to a float.'
    #    print '(Exiting.)'
    #    return None
    #procdone = None
    #while (procdone == None):
    #    print hidemon.poll()
    #    procdone = hidemon.poll() # wait for it
    #    time.sleep(checkflt)
    #    sys.stdout.flush()
    #    #fileme.flush()
    #    #os.fsync(fileme.fileno())
#    hidemon = subprocess.Popen([sys.executable,
#            'Uwrapper.py', 'Pqautorelax',
#            fullpath, numatoms, maxout, checksec, checklag, 
#            encomp, entype, volcomp, voltype],
#            stdout=fileme, stderr=fileme)
    fileme.close()
    return True
