#!/usr/bin/env python
##
#@package   Pdirmonitor
#@ingroup   vasp
#@ingroup   python
#@brief     monitor a job, but decoupled completely from job id
#@details   relies only on whether files are present and updating, and
#               on whether Total CPU time has been reached.
#@author    Tam Mayeshiba
#@date      1/5/11 copied from Pqueuemonitor and modified
###
def main(rundir, checksec=10, lagsec=1800):
    """
    Purpose: check to see if job is still running, repeatedly
       inputs:  
            [$1] run directory
                example: '/home/tam/NaCl1'
            [$2] number of seconds between checks - default for testing 10
                range between 10 and 100000 (about 1 day)
            [$3] number of seconds of acceptable lag between file
                updates (checks for hung jobs) - default 1800 (half an hour)
                range between 10 and 100000 (about 1 day)
       outputs:
            True - job has finished
            False - job was not on queue to begin with
            None - error
    """
#   Modules:
    import os
    import sys
    import time
    import subprocess
    import Xvaliddirectory
    import Xvalidinteger
    import Xtimestring
    import Vstillupdating
    import Vjobstarted
    import Vgetstatus
#   
#   validate
#   #   validate run directory
    rundir = Xvaliddirectory.main(rundir)
    if (rundir == None):
        print '(Exiting.)'
        return None
#   #   validate checksec
    checksec = Xvalidinteger.main(checksec, 10, 86400)
    if (checksec == None):
        print 'Seconds between check should be between 10 seconds'
        print 'and 1 day (86400 seconds).'
        print '(Exiting.)'
        return None
#   #   validate lagsec
    lagsec = Xvalidinteger.main(lagsec, 10, None)
    if (lagsec == None):
        print 'Seconds after which files must be updated or'
        print 'job will be considered hung should be at least 10 seconds.'
        print '(Exiting.)'
        return None
#   #
    started=False
    print 'Directory: ', rundir
#   #   has job started yet?
    started = Vjobstarted.main(rundir)
    while (started == False):
        time.sleep(checksec)
        started = Vjobstarted.main(rundir)
    if (started == None):
        print 'Error figuring out if job in', rundir, 'has started.'
        print '(Exiting.)'
        return None
#   #   is job continuing to update? has it finished yet?
    updating=True
    totaltime=None
    while updating and (totaltime == None):
        updating = Vstillupdating.main(rundir, lagsec)
        totaltime = Vgetstatus.main(rundir)
        if (updating and (totaltime == None)):
            time.sleep(checksec)
            print Xtimestring.main(), 'check'
    if (updating == None):
        print 'Error seeing if job in', rundir, 'is updating.'
        print '(Exiting.)'
        return None
    if (totaltime != None):
        print 'Total time: ', totaltime
    if (updating == False):
        print Xtimestring.main(), 'Job in ', rundir, 'seems to have hung.'
        return None
    return True
