#!/usr/bin/env python
##
#@package   Pdist1
#@ingroup   vasp
#@ingroup   python
#@brief     calculates absolute distance to a fixed coordinate
#@author    Tam Mayeshiba
#@date      2/20/12 looping wrapper to pathcalcs
###
def main(topdir, atom1=0, coord1="", adj="", fname='CONTCAR'):
    """
    Purpose:
        inputs:  
            [$1] neb directory
            example: /home/tam/NaClneb/staticim
            example: /home/tam/NaClneb
            [$2] atom position in POSCAR to search for
            [$3] fixed coordinate
            [$4] adjustment (if over or under this value, go to other side)
                example:"xlow, xhigh, ylow, yhigh, zlow, zhigh"
                "0.25,1,0,1,0,1"
            [$4] file name to use (default 'POSCAR')
        outputs:
            fractional path length
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Xvalidinteger
    import Pgetline
    import Upathcalcs
    import Xvector
    import Pfractocart
    import Platticevec
#   Validation
#   #  
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
    atomn1 = Xvalidinteger.main(atom1,1,None)
    if atomn1 == None:
        return None
   
    cartcoords=[]
    mydist=0
    mydistvec=[]
    av1=[]
    av2=[]
    lvecs = Platticevec.main(topdir, fname)
    cartcoords = Pfractocart.main(topdir, fname, adj)
    if not cartcoords == None:
        av1 = cartcoords[atomn1-1] #indexing starts at 0
        av2 = Xvector.frctocart(lvecs[0],lvecs[1],lvecs[2],Xvector.strtoflt(coord1))
        print av1
        print av2
        mydistvec = Xvector.add(av1,av2,1)
        # actually subtracts
        print mydistvec
        mydist = Xvector.magnitude(mydistvec)
    return mydist
#
def loop(loopdir, atomline1=0, fixedcoord1="", adj='', fname='CONTCAR', 
            filter='*bulk_static'):
    """
    Purpose:
        inputs:  
            [$1] top neb directory
            example: /home/tam/allnebs
            [$2] atomic position in file to use (VASP 5 files only)
            [$3] fixed coordinate, example: "0.5 0.1 0.1"
            [$4] adjustment (if over or under this value, go to other side)
                example:"xlow, xhigh, ylow, yhigh, zlow, zhigh"
                "0.25,1,0,1,0,1"
            [$3] file name to use (default 'POSCAR', since stat im eps have no CONTCAR)
            [$4] '*neb_static': folders marked 'neb_static' only
        outputs:
            difference, and trend string
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Pdist1
#   Validation
#   #  
    loopdir = Xvaliddirectory.main(loopdir)
    if (loopdir == None):
        print '(Exiting.)'
        return None
#   #   comparison threshold will be validated by Pminmax
    loopdirlist=[]
    loopdirlist = Xgetdirs.main(loopdir, 1, 5, filter)
    if (loopdirlist == None):
        print 'Error getting directory list.'
        print '(Exiting.)'
        return None
    loopdiritem=""
    myreturn=""
    returnlist=[]
    #print loopdirlist
    for loopdiritem in loopdirlist:
        #print loopdiritem
        myreturn = Pdist1.main(loopdiritem, atomline1, fixedcoord1, 
            adj, fname)
        if not myreturn == None:
            returnlist.append(loopdiritem + ': ' + str(myreturn))
    print loopdir, ': Distance from atom ',atomline1,' to ',fixedcoord1
    for myreturn in returnlist:
        print myreturn
    return returnlist
