#!/usr/bin/env python
##
#@package   Pdist2files
#@ingroup   vasp
#@ingroup   python
#@brief     calculates absolute distance between atoms in two files
#@author    Tam Mayeshiba
#@date      2/20/12 looping wrapper to pathcalcs
###
def main(topdir, dir1, dir2, atom1=0, atom2=0, adj="", fname='CONTCAR'):
    """
    Purpose:
        inputs:  
            [$1] top directory
            example: /home/tam/NaClneb/staticim
            example: /home/tam/NaClneb
            [$1a] first directory name
            [$1b] second directory name
            [$2] atom position in POSCAR to search for
            [$3] second atom position in POSCAR to search for
            [$4] adjustment (if over or under this value, go to other side)
                example:"xlow, xhigh, ylow, yhigh, zlow, zhigh"
                "0.25,1,0,1,0,1"
            [$4] file name to use (default 'POSCAR')
        outputs:
            fractional path length
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Xvalidinteger
    import Pgetline
    import Upathcalcs
    import Xvector
    import Pfractocart
#   Validation
#   #  
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
    topdir1 = os.path.join(topdir, dir1)
    topdir2 = os.path.join(topdir, dir2)
    topdir1 = Xvaliddirectory.main(topdir1)
    if (topdir1 == None):
        print '(Exiting.)'
        return None
    topdir2 = Xvaliddirectory.main(topdir2)
    if (topdir2 == None):
        print '(Exiting.)'
        return None
    atomn1 = Xvalidinteger.main(atom1,1,None)
    if atomn1 == None:
        return None
    atomn2 = Xvalidinteger.main(atom2,1,None)
    if atomn2 == None:
        return None
   
    cartcoords=[]
    mydist=0
    mydistvec=[]
    av1=[]
    av2=[]
    cartcoords1 = Pfractocart.main(topdir1, fname, adj)
    cartcoords2 = Pfractocart.main(topdir2, fname, adj)
    if not cartcoords1 == None and not cartcoords2==None:
        av1 = cartcoords1[atomn1-1] #indexing starts at 0
        av2 = cartcoords2[atomn2-1] #indexing starts at 0
        print av1
        print av2
        mydistvec = Xvector.add(av1,av2,1)
        # actually subtracts
        print mydistvec
        mydist = Xvector.magnitude(mydistvec)
    return mydist
#
def loop(loopdir, atomline1=0, atomline2=0, adj='', fname='CONTCAR', 
            filter='*bulk_static'):
    """
    Purpose:
        inputs:  
            [$1] top neb directory
            example: /home/tam/allnebs
            [$2,$3] atomic position in file to use (VASP 5 files only)
            [$4] adjustment (if over or under this value, go to other side)
                example:"xlow, xhigh, ylow, yhigh, zlow, zhigh"
                "0.25,1,0,1,0,1"
            [$3] file name to use (default 'POSCAR', since stat im eps have no CONTCAR)
            [$4] '*neb_static': folders marked 'neb_static' only
        outputs:
            difference, and trend string
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Pdistance
#   Validation
#   #  
    loopdir = Xvaliddirectory.main(loopdir)
    if (loopdir == None):
        print '(Exiting.)'
        return None
#   #   comparison threshold will be validated by Pminmax
    loopdirlist=[]
    loopdirlist = Xgetdirs.main(loopdir, 1, 5, filter)
    if (loopdirlist == None):
        print 'Error getting directory list.'
        print '(Exiting.)'
        return None
    loopdiritem=""
    myreturn=""
    returnlist=[]
    #print loopdirlist
    for loopdiritem in loopdirlist:
        #print loopdiritem
        myreturn = Pdistance.main(loopdiritem, atomline1, atomline2, 
            adj, fname)
        if not myreturn == None:
            returnlist.append(loopdiritem + ': ' + str(myreturn))
    print loopdir, ': Distance from atom ',atomline1,' to ',atomline2
    for myreturn in returnlist:
        print myreturn
    return returnlist
