#!/usr/bin/env python
##
#@package   Penergynebnew
#@ingroup   vasp
#@ingroup   python
#@brief     gets neb energy, but can work from other folders than just /00 etc
#           (e.g. from created static folders)
#           actually just gets list of energies, then gets
#           difference from endpoints to max or min from a list 
#           if there is only one min OR one max (not both)
#@author    Tam Mayeshiba
#@date      4/19/11 created
#@date      2/6/12 use more directories
###
def main(topdir, filter='*neb*_static', compthresh=0.001):
    """
    Purpose:
        inputs:  
            [$1] neb directory
            example: /home/tam/NaClneb/staticim
            example: /home/tam/NaClneb
            [$2] '*neb' folders marked 'neb' only
                 '*neb_static': folders marked 'neb_static' only (default)
            [$2] comparison threshold for endpoints (OBSOLETE)
                default 0.001 (as in 0.001 eV)
        outputs:
            difference, and trend string
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Vgetenergy
    import Pminmax
#   Validation
#   #  
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   comparison threshold will be validated by Pminmax
    dirlist = Xgetdirs.main(topdir, 1, 1)
    if (dirlist == None):
        print 'Error getting directories in', topdir
        print '(Exiting.)'
        return None
    if len(dirlist) == 0:
        print 'No entries in directory list.'
        print '(Exiting.)'
        return None
    diritem=""
    myenergy=""
    energylist=[]
    for diritem in dirlist:
        myenergy = Vgetenergy.main(diritem)
        if (myenergy == None):
            print 'Skipping directory', diritem
        else:
            energylist.append(myenergy)
    energystr=""
    energystr = Pminmax.main(energylist, compthresh)
    return energystr
#
def loop(loopdir, filter='*neb*_static', compthresh=0.001):
    """
    Purpose:
        inputs:  
            [$1] top neb directory
            example: /home/tam/allnebs
            [$2] '*neb': folders marked 'neb' only
                 '*neb_static': folders marked 'neb_static' only
            [$2] comparison threshold for endpoints [OBSOLETE]
                default 0.001 (as in 0.001 eV)
        outputs:
            difference, and trend string
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Penergynebnew
#   Validation
#   #  
    loopdir = Xvaliddirectory.main(loopdir)
    if (loopdir == None):
        print '(Exiting.)'
        return None
#   #   comparison threshold will be validated by Pminmax
    loopdirlist=[]
    loopdirlist = Xgetdirs.main(loopdir, 1, 5, filter)
    if (loopdirlist == None):
        print 'Error getting directory list.'
        print '(Exiting.)'
        return None
    loopdiritem=""
    myreturn=""
    returnlist=[]
    for loopdiritem in loopdirlist:
        myreturn = Penergynebnew.main(loopdiritem, filter, compthresh)
        if not myreturn == None:
            returnlist.append(loopdiritem + ': ' + str(myreturn))
    for myreturn in returnlist:
        print myreturn
    return returnlist
