#!/usr/bin/env python
##
#@package   Pfracpath
#@ingroup   vasp
#@ingroup   python
#@brief     calculates fractional path length 
#@author    Tam Mayeshiba
#@date      2/20/12 looping wrapper to pathcalcs
###
def main(topdir, linenumber=0, fname='POSCAR'):
    """
    Purpose:
        inputs:  
            [$1] neb directory
            example: /home/tam/NaClneb/staticim
            example: /home/tam/NaClneb
            [$2] line number in POSCAR to search for
            [$3] file name to use (default 'POSCAR')
        outputs:
            fractional path length
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Xvalidinteger
    import Pgetline
    import Upathcalcs
#   Validation
#   #  
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   comparison threshold will be validated by Pminmax
    lineno = Xvalidinteger.main(linenumber,1,None)
    if lineno == None:
        return None
    
    staticlist=[]
    staticlist = Xgetdirs.main(topdir, 1, 1)
    if (staticlist == None):
        print 'Error getting directories in', topdir
        print '(Exiting.)'
        return None
    if len(staticlist) == 0:
        print 'No entries in directory list.'
        print '(Exiting.)'
        return None
    
    staticdir=""
    myenergy=""
    pathlist=[]
    myline=""
    myfracpath=""
    for staticdir in staticlist:
        #print 'Static: ', staticdir
        if 'static' not in os.path.basename(staticdir):
            print 'WARNING: not a static directory.'
        myline = Pgetline.main(staticdir, fname, '0', lineno)
        #print myline
        if myline == None:
            print 'Skipping', topdir, 'because of None value in search.'
            return None
        pathlist.append(myline)
    print pathlist
    myfracpath = Upathcalcs.pathlen(pathlist)
    return myfracpath
#
def loop(loopdir, linenumber=0, fname='POSCAR', filter='*neb_static'):
    """
    Purpose:
        inputs:  
            [$1] top neb directory
            example: /home/tam/allnebs
            [$2] line number in file to use (add 8 to VASP 5 POSCAR/CONTCAR atom number)
            [$3] file name to use (default 'POSCAR', since stat im eps have no CONTCAR)
            [$4] '*neb_static': folders marked 'neb_static' only
        outputs:
            difference, and trend string
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Pfracpath
#   Validation
#   #  
    loopdir = Xvaliddirectory.main(loopdir)
    if (loopdir == None):
        print '(Exiting.)'
        return None
#   #   comparison threshold will be validated by Pminmax
    loopdirlist=[]
    loopdirlist = Xgetdirs.main(loopdir, 1, 5, filter)
    if (loopdirlist == None):
        print 'Error getting directory list.'
        print '(Exiting.)'
        return None
    loopdiritem=""
    myreturn=""
    returnlist=[]
    #print loopdirlist
    for loopdiritem in loopdirlist:
        #print loopdiritem
        myreturn = Pfracpath.main(loopdiritem, linenumber, fname)
        if not myreturn == None:
            returnlist.append(loopdiritem + ': ' + str(myreturn))
    for myreturn in returnlist:
        print myreturn
    return returnlist
