#!/usr/bin/env python
##
#@package   Pfractocart
#@ingroup   vasp
#@ingroup   python
#@brief     fractional coordinates to cartesian coordinates
#@author    Tam Mayeshiba
#@date      4/2/12
###
def main(dirname, fname, adj, printout=0):
    """
    Purpose:
        inputs:  
            [$1] directory 
            example:
            [$2] filename (CONTCAR or POSCAR, with species line 
            (atoms must start at line 9)
            [$3] adjustment (if over or under this value, go to other side)
                example:[xlow, xhigh, ylow, yhigh, zlow, zhigh]
                [0.25,1,0,1,0,1]
                0 - no (default)
                1 - yes
            [$4] print out
        outputs:
            matrix of cartesian coordinates
    """
#   Modules:
    import os
    import sys
    import Xvector
    import Pgetline
    import Xvaliddirectory
#   
#   #  
    dirname = Xvaliddirectory.main(dirname)
    if dirname == None:
        return None
    fullpath=""
    fullpath = os.path.join(dirname, fname)
    if not os.path.isfile(fullpath):
        print 'Could not find', fullpath, '. Exiting.'
        return None
    myline=Pgetline.main(dirname, fname, 0, 8)
    if myline == None:
        print 'Error getting line 8 from', fullpath, '. Exiting.'
        return None
    if (myline.strip()[0]).isdigit():
        print fullpath, 'is not in VASP version 5 format. Exiting.'
        return None
    fmat=[]
    myread=""
    myread = open(fullpath, 'rb')
    lmat=[]
    lmat = myread.readlines()
    myread.close()
    fmat = lmat[8:]
    if not float(lmat[1]) == 1:
        print 'Scaling vector must be 1 for this function. Exiting.'
        return None
    lv1=Xvector.strtoflt(lmat[2])
    lv2=Xvector.strtoflt(lmat[3])
    lv3=Xvector.strtoflt(lmat[4])
    #print lv1,lv2,lv3
    cmat=[]
    line=''
    coord=''
    adjust=[]
    if len(adj) == 0:
        adjust=[]
    else:
        adjust = Xvector.strtoflt(adj)
    print adjust
    print 'Converting fractional to Cartesian.'
    for line in fmat:
        coord = Xvector.strtoflt(line)
        if not coord == None:
            if not len(adjust) == 0:
                if coord[0] < adjust[0]:
                    coord[0] = coord[0] + 1
                elif coord[0] > adjust[1]:
                    coord[0] = coord[0] - 1
                if coord[1] < adjust[2]:
                    coord[1] = coord[1] + 1
                elif coord[1] > adjust[3]:
                    coord[1] = coord[1] - 1
                if coord[2] < adjust[4]:
                    coord[2] = coord[2] + 1
                elif coord[2] > adjust[5]:
                    coord[2] = coord[2] - 1
            cmat.append(Xvector.frctocart(lv1,lv2,lv3,coord))
    print 'Done converting.'
    if printout == 1:
        for line in fmat:
            print line
        for line in cmat:
            print line
    return cmat
