function importme(filename,sheetname,range)
%> @namespace importme
%> @ingroup   plotting
%> @ingroup   matlab
%> @brief     Import data into a dataset named 'mydata' from an excel spreadsheet.
%> @verbatim  importme @endverbatim
%> @param     filename Full file name in single quotes. Can set defaults within script.
%> @param     sheetname Excel sheet name in single quotes.
%> @param     range Excel range, for example 'A1:B3'. Do not include blank rows.
%> @note      All columns in the excel spreadsheet, including blank ones, 
%> must have a UNIQUE header that does not include special characters like '-' or '('.
%> @author    Tam Mayeshiba
%> @date      10/24/11 created
%> @date      1/13/12 removed extraneous lines
if (nargin < 1)
    filename='C:\your_file_here.xlsx';
    sheetname='4M';
    range='A1:AP127'; %do not include blank rows.
end

mydata = dataset('XLSFile',filename,'Sheet',sheetname,'Range',range);

headers = fieldnames(mydata); % or, headers = mydata.Properties.VarNames

numbered = {[1:length(headers)];headers};
nlist = transpose(numbered);
for i=1:length(headers)
    %display([nlist{1}(i),nlist{2}(i)]);
    fprintf('%d %s\n', nlist{1}(i), nlist{2}{i});
end

%replace '' strings with 'blank'
%loop through all values except built-in 'Properties' fieldname
for i=1:length(headers)
    if ~strcmp(headers{i},'Properties')
        if strcmp(class(mydata.(headers{i})),'cell')
            for row = 1:1:size(mydata.(headers{i}),1)
                if strcmp(mydata.(headers{i})(row),'')
                    mydata.(headers{i})(row) = {'blank'};
                end
            end
        end
    end
end

%nominalize all non-numeric columns to store bins of categories
for i=1:length(headers)
    if ~strcmp(headers{i},'Properties')
        if strcmp(class(mydata.(headers{i})),'cell')
            mydata.(headers{i}) = nominal(mydata.(headers{i}));
        end
    end
end

%mydata  %print all to screen

%push to current workspace
newData1.mydata = mydata;
assignin('base', 'mydata', newData1.mydata);
