function linearfit(figurenumber)
%> @namespace linearfit
%> @ingroup   plotting
%> @ingroup   matlab
%> @brief     Add a linear fit line to the current figure.
%> @verbatim  linearfit @endverbatim
%> @param     figurenumber Number of figure. If not specified, will use selected figure.
%> @note      Figure must be UNDOCKED from GUI property editor. Otherwise the fit line may overwrite.
%> @author    Tam Mayeshiba
%> @date      1/5/12 created

% some code copied from mathworks.com
% http://www.mathworks.com/support/solutions/en/data/1-A6PBV2/index.html?solution=1-A6PBV2
% also, http://chemwiki.ucdavis.edu/VV:_Mathematical_Concepts/Linear_Regression_in_Matlab
% and Jos and John D'Errico's posts on thread 122906 in the mathworks.de forum
% TM 1/5/12
%R^2

if nargin < 1
    h = gcf;
else
    h = figurenumber;
    figure(h);
end

%Find the line objects in the figure. This will return all the lines in the
%current figure.
line = findobj(h,'Type','hggroup');

%Extract X and Y data values from the first line:
dispmat=[];
endmat=[]; %display matrix
hold on
for i = 1:length(line)
    mytags = fieldnames(get(line(i)));
    if any(ismember(mytags,'XData'))
        dispname = get(line(i),'DisplayName');
        dispmat=['Series ', int2str(i), dispname];
        x_data = get(line(i), 'xdata');
        y_data = get(line(i), 'ydata');
        xkeep = ~isnan(x_data);
        ykeep = ~isnan(y_data);
        keepboth = xkeep & ykeep;
        goodx = x_data(keepboth); %only non-NaN values for both x and y
        goody = y_data(keepboth); %only non-NaN values for both x and y
        numpoints = length(goodx);
        if numpoints > 0
            [p, s] = polyfit(goodx, goody, 1); %1 = linear
            fitvalues = polyval(p,goodx);
            plot(goodx, fitvalues);
            correlation = corrcoef(goody, fitvalues);
            if (~any(isnan(correlation))) & (numel(correlation) > 1)
                Rsquared = correlation(1,2)^2;
                disptext = [dispname ' R2=' num2str(Rsquared,4) ' slope=' num2str(p(1),2)];
                text(goodx(1),fitvalues(1),disptext)
                endmat = vertcat(endmat,{disptext});
            end
        end
    end
end
hold off
endmat
end
     


