function miniplots(mydata, xcol, ycol, plotchoice)
%> @namespace miniplots
%> @ingroup   plotting
%> @ingroup   matlab
%> @brief     Create a set of plots using plotme.
%> @details   Use to create default sets of plots quickly. Plots each xcol
%> against each y col, and can save default plot configurations (filters, point labels).
%> @verbatim  miniplots(mydata, [3,6], 4, 2) @endverbatim
%> @param     xcol Number or matrix of x-column(s) to plot.
%> @param     ycol Number or matrix of y-column(s) to plot.
%> @author    Tam Mayeshiba
%> @date      10/24/11 created
%> @date      1/13/12 generalized for group

%can separate out by different labels. Here, column 3 is a label column.
calcmatrix={getlabels(mydata.(mydata.Properties.VarNames{3}))};

if nargin < 2
    pchoice=0; %default
   
    validx=[1]; %set these defaults according to your data - arbitrary
    validy=[2,3,4];

elseif nargin < 4
    validx = xcol;
    validy = ycol;
    pchoice = 0; %default
else
    validx = xcol;
    validy = ycol;
    pchoice = plotchoice;
end

%YOU CAN SET UP DIFFERENT 'plotme' LINES
if pchoice == 1
    %separated by labels
    for myx = validx
         for myy = validy
             for myf = 1:length(calcmatrix{1})
                 if (myx ~= myy)
                     plotme(mydata,myx,myy,3,calcmatrix{1}{myf},0,'',0,1,0)
                 end
             end
         end
    end    
elseif pchoice == 2
    %unfiltered
    for myx = validx
        for myy = validy
            plotme(mydata,myx,myy,0,'',0,'',3,1,0) 
        end
    end
else
    %unfiltered
    for myx = validx
        for myy = validy
            plotme(mydata,myx,myy,0,'',0,'',1,2,0) 
        end
    end
end
end

