function plotme(mydata,xcol,ycol,filtercol1,filterval1,filtercol2,filterval2,splitcol,label1col,label2col)
    %> @namespace plotme
    %> @ingroup   plotting
    %> @ingroup   matlab
    %> @brief     Makes a scatter plot of data. 
    %> @verbatim  plotme(mydata, 2, 3, 4, 'La', 0, '', 1, 5, 0)@endverbatim
    %> @param     mydata Dataset containing columns of data
    %> @param     xcol Number for x-column in spreadsheet to plot (use headers(mydata) to get column numbers).
    %> @param     ycol Number for y-column in spreadsheet to plot
    %> @param     filtercol1 Number for filter column. Use 0 if no filter.
    %> @param     filterval1 Value (string) for filter column. Use '' if no filter.
    %> @param     filtercol2 Number for filter column. Use 0 if no filter.
    %> @param     filterval2 Value (string) for filter column. Use '' if no filter.
    %> @param     splitcol Number for column to group data into separate lines. Use 0 if no column.
    %> @param     label1col Number for labeling column. Use 0 if no column.
    %> @param     label2col Number for second labeling column. Use 0 if no column.
    %> @note      Must have imported data from Excel spreadsheet through importme first. 
    %> @author    Tam Mayeshiba
    %> @date      10/24/11 created
    %> @date      1/13/12 last update
    
    figure
    
    x=xcol;   % example 1 for A-site cation
    y=ycol;   % example 5 for migration barrier
    filt1=filtercol1; % example 3 for calc detail
    filt1str = filterval1; % example 'FM 4M 40 atoms'
    filt2=filtercol2;
    filt2str = filterval2;
    split = splitcol; % column number to split out by in single plot
    l1 = label1col;
    l2 = label2col;
   
    markerlist={'o','s','^','+','x','d','h','<','p','>','v','o','s','^','+','x','d','h','<','p','>','v','o','s','^','+','x','d','h','<','p','>','v','o','s','^','+','x','d','h','<','p','>','v','o','s','^','+','x','d','h','<','p','>','v'};
    markerct=1;
    markersize=15^2;
    legmarksize=10; %legend marker size
    linewidth=3;
    leglinewidth=2.4;
    fontsize=16;
    colororder = [
    1.00  0.00  0.00
    0.75  0.00  0.75
	0.00  0.50  0.00
    0.00  0.00  1.00
	0.00  0.75  0.75
	0.75  0.75  0.00
	0.25  0.25  0.25
	0.75  0.25  0.25
	0.95  0.95  0.00
	0.25  0.25  0.75
	0.75  0.75  0.75
	0.00  1.00  0.00
	0.76  0.57  0.17
	0.54  0.63  0.22
	0.34  0.57  0.92
	1.00  0.10  0.60
	0.88  0.75  0.73
	0.10  0.49  0.47
	0.66  0.34  0.65
	0.99  0.41  0.23
    ];
    set(gcf,'DefaultAxesColorOrder', colororder);
    
    filterset = dataset;
    xset = [];
    yset = [];
    Aset = [];
    l1set=[];
    l2set=[];
    l1text='';
    l2text='';
    filt1append=[];
    filt2append=[];
    filtidx1=[];
    filtidx2=[];
    filtidxboth=[];
    
    if filt1 > 0
        filt1append=[' for ' mydata.Properties.VarNames{filt1} ' = ' filt1str ' '];
    end           
    if filt2 > 0
        filt2append = [' for ' mydata.Properties.VarNames{filt2} ' = ' filt2str ' '];
    end
        
    if filt1 > 0
        filtidx1 = mydata.(mydata.Properties.VarNames{filt1}) == filt1str;
    else
        filtidx1 = ones([size(mydata,1),1]); %filter nothing out
    end
    if filt2 > 0
        filtidx2 = mydata.(mydata.Properties.VarNames{filt2}) == filt2str;
    else
        filtidx2 = ones([size(mydata,1),1]); %filter nothing out
    end
    filtidxboth = filtidx1 & filtidx2;
    
    xset = mydata.(mydata.Properties.VarNames{x})(filtidxboth);
    yset = mydata.(mydata.Properties.VarNames{y})(filtidxboth);
    if split > 0
        Aset = mydata.(mydata.Properties.VarNames{split})(filtidxboth);
    else
        Aset = zeros(length(xset),1); %zero matrix
    end
    if l1 > 0
        l1set = mydata.(mydata.Properties.VarNames{l1})(filtidxboth);
    else
        l1set = zeros(length(xset),1); %zero matrix
    end
    if l2 > 0
        l2set = mydata.(mydata.Properties.VarNames{l2})(filtidxboth);
    else
        l2set = zeros(length(xset),1);
    end
    
    if strcmp(class(xset),'nominal') || strcmp(class(yset),'nominal')
        class(xset)
        class(yset)
        return
    end
    
    [mydata.Properties.VarNames(y) ' vs ' mydata.Properties.VarNames(x), filt1append, filt2append]
    if (split > 0)
        filterset = dataset(Aset,xset,yset,l1set,l2set);
        Acats=[];
        xsets=[];
        ysets=[];
        l1sets=[];
        l2sets=[];
        Acats_legend=[];
        Acats = nominal(getlabels(filterset.(filterset.Properties.VarNames{1})));
        xsets = cell(1,length(Acats)); % as many A sets as A categories
        ysets = cell(1,length(Acats)); % as many A sets as A categories
        l1sets = cell(1,length(Acats)); % as many A sets as A categories
        l2sets = cell(1,length(Acats)); % as many A sets as A categories
        Acats_legend = zeros(1,length(Acats));
        for cat=1:length(Acats)
            xsets{cat} = filterset.(filterset.Properties.VarNames{2})(filterset.(filterset.Properties.VarNames{1}) == Acats(cat));
            ysets{cat} = filterset.(filterset.Properties.VarNames{3})(filterset.(filterset.Properties.VarNames{1}) == Acats(cat));
            l1sets{cat} = filterset.(filterset.Properties.VarNames{4})(filterset.(filterset.Properties.VarNames{1}) == Acats(cat));
            l2sets{cat} = filterset.(filterset.Properties.VarNames{5})(filterset.(filterset.Properties.VarNames{1}) == Acats(cat));
        end
            
        hold on
        for cat = 1:length(Acats)
            scatter(xsets{cat},ysets{cat},'Marker',markerlist{markerct},'SizeData',markersize,'LineWidth',linewidth)
            if all(isnan(xsets{cat})) || all(isnan(ysets{cat}))
                Acats_legend(cat) = 0;
            else
                Acats_legend(cat) = 1;
            end
            %xsets{cat}
            %ysets{cat}
            markerct = markerct + 1;
            for i=1:length(xsets{cat})
                if isnumeric(l1sets{cat}(i))
                    if any(any(l1sets{cat})) == 0
                        l1text = '';
                    else
                        l1text = num2str(double(l1sets{cat}(i)),3);
                    end
                else
                    l1text = cellstr(l1sets{cat}(i));
                    if strcmp(l1text,'blank')
                        l1text = '';
                    end
                end
                if isnumeric(l2sets{cat}(i))
                    if any(any(l2sets{cat})) == 0
                        l2text = '';
                    else
                        l2text = num2str(double(l2sets{cat}(i)),3);
                    end
                else
                    l2text = cellstr(l2sets{cat}(i));
                    if strcmp(l2text,'blank')
                        l2text = '';
                    end
                end
                text(xsets{cat}(i),ysets{cat}(i),[l1text, ' ', l2text])
            end
        end
        hold off
    else
        %xusedlevels = droplevels(xset)
        %scatter(xusedlevels,yset,'Marker',markerlist{markerct},'SizeData',markersize,'LineWidth',linewidth)
        %xusedlevels
        scatter(xset,yset,'Marker',markerlist{markerct},'SizeData',markersize,'LineWidth',linewidth)
        %xset
        %yset
        for i=1:length(xset)
            if isnumeric(l1set(i))
                if any(any(l1set)) == 0
                    l1text = '';
                else
                    l1text = num2str(double(l1set(i)),3);
                end
            else
                l1text = cellstr(l1set(i));
            end
            if isnumeric(l2set(i))
                if any(any(l2set)) == 0
                    l2text = '';
                else
                    l2text = num2str(double(l2set(i)),3);
                end
            else
                l2text = cellstr(l2set(i));
            end
            text(xset(i),yset(i),[l1text, ' ', l2text])
        end
    end

    
    %SPECIFIC TO TAM'S PLOTS:
    if (x == 4) %B-site atomic number use letter labels instead
        Bmat={'Sc','Ti','V','Cr','Mn','Fe','Co','Ni','Cu','Zn','Ga'};
        xvals = transpose(unique(xset));
        tickmat={};
        count=1;
        for xval = xvals
            tickmat{count} = Bmat{xval - 20}; %starts at 21 = Sc
            count = count + 1;
        end
        set(gca,'XTick',xvals)
        set(gca,'XTickLabel',tickmat)
    elseif strcmp(class(mydata.(mydata.Properties.VarNames{x})),'nominal')
        set(gca,'XTick',[1:size(getlabels(xset),2)])
        set(gca,'XTickLabel',getlabels(xset))
    end
    
    %SPECIFIC TO TAM'S PLOTS:
    %scale VFE and migration barrier always to same amount
    if y == 6
        ylim([0, 2.5]);
    elseif y == 9
        ylim([0, 10]);
    end
    
    %SPECIFIC TO TAM'S PLOTS:
    %legend for split
    if (split > 0) && strcmp(class(mydata.(mydata.Properties.VarNames{split})),'nominal')
        Ausedlevels=[];
        for i=1:length(Acats)
            if Acats_legend(i) == 1
                Ausedlevels = horzcat(Ausedlevels,Acats(i));
            end
        end
        Ausedlevels = droplevels(Ausedlevels);
        h_legend=legend(getlabels(Ausedlevels),'Location','EastOutside'); %legend values
        set(h_legend,'FontSize',fontsize);
        legendobj = findobj(gcf,'tag','legend'); 
        legend_children = get(legendobj,'Children');
        for i=1:length(legend_children)
            if strcmp(get(legend_children(i),'Type'),'hggroup')
                set(get(legend_children(i),'Children'),'LineWidth',leglinewidth,'MarkerSize',legmarksize)
            end
        end
    end
    
    %label axes
    set(gca,'FontSize',fontsize)
    xlabel(mydata.Properties.VarNames(x),'FontSize',fontsize);
    ylabel(mydata.Properties.VarNames(y),'FontSize',fontsize);
    title([mydata.Properties.VarNames(y) ' vs ' mydata.Properties.VarNames(x), filt1append, filt2append]);
    

end