#!/usr/bin/env python
##
#@package   Pminmax
#@ingroup   vasp
#@ingroup   python
#@brief     takes a list of numbers and determines whether it has
#           more than one max or min, or if the endpoint numbers are
#           very different. If same endpoints and one max, takes difference.
#           NEB energy should really use the spline fit, but this is
#           supposed to be an estimate.
#@author    Tam Mayeshiba
#@date      4/19/11 created
#@date      2/6/12 use max minus min, tell how far eps are
###
def main(numlist, compthresh=0.001):
    """
    Purpose:
            Takes a list of numbers and determines whether it has
            more than one max or min, or if the endpoint numbers are
            very different. If same endpoints and one max, takes difference.
            If 1 endpoint is the max and the other is the min, takes diff.
            NEB energy should really use the spline fit, but this is
            supposed to be an estimate.
        inputs:  
            [$1] list of numbers (floating notation is okay eg. 3E-10)
            example: ['4E-5',1.3,12]
            [$2] comparison threshhold to determine if endpoints are the
                'same' - default 0.001 (as in, 0.001 eV)
        outputs:
            difference (if applicable) and a string to describe the list
            behavior ('max - x - x - min')
        notes:
            Cannot be run directly using Uwrapper
    """
#   Modules:
    import os
    import sys
#   Validation
#   #  
    if type(numlist) != list:
        print 'Number list should be a list.'
        print '(Exiting.)'
        return None
    fltthresh=0
    try:
        fltthresh = float(compthresh)
    except (ValueError, TypeError):
        print 'comparison threshold', compthresh, 'should be an'
        print 'integer or a decimal.'
        print '(Exiting.)'
        return None
#   loop
    onemax=0
    onemin=0
    globmax=0
    globmin=0
    listlen=0
    listlen = len(numlist)
    if (listlen == 0):
        print 'List has no elements.'
        print '(Exiting.)'
        return None
    fltlist=[]
    rawnum=""
    fltnum=0
    for rawnum in numlist:
        try:
            fltnum = float(rawnum)
        except (ValueError, TypeError):
            print rawnum, 'is not recognized as a valid number.'
            print '(Exiting.)'
            return None
        fltlist.append(fltnum)
    fltlen = len(fltlist)
    if (fltlen < 1):
        print 'Float list has no elements.'
        print '(Exiting.)'
        return None
    if (fltlen == 1):
        print 'Only one list item. Returning it.'
        return fltlist[0]
    globmax = max(fltlist)
    globmin = min(fltlist)
    diff=0
    thisflt=0
    lastflt=0
    fltct=0
    trendstr=""
    while (fltct < fltlen):
        if (fltct == 0):
            fltct = fltct + 1
            trendstr = trendstr + 'x'
        else:
            lastflt = fltlist[fltct - 1]
            thisflt = fltlist[fltct]
            diff = thisflt - lastflt
            if (diff == 0):
                trendstr = trendstr + '-same-x'
            elif (diff < 0):
                trendstr = trendstr + "-\-x"
            else:
                trendstr = trendstr + '-/-x'
            fltct = fltct + 1
    getmaxima=[]
    getmaxima = trendstr.split("-/-x-\-")
    getminima=[]
    getminima = trendstr.split("-\-x-/-")
    internalmax = len(getmaxima) - 1
    internalmin = len(getminima) - 1
    endptcompare=0
    ep2minusep1=0
    avgendpt=0
    mydiff=0
    ep1=0
    ep2=0
    ep1 = fltlist[0]
    ep2 = fltlist[fltlen - 1]
    endptcompare = abs(ep2 - ep1)
    ep2minusep1=ep2 - ep1
#
    retstr=""
    retstr = 'Ep2 minus ep1:' + str(ep2minusep1)
    retstr = retstr + ':Max min min:' + str(globmax - globmin)
    retstr = retstr + ':Pattern:' + trendstr
    #if (internalmax == 1) and (internalmin == 0):
    #    retstr=""
    #    if (endptcompare > fltthresh):
    #        retstr = 'Endpoints' + endptcompare + 'apart.'
    #        print 'One internal maximum found.'
    #        return 'diff eps : ' + trendstr
    #    avgendpt = (ep1 + ep2) / float(2)
    #    mydiff = globmax - avgendpt
    #    return str(mydiff) + ' : ' + trendstr
#   #   one internal minimum
    #if (internalmax == 0) and (internalmin == 1):
    #    if (endptcompare > fltthresh):
    #        print 'Endpoints too different to compare.'
    #        print 'One internal minimum found.'
    #        return 'diff eps : ' + trendstr
    #    avgendpt = (ep1 + ep2) / float(2)
    #    mydiff = globmin - avgendpt
    #    return str(mydiff) + ' : ' + trendstr
#   #   endpoints serve as max and min
    #if (internalmax == 0) and (internalmin == 0):
    #    mydiff = fltlist[fltlen - 1] - fltlist[0]        
    #    return str(mydiff) + ' : ' +  trendstr
    #print 'Trend has more than one maximum and/or minimum.'
    #return 'complicated : ' + trendstr
    return retstr
        
