#!/usr/bin/env python
##
#@package   Pmodfilenew
#@ingroup   vasp
#@ingroup   python
#@brief     deletes, cuts and pastes, copies and pastes, inserts,
#           or replaces a line
#@author    Tam Mayeshiba
#@date      11/1/10 created
#@date      11/30/10 updates to style
#@date      12/22/10: chopping existf into existdir and existname
#                       in order to facilitate looping
#@date      12/22/10: fixed modeact integer validation
#@date      12/22/10: adding looping tag
#@date      12/22/10: updated logic tag for findstr vs findln
#@date      8/12/11: add depth parameter to loop
###
def main(existdir, existname, findstr, findln='X', modeact='D', modeinst=''):
    """
    Purpose: deletes, cuts and pastes, copies and pastes, inserts, or 
             replaces a line
        inputs:  
            [$1] path for file to modify
                 example: /home/tam/newNaCl
            [$2] file name to modify
                 example: POSCAR
            [$2] string to search for, in quotes
                 example: "[VASP_POTCAR_FILE]"
                 enter 0 if searching by line number instead
            [$3] line number to search for
                 enter X if searching by string instead
            [$4] mode: 
                    D - delete (default) 
                    C - cut and paste
                    Y - copy and paste
                    I - inserts a line
                    R - replace
            [$5] if mode is:
                     D - this parameter will be ignored
                     C - line number (in original file) after which to paste
                            (0 is accepted)
                     Y - line number (in original file) after which to paste
                     R - whole line to replace with, in quotes;
                 no end-of-line characters (no '\\n')
                     I - whole line to be inserted below search, in quotes;
                 no end-of-line characters (no '\\n')
                 Following lines will be moved down.
    Note: right now this only finds the first occurrence in the file.
          also, the file should never have any blank lines in it
    tags:
        main
        loop
    """
#   Modules:
    import os
    import sys
    import Xvalidletter
    import Xvalidinteger
    import Xvaliddirectory
#   
    tgfile=-1
    readf=""
    workf=""
    linect=1
    readln=""
    sameln=""
#   validation
#   #   validate directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   validate file existence
    existf=""
    existf = existdir + '/' + existname
    if (os.path.isfile(existf)) == False:
        print existf, 'does not exist.'
        print '(Exiting.)'
        return None
#   #   validate input parameters
#   #   #   validate line number
    if (findln in ['X', 'x']):
        findln = str.upper(findln)
        if (findstr == '0') or (findstr == 0):
            print '0 is not a good string for which to search.'
            print 'Please try again with a different string'
            print 'or use a line number in the third parameter instead.'
            print '(Exiting.)'
            return None
    else:
        findln = Xvalidinteger.main(findln, 1, None)
        if (findln == None):
            print findln, 'should be a line number or X.'
            print '(Exiting.)'
            return None
        if (findstr != "0") and (findstr != 0):
            print 'Line number will be used instead of search string.'
#   #   #   validate mode
    modeact = Xvalidletter.main(modeact, ['d','c','y','i','r'], 1)
    if (modeact == None):
        print modeact, 'should be D, C, Y, I, or R.'
        print '(Exiting.)'
        return None
#   #   #   validate last parameter
    if (modeact in ['c', 'y']):
        modeinst = Xvalidinteger.main(modeinst, 0, None)
        if (modeinst == None):
            print 'Using option', modeact,', last parameter'
            print 'should be a line number after which to paste.'
            print '(Exiting.)'
            return None
    elif (modeact in ['r','i']):
        if (len(modeinst) == 0):
            print 'Using option', modeact,','
            print 'a string should be given in the last parameter.'
            print '(Exiting.)'
            return None
        if (len(modeinst) < 4):
            print 'Warning: the line to replace seems short. Check output.'
#   #   open target file and working file
    readf = open(existf, 'rb')
    workf = open(existf + '.work', 'wb')
#   #   copy over all lines into working file until string or line is found
    if (findln == 'X'):
        while (tgfile == -1):
            readln = readf.readline()
            if not readln:
                break                      # file should not have blank lines
            tgfile = readln.find(findstr)
            if tgfile == -1:
                workf.write(readln)
#   #   TTM debug: what if the string is never found?
        if (tgfile == -1):
            print findstr, 'was not found.'
            print '(Exiting.)'
            workf.close()
            readf.close()
            return None
    else:
        while (linect <= findln):
            readln = readf.readline()
            if not readln:
                break
            if (linect < findln):
                workf.write(readln)
            linect = linect + 1
#   #   readln is the search line or found string line
    sameln = readln
#   #   only write the line if in copy and paste or insert mode
    if (modeact in ['y', 'i']):
        workf.write(readln)
    if (modeact in ['i', 'r']):
        workf.write(modeinst + '\n')
    readln = readf.readlines()
    workf.writelines(readln)
    readf.close()
    workf.close()
    os.remove(existf)
    os.rename(existf + '.work', existf)
    if (modeact in ['c', 'y']):
        readf = open(existf, 'rb')
        workf = open(existf + '.work', 'wb')
        linect = 1
        while (linect <= modeinst):
            readln = readf.readline()
            if not readln:
                break
            workf.write(readln)     # always write the last line
            linect = linect + 1
        workf.write(sameln)
        readln = readf.readlines()
        workf.writelines(readln)
        readf.close()
        workf.close()
        os.remove(existf)
        os.rename(existf + '.work', existf)
    workf.close()
    readf.close()
    print existf, '-Done.'
    return True
#
#
def loop(topdir, existname, findstr, findln='X', modeact='D', modeinst='',
            depth=1):
    """
    Purpose: deletes, cuts and pastes, copies and pastes, inserts, or 
             replaces a line
        inputs:  
            [$1] top directory containing directories for files to modify
                 example: /home/tam/collection
            [$2] file name to modify
                 example: POSCAR
            [$2] string to search for, in quotes
                 example: "[VASP_POTCAR_FILE]"
                 enter 0 if searching by line number instead
            [$3] line number to search for
                 enter X if searching by string instead
            [$4] mode: 
                    D - delete (default) 
                    C - cut and paste
                    Y - copy and paste
                    I - inserts a line
                    R - replace
            [$5] if mode is:
                     D - this parameter will be ignored
                     C - line number (in original file) after which to paste
                            (0 is accepted)
                     Y - line number (in original file) after which to paste
                     R - whole line to replace with, in quotes;
                 no end-of-line characters (no '\\n')
                     I - whole line to be inserted below search, in quotes;
                 no end-of-line characters (no '\\n')
                 Following lines will be moved down.
            [$6] levels deep to search: default is 1 (directories below)
                 Values can be from 1 to 5.
    Note: right now this only finds the first occurrence in the file.
          also, the file should never have any blank lines in it
    tags:
        main
        loop
    """
#   import
    import Pmodfilenew
    import Xgetdirs
    import Xvaliddirectory
    import Xvalidinteger
#   validation
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
    depth = Xvalidinteger.main(depth, 1, 5)
    if (depth == None):
        print '(Exiting.)'
        return None
#   run Pmodfilenew
    dirlist=[]
    mydir=""
    returnval=""
    dirlist = Xgetdirs.main(topdir, 1, depth)
    for mydir in dirlist:
        returnval = Pmodfilenew.main(mydir, existname, findstr, 
                                    findln, modeact, modeinst)
    print 'Done.'
    return True
#
#
def prompt():
#   Modules
    import Xvalidletter
#   prompt for input
    getdir=""
    getname=""
    getstr=""
    getln=""
    getact=""
    getinst=""
    retlist=[]
    print 'Path for file to modify, or top directory of'
    getdir = raw_input('collection of files if using loop tag: ')
    retlist.append(getdir)
    getname = raw_input('File name to modify (ex: POSCAR): ')
    retlist.append(getname)
    getstr = raw_input('String to search for, or 0 to use line number: ')
    retlist.append(getstr)
    if (getstr != '0'):
        getln = 'X'
    else:
        getln = raw_input('Line number to search for: ')
    retlist.append(getln)
    print 'Mode: (D)elete, (C)ut and paste, Cop(Y) and paste, ',
    getact = raw_input('(I)nsert line, (R)eplace line: ')
    getact = Xvalidletter.main(getact, ['d', 'c', 'y', 'i', 'r'], 1)
    while (getact == None):
        print 'Mode: (D)elete, (C)ut and paste, Cop(Y) and paste, ',
        getact = raw_input('(I)nsert line, (R)eplace line: ')
        getact = Xvalidletter.main(getact, ['d', 'c', 'y', 'i', 'r'], 1)
    retlist.append(getact)
    if (getact == 'd'):
        pass # no need for last parameter
    elif getact in ['c', 'y']:
        print 'Line number (in original file) after which to paste.'
        getinst = raw_input('(0 is acceptable to paste at top): ')
    elif (getact == 'r'):
        getinst = raw_input('Whole line to replace with: ')
    elif (getact == 'i'):
        getinst = raw_input('Whole line to be inserted below search: ')
    else:
        pass
    retlist.append(getinst)
    return retlist
