#!/usr/bin/env python
##
#@package   Pqsubretidnew 
#@ingroup   vasp
#@ingroup   python
#@brief     submits script and returns job id
#@author    Tam Mayeshiba
#@date      12/3/10 created
#@date      3/1/11 copied from Pqueuesubretid; want to avoid writing to file
#@date      6/30/11 return subpath if can't get job ID: for running on Ranger
###
def main(subpath, subname):
    """
    Purpose: submits script and returns job id
        inputs:  
            [$1] submission script path
                 example: /home/tam/NaCl_rel1/NaClep1/
            [$2] submission script name
                example: parallel.sh
        returns:
            job ID of submission if succeeds
            None if error
    """
#   Modules:
    import os
    import sys
    import subprocess
    import Xvaliddirectory
    import Pgetseg
#
#   validation
#   #   validate submission script path
    subpath = Xvaliddirectory.main(subpath)
    if (subpath == None):
        print '(Exiting.)'
        return None
#   #   validate submission script name
    if type(subname) != str:
        print subname, 'should be a string of the submission script name.'
        print '(Exiting.)'
        return None
    if (subname[-3:] != '.sh'):
        print subname, 'does not end in .sh and may not be a valid',
        print 'submission script. Please use full file name, including',
        print '.sh'
        print '(Exiting.)'
        return None
    if not os.path.isfile(subpath + '/' + subname):
        print subname, 'does not exist in', subpath
        print '(Exiting.)'
        return None
#   submit run
    beforedir=""
    beforedir = os.getcwd()
    os.chdir(subpath)
    submitrun=""
    subretstr=""
    submitrun = subprocess.Popen('qsub ' + subname, shell=True, 
                stdout=subprocess.PIPE)
    subretstr = submitrun.communicate()[0]
    submitrun.wait()
#   extract job id
    jobseg=""
    jobseg = Pgetseg.main(subretstr, 1, '.')
    if (jobseg == None):
        print 'Job ID could not be extracted.'
        print '(Exiting.)'
        return subpath
    if (str.isdigit(jobseg) != True):
        print 'Job ID could not be extracted.'
        print '(Exiting.)'
        return subpath
    else:
        return jobseg
