#!/usr/bin/env python
##
#@package   Prunone 
#@ingroup   vasp
#@ingroup   python
#@brief     run submission script for one directory
#@author    Tam Mayeshiba
#@date      3/22/11 - copied from Prunindir
#@date      3/23/11 - modify to use Psubfile to find submission script
#@date      3/23/11 - add support for job monitoring
#@date	    3/28/11 - remove change of directories: handled by Pqueuesubretid
#@date      3/28/11 - add NEB flag (looks in 01 directory, which all NEB
#                   have, in order to determine if started
#@date      5/4/11 - modify to use Pqsubretidnew
###
def main(existdir, askfirst=1, checksec=900, lagsec=1800, emaddr="",
            isneb=0):
    """
    Purpose: run submission script for one directory
        inputs:  
            [$1] directory in which to run script
                 example: /home/tam/starts/
            [$2] 1 - ask before running (default)
                 0 - run without asking
            [$3] number of seconds between checks for still running
                 default 900; acceptable range between 10 and 100000
            [$4] number of seconds between updates for job to be
                 considered hung
                 default 1800; acceptable range between 10 and 100000
            [$5] email address for problem notification
            [$6] 0 - job is not an NEB run (default)
                 1 - job is an NEB run
        returns:
            None if error
            True if run
            False if run hung up
        outputs: outputs to screen
        note: only goes one level deep
              use Prunindir if you just want to send a bunch of jobs and
                   not worry about whether they complete
    """
#   Modules:
    import os
    import sys
    import shutil
    import subprocess
    import Xvaliddirectory
    import Xvalidinteger
    import Psubfile
    import Xvalidletter
    import Pdirmonitor
    import Psendemail
    import Pqsubretidnew
#   Validation
#   #   askfirst
    askfirst = Xvalidinteger.main(askfirst, 0, 1)
    if (askfirst == None):
        print '(Exiting.)'
        return None
#   #   validate directory path
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   validate checksec
    checksec = Xvalidinteger.main(checksec, 10, 100000)
    if (checksec == None):
        print '(Exiting.)'
        return None
#   #   validate lagsec
    lagsec = Xvalidinteger.main(lagsec, 10, 100000)
    if (lagsec == None):
        print '(Exiting.)'
        return None
#   #   validate isneb
    isneb = Xvalidinteger.main(isneb, 0, 1)
    if (isneb == None):
        print '(Exiting.)'
        return None
#   find submission scripts
    subfile=""
    subfile = Psubfile.main(existdir)
    if (subfile == None):
        return None
    subme=""
#   ask permission and submit run
    monitorone=""
    tellme=""
    subfilename=""
    subfilepath=""
    subfilename = os.path.basename(subfile)
    subfilepath = os.path.dirname(subfile)
    jobid=""
    if (askfirst == 0):
        okaytorun = 'y'
    else:
        okaytorun = raw_input('Run for ' + existdir + ' (Y/N)? ')
    okaytorun = Xvalidletter.main(okaytorun, ['y','n'], 1) 
    if (okaytorun == None):
        print '(Exiting.)'
        return None
    if (okaytorun == 'n'):
        print 'Not run.'
        return False
    if (okaytorun == 'y'):
        print "cwd: ", existdir
        print "subfile: ", subfile
        jobid = Pqsubretidnew.main(subfilepath, subfilename)
        if (isneb == 0):
            monitorone = Pdirmonitor.main(existdir, checksec, lagsec)
        elif (isneb == 1):
            monitorone = Pdirmonitor.main(existdir + '/01', checksec, lagsec)
        if (monitorone != True):
            print 'Error in job', jobid
            tellme = Psendemail.main(emaddr,'Error in job ' + jobid, 
                    'Error in ' + existdir)
            return False
    return True
