#!/usr/bin/env python
##
#@package   Prunonewait   
#@ingroup   vasp
#@ingroup   python
#@brief     run Prunone in the background, but wait for it
#@details   used within other programs - not useful from command line
#@author    Tam Mayeshiba
#@date      3/23/11 copied from Pdirautorelaxsilent
#@date      5/18/11 copied from Prunone
#@date      5/19/11 fix call to Prunone: askfirst = 0
#@date      5/19/11 rename checklag to lagsec for consistency
#@date      5/19/11 fix call to Prunone: askfirst = '0' (send as string)
###
def main(fullpath, checksec=1800, lagsec=1800, emaddr="", isneb=0): 
    """
    Purpose: run Pdirautorelax silently in the background
       inputs:  
	        [$1] full path of folder of job to submit
                example: '/home/tam/rocksalt_rel1/NaClep1'
            [$2] second between checks - default 1800
            [$3] acceptable number of seconds before job considered hung
                    - default 1800
            [$4] email address
            [$5] 0 - default - is not an NEB run
                 1 - is an NEB run
       outputs:
            True - job has finished
            None - error
    """
#   Modules:
    import os
    import sys
    import subprocess
    import Xtimestring
    import Xuniquename
    import Pmakemonfile
#   validation will be handled by Pqautorelax
#   convert inputs into strings
    try:
        fullpath = str(fullpath)
    except (TypeError, ValueError):
        print fullpath, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        checksec = str(checksec)
    except (TypeError, ValueError):
        print checksec, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        lagsec = str(lagsec)
    except (TypeError, ValueError):
        print lagsec, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        emaddr = str(emaddr)
    except (TypeError, ValueError):
        print emaddr, 'not valid.'
        print '(Exiting.)'
        return None
    try:
        isneb = str(isneb)
    except (TypeError, ValueError):
        print isneb, 'not valid.'
        print '(Exiting.)'
        return None
#   set up output file
    filestr=""
    fileme=""
    filestr = Pmakemonfile.main(fullpath)
    if (filestr == None):
        print 'Could not create monitor file.'
        print '(Exiting.)'
        return None
    fileme = open(filestr, 'ab')
#   run process
    hidemon=""
    hidemon = subprocess.Popen(['Uwrapper.py', 'Prunone',
            fullpath, '0', checksec, lagsec, 
            emaddr, isneb],
            stdout=fileme, stderr=fileme)
    hidemon.wait() # THIS LINE differentiates from Pdirautorelaxsilent
#    hidemon = subprocess.Popen([sys.executable,
#            'Uwrapper.py', 'Pqautorelax',
#            fullpath, numatoms, maxout, checksec, checklag, 
#            encomp, entype, volcomp, voltype],
#            stdout=fileme, stderr=fileme)
    fileme.close()
    print hidemon
    return True
