#!/usr/bin/env python
##
#@package   Pstrain 
#@ingroup   vasp
#@ingroup   python
#@brief     copy directories with strain in lattice params
#@author    Tam Mayeshiba
#@date      6/11/12
###
def main(topdir, strainmatrix, namematrix):
    """
    Purpose: Copy directories with strain in lattice params
        inputs:  
            [$1] top directory
            example: //home/tam/NaClcollection
            [$2] strain string in percent and lattice param a, b, c directions
            delimited by spaces and semicolons
            example: "-4 -4 0; -2 -2 0; 1 1 0"
            [$3] append for each strain criterion
            example: "stneg4;stneg2;st1"
        outputs:
            directories with strain
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Xvector
    import Vrelax
    import Platticevec
    import shutil
    import Pmodfilenew
    import Xuniquename
#   Validation
#   #  
    newdir = Xuniquename.main(topdir, 'strser')
    topdir = Xvaliddirectory.main(topdir)
    if topdir == None:
        return None
#   Get strain and append matrix
    indivstrain=[]
    indivapp=[]
    indivstrain = strainmatrix.split(';')
    indivapp = namematrix.split(';')
    if not (len(indivstrain) == len(indivapp)):
        print 'Number of strains does not match up with number of appends.'
        return None
    dct = 0
    initdirs=[]
    initdirs = Xgetdirs.main(topdir)
    try:
        os.mkdir(newdir)
    except OSError:
        print 'Error creating directory.'
        return None
    myidir=""
    namestr=""
    fullname=""
    lvecmat=[]
    strvec=[]
    newvstr=""
    for myidir in initdirs:
        dct = 0
        while dct < len(indivstrain):
            namestr = os.path.basename(myidir) + indivapp[dct].strip()
            fullname = os.path.join(newdir, namestr)
            if not os.path.isfile(myidir + "/CONTCAR"):
                try:
                    os.mkdir(fullname)
                    shutil.copy(myidir + "/POSCAR",fullname)
                    shutil.copy(myidir + "/POTCAR",fullname)
                    shutil.copy(myidir + "/INCAR",fullname)
                    shutil.copy(myidir + "/KPOINTS",fullname)
                except IOError:
                    'Could not copy.'
            else:
                Vrelax.main(myidir, newdir, namestr)
            lvecmat = Platticevec.main(fullname, 'POSCAR')
            shutil.copy(fullname + '/POSCAR', fullname + '/unstr_POSCAR')
            strvec = Xvector.strtoflt(indivstrain[dct].strip())
            newvstr=""
            newvstr= newvstr + str(lvecmat[0][0] * ((strvec[0] * 0.01) + 1)) + '   '
            newvstr= newvstr + str(lvecmat[0][1] * ((strvec[0] * 0.01) + 1)) + '   '
            newvstr= newvstr + str(lvecmat[0][2] * ((strvec[0] * 0.01) + 1)) + '   '
            Pmodfilenew.main(fullname, 'POSCAR', 0, 3, 'R', newvstr)
            newvstr=""
            newvstr= newvstr + str(lvecmat[1][0] * ((strvec[1] * 0.01) + 1)) + '   '
            newvstr= newvstr + str(lvecmat[1][1] * ((strvec[1] * 0.01) + 1)) + '   '
            newvstr= newvstr + str(lvecmat[1][2] * ((strvec[1] * 0.01) + 1)) + '   '
            Pmodfilenew.main(fullname, 'POSCAR', 0, 4, 'R', newvstr)
            newvstr=""
            newvstr= newvstr + str(lvecmat[2][0] * ((strvec[2] * 0.01) + 1)) + '   '
            newvstr= newvstr + str(lvecmat[2][1] * ((strvec[2] * 0.01) + 1)) + '   '
            newvstr= newvstr + str(lvecmat[2][2] * ((strvec[2] * 0.01) + 1)) + '   '
            Pmodfilenew.main(fullname, 'POSCAR', 0, 5, 'R', newvstr)
            dct = dct + 1
    return True

