#!/usr/bin/env python
##
#@package   Pstrainsingle 
#@ingroup   vasp
#@ingroup   python
#@brief     copy directory with strain in lattice params
#@author    Tam Mayeshiba
#@date      7/4/12
###
def main(myidir, strainmatrix, namematrix, existdir):
    """
    Purpose: Copy directories with strain in lattice params
        inputs:  
            [$1] directory
            example: //home/tam/NaClcollection/NaCl
            [$2] strain string in percent and lattice param a, b, c directions
            delimited by spaces and semicolons
            example: "-4 -4 0; -2 -2 0; 1 1 0"
            [$3] append for each strain criterion
            example: "stneg4;stneg2;st1"
            [$4] existing directory in which to insert strained case
        outputs:
            directories with strain
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Xvector
    import Vrelax
    import Platticevec
    import shutil
    import Pmodfilenew
#   Validation
#   #  
    existdir = Xvaliddirectory.main(existdir)
    if existdir == None:
        return None
    myidir = Xvaliddirectory.main(myidir)
    if myidir == None:
        return None
#   Get strain and append matrix
    indivstrain=[]
    indivapp=[]
    indivstrain = strainmatrix.split(';')
    indivapp = namematrix.split(';')
    if not (len(indivstrain) == len(indivapp)):
        print 'Number of strains does not match up with number of appends.'
        return None
    dct = 0
    namestr=""
    fullname=""
    lvecmat=[]
    strvec=[]
    newvstr=""
    dct = 0
    while dct < len(indivstrain):
        namestr = os.path.basename(myidir) + indivapp[dct].strip()
        fullname = os.path.join(existdir, namestr)
        if not os.path.isfile(myidir + "/CONTCAR"):
            try:
                os.mkdir(fullname)
                shutil.copy(myidir + "/POSCAR",fullname)
                shutil.copy(myidir + "/POTCAR",fullname)
                shutil.copy(myidir + "/INCAR",fullname)
                shutil.copy(myidir + "/KPOINTS",fullname)
            except IOError:
                'Could not copy.'
        else:
            Vrelax.main(myidir, existdir, namestr)
        lvecmat = Platticevec.main(fullname, 'POSCAR')
        shutil.copy(fullname + '/POSCAR', fullname + '/unstr_POSCAR')
        strvec = Xvector.strtoflt(indivstrain[dct].strip())
        newvstr=""
        newvstr= newvstr + str(lvecmat[0][0] * ((strvec[0] * 0.01) + 1)) + '   '
        newvstr= newvstr + str(lvecmat[0][1] * ((strvec[0] * 0.01) + 1)) + '   '
        newvstr= newvstr + str(lvecmat[0][2] * ((strvec[0] * 0.01) + 1)) + '   '
        Pmodfilenew.main(fullname, 'POSCAR', 0, 3, 'R', newvstr)
        newvstr=""
        newvstr= newvstr + str(lvecmat[1][0] * ((strvec[1] * 0.01) + 1)) + '   '
        newvstr= newvstr + str(lvecmat[1][1] * ((strvec[1] * 0.01) + 1)) + '   '
        newvstr= newvstr + str(lvecmat[1][2] * ((strvec[1] * 0.01) + 1)) + '   '
        Pmodfilenew.main(fullname, 'POSCAR', 0, 4, 'R', newvstr)
        newvstr=""
        newvstr= newvstr + str(lvecmat[2][0] * ((strvec[2] * 0.01) + 1)) + '   '
        newvstr= newvstr + str(lvecmat[2][1] * ((strvec[2] * 0.01) + 1)) + '   '
        newvstr= newvstr + str(lvecmat[2][2] * ((strvec[2] * 0.01) + 1)) + '   '
        Pmodfilenew.main(fullname, 'POSCAR', 0, 5, 'R', newvstr)
        dct = dct + 1
    return True

