#!/usr/bin/env python
##
#@package   Psumeg 
#@ingroup   vasp
#@ingroup   python
#@brief     sum electrons. Uses Wei Xie's psvasp and matlab occps.m
#@author    Tam Mayeshiba
#@date      9/13/11
#@date      9/14/11 remove default existdir so that error-checking works
###
def main(existdir, specstr="", colstr="", onlyspin=0):
    """
    Purpose: sum electrons. Uses Wei Xie's psvasp and matlab occps.m
        inputs:  
            [$1] directory
            example: /home/tam/NaCl
            [$2] species list, period-delimited
            example: "Na.K.Li.Cl"
            [$3] columns to sum (corresponds to orbitals): period-delimited
            example: "3.5" to sum columns 3 and 5
            [$4] onlyspin
                0 - take both spins (default)
                1 - take spin component 1 only
                2 - take spin component 2 only
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Xgetfiles
#   Validation
#   #  existdir
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
    fullpath=""
    fullpath = os.path.join(existdir, 'OCC_parsed') #name is hardcoded
    if not os.path.exists(fullpath):
        print 'No occps data to parse.' 
        print '(Exiting.)'
        return None
#   #   onlyspin
    onlyspin = Xvalidinteger.main(onlyspin, 0, 2)
    if (onlyspin == None):
        print '(Exiting.)'
        return None
#   #   species list
    speclist=[]
    speclist = specstr.split('.')
    if len(speclist) == 0:
        print 'Species list is empty.'
        print '(Exiting.)'
        return None
#   #   column list
    collist=[]
    collist = str(colstr).split('.')
    if len(collist) == 0:
        print 'Column list is empty.'
        print '(Exiting.)'
        return None
    colitem=""
    for colitem in collist:
        if not colitem.isdigit():
            print colitem, 'is not a number.'
            print '(Exiting.)'
            return None
#   get files
    filelist=[]
    filelist = Xgetfiles.main(fullpath)
    if (filelist == None):
        print 'Error retrieving files in', fullpath
        print '(Exiting.)'
        return None
    if len(filelist) == 0:
        print 'No files in', fullpath
        print '(Exiting.)'
        return None
#   loop through list looking for appropriate file
    fileitem=""
    shortname=""
    namescore=[]
    openfile=""
    readbl=""
    sumline=""
    sumlist=[]
    colnum=0
    colflt=0
    sumflt=0
    retcolstr=""
    retstr=""
    for fileitem in filelist:
        if '.csv' not in fileitem:
            continue #skip this file
        shortname = os.path.basename(fileitem)[:-4] # all named .csv
        namecore = shortname.split('_') # Al_up
        if not (namecore[0] in speclist):
            continue #skip this file
        if (onlyspin == 1) and (namecore[1] != 'up'):
            continue # not spin up
        elif (onlyspin == 2) and (namecore[1] != 'dn'):
            continue # not spin down
        elif (onlyspin == 0) and (namecore[1] != 'sum'):
            continue # not sum of spin up and spin down
        openfile = open(fileitem, 'rb')
        readbl = openfile.readlines()
        openfile.close()
        if (readbl == None):
            print 'No block. Continuing to next file.'
            continue
        bllen=0
        bllen = len(readbl)
        if bllen == 0:
            print 'Block is length 0. Continuing to next file.'
            continue
        sumline = readbl[bllen-1]
        sumlist = sumline.split(',') # comma-delimited
        sumflt = 0
        colitem=""
        for colitem in collist:
            colnum = int(colitem) # should be all digits, so int should work
            if (colnum < len(sumlist)):
                try:
                    colflt = float(sumlist[colnum - 1])
                except (ValueError, TypeError):
                    print sumlist[colnum - 1], 'is not a float.'
                    print 'Continuing to next file.'
                    continue
                retcolstr = retcolstr + colitem + ' '
                sumflt = sumflt + colflt
        retstr = retstr + fileitem + ' Cols: ' + retcolstr + ' Occ: ' 
        retstr = retstr + str(sumflt) + '\n'
    print retstr
    return retstr
#
#
def loop(existdir, specstr="", colstr="", onlyspin=0):
    """
    Purpose: sum electrons. Uses Wei Xie's psvasp and matlab occps.m
        inputs:  
            [$1] directory in which to loop
            example: /home/tam/NaClcollection
            [$2] species list, period-delimited
            example: "Na.K.Li.Cl"
            [$3] columns to sum (corresponds to orbitals): period-delimited
            example: "3.5" to sum columns 3 and 5
            [$4] onlyspin
                0 - take both spins (default)
                1 - take spin component 1 only
                2 - take spin component 2 only
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Psumeg
#   Validation will be handled by main
#   #  existdir
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   get directories
    dirlist=[]
    dirlist = Xgetdirs.main(existdir)
    if (dirlist == None):
        print 'Error getting directories in', existdir
        print '(Exiting.)'
        return None
    if len(dirlist) == 0:
        print 'No directories under', existdir
        print '(Exiting.)'
        return None
    diritem=""
    dirreturn=""
    retlist=[]
    retitem=""
    for diritem in dirlist:
        dirreturn = Psumeg.main(diritem, specstr, colstr, onlyspin)
        retlist.append(dirreturn)
    for retitem in retlist:
        print retitem
    return retlist


