#!//usr/bin/env python
##
#@package  Rdparse 
#@ingroup   
#@ingroup   python
#@brief     Rotations for d-orbitals
#@author    Tam Mayeshiba
#@date      2013-04-30
###
def parse_up_or_down_file(occfname, occatoms=[], l=2):
    """Parse up or down occupation file.
        occfname = occupation file name
        occatoms = list of atom numbers corresponding to the lines
                in the occupation file
        l = quantum number l (2 for d-orbitals)
    """
    myfile = open(occfname, 'rb')
    mylines = myfile.readlines()
    occdict=dict()
    linect=0
    while linect < len(occatoms):
        myline = mylines[linect].strip()
        linesplit = myline.split(',')
        pcct = 0
        newstr = ""
        while pcct < (2*l + 1):
            newstr = newstr + linesplit[pcct] + " " 
            pcct = pcct + 1
        occdict[occatoms[linect]] = newstr
        linect = linect + 1
    print "OCCDICT: ", occdict
    return occdict

def rotate_up_or_down_file(occfname, posfname, occatoms=[], l=2):
    """Rotate up or down occupation file.
        occfname = occupation file name
        posfname = POSCAR-type file name
        occatoms = list of atom numbers corresponding to the lines
            in the occupation file
        l = quantum number l (2 for d-orbitals)
        These files are in OCC_parsed generated by Wei Xie's script
        from the occupancies and eigenvalues VASP matrix
    """
    if len(occatoms) == 0:
        occatoms = range(9,17)
    #parse occupation file
    myoccs = parse_up_or_down_file(occfname, occatoms, l)
    import Rgetcoords
    coordset=Rgetcoords.main(posfname, 0.05)
    oextend=Rgetcoords.extend_cell(coordset[2])
    octdict=Rgetcoords.build_octahedra(coordset[1],oextend)
    myrealrotated=dict()
    import Rdrotate
    angledict=dict()
    for occatom in occatoms:
        getlines=Rgetcoords.get_lines_from_octahedron(octdict,occatom)
        if len(getlines) > 0 and len(getlines[0]) > 0:
            #avgrots=Rgetcoords.get_line_tilts_from_octahedron(getlines[0],getlines[1])
            #xrot=None
            #yrot=None
            #zrot=None
            #if 'xrot' in avgrots.keys():
            #    xrot = avgrots['xrot']
            #if 'yrot' in avgrots.keys():
            #    yrot = avgrots['yrot']
            #if 'zrot' in avgrots.keys():
            #    zrot = avgrots['zrot']
            #if (xrot and yrot and zrot):
            #    myrotated = Rdrotate.rotatereal(l, zrot, xrot, yrot,
            #        myoccs[occatom])
            #    myrealrotated[occatom] = myrotated
            alphabetagamma = Rgetcoords.get_euler_angles_from_octahedron(getlines[0],getlines[1])
            myrotated = Rdrotate.rotatereal(l, alphabetagamma[0],
                alphabetagamma[1], alphabetagamma[2], myoccs[occatom])
            myrealrotated[occatom] = myrotated
            angledict[occatom]=alphabetagamma
    print "ROTATEDOCT: "
    for key in myrealrotated.keys():
        print key, ":", myrealrotated[key]
    print "ANGLEDICT: "
    for key in angledict.keys():
        print key, ":", angledict[key]
    return myrealrotated

def get_rotated_total(occfnameup,occfnamedown,posfname,occatoms=[],l=2):
    myrotatedup = rotate_up_or_down_file(occfnameup, posfname,
                        occatoms, l)
    myrotateddown = rotate_up_or_down_file(occfnamedown, posfname,
                        occatoms, l)
    absrotated=dict()
    import math
    for key in myrotatedup.keys():
        if key in myrotateddown.keys():
            absrotated[key]=dict()
            for orbital in myrotatedup[key].keys():
                absrotated[key][orbital] = math.fabs(myrotatedup[key][orbital]) + math.fabs(myrotateddown[key][orbital])
    print "UP+DOWN, ROTATED:", absrotated
    display_absrotated_d(absrotated)
    return absrotated

def display_absrotated_d(absrotated):
    print "atom: dxy dyz dz2 dxz dx2"
    for atom in range(9,17):
        mystr = str(atom) + ": "
        if atom in absrotated.keys():
            for orbital in range(-2,3):
                mystr = mystr + str(absrotated[atom][orbital]) + " "
        print mystr

