#!/bin/env/python
#Author: Tam Mayeshiba
#Date: 2013-04-18
import numpy as np
import math

def main(fname, tolerance=0.0):
    """Get coordinates from POSCAR
        fname = POSCAR (or CONTCAR) file name
        tolerance: edge tolerance for adding/subtracting back to 
                    periodic fraction (e.g. if this is 0.05, then 
                    since 0.97 is within 0.05 of 1, 0.97 will become
                    -0.03
        assumes A B O ordering (e.g. 8 8 24)
    """
    myfile=open(fname,'rb')
    mylines = myfile.readlines()
    myfile.close()
    myspecs=mylines[6].split()
    numa=int(myspecs[0])
    numb=int(myspecs[1])
    numo=int(myspecs[2])
    numatoms = numa + numb + numo
    atomlines=mylines[8:numatoms+8]
    onesplit=[]
    onecoord=[]
    atomcoords=[]
    coord=""
    fltcoord=0.0
    for oneatom in atomlines:
        onecoord=[]
        onesplit = oneatom.split()[:3] #only the first 3 values
        for coord in onesplit:
            fltcoord = float(coord)
            if fltcoord > 1 - tolerance:
                onecoord.append(fltcoord - 1)
            elif fltcoord < 0 - tolerance:
                onecoord.append(fltcoord + 1)
            else:
                onecoord.append(fltcoord)
        atomcoords.append(onecoord)
    adict=dict()
    bdict=dict()
    odict=dict()
    for count in range(1,numa+1):
        adict[count]=atomcoords[count-1]
    for count in range(numa+1, numa+numb+1):
        bdict[count]=atomcoords[count-1]
    for count in range(numa+numb+1, numatoms+1):
        odict[count]=atomcoords[count-1]
    print adict
    print bdict
    print odict
    return [adict, bdict, odict]

def build_octahedra(bdict, odict, fracbond=0.3):
    octdict=dict()
    ocoord=""
    bcoord=""
    dist=0.0
    for bkey in bdict.keys():
        octdict[bkey]=dict()
        octdict[bkey][bkey]=bdict[bkey]
        bcoord=np.array([bdict[bkey]],float)
        for okey in odict.keys():
            ocoord=np.array([odict[okey]],float)
            dist = np.linalg.norm(bcoord-ocoord)
            if dist <= fracbond:
                octdict[bkey][okey]=odict[okey]
    for key in octdict.keys():
        print key, ":", octdict[key].keys()
    #print octdict
    return octdict

def print_back_out(origfname, octrotated, nameout, adict, bdict):
    myfile=open(origfname,'rb')
    mylines = myfile.readlines()
    myfile.close()
    mynewlines=mylines[0:8]
    foundlist=[]
    mystr=""
    for akey in adict.keys():
        mystr=""
        for value in adict[akey]:
            mystr = mystr + str(value) + " "
        mystr = mystr + "A" + str(akey) + '\n'
        mynewlines.append(mystr) 
    for bkey in bdict.keys():
        mystr=""
        for value in bdict[bkey]:
            mystr = mystr + str(value) + " "
        mystr = mystr + "B" + str(bkey) + '\n'
        mynewlines.append(mystr) 
        foundlist.append(bkey)
    for octkey in octrotated.keys():
        for ionkey in octrotated[octkey].keys():
            if not ionkey in foundlist:
                foundlist.append(ionkey)
                mystr=""
                myarr = octrotated[octkey][ionkey]
                mystr = mystr + str(myarr[0][0]) + " "
                mystr = mystr + str(myarr[0][1]) + " "
                mystr = mystr + str(myarr[0][2]) + " "
                mystr = mystr + "O" + str(ionkey) + '\n'
                mynewlines.append(mystr) 
    #print mynewlines
    newfile = open(nameout, 'wb')
    newfile.writelines(mynewlines)
    newfile.close()
    return mynewlines

def comparecoords(dictset1, dictset2):
    """Compare two dictionaries of coordinates, from Rgetcoords.main
        Return: average and stdev of fractional distance between each
                coordinate, per dictionary, and total

    """
    import numpy as np
    if not len(dictset1) == len(dictset2):
        print "Lists of dictionaries are not the same length."
        return None
    dct=0
    avgs=[]
    stdevs=[]
    alldist=dict()
    while dct < len(dictset1):
        if not (dictset1[dct].keys().sort() == dictset2[dct].keys().sort()):
            print "Dictionaries at index", dct, "do not have the same keys."
            return None
        mydists=[]
        for key in dictset1[dct].keys():
            coord1=np.array([dictset1[dct][key]],float)
            coord2=np.array([dictset2[dct][key]],float)
            distpair=np.linalg.norm(coord1-coord2)
            alldist[key]=distpair
            mydists.append(distpair)
            distarr=np.array([mydists],float)
        print mydists
        avgs.append(np.mean(distarr))
        stdevs.append(np.std(distarr))
        dct = dct + 1
    return [avgs,stdevs]

def get_lines_from_octahedron(octdict, whichoct, tolerance=0.01):
    """Get lines from an octahedron"""
    onedict=octdict[whichoct]
    centerdict=dict() #center around B-site cation
    coordb = np.array([onedict[whichoct]],float)
    for key in onedict.keys():
        coordo = np.array([onedict[key]],float)
        centerdict[key] = coordo - coordb
    print centerdict
    pairdict=dict()
    for key1 in centerdict.keys():
        if key1 == whichoct:
            pass
        else:
            key1x = centerdict[key1][0][0]
            key1y = centerdict[key1][0][1]
            key1z = centerdict[key1][0][2]
            for key2 in centerdict.keys():
                pairme=0
                if key2 == whichoct:
                    pass
                elif key1 == key2:
                    pass
                else: #add together, because should be on opp sides of zero
                    key2x = centerdict[key2][0][0]
                    key2y = centerdict[key2][0][1]
                    key2z = centerdict[key2][0][2]
                    if math.fabs(key1x + key2x) <= tolerance:
                        pairme = pairme + 1
                    if math.fabs(key1y + key2y) <= tolerance:
                        pairme = pairme + 1
                    if math.fabs(key1z + key2z) <= tolerance:
                        pairme = pairme + 1
                    if pairme == 3:
                        if math.fabs(key1x) > math.fabs(key1y) and math.fabs(key1x) > math.fabs(key1z):
                            if not 'x' in pairdict.keys():
                                pairdict['x'] = [key1, key2]
                        elif math.fabs(key1y) > math.fabs(key1x) and math.fabs(key1y) > math.fabs(key1z):
                            if not 'y' in pairdict.keys():
                                pairdict['y'] = [key1, key2]
                        elif math.fabs(key1z) > math.fabs(key1x) and math.fabs(key1z) > math.fabs(key1y):
                            if not 'z' in pairdict.keys():
                                pairdict['z'] = [key1, key2]
    #fill in the missing gaps
    for pairdir in ['x','y','z']:
        if not pairdir in pairdict.keys():
            maxdir=0
            maxdiratom=0
            mindir=0
            mindiratom=0
            for key in centerdict.keys():
                if key == whichoct:
                    pass
                else:
                    if pairdir == 'x':
                        keydir = centerdict[key][0][0]
                    elif pairdir == 'y':
                        keydir = centerdict[key][0][1]
                    else:
                        keydir = centerdict[key][0][2]
                    if keydir > maxdir:
                        maxdir = keydir
                        maxdiratom = key
                    elif keydir < mindir:
                        mindir = keydir
                        mindiratom = key
                if math.fabs(maxdir) > math.fabs(mindir): #one is close to 0.25 vs. 0.0
                    pairdict[pairdir] = [whichoct, maxdiratom]
                else:
                    pairdict[pairdir] = [whichoct, mindiratom]
    print pairdict
    return [pairdict, centerdict]

def get_line_tilts_from_octahedron(pairdict, centerdict):
    """Get line tilts from octahderon"""
    # counterclockwise z-rotation = phi ; tan(phi) = y/x
    allrots=dict()
    allrots['xrot']=list()
    allrots['yrot']=list()
    allrots['zrot']=list()
    smallrots=dict()
    smallrots['xrot']=list()
    smallrots['yrot']=list()
    smallrots['zrot']=list()
    for key in pairdict.keys():
        mycoord=centerdict[pairdict[key][0]]-centerdict[pairdict[key][1]]
        myx=mycoord[0][0]
        myy=mycoord[0][1]
        myz=mycoord[0][2]
        tan_zrot = myy/myx
        zrot = np.arctan(tan_zrot)*180/np.pi
        tan_yrot = myx/myz
        yrot = np.arctan(tan_yrot)*180/np.pi
        tan_xrot = myz/myy
        xrot = np.arctan(tan_xrot)*180/np.pi
        if key == 'x':
            allrots['yrot'].append(yrot)
            allrots['zrot'].append(zrot)
        elif key == 'y':
            allrots['xrot'].append(xrot)
            allrots['zrot'].append(zrot)
        elif key == 'z':
            allrots['xrot'].append(xrot)
            allrots['yrot'].append(yrot)
    for key in allrots.keys():
        for myangle in allrots[key]: 
            if math.fabs(myangle) > 45:
                if myangle < 0:
                    myangle = myangle + 90
                else:
                    myangle = myangle - 90
            smallrots[key].append(myangle)
    avgrots=dict()
    for key in smallrots.keys():
        if len(smallrots[key]) > 0:
            avgrots[key] = np.average(smallrots[key])
    print "All rotations: ",allrots
    print "Normalized so angles under 45 degrees: ", smallrots
    print "Averages: ", avgrots
    return avgrots

def extend_cell(mydict, tol=0.2):
    """Extend a dictionary (odict) so that all octahedra are full."""
    for key in mydict.keys():
        makenew=0
        myx=mydict[key][0]
        myy=mydict[key][1]
        myz=mydict[key][2]
        newx=myx
        newy=myy
        newz=myz
        if myx < tol:
            makenew = makenew + 1
            newx = myx + 1
        elif myx > (1-tol):
            makenew = makenew + 1
            newx = myx - 1
        if myy < tol:
            makenew = makenew + 1
            newy = myy + 1
        elif myy > (1-tol):
            makenew = makenew + 1
            newy = myy - 1
        if myz < tol:
            makenew = makenew + 1
            newz = myz + 1
        elif myz > (1-tol):
            makenew = makenew + 1
            newz = myz - 1
        if makenew > 0:
            mydict[key+1000000]=[newx, newy, newz]
    return mydict

def get_euler_angles_from_octahedron(pairdict, centerdict):
    """Get Euler tilts from octahderon.
        [$1] dictionary of pairs along closest axes ['x']=[20,21], etc.
        [$2] dictionary of coordinates, centered so B-cation is at (0,0,0)
    """
    # alpha = arccos(-Z2/sqrt(1-(Z3)^2))
    # beta = arccos(Z3)
    # gamma = arccos(Y3/sqrt(1-(Z3)^2))
    # Let's assume these octahedra have perfectly orthogonal axes.
    # Make long vectors out of each pair. One pair in a defected cell
    # might be made up of the center cation and an anion.
    mycoordsX = centerdict[pairdict['x'][0]] - centerdict[pairdict['x'][1]]
    mycoordsY = centerdict[pairdict['y'][0]] - centerdict[pairdict['y'][1]]
    mycoordsZ = centerdict[pairdict['z'][0]] - centerdict[pairdict['z'][1]]
    if mycoordsX[0][0] < 0: #ensure a right-hand coordinate system
        if np.absolute(mycoordsX[0][0]) < 0.00001:
            mycoordsX[0][0] = 0
        else:
            mycoordsX[0][0] = -1*mycoordsX[0][0]
        if np.absolute(mycoordsX[0][1]) < 0.00001:
            mycoordsX[0][1] = 0
        else:
            mycoordsX[0][1] = -1*mycoordsX[0][1]
        if np.absolute(mycoordsX[0][2]) < 0.00001:
            mycoordsX[0][2] = 0
        else:
            mycoordsX[0][2] = -1*mycoordsX[0][2]
    if mycoordsY[0][1] < 0:
        if np.absolute(mycoordsY[0][0]) < 0.00001:
            mycoordsY[0][0] = 0
        else:
            mycoordsY[0][0] = -1*mycoordsY[0][0]
        if np.absolute(mycoordsY[0][1]) < 0.00001:
            mycoordsY[0][1] = 0
        else:
            mycoordsY[0][1] = -1*mycoordsY[0][1]
        if np.absolute(mycoordsY[0][2]) < 0.00001:
            mycoordsY[0][2] = 0
        else:
            mycoordsY[0][2] = -1*mycoordsY[0][2]
    if mycoordsZ[0][2] < 0:
        if np.absolute(mycoordsZ[0][0]) < 0.00001:
            mycoordsZ[0][0] = 0
        else:
            mycoordsZ[0][0] = -1*mycoordsZ[0][0]
        if np.absolute(mycoordsZ[0][1]) < 0.00001:
            mycoordsZ[0][1] = 0
        else:
            mycoordsZ[0][1] = -1*mycoordsZ[0][1]
        if np.absolute(mycoordsZ[0][2]) < 0.00001:
            mycoordsZ[0][2] = 0
        else:
            mycoordsZ[0][2] = -1*mycoordsZ[0][2]
    print "UNNORMALIZED: ",mycoordsX,mycoordsY,mycoordsZ
    mycoordsX = mycoordsX/np.linalg.norm(mycoordsX) #normalize to 1
    mycoordsY = mycoordsY/np.linalg.norm(mycoordsY)
    mycoordsZ = mycoordsZ/np.linalg.norm(mycoordsZ)
    #NEW: Round these to 4 decimal places
    mycoordsX = np.round(mycoordsX,4)
    mycoordsY = np.round(mycoordsY,4)
    mycoordsZ = np.round(mycoordsZ,4)
    print "OCT COORDS: ", mycoordsX, mycoordsY, mycoordsZ
    myZ3 = mycoordsZ[0][2]
    myZ2 = mycoordsZ[0][1]
    myZ1 = mycoordsZ[0][0]
    myY3 = mycoordsY[0][2]
    myX3 = mycoordsX[0][2]
    print "X Y CROSS: ", np.cross(mycoordsX, mycoordsY)
    #myalpha = np.arccos(-1*myZ2/np.sqrt(1-(np.power(myZ3,2))))*180/np.pi
    mybeta = np.arccos(myZ3)*180/np.pi
    #mygamma = np.arccos(myY3/np.sqrt(1-(np.power(myZ3,2))))*180/np.pi
    #print "As-is alpha, beta, gamma: ",myalpha, mybeta, mygamma
    #alpha, gamma angles are too large.
    #alpha = atan2(Z1, -Z2)
    #gamma = atan2(X3, Y3)
    if np.absolute(myZ2) < 0.00001:
        newalpha = np.arctan2(myZ1, myZ2)
    else:
        newalpha = np.arctan2(myZ1, -1*myZ2)
    newgamma = np.arctan2(myX3, myY3)
    #print "My alpha, beta, gamma: ",newalpha, mybeta, newgamma
    #newalpha, newgamma angles are too small now. Forgot to turn
    # these back into degrees! No wonder.
    newalpha = newalpha*180/np.pi
    newgamma = newgamma*180/np.pi
    #But now they are too large again.
    print "My alpha, beta, gamma: ",newalpha, mybeta, newgamma
    normangles=[]
    myangles=[newalpha, mybeta, newgamma]
    #for angle in myangles:
    #    if angle > 90:
    #        normangles.append(angle-180)
    #    elif angle > 45:
    #        normangles.append(angle-90)
    #    else:
    #        normangles.append(angle)
    #print "Normalized alpha, beta, gamma: ", normangles
    #return normangles
    return myangles

def get_euler_angle_matrix_wadef_bad(alpha, beta, gamma):
    """Get euler angle matrix
        [$1] alpha in degrees
        [$2] beta in degrees
        [$3] gamma in degrees
        Returns: 
            numpy array, 3x3
    """
    #a_(11) =   cospsicosphi-costhetasinphisinpsi   
    #a_(12)  =   cospsisinphi+costhetacosphisinpsi   
    #a_(13)  =   sinpsisintheta  
    #a_(21)  =   -sinpsicosphi-costhetasinphicospsi  
    #a_(22)  =   -sinpsisinphi+costhetacosphicospsi  
    #a_(23)  =   cospsisintheta  
    #a_(31)  =   sinthetasinphi  
    #a_(32)  =   -sinthetacosphi 
    #a_(33)  =   costheta
    # where alpha = phi, beta = theta, and gamma = psi
    #from Wolfram alpha
    #however, their definition uses clockwise rotation around
    #axis as positive. Euler angles use COUNTERclockwise rotation
    #looking down the axis from the positive direction towards
    #zero as positive.
    try:
        phi = -1*float(alpha)*np.pi/180 #-1 to switch to ccwise
        theta = -1*float(beta)*np.pi/180
        psi = -1*float(gamma)*np.pi/180
    except ValueError, TypeError:
        print "alpha, beta, or gamma could not be converted to radians."
        return
    amat=np.zeros([3,3],float)
    amat[0][0] = np.cos(psi)*np.cos(phi)-(np.cos(theta)*np.sin(phi)*np.sin(psi))   
    amat[0][1] = np.cos(psi)*np.sin(phi)+(np.cos(theta)*np.cos(phi)*np.sin(psi))   
    amat[0][2] = np.sin(psi)*np.sin(theta)
    amat[1][0] = -1*np.sin(psi)*np.cos(phi)-(np.cos(theta)*np.sin(phi)*np.cos(psi))  
    amat[1][1] = -1*np.sin(psi)*np.sin(phi)+(np.cos(theta)*np.cos(phi)*np.cos(psi)) 
    amat[1][2] = np.cos(psi)*np.sin(theta)  
    amat[2][0] = np.sin(theta)*np.sin(phi)  
    amat[2][1] = -1*np.sin(theta)*np.cos(phi) 
    amat[2][2] = np.cos(theta)
    return amat

def apply_euler_angles_octa(alpha, beta, gamma):
    """Apply Euler angles to a fake octahedron.
        [$1] alpha in degrees
        [$2] beta in degrees
        [$3] gamma in degrees
        Returns: 
            row coordinates
    """
    eulermat = get_euler_angle_matrix(alpha, beta, gamma)
    returncoords=[]
    for mycoordstr in ["0.25 0 0","-0.25 0 0","0 0.25 0",
            "0 -0.25 0","0 0 0.25","0 0 -0.25"]:
        mycoord = np.array([mycoordstr.split()],float)
        endcoord = np.transpose(np.dot(eulermat,
                    np.transpose(mycoord))) 
        retstr=""
        for value in endcoord[0]:
            retstr = retstr + str(value) + " "
        returncoords.append(retstr)
    for mystr in returncoords:
        print mystr
    return returncoords

def apply_euler_angles_coordsys(alpha, beta, gamma):
    """Apply Euler angles to a unit-vector coordinate system.
        [$1] alpha in degrees
        [$2] beta in degrees
        [$3] gamma in degrees
        Returns: 
            row coordinates
    """
    eulermat = get_euler_angle_matrix(alpha, beta, gamma)
    returncoords=[]
    for mycoordstr in ["1 0 0","0 1 0","0 0 1"]:
        mycoord = np.array([mycoordstr.split()],float)
        endcoord = np.transpose(np.dot(eulermat,
                    np.transpose(mycoord))) 
        retstr=""
        for value in endcoord[0]:
            retstr = retstr + str(value) + " "
        returncoords.append(retstr)
    for mystr in returncoords:
        print mystr
    return returncoords

def get_euler_angle_matrix(alpha, beta, gamma):
    """Get euler angle matrix
        [$1] alpha in degrees
        [$2] beta in degrees
        [$3] gamma in degrees
        Returns: 
            numpy array, 3x3
    """
    #using wikipedia definition: Z1, X2, Z3
    #c1c3 - c2s1s3, -c1s3-c2c3s1, s1s2
    #c3s1+c1c2s3, c1c2c3-s1s3, -c1s2
    #s2s3, c3s2, c2
    # where alpha = Z1, beta = X2, and gamma = Z3 rotations
    try:
        arad = float(alpha)*np.pi/180 
        brad = float(beta)*np.pi/180
        grad = float(gamma)*np.pi/180
    except ValueError, TypeError:
        print "alpha, beta, or gamma could not be converted to radians."
        return
    c1 = np.cos(arad)
    c2 = np.cos(brad)
    c3 = np.cos(grad)
    s1 = np.sin(arad)
    s2 = np.sin(brad)
    s3 = np.sin(grad)
    amat=np.zeros([3,3],float)
    amat[0][0] = c1*c3 - (c2*s1*s3)
    amat[0][1] = -1*(c1*s3) - (c2*c3*s1)
    amat[0][2] = s1*s2
    amat[1][0] = c3*s1 + (c1*c2*s3)
    amat[1][1] = c1*c2*c3 - s1*s3
    amat[1][2] = -c1*s2
    amat[2][0] = s2*s3
    amat[2][1] = c3*s2
    amat[2][2] = c2
    return amat
