#!/bin/env/python
#Author: Tam Mayeshiba
#Date: 2013-05-03
import numpy as np
import math
import os

def get_occ_and_ev(opath):
    """Get occupancies and eigenvalues matrices from outcar
        opath = OUTCAR path
    """
    if not os.path.exists(opath):
        print "No file at ", opath
        return
    myfile = open(opath, 'rb')
    mylines = myfile.readlines()
    myfile.close()
    okay = 0 #want to make sure whole file is transmitted.
    nionline = ""
    for line in mylines:
        if "User" in line:
            print "User time found. Continue."
            print line
            okay = 1
        if "NIONS" in line:
            nionline = line
    if okay == 0:
        print "Error reading all of OUTCAR."
        return
    nions = int(nionline.split()[-1])
    print "NIONS: ",nions
    atomindices=[]
    numlines = len(mylines)
    linect = 0
    while linect < numlines:
        if "atom =" in mylines[linect]:
            atomindices.append(linect)
        linect = linect + 1
    #print atomindices
    #print len(atomindices)
    lastset = atomindices[-1*nions:]
    #print lastset
    print "number of occ, eigenv pairs: ", len(lastset)
    eigvaldict=dict()
    vupdict=dict()
    vdowndict=dict()
    for atomidx in lastset:
        linect = atomidx
        my_atom = int(mylines[linect].split()[2])
        #print "atom: ",str(my_atom)
        eigvaldict[my_atom]=list()
        vupdict[my_atom]=list()
        vdowndict[my_atom]=list()
        my_lval = int(mylines[linect].split()[-1])
        occlines = (my_lval*2+1)*2
        occup = my_lval*2 + 1
        #print "number of occ lines: ", occlines
        occline = 0
        myline=""
        while linect < numlines and occline < occlines:
            myline = mylines[linect]
            if "v = " in myline:
                eigvaldict[my_atom].append(float(myline.split()[2]))
                vstr = myline.split("v = ")[1].strip()
                vlist = vstr.split()
                vupstr = ""
                vdownstr = ""
                vct = 0
                while vct < occup:
                    vupstr = vupstr + vlist[vct] + " "
                    vct = vct + 1
                while vct < occlines:
                    vdownstr = vdownstr + vlist[vct] + " "
                    vct = vct + 1
                #print vstr
                #print vupstr
                #print vdownstr
                vupdict[my_atom].append(vupstr.strip())
                vdowndict[my_atom].append(vdownstr.strip())
                occline = occline + 1
            linect = linect + 1
    print "ODICT 13: ",eigvaldict[13]
    print "VUPDICT 13:",vupdict[13]
    print "VDOWNDICT 13:",vdowndict[13]
    return [eigvaldict, vupdict, vdowndict]

def rotate_occ_and_ev_eigv(contcarfile, outcarfile, occatoms=[], l=2):
    """Rotate occupancy eigenvectors for B-sites only.
        [$1] CONTCAR file name
        [$2] OUTCAR file name
        [$3] occatoms = list of atom numbers corresponding 
            to the lines in the CONTCAR
        [$4] quantum number l
    """
    #print "Do not use! rotating eigenvectors and multiplying the squares by old eigenvalues may not be correct."
    #return
    import Rdrotate
    import Rgetcoords
    if not os.path.isfile(contcarfile):
        print "No such file at ",contcarfile
        return
    if not os.path.isfile(outcarfile):
        print "No such file at ", outcarfile
        return
    if len(occatoms) == 0:
        occatoms = range(9,17)
    import Rgetcoords
    coordset=Rgetcoords.main(contcarfile, 0.05)
    oextend=Rgetcoords.extend_cell(coordset[2])
    octdict=Rgetcoords.build_octahedra(coordset[1],oextend)
    #parse occupation file
    mydicts = get_occ_and_ev(outcarfile)
    d_eigenval = mydicts[0]
    d_vup = mydicts[1]
    d_vdown = mydicts[2]
    d_vuprotsq = dict()
    d_vdnrotsq = dict()
    d_upsqorb = dict()
    d_dnsqorb = dict()
    linect=0
    maxlines=0
    import Rdrotate
    for atom in occatoms:
        getlines=Rgetcoords.get_lines_from_octahedron(octdict,atom)
        if len(getlines) > 0 and len(getlines[0]) > 0:
            alphabetagamma = Rgetcoords.get_euler_angles_from_octahedron(getlines[0],getlines[1])
            alp = alphabetagamma[0]
            bet = alphabetagamma[1]
            gam = alphabetagamma[2]
            linect = 0
            maxlines = len(d_eigenval[atom])
            d_vuprotsq[atom]=[]
            d_vdnrotsq[atom]=[]
            d_upsqorb[atom]=[]
            d_dnsqorb[atom]=[]
            while linect < maxlines:
                myvuprot = Rdrotate.rotatereal(l, alp, bet, gam,
                            d_vup[atom][linect])
                myvdnrot = Rdrotate.rotatereal(l, alp, bet, gam,
                            d_vdown[atom][linect])
                orbct=0
                upsq=0.0
                dnsq=0.0
                d_vuprotsq[atom].append(dict())
                d_vdnrotsq[atom].append(dict())
                d_upsqorb[atom].append(dict())
                d_dnsqorb[atom].append(dict())
                myev = d_eigenval[atom][linect]
                for orb in range(-l,l+1):
                    upsq = np.power(myvuprot[orb],2)
                    dnsq = np.power(myvdnrot[orb],2)
                    d_vuprotsq[atom][linect][orb] = upsq
                    d_vdnrotsq[atom][linect][orb] = dnsq
                    d_upsqorb[atom][linect][orb] = upsq * myev
                    d_dnsqorb[atom][linect][orb] = dnsq * myev
                    orbct = orbct + 1 
                linect = linect + 1
    rotocc=dict()
    for atom in occatoms:
        rotocc[atom]=dict()
        maxlines = len(d_eigenval[atom])
        for orb in range(-l, l+1):
            orbsum = 0
            linect = 0
            while linect < maxlines:
                orbsum = orbsum + d_upsqorb[atom][linect][orb]
                orbsum = orbsum + d_dnsqorb[atom][linect][orb]
                linect = linect + 1
            rotocc[atom][orb] = orbsum
    for atom in occatoms:
        mystr = str(atom) + ": "
        for orb in range(-l, l+1):
            mystr = mystr + str(rotocc[atom][orb]) + " "
        print mystr
    return
             
def rotate_occ_and_ev_sqfirst(mypath, occatoms=[], l=2):
    """Rotate occupancy eigenvectors for B-sites only.
        Do this AFTER getting squares.
        [$1] CONTCAR file name
        [$2] OUTCAR file name
        [$3] occatoms = list of atom numbers corresponding 
            to the lines in the CONTCAR
        [$4] quantum number l
    """
    import Rdrotate
    import Rgetcoords
    contcarfile = os.path.join(mypath, "CONTCAR")
    outcarfile = os.path.join(mypath, "OUTCAR")
    if not os.path.isfile(contcarfile):
        print "No such file at ",contcarfile
        return
    if not os.path.isfile(outcarfile):
        print "No such file at ", outcarfile
        return
    if len(occatoms) == 0:
        occatoms = range(9,17)
    import Rgetcoords
    coordset=Rgetcoords.main(contcarfile, 0.05)
    oextend=Rgetcoords.extend_cell(coordset[2])
    octdict=Rgetcoords.build_octahedra(coordset[1],oextend)
    #parse occupation file
    mydicts = get_occ_and_ev(outcarfile)
    d_eigenval = mydicts[0]
    d_vup = mydicts[1]
    d_vdown = mydicts[2]
    d_vuprotsq = dict()
    d_vdnrotsq = dict()
    d_upsqorb = dict()
    d_dnsqorb = dict()
    linect=0
    maxlines=0
    import Rdrotate
    for atom in occatoms:
        print "ATOM:",str(atom)
        getlines=Rgetcoords.get_lines_from_octahedron(octdict,atom)
        if len(getlines) > 0 and len(getlines[0]) > 0:
            alphabetagamma = Rgetcoords.get_euler_angles_from_octahedron(getlines[0],getlines[1])
            alp = alphabetagamma[0]
            bet = alphabetagamma[1]
            gam = alphabetagamma[2]
            linect = 0
            maxlines = len(d_eigenval[atom])
            d_vuprotsq[atom]=[]
            d_vdnrotsq[atom]=[]
            d_upsqorb[atom]=[]
            d_dnsqorb[atom]=[]
            while linect < maxlines:
                upsq=0.0
                dnsq=0.0
                myvup = d_vup[atom][linect].strip().split()
                myvdn = d_vdown[atom][linect].strip().split()
                upsqstr=""
                dnsqstr=""
                orbct = 0
                while orbct < (2*l+1):
                    upsq = np.power(float(myvup[orbct]),2)
                    dnsq = np.power(float(myvdn[orbct]),2)
                    upsqstr = upsqstr + str(upsq) + " "
                    dnsqstr = dnsqstr + str(dnsq) + " " 
                    orbct = orbct + 1
                upsqstr = upsqstr.strip()
                dnsqstr = dnsqstr.strip()
                print "UPSQSTR:",upsqstr
                print "DNSQSTR:",dnsqstr
                d_upsqorb[atom].append(dict())
                d_upsqorb[atom][linect] = dict()
                d_dnsqorb[atom].append(dict())
                d_dnsqorb[atom][linect] = dict()
                myev = d_eigenval[atom][linect]
                myvuprot = Rdrotate.rotatereal(l, alp, bet, gam,
                            upsqstr)
                myvdnrot = Rdrotate.rotatereal(l, alp, bet, gam,
                            dnsqstr)
                for orb in range(-l,l+1):
                    d_upsqorb[atom][linect][orb] = myvuprot[orb] * myev
                    d_dnsqorb[atom][linect][orb] = myvdnrot[orb] * myev
                linect = linect + 1
    rotocc=dict()
    for atom in occatoms:
        rotocc[atom]=dict()
        maxlines = len(d_eigenval[atom])
        for orb in range(-l, l+1):
            rotorbsum = 0
            linect = 0
            while linect < maxlines:
                rotorbsum = rotorbsum + d_upsqorb[atom][linect][orb]
                rotorbsum = rotorbsum + d_dnsqorb[atom][linect][orb]
                if rotorbsum < 0:
                    print "ATTENTION < 0:",rotorbsum
                linect = linect + 1
            rotocc[atom][orb] = rotorbsum
    print "ROTATED:"
    for atom in occatoms:
        mystr = str(atom) + ": "
        for orb in range(-l, l+1):
            mystr = mystr + str(rotocc[atom][orb]) + " "
        mystr = mystr + "SUM: "
        mystr = mystr + str(sum(rotocc[atom].values()))
        print mystr
    return
             
def norotate_occ(contcarfile, outcarfile, occatoms=[], l=2):
    """Occupancies for B-sites only.
        [$1] CONTCAR file name
        [$2] OUTCAR file name
        [$3] occatoms = list of atom numbers corresponding 
            to the lines in the CONTCAR
        [$4] quantum number l
    """
    import Rdrotate
    import Rgetcoords
    if not os.path.isfile(contcarfile):
        print "No such file at ",contcarfile
        return
    if not os.path.isfile(outcarfile):
        print "No such file at ", outcarfile
        return
    if len(occatoms) == 0:
        occatoms = range(9,17)
    import Rgetcoords
    coordset=Rgetcoords.main(contcarfile, 0.05)
    oextend=Rgetcoords.extend_cell(coordset[2])
    octdict=Rgetcoords.build_octahedra(coordset[1],oextend)
    #parse occupation file
    mydicts = get_occ_and_ev(outcarfile)
    d_eigenval = mydicts[0]
    d_vup = mydicts[1]
    d_vdown = mydicts[2]
    d_vuprotsq = dict()
    d_vdnrotsq = dict()
    d_upsqorb = dict()
    d_dnsqorb = dict()
    linect=0
    maxlines=0
    import Rdrotate
    for atom in occatoms:
        print "ATOM:",str(atom)
        getlines=Rgetcoords.get_lines_from_octahedron(octdict,atom)
        if len(getlines) > 0 and len(getlines[0]) > 0:
            linect = 0
            maxlines = len(d_eigenval[atom])
            d_vuprotsq[atom]=[]
            d_vdnrotsq[atom]=[]
            d_upsqorb[atom]=[]
            d_dnsqorb[atom]=[]
            while linect < maxlines:
                upsq=0.0
                dnsq=0.0
                myvup = d_vup[atom][linect].strip().split()
                myvdn = d_vdown[atom][linect].strip().split()
                upsqstr=""
                dnsqstr=""
                orbct = 0
                d_upsqorb[atom].append(dict())
                d_upsqorb[atom][linect] = dict()
                d_dnsqorb[atom].append(dict())
                d_dnsqorb[atom][linect] = dict()
                myev = d_eigenval[atom][linect]
                while orbct < (2*l+1):
                    upsq = np.power(float(myvup[orbct]),2)
                    d_upsqorb[atom][linect][orbct - l] = upsq * myev
                    dnsq = np.power(float(myvdn[orbct]),2)
                    d_dnsqorb[atom][linect][orbct - l] = dnsq * myev
                    orbct = orbct + 1
                linect = linect + 1
    rotocc=dict()
    for atom in occatoms:
        rotocc[atom]=dict()
        maxlines = len(d_eigenval[atom])
        for orb in range(-l, l+1):
            rotorbsum = 0
            linect = 0
            while linect < maxlines:
                rotorbsum = rotorbsum + d_upsqorb[atom][linect][orb]
                rotorbsum = rotorbsum + d_dnsqorb[atom][linect][orb]
                linect = linect + 1
            rotocc[atom][orb] = rotorbsum
    print "UNROTATED:"
    for atom in occatoms:
        mystr = str(atom) + ": "
        for orb in range(-l, l+1):
            mystr = mystr + str(rotocc[atom][orb]) + " "
        mystr = mystr + "SUM: "
        mystr = mystr + str(sum(rotocc[atom].values()))
        print mystr
    return
             
def rotate_occ_and_ev_normalized(contcarfile, outcarfile, occatoms=[], l=2):
    """Rotate occupancy eigenvectors for B-sites only.
        Do this AFTER getting squares.
        [$1] CONTCAR file name
        [$2] OUTCAR file name
        [$3] occatoms = list of atom numbers corresponding 
            to the lines in the CONTCAR
        [$4] quantum number l
    """
    print "DO NOT USE. Normalization turns out oddly for small-valued rotations."
    return
    import Rdrotate
    import Rgetcoords
    if not os.path.isfile(contcarfile):
        print "No such file at ",contcarfile
        return
    if not os.path.isfile(outcarfile):
        print "No such file at ", outcarfile
        return
    if len(occatoms) == 0:
        occatoms = range(9,17)
    import Rgetcoords
    coordset=Rgetcoords.main(contcarfile, 0.05)
    oextend=Rgetcoords.extend_cell(coordset[2])
    octdict=Rgetcoords.build_octahedra(coordset[1],oextend)
    #parse occupation file
    mydicts = get_occ_and_ev(outcarfile)
    d_eigenval = mydicts[0]
    d_vup = mydicts[1]
    d_vdown = mydicts[2]
    d_vuprotsq = dict()
    d_vdnrotsq = dict()
    d_upsqorb = dict()
    d_dnsqorb = dict()
    linect=0
    maxlines=0
    import Rdrotate
    for atom in occatoms:
        print "ATOM:",str(atom)
        getlines=Rgetcoords.get_lines_from_octahedron(octdict,atom)
        if len(getlines) > 0 and len(getlines[0]) > 0:
            alphabetagamma = Rgetcoords.get_euler_angles_from_octahedron(getlines[0],getlines[1])
            alp = alphabetagamma[0]
            bet = alphabetagamma[1]
            gam = alphabetagamma[2]
            linect = 0
            maxlines = len(d_eigenval[atom])
            d_vuprotsq[atom]=[]
            d_vdnrotsq[atom]=[]
            d_upsqorb[atom]=[]
            d_dnsqorb[atom]=[]
            while linect < maxlines:
                upsq=0.0
                dnsq=0.0
                myvup = d_vup[atom][linect].strip().split()
                myvdn = d_vdown[atom][linect].strip().split()
                upsqstr=""
                dnsqstr=""
                orbct = 0
                while orbct < (2*l+1):
                    upsq = np.power(float(myvup[orbct]),2)
                    dnsq = np.power(float(myvdn[orbct]),2)
                    upsqstr = upsqstr + str(upsq) + " "
                    dnsqstr = dnsqstr + str(dnsq) + " " 
                    orbct = orbct + 1
                upsqstr = upsqstr.strip()
                dnsqstr = dnsqstr.strip()
                print "UPSQSTR:",upsqstr
                print "DNSQSTR:",dnsqstr
                d_upsqorb[atom].append(dict())
                d_upsqorb[atom][linect] = dict()
                d_dnsqorb[atom].append(dict())
                d_dnsqorb[atom][linect] = dict()
                myev = d_eigenval[atom][linect]
                myvuprot = Rdrotate.rotatereal_verbose_normalize(l, alp, bet, gam,
                            upsqstr)
                myvdnrot = Rdrotate.rotatereal_verbose_normalize(l, alp, bet, gam,
                            dnsqstr)
                for orb in range(-l,l+1):
                    d_upsqorb[atom][linect][orb] = myvuprot[orb] * myev
                    d_dnsqorb[atom][linect][orb] = myvdnrot[orb] * myev
                linect = linect + 1
    rotocc=dict()
    for atom in occatoms:
        rotocc[atom]=dict()
        maxlines = len(d_eigenval[atom])
        for orb in range(-l, l+1):
            rotorbsum = 0
            linect = 0
            while linect < maxlines:
                rotorbsum = rotorbsum + d_upsqorb[atom][linect][orb]
                rotorbsum = rotorbsum + d_dnsqorb[atom][linect][orb]
                linect = linect + 1
            rotocc[atom][orb] = rotorbsum
    print "ROTATED:"
    for atom in occatoms:
        mystr = str(atom) + ": "
        for orb in range(-l, l+1):
            mystr = mystr + str(rotocc[atom][orb]) + " "
        mystr = mystr + "SUM: "
        mystr = mystr + str(sum(rotocc[atom].values()))
        print mystr
    return
             
def rotate_procarstr(contcarfile, procarstr, occatoms=[], l=2):
    """Rotate occupancy eigenvectors for B-sites only.
        Do this AFTER getting squares.
        [$1] CONTCAR file name
        [$2] PROCAR occupation values (dxy, dyz, dz2, dxz, dx2
            for each atom, in a space-delimited string)
        [$3] occatoms = list of atom numbers corresponding 
            to the lines in the CONTCAR
        [$4] quantum number l
    """
    import Rdrotate
    import Rgetcoords
    if not os.path.isfile(contcarfile):
        print "No such file at ",contcarfile
        return
    if len(occatoms) == 0:
        occatoms = range(9,17)
    import Rgetcoords
    coordset=Rgetcoords.main(contcarfile, 0.05)
    oextend=Rgetcoords.extend_cell(coordset[2])
    octdict=Rgetcoords.build_octahedra(coordset[1],oextend)
    #parse procar occupation string
    procdict=dict()
    proclist = procarstr.split()
    atomct=0
    pidx=0
    while pidx < len(proclist):
        atom = occatoms[atomct]
        orbstr = ""
        for orb in range(-l,l+1):
            orbstr = orbstr + str(proclist[pidx]) + " "
            pidx = pidx + 1
        procdict[atom] = orbstr.strip()
        atomct = atomct + 1
    import Rdrotate
    rotocc=dict()
    for atom in occatoms:
        print "ATOM:",str(atom)
        getlines=Rgetcoords.get_lines_from_octahedron(octdict,atom)
        if len(getlines) > 0 and len(getlines[0]) > 0:
            alphabetagamma = Rgetcoords.get_euler_angles_from_octahedron(getlines[0],getlines[1])
            alp = alphabetagamma[0]
            bet = alphabetagamma[1]
            gam = alphabetagamma[2]
            myrot = Rdrotate.rotatereal(l, alp, bet, gam, 
                            procdict[atom])
            rotocc[atom] = myrot
    print "ROTATED FROM PROCAR STRING:"
    for atom in occatoms:
        mystr = str(atom) + ": "
        for orb in range(-l, l+1):
            mystr = mystr + str(rotocc[atom][orb]) + " "
        mystr = mystr + "SUM: "
        mystr = mystr + str(sum(rotocc[atom].values()))
        print mystr
    return
             
def rotate_procarstr_byspin(contcarfile, procarstrup, procarstrdn, occatoms=[], l=2):
    """Rotate occupancy eigenvectors for B-sites only.
        Do this AFTER getting squares.
        [$1] CONTCAR file name
        [$2] PROCAR up occupation values (dxy, dyz, dz2, dxz, dx2
            for each atom, in a space-delimited string, format
                "atomnumber: dxy dyz dz2 dxz dx2 atomnumber: ..."
        [$2.5] PROCAR down occupation values
        [$3] occatoms = list of atom numbers corresponding 
            to the lines in the CONTCAR
        [$4] quantum number l
    """
    import Rdrotate
    import Rgetcoords
    if not os.path.isfile(contcarfile):
        print "No such file at ",contcarfile
        return
    if len(occatoms) == 0:
        occatoms = range(9,17)
    import Rgetcoords
    coordset=Rgetcoords.main(contcarfile, 0.05)
    oextend=Rgetcoords.extend_cell(coordset[2])
    octdict=Rgetcoords.build_octahedra(coordset[1],oextend)
    #parse procar occupation string
    procdictup=dict()
    proclistup = procarstrup.split()
    procdictdn=dict()
    proclistdn = procarstrdn.split()
    atomct=0
    pidx=0
    while pidx < len(proclistup):
        atom = occatoms[atomct]
        orbstrup = ""
        orbstrdn = ""
        if ":" in proclistup[pidx]:
            print "atom:",proclistup[pidx]
            pidx = pidx + 1 #skip the atom designation
        for orb in range(-l,l+1):
            orbstrup = orbstrup + str(proclistup[pidx]) + " "
            orbstrdn = orbstrdn + str(proclistdn[pidx]) + " "
            pidx = pidx + 1
        procdictup[atom] = orbstrup.strip()
        procdictdn[atom] = orbstrdn.strip()
        atomct = atomct + 1
    import Rdrotate
    rotoccup=dict()
    rotoccdn=dict()
    for atom in occatoms:
        print "ATOM:",str(atom)
        getlines=Rgetcoords.get_lines_from_octahedron(octdict,atom)
        if len(getlines) > 0 and len(getlines[0]) > 0:
            alphabetagamma = Rgetcoords.get_euler_angles_from_octahedron(getlines[0],getlines[1])
            alp = alphabetagamma[0]
            bet = alphabetagamma[1]
            gam = alphabetagamma[2]
            myrotup = Rdrotate.rotatereal(l, alp, bet, gam, 
                            procdictup[atom])
            rotoccup[atom] = myrotup
            myrotdn = Rdrotate.rotatereal(l, alp, bet, gam, 
                            procdictdn[atom])
            rotoccdn[atom] = myrotdn
    print "ROTATED FROM PROCAR STRING:"
    sumdict=dict()
    for atom in occatoms:
        mystr = str(atom) + ": "
        sumdict[atom] = dict()
        for orb in range(-l, l+1):
            sumdict[atom][orb] = math.fabs(rotoccup[atom][orb]) + math.fabs(rotoccdn[atom][orb])
            mystr = mystr + str(sumdict[atom][orb]) + " "
        mystr = mystr + "SUM: "
        mystr = mystr + str(sum(sumdict[atom].values()))
        print mystr
    return
             
def rotate_occ_and_ev_matrix(mypath, occatoms=[], l=2):
    """Rotate occupancies for B-sites only.
        Get matrix from eigenvals and vectors, rotate, extract 
        eigvals and vectors
        [$1] CONTCAR file name
        [$2] OUTCAR file name
        [$3] occatoms = list of atom numbers corresponding 
            to the lines in the CONTCAR
        [$4] quantum number l
    """
    import Rdrotate
    import Rgetcoords
    contcarfile = os.path.join(mypath, "CONTCAR")
    outcarfile = os.path.join(mypath, "OUTCAR")
    if not os.path.isfile(contcarfile):
        print "No such file at ",contcarfile
        return
    if not os.path.isfile(outcarfile):
        print "No such file at ", outcarfile
        return
    if len(occatoms) == 0:
        occatoms = range(9,17)
    coordset=Rgetcoords.main(contcarfile, 0.05)
    oextend=Rgetcoords.extend_cell(coordset[2])
    octdict=Rgetcoords.build_octahedra(coordset[1],oextend)
    #parse occupation file
    mydicts = get_occ_and_ev_arrays(outcarfile)
    d_evals = mydicts[0]
    d_evecs = mydicts[1]
    #get original matrices back (one for up, one for down)
    d_origmats = dict()
    myprintlist=[]
    myprintlist.append("outcarfile: " + str(outcarfile))
    print "outcarfile: ",outcarfile
    myprintlist.append("contcarfile: " + str(contcarfile))
    print "contcarfile: ",contcarfile
    if l == 2:
        orbstr= "dxy dyz dz2 dxz dx2"
        print orbstr
        myprintlist.append(orbstr)
    for atom in occatoms:
        myorigmat = get_original_matrix(d_evals[atom],d_evecs[atom])
        print "Atom ",str(atom), "original matrix:"
        print_matrix(myorigmat)
        mystrdict = arrange_matrix_into_strings(myorigmat)
        getlines=Rgetcoords.get_lines_from_octahedron(octdict,atom)
        if not(len(getlines) > 0 and len(getlines[0]) > 0):
            print "Error getting coordinates!"
            return
        alphabetagamma = Rgetcoords.get_euler_angles_from_octahedron(getlines[0],getlines[1])
        alp = alphabetagamma[0]
        bet = alphabetagamma[1]
        gam = alphabetagamma[2]
        myrotatedict = rotate_string_dict(alp,bet,gam,mystrdict)
        myrotmat = transfer_spin_rotations_to_matrix(myrotatedict)
        print "Atom ",str(atom), "rotated matrix:"
        print_matrix(myrotmat)
        mysolved = eig_matrix(myrotmat)
        myrotevals = mysolved[0]
        myrotevecs = mysolved[1]
        myoccmat = get_occupation_matrix(myrotevals, myrotevecs)
        print "Atom ",str(atom), "occ matrix:"
        print_matrix(myoccmat)
        mytotdict = get_occupations(myrotevals, myrotevecs)
        myprintstr = print_occupations(mytotdict, True)
        myprintstr = "Atom " + str(atom) + ": " + myprintstr
        print myprintstr
        myprintlist.append(myprintstr)
    for myline in myprintlist:
        print myline
    return myprintlist

def get_occ_and_ev_arrays(opath):
    """Get occupancies and eigenvectors as numpy arrays per atom, 
        from OUTCAR. Note that eigenvalues will be in a single row,
        while their corresponding eigenvectors will be column
        vectors.
        opath = OUTCAR path
    """
    if not os.path.exists(opath):
        print "No file at ", opath
        return
    myfile = open(opath, 'rb')
    mylines = myfile.readlines()
    myfile.close()
    okay = 0 #want to make sure whole file is transmitted.
    nionline = ""
    for line in mylines:
        if "User" in line:
            print "User time found. Continue."
            print line
            okay = 1
        if "NIONS" in line:
            nionline = line
    if okay == 0:
        print "Error reading all of OUTCAR."
        return
    nions = int(nionline.split()[-1])
    print "NIONS: ",nions
    atomindices=[]
    numlines = len(mylines)
    linect = 0
    while linect < numlines:
        if "atom =" in mylines[linect]:
            atomindices.append(linect)
        linect = linect + 1
    #print atomindices
    #print len(atomindices)
    lastset = atomindices[-1*nions:]
    #print lastset
    print "number of occ, eigenv pairs: ", len(lastset)
    d_eigvals=dict() 
    d_eigvecs=dict()
    #each dictionary entry will be a [(2l+1)*2,(2l+1)*] array,
    #with one of the (2l+1) sets being for each spin.
    for atomidx in lastset:
        linect = atomidx
        my_atom = int(mylines[linect].split()[2])
        #print "atom: ",str(my_atom)
        my_lval = int(mylines[linect].split()[-1])
        occlines = (my_lval*2+1)*2
        d_eigvals[my_atom]=np.zeros([1, occlines],float)
        d_eigvecs[my_atom]=np.zeros([occlines, occlines],float)
        #print "number of occ lines: ", occlines
        occline = 0
        myline=""
        while linect < numlines and occline < occlines:
            myline = mylines[linect]
            if "v = " in myline:
                d_eigvals[my_atom][0,occline]=float(myline.split()[2])
                vstr = myline.split("v = ")[1].strip()
                vlist = vstr.split()
                vupstr = ""
                vdownstr = ""
                vct = 0
                while vct < occlines:
                    d_eigvecs[my_atom][vct,occline] = float(vlist[vct])
                    vct = vct + 1
                #print vstr
                #print vupstr
                #print vdownstr
                occline = occline + 1
            linect = linect + 1
    #print "ODICT 13: ",d_eigvals[13]
    #print "VECDICT 13:",d_eigvecs[13]
    return [d_eigvals, d_eigvecs]

def get_original_matrix(eigvals, eigvecs):
    """Get original matrix from set of eigenvalues and eigenvectors.
        The eigenvectors should be column vectors corresponding
        to the eigenvalues listed in order.
        [A-(lambda)I]v = 0
        P = matrix with column vectors as eigenvalues v
        D = diagonal matrix with entries as eigenvectors
        A = PD(P^-1)
    """
    my_p = eigvecs
    my_pinv = np.linalg.pinv(eigvecs)
    mysize = np.size(eigvals)
    my_d = eigvals*np.identity(mysize,float)
    #print my_p
    #print my_d
    my_pd = np.dot(my_p,my_d)
    my_pdpinv = np.dot(my_pd, my_pinv)
    return my_pdpinv

def make_matrix_and_list_from_evecs_evals(vecd,vald="", printnow=False):
    """Make a matrix from a dictionary like:
        D[lineno][-2]=3
        D[lineno][-1]=3
        Assume it is a square matrix.
    """
    linelist = vecd.keys()
    linelist.sort()
    if len(linelist) == 0:
        print "No lines in dictionary."
        return
    mysize = len(vecd[linelist[0]].keys())
    myarr = np.zeros([mysize,mysize],complex)
    mylist=list()
    colct=0
    for myline in linelist:
        if not sum(vecd[myline].values()) == 0:
            if not (vald==""):
                mylist.append(vald[myline])
            numlist = vecd[myline].keys()
            numlist.sort()
            rowct = 0
            for mynum in numlist:
                myarr[rowct][colct]=vecd[myline][mynum]
                rowct = rowct + 1
            colct = colct + 1
    return [myarr,mylist]

def make_list_from_evals(eval_d,printnow=False):
    """Make a list from eigenvalues is D[lineno]=3, etc."""
    linelist = eval_d.keys()
    linelist.sort()
    evallist=list()
    for myline in linelist:
        evallist.append(eval_d[myline])
    if printnow:
        print evallist
    return evallist


def make_orbdict_from_matrix(mymat, printnow=False):
    """Make an orbital dictionary list from a matrix, which
        is assumed to be square, with indices -l...0...l in order
        as rows, and arbitrary line numbers as columns.
    """
    mysize = np.size(mymat,0)
    myl = (mysize - 1)/2
    colct = 0
    mydict=dict()
    while colct < mysize:
        rowct = 0
        mydict[colct] = dict()
        while rowct < mysize:
            mydict[colct][rowct - myl] = mymat[rowct][colct]
            rowct = rowct + 1
        colct = colct + 1
    if printnow:
        import Rdorb
        Rdorb.print_two_level_dictionary(mydict)
    return mydict



def arrange_matrix_into_strings(mymat):
    """Arrange a square numpy array into strings for rotation.
        This function assumes that a matrix has vectors in columns,
        the first half of whose rows signify one spin,
        and the second half of whose rows signify another spin.
        
        Returns:
            Dict of list of strings, in order of the column vectors.
            First key is 'spin1' and second is 'spin2'
    """
    mysize = np.size(mymat,0)
    numorbs = mysize/2 
    str1list=list()
    str2list=list()
    strdict=dict()
    colct = 0
    #print mymat
    #print numorbs
    while colct < mysize:
        orbct = 0
        mystr1 = ""
        mystr2 = ""
        while orbct < numorbs:
            mystr1 = mystr1 + str(mymat[orbct,colct]) + " "
            mystr2 = mystr2 + str(mymat[orbct + numorbs,colct]) + " "
            orbct = orbct + 1
        str1list.append(mystr1.strip())
        str2list.append(mystr2.strip())
        colct = colct + 1
    strdict['spin1'] = str1list
    strdict['spin2'] = str2list
    print strdict
    return strdict
def rotate_string_dict(alpha, beta, gamma, stringdict):
    """Rotate a dictionary of strings.
        This function assumes 
        1. That alpha, beta, and gamma are Euler angles
        2. That the dictionary has keys 'spin1' and 'spin2'
        3. That each spin string has a number of values equal to
            2l+1 where m in -2l...0...2l is the real 
            (linear combination) value; e.g. dxy dyz dz2 dxz dx2

    """
    import Rdrotate
    list1 = stringdict['spin1']
    list2 = stringdict['spin2']
    numorbs = len(list1[0].split())
    numvecs = numorbs * 2
    myl = (numorbs - 1)/2
    rotate_spin1 = list()
    rotate_spin2 = list()
    rotatedict = dict()
    stridx = 0
    #print "numorbs: ",numorbs
    #print "My l: ",myl
    while stridx < numvecs:
        #print "stridx:",stridx
        #print "entry: ",list1[stridx]
        rotate1 = Rdrotate.rotatereal(myl, alpha, beta, gamma,
                    list1[stridx])
        rotate2 = Rdrotate.rotatereal(myl, alpha, beta, gamma,
                    list2[stridx])
        rotate_spin1.append(rotate1)
        rotate_spin2.append(rotate2)
        stridx = stridx + 1
    rotatedict['spin1'] = rotate_spin1
    rotatedict['spin2'] = rotate_spin2
    print rotatedict
    return rotatedict

def transfer_spin_rotations_to_matrix(rotatedict):
    """Transfer rotated matrix lines into numpy array.
        Assumes a dictionary of 'spin1' and 'spin2'
        with keys matching 2l...0...2l
    """
    spin1list=rotatedict['spin1']
    spin2list=rotatedict['spin2']
    numorbs = len(spin1list[0]) #2l+1
    myl = (numorbs - 1)/2
    orball = 2*numorbs
    rotmat = np.zeros([orball,orball],float)
    colct = 0
    while colct < orball:
        orbct = 0
        while orbct < numorbs:
            rotmat[orbct,colct] = spin1list[colct][orbct - myl]
            rotmat[orbct + numorbs, colct] = spin2list[colct][orbct - myl]
            orbct = orbct + 1
        colct = colct + 1
    return rotmat    
def eig_matrix(mymat):
    """Get eigneeigenvalues and eigenvectors from numpy array.
    """
    myevals, myevecs = np.linalg.eig(mymat)
    return [myevals, myevecs]

def get_occupation_matrix(evals, evecs):
    """Get occupation matrix from eigenvalues and eigenvectors.
        Square the eigenvector coefficients and multiply them
        by the eigenvalues (each column is an eigenvector).
    """
    mysq = np.multiply(evecs,np.conj(evecs))
    mysqtransp = np.transpose(mysq)
    mysize = np.size(evals)
    my_diag = evals*np.identity(mysize,float)
    myocctransp = np.dot(my_diag,mysqtransp)
    myoccmat = np.transpose(myocctransp)
    return myoccmat

def get_occupations(evals, evecs):
    """Get occupations from eigenvalues and eigenvectors.
        Square the eigenvector coefficients and multiply them
        by the eigenvalues (each column is an eigenvector).
        Then add up all the contributions in one row to get
        the contribution in an orbital.
    """
    mysize = np.size(evecs,0)
    mysq = np.zeros([mysize,mysize],float)
    colct = 0
    while colct < mysize:
        rowct = 0
        while rowct < mysize:
            myevecval = evecs[rowct][colct]
            myeigval = evals[colct]
            myevecsq = np.dot(myevecval, np.conj(myevecval)) 
            #magnitude is psi*conj(psi)
            #print evecs[rowct][colct], myevecval
            #print evals[colct], myeigval
            myval = myevecsq * myeigval
            if myval < 0:
                print "ATTENTION < 0: ",myval
                print "eigval: ",myeigval
                print "myevecsq: ",myevecsq
                print "myevecval: ",myevecval
            mysq[rowct][colct] = myevecsq * myeigval
            rowct = rowct + 1
        colct = colct + 1
    mysum = np.sum(mysq,1) #1 index sums across columns, or orbitals
    numorbs = mysize/2
    myl = (numorbs - 1)/2
    orbct = 0
    my_up = dict()
    my_dn = dict()
    my_tot_dict = dict()
    while orbct < numorbs:
        orbidx = orbct - myl
        my_up[orbidx] = mysum[orbct]
        my_dn[orbidx] = mysum[orbct + numorbs]
        if my_up[orbidx] < 0:
            print "ATTENTION < 0:",my_up[orbidx]
        if my_dn[orbidx] < 0:
            print "ATTENTION < 0:",my_dn[orbidx]
        my_tot_dict[orbidx] = my_up[orbidx] + my_dn[orbidx]
        orbct = orbct + 1
    return my_tot_dict

def print_occupations(totaloccdict,returnonly=False):
    """Print occupations nicely, from a dictionary whose keys
        are the orbital m numbers (real, so for example l=2, m=0
        is dz2-r2)
        
        returnonly = 0 (default); set this to 1 to only return,
        not print the string
    """
    numorbs = len(totaloccdict)
    myl = (numorbs - 1)/2
    orbct = 0
    printstr=""
    while orbct < numorbs:
        orbidx = orbct - myl
        printstr = printstr + str(totaloccdict[orbidx]) + " "
        orbct = orbct + 1
    if not(returnonly):
        print printstr
    return printstr

def print_matrix(mymat):
    """Print a numpy array."""
    numrows = np.size(mymat,0)
    numcols = np.size(mymat,1)
    rowct = 0
    print "PRINTING MATRIX:"
    while rowct < numrows:
        printstr = ""
        colct = 0
        while colct < numcols:
            printstr = printstr + "%4.4f+%4.4fj" % (np.real(mymat[rowct,colct]),np.imag(mymat[rowct,colct])) + " "
            colct = colct + 1
        print printstr
        rowct = rowct + 1

def get_occ_and_ev_dict(opath, printnow=False):
    """Get occupancies and eigenvalues matrices from outcar.
        opath: OUTCAR path
        printnow: set to True to print output

        Returns:
            dictionary of occupancies and eigenvalues, indexed
            arbitrarily by line number (VASP sorts them high
            to low).
            Each atom gets keys 'spin1' and 'spin2'.
            Each spin gets keys 'eval' for eigenvalues or
                'evec' for eigenvectors.
            Each 'eval' and 'evec' key gets line number keys.
            Each line number under 'evec'
            gets additional keys corresponding to the
            real m number (e.g. little m = -2 for dxy in l=2)
            Ex: for atom 3, first occupancies and eigenvectors line,
            dict[3]['spin1']['eval'][0]=3
            dict[3]['spin2']['evec'][0][-2]=-0.4
            dict[3]['spin2']['evec'][0][-1]=-0.4
    """
    if not os.path.exists(opath):
        print "No file at ", opath
        return
    myfile = open(opath, 'rb')
    mylines = myfile.readlines()
    myfile.close()
    okay = 0 #want to make sure whole file is transmitted.
    nionline = ""
    for line in mylines:
        if "User" in line:
            print "User time found. Continue."
            print line
            okay = 1
        if "NIONS" in line:
            nionline = line
    if okay == 0:
        print "Error reading all of OUTCAR."
        return
    nions = int(nionline.split()[-1])
    print "NIONS: ",nions
    atomindices=[]
    numlines = len(mylines)
    linect = 0
    while linect < numlines:
        if "atom =" in mylines[linect]:
            atomindices.append(linect)
        linect = linect + 1
    #print atomindices
    #print len(atomindices)
    lastset = atomindices[-1*nions:]
    #print lastset
    print "number of occ, eigenv pairs: ", len(lastset)
    occev_d=dict()
    for atomidx in lastset:
        linect = atomidx
        my_atom = int(mylines[linect].split()[2])
        #print "atom: ",str(my_atom)
        occev_d[my_atom]=dict()
        my_lval = int(mylines[linect].split()[-1])
        occlines = (my_lval*2+1)*2
        occup = my_lval*2 + 1
        #print "number of occ lines: ", occlines
        occline = 0
        myline=""
        occev_d[my_atom]=dict()
        occev_d[my_atom]['spin1']=dict()
        occev_d[my_atom]['spin1']['eval']=dict()
        occev_d[my_atom]['spin1']['evec']=dict()
        occev_d[my_atom]['spin2']=dict()
        occev_d[my_atom]['spin2']['eval']=dict()
        occev_d[my_atom]['spin2']['evec']=dict()
        while linect < numlines and occline < occlines:
            myline = mylines[linect]
            if "v = " in myline:
                eigval = float(myline.split()[2])
                occev_d[my_atom]['spin1']['eval'][linect] = eigval
                occev_d[my_atom]['spin1']['evec'][linect] = dict()
                occev_d[my_atom]['spin2']['eval'][linect] = eigval
                occev_d[my_atom]['spin2']['evec'][linect] = dict()
                vstr = myline.split("v = ")[1].strip()
                vlist = vstr.split()
                vct = 0
                orbct = -my_lval
                while vct < occup:
                    occev_d[my_atom]['spin1']['evec'][linect][orbct]=float(vlist[vct])
                    vct = vct + 1
                    orbct = orbct + 1
                orbct = -my_lval
                while vct < occlines:
                    occev_d[my_atom]['spin2']['evec'][linect][orbct]=float(vlist[vct])
                    vct = vct + 1
                    orbct = orbct + 1
                occline = occline + 1
            linect = linect + 1
    if printnow:
        import Rdorb
        atomlist = occev_d.keys()
        atomlist.sort()
        for myatom in atomlist:
            print "Atom: ",str(myatom)
            for myspin in ['spin1','spin2']:
                print myspin
                print "Eigenvectors:"
                Rdorb.print_two_level_dictionary(occev_d[myatom][myspin]['evec'])
                print "Eigenvalues:"
                Rdorb.print_one_level_dictionary(occev_d[myatom][myspin]['eval'])
    return occev_d

def rotate_occ_and_ev_through_complex(mypath,l=2,occatoms=[]):
    """Rotate occupancies by going through the complex orbitals
        first.
        mypath: directory
    """
    import Rdrotate
    import Rgetcoords
    import Rdorb
    contcarfile = os.path.join(mypath, "CONTCAR")
    outcarfile = os.path.join(mypath, "OUTCAR")
    if not os.path.isfile(contcarfile):
        print "No such file at ",contcarfile
        return
    if not os.path.isfile(outcarfile):
        print "No such file at ", outcarfile
        return
    if len(occatoms) == 0:
        occatoms = range(9,17)
    occatoms = [9] #TTM DEBUG
    coordset=Rgetcoords.main(contcarfile, 0.05)
    oextend=Rgetcoords.extend_cell(coordset[2])
    octdict=Rgetcoords.build_octahedra(coordset[1],oextend)
    #parse occupation file
    occev_d = get_occ_and_ev_dict(outcarfile)
    myprintlist=[]
    myprintlist.append("outcarfile: " + str(outcarfile))
    print "outcarfile: ",outcarfile
    myprintlist.append("contcarfile: " + str(contcarfile))
    print "contcarfile: ",contcarfile
    if l == 2:
        orbstr= "dxy dyz dz2 dxz dx2"
        print orbstr
        myprintlist.append(orbstr)
    rot_d = dict()
    for atom in occatoms:
        print "ATOM: ",str(atom)
        #For each spin, get original matrix, 
        #convert to complex, rotate, convert
        #to real, and re-save
        rot_d[atom] = dict()
        getlines=Rgetcoords.get_lines_from_octahedron(octdict,atom)
        if not(len(getlines) > 0 and len(getlines[0]) > 0):
            print "Error getting coordinates!"
            return
        alphabetagamma = Rgetcoords.get_euler_angles_from_octahedron(getlines[0],getlines[1])
        alp = alphabetagamma[0]
        bet = alphabetagamma[1]
        gam = alphabetagamma[2]
        #occupation dict is d[atom]['spin1']['evec'][lineno][orbital]
        # or d[atom]['spin1']['eval'][lineno]
        totaloccmat = np.zeros([(2*l+1),(2*l+1)],complex)
        for myspin in ['spin1','spin2']:
            print myspin
            evals = occev_d[atom][myspin]['eval']
            evecs = occev_d[atom][myspin]['evec']
            [evecmat, evallist] = make_matrix_and_list_from_evecs_evals(evecs,evals)
            print "vector matrix:"
            print_matrix(evecmat)
            myorigmat = get_original_matrix(evallist, evecmat)
            print "original matrix:"
            print_matrix(myorigmat)
            myorigorbs = make_orbdict_from_matrix(myorigmat)
            rotevecs=dict()
            for myline in myorigorbs.keys():
                complex_dict = Rdorb.get_complex_from_real(myorigorbs[myline])
                rot_dict = Rdorb.rotate_complex_orbitals(alp,bet,gam, complex_dict)
                real_back = Rdorb.get_real_from_complex(rot_dict)
                rotevecs[myline] = real_back
            [rotmat,noval] = make_matrix_and_list_from_evecs_evals(rotevecs)
            print "rotated matrix:"
            print_matrix(rotmat)
            mysolved = eig_matrix(rotmat)
            myrotevals = mysolved[0]
            print "rotated eigenvals: "
            print myrotevals
            myrotevecs = mysolved[1]
            myoccmat = get_occupation_matrix(myrotevals, myrotevecs)
            print "occ matrix:"
            print_matrix(myoccmat)
            totaloccmat = totaloccmat + myoccmat
        print "total occ matrix:"
        print_matrix(totaloccmat)
        myprintstr = '   '.join(map(str,totaloccmat.sum(0))) #sum of rows
        myprintstr = "Atom " + str(atom) + ": " + myprintstr
        print myprintstr
        myprintlist.append(myprintstr)
    for myline in myprintlist:
        print myline
    return myprintlist

