#!/usr/bin/env python
##
#@package   Vbands 
#@ingroup   vasp
#@ingroup   python
#@brief     get bands and kpoints
#@author    Tam Mayeshiba
#@date      9/6/11
###
def main(existdir, fname='OUTCAR'):
    """
    Purpose: get bands for kpoints
        inputs:  
            [$1] directory
            example: /home/tam/NaCl
            [$2] file name - default OUTCAR
        outputs:
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgrep
    import Pgetline
    import Pgetseg
    import Xvalidinteger
    import Xuniquename
#   validation
#   #  existdir
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   fname
    if not os.path.isfile(os.path.join(existdir, fname)):
        print fname, 'does not exist in', existdir
        print '(Exiting.)'
        return None
#   #   get number of kpts
    kptline=""
    kptstr=""
    kpts=0
    kptline = Pgetline.main(existdir, fname, 'NKPTS')
    if (kptline == None):
        print 'NKPTS not found in', fname
        print '(Exiting.)'
        return None
    kptstr = Pgetseg.main(kptline, 'NKPTS =', 'k-points')
    if (kptstr == None):
        print 'NKPTS line could not be parsed from', fname
        print '(Exiting.)'
        return None
    try:
        kpts = int(kptstr)
    except (ValueError, TypeError):
        print 'Kpoints could not be converted to an integer value.'
        print '(Exiting.)'
        return None
#   #   get spin or no spin
    spinline=""
    spinstr=""
    spin=0
    spinline = Pgetline.main(existdir, fname, 'ISPIN')
    if (spinline == None):
        print 'ISPIN not found in', fname
        print '(Exiting.)'
        return None
    spinstr = Pgetseg.main(spinline, 'ISPIN  =', 'spin polarized')
    if (spinstr == None):
        print 'Spin-polarized calc line could not be parsed from', fname
        print '(Exiting.)'
        return None
    try:
        spin = int(spinstr)
    except (ValueError, TypeError):
        print 'Spin could not be converted to an integer value.'
        print '(Exiting.)'
        return None
    spin = Xvalidinteger.main(spin, 1, 2)
    if (spin == None):
        print 'ISPIN does not appear to be either 1 or 2.'
        print '(Exiting.)'
        return None
#   #   get number of bands NBANDS
    nbandline=""
    nbandstr=""
    nbands=0
    nbandline = Pgetline.main(existdir, fname, 'NBANDS')
    if (nbandline == None):
        print 'NBANDS not found in', fname
        print '(Exiting.)'
        return None
    nbandstr = Pgetseg.main(nbandline, 'NBANDS=')
    if (nbandstr == None):
        print 'NBANDS could not be parsed from', fname
        print '(Exiting.)'
        return None
    try:
        nbands = int(nbandstr)
    except (ValueError, TypeError):
        print 'NBANDS could not be converted to an integer value.'
        print '(Exiting.)'
        return None
#   #   grep me
    bandenergies=[]
    bandenergies = Xgrep.main(existdir, fname, 'band energies', 1)
#   #   get starting lines starting from bottom
    numenlines=0
    numtocount=0
    truncmat=[]
    truncitem=""
    truncsplit=""
    truncline=""
    truncint=0
    linemat=[]
    numenlines = len(bandenergies)
    if (spin == 2):
        numtocount = kpts*2
    else:
        numtocount = kpts
    truncmat = bandenergies[numenlines - numtocount:]
    for truncitem in truncmat:
        truncsplit = truncitem.split(':')
        try:
            truncint = int(truncsplit[1])
        except (ValueError, TypeError):
            print 'No line number found in', truncitem
            print '(Exiting.)'
            return None
        linemat.append(truncint - 1) #truncint - 1 because want kpoint information
    print linemat
    halfway=0
    lenlinemat=0
    lenlinemat = len(linemat)
    halfway = lenlinemat/2
#   #   for each line, start reading in band number, band energy, and occupation
    spin1=[]
    spin2=[]
    matlinect=""
    occmat=[]
    nonoccmat=[]
    kline=""
    nline=""
    bandct=1
    bandline=""
    bandmat=[]
    linect=0
    numcycles=0
    savestr=""
    savestrocc=""
    savestrnonocc=""
    occ=0
    ener=0
    for matlinect in linemat:
        numcycles = numcycles + 1
        savestr = ""
        savestrocc = ""
        linect = int(matlinect)
        kline = Pgetline.main(existdir, fname, '0', linect)
        kline = kline.strip()
        linect = linect + 1
        nline = Pgetline.main(existdir, fname, '0', linect)
        linect = linect + 1
        savestr = kline
        savestrocc = kline
        savestrnonocc = kline
        bandct = 1
        while (bandct <= nbands):
            bandline = Pgetline.main(existdir, fname, '0', linect)
            bandmat = bandline.split()
            occ = bandmat[2]
            ener = bandmat[1]
            savestr = savestr + ' : ' + ener 
            if (float(occ) > 0):
                savestrocc = savestrocc + ' : ' + ener
                savestrnonocc = savestrnonocc + ' : '
            else:
                savestrnonocc = savestrnonocc + ' : ' + ener
                savestrocc = savestrocc + ' : '
            linect = linect + 1
            bandct = bandct + 1
        if (spin == 2) and (numcycles > halfway):
            spin2.append(savestr)
        else:
            spin1.append(savestr)
        occmat.append(savestrocc)
        nonoccmat.append(savestrnonocc)
        print numcycles, 'out of', lenlinemat  # so you can see this working
#   output to file
    newfilename=""
    newfile=""
    newfilename = Xuniquename.main(existdir, 'bands', 'f')
    if (newfilename == None):
        print 'New file could not be created.'
        print '(Exiting.)'
        return None
    newfile = open(newfilename, 'ab')
    for lineitem in spin1:
        newfile.write(lineitem + '\n')
    newfile.write('Spin 2\n')
    for lineitem in spin2:
        newfile.write(lineitem + '\n')
    newfile.write('Occupied\n')
    for lineitem in occmat:
        newfile.write(lineitem + '\n')
    newfile.write('Unoccupied\n')
    for lineitem in nonoccmat:
        newfile.write(lineitem + '\n')
    newfile.close()
    return [spin1, spin2]

