#!/usr/bin/env python
##
#@package   Vcontinueprev
#@ingroup   vasp
#@ingroup   python
#@brief     Continue a stopped run: save some files and use CONTCAR of prev.
#@author    Tam Mayeshiba
#@date      5/20/11 created
###
def main(existdir):
    """
    Purpose: Continue as stopped run: save OUTCAR, OSZICAR, CONTCAR, POSCAR
             and use the CONTCAR as POSCAR
        inputs: 
            [$1] directory of the stopped run
            example: /home/tam/NaCl
        outputs:
            None if error
            True if succeeded
            total time if run had already finished
    """
#   Modules:
    import os
    import sys
    import shutil
    import subprocess
    import Xvaliddirectory
    import Vgetstatus
    import Xuniquename
#   Validation
#   #  
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   check appropriateness
    archivename=""
    niceme=""
    totaltime=""
    if not os.path.isfile(existdir + '/CONTCAR'):
        print 'CONTCAR does not exist in', existdir
        print '(Exiting.)'
        return None
    if (os.stat(existdir + '/CONTCAR').st_size == 0):
        print 'CONTCAR exists in', existdir, 'but is empty.'
        print '(Exiting.)'
        return None
    totaltime = Vgetstatus.main(existdir)
    if not (totaltime == None):
        print 'Run has finished, not stopped midway.'
        print '(Exiting.)'
        return totaltime
#   save CONTCAR
    archivename = Xuniquename.main(existdir, 'CONTCAR', 'f', "")
    if (archivename == None):
        print 'CONTCAR exists but a copy could not be saved.'
        print '(Exiting.)'
        return None
    niceme = os.nice(19)
    shutil.copy(existdir + '/CONTCAR', existdir + '/' + archivename)
    print 'CONTCAR saved as', archivename
#   save POSCAR
    if not os.path.isfile(existdir + '/POSCAR'):
        print 'POSCAR not found in', existdir
        print '(Exiting.)'
        return None
    archivename = Xuniquename.main(existdir, 'POSCAR', 'f', "")
    if (archivename == None):
        print 'POSCAR exists but a copy could not be saved.'
        print '(Exiting.)'
        return None
    niceme = os.nice(19)
    shutil.copy(existdir + '/POSCAR', existdir + '/' + archivename)
    print 'POSCAR saved as', archivename
#   save OSZICAR
    if not os.path.isfile(existdir + '/OSZICAR'):
        print 'OSZICAR not found in', existdir
        print '(Exiting.)'
        return None
    archivename = Xuniquename.main(existdir, 'OSZICAR', 'f', "")
    if (archivename == None):
        print 'OSZICAR exists but a copy could not be saved.'
        print '(Exiting.)'
        return None
    niceme = os.nice(19)
    shutil.copy(existdir + '/OSZICAR', existdir + '/' + archivename)
    print 'OSZICAR saved as', archivename
    os.remove(existdir + '/OSZICAR')
#   save OUTCAR
    if not os.path.isfile(existdir + '/OUTCAR'):
        print 'OUTCAR not found in', existdir
        print '(Exiting.)'
        return None
    archivename = Xuniquename.main(existdir, 'OUTCAR', 'f', "")
    if (archivename == None):
        print 'OUTCAR exists but a copy could not be saved.'
        print '(Exiting.)'
        return None
    niceme = os.nice(19)
    shutil.copy(existdir + '/OUTCAR', existdir + '/' + archivename)
    print 'OUTCAR saved as', archivename
    os.remove(existdir + '/OUTCAR')
#   copy CONTCAR to POSCAR
    shutil.copy(existdir + '/CONTCAR', existdir + '/POSCAR')
    print 'CONTCAR copied to POSCAR'
    return True
