#!/usr/bin/env python
##
#@package   Vdoscarinsulator 
#@ingroup   vasp
#@ingroup   python
#@brief     see if from DOSCAR, system appears insulating to VASP
#@author    Tam Mayeshiba
#@date      1/21/11
###
def main(existdir, ofile='DOSCAR'):
    """
    Purpose: get middle lines of DOSCAR, around Fermi energy
        inputs:  
            [$1] directory of run
            example: '/home/tam/NaCl'
            [$2] file name (default DOSCAR)
            example: 'DOSCARU'
        outputs:
            whether the system appears to be insulating, plus the gap range 
            None if error
        notes:
            Currently this ONLY works for spin without specified RWIGS,
            so only columns 1, 2, and 3 are read in. 
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Vdoscargap
    import pdb
#   Validation
#   #  directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   file
    fullfile=""
    fullfile = existdir + '/' + ofile
    if not os.path.isfile(fullfile):
        print fullfile, 'is not a valid file.'
        print '(Exiting.)'
        return None
#   get doscar gap
    gaplist=[]
    gaplist = Vdoscargap.main(existdir, ofile)
    if (gaplist == None):
        print 'Error obtaining min and max gap.'
        print '(Exiting.)'
        return None
    if len(gaplist) < 2:
        return 'No gap detected.'
    fltmingap = gaplist[0]
    fltmaxgap = gaplist[1]
    fltgapwidth = fltmaxgap - fltmingap
    strdecide=""
    if (fltmingap < 0.25) and (fltgapwidth > 4):
        strdecide = 'Insulator?'
    elif (fltmingap < 0.25) and (fltgapwidth > 1):
        strdecide = 'Semiconductor?'
    elif (fltmingap < 0.25):
        strdecide = 'Semimetal?'
    else:
        strdecide = 'No gap around E(Fermi) detected.'
    strdecide = strdecide + ' : ' + str(fltmingap) + ', ' + str(fltmaxgap)
    strdecide = strdecide + ' : ' + str(fltgapwidth)
    return strdecide 
#
#
def loop(topdir, ofile='DOSCAR'):
    """
    Purpose: gets bandgap determination for all files in top directory
        inputs:  
            [$1] top directory 
            example: '/home/tam/NaClcollection'
            [$2] DOSCAR file name (default 'DOSCAR')
        outputs:
            None - error
            list of bandgap determinations
    """
#   Modules
    import os
    import sys
    import Vdoscarinsulator
    import Xvaliddirectory
    import Xgetdirs
#   Validate directory
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   other validation will be handled by Vgetenergy
    dirlist=[]
    dirlist = Xgetdirs.main(topdir, 1, 5)
    mydir=""
    resultlist=[]
    noresultlist=[]
    for mydir in dirlist:
        myresult = Vdoscarinsulator.main(mydir, ofile)
        if (myresult != None):
            resultlist.append(mydir + ': ' + str(myresult))
        else:
            noresultlist.append(mydir)
    print 'Type determinations: '
    for myresult in resultlist:
        print myresult
    if len(noresultlist) > 0:
        print 'Runs without a determination: '
        for myresult in noresultlist:
            print myresult
    return True

            
