#!/usr/bin/env python
##
#@package   Vdoscarmid
#@ingroup   vasp
#@ingroup   python
#@brief     get middle lines of DOSCAR, around Fermi energy
#@author    Tam Mayeshiba
#@date      1/21/11 
###
def main(existdir, ofile='DOSCAR'):
    """
    Purpose: get middle lines of DOSCAR, around Fermi energy
        inputs:  
            [$1] directory of run
            example: '/home/tam/NaCl'
            [$2] file name (default DOSCAR)
            example: 'DOSCARU'
        outputs:
            list of energy:total DOS around Fermi energy
            None if error
        notes:
            Currently this ONLY works for spin without specified RWIGS,
            so only columns 1, 2, and 3 are read in. 
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Vfermi
    import pdb
#   Validation
#   #  directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   file
    fullfile=""
    fullfile = existdir + '/' + ofile
    if not os.path.isfile(fullfile):
        print fullfile, 'is not a valid file.'
        print '(Exiting.)'
        return None
#   get Fermi energy
    fltfermi=0
    fltfermi = Vfermi.main(existdir, ofile)
    if (fltfermi == None):
        print 'Could not isolate Fermi energy.'
        print '(Exiting.)'
        return None
#   open doscar
    doscarfile=""
    readln=""
    linesplit=[]
    dosup=""
    fltdosup=0
    dosdown=""
    fltdosdown=0
    rawenergy=""
    fltrawenergy=0
    fltadjenergy=0
    fltdossum=0
    strdossum=""
    stradjenergy=""
    strreturn=""
    returnme=[]
    doscarfile = open(fullfile, 'rb')
    readln = doscarfile.readline() #skip line 1
    readln = doscarfile.readline() #skip line 2
    readln = doscarfile.readline() #skip line 3
    readln = doscarfile.readline() #skip line 4
    readln = doscarfile.readline() #skip line 5
    readln = doscarfile.readline() #skip line 6
    readln = doscarfile.readline() #read line 7
    #pdb.set_trace() #DEBUG
#line 7 is usually way in the core anyway.
    linesplit = readln.split()
    if len(linesplit) < 5:
        print 'This DOSCAR does not seem to conform to the format'
        print 'Energy, DOS up, DOS down, integrated up, integrated down.'
        print '(Exiting.)'
        doscarfile.close()
        return None
    while (readln):
        linesplit = readln.split()    
        rawenergy = linesplit[0]
        try:
            fltrawenergy = float(rawenergy)
        except (ValueError, TypeError):
            print 'Bad value', rawenergy, 'for energy.'
            print '(Exiting.)'
            doscarfile.close()
            return None
        fltadjenergy = fltrawenergy - fltfermi
#   #   only continue loop for a small range of values
        if (fltadjenergy > -1) and (fltadjenergy < 10):
            dosup = linesplit[1]
            dosdown = linesplit[2]
            try:
                fltdosup = float(dosup)
            except (ValueError, TypeError):
                print 'Bad value', dosup, 'for DOS up.'
                print '(Exiting.)'
                doscarfile.close()
                return None
            try:
                fltdosdown = float(dosdown)
            except (ValueError, TypeError):
                print 'Bad value', dosup, 'for DOS up.'
                print '(Exiting.)'
                doscarfile.close()
                return None
            fltdossum = fltdosup + fltdosdown
            strdossum = str(fltdossum)
            stradjenergy = str(fltadjenergy)
            strreturn = stradjenergy + ':' + strdossum
            returnme.append(strreturn)
        readln = doscarfile.readline()
    doscarfile.close()
    return returnme
