#!/usr/bin/env python
##
#@package   Vdoscarsumlimit 
#@ingroup   vasp
#@ingroup   python
#@brief     Sum up doscar for a range of atoms
#@author    Tam Mayeshiba
#@date      8/23/11 copied from Vdoscarsum, added limit parameters    
###
def main(existdir, fname='DOSCAR', atomstart=0, atomfin=0, atomlist=""):
    """
    Purpose: Sum up doscar for all atoms.
        inputs:  
            [$1] directory
            example: /home/tam/Nacl
            [$2] file name - default 'DOSCAR'
            [$3] atom with which to start - default 0: look to atomlist
            [$4] atom with which to finish - default 0: look to atomlist
                    Range for atomstart to atomfin is inclusive.
            [$5] period-delimited list of atoms to look for
            example: "9.10.11.12"
        outputs:
            dos_parsed with same columns as per-atom sections in DOSCAR
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Pgetline
    #import pdb
#   validation
#   #  existdir
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   # file name
    if type(fname) != str:
        print fname, 'should be a string.'
        print '(Exiting.)'
        return None
#   Validate file
    fullpath=""
    fullpath = os.path.join(existdir, fname)
    if not os.path.isfile(fullpath):
        print fullpath, 'is not a file.'
        print '(Exiting.)'
        return None
#   #   atomstart
    atomstart = Xvalidinteger.main(atomstart, 0, 1000)
    if (atomstart == None):
        return None
#   #   atomfin
    atomfin = Xvalidinteger.main(atomfin, 0, 1000)
    if (atomfin == None):
        return None
#   #   atomlist
    atommat=[]
    atomstr=""
    atomint=0
    atomnums=[]
    atominit=0
    if (atomstart == 0) and (atomfin == 0):
        if (atomlist == ""):
            print 'Atomlist is empty and start and finish have not been set.'
            print '(Exiting.)'
            return None
        try:
            atomlist = str(atomlist)
        except (ValueError, TypeError):
            print atomlist, 'should be a string.'
            print '(Exiting).'
            return None
        atommat = atomlist.split('.')
        if len(atommat) == 0:
            print 'No atoms detected in atomlist.'
            print '(Exiting.)'
            return None
        for atomstr in atommat:
            try:
                atomint = int(atomstr)
            except (ValueError, TypeError):
                print 'Error converting', atomstr, 'into integer.'
                print '(Exiting.)'
                return None
            atomnums.append(atomint)
    else:
        atominit = atomstart
        while (atominit <= atomfin):
            atomnums.append(atominit)
            atominit = atominit + 1
#   get number of atoms and number of energy lines and Fermi energy
    line1=""
    line6=""
    list1=[]
    list6=[]
    numatoms=0
    elines=0
    fermi=0
    line1 = Pgetline.main(existdir, fname, '0', 1)
    if (line1 == None):
        print 'Error getting first line from', fullpath
        print '(Exiting.)'
        return None
    line1 = line1.strip()
    line6 = Pgetline.main(existdir, fname, '0', 6)
    if (line6 == None):
        print 'Error getting sixth line from', fullpath
        print '(Exiting.)'
        return None
    line6 = line6.strip()
    list1 = line1.split()
    if len(list1) < 1:
        print 'Error separating first line', line1
        print '(Exiting.)'
        return None
    numatoms = list1[0]
    print 'Number of atoms:', numatoms
    list6 = line6.split()
    if len(list6) < 5:
        print 'Error separating sixth line', line6
        print '(Exiting.)'
        return None
    elines = float(list6[2])
    print 'Number of energy data points per atom:', elines
    fermi = float(list6[3])
    print 'Fermi energy (eV):', fermi
#   start summing up for each atom. Skip the first matrix.
#   #   initialize matrix
    bigmat=[]
    starthere=0
    numcols=0
    numcolline=""
    numcollist=[]
    ect=0
    colct=0
    starthere = 6 + elines
    numcolline = Pgetline.main(existdir, fname, '0', int(starthere + 2))
    if (numcolline == None):
        print 'Error with number of columns.'
        print '(Exiting.)'
        return None
    numcollist = numcolline.split()
    numcols = len(numcollist)
    while (ect < elines):
        bigmat.append([])
        colct = 0
        while (colct < numcols):
            bigmat[ect].append(0)
            colct = colct + 1
        ect = ect + 1
#   #   start reading
    myfile=""
    readline=""
    linesplit=[]
    lineflt=0
    onect=0
    colct=0
    whichatom=1
    count=1 # will store the number line that is currently read
    starthere = 6 + elines + 2 # Skip the first matrix and first leading line
    myfile = open(fullpath, 'rb')
    readline = myfile.readline() #count = 1
    while (readline):
        if (onect == elines):
            onect = -1
            whichatom = whichatom + 1
        if (count < starthere):
            onect = -1
        elif (onect < elines) and (onect >= 0) and (whichatom in atomnums):
            linesplit = readline.split()
            colct = 0
            #pdb.set_trace()
            while (colct < numcols):
                lineflt = float(linesplit[colct])
                #print 'onect: ',onect,' colct:',colct
                bigmat[onect][colct] = bigmat[onect][colct] + lineflt
                colct = colct + 1
            #print bigmat[onect]
        readline = myfile.readline()
        onect = onect + 1
        count = count + 1
    myfile.close()
    myfile = open(os.path.join(existdir, 'dos_limited_' + fname), 'wb')
    oneitem=""
    #pdb.set_trace()
    ect=0
    colct=0
    atominit = 0
    myfile.write('Atoms: ')
    for atominit in atomnums:
        myfile.write(str(atominit) + ' ')
    myfile.write('\n')
    myfile.write('Num atoms: ' + str(numatoms) + '\n')
    myfile.write('Number of energy lines per atom: ' + str(elines) + '\n')
    myfile.write('Fermi energy: ' + str(fermi) + '\n')
    while (ect < elines):
        colct = 0
        while (colct < numcols):
            myfile.write(str(bigmat[ect][colct]))
            myfile.write(' ')
            colct = colct + 1
        myfile.write('\n')
        ect = ect + 1
    myfile.close()
    return True



        
