#!/usr/bin/env python
##
#@package   Velasticmodel
#@ingroup   other
#@ingroup   python
#@brief     From epitaxial strain of cube, get Young's modulus and 
#           Poisson's ratio
#@author    Tam Mayeshiba
#@date      11/18/2012
###

def main(mydir, flaglist="", zerodir="", searchstr="*bulk_static"):

    """
    Epitaxial strain fitting for 2*Youngs modulus/(3*(1-Poissons ratio))
    Assumes equal epitaxial strain.
    Assumes that the unstrained lattice is approximately cubic
    INPUTS:
        mydir <str> = directory
        flaglist <str> = space-delimited list of values to include (designate as 1's) or exclude (designate as 0's)
        zerodir <str> = directory of zero bulk static
        searchstr <str>: string to wildcard search
    """
    import Vgetenergy
    import Vgetvolume
    import Vgettotmom
    import Platticevec
    import Pgetline
    import math
    import os
    import sys
    import pdb
    import Xgetdirs
    import numpy
    #
    mylistall=[]
    mylist=[]
    edenslist=[]
    poissonlist=[] #using approx of poisson's ratio approx delta z / delta x 
    xstrainlist=[]
    ystrainlist=[]
    zstrainlist=[]
    ymodlist=[]
    maglist=[]
    if "neb" in searchstr:
        mylistall = Xgetdirs.main(mydir, 2, 3, searchstr)
    else:
        mylistall = Xgetdirs.main(mydir, 1, 2, searchstr)
    finishedlist=[]
    alldir=""
    for alldir in mylistall:
        if not os.path.isfile(alldir + "/OUTCAR"):
            pass
        elif Pgetline.main(alldir, "OUTCAR", "User", "X") == None:
            pass
        else:
            finishedlist.append(alldir)
    if flaglist=="":
        mylist = finishedlist
        idx=0
        okays=[]
        while idx < len(mylist):
            okays.append(1)
            idx = idx + 1
    else:
        idx = 0
        okays = flaglist.split()
        while idx < len(okays):
            okays[idx] = int(okays[idx])
            idx = idx + 1
        idx = 0
        if not (len(okays) == len(finishedlist)):
            print "Error in flag list size."
            return None
        while idx < len(okays):
            if okays[idx] == 1:
                mylist.append(finishedlist[idx])
            idx = idx + 1
        idx = 0
    onedir=""
    if mylist=="" or mylist==None or mylist==[]:
        print "Empty list. Exiting."
        return None
    myvol=""
    myenergy=""
    mymag=""
    flag=0
    dlenx=0
    dleny=0
    dlenz=0
    xlen=0
    ylen=0
    zlen=0
    epsx=0
    prhs=0
    plhs=0
    poisson=0
    latticeparams=[]
    zerolattice=0
    mypoisson=0
    zerolatt = Platticevec.main(zerodir, "CONTCAR")
    zerovol = Vgetvolume.main(zerodir)
    try:
        zerovol = float(zerovol)
    except (TypeError, ValueError):
        print "Could not convert volume."
        return None
    zeroenergy = Vgetenergy.main(zerodir)
    for onedir in mylist:
        myvol=Vgetvolume.main(onedir)
        myenergy=Vgetenergy.main(onedir)
        mymag=Vgettotmom.main(onedir)
        edenslist.append((float(myenergy)-float(zeroenergy))/float(myvol))
        if not (mymag == None):
            maglist.append(float(mymag))
        else:
            maglist.append(0)
        latticeparams=Platticevec.main(onedir,"CONTCAR")
        if latticeparams==None:
            print "Error getting lattice parameters for ",onedir
            flag=1
        if flag==1:
            poissonlist.append(0)
            xstrainlist.append(0)
            ystrainlist.append(0)
            zstrainlist.append(0)
            flag=0
        else:
            xlen = float(latticeparams[0][0])
            ylen = float(latticeparams[1][1])
            zlen = float(latticeparams[2][2])
            dlenx = xlen-zerolatt[0][0]
            dleny = ylen-zerolatt[1][1]
            dlenz = zlen-zerolatt[2][2]
            xstrainlist.append(dlenx/zerolatt[0][0])
            ystrainlist.append(dleny/zerolatt[1][1])
            zstrainlist.append(dlenz/zerolatt[2][2])
            if math.fabs(dlenx) < 0.005:
                pass
            else:
                prhs = 1 - ((math.fabs(dlenz)/4)/zerolatt[2][2])
                plhs = 1 + (math.fabs(dlenx)/zerolatt[0][0])
                prhs = math.log(prhs)
                plhs = math.log(plhs)
                mypoisson = -1 * prhs / plhs
                poissonlist.append(mypoisson) #trans delta L = 2*dlenz; and stretching on 2 axes is another factor of 2?
                #poissonlist.append(-1*(dlenz/4)/dlenx) #trans delta
    poissonmat = numpy.mat(poissonlist)
    poisson=0
    poisson = numpy.mean(poissonmat)
    yfit = numpy.polyfit(xstrainlist, edenslist, 2, None, True)
    ymod = yfit[0][0]*(1-poisson)
    twoyover31minusv = 2*ymod/(3*(1-poisson))
    idx=0
    print "Dirs: "
    while idx < len(okays):
        print okays[idx], ": ", finishedlist[idx]
        idx = idx + 1
    idx=0
    print "Dirs used: "
    while idx < len(mylist):
        print mylist[idx]
        idx = idx + 1
    idx=0
    print "Y fit: "
    while idx < len(yfit[0]):
        print yfit[0][idx]
        idx = idx + 1
    idx=0
    print "Approx Y mod: ",ymod
    print "Avg poisson: ",poisson
    print "2/3 * (Y/(1-v)): ",str(0.66666666 * yfit[0][0])
    print "Residuals etc: ", yfit[1]
    #return printing
    retstr=""
    retstr = retstr + " :MAG: "
    idx=0
    while idx < len(maglist):
        retstr = retstr + str(maglist[idx]) + " "
        idx = idx + 1
    retstr = retstr + " :strainxx: "
    idx=0
    while idx < len(xstrainlist):
        retstr = retstr + str(xstrainlist[idx]) + " "
        idx = idx + 1
    retstr = retstr + " :strainyy: "
    idx=0
    while idx < len(ystrainlist):
        retstr = retstr + str(ystrainlist[idx]) + " "
        idx = idx + 1
    retstr = retstr + " :strainzz: "
    idx=0
    while idx < len(zstrainlist):
        retstr = retstr + str(zstrainlist[idx]) + " "
        idx = idx + 1
    retstr = retstr + " :Energdens: "
    idx=0
    while idx < len(edenslist):
        retstr = retstr + str(edenslist[idx]) + " "
        idx = idx + 1
    retstr = retstr + " :Okay_is_1: " + flaglist
    retstr = retstr + " :Y_in_eV/A^3: " + str(ymod)    
    retstr = retstr + ":Poisson:" + str(poisson)
    retstr = retstr + ":2Y/3(1-v): " + str(twoyover31minusv) + " "
    retstr = retstr + ":Resid in yfit: " + str(yfit[1]) + " "
    return retstr
