#!/usr/bin/env python
##
#@package   Vencut.py
#@ingroup   vasp
#@ingroup   python
#@brief     get max ENMAX from POTCAR, mult by certain value
#@author    Tam Mayeshiba
#@date      
###
def main(existdir, multby=1.5, fname='POTCAR'):
    """
    Purpose:
        inputs:  get max ENMAX from POTCAR, mult by certain value
            [$1] directory containing POTCAR
            example: /home/tam/NaCl
            [$2] float by which to multiply - default 1.5
                (1.5 for ionic relaxations, 1.3 for certain other uses?)
            [$3] file name to use - default 'POTCAR'
        outputs:
            None if error
            suitable ENMAX, rounded to nearest eV
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgrep
    import Pgetseg
#   Validation
#   #  existdir
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   # multby
    multflt=""
    try:
        multflt = float(multby)
    except (ValueError, TypeError):
        print multby, 'could not be converted into a float.'
        print '(Exiting.)'
        return None
#   #   fname
    fullpath=""
    try:
        fname = str(fname)
    except (ValueError, TypeError):
        print fname, 'should be a string.'
        print '(Exiting.)'
        return None
    fullpath = existdir + '/' + fname
    if not os.path.isfile(fullpath):
        print fullname, 'was not found.'
        print '(Exiting.)'
        return None
#   get ENMAX list
    linelist=[]
    linelist = Xgrep.main(existdir, fname, 'ENMAX')
    if (linelist == None):
        print 'Error getting ENMAX values from', fullpath
        print '(Exiting.)'
        return None
    if len(linelist) == 0:
        print 'List for ENMAX lines for', fullpath, 'is empty.'
        print '(Exiting.)'
        return None
#   parse list
    lineitem=[]
    enmaxlist=[]
    enmaxseg=""
    enmaxflt=0
    for lineitem in linelist:
        enmaxseg = Pgetseg.main(lineitem, 'ENMAX', ';')
        if (enmaxseg == None):
            print 'Error getting ENMAX from', lineitem
            print '(Exiting.)'
            return None
        enmaxseg = enmaxseg.strip()
        if len(enmaxseg) == 0:
            print 'ENMAX could not be isolated from', lineitem
            print '(Exiting.)'
            return None
        if enmaxseg[0] == '=':
            enmaxseg = enmaxseg[1:].strip()
        try:
            enmaxflt = float(enmaxseg)
        except (ValueError, TypeError):
            print enmaxseg, 'could not be converted into an ENMAX float.'
            print '(Exiting.)'
            return None
        enmaxlist.append(enmaxflt)
#   get max
    maxenmax=0
    encutreturn=0
    maxenmax = max(enmaxlist)
    encutreturn = multflt * maxenmax 
    return int(round(encutreturn))


