#!/usr/bin/env python
##
#@package   Vfermi
#@ingroup   vasp
#@ingroup   python
#@brief     get Fermi energy from DOSCAR
#@author    Tam Mayeshiba
#@date      1/21/11 
###
def main(existdir, ofile='DOSCAR'):
    """
    Purpose: Get Fermi energy from DOSCAR
        inputs:  
            [$1] description 
            example:
            [$2] file name (default DOSCAR)
            example: 'DOSCARU'
        outputs:
            fermi energy as a FLOAT
            None if error
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Pgetline
#   Validation
#   #  directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   file
    fullfile=""
    fullfile = existdir + '/' + ofile
    if not os.path.isfile(fullfile):
        print fullfile, 'is not a valid file.'
        print '(Exiting.)'
        return None
#   get 6th line of DOSCAR
    fermiline=""
    fermiline = Pgetline.main(existdir, ofile, 0, 6)
#   get 4th segment
    linesplit=[]
    linesplit = fermiline.split()
    #print linesplit
    fermienergy=""
    try:
        fermienergy = linesplit[3]
    except (IndexError, TypeError):
        print 'Error isolating Fermi energy from', fullfile
        print '(Exiting.)'
        return None
    try:
        fermienergy = float(fermienergy)
    except (ValueError, TypeError):
        print 'Error converting', fermienergy, 'to float.'
        print '(Exiting.)'
        return None
    return fermienergy
