#!/usr/bin/env python
##
#@package   Vgetnebstatus
#@ingroup   vasp
#@ingroup   python
#@brief     gets status of neb run
#@details   checks whether all images have finished
#@author    Tam Mayeshiba
#@date      3/1/11 created 
###
def main(mydir, ofile='OUTCAR', detail=0):
    """
    Purpose: gets status of neb run (i.e. whether all images have finished)
        inputs:  
            [$1] directory 
            example: '/home/tam/NaClneb'
            [$2] file name (default 'OUTCAR')
            example: 'OUTCAR_unfinishedrun'
            [$3] print result by image
                 1 - yes
                 0 - no (default)
        outputs:
            None - error
            True - neb has finished (all images have finished)
            False - neb has not finished
    """
#   Modules:
    import os
    import sys
    import Vgetstatus
    import Xgetdirs
    import Xvaliddirectory
    import Xvalidinteger
#   Validation
#   # directory
    mydir = Xvaliddirectory.main(mydir)
    if (mydir == None):
        print '(Exiting.)'
        return None
#   # file name validation will be handled by Vgetstatus
#   # detail
    detail = Xvalidinteger.main(detail, 0, 1)
    if (detail == None):
        print '(Exiting.)'
        return None
#   get image directories
    imdirlist=[]
    imdirlist = Xgetdirs.main(mydir, 1, 1)
    lendirlist=0
    numims=0
    lendirlist = len(imdirlist)
#   #   there should be at least 3 folders in any neb: 00, 01, and 02
    if (lendirlist < 3):
        print 'No image directories detected.'
        print '(Exiting.)'
        return None
    numims = lendirlist - 2 #not counting the endpoint images
    thisimdir=""
    imcount=0
    imresult=""
    overalldone=True
    for thisimdir in imdirlist:
        if (imcount > numims):
            break #have processed all images
        if (imcount == 0):
            imcount = imcount + 1 #skip the first folder, 00
            continue
        imresult = Vgetstatus.main(thisimdir, ofile)
        if (imresult == None):
            overalldone = False
        if (detail == 1):
            print thisimdir, ': ', imresult 
        imcount = imcount + 1
    return overalldone
#
#
def loop(topdir, ofile='OUTCAR', detail=0):
    """
    Purpose: gets status of neb run (i.e. whether all images have finished)
        inputs:  
            [$1] top directory 
            example: '/home/tam/NaClnebcollection'
            [$2] file name (default 'OUTCAR')
            example: 'OUTCAR_unfinishedrun'
            [$3] print result by image
                 1 - yes
                 0 - no (default)
        outputs:
            None - error
            prints list of results for whether neb runs have finished
            True if all completed
            False otherwise
    """
#   Modules:
    import os
    import sys
    import Vgetnebstatus
    import Xvaliddirectory
    import Xvalidinteger
    import Xgetdirs
#   Validation
#   # directory
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   # ofile will be handled by Vgetnebstatus
#   # detail
    detail = Xvalidinteger.main(detail, 0, 1)
    if (detail == None):
        print '(Exiting.)'
        return None
#   loop
    dirlist=[]
    dirlist = Xgetdirs.main(topdir, 1, 1)
    mydir=""
    myresult=""
    resultlist=[]
    noresultlist=[]
    alltrue = True
    for mydir in dirlist:
        myresult = Vgetnebstatus.main(mydir, ofile, detail)
        if (myresult != None):
            if (myresult == False):
                alltrue = False
            resultlist.append(mydir + ': ' + str(myresult))
        else:
            alltrue = False
            noresultlist.append(mydir)
    print 'NEB status:'
    for myresult in resultlist:
        print myresult
    if len(noresultlist) > 0:
        print 'Error evaluating:'
        for myresult in noresultlist:
            print myresult
    return alltrue
