#!/usr/bin/env python
##
#@package   Vgetstats
#@ingroup   vasp
#@ingroup   python
#@brief     gets line of parameter information
#@author    Tam Mayeshiba
#@date      7/14/11
###
def main(existdir="", noheader=0, dlc="|"):
    """
    Purpose:
        inputs:  
            [$1] directory of VASP run
            example: /home/tam/NaCl
            [$2] suppress header
                0 - no (default)
                1 - yes
            [$3] delimiting character
                default |
                can choose another
        outputs:
            one line with volume, pressure, last E0 from OSZICAR, etc.
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Vgetstatus
    import Pgetline
    import Pgetseg
#   Validation
#   #  directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == ""):
        print '(Exiting.)'
        return None
#   # suppress header
    noheader = Xvalidinteger.main(noheader, 0, 1)
    if (noheader == ""):
        print '(Exiting.)'
        return None
#   # delimchar
    try:
        dlc = str(dlc)
    except (ValueError, TypeError):
        print 'Delimiting character', dlc, 'should be a string or character.'
        print '(Exiting.)'
        return None
#   Initialize fields
    FullPath="None"
    ShortName="None"
    E0="0"
    mag="0"
    pressure="0"
    volume="0"
    CPUtime="0"
    kpoints="None"
    PREC="None"
    LCLIMB="None"
    ISMEAR="None"
    SIGMA="None"
    IBRION="None"
    ISIF="None"
    NELECT="None"
    NUPDOWN="None"
    MAGMOM="None"
#   only proceed for finished runs
    CPUtime = Vgetstatus.main(existdir)
    if (CPUtime == None):
        print 'Run in', existdir, 'does not appear to be complete. Skipping.'
        return None
    CPUtime = str.strip(CPUtime)
#   get each field
    FullPath = existdir
    ShortName = os.path.basename(existdir)
    E0andmag=""
    E0andmag = Pgetline.main(existdir, 'OSZICAR', 'E0', 'X', 1)
    if (E0andmag != None):
        E0 = str.strip(str(Pgetseg.main(E0andmag, 'E0=', 'd')))
        mag = str.strip(str(Pgetseg.main(E0andmag, 'mag=', 0)))
    pressstr=""
    pressstr = Pgetline.main(existdir, 'OUTCAR', 'pressure', 'X', 1)
    if (pressstr != None):
        pressure = str.strip(str(Pgetseg.main(pressstr, 'pressure =', 'kB')))
    volstr=""
    volstr = Pgetline.main(existdir, 'OUTCAR', 'volume', 'X', 1)
    if (volstr != None):
        volume = str.strip(str(Pgetseg.main(volstr, 'volume of cell :', 0)))
    #CPUtime already retrieved above
    kpoints = str.strip(str(Pgetline.main(existdir, 'KPOINTS', 0, 4)))
    PREC = str.strip(str(Pgetline.main(existdir, 'INCAR', 'PREC', 'X', 1)))
    LCLIMB = str.strip(str(Pgetline.main(existdir, 'INCAR', 'LCLIMB', 'X', 1)))
    ISMEAR = str.strip(str(Pgetline.main(existdir, 'INCAR', 'ISMEAR', 'X', 1)))
    SIGMA = str.strip(str(Pgetline.main(existdir, 'INCAR', 'SIGMA', 'X', 1)))
    IBRION = str.strip(str(Pgetline.main(existdir, 'INCAR', 'IBRION', 'X', 1)))
    ISIF = str.strip(str(Pgetline.main(existdir, 'INCAR', 'ISIF', 'X', 1)))
    NELECT = str.strip(str(Pgetline.main(existdir, 'INCAR', 'NELECT', 'X', 1)))
    NUPDOWN = str(Pgetline.main(existdir, 'INCAR', 'NUPDOWN', 'X', 1))
    NUPDOWN = str.strip(NUPDOWN)
    MAGMOM = str.strip(str(Pgetline.main(existdir, 'INCAR', 'MAGMOM', 'X', 1)))
#   return string
    returnstr=""
    returnstr = FullPath + dlc + ShortName + dlc + E0 + dlc + mag + dlc
    returnstr = returnstr + pressure + dlc + volume + dlc 
    returnstr = returnstr + CPUtime + dlc + kpoints + dlc
    returnstr = returnstr + PREC + dlc + LCLIMB + dlc + ISMEAR + dlc
    returnstr = returnstr + SIGMA + dlc + IBRION + dlc + ISIF + dlc
    returnstr = returnstr + NELECT + dlc + NUPDOWN + dlc + MAGMOM
#   header
    header=""
    if (noheader == 0):
        header = "FullPath" + dlc + "ShortName" + dlc + "E0" + dlc
        header = header + "mag" + dlc + "pressure" + dlc + "volume" + dlc
        header = header + "CPUtime" + dlc
        header = header + "kpoints" + dlc + "PREC" + dlc + "LCLIMB" + dlc 
        header = header + "ISMEAR" + dlc + "SIGMA" + dlc + "IBRION" + dlc 
        header = header + "ISIF" + dlc + "NELECT" + dlc + "NUPDOWN" + dlc 
        header = header + "MAGMOM"
        header = header + '\n'
        returnstr = header + returnstr
    return returnstr
#
#
def loop(topdir="", dlc="|"):
    """
    Purpose:
        inputs:  
            [$1] directory of VASP run
            example: /home/tam/NaCl
            [$2] delimiting character
                default |
                can choose another
        outputs:
            one line with volume, pressure, last E0 from OSZICAR, etc.
    """
#   Modules:
    import os
    import sys
    import Vgetstats
    import Xvaliddirectory
    import Xgetdirs
    import Pmakemonfile
#   Validation
#   #  directory
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == ""):
        print '(Exiting.)'
        return None
#   # delimchar
    try:
        dlc = str(dlc)
    except (ValueError, TypeError):
        print 'Delimiting character', dlc, 'should be a string or character.'
        print '(Exiting.)'
        return None
#   get directories
    dirlist=[]
    onedir=""
    onedirstats=""
    dirlist = Xgetdirs.main(topdir, 1, 5)
    if (dirlist == None):
        print 'Error getting subdirectories for', topdir
        print '(Exiting.)'
        return None
    if len(dirlist) == 0:
        print 'No subdirectories found for', topdir
        print '(Exiting.)'
        return None
    firstct=0
    myfilename=""
    myfilename = Pmakemonfile.main(topdir)
    if (myfilename == None):
        print 'Could not create unique file name for writing.'
        print '(Exiting.)'
        return None
    myfile=""
    myfile = open(myfilename, 'ab')
    for onedir in dirlist:
        if (firstct == 1):
            onedirstats = Vgetstats.main(onedir, 1, dlc)
        else:
            onedirstats = Vgetstats.main(onedir, 0, dlc)
        firstct = 1
        myfile.write(str(onedirstats) + '\n')
    myfile.close()
    print 'Done.'
    return True

