#!/usr/bin/env python
##
#@package   Vgetstatus
#@ingroup   vasp
#@ingroup   python
#@brief     gets total CPU time, or if haven't finished
#@author    Tam Mayeshiba
#@date      1/3/11 
#@date      1/11/11 allow to look at differently named files (OUTCAR, OUTCARU)
#                   for files that might have been renamed
#@date      1/11/11 removing error notification messages in main; return of
#                   None should be assumed to be an error
###
def main(mydir, ofile='OUTCAR'):
    """
    Purpose: gets total CPU time from OUTCAR, or if run hasn't finished
        inputs:  
            [$1] directory 
            example: '/home/tam/NaCl1'
            [$2] file name (default 'OUTCAR')
            example: 'OUTCAR_unfinishedrun'
        outputs:
            None - error
            total CPU time
    """
#   Modules:
    import os
    import sys
    import Pgetline
    import Pgetseg
#   validation handled by Pgetline
#   #
#   get total CPU time
    myline=""
    myseg=""
    myline = Pgetline.main(mydir, ofile, 'Total CPU', 'X', 1)
    if (myline == None):
        #print 'Error reading', ofile,'for', mydir
        #print '(Exiting.)'
        return None
    myseg = Pgetseg.main(myline, '(sec):')
    if (myseg == None):
        #print 'Error isolating total CPU time for', mydir
        #print '(Exiting.)'
        return None
    return myseg
#
#
def loop(topdir, ofile='OUTCAR'):
    """
    Purpose: gets total CPU time from directories under directory
        inputs:  
            [$1] top directory 
            example: '/home/tam/NaClcollection'
            [$2] file name to use (default OUTCAR)
            example: 'OUTCAR_unfinished'
        outputs:
            None - error
            total CPU time from OUTCAR
    """
#   Modules
    import os
    import sys
    import Vgetstatus
    import Xvaliddirectory
    import Xgetdirs
#   Validate directory
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   other validation will be handled by Vgetenergy
    dirlist=[]
    dirlist = Xgetdirs.main(topdir, 1, 5)
    mydir=""
    resultlist=[]
    noresultlist=[]
    for mydir in dirlist:
        myresult = Vgetstatus.main(mydir, ofile)
        if (myresult != None):
            resultlist.append(mydir + ': ' + str(myresult))
        else:
            noresultlist.append(mydir)
    print 'Total CPU time: '
    for myresult in resultlist:
        print myresult
    if len(noresultlist) > 0:
        print 'Runs not completed: '
        for myresult in noresultlist:
            print myresult
    return True
