#!/usr/bin/env python
##
#@package   Vgetstrain
#@ingroup   other
#@ingroup   python
#@brief     Get strain tensor from activated stress tensor (assume only principal stress) and energy difference
#@author    Tam Mayeshiba
#@date      11/18/2012
###

def main(mydir, initstring, actstring, basestring="", zerodelh="", cond=0.005, verbose=0):

    """
    Get strain tensor from activated stress tensor and energy difference between act and init state
    Assumes equal epitaxial strain.
    Assumes that the unstrained lattice is approximately cubic
    INPUTS:
        mydir <str> = directory housing strain series
        initstring <str> = string to match initial directories; include asterices for wildcards
        actstring <str> = string to match activated directories; include asterices for wildcards
        basestring <str> = string to match a baseline directory, for example, a bulk directory if the initial state is not the bulk
        zerodelh <float> = zero-strain change in energy
        condset <float> = conditional cutoff for scipy.linalg.lstsq
        verbose <int> = 0 - do not print extra information (default)
                        1 - print extra information
    """
    import Vgetenergy
    import Vgetvolume
    import Vgetstress
    import Platticevec
    import scipy
    import math
    import os
    import sys
    import pdb
    import Xgetdirs
    import numpy
    from scipy import linalg
    #
    mylist=[]
    mylist = Xgetdirs.main(mydir, 1, 1)
    strdir=""
    actdirs=[]
    initdirs=[]
    basedirs=[]
    for strdir in mylist:
        actlist=[]
        actlist = Xgetdirs.main(strdir, 1, 5, actstring)
        initlist=[]
        initlist = Xgetdirs.main(strdir, 1, 5, initstring)
        baselist=[]
        flag=0
        if not (basestring == ""):
            baselist = Xgetdirs.main(strdir, 1, 5, basestring)
        if not (len(actlist) == 1):
            flag = 1
        if not (len(initlist) == 1):
            flag = 1
        if not (basestring == ""):
            if not (len(baselist) == 1):
                flag = 1
        if flag == 1:
            print "Ambiguity finding folders under ", strdir, ". Skipping."
        else:
            actdirs.append(actlist[0])
            initdirs.append(initlist[0])
            if not (basestring == ""):
                basedirs.append(baselist[0])
    numpairs = len(actdirs)
    idx = 0
    stressstring=""
    stresslist=[]
    strstring=""
    ediffstr=""
    myvol=""
    volstr=""
    stressbasestring=""
    basestresslist=[]
    strbasestr=""
    if numpairs == 0:
        print "No pairs found in ", mydir
        return None
    while idx < numpairs:
        flag = 0
        if not (basestring==""):
            stressbasestring = Vgetstress.main("", basedirs[idx])
            if stressbasestring == None:
                print "Error getting base stress. Skipping."
                flag = 1
            else:
                basestresslist = stressbasestring.split(" ", 1)
                if verbose == 1:
                    print "Stress base string: ", stressbasestring
                    print "Base stress list: ", basestresslist
        stressstring = Vgetstress.main(initdirs[idx],actdirs[idx])
        if stressstring == None:
            print "Error getting stress. Skipping."
        else:
            stresslist = stressstring.split(" ", 1) # energy diff, sigma11, sigma22, sigma33
            ediffstr = ediffstr + str(stresslist[0]) + ";"
            if not (basestring == ""):
                if flag == 1:
                    print "No base stress. Exiting."
                    return None
                else:
                    strbasestr = strbasestr + str(basestresslist[1]) + " 0;"
            strstring = strstring + str(stresslist[1]) + " "
            myvol = Vgetvolume.main(actdirs[idx])
            strstring = strstring + str(1/float(myvol)) + ";" #fit the non-strain contribution of strain-dependent migration energy
            volstr = volstr + str(myvol).strip() + ";"
        idx = idx + 1
    strstring = strstring[:-1] #remove last "1;"
    ediffstr = ediffstr[:-1] #remove last ";"
    volstr = volstr[:-1]
    stressmat = scipy.mat("[" + strstring + "]",float)
    ediffmat = scipy.mat("[" + ediffstr + "]",float)
    volmat = scipy.mat("[" + volstr + "]", float)
    if verbose == 1:
        print "Stress mat: ", stressmat
        print "Ediff mat: ", ediffmat
        print "Vol mat: ", volmat
    if not (basestring == ""):
        strbasestr = strbasestr[:-1]
        basemat = scipy.mat("[" + strbasestr + "]", float)
        if verbose == 1:
            print "Base mat: ", basemat
        stressmat = stressmat - basemat
    # using deltaH = V * [delta sigma tensor * strain tensor] or equiv:
    #       deltaH = V * [sigma tensor * delta strain tensor]
    if not (zerodelh == ""):
        ediffmat = ediffmat - float(zerodelh)
    ediffmatovervol=ediffmat/volmat
    #pdb.set_trace()
    if not (cond == ""):
        myreturn=linalg.lstsq(stressmat, ediffmatovervol, cond)
    else:
        myreturn=linalg.lstsq(stressmat, ediffmatovervol)
    try:
        eps11=myreturn[0][0][0]
        eps22=myreturn[0][1][0]
        eps33=myreturn[0][2][0]
        extra=myreturn[0][3][0]
        resid=myreturn[1][0]
    except IndexError:
        try:
            eps11=myreturn[0][0]
            eps22=myreturn[0][1]
            eps33=myreturn[0][2]
            extra=myreturn[0][3]
            resid=myreturn[1]
        except IndexError:
            print "Error in least-squares output!"
            return None
    if verbose == 1:
        print "Initial dirs: ", initdirs
        print "Activated dirs: ", actdirs
        print "Activated stress mat (adjusted if nec.): ", stressmat
        if not (basestring == ""):
            print "Base stress mat: ", basemat
        print "Energy differences: ", ediffmat
    return "eps11: " + str(eps11) + " eps22: " + str(eps22) + " eps33: " + str(eps33) + " extra: " + str(extra) + " resid: " + str(resid)

