#!/usr/bin/env python
##
#@package   Vgetstress
#@ingroup   other
#@ingroup   python
#@brief     From change in energies with strain, use stress tensor of activated state to get strain differences between two states
#           Poisson's ratio
#@author    Tam Mayeshiba
#@date      11/18/2012
###

def main(mydirinit, mydiract, verbose=0):

    """
    Get strain energy contribution between two states
    Assumes equal epitaxial strain.
    Assumes that the unstrained lattice is approximately cubic
    INPUTS:
        mydirinit <str> = directory (bulk, or initial state)
                        use "" to get only activated stress
        mydiract <str> = directory (defected, or transition state)
        verbose <int> = 1 - yes, print verbose
                        0 - no, just print end
    OUTPUTS:
       <str> : energy difference (eV)
               sigma11 (eV/Angs^3)
               sigma22
               sigma33
    """
    import Vgetenergy
    import Vgetvolume
    import Platticevec
    import math
    import os
    import sys
    import pdb
    import Xgetdirs
    import numpy
    import Pgetline
    #
    mylist=[]
    edenslist=[]
    poissonlist=[] #using approx of poisson's ratio approx delta z / delta x
    strainlist=[]
    zstrainlist=[]
    ymodlist=[]
    myvol=""
    myenergy=""
    flag=0
    stressactline=""
    stressact=[]
    stressacteV=[]
    stressactline = Pgetline.main(mydiract, "OUTCAR", "in kB", "X", 1)
    if stressactline == None:
        print "No stress tensor in ", mydiract, ". Exiting."
        return None
    stressact=stressactline.split() #first entry is "in", second is "kB"
    stressacteV.append(float(stressact[2])/1602)
    stressacteV.append(float(stressact[3])/1602)
    stressacteV.append(float(stressact[4])/1602)
    energyact=0
    energyinit=0
    energyact = Vgetenergy.main(mydiract)
    if energyact == None:
        print "No energy in ",mydiract, ". Exiting."
        return None
    if not (mydirinit == ""):
        energyinit = Vgetenergy.main(mydirinit)
        if energyinit == None:
            print "No energy in ",mydirinit, ". Exiting."
            return None
        delh = float(energyact) - float(energyinit)
    else:
        delh = float(energyact)
    #return energy difference and principal stress tensor
    return str(delh) + " " + str(stressacteV[0]) + " " + str(stressacteV[1]) + " " + str(stressacteV[2])
