#!/usr/bin/env python
##
#@package   Vibz
#@ingroup   vasp
#@ingroup   python
#@brief     get number of irreducible and total kpoints
#@author    Tam Mayeshiba
#@date      1/11/11 
###
def main(existdir, ofile='IBZKPT', vaspversion=4):
    """
    Purpose: get number of irreducible and total kpoints
        inputs:  
            [$1] folder
            example: /home/tam/NaCl
            [$2] IBZKPT file name (default IBZKPT)
            (sometimes it's been renamed from IBZKPT)
            example: IBZKPT2
            [$3] 4 - VASP version 4 (default)
                 5 - VASP version 5
        outputs:
            # irreducible, # total
            None if error
    """
#   Modules:
    import os
    import sys
    import subprocess
    import Xvaliddirectory
    import Xvalidinteger
    import Pgetseg
#   Validation
#   #  validate directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   validate file
    fullfile = existdir + '/' + ofile
    if not os.path.isfile(fullfile):
        print fullfile, 'could not be found.'
        print '(Exiting.)'
        return None
#   #   validate version
    vaspversion = Xvalidinteger.main(vaspversion, 4, 5)
    if (vaspversion == None):
        return None
#   sum up total number of kpoints (number of each irreducible kpoint)
    #ctproc = subprocess.Popen([ctprocstr], shell=True, stdout=subprocess.PIPE)
    #ctloops = ctproc.communicate()[0]
    #ctproc.wait()
    #print ctloops
#   #   get time per loop
    openibz=""
    openibz = open(fullfile, 'rb')
    readln=""
    readln = openibz.readline() # gets "Automatically generated mesh"
    numirrstr=""
    numirr=0
    numirrstr = openibz.readline() # gets number of irreducible kpoitns
    try:
        numirr = int(numirrstr)
    except (TypeError, ValueError):
        print 'Error getting number of irreducible kpoints for', fullfile
        print '(Exiting.)'
        openibz.close()
        return None
    readln = openibz.readline() # gets "Reciprocal lattice"
    readln = openibz.readline() # gets first kpoint line
    kptstr=""
    kptint=0
    kptsum=0
    while(readln):
        kptstr = Pgetseg.main(readln, 61, 0)
        try:
            kptint = int(kptstr)
        except (TypeError, ValueError):
            print 'Error converting kpoints for', kptstr
            kptint = 0
        kptsum = kptsum + kptint
        readln = openibz.readline()
    openibz.close()
    return str(numirr) + ',' + str(kptsum)
#
#
def loop(topdir, ofile='IBZKPT', vaspversion=4):
    """
    Purpose: get number of irreducible and total kpoints
        inputs:  
            [$1] folder
            example: /home/tam/NaCl
            [$2] IBZKPT file name (default IBZKPT)
            (sometimes it's been renamed from IBZKPT)
            example: IBZKPT2
            [$3] 4 - VASP version 4 (default)
                 5 - VASP version 5
        outputs:
            # irreducible, # total
            None if error
    """
#   Modules:
    import os
    import sys
    import Vibz
    import Xvaliddirectory
    import Xgetdirs
#   Validation
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   other validation will be handled by Vtotaltime
#   Run for list
    dirlist=[]
    dirlist = Xgetdirs.main(topdir)
    mydir=""
    myresult=""
    resultlist=[]
    noresultlist=[]
    for mydir in dirlist:
        myresult = Vibz.main(mydir, ofile, vaspversion)
        if (myresult != None):
            resultlist.append(mydir + ': ' + str(myresult))
        else:
            noresultlist.append(mydir)
    print 'Directory: irreducible kpoints, total kpoints'
    for myresult in resultlist:
        print myresult
    if len(noresultlist) > 0:
        print 'No result for: '
        for myresult in noresultlist:
            print myresult
    return True
