#!/usr/bin/env python
##
#@package   Vjumpdirsnew
#@ingroup   vasp
#@ingroup   python
#@brief     jump in each of several nearest-neighbor directions
#@note      bulk directory should contain full inputfile for an neb
#           Note that 'vac2line' should have a period-delimited list
#           of line numbers in it.
#           Refer to Vonenebcontrol for what should go into inputfile.
#@author    Tam Mayeshiba
#@date      4/11/11 created (copied from Voneneb) 
#@date      5/20/11 copied from Vjumpdirs; use Vonenebcontrol
#@date      5/20/11 get vac1line and vac2str from inputfile
###
def main():
    """
    Purpose: NEB one-vacancy jump in each of several nearest-neighbor
             directions
        inputs: None, but should be run in a directory that contains
            1) a bulk directory ready to run and
            2) a full neb inputfile that references that bulk directory,
            with the following change:
            vac2line = string of line numbers from bulk POSCAR indicating end
                 position for vacancy (will be replaced by starting position
                 line to create ep2), separated by .
                 example: 25.27.37.40.45
        outputs:
            None - if error
        notes:
            refer to Vonenebcontrol for other parameters for the inputfile
    """
#   Modules:
    import os
    import sys
    import shutil
    import subprocess
    import Xvaliddirectory
    import Xvalidinteger
    import Xgetdirs
    import Xinputmatch
    import Vnumatoms
    import Vlastspecdir
    import Vgetmon
    import Vrelax
    import Vgetstatus
    import Vmodstatic
    import Vmakeendpt
    import Prunonewait
    import Pdirautorelaxwait
    import Pmodfilenew
    import Vonenebcontrolsilent
#   get params from inputfile
    mycwd=""
    bulkparams=""
    bulkstartdir=""
    bulkdict={}
    mycwd = os.getcwd()
    bulkparams = 'bulkstartdir.checksec.lagsec.'
    bulkparams = bulkparams + 'encomp.entype.volcomp.voltype.emaddr.maxout.'
    bulkparams = bulkparams + 'vac1line.vac2line.nelectadj.vaspvers'
    bkdict = Xinputmatch.main(mycwd + '/inputfile', bulkparams, 1)
    if (bkdict == None):
        print 'Error getting parameters from input file in', mycwd
        print '(Exiting.)'
        return None
    bulkstartdir = bkdict['bulkstartdir']
#   define top directory
    topdir=""
    topdir = os.path.dirname(bulkstartdir)
#   get number of atomic species and number of each type
    numatomstr=""
    numatomsbulktot=""
    numatomstr = Vnumatoms.main(bulkstartdir)
    if (numatomstr == None):
        print '(Exiting.)'
        return None
    numatomssplit=[]
    numatomssplit = numatomstr.split(':')
    if len(numatomssplit) < 2:
        print 'Error retrieving number of atoms in system.'
        print '(Exiting.)'
        return None
    numatomsbulktot=numatomssplit[0]
#   parse list of vacancy lines
    vac2list=[]
    vac2list = bkdict['vac2line'].split('.')
    vac2item=""
    vac2check=""
    vac2mlist=[]
    if len(vac2list) < 1:
        print 'Vacancy 2 list from,', bkdict['vac2line'], ' is empty.'
        print '(Exiting.)'
        return None
    for vac2item in vac2list:
        vac2check = Xvalidinteger.main(vac2item, 1, None)
        if (vac2check == None):
            print 'Error in vacancy 2 string list,', bkdict['vac2line']
            print '(Exiting.)'
            return None
        vac2mlist.append(vac2check) 
    if len(vac2mlist) < 2:
        print 'Error with vacancy 2 list: fewer than 2 items'
        print '(Exiting.)'
        return None
    print vac2mlist
#   relax bulk
    bulkrel=""
    bulkrel = Vgetmon.main(bulkstartdir)
    if (bulkrel == None) or ('True' not in bulkrel):
        bulkrel = Pdirautorelaxwait.main(bulkstartdir, numatomsbulktot, bkdict['maxout'], 
                bkdict['checksec'], bkdict['lagsec'], bkdict['encomp'],
                bkdict['entype'], bkdict['volcomp'], bkdict['voltype'], bkdict['emaddr'])
        if (bulkrel != True):
            print 'Error starting bulk relaxation in ', bulkstartdir
            return None
    bulkrel = Vgetmon.main(bulkstartdir) #this gives back a STRING   
    if (bulkrel == None) or ('True' not in bulkrel):
        print 'Error with bulk relaxation in ', bulkstartdir
        return None
    bulkfindir=""
    bulkfindir = Vlastspecdir.main(bulkstartdir, '_rel')
    if (bulkfindir == None):
        print 'Error: no bulk relaxation finish.'
        return None
    print '---> Bulk relaxed in', bulkstartdir
#   make and relax endpoint 1 and run static
    numatomsforeps=0
    numatomsforeps = int(numatomsbulktot) - 1
    reltoep1=""
    ep1dir=""
    makeep1=""
    relep1=""
    runep1=""
#   #   make endpoint folder
    ep1dir = bulkfindir + '_ep1'
    if (os.path.exists(ep1dir)):
        pass #ep 1 directory has already been created
    else:
        reltoep1 = Vrelax.main(bulkfindir, topdir, os.path.basename(ep1dir))
        makeep1 = Vmakeendpt.main(bulkfindir, ep1dir, bkdict['vac1line'],
                0, bkdict['nelectadj'], bkdict['vaspvers'])
        if (makeep1 == None):
            print 'Error modifying ep1 files.'
            return None
    print '---> Endpoint 1 start in: ', ep1dir
#   loop for endpoint 1
    ep1mon=""
    ep1mon = Vgetmon.main(ep1dir)
    if (ep1mon == None) or ('True' not in ep1mon):
        runep1= Pdirautorelaxwait.main(ep1dir,
                numatomsforeps,
                bkdict['maxout'], bkdict['checksec'], bkdict['lagsec'],
                bkdict['encomp'], bkdict['entype'], bkdict['volcomp'],
                bkdict['voltype'], bkdict['emaddr'])
    ep1mon = Vgetmon.main(ep1dir)
    if (ep1mon == None) or ('True' not in ep1mon):
        print 'Error in ep1 relaxation for', ep1dir
        return None
    print '---> Endpoint 1', ep1dir, 'relaxed.'
    ep1findir=""
    ep1findir = Vlastspecdir.main(ep1dir, '_rel')
    if (ep1findir == None):
        print 'Error: no ep1 relaxation finish.'
        return None
    print '---> Finished ep1 relaxation: ', ep1findir
#   static endpoint 1
    reltostat1=""
    stat1dir=""
    modstat1=""
    runstat1=""
    stat1dir = ep1findir + '_static'
    if (os.path.exists(stat1dir)):
        pass #static ep1 directory has already been created
    else:
        reltostat1 = Vrelax.main(ep1findir, topdir, 
                        os.path.basename(stat1dir))
        modstat1 = Vmodstatic.main(stat1dir)
        if (modstat1 == None):
            print 'Error modifying static ep1 files.'
            return None
    print '---> Static ep1 start in: ', stat1dir
#   run static
    #pdb.set_trace()
    if Vgetstatus.main(stat1dir) == None:
        runstat1 = Prunonewait.main(stat1dir, bkdict['checksec'],
                bkdict['lagsec'], bkdict['emaddr'], 0)
        if (runstat1 == None):
            print 'Error running', stat1dir
            return None
    if Vgetstatus.main(stat1dir) == None:
        print 'Error in static ep1 run in: ', stat1dir
        return None
    else:
        print '---> Static 1 endpoint finished in: ', stat1dir
#   #   make a directory for each of the other endpoints and copy relaxed bulk and ep1 over
    maxct=0
    maxct = len(vac2mlist) + 2  #start at ep2, etc.
    epct=2
    idxct=0
    newname=""
    newpath=""
    niceme=""
    curdir=""
    newstart=""
    vac2num=""
    modme=""
    curdir = os.getcwd()
    while (epct < maxct):
        vac2num = str(vac2mlist[idxct])
        newname = 'jump1to' + str(epct) + 'line' + vac2num
        newpath = topdir + '/' + newname
        try:
            os.mkdir(newpath)
        except (OSError):
            print newpath, 'already exists.'
            print '(Exiting.)'
            return None
        newstart = newpath + '/' + os.path.basename(bulkstartdir)
        niceme = os.nice(19)
        shutil.copytree(bulkstartdir, newstart)
        shutil.copytree(bulkfindir, newpath + '/' + os.path.basename(bulkfindir))
        shutil.copytree(ep1dir, newpath + '/' + os.path.basename(ep1dir))
        shutil.copytree(ep1findir, newpath + '/' + os.path.basename(ep1findir))
        shutil.copy(topdir + '/inputfile', newpath + '/inputfile')
        modme = Pmodfilenew.main(newpath, 'inputfile', 'bulkstartdir', 
                        "X", "R", 'bulkstartdir = ' + newstart)
        if (modme == None):
            print 'Error modifying bulkstartdir in inputfile in', newpath
            print '(Exiting.)'
            return None
        modme = Pmodfilenew.main(newpath, 'inputfile', 'nebtopdir', 
                        "X", "R", 'nebtopdir = ' + newpath)
        if (modme == None):
            modme = Pmodfilenew.main(newpath, 'inputfile', 0, 3, "I",
                            'nebtopdir = ' + newpath)
        if (modme == None):
            print 'Error modifying nebtopdir in inputfile in', newpath
            print '(Exiting.)'
            return None
        modme = Pmodfilenew.main(newpath, 'inputfile', 'vac2line', 
                            "X", "R", 'vac2line = ' + vac2num)
        if (modme == None):
            modme = Pmodfilenew.main(newpath, 'inputfile', 0, 3, "I",
                            'vac2line = ' + vac2num)
        if (modme == None):
            print 'Error modifying vac2line in inputfile in', newpath
            print '(Exiting.)'
            return None
        os.chdir(newpath)
        Vonenebcontrolsilent.main()
        epct = epct + 1
        idxct = idxct + 1
    os.chdir(curdir)
    return True
