#!/usr/bin/env python
##
#@package   Vlastspecdir.py
#@ingroup   vasp
#@ingroup   python
#@brief     gets last of a given string + append + number
#@details   NaCl_rel1, NaCl_rel2, NaCl_rel3, gets NaCl_rel3
#@author    Tam Mayeshiba
#@date      12/21/10 created as Xgetlastdir
#@date      1/4/2011 update purpose statement
#@date      1/12/11 add toggle to get last VASP finished directories
#@date      1/12/11 changed name to Vgetlastdir because has VASP component
#@date      1/13/11 change name to Vgetlastdirs becausse getting many
#                   directories
#@date      1/26/11 add only fully relaxed directories
#@date      3/24/11 make exact match really exact match to exact + app + num
#@date      5/18/11 fix typo and error where triple can be referenced if empty
#@date      5/18/11 if exact match, make sure only number follows append
#@date      5/18/11 copied from Vgetlastdirs
#@date      5/20/11 removing unnecessary import statements, default appendstr
###
def main(matchdir, appendstr='_rel', vaspfin=1, onlyrel=1):
    """
    Purpose: gets last relaxation directory
        inputs:  
            [$1] root directory to match
                    example: '/home/tam/collection/NaCl_ep1'
            [$2] append string to match folders - default '_rel'
            [$3] only get directories where VASP has finished running
                0 - not implemented
                1 - yes, do this (default)
            [$4] only get directories where relaxation has finished
                 (according to the monitor file)
                0 - not implemented
                1 - yes, do this (default)
        outputs:
            list of subdirectories
        returns:
            True if successful
            None if error
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Xvalidname
    import Xgetdirs
    import Vgetstatus
    import Vgetmon
#    import pdb
#   validation
#   #   validate matchdir
    matchdir = Xvaliddirectory.main(matchdir)
    if (matchdir == None):
        print '(Exiting.)'
        return None
#   #   validate appendstr
    if type(appendstr) != str:
        print appendstr, 'should be a string.'
        print '(Exiting.)'
        return None
#   #   validate vaspfin
    vaspfin = Xvalidinteger.main(vaspfin, 0, 1)
    if (vaspfin == None):
        print '(Exiting.)'
        return None
#   #   validate onlyrel
    onlyrel = Xvalidinteger.main(onlyrel, 0, 1)
    if (onlyrel == None):
        print '(Exiting.)'
        return None
#   has directory to match finished?
    monres=""
    if (onlyrel == 1):
        monres = Vgetmon.main(matchdir)
        if (monres == None):
            print 'No monitor file results for', matchdir
            print '(Exiting.)'
            return None
        if not ('True' in monres):
            print matchdir, 'did not finish relaxing according to monitor.'
            print '(Exiting.)'
            return None
#   get directories
    lookindir=""
    lookindir = os.path.dirname(matchdir)
    rawdirlist=[]
    rawdirlist = Xgetdirs.main(lookindir, 1, 1) #only gets very next set
    if (rawdirlist == None):
        print 'No subdirectories under', lookindir
        print '(Exiting.)'
        return None
    rawdirlist.sort()
#   initializations
    mydir=""
    runfin=""
    unfindirlist=[]
    findirlist=[]
    #   only finished runs
    if (vaspfin == 1):
        for mydir in rawdirlist:
            runfin = Vgetstatus.main(mydir)
            if (runfin == None):
                unfindirlist.append(mydir)
            else:
                findirlist.append(mydir)
        rawdirlist = findirlist
    #
#   remove all entries not following the pattern matchdir + appendstr + digits
    matchme=""
    leftover=""
    lenmatch=0
    matchme = matchdir + appendstr
    lenmatch = len(matchme)
    leftint=0
    greatestleft=0
    greatestmatch=""
    for mydir in rawdirlist:
        if matchme in mydir:
            leftover = mydir[lenmatch:]
            if leftover.isdigit():
                leftint = int(leftover)
                if leftint > greatestleft:
                    greatestleft = leftint
                    greatestmatch = mydir
    if (greatestmatch == ""):
        return None
    else:
        return greatestmatch
