#!/usr/bin/env python
##
#@package   Vlinetospec
#@ingroup   vasp
#@ingroup   python
#@brief     from a line in CONTCAR/POSCAR, tells what number atomic species
#@author    Tam Mayeshiba
#@date      3/22/11 for use in Voneneb 
###
def main(mydir, whichline, filetouse='POSCAR', vaspvers=5):
    """
    Purpose: from a line in CONTCAR/POSCAR, tells what number atomic species
            for example, 1, 2, or 3
        inputs:  
            [$1] directory of VASP run or files
            example: /home/tam/NaCl
            [$2] line number in CONTCAR/POSCAR
            [$3] filename to use (default 'POSCAR')
            [$4] 5 - VASP version 5 (default)
                 4 - VASP version 4 (affects which line number to use)
        outputs: 
            number indicating which number atomic species
            example: line 50 in a
            CONTCAR with 8 Ag and many more Au, with order Ag Au in CONTCAR
            would return '2' for Au
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Vnumatoms
#   Validation
#   #  directory
    mydir = Xvaliddirectory.main(mydir)
    if (mydir == None):
        print ('Exiting.')
        return None
#   # line
    whichline = Xvalidinteger.main(whichline, 1, None)
    if (whichline == None):
        print ('Exiting.')
        return None
#   # version
    vaspvers = Xvalidinteger.main(vaspvers, 4, 5)
    if (vaspvers == None):
        print ('Exiting.')
        return None
#   get atom string
    atomstring=""
    atomstring = Vnumatoms.main(mydir, filetouse)
    if (atomstring == None):
        return None
    totatoms=""
    atomstringsplit=[]
    atomstringsplit = atomstring.split(':') 
    totatoms = atomstringsplit[0]
    speciesstring = atomstringsplit[1]
    specieslist=[]
    specieslist = speciesstring.split(',')
#   get line of species
    lineofspec=0
    if (vaspvers == 5):
        lineofspec = whichline - 8
    else:
        lineofspec = whichline - 7
#   find which number of species
    numspecs=0
    numspecs = len(specieslist)
    numsofar=0
    ctspecs=0
    intspecct=0
    while (ctspecs < numspecs):
        try:
            intspecct = int(specieslist[ctspecs])
        except (ValueError, TypeError):
            return None
        numsofar = numsofar + intspecct
        if (lineofspec <= numsofar):
            return ctspecs + 1
        ctspecs = ctspecs + 1
    print 'Line number exceeded number of species.'
    return None

