#!/usr/bin/env python
##
#@package   Vmakeendpt 
#@ingroup   vasp
#@ingroup   python
#@brief     make single vacancy endpoint from a bulk directory
#@author    Tam Mayeshiba
#@date      3/29/11 - created from Voneneb
###
def main(bulkdir, epdir, vac1line, vac2line=0, nelectadj=0, vaspvers=5):
    """
    Purpose:
        inputs:  
            [$1] bulk directory 
            example: /home/tam/NaClbulk
            [$2] endpoint directory 
                This directory must have already been created, and must
                contain the bulk CONTCAR as its POTCAR, and the same
                INCAR from the bulk.
            example: /home/tam/NaClep1
            [$3] line number in POSCAR/CONTCAR to remove to create vacancy
                 in endpoint 1
                 check this value carefully - program does not error check it.
            [$4] line number in POSCAR/CONTCAR to replace with line removed
                 designated by vac1line
                 0 - ignore this parameter; making first endpoint
                 check this value carefully - program does not error check it.
            [$5] NELECT adjustment (see Vnelectadjust.py)
                 0 - default
            [$6] VASP version
                 5 - default
                 4 (the line numbering is different in POSCAR/CONTCAR)
        outputs:
            True if successful - files modified in specified directory
            None if error
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Pmodfilenew
    import Vlinetospec
    import Vnewspecstring
    import Vnelectadjust
    import Pgetline
    import Vnewmagmomstr
#   Validation
#   #   bulk directory
    bulkdir = Xvaliddirectory.main(bulkdir)
    if (bulkdir == None):
        print '(Exiting.)'
        return None
#   #   endpoint directory
    epdir = Xvaliddirectory.main(epdir)
    if (epdir == None):
        print '(Exiting.)'
        return None
#   #   vacancy 1 line
    vac1line = Xvalidinteger.main(vac1line, 1, None)
    if (vac1line == None):
        print '(Exiting.)'
        return None
#   #   vacancy 2 line
    vac2line = Xvalidinteger.main(vac2line, 0, None)
    if (vac2line == None):
        print '(Exiting.)'
        return None
#   #   NELECT adjustment
    nelectadj = Xvalidinteger.main(nelectadj, None, None)
    if (nelectadj == None):
        print '(Exiting.)'
        return None
#   #   VASP version
    vaspvers = Xvalidinteger.main(vaspvers, 4, 5)
    if (vaspvers == None):
        print' (Exiting.)'
        return None
#   modify files so they are no longer bulk files, but endpoint files
    modep=""
    epstring=""
    epstring = Pgetline.main(bulkdir, 'POSCAR', 0, 1)
    epstring = epstring.strip() #   remove carriage return
    if (epstring == None):
        print 'Error getting bulk description.'
        return None
    if (vac2line == 0):
        epstring = epstring + ' endpoint 1'
    else:
        epstring = epstring + ' endpoint 2'
#   #   #   name change
    modep = Pmodfilenew.main(epdir, 'POSCAR', 0, 1, 'R', epstring)
    if (modep == None):
        print 'Error in name change.'
        return None
#   #   #   constant volume
    modep = Pmodfilenew.main(epdir, 'INCAR', 'ISIF', 'X', 'R', 'ISIF = 2')
    if (modep == None):
        print 'Error in constant volume change.'
        return None
#   #   #   only 1 vacancy NEB is supported by this script. Figure out
#   #   #   which atomic species type it is, make new species string
    whichspec=0
    whichspec = Vlinetospec.main(bulkdir, vac1line, 'POSCAR', vaspvers)
    if (whichspec == None):
        print 'Error getting species from bulk.'
        return None
    newstringforeps = Vnewspecstring.main(bulkdir, whichspec, -1, 'POSCAR',
                        vaspvers)
    if (newstringforeps == None):
        print 'Error getting new species string.'
        return None
    if (vaspvers == 5):
        modep = Pmodfilenew.main(epdir, 'POSCAR', 0, 7, 'R', 
                    newstringforeps)
        if (modep == None):
            print 'Error replacing species string, VASP version 5.'
            return None
    elif (vaspvers == 4):
        modep = Pmodfilenew.main(epdir, 'POSCAR', 0, 6, 'R', 
                    newstringforeps)
        if (modep == None):
            print 'Error replacing species string, VASP version 4.'
            return None
    else:
        print 'Unsupported VASP version.'
        return None
#   #   create vacancy. 
#   #   #   For endpoint 1, just deleting the vacancy 1 line
    getvac1line=""
    if (vac2line == 0):
        modep = Pmodfilenew.main(epdir, 'POSCAR', 0, vac1line, 'D')
        if (modep == None):
            print 'Error creating ep1 vacancy.'
            return None
#   #   #   For endpoint 2, replace vac2line with vac1line's actual line
#   #   #   Have to do this BEFORE can delete the vac1line.
    else:
        getvac1line = Pgetline.main(bulkdir, 'CONTCAR', 0, vac1line)
        if (getvac1line == None):
            print 'Error getting vacancy 1 line.'
            return None
        getvac1line = getvac1line.rstrip()   #   remove carriage return
        modep = Pmodfilenew.main(epdir, 'POSCAR', 0, vac2line, 'R',
                getvac1line)
        if (modep == None):
            print 'Error replacing vacancy line.'
            return None
        modep = Pmodfilenew.main(epdir, 'POSCAR', 0, vac1line, 'D')
        if (modep == None):
            print 'Error removing vacancy 1 line.'
            return None
#   #   NELECT adjustment 
    nelectep=""
    if (nelectadj != 0):
        nelectep = Vnelectadjust.main(epdir, nelectadj)
        if (nelectep != True):
            print 'Error in NELECT adjustment.'
            return None
#   #   #   magnetic moment: MAGMOM
    magmomstr=""
    gotmagmom=""
#   #   #   whether ep1 or ep2, vac1line is the only one deleted, so
#   #   #   magmom should follow this and only delete the vac1 moment
    magmomstr = Vnewmagmomstr.main(bulkdir, vac1line, vaspvers)
    if (magmomstr == None):
        print 'Error retrieving magnetic moment string from bulk.'
        return None
    gotmagmom = Pgetline.main(epdir, 'INCAR', 'MAGMOM', 'X')
    if (gotmagmom == None):
        pass
    else:
        modep = Pmodfilenew.main(epdir, 'INCAR', 'MAGMOM', 'X', 'R',
            magmomstr)
        if (modep == None):
            print 'Error replacing magnetic moment string.'
            return None
    return True
