#!/usr/bin/env python
##
#@package   Vmodneb
#@ingroup   vasp
#@ingroup   python
#@brief     modify neb's INCAR (copied from a non-static endpoint) 
#           to include SPRING, etc.
#@author    Tam Mayeshiba
#@date      4/7/11 made static endpoints make unique names
#@date      4/7/11 copied from Voneneb.py, removed extra parameters
#@date      4/19/11 modified to use Vrunstatic
#@date      4/19/11 removed unnecessary import statements
#@date      4/19/11 modified to return neb path
#@date      5/18/11 copied from Vnebfromendpts
#@date      8/11/11 Tom Angsten: added LCLIMB = False insertion if climb == 0
#@date      8/12/11 TM: try to replace LCLIMB if exists. Otherwise insert.
#@date      8/12/11 TM: add this support for all other tags 
###
def main(nebpath, numim=1, spring=-5, climb=1, nelectadj=0):
    """
    Purpose:
        inputs:  
            [$1] neb directory
            example: /home/tam/bulk/NaClneb
            [$2] number of images (default 1)
            [$3] spring constant (default -5)
            [$4] climbing NEB   
                1 - yes (default; adds LCLIMB tag to INCAR)
                0 - no
            [$5] adjust NELECT for endpoints and NEB (default 0)
                    negative number to remove electrons (positive vacancy)
                    positive number to add electrons (negative vacancy???)
        outputs:
            None - if error
            True - if successfully modified
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Vnelectadjust
    import Pmodfilenew
#   Validation
#   #  directory
    nebpath = Xvaliddirectory.main(nebpath)
    if (nebpath == None):
        print '(Exiting.)'
        return None
#   #   numim
    numim = Xvalidinteger.main(numim, 1, None)
    if (numim == None):
        print '(Exiting.)'
        return None
#   #   spring
    spring = Xvalidinteger.main(spring, None, None)
    if (spring == None):
        print '(Exiting.)'
        return None
#   #   climb
    climb = Xvalidinteger.main(climb, 0, 1)
    if (climb == None):
        print '(Exiting.)'
        return None
#   #   NELECT adjustment
    nelectadj = Xvalidinteger.main(nelectadj, None, None)
    if (nelectadj == None):
        print '(Exiting.)'
        return None
#   #   get starting name
    modneb=""
    modneb = Pmodfilenew.main(nebpath, 'INCAR', 'PREC', 'X', 'R', 
                'PREC = Accurate')
    if (modneb == None):
        print 'Trying PREC insert.'
        modneb = Pmodfilenew.main(nebpath, 'INCAR', 0, 3, 'I',
                'PREC = Accurate')
        if (modneb == None):
            print 'PREC tag failed.'
            return None
    modneb = Pmodfilenew.main(nebpath, 'INCAR', 'IBRION', 'X', 'R', 
                'IBRION = 1')
    if (modneb == None):
        print 'Trying IBRION insert.'
        modneb = Pmodfilenew.main(nebpath, 'INCAR', 0, 3, 'I',
                'IBRION = 1')
        if (modneb == None):
            print 'IBRION tag failed.'
            return None
    modneb = Pmodfilenew.main(nebpath, 'INCAR', 'POTIM', 'X', 'R',
                'POTIM = 0.5')
    if (modneb == None):
        print 'Trying POTIM insert.'
        modneb = Pmodfilenew.main(nebpath, 'INCAR', 'IBRION', 'X', 'I', 
                'POTIM = 0.5')
        if (modneb == None):
            print 'POTIM tag failed.'
            return None
    modneb = Pmodfilenew.main(nebpath, 'INCAR', 'IMAGES', 'X', 'R',
                'IMAGES = ' + str(numim))
    if (modneb == None):
        print 'Trying IMAGES insert.'
        modneb = Pmodfilenew.main(nebpath, 'INCAR', 'POTIM', 'X', 'I', 
                'IMAGES = ' + str(numim))
        if (modneb == None):
            print 'IMAGES tag failed.'
            return None
    modneb = Pmodfilenew.main(nebpath, 'INCAR', 'SPRING', 'X', 'R',
                'SPRING = ' + str(spring))
    if (modneb == None):
        print 'Trying SPRING insert.'
        modneb = Pmodfilenew.main(nebpath, 'INCAR', 'IMAGES', 'X', 'I',
                'SPRING = ' + str(spring))
        if (modneb == None):
            print 'SPRING tag failed.'
            return None
    if (climb == 1):
        modneb = Pmodfilenew.main(nebpath, 'INCAR', 'LCLIMB', 'X', 'R',
                'LCLIMB = True')
        if (modneb == None):
            print 'Trying LCLIMB True insert.'
            modneb = Pmodfilenew.main(nebpath, 'INCAR', 'SPRING', 'X', 'I',
                'LCLIMB = True')
            if (modneb == None):
                print 'LCLIMB True tag failed.'
                return None
    if (climb == 0):
        modneb = Pmodfilenew.main(nebpath, 'INCAR', 'LCLIMB', 'X', 'R',
                'LCLIMB = False')
        if (modneb == None):
            print 'Trying LCLIMB False insert.'
            modneb = Pmodfilenew.main(nebpath, 'INCAR', 'SPRING', 'X', 'I',
                'LCLIMB = False')
            if (modneb == None):
                print 'LCLIMB False tag failed.'
                return None
    if (nelectadj !=0):
        Vnelectadjust.main(nebpath, nelectadj)
    return True
