#!/usr/bin/env python
##
#@package   Vnebstaticnew
#@ingroup   vasp
#@ingroup   python
#@brief     Set up all images in an neb as static. Endpoints should already
#           have been run statically. NEB should already have been run.
#@author    Tam Mayeshiba
#@date      4/19/11 created
#@date      5/18/11 copied from Vrunnebstatic. 
#                   Make the folder outside.
#                   Do not run.
#@date      7/20/12 use morgan1 script
###
def main(nebdir, newname):
    """
    Purpose: Set up all images in an neb as static. Endpoints should already
             have been run statically. NEB should already have been run.
             Endpoints should already have been run statically.
        inputs:  
            [$1] neb directory
            example: /home/tam/NaClneb
            [$2] new name for directory in same directory housing the neb
        outputs:
            None if error
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Xgetdirs
    import Vmodstatic
    import Pmodfilenew
    import Psubfile
    import shutil
    import Pmodsubs
#   Validation
#   # directory
    nebdir = Xvaliddirectory.main(nebdir)
    if (nebdir == None):
        print '(Exiting.)'
        return None
#   # newname
    if type(newname) != str:
        print newname, 'should be a string.'
        print '(Exiting.)'
        return None
#   get folders
    rawfolderlist=[]
    folderlist=[]
    rawfolderlist = Xgetdirs.main(nebdir, 1, 1)   #search 1 level deep
    if (rawfolderlist == None):
        print 'Error getting folders for', nebdir
        print '(Exiting.)'
        return None
    rawfolder=""
    rawfolderbase=""
    for rawfolder in rawfolderlist:
        rawfolderbase = os.path.basename(rawfolder)
        if (rawfolderbase.isdigit() == False):
            pass
        else:
            folderlist.append(rawfolder)
    numfolders=0
    numfolders = len(folderlist)
    if numfolders < 3:
        print nebdir, 'should contain at least 3 folders if it is an neb.'
        print '(Exiting.)'
        return None
    folderlist.sort()
#   make static folder (will make it easier to parse data later)
    statdir=""
    statdir = os.path.dirname(nebdir) + '/' + newname
    if os.path.exists(statdir):
        print 'Static directory', statdir, 'already created.'
        print '(Exiting.)'
        return None
    os.mkdir(statdir)
#   run each image statically. Copy endpoints into similar folders.
    fct=0
    folderstr=""
    folderbase=""
    newdirname=""
    maxnum=0
    maxnum = numfolders - 1
#   seed folder for images with an INCAR, KPOINTS, POTCAR, submission script.
#   CONTCAR as POSCAR will be carried over per folder.
    tempdir=""
    tempdir = statdir + '/static'
    os.mkdir(tempdir)
    shutil.copy(nebdir + '/KPOINTS', tempdir)
    shutil.copy(nebdir + '/INCAR', tempdir)
    modincar=""
    modincar = Pmodfilenew.main(tempdir, 'INCAR', 'LCLIMB', 'X', 'D')
    modincar = Pmodfilenew.main(tempdir, 'INCAR', 'IMAGES', 'X', 'D')
    modincar = Pmodfilenew.main(tempdir, 'INCAR', 'SPRING', 'X', 'D')
    shutil.copy(nebdir + '/POTCAR', tempdir)
    shutil.copy('//home/tam/tsub/morgan1.sh',tempdir)
    #subfile = Psubfile.main(nebdir)
    #if (subfile == None):
    #    print 'No submission script.'
    #    print '(Exiting.)'
    #    return None
    #else:
    #    shutil.copy(subfile, tempdir)
#   loop
    returnlist=[]
    modstat=""
    while (fct < numfolders):
        folderstr = folderlist[fct]
        if (fct == 0) or (fct == maxnum):
            folderbase = os.path.basename(folderstr)
            newdirname = statdir + '/static' + str(fct)
            shutil.copytree(folderstr, newdirname)
        else:
            newdirname = statdir + '/static' + str(fct)
            if not os.path.exists(newdirname):
                shutil.copytree(tempdir, newdirname)
            shutil.copy(folderstr + '/CONTCAR', newdirname + '/POSCAR')
            modstat = Vmodstatic.main(newdirname)
            if (modstat == None):
                print 'Error creating static run for', newdirname
                print 'Continuing.'
            else:
                print newdirname
                returnlist.append(newdirname)
            Pmodsubs.main(newdirname)
        fct = fct + 1
    os.remove(tempdir + '/KPOINTS')
    os.remove(tempdir + '/POTCAR')
    os.remove(tempdir + '/INCAR')
    os.remove(tempdir + '/morgan1.sh')
    os.rmdir(tempdir)
    return returnlist
