#!/usr/bin/env python
##
#@package   Vnelectadjust
#@ingroup   vasp
#@ingroup   python
#@brief     adjust NELECT in INCAR in order to "run as insulator"
#@author    Tam Mayeshiba
#@date      2/25/11 
###
def main(existdir, numadjust=0):
    """
    Purpose: adjust NELECT in INCAR in order to "run as insulator"
        inputs:  
            [$1] directory 
            example: /home/tam/NaCl
            [$2] number of electrons to adjust up or down (pos or negative)
        outputs:
            True if successful
            None if error
        notes:
            if you have "NELECT" commented in and out in multiple
            places in the INCAR, this routine may fail.
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Vnelect
    import Pmodfilenew
    import subprocess
#   Validation
#   #  directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #  adjustment number
    numadjust = Xvalidinteger.main(numadjust, None, None)
    if (numadjust == None):
        print '(Exiting.)'
        return None
#   find unadjusted value of NELECT
    nelectorig=0
    nelectnew=0
    nelectorig = Vnelect.main(existdir)
    if (nelectorig == None):
        print 'Error finding original NELECT number.'
        print '(Exiting.)'
        return None
    try:
        nelectnew = nelectorig + numadjust
    except (ValueError, TypeError):
        print 'Error calculating adjustment by specified amount.'
        print '(Exiting.)'
        return None
    if (nelectnew <= 0):
        print 'New NELECT value would be less than or equal to zero; not valid.'
        print '(Exiting.)'
        return None
#   does NELECT line exist in INCAR?
    getnelect=""
    nelectline=""
    curdir=""
    curdir = os.getcwd()
    os.chdir(existdir)
    getnelect = subprocess.Popen(['grep NELECT INCAR'], shell=True,
                stdout=subprocess.PIPE)
    nelectline = getnelect.communicate()[0]
    getnelect.wait()
    if (nelectline == None) or (nelectline == ''):
        Pmodfilenew.main(existdir, 'INCAR', '0', 2, 'I', 
            'NELECT = ' + str(nelectnew))
    else:
        Pmodfilenew.main(existdir, 'INCAR', 'NELECT', 'X', 'R', 
            'NELECT = ' + str(nelectnew))
    os.chdir(curdir)
    print existdir, ' old: ', nelectorig, ' new: ', nelectnew
    return True
#
def loop(topdir, numadjust=0):
    """
    Purpose: adjust NELECT in INCAR in order to "run as insulator"
        inputs:  
            [$1] top directory 
            example: /home/tam/NaClcollection
            [$2] number of electrons to adjust up or down (pos or negative)
        outputs:
            True if successful
            None if error
    """
#   Modules:
    import os
    import sys
    import Vnelectadjust
    import Xvaliddirectory
    import Xvalidinteger
    import Xgetdirs
#   Validation
#   #  directory
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   loop
    dirlist=[]
    dirlist = Xgetdirs.main(topdir, 1, 1)
    mydir=""
    noresultlist=[]
    myresult=""
    for mydir in dirlist:
        myresult = Vnelectadjust.main(mydir, numadjust)
        if (myresult != True):
            noresultlist.append(mydir + ': ' + str(myresult))
    for myresult in noresultlist:
        print myresult
    return True
