#!/usr/bin/env python
##
#@package   Vnewmagmomstr
#@ingroup   vasp
#@ingroup   python
#@brief     make new MAGMOM string for INCAR based on removal
#@author    Tam Mayeshiba
#@date      3/28/11 created (copied from Vnewspecstring) 
###
def main(mydir, whichnum, vaspvers=5, newauto=1):
    """
    Purpose: make new numbers of species string for POSCAR/CONTCAR
        inputs:  
            [$1] directory of VASP files
            example: /home/tam/NaCl
            [$2] line number in POSCAR/CONTCAR of species being skipped
            [$3] 5 - VASP version 5 (default)
                 4 - VASP version 4 (affects POSCAR line numbers)
            [$4] automatic magnetic moment to be generated if none
                 previously existed
                 1 - default
        outputs:
            None if error
            new string if successful, in format number space number etc.
            for example, '1 1 1 1 5 5 5 5 1 1 1 1 1 1 1 1 1 1'
            if no MAGMOM string is found in INCAR, conforms to VASP default of
                setting moment for each atom to 1 (or to newauto value specified.)
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Pgetline
    import Pgetseg
    import Vnumatoms
#   Validation
#   # directory
    mydir = Xvaliddirectory.main(mydir)
    if (mydir == None):
        print '(Exiting.)'
        return None
#   # species ordinal
    whichnum = Xvalidinteger.main(whichnum, 1, None)
    if (whichnum == None):
        print '(Exiting.)'
        return None
#   # version number
    vaspvers = Xvalidinteger.main(vaspvers, 4, 5)
    if (vaspvers == None):
        print '(Exiting.)'
        return None
#   #   magnetic moment default
    newauto = Xvalidinteger.main(newauto, -500, 500)
    if (newauto == None):
        print '(Exiting.)'
        return None
#   get actual species number
    specvacnum=0
    specvacindex=0
    if (vaspvers == 5):
        specvacnum = whichnum - 8
    else:
        specvacnum = whichnum - 7    #   no species identification line
#   #   validate
    if (specvacnum < 1):
        print 'Not a species line in POSCAR/CONTCAR.'
        if (specvacnum == 0):
            print 'Did you mean VASP version 4 instead of VASP version 5?'
        print '(Exiting.)'
        return None
    specvacindex = specvacnum - 1   #   so it starts at 0
#   #   find old MAGMOM string
    oldmagmom=""
    oldmagstr=""
    newmagstr=""
    strnewauto=""
    numatomstr=""
    totnumatomstr=""
    totnumatoms=0
    momlist=[]
    lenoldmag=0
    oldmagct=0
    oldmagmom = Pgetline.main(mydir, 'INCAR', 'MAGMOM')
    if (oldmagmom != None):
        oldmagstr = Pgetseg.main(oldmagmom, '=')
        momlist = oldmagstr.split()
        lenoldmag = len(momlist)
        if (specvacnum > lenoldmag):
            print 'Attempting to remove the moment of an atom that does not exist.'
            print '(Exiting.)'
            return None
        while (oldmagct < lenoldmag):
            if (oldmagct != specvacindex):
                newmagstr = newmagstr + momlist[oldmagct] + ' '
            oldmagct = oldmagct + 1
    else:
        numatomstr = Vnumatoms.main(mydir)
        if (numatomstr == None):
            print '(Exiting.)'
            return None
        totnumatomstr = numatomstr.split(':')[0]
        totnumatoms = int(totnumatomstr)
        oldmagct = 1 # bump up the index; just removing one from MAGMOM string
        strnewauto = str(newauto)
        while (oldmagct < totnumatoms):
            newmagstr = newmagstr + strnewauto + ' '
            oldmagct = oldmagct + 1
    newmagstr = 'MAGMOM = ' + newmagstr
    return newmagstr
