#!/usr/bin/env python
##
#@package   Vnewspecstring
#@ingroup   vasp
#@ingroup   python
#@brief     make new numbers of species string for POSCAR/CONTCAR
#@author    Tam Mayeshiba
#@date      3/22/11 
###
def main(mydir, whichspec, adjusthow, filename='POSCAR', vaspvers=5):
    """
    Purpose: make new numbers of species string for POSCAR/CONTCAR
        inputs:  
            [$1] directory of VASP files
            example: /home/tam/NaCl
            [$2] which species (ordinal) to adjust: 1 for first species, etc.
            [$3] by how many to adjust (e.g. 1 or -1)
            [$4] file name to edit - default POSCAR
            [$5] 5 - VASP version 5 (default)
                 4 - VASP version 4 (affects POSCAR line numbers)
        outputs:
            None if error
            new string if successful, in format number space number etc.
            for example, '8 8 23'
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Vnumatoms
#   Validation
#   # directory
    mydir = Xvaliddirectory.main(mydir)
    if (mydir == None):
        print '(Exiting.)'
        return None
#   # species ordinal
    whichspec = Xvalidinteger.main(whichspec, 1, None)
    if (whichspec == None):
        print '(Exiting.)'
        return None
#   # adjusting number
    adjusthow = Xvalidinteger.main(adjusthow, None, None)
    if (adjusthow == None):
        print '(Exiting.)'
        return None
#   # version number
    vaspvers = Xvalidinteger.main(vaspvers, 4, 5)
    if (vaspvers == None):
        print '(Exiting.)'
        return None
#   get old atom string
    atomstring=""
    atomstring = Vnumatoms.main(mydir, filename)
    if (atomstring == None):
        print 'Could not get atom string.'
        print '(Exiting.)'
        return None
    atomstringsplit=[]
    atomstringsplit = atomstring.split(':')
    speciesstring=""
    speciesstring = atomstringsplit[1]
    specieslist=[]
    specieslist = speciesstring.split(',')
    numspecs=0
    numspecs = len(specieslist)
    if (whichspec > numspecs):
        print 'Species ordinal', whichspec, 'does not exist.'
        print 'Only ', numspecs, 'species.'
        print '(Exiting.)'
        return None
    tomod = whichspec - 1 #convert ordinal into list index number
    oldnum = specieslist[tomod]
    try:
        oldnum = int(oldnum)
    except (ValueError, TypeError):
        print 'Error in the atomic species string. Could not get an integer.'
        print '(Exiting.)'
        return None
    if (adjusthow < 0):
        if (abs(adjusthow) > oldnum):
            print 'Cannot make species negative.'
            print '(Exiting.)'
            return None
        if (abs(adjusthow) == oldnum):
            print 'WARNING: a species is being lost altogether.'
            print 'Please check your POTCAR and POSCAR carefully.'
    newnum=0
    newnum = oldnum + adjusthow
    newcount=0
    newstring=""
#   #   in following loop, if newnum == 0, skip species altogether
    while (newcount < numspecs):
        if (newcount == tomod):
            if (newnum == 0):
                pass
            else:
                newstring = newstring + str(newnum) + ' '
        else:
            newstring = newstring + specieslist[newcount] + ' '
        newcount = newcount + 1
    newstring = newstring.rstrip()
    return newstring
