#!/usr/bin/env python
##
#@package   Vokayvolume
#@ingroup   vasp
#@ingroup   python
#@brief     compares volumes in two directories to see if they're close
#           enough, or if they need another relaxation
#@author    Tam Mayeshiba
#@date      12/3/10 
#@date      12/22/10 change Pgetline call to reflect new inputs
###
def main(mydir1, mydir2, divby=1, usecomp='t', usethr=0.01, useperc=10):
    """
    Purpose: compare volumes and see if they're close enough, or if they
            need another relaxation
        inputs:  
            [$1] first directory
                 example: /home/tam/starts/NaCl1
            [$2] second directory
                 example: /home/tam/relaxes/NaCl1_r1
            [$3] number of atoms - default 1 (acts as divisor)
            [$4] T - use threshhold to compare (default)
                 P - use percentage to compare
            [$4] threshhold - default 0.01 angstroms cubed
            [$5] percentage - default 10 (for 10 percent)
        outputs:
            returns:
                True - close enough
                False - need another relaxation
                None - there was an error
    """
#   Modules:
    import os
    import sys
    import Pcompseg
    import Pgetseg
    import Pgetline
    import Xvaliddirectory
    import Xvalidinteger
#
#   validation
#   #   validate directories
    mydir1 = Xvaliddirectory.main(mydir1)
    if (mydir1 == None):
        print mydir1, 'is not a valid directory.'
        print '(Exiting.)'
        return None
    mydir2 = Xvaliddirectory.main(mydir2)
    if (mydir2 == None):
        print mydir2, 'is not a valid directory.'
        print '(Exiting.)'
        return None
#   #   validate number of atoms
    divby = Xvalidinteger.main(divby, 1, None)
    if (divby == None):
        print divby, 'is not a valid number of atoms. Using 1.'
        divby = 1
#   get lines
    myline1=""
    myline2=""
    myline1 = Pgetline.main(mydir1, 'OUTCAR', 'volume', 'X', 1)
    if (myline1 == None):
        print 'Error reading OUTCAR for', mydir1
        print '(Exiting.)'
        return None
    myline2 = Pgetline.main(mydir2, 'OUTCAR', 'volume', 'X', 1)
    if (myline2 == None):
        print 'Error reading OUTCAR for', mydir2
        print '(Exiting.)'
        return None
#   get segments
    myseg1=""
    myseg2=""
    myseg1 = Pgetseg.main(myline1, ':', 0)
    myseg2 = Pgetseg.main(myline2, ':', 0)
#   compare segments
    withindiff=""
    withindiff = Pcompseg.main(myseg1, myseg2, usecomp, usethr, divby,
                    useperc)
    if (withindiff == None):
        print 'Error processing results.'
        print '(Exiting.)'
        return None
    else:
        print mydir1, ': ', myseg1,
        print mydir2, ': ', myseg2,
        print 'when divided by: ', divby
        if (usecomp in ['t', 'T']):
            print 'Is within ', usethr, '?'
        else:
            print 'Is within ', useperc, 'percent change?'
        return withindiff
