#!/usr/bin/env python
##
#@package   Vonenebcontrol
#@ingroup   vasp
#@ingroup   python
#@brief     controls running of one neb. Switches to see which step to run.
#@author    Tam Mayeshiba
#@date      5/18/11 created
#@date      5/19/11 fix Prunonewait calls
#@date      5/20/11 remove unnecessary import statements
###
def main(bulkstartdir="", nebtopdir="", templdir="", vac1line="", vac2line="",
            numim=1, spring=-5, climb=1, encomp=0.001, entype='T',
            volcomp=1, voltype='P', emaddr="", 
            checksec=900, lagsec=1800, nelectadj=0, vaspvers=5,
            maxout=5):
    """
    Purpose:
        inputs:  
            [$1] starting bulk run directory 
            example: /home/tam/bulk/NaCl
            [$2] top directory in which to create neb directory
            [$2.5] template directory for NEB (should contain a 
                    submission script optimized for the number of images)
            [$3] line number from bulk CONTCAR/POSCAR indicating starting 
                 position for vacancy (will be removed to create ep1)
            [$4] line number from bulk CONTCAR/POSCAR indicating ending 
                 position for vacancy (will be replaced by starting position
                 line to create ep2)
            [$5] number of images (default 1)
            [$6] spring constant (default -5)
            [$7] climbing NEB   
                1 - yes (default; adds LCLIMB tag to INCAR)
                0 - no
            [$8] energy comparison value (see Vokayenergy)
            [$9] energy comparison type
            [$10] volume comparison value (see Vokayvolume)
            [$11] volume comparison type
            [$12] email address for error notification
            [$13] seconds between checks - default 900; between 10 and 100000
            [$14] seconds before considered hung - default 1800
                  between 10 and 100000
            [$15] adjust NELECT for endpoints and NEB (default 0)
                    negative number to remove electrons (positive vacancy)
                    positive number to add electrons (negative vacancy???)
            [$16] 5 - VASP version 5 (default)
                  4 - VASP version 4 (affects line in POSCAR for # atoms)
            [$17] maximum number of relaxations in each step - default 5
        outputs:
            None - if error
        notes:
            having too many parameters makes this unusable with Uwrapper.py
    """
#   Modules:
    import os
    import sys
    import time
    import Xinputmatch
    import Xvalidinteger
    import Vnumatoms
    import Pdirautorelaxwait
    import Vlastspecdir
    import Vmakeendpt
    import Vgetmon
    import Vrelax
    import Vmodstatic
    import Vgetstatus
    import Prunonewait
    import Veptoneb
    import Vgetnebstatus
    import Penergynebnew
    import Vnebstaticnew
#   
    #import pdb
#   Input file
#   get cwd
    mycwd=""
    mydict={}
    catparams=""
    mycwd = os.getcwd()
    ifilename=""
    ifilename = 'inputfile'
    catparams='bulkstartdir.nebtopdir.templdir.vac1line.vac2line.'
    catparams = catparams + 'numim.spring.climb.encomp.entype.volcomp.voltype.'
    catparams = catparams + 'emaddr.checksec.lagsec.nelectadj.vaspvers.maxout'
    mydict = Xinputmatch.main(mycwd + '/' + ifilename, catparams, 1)
#   #   Xinputmatch third parameter = 1 to require all values
    if (mydict == None):
        print 'Error getting parameters from input file.'
        print '(Exiting.)'
        return None
#   Validation
    checksec = Xvalidinteger.main(checksec, 10, 180000)
    if (checksec == None):
        print 'Seconds between check not within range.'
        print '(Exiting.)'
        return None
    lagsec = Xvalidinteger.main(lagsec, 300, 180000)
    if (lagsec == None):
        print 'Seconds to count job as hung not within range.'
        print '(Exiting.)'
        return None
#   get number of atoms
    numatomstr=""
    numatomsbulktot=""
    numatomstr = Vnumatoms.main(mydict['bulkstartdir'])
    if (numatomstr == None):
        print 'Number of atoms cannot be determined.'
        print '(Exiting.)'
        return None
    numatomssplit=[]
    numatomssplit = numatomstr.split(':')
    if len(numatomssplit) < 2:
        print 'Error retrieving number of atoms in system.'
        print '(Exiting.)'
        return None
    numatomsbulktot = numatomssplit[0]
#   get path above
    relaxdir = os.path.dirname(mydict['bulkstartdir'])
#   Relax bulk
    runbulk=""
    print '---> Relax bulk in', mydict['bulkstartdir']
    bulkmon=""
    bulkmon = Vgetmon.main(mydict['bulkstartdir'])
    if (bulkmon == None) or ('True' not in bulkmon):
        runbulk = Pdirautorelaxwait.main(mydict['bulkstartdir'],
            numatomsbulktot,
            mydict['maxout'], mydict['checksec'], mydict['lagsec'],
            mydict['encomp'], mydict['entype'], mydict['volcomp'],
            mydict['voltype'], mydict['emaddr'])
    bulkmon = Vgetmon.main(mydict['bulkstartdir'])
    if (bulkmon == None) or ('True' not in bulkmon):
        print 'Error in bulk relaxation for', mydict['bulkstartdir']
        return None
    print '---> Bulk', mydict['bulkstartdir'], 'relaxed.'
    bulkfindir=""
    bulkfindir = Vlastspecdir.main(mydict['bulkstartdir'], '_rel')
    if (bulkfindir == None):
        print 'Error: no bulk relaxation finish.'
        return None
    print '---> Finished bulk relaxation: ', bulkfindir
#   ENDPOINTS
#   num of atoms
    numatomsforeps=0
    numatomsforeps = int(numatomsbulktot) - 1
#   endpoint 1
    reltoep1=""
    ep1dir=""
    makeep1=""
    relep1=""
    runep1=""
    ep1dir = bulkfindir + '_ep1'
    if (os.path.exists(ep1dir)):
        pass #ep 1 directory has already been created
    else:
        reltoep1 = Vrelax.main(bulkfindir, relaxdir, os.path.basename(ep1dir))
        makeep1 = Vmakeendpt.main(bulkfindir, ep1dir, mydict['vac1line'],
                0, mydict['nelectadj'], mydict['vaspvers'])
        if (makeep1 == None):
            print 'Error modifying ep1 files.'
            return None
    print '---> Endpoint 1 start in: ', ep1dir
#   loop for endpoint 1
    ep1mon=""
    ep1mon = Vgetmon.main(ep1dir)
    if (ep1mon == None) or ('True' not in ep1mon):
        runep1= Pdirautorelaxwait.main(ep1dir,
                numatomsforeps,
                mydict['maxout'], mydict['checksec'], mydict['lagsec'],
                mydict['encomp'], mydict['entype'], mydict['volcomp'],
                mydict['voltype'], mydict['emaddr'])
    ep1mon = Vgetmon.main(ep1dir)
    if (ep1mon == None) or ('True' not in ep1mon):
        print 'Error in ep1 relaxation for', ep1dir
        return None
    print '---> Endpoint 1', ep1dir, 'relaxed.'
    ep1findir=""
    ep1findir = Vlastspecdir.main(ep1dir, '_rel')
    if (ep1findir == None):
        print 'Error: no ep1 relaxation finish.'
        return None
    print '---> Finished ep1 relaxation: ', ep1findir
#   endpoint 2
    reltoep2=""
    ep2dir=""
    makeep2=""
    relep2=""
    runep2=""
    ep2dir = bulkfindir + '_ep2'
    if (os.path.exists(ep2dir)):
        pass #ep 2 directory has already been created
    else:
        reltoep2 = Vrelax.main(bulkfindir, relaxdir, os.path.basename(ep2dir))
        makeep2 = Vmakeendpt.main(bulkfindir, ep2dir, mydict['vac1line'],
                mydict['vac2line'], mydict['nelectadj'], mydict['vaspvers'])
        if (makeep2 == None):
            print 'Error modifying ep2 files.'
            return None
    print '---> Endpoint 2 start in: ', ep2dir
#   loop for endpoint 2
    ep2mon=""
    ep2mon = Vgetmon.main(ep2dir)
    if (ep2mon == None) or ('True' not in ep2mon):
        runep2 = Pdirautorelaxwait.main(ep2dir,
                numatomsforeps,
                mydict['maxout'], mydict['checksec'], mydict['lagsec'],
                mydict['encomp'], mydict['entype'], mydict['volcomp'],
                mydict['voltype'], mydict['emaddr'])
    ep2mon = Vgetmon.main(ep2dir)
    if (ep2mon == None) or ('True' not in ep2mon):
        print 'Error in ep2 relaxation for', ep2dir
        return None
    print '---> Endpoint 2', ep2dir, 'relaxed.'
    ep2findir=""
    ep2findir = Vlastspecdir.main(ep2dir, '_rel')
    if (ep2findir == None):
        print 'Error: no ep2 relaxation finish.'
        return None
    print '---> Finished ep2 relaxation: ', ep2findir
#   #   STATIC ENDPOINTS
#   static endpoint 1
    reltostat1=""
    stat1dir=""
    modstat1=""
    runstat1=""
    stat1dir = ep1findir + '_static'
    if (os.path.exists(stat1dir)):
        pass #static ep1 directory has already been created
    else:
        reltostat1 = Vrelax.main(ep1findir, relaxdir, 
                        os.path.basename(stat1dir))
        modstat1 = Vmodstatic.main(stat1dir)
        if (modstat1 == None):
            print 'Error modifying static ep1 files.'
            return None
    print '---> Static ep1 start in: ', stat1dir
#   run static
    #pdb.set_trace()
    if Vgetstatus.main(stat1dir) == None:
        runstat1 = Prunonewait.main(stat1dir, mydict['checksec'],
                mydict['lagsec'], mydict['emaddr'], 0)
        if (runstat1 == None):
            print 'Error running', stat1dir
            return None
    if Vgetstatus.main(stat1dir) == None:
        print 'Error in static ep1 run in: ', stat1dir
        return None
    else:
        print '---> Static 1 endpoint finished in: ', stat1dir
#   static endpoint 2
    reltostat2=""
    stat2dir=""
    modstat2=""
    runstat2=""
    stat2dir = ep2findir + '_static'
    if (os.path.exists(stat2dir)):
        pass #static ep2 directory has already been created
    else:
        reltostat2 = Vrelax.main(ep2findir, relaxdir, 
                        os.path.basename(stat2dir))
        modstat2 = Vmodstatic.main(stat2dir)
        if (modstat2 == None):
            print 'Error modifying static ep2 files.'
            return None
    print '---> Static ep2 start in: ', stat2dir
#   run static
    if Vgetstatus.main(stat2dir) == None:
        runstat2 = Prunonewait.main(stat2dir, mydict['checksec'],
                mydict['lagsec'], mydict['emaddr'], 0)
        if (runstat2 == None):
            print 'Error running', stat2dir
            return None
    if Vgetstatus.main(stat2dir) == None:
        print 'Error in static ep2 run in: ', stat2dir
        return None
    else:
        print '---> Static 2 endpoint finished in: ', stat2dir
#   NEB - nonstatic
    nebpath=""
    makeneb=""
    nebpath = mydict['nebtopdir'] + '/' + os.path.basename(bulkfindir) + '_neb'
    modneb=""
    runneb=""
    if os.path.exists(nebpath):
        pass #neb already created
    else:
        makeneb = Veptoneb.main(stat1dir, stat2dir, mydict['nebtopdir'],
            os.path.basename(nebpath), mydict['templdir'], ep1findir,
            mydict['numim'], mydict['spring'],
            mydict['climb'], mydict['nelectadj'])
        if (makeneb == None):
            print 'Error creating neb from', stat1dir, 'and', stat2dir
            return None
        else:
            print '---> NEB created in', nebpath
    if (Vgetnebstatus.main(nebpath) != True):
        runneb = Prunonewait.main(nebpath, mydict['checksec'], 
                mydict['lagsec'], mydict['emaddr'], 1)
        if (runneb == None):
            print 'Error running neb in', nebpath
            return None
    if (Vgetnebstatus.main(nebpath) != True):
        print 'NEB in', nebpath, 'failed to finish.'
        return None
    print '---> nonstatic NEB finished in: ', nebpath
#   NEB static
    nebstatpath=""
    makestatneb=""
    nebstatpath = nebpath + '_static'
    modstatneb=""
    runstatneb=""
    if os.path.exists(nebstatpath):
        pass #neb already created
    else:
        makestatneb = Vnebstaticnew.main(nebpath, 
                        os.path.basename(nebstatpath))
        if (makestatneb == None):
            print 'Error creating static tree from', nebpath, 'in', nebstatpath
            return None
        else:
            print '---> Static tree created in', nebstatpath
#   run static images
    statim=1
    stattorun=""
    runstat=""
    while (statim <= numim):
        stattorun = nebstatpath + '/static' + str(statim)
        if (Vgetstatus.main(stattorun) == None):
            runstat = Prunonewait.main(stattorun, mydict['checksec'], 
                    mydict['lagsec'], mydict['emaddr'], 0)
            if (runstat == None):
                print 'Error running static image in', stattorun
                return None
        if (Vgetstatus.main(stattorun) == None):
            print 'Static image run in', stattorun, 'failed to finish.'
            return None
        print 'Static image finished in: ', stattorun
        statim = statim + 1
    print '---> Static images finished in', nebstatpath
#   get migration barriers
    nonstatbarrier=""
    statbarrier=""
    nonstatbarrier = Penergynebnew.main(nebpath, 0.01)
    statbarrier = Penergynebnew.main(nebstatpath, 0.01)
    print 'Nonstatic image migration barrier: ', nonstatbarrier
    print 'Static image migration barrier: ', statbarrier
    return statbarrier
