#!/usr/bin/env python
##
#@package   Vonenebsymm
#@ingroup   vasp
#@ingroup   python
#@brief     do one single-vacancy neb run that is symmetric, 1 image
#           then takes staticized image, makes that endpt 2, runs
#           another neb with number of images specified
#@author    Tam Mayeshiba
#@date      4/19/11 modified off of Vonenebwithstat 
###
def main(bulkstartdir, nebtopdir, templdir, vac1line, vac2line,
            numim=1, spring=-5, climb=1, encomp=0.001, entype='T',
            volcomp=1, voltype='P', emaddr="", 
            checksec=900, lagsec=1800, nelectadj=0, vaspvers=5):
    """
    Purpose:
        inputs:  
            [$1] starting bulk run directory 
            example: /home/tam/bulk/NaCl
            [$2] top directory in which to create neb directory
            [$2.5] template directory for NEB (should contain a 
                    submission script optimized for the number of images)
            [$3] line number from bulk CONTCAR/POSCAR indicating starting 
                 position for vacancy (will be removed to create ep1)
            [$4] line number from bulk CONTCAR/POSCAR indicating ending 
                 position for vacancy (will be replaced by starting position
                 line to create ep2)
            [$5] number of images (default 1)
            [$6] spring constant (default -5)
            [$7] climbing NEB   
                1 - yes (default; adds LCLIMB tag to INCAR)
                0 - no
            [$8] energy comparison value (see Vokayenergy)
            [$9] energy comparison type
            [$10] volume comparison value (see Vokayvolume)
            [$11] volume comparison type
            [$12] email address for error notification
            [$13] seconds between checks - default 900; between 10 and 100000
            [$14] seconds before considered hung - default 1800
                  between 10 and 100000
            [$15] adjust NELECT for endpoints and NEB (default 0)
                    negative number to remove electrons (positive vacancy)
                    positive number to add electrons (negative vacancy???)
            [$16] 5 - VASP version 5 (default)
                  4 - VASP version 4 (affects line in POSCAR for # atoms)
        outputs:
            None - if error
        notes:
            having too many parameters makes this unusable with Uwrapper.py
    """
#   Modules:
    import os
    import sys
    import subprocess
    import Xvaliddirectory
    import Xvalidinteger
    import Xvalidletter
    import Vnumatoms
    import Vnelectadjust
    import Pdirautorelaxwait
    import Vgetlastdirs
    import Vrelax
    import Vstillupdating
    import Pmodfilenew
    import Pgetline
    import Vlinetospec
    import Vnewspecstring
    import Vnebmake
    import Prunone
    import Vgetmon
    import time
    import Pmultkpts
    import shutil
#    import pdb
    import Vnewmagmomstr
    import Vmakeendpt
    import Pfindinlist
    import Vnebfromendpts
    import Vrunnebstatic
    import Penergynebnew
    import Xuniquename
    import Vonenebwithstat
    import Psubfile
#   Validation
#   #  directories
    bulkstartdir = Xvaliddirectory.main(bulkstartdir)
    if (bulkstartdir == None):
        print '(Exiting.)'
        return None
    nebtopdir = Xvaliddirectory.main(nebtopdir)
    if (nebtopdir == None):
        print '(Exiting.)'
        return None
    templdir = Xvaliddirectory.main(templdir)
    if (templdir == None):
        print '(Exiting.)'
        return None
#   #   vacancy line numbers
    vac1line = Xvalidinteger.main(vac1line, 1, None)
    if (vac1line == None): 
        print '(Exiting.)'
        return None
    vac2line = Xvalidinteger.main(vac2line, 1, None)
    if (vac2line == None):
        print '(Exiting.)'
        return None
#   #   numim
    numim = Xvalidinteger.main(numim, 1, None)
    if (numim == None):
        print '(Exiting.)'
        return None
#   #   spring
    spring = Xvalidinteger.main(spring, None, None)
    if (spring == None):
        print '(Exiting.)'
        return None
#   #   climb
    climb = Xvalidinteger.main(climb, 0, 1)
    if (climb == None):
        print '(Exiting.)'
        return None
#   #   energy comparison
    encompflt=0
    try:
        encompflt = float(encomp)
    except (ValueError, TypeError):
        print '(Exiting.)'
        return None
#   #   energy comparison type
    entype = Xvalidletter.main(entype, ['t','p'], 1)
    if (entype == None):
        print '(Exiting.)'
        return None
#   #   volume comparison
    volcompflt=0
    try:
        volcompflt = float(volcomp)
    except (ValueError, TypeError):
        print '(Exiting.)'
        return None
#   #   volume comparison type
    voltype = Xvalidletter.main(voltype, ['t','p'], 1)
    if (voltype == None):
        print '(Exiting.)'
        return None
#   #   email address
    if type(emaddr) != str:
        print 'Email address is not a string.'
        print '(Exiting.)'
        return None
#   #   checksec
    checksec = Xvalidinteger.main(checksec, 10, 100000)
    if (checksec == None):
        print '(Exiting.)'
        return None
#   #   lagsec
    lagsec = Xvalidinteger.main(lagsec, 10, 100000)
    if (lagsec == None):
        print '(Exiting.)'
        return None
#   #   NELECT adjustment
    nelectadj = Xvalidinteger.main(nelectadj, None, None)
    if (nelectadj == None):
        print '(Exiting.)'
        return None
#   #   VASP version
    vaspvers = Xvalidinteger.main(vaspvers, 4, 5)
    if (vaspvers == None):
        print '(Exiting.)'
        return None
#   get number of atomic species and number of each type
    oneimageneb=""
    oneimageneb = Vonenebwithstat.main(bulkstartdir, nebtopdir, templdir,
                    vac1line, vac2line, 1, spring, climb, encomp,
                    entype, volcomp, voltype, emaddr,
                    checksec, lagsec, nelectadj, vaspvers)
    if (oneimageneb == None):
        print 'Error in one neb with stat for', bulkstartdir
        print '(Exiting.)'
        return None
    oldnebpath=""
    newnebpath=""
    staticpath=""
    ep1path=""
    nebbase=""
    oneimsplit=[]
    oneimnebpath=""
    oneimsplit = oneimageneb.split(':')
    oneimnebpath = oneimsplit[0]
#   #   hardcoding the paths based on Vonenebwithstat:
#   #   static image to use as ep2 should be oneimageneb + '/static1'
#   #   neb files should be two folders up from that
#   #   endpoint 1 should be in the neb folder/00
    oldnebpath = os.path.dirname(oneimnebpath)
    staticpath = oneimnebpath + '/static1'
    ep1path = oldnebpath + '/00'
    ep1path = Xvaliddirectory.main(ep1path)
    if (ep1path == None):
        return None
    staticpath = Xvaliddirectory.main(staticpath)
    if (staticpath == None):
        return None
    nebbase = os.path.basename(oldnebpath)
    nebprev = os.path.dirname(oldnebpath)
    nebstatuniq = Xuniquename.main(nebprev, nebbase + '_symm', 'd')
    if (nebstatuniq == None):
        print 'Cannot create unique symmetric neb name.'
        print '(Exiting.)'
        return None
    nebpathsymm=""
    nebpathsymm = nebprev + '/' + nebstatuniq
#   make neb from endpoints. Do this directly with convasp since we're
#   missing too many files to use Vnebmake
    os.mkdir(nebpathsymm)
    os.chdir(nebpathsymm)
    intpolstr=""
    intpolstr = 'nice -n 19 convasp -intpol '
    intpolstr = intpolstr + ep1path + '/POSCAR' + ' '
    intpolstr = intpolstr + staticpath + '/CONTCAR' + ' '
    intpolstr = intpolstr + str(numim) + ' n'
    subintpol=""
    subintpol = subprocess.Popen(intpolstr, shell=True)
    subintpol.wait()
#   copy essential files from previous neb run.
    shutil.copy(oldnebpath +'/KPOINTS', nebpathsymm)
    shutil.copy(oldnebpath +'/INCAR', nebpathsymm)
    shutil.copy(oldnebpath +'/POTCAR', nebpathsymm)
    oldsubfile=""
    oldsubfile = Psubfile.main(oldnebpath)
    if (oldsubfile == None):
        print 'submission file not found.'
        print '(Exiting.)'
        return None
    shutil.copy(oldsubfile, nebpathsymm)
    shutil.copy(oldnebpath + '/00/OSZICAR', nebpathsymm + '/00')
    if (numim >= 9):
        shutil.copy(staticpath + '/OSZICAR', 
            nebpathsymm + '/' + str(numim + 1))
    else:
        shutil.copy(staticpath + '/OSZICAR',
            nebpathsymm + '/0' + str(numim + 1))
#   readjust images
    adjustim=""
    adjustim = Pmodfilenew.main(nebpathsymm, 'INCAR', 'IMAGES', 'X', 'R',
                            'IMAGES = ' + str(numim))
    if (adjustim == None):
        print '(Exiting.)'
        return None
    runsymmneb=""
    runsymmneb = Prunone.main(nebpathsymm, 0, checksec, lagsec, emaddr, 1)
    if (runsymmneb == None):
        print 'Error running symmetric neb.'
        print '(Exiting.)'
        return None
    symmstatlist=""
    symmstatlist = Vrunnebstatic.main(nebpathsymm, checksec, lagsec, emaddr)
    if (symmstatlist == None):
        print 'Error running finished symmetric neb as static.'
        print '(Exiting.)'
        return None
    if len(symmstatlist) == 0:
        print 'No static images run.'
        print '(Exiting.)'
        return None
    statpath=""
    statpath = os.path.dirname(symmstatlist[0])
    myenergy=""
    myenergy = Penergynebnew.main(statpath, encomp)
    return statpath + ': ' + myenergy
