#!/usr/bin/env python
#######################################
# PLS prediction for single directory
# Tam Mayeshiba
# 2015-03-31
#######################################
import os
import Vvfe
import Xvaliddirectory
import Pdist1
import numpy as np
import pdb
def main(mydir):
    """PLS prediction for a single directory.
        Args:
            mydir <str>: Directory
        Return:
            PLS prediction
    """
    #pdb.set_trace()
    mydir=Xvaliddirectory.main(mydir)
    perfdir=os.path.join(mydir,"perf_stat")
    defdirq0=os.path.join(mydir,"defect_vac1q0_q=p0_stat")
    defdirq2=os.path.join(mydir,"defect_vac1q2_q=p2_stat")
    retlist=list()
    for chg in ["0","2"]:
        defdir=os.path.join(mydir,"defect_vac1q%s_q=p%s_stat" % (chg, chg))
        
        ######VFE
        vfe=Vvfe.main(perfdir, defdir, 1173, 0.1, 1, int(chg))

        
        ######Geometric o20x to b9x
        o20=get_coord_match(defdir, "0.25 0.25 0.5", 0.1, "CONTCAR", False)
        if o20 == None:
            raise ValueError("o20 is None")
        b9=get_coord_match(defdir, "0.25 0.25 0.25", 0.1, "CONTCAR", False)
        if b9 == None:
            raise ValueError("b9 is None")
        ##structure index files may have scrambled around positions
        #o20=get_coord(defdir, 20, "", "CONTCAR", True)
        #b9=get_coord(defdir, 9, "", "CONTCAR", True)
        o20minb9x=o20[0]-b9[0]
        
        ######Geometric distance o29 to ideal (fracpath)
        o29=get_coord_match(defdir, "0.75 0.25 0.00", 0.1, "CONTCAR", False)
        if o29 == None:
            raise ValueError("o29 is None")
        ideal=np.array([0.5, 0.25, 0.25], "float")
        o29toidealdist=np.linalg.norm(ideal-o29)

        #oxygen p-band width?
        #oxygen p-band center?
        #magnetic moment per b-site cation?

        #PLS1 -16.59 * o20minb9x + 22.94 * o29toidealdist + -6.67
        pls1 = (-16.59*o20minb9x) + (22.94*o29toidealdist) - 6.67

        #PLS2 0.15 * vfe at 1173K, 1 atm pO2 + -8.77 * o20minb9x + 0.16
        pls2 = (0.15*vfe) + (-8.77*o20minb9x) + 0.16

        #PLS3 0.29 * opbandwidth + -9.93 * o20minb9x + -0.50
        
        #PLS4 0.23 * vfe at 1173K, 1 atm pO2 + -0.01 * magmomperbsite + 0.32
        
        #PLS5 -0.25 * opbandcenter + -10.26 * o20minb9x + -0.19

        vfestr="vfe1173K1atmpO2:%3.3f" % vfe 
        o20minb9xstr="o20minb9x:%3.3f" % o20minb9x
        o29toidealdiststr="o29toidealdist:%3.3f" % o29toidealdist
        pls1str="pls1:%3.3f" % pls1
        pls2str="pls2:%3.3f" % pls2
        retstr="%s:%s:%s:%s:%s:%s:%s" % (mydir, chg, vfestr, o20minb9xstr, o29toidealdiststr, pls1str, pls2str)
        retlist.append(retstr)
        print retstr
    return retlist

def get_coord_match(topdir, matchstr="", tol="", fname='CONTCAR', matchmult=False):
    """
    Purpose: Get coordinate from a vector match, using fractional coordinates ONLY
        inputs:  
            [$1] directory
            [$2] match string to search for
            [$3] tolerance
            [$4] file name to use (default 'CONTCAR')
            [$5] match multiple (default False; only accept single match)
        outputs:
            coordinate vector
    """
#   Modules:
    import os
    import sys
    import pymatgen
    import Xvaliddirectory
    import Xgetdirs
    import Xvalidinteger
    import Pgetline
    import Upathcalcs
    import Xvector
    import Pfractocart
#   Validation
#   #  
    mycoord = np.array(matchstr.split(), 'float')
    mystr = pymatgen.io.vaspio.Poscar.from_file("%s/%s" % (topdir, fname)).structure
    myfind = pymatgen.util.coord_utils.find_in_coord_list_pbc(mystr.frac_coords, mycoord, tol)
    if len(myfind) == 0:
        print "NO MATCHES FOUND."
        return None
    matchcoords=list()
    
    for findidx in myfind:
        matchcoords.append(mystr.frac_coords[findidx])
    print matchcoords

    if len(myfind) > 1:
        if matchmult:
            return matchcoords
        else:
            print "MORE THAN ONE MATCH FOUND. Error out."
            print myfind
            print matchcoords
            return None
    if matchmult: 
        return matchcoords
    else:
        return matchcoords[0]

def get_coord(topdir, atomnum=0, adj="", fname='CONTCAR', usefrac=True):
    """
    Purpose: Get coordinate
        inputs:  
            [$1] directory
            [$2] atom position in POSCAR to search for
            [$3] adjustment (if over or under this value, go to other side)
                example:"xlow, xhigh, ylow, yhigh, zlow, zhigh"
                "0.25,1,0,1,0,1"
            [$4] file name to use (default 'CONTCAR')
            [$5] set to True to use fractional coordinates 
        outputs:
            coordinate vector
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xgetdirs
    import Xvalidinteger
    import Pgetline
    import Upathcalcs
    import Xvector
    import Pfractocart
#   Validation
#   #  
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
    atomn1 = Xvalidinteger.main(atom1,1,None)
    if atomn1 == None:
        return None
    cartcoords=[]
    mydist=0
    mydistvec=[]
    av1=[]
    av2=[]
    if bool(usefrac) == True:
        avstr1 = Pgetline.main(topdir, fname, 0,8+atomn1)
        #print avstr1
        av1 = Xvector.valflt(avstr1.strip().split())
        return av1
    cartcoords = Pfractocart.main(topdir, fname, adj)
    return cartcoords
