#!/usr/bin/env python
##
#@package   Vpotalign 
#@ingroup   vasp
#@ingroup   python
#@brief     Uses Brian Puchala's convasp -potalign function.
#@author    Tam Mayeshiba
#@date      
###
def main(thisdir, refdir, stubfile="", vaspvers=5):
    """
    Purpose: Uses Brian Puchala's convasp -potalign function in order
             to automatically get the farthest potential alignment
        inputs:  thisdir
            [$1] defected directory 
            example: /home/tam/NaCl/endpoint1
            [$2] reference directory (i.e. the undefected bulk)
            example: /home/tam/NaCl/bulk
            [$3] stub file with defects and locations, and number of atoms
                 close to use (see convasp -potalign) for help. Use full path.
            example: /home/tam/NaCl/potalign_template.txt
            [$4] VASP version 4 or 5
                4 - version 4
                5 - version 5 (default)
        outputs:
            potalign_input.txt
            runs convasp -potalign
    """
#   Modules:
    import os
    import sys
    import shutil
    import Xvaliddirectory
    import Xvalidinteger
    import Xgrep
    import Pgetline
    import Vnumatoms
    import Xuniquename
    import Pmodfilenew
    import Pgetseg
    import subprocess
#   Validation
#   #   thisdir
    thisdir = Xvaliddirectory.main(thisdir)
    if (thisdir == None):
        print '(Exiting.)'
        return None
#   #   refdir
    refdir = Xvaliddirectory.main(refdir)
    if (refdir == None):
        print '(Exiting.)'
        return None
#   #  vaspvers
    vaspvers = Xvalidinteger.main(vaspvers, 4, 5)
    if (vaspvers == None):
        print '(Exiting.)'
        return None
#   #   stubfile
    if not os.path.isfile(stubfile):
        print stubfile, 'could not be found.'
        print '(Exiting.)'
        return None
#   #   SECTION 1: Form the potalign_input.txt file.
#   get TITEL in POTCAR from reference directory
    mytitles=[]
    mytitles = Xgrep.main(refdir, 'POTCAR', 'TITEL')
    if (mytitles == None):
        print 'Error finding TITEL in POTCAR.'
        print '(Exiting.)'
        return None
    if len(mytitles) == 0:
        print 'No results for TITEL in POTCAR.'
        print '(Exiting.)'
        return None
    titleitem=""
    titlesplit=[]
    tlist=[]
    for titleitem in mytitles:
        titlesplit = titleitem.split() # TITEL, = , type, Name, Date
        if (titlesplit == None):
            pass
        elif len(titlesplit) == 0:
            pass
        else:
            tlist.append(titlesplit[3])
#   get electrostatic potentials from reference directory
    eleclines=[]
    fermilines=[]
    elecline=""
    elecstr=[]
    elecsplit=[]
    fermiline=""
    fermistr=[]
    fermisplit=[]
#   #   find all the 'electrostatic' lines
    eleclines = Xgrep.main(refdir, 'OUTCAR', 'electrostatic', 1)
    if (eleclines == None):
        print 'Error finding electrostatic lines in OUTCAR.'
        print '(Exiting.)'
        return None
    if len(eleclines) == 0:
        print 'No electrostatic lines in OUTCAR.'
        print '(Exiting.)'
        return None
#   #   find all the 'E-fermi' lines
    fermilines = Xgrep.main(refdir, 'OUTCAR', 'E-fermi', 1)
    if (fermilines == None):
        print 'Error finding E-fermi lines in OUTCAR.'
        print '(Exiting.)'
        return None
    if len(fermilines) == 0:
        print 'No E-fermi lines in OUTCAR.'
        print '(Exiting.)'
        return None
#   #   take the last one in each case 
    elecstr = eleclines[len(eleclines) - 1]
    elecsplit = elecstr.split(':')
    elecline = elecsplit[len(elecsplit) - 1]
    fermistr = fermilines[len(fermilines) - 1]
    fermisplit = fermistr.split(':')
    fermiline = fermisplit[len(fermisplit) - 1]
#   #   get the lines in between them and parse
    ferminum=0
    linect=0
    epotlines=[]
    potvals=[]
    myline=""
    linesplit=[]
    splitseg=""
    linect = int(elecline)
    ferminum = int(fermiline)
    while (linect < ferminum):
        myline = Pgetline.main(refdir, 'OUTCAR', '0', linect)
        if ('electrostatic') in myline:
            pass
        elif ('test charge') in myline:
            pass
        else:
            epotlines.append(myline)
        linect = linect + 1
    myline = ""
    for myline in epotlines:
        linesplit = myline.split()
        for splitseg in linesplit:
            if '-' in splitseg: #TTM+1 12/19/11 allow for -100, etc.
                potvals.append(splitseg[splitseg.find('-'):])
#   match potential values up with the number of atoms
    atomstr=""
    atomsplitnum=[]
    atomsplitspec=[]
    totnum=0
    atomitem=""
    numeach=[]
    atomstr = Vnumatoms.main(refdir, 'POSCAR')
    if (atomstr == None):
        print 'Error retrieving number of atoms from', refdir
        print '(Exiting.)'
        return None
    atomsplitnum = atomstr.split(':')
    totnum = int(atomsplitnum[0])
    if (totnum != len(potvals)):
        print 'Error: number of potentials and number of atoms do not match.'
        print '(Exiting.)'
        return None
    atomsplitspec = atomsplitnum[1].split(',')
    for atomitem in atomsplitspec:
        numeach.append(int(atomitem))
#   # get averages for each atom type
    typeavg=0
    potavgs=[]
    potitem=""
    atomct=0
    totalct=0
    numatoms=0
    for numatoms in numeach:
        typeavg = 0
        atomct = 0
        while (atomct < numatoms):
            try:
                typeavg = typeavg + float(potvals[totalct])
            except ValueError:
                print potvals[totalct], 'could not be converted to float.'
                return
            totalct = totalct + 1
            atomct = atomct + 1
        typeavg = typeavg / numatoms
        potavgs.append(typeavg)
#   #   create a text file from the stub and append the titles and averages.
    inputf=""
    inputpath=""
    openinput=""
    numavgs=0
    avgct=0
    inputf = Xuniquename.main(thisdir, 'potalign_input', 'f')
    if (inputf == None):
        print 'Error finding unique name in', thisdir
        print '(Exiting.)'
        return None
    inputpath = os.path.join(thisdir, inputf)
    shutil.copy(stubfile, inputpath)
    openinput = open(inputpath, 'ab') #append
    numavgs = len(potavgs)
    while (avgct < numavgs):
        openinput.write(tlist[avgct] + ' ' + str(potavgs[avgct]) + '\n')
        avgct = avgct + 1
    openinput.close()
#   #   if VASP version is 5, copy CONTCAR and remove species line
    contcarname=""
    modcontcar=""
    if (vaspvers == 4):
        contcarname = 'CONTCAR'
    else:
        contcarname = Xuniquename.main(thisdir, 'potalign_CONTCAR', 'f')
        if (contcarname == None):
            print 'Cannot create new name for species-list-free CONTCAR.'
            print '(Exiting.)'
            return None
        shutil.copy(os.path.join(thisdir, 'CONTCAR'),
                    os.path.join(thisdir, contcarname))
        modcontcar = Pmodfilenew.main(thisdir, contcarname, '0', 6) #species ln
        if (modcontcar == None):
            print 'Error modifying', contcarname
            print '(Exiting.)'
            return None
#   #   cd to thisdir, run potalign, and capture output into a file
    currentdir=""
    currentdir = os.getcwd()
    os.chdir(thisdir)
    outputf=""
    outstr=""
    outopen=""
    potproc=""
    runstring=""
    runstring = 'convasp -potalign ' + inputf + ' OUTCAR ' + contcarname
    outputf = Xuniquename.main(thisdir, 'potalign_output', 'f')
    if (outputf == None):
        print 'Could not make unique name for output.'
        print '(Exiting.)'
        return None
    outstr = os.path.join(thisdir, outputf)
    outopen = open(outstr, 'ab')
    potproc = subprocess.Popen(runstring, shell=True, stdout=outopen,
                                stderr=outopen)
    potproc.wait()
    outopen.close()
    os.chdir(currentdir)
#   Parse output for 'mean farthest host atom potential shift'
    farlist=[]
    farsplit=[]
    farthest=""
    farlist = Xgrep.main(thisdir, outputf, 'mean farthest host atom potential')
    if (farlist == None):
        print 'Error finding mean farthest shift. Problem in potalign.'
        print '(Exiting.)'
        return None
    if len(farlist) == 0:
        print 'No mean farthest potential shift found. Problem in potalign.'
        print '(Exiting.)'
        return None
    farsplit = farlist[0].split(':')
    farthest = farsplit[1]
    farthest = farthest.strip()
#   Get fermi energy from fermistr, which is actually the valence band max.
    fermien=""
    fermien = Pgetseg.main(fermistr, 'E-fermi :', 'XC')
    returnstr=""
    returnstr = 'Mean farthest shift: ' + farthest + ' E_VBM: ' + fermien
    return returnstr

