#!/usr/bin/env python
##
#@package   Vpotcardirs
#@ingroup   vasp
#@ingroup   python
#@brief     from folder full of potcars, make new runs with template
#           files for KPOINTS, POSCAR, INCAR, and submission script
#@details   (python version of Tpotcardirs)
#@author    Tam Mayeshiba
#@date      12/28/10 
#@date      5/20/11 create ability to copy and modify inputfile from
#                   template directory
#@date      5/25/11 house each bulk inside its own directory
###
def main(potcardir, templdir, newdir):
    """
    Purpose: from folder full of potcars, make new runs with template
             files for KPOINTS, POSCAR, INCAR, and submission script
        inputs:  
            [$1] directory full of POTCAR files
            example: /home/tam/A3B3potcars
            [$2] directory of template files
            example: /home/tam/A3B3template
            [$3] new directory in which to create directories
            example: /home/tam/mynewpotcars
        outputs:
            makes new folders where indicated and returns True
            None if error
    """
#   Modules:
    import os
    import sys
    import shutil
    import Xvaliddirectory
    import Xvalidname
    import Xgetfiles
    import Pmodfilenew
#   Validation
#   # validate POTCAR directory
    potcardir = Xvaliddirectory.main(potcardir)
    if (potcardir == None):
        print '(Exiting.)'
        return None
#   # validate template directory
    templdir = Xvaliddirectory.main(templdir)
    if (templdir == None):
        print '(Exiting.)'
        return None
#   # validate newdirectory
    newname = os.path.basename(newdir)
    newname = Xvalidname.main(newname)
    if (newname == None):
        print '(Exiting.)'
        return None
    if os.path.exists(newdir):
        print 'Directory already exists.'
        return None
#   Template directory check for necessary files
    templfiles=[]
    templf=""
    foundscript=False
    templfiles = Xgetfiles.main(templdir)
    print 'Files to be copied over:'
    for templf in templfiles:
        print templf
        if '.sh' in templf:
            foundscript = True
    if (templdir + '/KPOINTS') not in templfiles:
        print 'Warning: KPOINTS missing from template directory.'
    if (templdir + '/INCAR') not in templfiles:
        print 'Warning: INCAR missing from template directory.'
    if (templdir + '/POSCAR') not in templfiles:
        print 'Warning: POSCAR missing from template directory.'
    if foundscript == False:
        print 'Warning: submission script missing from template directory.'
#   make new directory and directories in directories
    os.mkdir(newdir)
    newdir = Xvaliddirectory.main(newdir) #properly format this
    potcarlist=[]
    potcarfile=""
    potcarname=""
    onedir=""
    bulkdir=""
    jobstr=""
    scriptname=""
    potcarlist = Xgetfiles.main(potcardir)
    #for potcarfile in potcarlist:
    #    print potcarfile
    print 'Making POTCAR-derived directories:'
    for potcarfile in potcarlist:
        potcarname = os.path.basename(potcarfile)
        os.nice(19)
        onedir = newdir + '/' + potcarname + '_relaxes'
        bulkdir = onedir + '/' + potcarname
        try:
            os.mkdir(onedir)         # make directory
        except OSError:
            print onedir, 'already exists or could not be created.'
            continue
        os.mkdir(bulkdir)
        shutil.copy(potcarfile, bulkdir + '/POTCAR')    # copy POTCAR
        jobstr = '#PBS -N ' + potcarname
        for templf in templfiles:
            if (os.path.basename(templf) == 'inputfile'):
                shutil.copy(templf, onedir)
                Pmodfilenew.main(onedir, 'inputfile', 'bulkstartdir',
                        "X", "R", 'bulkstartdir = ' + bulkdir)
            else:
                shutil.copy(templf, bulkdir)      # copy other files
                if '.sh' in templf:
                    scriptname = os.path.basename(templf)
                    Pmodfilenew.main(bulkdir, scriptname, 'PBS -N', 
                                    "X", "R", jobstr)
    print 'Done.'
    return True
#
#
def prompt():
    dirlist=[]
    potcardir = raw_input('Directory containing POTCAR files: ')
    templdir = raw_input('Directory containing template files: ')
    newdir = raw_input('Directory to create: ')
    dirlist.append(potcardir)
    dirlist.append(templdir)
    dirlist.append(newdir)
    return dirlist
