#!/usr/bin/env python
##
#@package   Vpotcarmake
#@ingroup   vasp
#@ingroup   python
#@brief     concatenates set of POTCARs
#@author    Tam Mayeshiba
#@date      5/23/11 created from Tpotcarmake
###
def main(outputdir, potcarsA="", potcarsB="", potcarsC="", potcarsD="", 
        potcarsE="", potcarsF=""):
    """
    Purpose: concatenates set of POTCAR files to make muliple-species
             POTCAR file
        inputs: 
            [$1] directory to which to output POTCAR files
            example: /home/tam/rocksalts
            [$2 through $7] directories containing POTCAR files for
            concatenation
        outputs:
            Labeled POTCAR files in the directory specified
        notes:
            Will make combinations of choose 1 from A, choose 1 from B,
            etc.
            starting POTCAR files should be named "La.POTCAR" and so on
    """
#   Modules:
    import os
    import subprocess
    import sys
    import Xvaliddirectory
    import Xgetfiles
    import Pgetseg
    #import pdb
#   Validation
#   #  
    outputdir = Xvaliddirectory.main(outputdir)
    if (outputdir == None):
        print '(Exiting.)'
        return None
#   #
    dirlist=[]
    gooddirlist=[]
    thisdir=""
    numdirs=0
    if (potcarsA != ""):
        dirlist.append(potcarsA)
    if (potcarsB != ""):
        dirlist.append(potcarsB)
    if (potcarsC != ""):
        dirlist.append(potcarsC)
    if (potcarsD != ""):
        dirlist.append(potcarsD)
    if (potcarsE != ""):
        dirlist.append(potcarsE)
    if (potcarsF != ""):
        dirlist.append(potcarsF)
    for thisdir in dirlist:
        if (Xvaliddirectory.main(thisdir) == None):
            print thisdir, 'is not a valid directory.'
        else:
            gooddirlist.append(thisdir)
    numdirs = len(gooddirlist)
    if numdirs == 0:
        print 'Directory list for POTCARs is empty.'
        print '(Exiting.)'
        return None
#   Combinations
    onepotcarlist=[]
    potcarlist=[]
    dirct=0
    thatdir=""
    onepotcar=""
    while (dirct < numdirs):
        thatdir = gooddirlist[dirct]
        onepotcarlist = Xgetfiles.main(thatdir, 1, 1)
        if (onepotcarlist == None):
            print 'Error getting files from', thatdir
            print '(Exiting.)'
            return None
        if len(onepotcarlist) == 0:
            print 'No files in', thatdir
        potcarlist.append(onepotcarlist)
        dirct = dirct + 1
#   put POTCARs together
    potcar1=""
    potcar2=""
    potcar3=""
    potcar4=""
    potcar5=""
    potcar6=""
    base1=""
    base2=""
    base3=""
    base4=""
    base5=""
    base6=""
    onepotcarname=""
    onepotcarsplit=[]
    onepotcarlabel=""
    onestring=""
    catlist=[]
    basename=""
    namestr=""
    namelist=[]
    #pdb.set_trace()
    for potcar1 in potcarlist[0]:
        base1 = Pgetseg.main(os.path.basename(potcar1), 1, '.POTCAR')
        if numdirs < 2:
            onestring = potcar1
            namestr = base1
            catlist.append(onestring)
            namelist.append(namestr)
            continue
        for potcar2 in potcarlist[1]:
            base2 = Pgetseg.main(os.path.basename(potcar2), 1, '.POTCAR')
            if numdirs < 3:
                onestring = potcar1 + ' ' + potcar2
                namestr = base1 + base2
                catlist.append(onestring)
                namelist.append(namestr)
                continue
            for potcar3 in potcarlist[2]:
                base3 = Pgetseg.main(os.path.basename(potcar3), 1, '.POTCAR')
                if numdirs < 4:
                    onestring = potcar1 + ' ' + potcar2 + ' ' + potcar3
                    namestr = base1 + base2 + base3
                    catlist.append(onestring)
                    namelist.append(namestr)
                    continue
                for potcar4 in potcarlist[3]:
                    base4 = Pgetseg.main(os.path.basename(potcar4), 
                                1, '.POTCAR')
                    if numdirs < 5:
                        onestring = potcar1 + ' ' + potcar2 + ' ' + potcar3
                        onestring = onestring + ' ' + potcar4
                        namestr = base1 + base2 + base3 + base4
                        catlist.append(onestring)
                        namelist.append(namestr)
                        continue
                    for potcar5 in potcarlist[4]:
                        base5 = Pgetseg.main(os.path.basename(potcar5), 
                            1, '.POTCAR')
                        if numdirs < 6:
                            onestring = potcar1 + ' ' + potcar2 + ' ' + potcar3
                            onestring = onestring + ' ' + potcar4 + ' '
                            onestring = onestring + potcar5
                            namestr = base1 + base2 + base3 + base4 + base5
                            catlist.append(onestring)
                            namelist.append(namestr)
                            continue
                        for potcar6 in potcarlist[6]:
                            base6 = os.path.basename(potcar6)
                            base6 = Pgetseg.main(base6, 1, '.POTCAR')
                            onestring = potcar1 + ' ' + potcar2 + ' ' + potcar3
                            onestring = onestring + ' ' + potcar4 + ' '
                            onestring = onestring + potcar5 + ' ' + potcar6
                            namestr = base1 + base2 + base3 + base4 + base5
                            namestr = namestr + base6
                            catlist.append(onestring)
                            namelist.append(namestr)
#   concatenate
    catnum=0
    catct=0
    curdir=""
    subme=""
    substr=""
    catnum = len(catlist)
    if not (catnum == len(namelist)):
        print 'Name list and directory list do not match. Error in code.'
        print '(Exiting.)'
        return None
    print 'Expect', catnum, 'POTCARs to be created.'
    curdir = os.getcwd()
    os.chdir(outputdir)
    while (catct < catnum):
        substr = 'cat ' + catlist[catct] + ' >> ' + namelist[catct]
        subme = subprocess.Popen([substr], shell=True)
        catct = catct + 1
    os.chdir(curdir)
    return True
#
def prompt():
    """
        Prompter function.
    Purpose: concatenates set of POTCAR files to make muliple-species
             POTCAR file
        Will ask for: 
            [$1] directory to which to output POTCAR files
            example: /home/tam/rocksalts
            [$2 through $7] directories containing POTCAR files for
            concatenation
        outputs:
            Labeled POTCAR files in the directory specified
        notes:
            Will make combinations of choose 1 from A, choose 1 from B,
            etc.
            starting POTCAR files should be named "La.POTCAR" and so on
    """
#   Modules
    import os
    import sys
    import Vpotcarmake
#   prompt for input
    getoutputdir=""
    getdir1=""
    getdir2=""
    getdir3=""
    getdir4=""
    getdir5=""
    getdir6=""
    returnme=""
    currentdir = os.getcwd()
    print 'Concatenates POTCAR files named Xx.POTCAR.'
    getoutputdir = raw_input('Path where new POTCARs will be created: ')
    getdir1 = raw_input('Directory for first set of POTCAR files: ')
    getdir2 = raw_input('Directory for second set (Enter to skip): ')
    getdir3 = raw_input('Directory for third set (Enter to skip): ')
    getdir4 = raw_input('Directory for fourth set (Enter to skip): ')
    getdir5 = raw_input('Directory for fifth set (Enter to skip): ')
    getdir6 = raw_input('Directory for sixth set (Enter to skip): ')
    returnme = Vpotcarmake.main(getoutputdir, getdir1, getdir2, getdir3, 
                    getdir4, getdir5, getdir6)
    return returnme

