#!/usr/bin/env python
##
#@package   Vrelax
#@ingroup   vasp
#@ingroup   python
#@brief     takes information from an existing run and creates a new directory
#@details   CONTCAR as POSCAR
#@author    Tam Mayeshiba
#@date      12/3/10 
#@date      12/22/10 modifying Pmodfilenew call to reflect new inputs
#@date      2/16/11 throw error if CONTCAR is empty
#@date      6/30/11 modify jobstring mod call to work on Ranger
###
def main(existdir, relaxdir, relaxname):
    """
    Purpose: takes information from an existing run and creates a new directory
        inputs:  
            [$1] directory of the existing run
                example: '/home/tam/rocksalt/NaCl'
            [$2] directory in which to create the new run (full directory)
                example: '/home/tam/rocksalt_rel1/'
            [$3] new folder name
                example: 'NaCl_rel'
        outputs: new files in new directory
        returns:
            True - relax succeeded
            None - error
    """
#   Modules:
    import os
    import sys
    import shutil
    import glob
    import Xvaliddirectory
    import Xvalidname
    import Pmodfilenew
#   validation
#   #   validate existing directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   validate new run directory
    relaxdir = Xvaliddirectory.main(relaxdir)
    if (relaxdir == None):
        print '(Exiting.)'
        return None
#   #   validate folder name
    relaxname = Xvalidname.main(relaxname)
    if (relaxname == None):
        print '(Exiting.)'
        return None
    newdir=""
    newdir = relaxdir + '/' + relaxname
    if os.path.exists(relaxdir + '/' + relaxname):
        print relaxname, 'already exists in', relaxdir
        print '(Exiting.)'
        return None
#   make folder and copy files
    os.mkdir(newdir)
#   #   replace KPOINTS, INCAR, POTCAR, and CONTCAR as POSCAR
    essfile=""
    existf=""
    newf=""
    readnew=""
    contcarsize=0
    for essfile in ['KPOINTS', 'INCAR', 'CONTCAR', 'POTCAR']:
        existf = existdir + '/' + essfile
        if os.path.isfile(existf) == False:
            print existf, 'does not exist.'
            print '(Exiting.)'
            return None
        else:
            if essfile == 'CONTCAR':
                contcarsize = os.stat(existf).st_size
                if (contcarsize == 0):
                    print existf, 'is empty.'
                    print '(Exiting.)'
                    return None
                newf = newdir + '/' + 'POSCAR'
            else:
                newf = newdir + '/' + essfile
            niceme = os.nice(19)
            shutil.copy(existf, newf)
            readnew = open(newf, 'rb')
            if readnew.readline() == "":
                print 'Warning: ', newf, 'is empty.'
            readnew.close()
#   #   find and copy submission script
    subsearch=""
    subsearch = glob.glob(existdir + '/' + '*.sh')
    subfile=""
    subname=""
    jobstrmod=""
    if len(subsearch) < 1:
        print 'Warning: no submission script detected in ', existdir
    else:
        if len(subsearch) > 1:
            print 'Warning: more than one submission script',
            print 'detected in ', existdir
            print 'First script is being copied.'
        subname = os.path.basename(subsearch[0])
        subfile = newdir + '/' + subname 
        shutil.copy(subsearch[0], subfile)
#   #   change job name in submission scripte
        jobstring='#PBS -N ' + relaxname
        jobstrmod = Pmodfilenew.main(newdir, subname, 
                    'PBS -N', "X", "R", jobstring)
        if (jobstrmod == None):
            jobstring = '#$ -N ' + relaxname
            jobstrmod = Pmodfilenew.main(newdir, subname, '#$ -N',
                        'X', 'R', jobstring)
    print ('Done.')
    return True
