#!/usr/bin/env python
##
#@package   Vrunstatic
#@ingroup   vasp
#@ingroup   python
#@brief     copy a run folder, CONTCAR to POSCAR, and modify so it is static
#           with ISMEAR = -5, and run
#@author    Tam Mayeshiba
#@date      4/19/11
def main(reldir, append='_static_', checksec=900, lagsec=1800, emaddr="",
            askfirst=1): 
    """
    Purpose:
        inputs:  
            [$1] starting fully relaxed run
            example: /home/tam/bulk/NaCl
            [$2] append to make new folder name (will be created in directory
                 above the relaxed directory)
            [$13] seconds between checks - default 900; between 10 and 100000
            [$14] seconds before considered hung - default 1800
                  between 10 and 100000
            [$12] email address for error notification
            [$6] 1 - ask before running (default)
                 0 - don't ask before running
        outputs:
            None - if error
            static endpoint directory if run
            'not run: static endpoint directory' if completely made
            successfully but not run
    """
#   Modules:
    import os
    import sys
    import subprocess
    import Xvaliddirectory
    import Xvalidinteger
    import Xvalidletter
    import Xuniquename
    import Vrelax
    import Pmodfilenew
    import Prunone
    import Vgetmon
    import Pmultkpts
#   Validation
#   #  directories
    reldir = Xvaliddirectory.main(reldir)
    if (reldir == None):
        print '(Exiting.)'
        return None
#   #   string
    if type(append) != str:
        print append, 'should be a string.'
        print '(Exiting.)'
        return None
#   #   email address
    if type(emaddr) != str:
        print 'Email address is not a string.'
        print '(Exiting.)'
        return None
#   #   checksec
    checksec = Xvalidinteger.main(checksec, 10, 100000)
    if (checksec == None):
        print '(Exiting.)'
        return None
#   #   lagsec
    lagsec = Xvalidinteger.main(lagsec, 10, 100000)
    if (lagsec == None):
        print '(Exiting.)'
        return None
#   #   askfirst
    askfirst = Xvalidinteger.main(askfirst, 0, 1)
    if (askfirst == None):
        print '(Exiting.)'
        return None
#   get parent directory and base name for relax
    parentdir=""
    basename=""
    parentdir = os.path.dirname(reldir)
    basename = os.path.basename(reldir)
#   make unique folder for static run, with CONTCAR as POSCAR
    statuniq=""
    statuniq = Xuniquename.main(parentdir, basename + append)
    if (statuniq == None):
        print 'Could not make a unique static folder.'
        return None
    reltostat=""
    reltostat = Vrelax.main(reldir, parentdir, statuniq)
    if (reltostat == None):
        print 'Error in Vrelax for static creation.'
        return None
    statdir=""
    statdir = parentdir + '/' + statuniq
#   modify files for static run
    modstat=""
    modstat = Pmodfilenew.main(statdir, 'INCAR', 'IBRION', 'X', 'R',
                                'IBRION = -1')
    if (modstat == None):
        return None
    modstat = Pmodfilenew.main(statdir, 'INCAR', 'NSW', 'X', 'R',
                                'NSW = 0')
    if (modstat == None):
        return None
    modstat = Pmodfilenew.main(statdir, 'INCAR', 'PREC', 'X', 'R',
                                'PREC = Accurate')
    if (modstat == None):
        return None
    mynumkpts=0
    mynumkpts = Pmultkpts.main(reldir)
    if (mynumkpts == None):
        return None
    if (mynumkpts > 8):
        modstat = Pmodfilenew.main(statdir, 'INCAR', 'ISMEAR', 'X', 'R',
                                    'ISMEAR = -5')
        if (modstat == None):
            return None
#   run
    monstatic=""
    monstatic = Prunone.main(statdir, askfirst, checksec, lagsec, emaddr)
    if (monstatic == None):
        print 'Error in run in', statdir
        return None
    elif (monstatic == False):
        print 'Run for', statdir, 'refused by user.'
        print 'Directory made.'
        return 'not run:' + statdir
    else:
        return statdir
