#!/usr/bin/env python
##
#@package   Vscalehighk
#@ingroup   vasp
#@ingroup   python
#@brief     rescale low KPOINT CONTCAR to fixed volume of a higher KPOINT run
#@author    Tam Mayeshiba
#@date      3/1/11 
###
def main(lowdir, highdir, newdir, onlydone=0, ofile='CONTCAR'):
    """
    Purpose: rescale low KPOINT CONTCAR to fixed volume of a higher KPOINT run
        inputs:  
            [$1] low KPOINT completed run directory
            example:
            [$2] high KPOINT completed run directory
            [$3] directory in which to place new run folder
            [$4] only accept both finished runs
                 1 - yes 
                 0 - no, accept any CONTCAR (default)
            [$5] file name (either CONTCAR or POSCAR)
        outputs:
            None - if error
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import Vrelax
    import Vgetstatus
    import shutil
    import Vgetvolume
    import Pmodfilenew
    import Xuniquename
#   Validation
#   #  directories
    lowdir = Xvaliddirectory.main(lowdir)
    if (lowdir == None):
        print '(Exiting.)'
        return None
    highdir = Xvaliddirectory.main(highdir)
    if (highdir == None):
        print '(Exiting.)'
        return None
    newdir = Xvaliddirectory.main(newdir)
    if (newdir == None):
        print '(Exiting.)'
        return None
#   #   filename should be handled by getline functions
    if (ofile != 'CONTCAR') and (ofile != 'POSCAR'):
        print 'Warning: file not CONTCAR or POSCAR. Check output carefully.'
    if os.path.isfile(highdir + '/' + ofile) == False:
        print ofile, 'does not exist in', highdir
        print '(Exiting.)'
        return None
    if os.path.isfile(lowdir + '/' + ofile) == False:
        print ofile, 'does not exist in', lowdir
        print '(Exiting.)'
        return None
#   #   onlydone
    onlydone = Xvalidinteger.main(onlydone, 0, 1)
    if (onlydone == None):
        print '(Exiting.)'
        return None
#   If only done files, check that the runs are done.
    highdone=""
    lowdone=""
    if (onlydone == 1):
        highdone = Vgetstatus.main(highdir)
        if (highdone == None):
            print highdir, 'did not finish.'
            print '(Exiting.)'
            return None
        lowdone = Vgetstatus.main(lowdir)
        if (lowdone == None):
            print lowdir, 'did not finish.'
            print '(Exiting.)'
            return None
#   make new run directory
    relaxme=""
    newnameseg=""
    relaxpath=""
    newnameseg = os.path.basename(lowdir)
    newnameseg = Xuniquename.main(newdir, newnameseg, 'D', '_scale')
    relaxme = Vrelax.main(lowdir, newdir, newnameseg)
    if (relaxme != True):
        print 'Error relaxing.'
        print '(Exiting.)'
        return None
    relaxpath = newdir + '/' + newnameseg
#   get high kmesh volume
    highvol=""
    highvol = Vgetvolume.main(highdir)
    if (highvol == None):
        print 'Error retrieving volume.'
        print '(Exiting.)'
        return None
    highvol = highvol.lstrip()
    highvol = highvol.rstrip()
    highvolflt=0
    try:
        highvolflt = float(highvol)
    except (TypeError, ValueError):
        print 'Error retrieving volume.'
        print '(Exiting.)'
        return None
#   rescale new run's POSCAR to that volume by setting scale factor to -vol
    modPOSCAR=""
    modPOSCAR = Pmodfilenew.main(relaxpath, 'POSCAR', 0, 2, 'R', 
                                    '-' + highvol)
    if (modPOSCAR == None):
        print 'Error modifying POSCAR in', relaxpath
        print '(Exiting.)'
        return None
#   modify new run's INCAR to use constant volume
    modINCAR=""
    modINCAR = Pmodfilenew.main(relaxpath, 'INCAR', 'ISIF', 'X', 'R',
                                    'ISIF = 2')
    if (modINCAR == None):
        print 'Error modifying INCAR in', relaxpath
        print '(Exiting.)'
        return None
#   copy high KPOINTS to new run
    if (os.path.isfile(highdir + '/KPOINTS')):
        if (os.path.isfile(relaxpath + '/KPOINTS')):
            shutil.copy(relaxpath + '/KPOINTS', relaxpath + '/KPOINTS_orig')
        shutil.copy(highdir + '/KPOINTS', relaxpath)
    else:
        print 'KPOINTS not modified in new directory.'
    return True
