#!/usr/bin/env python
##
#@package   Vstillupdating
#@ingroup   vasp
#@ingroup   python
#@brief     check to see if job is hung (no updates)
#@author    Tam Mayeshiba
#@date      12/14/10 
#@date      5/19/11 specifically check OSZICAR and OUTCAR files 
#@date      5/19/11 fix to actually extract data from os.stat
#@date      5/19/11 flush sys.stdout at every check
#@date      5/20/11 change [8] piece to st_mtime for robustness
###
def main(rundir, lagsec=1800):
    """
    Purpose: check to see if updates to folder have been made within
             a specific time period
       inputs:  
            [$1] directory to check
                example: '/home/tam/NaCl1'
            [$2] number of seconds of acceptable lag between file
                updates (checks for hung jobs) - default 1800 (half an hour)
       outputs:
            True - directory was updated within the specified amount of time
            False - directory has not updated within that time period
            None - error
    """
#   Modules:
    import os
    import sys
    import time
    import Xvaliddirectory
    import Xvalidinteger
    import Xgetfiles
#   
#   validate
#   #   validate run directory
    rundir = Xvaliddirectory.main(rundir)
    if (rundir == None):
        print '(Exiting.)'
        return None
#   #   validate lagsec
    lagsec = Xvalidinteger.main(lagsec, 1, None)
    if (lagsec == None):
        print 'Seconds after which files must be updated',
        print 'could not be read.'
        print '(Exiting.)'
        return None
#   compare current time with last timestamp in folder
    lastuposz=0
    lastupoutcar=0
    current=0
    calclag=0
    osztime=0
    outcartime=0
    lasttime=0
#   #   flush stdout
    sys.stdout.flush()
#   #   get last timestamp in folder
    if os.path.isfile(rundir + '/OSZICAR'):
        lastuposz = os.stat(rundir + '/OSZICAR') # get OSZICAR timestamp
        osztime = lastuposz.st_mtime
    if os.path.isfile(rundir + '/OUTCAR'):
        lastupoutcar = os.stat(rundir + '/OUTCAR') # get OUTCAR timestamp
        outcartime = lastupoutcar.st_mtime
    if (outcartime > osztime):
        lasttime = outcartime
    else:
        lasttime = osztime
#   #   compare with current timestamp
    current = int(time.time())
    #print 'Current: ', current, ' Last: ',lasttime 
    try:
        calclag = current - lasttime
    except (ValueError, TypeError):
        print 'Unable to determine time difference.'
        print '(Exiting.)'
        return None
    #print 'Last update: ', time.ctime(lastupdate)
    if (calclag <= lagsec):
        return True
    else:
        print 'Last update: ', time.ctime(lasttime)
        print 'Has not been updated for ',calclag, 'seconds.'
        return False
