#!/usr/bin/env python
##
#@package   Vstrainenergy
#@ingroup   other
#@ingroup   python
#@brief     From epitaxial strain of cube, use strain tensor to get approximate strain energy contribution between two states
#           Poisson's ratio
#@author    Tam Mayeshiba
#@date      11/18/2012
###

def main(mydir1, mydir2, zerovol, verbose=0, isotrop=0):

    """
    Get strain energy contribution between two states
    Assumes equal epitaxial strain.
    Assumes that the unstrained lattice is approximately cubic
    INPUTS:
        mydir1 <str> = directory (bulk, or initial state)
        mydir2 <str> = directory (defected, or transition state)
        zerovol <float> = zero-strain approx. cubic volume
        verbose <int> = 1 - yes, print verbose
                        0 - no, just print end
        isotrop <int> = 1 - yes, append isotropic approximation
                        0 - no, do not append isotropic approx.
    """
    import Vgetenergy
    import Vgetvolume
    import Platticevec
    import math
    import os
    import sys
    import pdb
    import Xgetdirs
    import numpy
    import Pgetline
    #
    mylist=[]
    edenslist=[]
    poissonlist=[] #using approx of poisson's ratio approx delta z / delta x 
    strainlist=[]
    zstrainlist=[]
    ymodlist=[]
    myvol=""
    myenergy=""
    flag=0
    stress1line=""
    stress2line=""
    stress1=[]
    stress2=[]
    stress1line = Pgetline.main(mydir1, "OUTCAR", "in kB", "X", 1)
    if stress1line == None:
        print "No stress tensor in ", mydir1, ". Exiting."
        return None
    stress2line = Pgetline.main(mydir2, "OUTCAR", "in kB", "X", 1)
    if stress2line == None:
        print "No stress tensor in ", mydir2, ". Exiting."
        return None
    stress1=stress1line.split() #first entry is "in", second is "kB"
    stress2=stress2line.split()
    myvol = Vgetvolume.main(mydir1)
    dlenx=0
    dlenz=0
    xlen=0
    zlen=0
    eps=[]
    latticeparams=[]
    zerolattice=0
    latticeparams=Platticevec.main(mydir1,"CONTCAR")
    if latticeparams==None:
        print "Error getting lattice parameters for ",mydir1
        return None
    zerolattice = math.pow(float(zerovol),0.333333333333)
    xlen = float(latticeparams[0][0])
    zlen = float(latticeparams[2][2])
    dlenx = xlen-zerolattice
    dlenz = zlen-zerolattice
    eps.append(dlenx/zerolattice)
    eps.append(dlenx/zerolattice) #epitaxial strain epsx = epsy
    eps.append(dlenz/zerolattice)
    princdelsig=[]
    for idx in [2, 3, 4]:
        princdelsig.append(float(stress2[idx])-float(stress1[idx]))
    strsum=0
    for idx in [0, 1, 2]:
        strsum = strsum + princdelsig[idx]*eps[idx]
    delh=0
    delh = -1*float(myvol)*strsum
    delheV = delh/10/160.2 #GPa*A^3 = 10 kbar*A^3; eV= 160.2 GPa*A^3
    if verbose == 1:
        print "Stress 1 in kB: ", str(stress1)
        print "Stress 2 in kB: ", str(stress2)
        print "Strain a, b, c: ", str(eps)
        print "Delta H in kB*A^3: ", str(delh)
        print "Delta H in eV: ", str(delheV)
    if isotrop == 1:
        avgstress=0
        avgstrain=0
        isodelh=0
        isodelheV=0
        for idx in [0, 1, 2]:
            avgstress = avgstress + princdelsig[idx]
            avgstrain = avgstrain + eps[idx]
        avgstress = avgstress/3
        avgstrain = avgstrain/3
        isodelh = -1*float(myvol)*3*avgstress*avgstrain
        isodelheV = isodelh/10/160.2
        return mydir1 + " : Delta H in eV : " + str(delheV) + ":isotropic approx Delta H in eV : " + str(isodelheV)
    else:
        return mydir1 + " : Delta H in eV : " + str(delheV)
