#!/usr/bin/env python
##
#@package   Vtotaltime
#@ingroup   vasp
#@ingroup   python
#@brief     get total time of run (sum CPU times if no total at end)
#@author    Tam Mayeshiba
#@date      1/11/11 
###
def main(existdir, ofile='OUTCAR', vaspversion=4):
    """
    Purpose: get total time of run (sum CPU times if did not finish)
        inputs:  
            [$1] folder
            example: /home/tam/NaCl
            [$2] OUTCAR file name (default OUTCAR)
            (sometimes it's been renamed from OUTCAR)
            example: OUTCARU
            [$3] 4 - VASP version 4 (default)
                 5 - VASP version 5
        outputs:
            total time
            None if error
        notes:
            summed time for unfinished OUTCAR is APPROXIMATE and is indicated
            by an appended ':unfinished'
    """
#   Modules:
    import os
    import sys
    import subprocess
    import Xvaliddirectory
    import Xvalidinteger
    import Vgetstatus
    import Pgetseg
#   Validation
#   #  validate directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   validate file
    fullfile = existdir + '/' + ofile
    if not os.path.isfile(fullfile):
        print fullfile, 'could not be found.'
        print '(Exiting.)'
        return None
#   #   validate version
    vaspversion = Xvalidinteger.main(vaspversion, 4, 5)
    if (vaspversion == None):
        return None
#   try total time first
    totaltime=0
    totaltime = Vgetstatus.main(existdir, ofile)
    if (totaltime != None):
        return totaltime
#   else need to sum up
#   #   get number of loops
    #ctproc=""
    #ctloops=0
    #ctprocstr = 'grep LOOP ' + fullfile + ' -c'
    #ctproc = subprocess.Popen([ctprocstr], shell=True, stdout=subprocess.PIPE)
    #ctloops = ctproc.communicate()[0]
    #ctproc.wait()
    #print ctloops
#   #   get time per loop
    openoutcar=""
    openoutcar = open(fullfile, 'rb')
    readln=""
    readln = openoutcar.readline()
    timestr=""
    timefloat=0
    timesum=0
    trailingsum=0
    trailingfloat=0
    while(readln):
        if ('LOOP+' in readln):
            timefloat = 0
            if (vaspversion == 5):
                timestr = Pgetseg.main(readln, 'cpu time', ': real')
            else:
                timestr = Pgetseg.main(readln, 'CPU time', 0)
            if timestr == None:
                print 'Error finding CPU time in', readln
            else:
                try:
                    timefloat = float(timestr)
                except (TypeError, ValueError):
                    print 'Error converting time for', timestr
                    timefloat = 0
                timesum = timesum + timefloat
            trailingsum = 0    # trailing loops encapsulated by big loop
        elif ('LOOP' in readln):
            if (vaspversion == 5):
                timestr = Pgetseg.main(readln, 'cpu time', ': real')
            else:
                timestr = Pgetseg.main(readln, 'CPU time', 0)
            if timestr == None:
                print 'Error finding CPU time in', readln
            else:
                try:
                    trailingfloat = float(timestr)
                except (TypeError, ValueError):
                    print 'Error converting time for', timestr
                    trailingfloat = 0
                trailingsum = trailingsum + trailingfloat
        readln = openoutcar.readline()
    openoutcar.close()
    return str(timesum + trailingsum) + ': unfinished'
#
#
def loop(topdir, ofile='OUTCAR', vaspversion=4):
    """
    Purpose: get total time of run (sum CPU times if did not finish)
        inputs:  
            [$1] top folder
            example: /home/tam/NaClcollection
            [$2] OUTCAR file name (default OUTCAR)
            (sometimes it's been renamed from OUTCAR)
            example: OUTCARU
            [$3]4 -  VASP version 4 (default)
                5 - VASP version 5
        outputs:
            total time
            None if error
        notes:
            summed time for unfinished OUTCAR is APPROXIMATE and is indicated
            by an appended ':unfinished'
    """
#   Modules:
    import os
    import sys
    import Vtotaltime
    import Xvaliddirectory
    import Xgetdirs
#   Validation
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   other validation will be handled by Vtotaltime
#   Run for list
    dirlist=[]
    dirlist = Xgetdirs.main(topdir)
    mydir=""
    myresult=""
    resultlist=[]
    noresultlist=[]
    for mydir in dirlist:
        myresult = Vtotaltime.main(mydir, ofile, vaspversion)
        if (myresult != None):
            resultlist.append(mydir + ': ' + str(myresult))
        else:
            noresultlist.append(mydir)
    print 'Total times (seconds):'
    for myresult in resultlist:
        print myresult
    if len(noresultlist) > 0:
        print 'No result for: '
        for myresult in noresultlist:
            print myresult
    return True
