#!/usr/bin/env python
##
#@package   Vvfeloopatmt
#@ingroup   vasp
#@ingroup   python
#@brief     loop through vacancy formation energy for one oxygen vacancy in perovskite
#@author    Tam Mayeshiba
#@date      12/2/11
#@date      2/7/12 more definitely use new folder conventions
#@date      2/7/12 add results matrix at the end
###
def main(topdir, epstr='ep1', temp=1173, poxygen=0.1, insulator=0, potstub=''):
    """
    Purpose: vfe for single oxygen vacancy in perovskite
             Use this version for dir/dir/ format
        inputs:  
            [$1] directory above directories housing statics
            [$2] string to find endpoint files: default 'ep1'
            [$2] temperature (default 1173 K)
            [$3] oxygen partial pressure (default 0.1 atm)
            [$4] insulator
                0-no
                1-yes (have to do potalign)
            [$6] potstub: full path to potalign stub to use
            example:
        outputs:
    """
#   Modules:
    import os
    import sys
    import Vvfe
    import Vpotalign
    import Xvaliddirectory
    import Xvalidinteger
#   Validation
#   #  
    topdir = Xvaliddirectory.main(topdir)
    if topdir == None:
        return None
    tempflt=0
    try:
        tempflt = float(temp)
    except (ValueError, TypeError):
        print 'Temperature', temp, 'could not be converted to float.'
        return None
    poxygenflt=0
    try:
        poxygenflt = float(poxygen)
    except (ValueError, TypeError):
        print 'Oxygen partial pressure', poxygen
        print 'Could not be converted to float.'
        return None
    insulator = Xvalidinteger.main(insulator,0,1)
    if insulator == None:
        return None
#   #
    #loop through directory and get statics
    toplist=[]
    topitem=""
    toppath=""
    statlist=[]
    statitem=""
    bulkdir=""
    epdir=""
    mybasevfe=""
    mypotalign=""
    toplist = os.listdir(topdir)
    results=[]
    resultstr=""
    for topitem in toplist:
        epdir = ""
        bulkdir = ""
        statlist = []
        mybasevfe=""
        mypotalign=""
        toppath = os.path.join(topdir, topitem, topitem)
        try:
            statlist = os.listdir(toppath)
        except OSError:
            pass
        if not statlist == []:
            for statitem in statlist:
                if epstr + '_static' in statitem:
                    epdir = os.path.join(toppath, statitem)
                elif 'bulk_static' in statitem:
                    bulkdir = os.path.join(toppath, statitem)
            if (not epdir == "") and (not bulkdir == ""):
                print ""
                print toppath
                mybasevfe = Vvfe.main(bulkdir, epdir, temp, poxygen)
                if insulator == 1:
                    mypotalign = Vpotalign.main(epdir, bulkdir, potstub)
                print mybasevfe
                print mypotalign
                resultstr = toppath + '; base vfe;' + str(mybasevfe)
                resultstr = resultstr + '; potalign;' + str(mypotalign)
                results.append(resultstr)
            else:
                print statitem, 'bulk or endpoint static is missing.'
    resultstr=""
    for resultstr in results:
        print resultstr
    return results
