#!/usr/bin/env python
##
#@package   Xgetdirs
#@ingroup   vasp
#@ingroup   python
#@brief     get list of subdirectories in a directory (full)
#@author    Tam Mayeshiba
#@date      12/21/10
#@date      1/5/11 updating so maxdepth should be >= mindepth
#@date      2/3/12 adding fnmatch capability
###
def main(existdir, mindepth=1, maxdepth=1, matchme=""):
    """
    Purpose: finds directories in a directory
        inputs:  
            [$1] directory over which to loop
                    example: /home/tam/collection/
            [$2] minimum levels deep to search - default 1
            [$3] maximum levels deep to search - default 1, maximum 5
            [$4] (optional) string to pattern match in quotes, like '*/*static'
            examples:
                starting from '/home/tam/collect':
                /home/tam/collect/NaCl1/00
                /home/tam/collect/NaCl1/01
                /home/tam/collect/NaCl1/02
                /home/tam/collect/NaCl2
                /home/tam/collect/CsCl/01
                /home/tam/collect/CsCl/02
                mindepth 1, maxdepth 1 will get
                /home/tam/collect/NaCl1
                /home/tam/collect/NaCl2
                /home/tam/collect/CsCl
                mindepth 1, maxdepth 2 will get the above plus
                /home/tam/collect/NaCl1/00
                /home/tam/collect/NaCl1/01
                /home/tam/collect/NaCl1/02
                /home/tam/collect/CsCl/01
                /home/tam/collect/CsCl/02
                mindepth 2, maxdepth 2 will get only the above
                mindepth 0 will get the top directory also
        outputs:
        returns:
            list of subdirectories
            None if error
    """
#   Modules:
    import os
    import sys
    import Xvaliddirectory
    import Xvalidinteger
    import fnmatch
#   validation
#   # validate existing directory
    existdir = Xvaliddirectory.main(existdir)
    if (existdir == None):
        print '(Exiting.)'
        return None
#   #   validate level depth
    mindepth = Xvalidinteger.main(mindepth, 0, None)
    if (mindepth == None):
        print '(Exiting.)'
        return None
    maxdepth = Xvalidinteger.main(maxdepth, mindepth, 5)
    if (maxdepth == None):
        print '(Exiting.)'
        return None
#   walk and make main list
    walkme=""
    walkentry=""
    walkme = os.walk(existdir)
    bigdirlist=[]
    if walkme == ():
        print 'no folders found in', existdir
        print '(Exiting.)'
        return None
    for walkentry in walkme:
        bigdirlist.append(walkentry[0])
#   sort list and pare down according to min/max depth
    numsep=0
    minnumsep=0
    maxnumsep=0
    numsep = existdir.count('/') # find base number of forward slashes
    minnumsep = numsep + mindepth
    maxnumsep = numsep + maxdepth
    onedir=""
    onenumsep=0
    smalldirlist=[]
    for onedir in bigdirlist:
        onenumsep = onedir.count('/')
        if (onenumsep >= minnumsep) and (onenumsep <= maxnumsep):
            smalldirlist.append(onedir)
    smalldirlist.sort()

    if not matchme == "":
        matchdirlist=[]
        for mydir in smalldirlist:
            if fnmatch.fnmatch(mydir, matchme):
                matchdirlist.append(mydir)
        return matchdirlist
    else:
        return smalldirlist

