#!/usr/bin/env python
##
#@package   Xinputmatch
#@ingroup   other
#@ingroup   python
#@brief     matches inputs in a text file to parameters. Returns parameters in order.
#@author    Tam Mayeshiba
#@date      5/18/11
###
def main(ifile, paramstr="", matchall=0):
    """
    Purpose:
        inputs:  
            [$1] input file (full path)
            example: /home/tam/NaCl/inputfile
            The input file must have format of 
            parameter = parameter value
            parameter2 = parameter 2's value
            1. Lines where # is the first character aside from spaces
            will be ignored.
            2. Multiple matches of the same parameter will throw an error.
            3. Input files only work for functions that have inputs that are 
            strings, integers, floats, characters - not lists.
            [$2] list of parameter names, in a period-delimited string.
            example: checksec.lagsec.emaddr
            [$3] 0 - finds as many parameters as exist in input file (default)
                 1 - must match all of the parameters in paramstr.
                     Otherwise, throws an error.
        outputs:
            None if error
            list of parameter values from input file
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
#   Validation
#   #  ifile
    if not os.path.isfile(ifile):
        print ifile, 'is not a valid file.'
        print '(Exiting.)'
        return None
#   #  paramstr
    if (len(paramstr) == 0):
        print 'No parameters listed.'
        print '(Exiting.)'
        return None
    if type(paramstr) != str:
        print paramstr, 'should be a string.'
        print '(Exiting.)'
        return None
#   #   matchall
    matchall = Xvalidinteger.main(matchall, 0, 1)
    if (matchall == None):
        print '(Exiting.)'
        return None
#   parse paramstr
    paramlist=[]
    paramstr = paramstr.strip()
    paramlist = paramstr.split('.')
    if (len(paramlist) == 0):
        print 'No parameters in parameter list.'
        print '(Exiting.)'
        return None
#   search in ifile
    readfile=""
    readln=""
    commentsplit=[]
    readwoc=""
    readsplit=[]
    paramname=""
    paramval=""
    paramdic={}
#   
    readfile = open(ifile, 'rb')
    readln = readfile.readline()
    while (readln):
        readln = readln.strip() # remove whitespace
        if (len(readln) == 0):
            pass
        elif (readln[0] == '#'):
            pass
        else:
            commentsplit = readln.split('#')   #split off trailing comments
            readwoc = commentsplit[0].strip()  # take non-trailing comment, strip out
            readsplit = readwoc.split('=')  # split 'parameter = value'
            if len(readsplit) < 2:
                pass #  improper format: not 'parameter = value'
            else:
                paramname = readsplit[0].strip()
                paramval = readsplit[1].strip()
                if (paramname in paramlist):
                    if (paramname in paramdic):
                        print 'Multiple values for', paramname, 'detected.'
                        print '(Exiting.)'
                        return None
                    else:
                        paramdic[paramname] = paramval
        readln = readfile.readline()
    readfile.close()
    errorstr=""
    throwerror=0
    errorstr = 'Input file is missing requested parameters: '
    if (matchall == 1):
        for paramname in paramlist:
            if (paramname not in paramdic):
                throwerror = 1
                errorstr = errorstr + paramname + ' '
        if (throwerror == 1):
            print errorstr
            print '(Exiting.)'
            return None
    print paramdic
    return paramdic

