#!/usr/bin/env python
##
#@package   Xuniquename
#@ingroup   other
#@ingroup   python
#@brief     returns the next unique name for either a file or a directory,
#           depending on an optional append and an optional start count
#@author    Tam Mayeshiba
#@date      11/30/10 
###
def main(basedir, checkname, typelett='D', appendstr='', startnum=1, stopnum=20):
    """
    Purpose:
        inputs:
            [$1] base directory
                example: '/home/tam/' as the top directory
            [$2] name to consider
                example: 'perovbulks'
            [$3] D - directory (default)
                 F - file
            [$4] string to append (empty string is default)
                example: '_rel'
            [$5] positive integer at which to start incrementing (default 1)
                example: 5, if want to start at _rel5 and go to _rel6 if
                _rel5 is taken
            [$6] positive number of iterations to attempt (default 20)
                example: if startnum is 5, append is _rel, and _rel5 through 
                _rel25 are all taken, will return None
        outputs:
            returns a suitable name
            None - error occurred.
        notes:
            does not allow a file to be named the same as an existing
            directory and vice versa
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import Xvalidletter
    import Xvaliddirectory
    import Xvalidname
#   validation
#   #   base directory
    basedir = Xvaliddirectory.main(basedir, 0)
    if (basedir == None):
        return None
#   #   name to try, as string (alphanumeric plus underscores)
    checkname = Xvalidname.main(checkname)
    if (checkname == None):
        print '(Exiting.)'
        return None
#   #   type
    choicelist = ['d','f']
    typelett = Xvalidletter.main(typelett, choicelist, 1)
    if (typelett == None):
        print '(Exiting.)'
        return None
#   #   append string (alphanumeric plus underscores)
    appendstr = Xvalidname.main(appendstr)
    if (appendstr == None):
        print '(Exiting.)'
        return None
#   #   starting number
    startnum = Xvalidinteger.main(startnum, 1, None)
    if (startnum == None):
        startnum = 1
#   #   stopping number
    stopnum = Xvalidinteger.main(stopnum, 1, None)
    if (stopnum == None):
        stopnum = 20
#   increment until no file is found
    roottry=""
    fulltry=""
    roottry = basedir + '/' + checkname + appendstr
    tryct=1
    currnum=startnum
    for tryct in range(stopnum):
        fulltry = roottry + str(currnum)
        if not (os.path.exists(fulltry)):
            if (typelett == 'f'):
                if not (os.path.isfile(fulltry)):
                    fulltry = os.path.normpath(fulltry)
                    return os.path.basename(fulltry)
            elif (typelett == 'd'):
                if not (os.path.isdir(fulltry)):
                    fulltry = os.path.normpath(fulltry)
                    return os.path.basename(fulltry)
        currnum = currnum + 1
    return None
