#!/usr/bin/env python
##
#@package   Xvector
#@ingroup   other
#@ingroup   python
#@brief     Vector functions. I know numpy and pylab exist but I want this
#           code to be able to function without them.
#@author    Tam Mayeshiba
#@date    
#@notes     tags: valflt, dot, magnitude, add, cross, angle, frctocart, strtoflt
###
def valflt(vec1=[]):
    """
    Purpose: validate that vector exists and is not null, and convert values
             to floats
        inputs:  
            [$1] vector (string values will be converted to floats)
            example: ['1','0','0'] or [1,0,0]
        outputs:
            validated vector as floats
    """
#   Modules:
    import os
    import sys
#   Validation
#   #  
    if (vec1 == "") or (vec1 == None):
        print 'Vector is null.'
        print '(Exiting.)'
        return None
#   get length
    len1=0
    len1 = len(vec1)
    if len1 == 0:
        print 'Vector is empty.'
        print '(Exiting.)'
        return None
#   convert to floats
    whichct=0
    val1=0
    returnvec=[]
    while (whichct < len1):
        try:
            val1 = float(vec1[whichct])
        except (ValueError, TypeError):
            print vec1[whichct], 'could not be converted to float.'
            print '(Exiting.)'
            return None
        returnvec.append(val1)
        whichct = whichct + 1
    return returnvec
#
#    
def dot(vec1=[],vec2=[]):
    """
    Purpose: dot product
        inputs:  
            [$1] first list (string values will be converted to floats)
            example: ['1','0','0'] or [1,0,0]
            [$2] second list of same size
        outputs:
            dot product
    """
#   Modules:
    import os
    import sys
    import Xvector
#   Validation
#   #  
    vec1 = Xvector.valflt(vec1)
    if (vec1 == None):
        print 'Error in vector 1.'
        print '(Exiting.)'
        return None
    vec2 = Xvector.valflt(vec2)
    if (vec2 == None):
        print 'Error in vector 2.'
        print '(Exiting.)'
        return None
#   get lengths
    len1=0
    len2=0
    len1 = len(vec1) # valflt ensures that these vectors are not empty
    len2 = len(vec2)
    if len1 != len2:
        print 'Lengths do not match. Cannot take dot product.'
        print '(Exiting.)'
        return None
#   take dot product
    whichct=0
    dotproduct=0
    while (whichct < len1):
        dotproduct = dotproduct + (vec1[whichct] * vec2[whichct])
        whichct = whichct + 1
    return dotproduct
#
def magnitude(vec1=[]):
    """
    Purpose: magnitude of vector
        inputs:  
            [$1] vector (string values will be converted to floats)
            example: ['1','0','0'] or [1,0,0]
            [$2] second list of same size
        outputs:
            dot product
    """
#   Modules:
    import os
    import sys
    import math
    import Xvector
#   validation will be handled by dot
#   get dot product
    mydot=0
    vecmag=0
    mydot = Xvector.dot(vec1, vec1)
    if (mydot == None):
        print 'Error calculating dot product.'
        print '(Exiting.)'
        return None
    vecmag = math.sqrt(mydot)
    return vecmag
#
def add(vec1=[],vec2=[],issubtract=0):
    """
    Purpose: vector addition
        inputs:  
            [$1] first list (string values will be converted to floats)
            example: ['1','0','0'] or [1,0,0]
            [$2] second list of same size
            [$3] subtract second vector from first
                    0 - no, add (default)
                    1 - yes, subtract vec2 from vec1
        outputs:
            subtracted vector
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
    import Xvector
#   Validation
#   #  
    vec1 = Xvector.valflt(vec1)
    if (vec1 == None):
        print 'Error in vector 1.'
        print '(Exiting.)'
        return None
    vec2 = Xvector.valflt(vec2)
    if (vec2 == None):
        print 'Error in vector 2.'
        print '(Exiting.)'
        return None
    issubtract = Xvalidinteger.main(issubtract, 0, 1)
    if (issubtract == None):
        print issubtract, 'is not valid.'
        print 'Use 0 to add or 1 to subtract.'
        print '(Exiting.)'
        return None
#   get lengths
    len1=0
    len2=0
    len1 = len(vec1) # valflt ensures that these vectors are not empty
    len2 = len(vec2)
    if len1 != len2:
        print 'Lengths do not match. Cannot add or subtract vectors.'
        print '(Exiting.)'
        return None
#   add
    whichct=0
    sumvec=[]
    sumval=0
    val1=0
    val2=0
    while (whichct < len1):
        if (issubtract == 0):
            sumval = vec1[whichct] + vec2[whichct]
        else:
            sumval = vec1[whichct] - vec2[whichct]
        sumvec.append(sumval)
        whichct = whichct + 1
    return sumvec
#
def cross(vec1=[],vec2=[]):
    """
    Purpose: vector cross product. ONLY for vectors of LENGTH 3
        inputs:  
            [$1] first list (string values will be converted to floats)
            example: ['1','0','0'] or [1,0,0]
            [$2] second list of same size
        outputs:
            cross product vector
    """
#   Modules:
    import os
    import sys
    import Xvector
#   Validation
#   #  
    vec1 = Xvector.valflt(vec1)
    if (vec1 == None):
        print 'Error in vector 1.'
        print '(Exiting.)'
        return None
    vec2 = Xvector.valflt(vec2)
    if (vec2 == None):
        print 'Error in vector 2.'
        print '(Exiting.)'
        return None
    if len(vec1) != 3:
        print 'Vector 1 must have a length of 3.'
        print '(Exiting.)'
        return None
    if len(vec2) != 3:
        print 'Vector 2 must have a length of 3.'
        print '(Exiting.)'
        return None
#   cross product: 3x3 matrix of [i j k ; vec1 ; vec2]
#   call it [i, j, k; A, B, C; D, E, F]
#   cross product is [(BF-CE)i - (AF-CD)j + (AE-BD)k]
    vali=0
    valj=0 # remember this has a negative sign
    valk=0
    valA=0
    valB=0
    valC=0
    valD=0
    valE=0
    valF=0
    crossvec=[]
    valA=vec1[0]
    valB=vec1[1]
    valC=vec1[2]
    valD=vec2[0]
    valE=vec2[1]
    valF=vec2[2]
    vali = (valB * valF) - (valC * valE)
    valj = -1 * ((valA * valF) - (valC * valD))
    valk = (valA * valE) - (valB * valD)
    crossvec.append(vali)
    crossvec.append(valj)
    crossvec.append(valk)
    return crossvec
#
def angle(vec1=[],vec2=[]):
    """
    Purpose: find angle between two vectors using the dot product
        inputs:  
            [$1] first list (string values will be converted to floats)
            example: ['1','0','0'] or [1,0,0]
            [$2] second list of same size
        outputs:
            angle between vectors (in degrees)
    """
#   Modules:
    import os
    import sys
    import math
    import Xvector
#   Validation
#   #  
    vec1 = Xvector.valflt(vec1)
    if (vec1 == None):
        print 'Error in vector 1.'
        print '(Exiting.)'
        return None
    vec2 = Xvector.valflt(vec2)
    if (vec2 == None):
        print 'Error in vector 2.'
        print '(Exiting.)'
        return None
#   A dot B = magA * magB * cos(angle between)
    cosangle=0
    anglerad=0
    angledeg=0
    mag1=0
    mag2=0
    dot1and2=0
    mag1 = Xvector.magnitude(vec1)
    if (mag1 == None):
        print 'Error getting vector 1 magnitude.'
        print '(Exiting.)'
        return None
    mag2 = Xvector.magnitude(vec2)
    if (mag2 == None):
        print 'Error getting vector 2 magnitude.'
        print '(Exiting.)'
        return None
    dot1and2 = Xvector.dot(vec1, vec2)
    if (dot1and2 == None):
        print 'Error getting dot product.'
        print '(Exiting.)'
        return None
    cosangle = dot1and2 / (mag1 * mag2)
    anglerad = math.acos(cosangle)
    angledeg = 180 * anglerad / math.pi
    return angledeg
    valj=0 # remember this has a negative sign
    valk=0
    valA=0
    valB=0
    valC=0
    valD=0
    valE=0
    valF=0
    crossvec=[]
    valA=vec1[0]
    valB=vec1[1]
    valC=vec1[2]
    valD=vec2[0]
    valE=vec2[1]
    valF=vec2[2]
    vali = (valB * valF) - (valC * valE)
    valj = -1 * ((valA * valF) - (valC * valD))
    valk = (valA * valE) - (valB * valD)
    crossvec.append(vali)
    crossvec.append(valj)
    crossvec.append(valk)
    return crossvec
#
def scalprod(scalar=0, vec1=[]):
    """
    Purpose: find scalar product between a number and a vector
        inputs:  
            [$1] scalar to multiply by
            [$2] list (string values will be converted to floats)
            example: ['1','0','0'] or [1,0,0]
        outputs:
            vector multiplied by scalar
    """
#   Modules:
    import os
    import sys
    import math
    import Xvector
#   Validation
#   #  
    try:
        scalar = float(scalar)
    except ValueError, TypeError:
        print scalar, 'needs to be convertible into float.'
        return None
    vec1 = Xvector.valflt(vec1)
    if (vec1 == None):
        print 'Error in vector 1.'
        print '(Exiting.)'
        return None
    multvec=[]
    mult=0
    for myflt in vec1:
        mult = scalar * myflt
        multvec.append(mult)
    return multvec

def frctocart(l1,l2,l3,frac):
    """
    Purpose: Return cartesian coordinates given lattice vectors and
            fractional coordinates
        inputs:  
            [$1,$2,$3] lattice vectors 1, 2, and 3
            [$4] fractional coordinates
            example: ['1','0','0'] or [1,0,0]
        outputs:
            vector
    """
#   Modules:
    import os
    import sys
    import math
    import Xvector
#   Validation
#   #  
    l1 = Xvector.valflt(l1)
    l2 = Xvector.valflt(l2)
    l3 = Xvector.valflt(l3)
    frac = Xvector.valflt(frac)
    if len(frac) != 3:
        print 'Only three lattice vectors allowed for fractional coordinate.'
        return None
    l1mult=0
    l2mult=0
    l3mult=0
    totvec=0
    l1mult = Xvector.scalprod(frac[0],l1)
    l2mult = Xvector.scalprod(frac[1],l2)
    l3mult = Xvector.scalprod(frac[2],l3)
    totvec=add(add(l1mult,l2mult),l3mult)
    return totvec

def strtoflt(mystr):
    """
    Purpose: Return cartesian coordinates given lattice vectors and
            fractional coordinates
        inputs:  
            [$1,$2,$3] lattice vectors 1, 2, and 3
            [$4] fractional coordinates
            example: ['1','0','0'] or [1,0,0]
        outputs:
            vector
    """
#   Modules:
    import os
    import sys
    import math
    import Xvector
#   Validation
#   #  
    mymat=[]
    if len(mystr) == 0:
        print 'Empty string.'
        return None
    if ',' in mystr:
        mymat = mystr.split(',')
    else:
        mymat = mystr.split()
    myvec = Xvector.valflt(mymat)
    return myvec
