#!/usr/bin/env python
# Purpose: class structure for running VASP
# Authors: Tam Mayeshiba and Tom Angsten
# 9/30/11 convention: any class taking input will have 
#           classname_input as the input class
#         - class convention: derived class xxx_ancestorname
#         - instance name convention: xxx_[2 char from ancestorname]
#TOM TASKS = make output_file class - in run change the outcar and contcar to output_file classes
import os
import shutil
import subprocess
import util 
import time
import math
import pdb

#Path to temporary files used (ensures cross-user consistency)
temp_path = os.path.expanduser('~/bin/tscripts/temps')
potcar_path = "/export/apps/vasp_pseudopotentials/"

class file:
    #data = [] #TTM 11/16/11 remove from here to avoid confusing reinit.

    #TTM 10/31/11 allow to initialize without a file path
    def __init__(self,file_path=""):
        self.data=[] # TTM 11/16/11 add here
        if file_path != "":
            self.from_file(file_path)
        else:
            pass

    def from_file(self,file_path):
        self.data = []
        #TTM+2 10/7/11 add error checking in case of no file
        if not os.path.isfile(file_path):
            return
        readf = open(file_path,'rb')
        lines = readf.readlines()
        for line in lines:
            self.data.append(line)
        readf.close()    
    
    def to_file(self,file_path):
        #TTM+2 10/7/11 add error checking in case of no path
        if (file_path == "") or (file_path == None):
            return
        writef = open(file_path,'wb')
        for line in self.data:
            writef.write(line)
        writef.close()

    #TTM 11/10/11 created to solve archiving problems
    def to_unique_file(self, parent_path="", try_name="", suffix="", max=10):
        if (parent_path == "") or (parent_path == None):
            return
        if (try_name == "") or (try_name == None):
            try_name = str(time.time())
        basicname = os.path.join(parent_path, try_name)
        tryme = basicname + suffix
        ct = 1
        while os.path.isfile(tryme):
            tryme = basicname + str(ct) + suffix
            ct = ct + 1
            if ct > max: #TTM 11/23/11 increase max threshold
                print 'Over', max, 'of the same file. Erroring out.'
                return
        self.to_file(tryme)

    def to_stdout(self):
        for line in self.data:
            print line,

    #TTM+block 10/5/11 created (Pgetline)
    def get_line_number(self, line_number):
        matidx = line_number - 1 # matrix indices start at 0
        if matidx < 0:
            return None
        if len(self.data) > matidx:
            return self.data[line_number - 1]
        else:
            return None

    #TTM+block 10/5/11 created (Pgetline)
    def get_line_match(self, string_to_match):
        for line in self.data:
            if string_to_match in line:
                return line
        return None

    #TTM+block 11/7/11 created to return line number
    def get_line_match_number(self, string_to_match):
        linect = len(self.data)
        ct = 0
        myline=""
        while ct < linect:
            myline = self.data[ct]
            if string_to_match in myline:
                return ct + 1 #return line numbers starting from 1
            ct = ct + 1


    #TTM+block 10/5/11 created
    def get_last_line_match(self, string_to_match):
        lastmatch=""
        for line in self.data:
            if string_to_match in line:
                lastmatch = line
        if lastmatch == "":
            return None
        return lastmatch

    #TTM+block 11/8/11 separated from get_segment_from_last_line_match
    def get_segment(self, string_to_chop, start_string="", end_string=""):
        start_found=""
        startpos=""
        endpos=""
        if start_string == "":
            startpos = 0 # start from beginning
        else:
            start_found = string_to_chop.find(start_string)
            if start_found == -1:
                return None # not found
            startpos = start_found + len(start_string) # start after the string
        if end_string == "":
            endpos = len(string_to_chop) # go to end
        else:
            endpos = string_to_chop.find(end_string)
            if endpos == -1:
                return None #not found
        return string_to_chop[startpos:endpos].strip() # strip off white space


    #TTM+block 10/5/11 created (Pgetseg)
    def get_segment_from_last_line_match(self, string_to_match, start_string="", end_string="", match_last_line=True):
        found=""
        if match_last_line:
            found = self.get_last_line_match(string_to_match)
        else:
            found = self.get_line_match(string_to_match)
        if not found:
            return None
        myseg = self.get_segment(found, start_string, end_string)
        return myseg

    #TTM 11/7/11 from Pmodfilenew.py
    def modify_file_by_line_number(self, lineno="", mode="", param=""):
        #mode: D - Delete, R - Replace, I - insert
        #param: if mode is:
        #       D - ignored; leave blank
        #       R - string with which to replace whole line at lineno
        #       I - string to insert below lineno as a whole line
        try:
            lineno = int(lineno)
        except (ValueError, TypeError):
            print 'Invalid line number.'
            return
        try:
            mode = str.upper(mode)
        except (ValueError, TypeError):
            print 'Invalid mode.'
            return
        if mode not in ['D','R','I']:
            print 'Only D-delete, R-replace, I-insert supported.'
            return
        dataidx = lineno - 1 #sync line number to matrix elements
        origlen = len(self.data)
        before=[]
        after=[]
        if (dataidx > origlen - 1):
            print 'File does not contain', lineno, 'lines.'
            return
        if (dataidx < 0):
            print 'Negative line number cannot be found.'
            return
        #
        if (dataidx == 0):
            before=[]
        else:
            before = self.data[0:dataidx]
        if (dataidx == origlen - 1):
            after=[]
        else:
            after = self.data[dataidx+1:]
        if (mode == 'D'):
            pass # no special action
        if (mode == 'R'):
            before.append(param)# put new line where old one went
        if (mode == 'I'):
            before.append(self.data[dataidx]) # put line back in
            before.append(param)
        self.data = before + after
        return

#TTM 10/31/11 created
class output_file(file):
    #path = "" #TTM 11/16/11 removing from here to avoid odd initialization

    def __init__(self):
        file.__init__(self)
        self.path = ""   #allow creation without reading from a file path
        
class input:
    #path = "" #TTM 11/16/11 removing from here to avoid odd initialization

    #TTM 11/1/11 added to initialize properly
    def __init__(self):
        self.path = "" 

    def reconcile_path(self):
        self.path = os.path.expanduser(self.path)
        self.path = os.path.abspath(self.path)

class counter: #TA 10/15/11 simple counter class (put this in util file eventually)
    #c = 0 #TTM 11/16/11 removing from here to avoid odd initialization
    #limit = 0 #TTM 11/16/11 removing to avoid odd initialization
    def __init__(self,i_limit):
        self.c = 0
        self.limit = i_limit
    def increment(self):
        self.c += 1
    def reset(self):
        self.c = 0
    def count(self):
        return self.c
    def output(self):
        return "Count: " + str(self.count()) + " Limit: " + str(self.limit)#for debugging
    def expired(self):
        return (self.c >= self.limit)
    
class error_tracker: #TA 10/15/11 used to track error stats
    #TTM does this work correctly without an __init__function?
    no_outcar_count = counter(1)
    no_contcar_count = counter(2)
    wfall_time_count = counter(2)

#TTM+block 11/9/11 update to use whole_input class
class calculation:
    #calc_in = None #TTM 11/16/11 removing variables from here for initializ.
    #run_count = 0
    #run = []
    #run_inputs = []
    #error = [] #TA 10/15/11 set of error tracking classes
    #    bulk_completed = False #TTM 10/3/11 removed; specific to relax_calculation
    #status = 'N' #TA 10/20/11 status of the run: 
    #   N - null, E - error, R - relaxing, 
    #   X - relax completed, S - static phase, C - complete
    #relax_path = "" #TA 10/21/11 this should be declared as a member variable if located in other parts of class
    
    def __init__(self, i_calc_input):
        self.run_count = 0 #TTM+5 adding initialization of parameters
        self.run = []
        self.run_inputs = []
        self.error = []
        self.status = 'N' 
        # Null, Error, Relaxing, relaX complete, Static phase, Completed
        self.calc_in = i_calc_input
        self.calc_in.reconcile_path()
        self.relax_path = os.path.join(self.calc_in['calc_path'], 
            self.calc_in['calc_name']) #TTM 10/6/11 added so don't have to keep retyping
        if 'append' not in self.calc_in.keys():
            self.calc_in['append'] = ""  #TTM 11/16/11 added for string concat.

        if(os.path.exists(self.calc_in['calc_path'])):
            print 'Directory at location ' + self.calc_in['calc_path'] + ' already exists.'
        else:
            os.mkdir(self.calc_in['calc_path'])

        self.set_up_first_run()

    def set_up_first_run(self):
        #self.calc_in.run_in.path = self.relax_path  #TTM 10/31/11 never change input!  #TTM 10/3/11 don't append 0 to first run
        new_run_in=""
        new_run_in = run_input()
        self.calc_in.clone_to_new_input(new_run_in)
        new_run_in['run_path'] = self.relax_path
        new_run_in.make_name()
        self.run_inputs.append(new_run_in)
        self.run.append(run(self.current_run_input()))
        print 'self.current_run().run_in.path', self.current_run().run_in['run_path']
        self.error.append(error_tracker())

    #TTM 11/1/11 separate initialization from starting the run
    def start_calc(self):
        self.current_run().update_status()
        self.current_run().submit_to_queue() #TTM 10/31/11 add submit to queue
        self.status = 'R' #TTM 10/31/11 list as 'relaxing'
        self.rapid_update()                                       #TTM+4 10/4/11 added status updates #TTM 11/22/11 use rapid_update()

    #TTM 11/22/11 rapid updating until reach left-off point, for restarting calc.s
    def rapid_update(self):
        self.update()
        oldstatus=""
        oldstatus = self.status
        self.update()
        while not oldstatus == self.status:
            oldstatus = self.status
            self.update()

    def current_run(self):
        return self.run[self.run_count]

    def prev_run(self): #TTM 10/31/11 added
        return self.run[self.run_count - 1]

    def current_run_input(self): #TTM 10/31/11 added
        return self.run_inputs[self.run_count]

    def prev_run_input(self): #TTM 10/31/11 added
        return self.run_inputs[self.run_count - 1]

    def current_error(self):
        return self.error[self.run_count]

    #TTM+6 10/31/11 remove clone_calc_run_input to run_input class
    
    #TTM 10/3/11 make iterate_path() function
    def get_next_path(self):
        return self.relax_path + str(self.run_count)

    def update(self):
        #print 'run_count', self.run_count
        #print self.run
        #print self.run[self.run_count].run_in.path
        self.current_run().update_status()
        if self.current_run().status == 'E': #TTM+1 10/31/11 if error in run, error in calc
            self.status = 'E'
        #TA 10/9/11 error checking section
        if(self.error_check() or self.status == 'E'):
            return
        elif self.status == 'C':
            print 'Complete.'
            disprun=""
            for disprun in self.run:
                disprun.print_finished()
            return
        elif self.status == 'S':
            if self.current_run().is_complete():
                self.status = 'C'
                print 'Complete.'
                disprun=""
                for disprun in self.run:
                    disprun.print_finished()
                return
        else:
            if self.current_run().is_complete():
                if self.run_count == 0: #TTM 10/31/11 remove ability to go static without having to have been relaxed, and change to elif
                    self.relax_path = self.relax_path + self.calc_in['append'] + 'r'
                    self.increment_relaxation() #TTM 10/31/11 use separated out function
                else:
                    self.evaluate_relax_completed_and_set_status()       
                    if not self.status == 'X':
                        if self.run_count == self.calc_in['max_relax']:  #TTM 10/31/11 use max relax instead of -1 TTM+1 10/7/11 limit number of relaxations
                            self.status = 'E' 
                            print 'Maximum of', self.calc_in['max_relax'], 'relaxations reached.' #TTM 10/31/11 removing skip to static.  Skipping to static.'#TA??why skip without further action? (error checking?)
                            print 'Not fully relaxed. Please check calculation.' #TTM 10/31/11 error.
                            return
                        else:
                            self.increment_relaxation() #TTM 10/31/11 use separated out function
                    #if are now in static phase, create static run
                    else:
                        self.status = 'S'
                        static_run_input = run_input() #new run_input
                        self.current_run_input().clone_with_contcar_to_poscar(static_run_input)
                        static_run_input['run_path'] = self.relax_path[:-1] + 'static' #remove the 'r'
                        try: #TTM 11/16/11 use override static_incar if one exists
                            static_run_input['incar'] = self.calc_in['static_incar']
                        except KeyError: #otherwise, create a static incar
                            static_incar = incar()
                            if self.current_run_input()['kpoints'].multiplied_kpoints() >= 27:
                                self.current_run_input()['incar'].create_static_tetrahedral(static_incar)
                            else:
                                self.current_run_input()['incar'].create_static(static_incar)
                            static_run_input['incar'] = static_incar
                        static_run_input['run_name'] = os.path.basename(static_run_input['run_path'])
                        self.run.append(run(static_run_input))
                        self.run_count += 1                         #TTM 10/28/11 changed order
                        if not self.current_run().is_complete(): #TTM+1 10/28/11 adding submit to queue separately
                            self.current_run().submit_to_queue() 
                        self.error.append(error_tracker())

    #TTM 10/31/11 separated out from update
    def evaluate_relax_completed_and_set_status(self):
        #print 'last run: ', self.run[self.run_count-1]
        #print self.run[self.run_count-1].run_in.path
        #self.current_run().is_complete() #TTM 10/31/11 remove - redundant; already checked
        last_energy = self.prev_run().get_and_set_energy_per_atom()
        #print 'last energy per atom: ', last_energy
        last_volume = self.prev_run().get_and_set_volume_per_atom()
        #print 'last volume per atom: ', last_volume
        #print 'current run: ', self.current_run()
        #print self.current_run().run_in.path
        current_energy = self.current_run().get_and_set_energy_per_atom()
        current_volume = self.current_run().get_and_set_volume_per_atom()
        #print 'current energy: ', current_energy
        #print 'current volume: ', current_volume
        delta_energy = math.fabs(current_energy-last_energy)
        #print 'delta energy: ', delta_energy
        delta_volume = math.fabs(current_volume-last_volume)
        energy_percent_change = (delta_energy/last_energy)*100
        vol_percent_change = (delta_volume/last_volume)*100

        self.status = 'X'  #TTM 10/31/11 use self.status

        if (self.calc_in['entype'] in ['T','t']):
            if(delta_energy > float(self.calc_in['encomp'])):
                self.status = 'R'
        elif (self.calc_in['entype'] in ['P','p']):
            if(energy_percent_change > float(self.calc_in['encomp'])):
                self.status = 'R'
        if (self.calc_in['voltype'] in ['T','t']):
            if(delta_volume > float(self.calc_in['volcomp'])):
                self.status = 'R'
        elif (self.calc_in['voltype'] in ['P','p']):
            if(vol_percent_change > float(self.calc_in['volcomp'])):
                self.status = 'R'
        print 'Relax status (R - relaxing, X - relax completed): ', self.status

    
    #TTM 10/31/11 separated out from update
    def increment_relaxation(self):
        self.run_inputs.append(run_input()) #TTM+block 10/7/11 use unique run_input for the appended run
        self.run_count +=1
        self.prev_run_input().clone_with_contcar_to_poscar(self.current_run_input()) #TTM 10/31/11 use new clone function
        self.current_run_input()['run_path'] = self.get_next_path()
        self.current_run_input().make_name()
        self.run.append(run(self.current_run_input()))
        if not self.current_run().is_complete(): #TTM+1 10/28/11 adding submit to queue separately
            self.current_run().submit_to_queue() 
        self.error.append(error_tracker())



    def error_check(self): #TA 10/9/11 added (called in update)
        print "in calculation.error_check"
        if(self.current_run().status == "E"):
            if(self.current_run().error == "no_outcar"):
                if self.current_error().no_outcar_count.expired():
                    print "Catastrophe: no outcar error has triggered too many times." #TTM 10/28/11 catastrophe
                    self.status = 'E'
                    return True
                self.current_error().no_outcar_count.increment()
                print "Run at " + self.current_run().run_in['run_path'] + " is running on the queue without an OUTCAR."
                self.current_run().remove_from_queue()
                self.current_run().archive_output()
                self.current_run().clean_directory()
                self.current_run().recycle_contcar() #must recycle before write so that new POSCAR is written
                self.current_run().write_files()
                time.sleep(2) #TA 10/22/11 perhaps there is a better way to ensure smooth transition        
                self.current_run().submit_to_queue()
            if(self.current_run().error == "no_contcar"):    
                if self.current_error().no_contcar_count.expired():
                    print "Catastrophy: no contcar error has triggered too many times."
                    self.status = 'E'
                    return True
                self.current_error().no_contcar_count.increment()
                print "Run at " + self.current_run().run_in['run_path'] + " is running on the queue without a CONTCAR."
                self.current_run().remove_from_queue()
                self.current_run().archive_output()
                self.current_run().clean_directory()
                self.current_run().recycle_contcar() #must recycle before write so that new POSCAR is written
                self.current_run().write_files()
                time.sleep(2) #TA 10/22/11 perhaps there is a better way to ensure smooth transition        
                self.current_run().submit_to_queue()
                
            if(self.current_run().error == "walltime"):
                if(self.current_error().wall_time_count.expired()):
                    print "Catastrophy: too many walltime expirations." #TA 10/15/11 error has yet to be implemented
                    self.status = 'E'
                    return True
                self.current_error().wall_time_count.increment()
            if(self.current_run().error == 'unknown'):
                print "Unknown error: Catastrophic failure, terminating."
                self.status = 'E'
                return True
        return False

#TTM+block 11/9/11 update to use whole_input class
class neb_calculation(calculation):
    #ep1 = "" #expect a run class, static #TTM 11/16/11 removing for initializ.
    #ep2 = ""
    #static_run_inputs = []
    #static_runs = []
    #image_runs = []
    #image_run_inputs = []
    #num_im_int = 0

    #TTM+block 10/3/11 override init so run does not start automatically
    def __init__(self, i_calc_input, ep1_path="", ep2_path=""):
        self.run_count = 0 #TTM+5 adding initialization of parameters
        self.run = []
        self.run_inputs = []
        self.error = []
        self.status = 'N'
        # Null, Error, Relaxing, relaX complete, Static phase, Completed
        self.calc_in = i_calc_input
        self.calc_in.reconcile_path()
        self.static_run_inputs = [] #TTM+3 11/1/11 initialize properly
        self.static_runs = []
        self.image_runs = []
        self.image_run_inputs = []
        self.num_im_int = 0
        self.ep1=""  #run class
        self.ep2=""
    
        if ep1_path == "":
            print 'Error: no endpoint 1 path given.'
            self.status = 'E'
            return
        if ep2_path == "":
            print 'Error: no second endpoint path given.'
            self.status = 'E'
            return
        
        #TTM+7 10/4/11 error-checking
        self.relax_path = self.calc_in['calc_path']
        if(os.path.exists(self.relax_path)):
            print 'Directory at location ' + self.relax_path + ' already exists.'
        else:
            os.mkdir(self.relax_path)        
       
        self.set_up(ep1_path, ep2_path)

    def start_calc(self):
        if not self.images_complete(): #TTM 11/14/11 add check
            self.current_run().submit_to_queue() #TTM 10/28/11 added
        self.update()

        if self.status == 'X':
            print 'Images relaxed.'
        elif self.status == 'C':
            print 'NEB marked as complete.'
        self.status = 'R' 
        self.update()

    #TTM 11/14/11 added
    def images_complete(self):
        missing_complete=0
        for image_run in self.image_runs:
            if not image_run.is_complete():
                missing_complete = 1
        if missing_complete == 1:
            return False
        return True

    def set_up(self, ep1_path, ep2_path):
        neb_in = run_input()
        self.calc_in.clone_to_new_input(neb_in)
        neb_in['run_path'] = os.path.join(self.relax_path, self.calc_in['calc_name'])
        neb_in.make_name()
        self.run_inputs.append(neb_in)
        self.run.append(nebtop_run(neb_in))
        
        self.num_im_int = int(neb_in['num_im']) # actual val is a string

        self.ep1 = run()
        self.ep2 = run()

        self.ep1.run_in = run_input()
        self.ep2.run_in = run_input()

        self.ep1.run_in.populate_run_input_from_path(ep1_path)
        self.ep1.set_up_output_paths()
        self.ep2.run_in.populate_run_input_from_path(ep2_path)
        self.ep2.set_up_output_paths()

        if not os.path.isfile(neb_in['run_path'] + '/CONTCARep1'):
            # copy endpoint CONTCARs
            shutil.copy(self.ep1.contcar.path, neb_in['run_path'] + '/CONTCARep1')
            shutil.copy(self.ep2.contcar.path, neb_in['run_path'] + '/CONTCARep2')
            # run_in should take care of KPOINTS, POTCAR, and INCAR (somehow...)
            convintstr=""
            convintstr='nice -n 19 convasp -intpol CONTCARep1 CONTCARep2'
            convintstr = convintstr + ' ' + str(neb_in['num_im']) + ' n'
            oldwd=""
            try:
                oldwd = os.getcwd()
            except OSError:
                pass
            os.chdir(neb_in['run_path'])
            makeme = subprocess.Popen(convintstr, shell=True)
            makeme.wait()
            if not oldwd == "":
                os.chdir(oldwd)
            # put OSZICARs into endpoint folders
            shutil.copy(self.ep1.oszicar.path, neb_in['run_path'] + '/00/OSZICAR')
            shutil.copy(self.ep2.oszicar.path, neb_in['run_path'] + '/0' + str(self.num_im_int + 1) + '/OSZICAR')
        self.run_count = 0 #there is only 1 run, the neb_top run
        
        self.set_up_image_runs()

    #TTM+block 10/4/11 created so can call again
    def set_up_image_runs(self):
        # set up run classes for images but don't run them
        idx = 0
        impath=""
        while (idx < self.num_im_int):
            impath = os.path.join(self.current_run().run_in['run_path'], str(idx + 1).zfill(2)) #pad to 2 spaces: 01, 02, etc.
            self.image_run_inputs.append(run_input())
            self.current_run_input().clone_to_new_input(self.image_run_inputs[idx])
            self.image_run_inputs[idx]['run_path'] = impath
            self.image_run_inputs[idx]['poscar'] = poscar(impath + '/POSCAR')
            self.image_runs.append(run(self.image_run_inputs[idx]))  #TTM 10/4/11 use run class now that can error check
            idx = idx + 1
    
    def set_up_static_runs(self):
        idx=0
        nebtoppath=""
        statpath=""
        statname=""
        # set up number of static nodes
        stat_nodes=0
        stat_nodes = int(self.current_run_input()['nodes'])/self.num_im_int
        if stat_nodes <= 0: #TTM 11/17/11 less than or equal to zero nodes not allowed
            stat_nodes = 1
        statname = self.current_run_input()['run_name'] + self.current_run_input()['append'] + 'static'
        #current run is the nebtop_run
        nebtoppath = self.current_run_input()['run_path']
        statpath = os.path.join(os.path.dirname(nebtoppath),statname)
        if os.path.exists(statpath):
            print 'Static path at ', statpath, 'already exists.'
        else:
            os.mkdir(statpath)
        while (idx < self.num_im_int):
            namestr=""
            namestr = '0' + str(idx + 1) #01, 02, 03,... etc
            self.static_run_inputs.append(run_input())
            self.current_run_input().clone_to_new_input(self.static_run_inputs[idx])
            self.static_run_inputs[idx]['run_path'] = os.path.join(statpath, namestr + 'static')
            self.static_run_inputs[idx]['poscar'] = poscar(self.image_run_inputs[idx]['run_path'] + '/CONTCAR')
            try: #TTM 11/16/11 use override static_incar if one exists
                self.static_run_inputs[idx]['incar'] = self.calc_in['static_incar']
            except KeyError: #otherwise, create a static incar
                static_incar = incar()
                if self.ep1.run_in['kpoints'].multiplied_kpoints() >= 27:
                    self.ep1.run_in['incar'].create_static_tetrahedral(static_incar)
                else:
                    self.ep1.run_in['incar'].create_static(static_incar)
                self.static_run_inputs[idx]['incar'] = static_incar
            self.static_run_inputs[idx]['run_name'] = os.path.basename(self.static_run_inputs[idx]['run_path'])
            self.static_run_inputs[idx]['nodes'] = stat_nodes #take number of nodes down from the neb calculation. Number of processors should stay the same
            #self.static_run_inputs[idx].script = submit_script(self.static_run_inputs[idx].inp['run_name'],'default',1,1,2000,3) #TTM 11/13/11 let run handle script
            self.static_runs.append(run(self.static_run_inputs[idx]))
            if not self.static_runs[idx].is_complete(): #TTM+1 10/28/11 add submit to queue
                self.static_runs[idx].print_inputs() #TTM 11/14/11 DEBUG
                self.static_runs[idx].submit_to_queue() 
            idx = idx + 1
        if not os.path.exists(statpath + '/00static'):
            shutil.copytree(nebtoppath + '/00', statpath + '/00static')
        if not os.path.exists(statpath +'/0'+str(self.num_im_int + 1)+'static'):
            shutil.copytree(nebtoppath + '/0' + str(self.num_im_int + 1), statpath + '/0' + str(self.num_im_int + 1) + 'static')


    #TTM 10/31/11 use self.status
    def update(self):
        idx = 0
        upperlim = 0
        run_item = ""

        print 'neb_calc status: ', self.status
        if self.status == 'R':
            self.status = 'X'
            for run_item in self.image_runs:
                run_item.jobid = self.run[self.run_count].jobid
                run_item.update_status()
                print '.'
                #print 'Status: ', run_item.status
                #print 'OUTCAR: ', run_item.outcar
                #print 'Complete: ', run_item.is_complete()
                if not run_item.is_complete():
                    self.status = 'R'
            self.current_run().update_status() #TTM 11/17/11 added
            if self.current_run().status in ['R','Q'] and self.current_run().queue_status() == 'X':
                self.status = 'E'

        if self.status == 'X':
            self.status = 'S'
            self.set_up_static_runs()
        
        if self.status == 'S':
            self.status = 'C'
            for run_item in self.static_runs:
                if not run_item.is_complete():
                    self.status = 'S'
        
        if self.status == 'C':
            pass

#TTM+block 11/9/11 update to use whole_input class
class run: 
    '''
    Encapsulation of a run in VASP, defined as a directory with all of the 
    input files along with the submission script. The run object can be queried 
    with regard to the state of the run, the time elapsed, and so forth.
    '''
    #status = ""  #TTM 11/16/11 removing for clear initialization
    #   N (null), Q (queued), E (errored), 
    #   R (running), C (complete)
    #error = "" #TA 10/9/11 added (error code list: walltime, no_outcar...)
    #run_in = None
    #contcar = "" 
    #outcar = ""
    #oszicar = "" #TTM 10/3/11 added
    #jobid = -1
    #energy = 0
    #volume = 0
    #energy_per_atom = 0
    #volume_per_atom = 0
    #submit_time = 0 #time of submission in seconds since the first epoch
    #life_time = 0   #time of existence of run in seconds
    #run_time = 0    #time at which run last went into R status on queue TA 10/20/11
    #num_atoms = 0 # TTM 10/3/11 added
    #arch_count = counter(100000) #TA 10/21/11 used for archiving data (tracks num. saves)
    #arch_contcar = [] #TA 10/21/11 archiving contcar and outcar virtually
    #arch_outcar = []
    #script = ""

    #TTM 10/28/11 removing submit to queue from initialization.
    # __init__ creates a new instance. Reset all variables.
    # Can also be used to rerun runs. 
    def __init__(self,i_input_files=""): #TTM allow input to be empty
        self.status = 'N'
        #Null, Queued, Running, Error, Complete
        self.error = ""
        self.energy = 0
        self.energy_per_atom = 0
        self.volume = 0
        self.volume_per_atom = 0
        self.submit_time =0
        self.life_time = 0
        self.num_atoms = 0
        self.arch_count = counter(100000)
        self.arch_contcar=[]
        self.arch_outcar=[]
        self.run_time = 0
        self.run_in = i_input_files
        self.script = ""
        self.contcar = ""
        self.outcar = ""
        self.oszicar = ""
        self.jobid = -1
        if not self.run_in == "":
            print 'run path: ', self.run_in['run_path']
            self.set_up_output_paths()
            
            if(os.path.exists(self.run_in['run_path'])):
                print 'Run at location ' + self.run_in['run_path'] + ' already exists.'
            else:
                os.mkdir(self.run_in['run_path'])
            self.print_inputs() #TTM 11/22/11 debug 

    #TTM+block 11/9/11 created
    def set_up_output_paths(self):
        self.contcar = output_file()                        #TTM+5 10/31/11 moved here so can check already-run runs. Use output_file class.
        self.contcar.path = self.run_in['run_path'] + '/CONTCAR'
        self.outcar = output_file()
        self.outcar.path = self.run_in['run_path'] + '/OUTCAR'
        self.oszicar = output_file()
        self.oszicar.path = self.run_in['run_path'] + '/OSZICAR'

    #TTM+block 10/6/11 created so can call and override independently (only set up certain files for some runs)
    def create_essential_files(self):
        print 'Creating essential files.'
        self.run_in['incar'].to_file(self.run_in['run_path']+'/INCAR')
        self.run_in['poscar'].to_file(self.run_in['run_path']+'/POSCAR')
        self.run_in['kpoints'].to_file(self.run_in['run_path']+'/KPOINTS')
        self.run_in['potcar'].to_file(self.run_in['run_path']+'/POTCAR')
        self.create_submission_script()

    def create_submission_script(self):
        #self.run_in['run_name'] = os.path.basename(self.run_in.inp['run_path']) #TTM 11/16/11 move to run_input
        self.script = script(self.run_in) #script is no longer in input class, it is created from whole_input
        if self.run_in['location'] in ['bardeen','curie','korczak']:
            self.script.validate_CMG_script_values()
            self.script.make_CMG_script()
        elif self.run_in['location'] == 'ranger': #TTM 11/23/11 add ranger support
            self.script.validate_ranger_script_values()
            self.script.make_ranger_script()
        self.script.to_unique_file(self.run_in['run_path'], 'parallel', '.sh')

    #TA 10/15/11 cleans out directory - ALL files are removed
    def clean_directory(self):
        for the_file in os.listdir(self.run_in['run_path']):
            if(the_file[0] == '.'):
                continue #protect the archived output
            file_path = os.path.join(self.run_in['run_path'],the_file)
            if os.path.isfile(file_path):
                os.remove(file_path)
        return

    #TTM+block 11/1/11 created
    def missing_restart_files(self):
        flag=0
        stat=""
        if not os.path.isfile(self.contcar.path):
            print 'Missing CONTCAR for restart.'
            flag = 1
        else:
            stat = os.stat(self.contcar.path)
            if (stat.st_size) == 0:
                print 'CONTCAR is empty for restart.'
                flag = 1
        if flag:
            return True

    #TTM+block 10/31/11 created
    def missing_essential_files(self):
        flag=0
        stat=""
        if not os.path.isfile(self.run_in['run_path']+'/INCAR'):
            print 'Missing INCAR.'
            flag = 1
        else:
            stat = os.stat(self.run_in['run_path']+'/INCAR')
            if (stat.st_size) == 0:
                print 'INCAR is empty.'
                flag = 1
        if not os.path.isfile(self.run_in['run_path']+'/POSCAR'):
            print 'Missing POSCAR.'
            flag = 1
        else:
            stat = os.stat(self.run_in['run_path']+'/POSCAR')
            if (stat.st_size) == 0:
                print 'POSCAR is empty.'
                flag = 1
        if not os.path.isfile(self.run_in['run_path']+'/POTCAR'):
            print 'Missing POTCAR.'
            flag = 1
        else:
            stat = os.stat(self.run_in['run_path']+'/POTCAR')
            if (stat.st_size) == 0:
                print 'POTCAR is empty.'
                flag = 1
        if not os.path.isfile(self.run_in['run_path']+'/KPOINTS'):
            print 'Missing KPOINTS.'
            flag = 1
        else:
            stat = os.stat(self.run_in['run_path']+'/KPOINTS')
            if (stat.st_size) == 0:
                print 'KPOINTS is empty.'
                flag = 1
        if flag:
            return True

    #TTM+block 11/1/11 created - expect this to get more complicated later
    def missing_submission_script(self):
        if not self.find_last_submission_script():
            return True
        return False
        #if self.run_in.inp['script'] == "":
        #    print 'No submission script.'
        #    self.status = 'E'

    #TTM+block 11/10/11 created
    def find_last_submission_script(self):
        myfile=""
        latesttime=0
        myfiles=[]
        filedict={}
        runpath=""
        runpath = self.run_in['run_path']
        myfiles = os.listdir(runpath)
        for myfile in myfiles:
            if '.sh' in myfile:
                filedict[os.stat(runpath + '/' + myfile).st_mtime] = myfile
        if len(filedict) > 0:
            return filedict[max(filedict)]
        return None

    #TTM+block 11/1/11 split out from submit_to_queue
    def okay_to_submit(self):
        self.jobid = self.get_latest_jobid()
        print 'Retrieved job id: ', self.jobid
        if self.queue_status() in ['R','Q']:
            return False
        if self.missing_essential_files():
            self.create_essential_files()
            return True
        if self.is_complete():
            self.status = 'C'
            self.print_finished()
            return False
        if self.missing_restart_files():
            self.archive_output()
            self.create_submission_script() # in case changed platforms
            return True
        else:
            self.recycle_contcar()
            self.archive_output()
            self.create_submission_script() # in case changed platforms
            return True
        return False

    #TTM+block 10/4/11 created in case need to resubmit
    def submit_to_queue(self):
        if self.okay_to_submit():
            cwd=""
            try:
                cwd = os.getcwd()
            except OSError:
                pass #running from silent script disallows getcwd
            os.chdir(self.run_in['run_path'])
            lastscript = self.find_last_submission_script()
            submitted = subprocess.Popen('qsub ' + lastscript,
                        shell=True,stdout=subprocess.PIPE)
            submitted.wait()
            self.jobid = util.extractJobid(submitted.communicate()[0])
            print 'job', self.jobid, 'submitted.'
            if not cwd == "":
                os.chdir(cwd)
            self.submit_time = time.time()
            self.run_time = 0
            self.store_jobid()  #TTM add
        else:
            print 'Not submitted. Job', self.jobid, 'status is', self.status

    #TTM 11/1/11 add block
    def store_jobid(self):
        jobfile=""
        jobfile = open(self.run_in['run_path'] + '/jobids','ab')
        jobfile.write(str(self.jobid) + '\n')
        jobfile.close()

    #TTM 11/1/11 add block
    def get_latest_jobid(self):
        jobfile=""
        ids=[]
        if not os.path.isfile(self.run_in['run_path'] + '/jobids'):
            return None
        jobfile = open(self.run_in['run_path'] + '/jobids','rb')
        ids = jobfile.readlines()
        if len(ids) > 0:
            return ids[len(ids)-1].strip()
        else:
            return None

    def remove_from_queue(self):
        termination = subprocess.Popen('qdel ' + str(self.jobid), shell=True,
                        stdout=subprocess.PIPE)
        termination.wait()
        print 'Run number ' + str(self.jobid) + ' terminated.'
        
        while not (self.queue_status() == None): #TA 10/15/11 now it waits until removed
            print "Waiting for job to leave queue."
        return
    
    #TTM 11/1/11 sorry Tom, but I'm going to make these real instead of virtual.
    def recycle_contcar(self): #used for restarting runs with an outcar stored #TA 10/21/11
        self.run_in['poscar'].to_unique_file(self.run_in['run_path'],'POSCAR') #TTM 11/10/11 no archive counting - just straight numeric incrementation
        print 'saved old POSCAR'
        shutil.copy(self.contcar.path, self.run_in['run_path'] + '/POSCAR')
        print 'copied CONTCAR to POSCAR'
        #self.run_in.inp['poscar'] = poscar(self.run_in.inp['run_path'] + '/POSCAR') #TTM 11/16/11 removing this line as non-functional, and also, run should never change anything in its run input
        #print "in recycle_contcar:"
        #if(self.arch_count.count() == 0 or self.arch_contcar[self.arch_count.count()-1] == None):
        #    print "no contcar archived, not restarting from last contcar"
        #    return
        #else:
        #    print "recycling contcar"
        #    self.run_in.poscar = self.arch_contcar[self.arch_count.count()-1] #TA be aware of -1 in index and SHALLOW copy
        #    #this could present a possible bug if the POSCAR were modified after a copy (unlikely but possible)


    #TTM 11/1/11 sorry Tom, but I'm going to make this real instead of virtual, too.
    def archive_output(self):
        os.listdir(self.run_in['run_path'])
        print "in archive_output: "
        if not self.arch_count.expired():
            self.arch_count.increment()
        else:
            print "count has expired"
            return
        temp_contcar = None
        temp_outcar = None
        if os.path.isfile(self.contcar.path):  #TTM 10/31/11 path to CONTCAR
            print "saving contcar"
            self.contcar.to_unique_file(self.run_in['run_path'],'CONTCAR') #TTM 11/10/11 use increasing numbers
            self.arch_contcar.append(file(self.contcar.path)) #TTM 10/31/11 path to CONTCAR
        else:
            print "contcar doesn't exist, not saving"
            self.arch_contcar.append(None) #must do this so count stays consistent
        if os.path.isfile(self.outcar.path): #TTM 10/31/11 path to OUTCAR
            print "saving outcar"
            self.arch_outcar.append(file(self.outcar.path)) #TTM 10/31/11 path to OUTCAR
            self.outcar.to_unique_file(self.run_in['run_path'],'OUTCAR') #TTM 11/10/11 use increasing numbers
            #shutil.copy(self.outcar.path, 
                #self.outcar.path + str(self.arch_count.c)) #TTM 11/1/11 added
        else:
            print "outcar doesn't exist, not saving"
            self.arch_outcar.append(None)

    def queue_status(self):
        # Common statuses: R - running, Q - queued, E - error, H - held
        # more statuses are in 'man qstat'
        # Additional status here is 'X' for no status.
        #TTM 11/1/11 changing so always returns a letter
        queue = subprocess.Popen('qstat -a ' +str(self.jobid)+ ' 2> /dev/null',
                    shell=True, stdout = subprocess.PIPE)
        queue.wait()
        queue_output = queue.communicate()[0]
        
        begin = queue_output.find(str(self.jobid))
        if(begin == -1):
            return 'X'
        
        queue_output = queue_output[begin:]
        end = queue_output.find('\n')
        queue_output = queue_output[:end]
        output = util.compressString(queue_output)
        final_out = output.split(' ')[9]    
        return final_out
    
    def time_running(self):
        return (time.time()-self.run_time)
                
    def update_status(self):
        print "in run.update_status, lifetime is " + str(self.life_time) + " runtime is " + str(self.time_running())
        if self.is_complete():
            self.print_finished()
            self.status = 'C'
            return
        qstatus = self.queue_status()
        print "qstatus: ", self.queue_status()
        print "mystatus: ", self.status
        self.life_time = time.time() - self.submit_time
        #self.print_finished() #TTM debug
        #TTM 10/3/11 moved self.energy and volume to only check underneath the 'run' status 
        if(qstatus == 'R'):
            if(self.run_time == 0):
                self.run_time = time.time() #TTM 10/31/11 removed file setting from here. Added to init.
                
            self.status = 'R'
            self.check_running_errors()

            #TA? why are these here
            #TTM+4 10/31/11 removed, redundant
            #self.get_energy() #TTM +3 10/3/11 moved here, use encapsulating functions
            #self.get_energy_per_atom()
            #self.get_volume()
            #self.get_volume_per_atom()
                    
        elif(qstatus == 'Q'):
            self.status = 'Q'
        elif(qstatus == 'X'):
            if not self.is_complete():
                self.error = self.parse_efile()
            print 'self.error: ', self.error
            if self.status in ['R','Q']: #TTM 11/1/11 added
                print 'Job', self.jobid, 'dropped off of queue.'
                self.status = 'E'
            #TTM 11/1/11 removing: make parse_efile set self.status
            #if self.error != "":
                #self.status = 'E'
        
    def check_running_errors(self):
        print "in check_running_errors: "
        os.listdir(self.run_in['run_path']) #NEED THIS for some reason or files won't register #TA10/21/11
        
        if not (os.path.isfile(self.outcar.path)) and (self.time_running() > 5):#TA10/9/11 added error checking #TTM 10/31/11 use .path
            self.status = "E"
            self.error = "no_outcar"
            return
        
        print "Has CONTCAR?: " + str(os.path.isfile(self.contcar.path))        #TTM 10/31/11 use .path
        print "CONTCAR PATH: " + self.contcar.path
        if not (os.path.isfile(self.contcar.path)) and (self.time_running()>5):
            self.status = "E"
            self.error = "no_contcar"
            return
        
    def parse_efile(self): #TA 10/20/11 look at the .e file and figure out pertinent info return error name
        #TTM 11/1/11 use file path check. Also, * does not work; using hostname
        efilestr = self.run_in['run_path'] + '/error.' 
        efilestr = efilestr + str(self.jobid) + os.getenv('HOSTNAME')
        if os.path.isfile(efilestr):
            efile = file(efilestr)
            if efile.data != []: #TTM+4 11/1/11 added for walltime error
                walltime = efile.get_line_match('walltime')
                if walltime != None:
                    self.error = "walltime_error"
                    self.status = "E"
        #TTM+4 commenting out. When is this useful? Restarting stopped jobs looks just like this.
        #else:
        #    print "run is not finished, is not on the queue, and the error file cannot be located."
        #    self.status = "E" #TTM 11/1/11 added
        #    return "unknown"
        return "" #TTM 11/1/11 return empty string for consistency
            
    #TTM 10/31/11 renamed check_completion to is_complete
    def is_complete(self):    #TTM+3 10/31/11 removed path sets. Now in init. 10/4/11 added so can get OUTCAR from already completed run
        if self.queue_status() in ['R','Q']:
            return False #TTM 11/2/11 if job is still on queue, don't check OUTCAR. HOWEVER, I have seen once a job that has finished but still appeared on the queue. We can check for this special case separately.
        if not os.path.isfile(self.outcar.path):
            return False
        self.outcar.from_file(self.outcar.path)  #TTM 10/31/11 use new structure #TTM+7 10/5/11 use file class
        usertime=""
        usertime = self.outcar.get_segment_from_last_line_match('User time (sec)', ':', '')  #TTM 10/31/11 use new output_file class
        if usertime:
            self.life_time = usertime # reset time to User time if complete - otherwise when restart a job after a few days, # seconds too high
            self.get_and_set_energy()  #TTM+3 10/31/11 renamed functions 10/3/11 get final volume and energy
            self.get_and_set_energy_per_atom()
            self.get_and_set_volume()
            self.get_and_set_volume_per_atom()
            self.print_inputs() #TTM 11/22/11 DEBUG
            return True    
        return False
            
    #TTM+block 10/5/11 added
    def print_finished(self):
        print 'Run at location ' + str(self.run_in['run_path']) + ' finished at ' + str(self.life_time) + ' seconds with an energy of ' + str(self.energy) 

    #TTM 10/5/11 created: moving here from util, and use new file class function
    def get_and_set_energy(self):
        if os.path.isfile(self.oszicar.path): #TTM+1 10/31/11 use .path
            self.oszicar.from_file(self.oszicar.path)
            try:
                self.energy = float(self.oszicar.get_segment_from_last_line_match('E0', 'E0=', 'd E'))
            except (ValueError, TypeError):
                pass
        return self.energy

    def get_and_set_volume(self):
        if os.path.isfile(self.outcar.path): #TTM+1 10/31/11 use .path
            self.outcar.from_file(self.outcar.path)
            try:
                self.volume = float(self.outcar.get_segment_from_last_line_match('volume of cell', ':', ''))
            except (ValueError, TypeError):
                pass
        return self.volume

    #TTM 10/3/11 created
    def get_and_set_num_atoms(self):
        #TTM 11/8/11 use new poscar file class instead.
        numatoms=0
        numatoms = self.run_in['poscar'].get_num_atoms()
        if numatoms == None:
            return
        self.num_atoms = numatoms
        return self.num_atoms
   
    #TTM 10/5/11 updated
    def get_and_set_energy_per_atom(self):
        if not self.energy:
            self.get_energy()
        if not self.num_atoms:
            self.get_and_set_num_atoms()
        if (self.num_atoms) > 0:
            self.energy_per_atom = self.energy / self.num_atoms
        return self.energy_per_atom
    
    #TTM+block 10/5/11 updated
    def get_and_set_volume_per_atom(self):
        if not self.volume:
            self.get_volume()
        if not self.num_atoms:
            self.get_and_set_num_atoms()
        if (self.num_atoms) > 0:
            self.volume_per_atom = self.volume / self.num_atoms
        return self.volume_per_atom
   
    #TTM 11/14/11 created
    def print_inputs(self):
        self.run_in.print_to_file(os.path.join(self.run_in['run_path'], 'inputs'))

#TTM+block 11/9/11 update to use whole_input class
class nebtop_run(run):
    #no POSCAR, since POSCARs are taken care of in image folders
    def create_essential_files(self):
        self.run_in['incar'].to_file(self.run_in['run_path']+'/INCAR')
        self.run_in['kpoints'].to_file(self.run_in['run_path']+'/KPOINTS')
        self.run_in['potcar'].to_file(self.run_in['run_path']+'/POTCAR')
        self.create_submission_script()

    #TTM+block 11/14/11 created
    def create_submission_script(self):
        self.run_in['run_name'] = os.path.basename(self.run_in['run_path'])
        self.script = script(self.run_in) #script is no longer in input class, it is created from whole_input
        if self.run_in['location'] in ['bardeen','curie','korczak']:
            self.script.validate_CMG_script_values()
            self.script.make_CMG_script()
        self.script.to_unique_file(self.run_in['run_path'], 'parallel', '.sh')
    #TTM+block 11/1/11 created, override so can exclude POSCAR
    def missing_essential_files(self):
        flag=0
        stat=""
        if not os.path.isfile(self.run_in['run_path']+'/INCAR'):
            print 'Missing INCAR.'
            flag = 1
        else:
            stat = os.stat(self.run_in['run_path']+'/INCAR')
            if (stat.st_size) == 0:
                print 'INCAR is empty.'
                flag = 1
        if not os.path.isfile(self.run_in['run_path']+'/POTCAR'):
            print 'Missing POTCAR.'
            flag = 1
        else:
            stat = os.stat(self.run_in['run_path']+'/POTCAR')
            if (stat.st_size) == 0:
                print 'POTCAR is empty.'
                flag = 1
        if not os.path.isfile(self.run_in['run_path']+'/KPOINTS'):
            print 'Missing KPOINTS.'
            flag = 1
        else:
            stat = os.stat(self.run_in['run_path']+'/KPOINTS')
            if (stat.st_size) == 0:
                print 'KPOINTS is empty.'
                flag = 1
        if flag:
            return True

    #TTM+block 11/1/11 created, override so can exclude POSCAR
    def missing_essential_files(self):
        flag=0
        stat=""
        if not os.path.isfile(self.run_in['run_path']+'/INCAR'):
            print 'Missing INCAR.'
            flag = 1
        else:
            stat = os.stat(self.run_in['run_path']+'/INCAR')
            if (stat.st_size) == 0:
                print 'INCAR is empty.'
                flag = 1
        if not os.path.isfile(self.run_in['run_path']+'/POTCAR'):
            print 'Missing POTCAR.'
            flag = 1
        else:
            stat = os.stat(self.run_in['run_path']+'/POTCAR')
            if (stat.st_size) == 0:
                print 'POTCAR is empty.'
                flag = 1
        if not os.path.isfile(self.run_in['run_path']+'/KPOINTS'):
            print 'Missing KPOINTS.'
            flag = 1
        else:
            stat = os.stat(self.run_in['run_path']+'/KPOINTS')
            if (stat.st_size) == 0:
                print 'KPOINTS is empty.'
                flag = 1
        if flag:
            return True

#TTM+block 11/10/11 copied from old submit_script class to use whole_input
#so that it can respond to needs of different levels of calculation.
#Also, queue, nodes, etc. can then be set at a high level.
#And, give it file-level functions, but a completely different init function.
class script(file):
    '''
    Encapsulation of the submission scripts used in VASP. All of the
    submission script variables (i.e. queue name, number of nodes...) are
    retained by member variables within this class.
    ''' 
    #script_in = ""     #TTM 11/16/11 moving within __init__ for initialization

    #name = ""                #name of the run, should equal input's run_name
    #queue = ""                   #queue on which to run
    #nodes = 0                   
    #processors = 0
    #memory = 0                   #specify in megabytes
    #walltime = 0                 #single integer representing number of hours
    #script = ""                  #holds script text to be sent to a .sh file
    #execline = "" # take whole line, because this is different on ranger

    def __init__(self, i_input):
        file.__init__(self)
        self.name = ""       #name of the run, should equal input's run_name
        self.queue = ""          #queue on which to run
        self.nodes = 0                   
        self.processors = 0
        self.memory = 0          #specify in megabytes
        self.walltime = 0        #single integer representing number of hours
        self.script = ""         #holds script text to be sent to a .sh file
        self.execline = ""# take whole line, because this is different on ranger
        self.account = "" #TTM 11/23/11 added for ranger
        self.script_in = i_input
        #self.populate()
    
    def validate_CMG_script_values(self):
        try:
            self.name = self.script_in['run_name']
        except KeyError:
            self.name = 'job'
        try:
            self.nodes = self.script_in['nodes']
        except KeyError:
            self.nodes = 1
        try:
            self.processors = self.script_in['processors']
        except KeyError:
            self.processors = 8
        try:
            self.memory = self.script_in['memory']
        except KeyError:
            self.memory = 2000
        try:
            self.walltime = self.script_in['walltime']
        except KeyError:
            self.walltime = 24
        try:
            self.queue = self.script_in['queue']
        except KeyError:
            self.optimize_queue() #TTM 11/23/11 use new optimize function
        try:
            self.execline = self.script_in['execline']
        except KeyError:
            self.execline = '/opt/mpiexec/bin/mpiexec ~/bin/vasp5.2_CNEB'
    
    def make_CMG_script(self): 
        #TTM 11/13/11 use self.data structure so if necessary can modify line
        # by line
        self.data = []
        self.data.append('#!/bin/bash\n')
        self.data.append('#PBS -N ' + self.name + '\n')
        self.data.append('#PBS -q ' + self.queue + '\n')
        self.data.append('#PBS -l nodes=' + str(self.nodes) + ':ppn=' + str(self.processors) + ',pvmem=' + str(self.memory) +'mb\n')
        self.data.append('#PBS -l walltime=' + str(self.walltime) + ':00:01\n') #TA 10/9/11 (changed to 5 sec for error check)
        self.data.append('##PBS -j oe\n')
        self.data.append('##PBS -k eo\n')
        self.data.append('##PBS -o output.$PBS_JOBID\n')
        self.data.append('#PBS -e error.$PBS_JOBID\n')
        self.data.append('NN=`cat $PBS_NODEFILE | wc -l`\n')
        self.data.append('echo "Processors received = "$NN\n')
        self.data.append('echo "script running on host `hostname`"\n')
        self.data.append('cd $PBS_O_WORKDIR\n')
        self.data.append('echo "PBS_NODEFILE"\n')
        self.data.append('cat $PBS_NODEFILE\n')
        self.data.append(self.execline + '\n')
        line=""    
        for line in self.data:
            self.script += line

    #TTM 11/23/11 created
    def validate_ranger_script_values(self):
        try:
            self.name = self.script_in['run_name']
        except KeyError:
            self.name = 'job'
        try:
            self.nodes = self.script_in['nodes']
        except KeyError:
            self.nodes = 1
        try:
            self.processors = self.script_in['processors']
        except KeyError:
            self.processors = 16
        try:
            self.walltime = self.script_in['walltime']
        except KeyError:
            self.walltime = 24
        try:
            self.queue = self.script_in['queue']
        except KeyError:
            self.optimize_queue() #TTM 11/23/11 well, this only returns 1 value.
        try:
            self.execline = self.script_in['execline']
        except KeyError:
            self.execline = 'ibrun /' + os.getenv('HOME') + '/bin/vasp_ncl_vtst'
        try:
            self.account = self.script_in['account']
        except KeyError:
            self.account = 'TG-DMR090023'

    #TTM 11/23/11 created
    def make_ranger_script(self):
        self.data = []
        multproc = int(self.nodes)*int(self.processors)  #ex: 3 x 16 = 48
        self.data.append('#$ -V\n')
        self.data.append('#$ -N ' + self.name + '\n')
        self.data.append('#$ -q ' + self.queue + '\n')
        self.data.append('#$ -o output.$JOB_NAME.$JOB_ID\n')
        self.data.append('#$ -e error.$JOB_NAME.$JOB_ID\n')
        self.data.append('#$ -pe 16way ' + str(multproc) + '\n')
        self.data.append('#$ -l h_rt=' + str(self.walltime) + ':00:01\n') #TA 10/9/11 (changed to 5 sec for error check)
        self.data.append('#$ -A ' + self.account + '\n')
        self.data.append(self.execline + '\n')
        line=""    
        for line in self.data:
            self.script += line

    #TTM 11/23/11 created
    def optimize_queue(self):
        myloc=""
        numproc=""
        numnodes=""
        numwall=""
        procqueue=""
        nodequeue=""
        wallqueue=""
        queuestr=""
        myloc = self.script_in['location']
        numproc = int(self.processors)
        numnodes = int(self.nodes)
        numwall = int(self.walltime)
        if myloc == 'bardeen':
            if numproc <= 8:
                procqueue = 'morganshort'
            elif numproc <= 12:
                procqueue = 'morgan2'
            elif numproc <= 32:
                procqueue = 'morgan3'
            else:
                procqueue = ""
                queuestr = queuestr + 'REQUESTED TOO MANY PPN: ' + str(numproc)

            if procqueue == 'morganshort':
                if numnodes <= 1:
                    nodequeue = 'morganshort'
                elif numnodes <= 2:
                    nodequeue = 'morgan1'
                else:
                    queuestr = queuestr + 'REQUESTED TOO MANY NODES for morgan1: ' + str(numnodes)
                    nodequeue = ""
            elif procqueue == 'morgan2':
                if numnodes <= 12:
                    nodequeue = 'morgan2'
                else:
                    queuestr = queuestr + 'REQUESTED TOO MANY NODES for morgan2: ' + str(numnodes)
                    nodequeue = ""
            elif procqueue == 'morgan3':
                if numnodes <= 4:
                    nodequeue = 'morgan3'
                else:
                    queuestr = queuestr + 'REQUESTED TOO MANY NODES for morgan3: ' + str(numnodes)
                    nodequeue = ""
            else:
                nodequeue = ""
            
            if (nodequeue == 'morganshort') or (nodequeue == 'morgan1'):
                if numwall <= 24:
                    wallqueue = 'morganshort'
                elif numwall <= 96:
                    wallqueue = 'morgan1'
                else:
                    queuestr = queuestr + 'WALLTIME TOO LONG for morgan1: ' + str(numwall)
                    wallqueue = ""
            elif nodequeue == 'morgan2':
                if numwall <= 336:
                    wallqueue = 'morgan2'
                else:
                    queuestr = queuestr + 'WALLTIME TOO LONG for morgan2: ' + str(numwall)
                    wallqueue = ""
            elif nodequeue == 'morgan3':
                if numwall <= 672:
                    wallqueue = 'morgan3'
                else:
                    queuestr = queuestr + 'WALLTIME TOO LONG for morgan3: ' + str(numwall)
                    wallqueue = ""
            else:
                wallqueue = ""
            if not wallqueue == "":
                self.queue = wallqueue
            else:
                self.queue = queuestr #print errors - this won't run successfully on any queue
        elif myloc == 'curie':
            if numproc <= 2:
                procqueue = 'morgan'
            elif numproc <= 4:
                procqueue = 'morganIB'
            else:
                queuestr = queuestr + 'Too many ppn for curie: ' + str(numproc)
                procqueue = ""
            if not procqueue == "":
                self.queue = procqueue
            else:
                self.queue = queuestr
        elif myloc == 'korczak':
            self.queue = 'default' # only one queue here
        elif myloc == 'ranger':
            self.queue = 'normal' #only support normal queue for now
        else:
            self.queue = 'NO QUEUE COULD BE OPTIMIZED.'

    #def send_to_file(self,path=""):
    #    writef = open(path,'wb')
    #    writef.write(self.script)
    #    writef.close() #TTM 10/7/11

class calcgroup:

    def __init__(self):
        pass

class neb_calcgroup(calcgroup):
    # move shared values into calcgroup after have finished with them
    #calcgroup_in = ""
    #status='I' # for NEB, statuses are B-bulk, P-endpoints, N-neb, E-error,
                # C-complete, I-initial
    #bulkcalc=""
    #ep1calc=""
    #ep2calc=""
    #nebcalc=""
    #bulkcalc_in = ""
    #ep1calc_in = ""
    #ep2calc_in = ""
    #nebcalc_in = ""
    #ep1poscar=""
    #ep2poscar=""
    #epincar=""
    #nebincar=""
    #group_path=""

    def __init__(self,i_calcgroup_input):
        self.status = 'I'
        #Bulk, endPoints, Neb, Error, Complete, Initial
        self.calcgroup_in = i_calcgroup_input
        self.bulkcalc = ""
        self.ep1calc = ""
        self.ep2calc = ""
        self.bulkcalc_in = calculation_input()
        self.ep1calc_in = calculation_input()
        self.ep2calc_in = calculation_input()
        self.nebcalc_in = calculation_input()
        self.nebincar = ""
        self.nebcalc = ""
        self.ep1poscar = ""
        self.ep2poscar = ""
        self.ep1incar = "" #TTM+1 11/22/11 differentiate ep1 and ep2 incars
        self.ep2incar = ""

        self.group_path = self.calcgroup_in['calcgroup_path']
        if os.path.exists(self.group_path):
            print 'Calculation group at', self.group_path, 'exists.'
        else:
            os.mkdir(self.group_path)


    def set_up_bulk(self):
        self.calcgroup_in.clone_to_new_input(self.bulkcalc_in)
        self.bulkcalc_in.ensure_append()
        self.bulkcalc_in['calc_path'] = os.path.join(self.group_path, self.calcgroup_in['calcgroup_name'])
        self.bulkcalc_in['calc_name'] = self.calcgroup_in['calcgroup_name'] + self.bulkcalc_in['append'] + 'bulk'
        self.bulkcalc = calculation(self.bulkcalc_in)
    
    def start_calcgroup(self):
        if self.status == 'I':
            self.set_up_bulk()
            self.bulkcalc.start_calc()
            self.status = 'B'
            self.rapid_update() #TTM 11/22/11 change to use rapid update
        else:
            self.status == 'E'
            return

    #TTM 11/22/11 rapid updating until reach left-off point, for restarting calc.s
    def rapid_update(self):
        self.update()
        oldstatus=""
        oldstatus = self.status
        self.update()
        while not oldstatus == self.status:
            oldstatus = self.status
            self.update()

    def update(self):
        if self.status == 'B':
            self.bulkcalc.update()
            if self.bulkcalc.status == 'C':
                self.status = 'P'
                self.make_ep1_from_bulk()
                self.make_ep2_from_bulk()
                self.adjust_incars_for_vacancy()
                self.set_up_ep_inputs()
                self.ep1calc = calculation(self.ep1calc_in)
                self.ep2calc = calculation(self.ep2calc_in)
                self.ep1calc.start_calc()
                self.ep2calc.start_calc()
            elif self.bulkcalc.status == 'E': #TTM 11/17/11 add error statuses
                self.status = 'E'
        elif self.status == 'P':
            self.ep1calc.update()
            self.ep2calc.update()
            if (self.ep1calc.status == 'C') and (self.ep2calc.status == 'C'):
                self.status = 'N'
                self.set_up_neb()
                self.nebcalc = neb_calculation(self.nebcalc_in, self.ep1calc.current_run().run_in['run_path'], self.ep2calc.current_run().run_in['run_path'])
                self.nebcalc.start_calc() #TTM 11/17/11 added
            elif self.ep1calc.status == 'E' or self.ep2calc.status == 'E':
                self.status = 'E'
        elif self.status == 'N':
            self.nebcalc.update()
            if self.nebcalc.status == 'C':
                self.status = 'C'
            elif self.nebcalc.status == 'E':
                self.status = 'E'
    

    def make_ep1_from_bulk(self):
        vac1line=0
        self.ep1poscar = poscar(self.bulkcalc.current_run().contcar.path)
        if self.ep1poscar.data == []:
            print 'Error: could not find bulk static contcar.'
            return
        vac1line = int(self.calcgroup_in['vac1pos']) + self.ep1poscar.get_num_atoms_line_number() + 1
        self.ep1poscar.substitute_vacancy_species_string(self.calcgroup_in['vac1pos'])
        self.ep1poscar.modify_file_by_line_number(vac1line,'D',"") # delete vacancy 1 line from POSCAR
        self.ep1poscar.to_file(self.group_path + '/make_ep1_POSCAR')


    def make_ep2_from_bulk(self):
        vac1line=0
        vac2line=0
        self.ep2poscar = poscar(self.bulkcalc.current_run().contcar.path)
        if self.ep2poscar.data == []:
            print 'Error: could not find bulk static contcar.'
            return
        vac1line = int(self.calcgroup_in['vac1pos']) + self.ep2poscar.get_num_atoms_line_number() + 1
        vac2line = int(self.calcgroup_in['vac2pos']) + self.ep2poscar.get_num_atoms_line_number() + 1 #the +1 is for the 'Direct' or 'Cartesian' line
        self.ep2poscar.substitute_vacancy_species_string(self.calcgroup_in['vac1pos'])
        vac1coords=""
        vac1coords = self.ep2poscar.get_line_number(vac1line)
        if vac1coords == None:
            print 'Error getting vacancy 1 coordinates.'
            return
        # replace vacancy 2 line with vacancy 1 line, so atom is now at
        # vacancy 1's coordinates, and there is nothing at vacancy 2
        self.ep2poscar.modify_file_by_line_number(vac2line, 'R', vac1coords) 
        # delete the old vacancy 1 line
        self.ep2poscar.modify_file_by_line_number(vac1line, 'D', "")
        self.ep2poscar.to_file(self.group_path + '/make_ep2_POSCAR')

    def adjust_incars_for_vacancy(self):
        self.ep1incar = incar()
        self.ep2incar = incar()
        self.ep1incar.file_to_dictionary(self.bulkcalc.current_run().run_in['run_path'] + '/INCAR')
        self.ep2incar.file_to_dictionary(self.bulkcalc.current_run().run_in['run_path'] + '/INCAR')
        self.ep1incar.use_vacancy_magmom_string(self.calcgroup_in['vac1pos'])
        self.ep2incar.use_vacancy_magmom_string(self.calcgroup_in['vac2pos'])
        zvals=[]
        atoms=[]
        atoms = self.ep1poscar.get_num_atoms_list()
        zvals = self.calcgroup_in['potcar'].get_zval_list()
        if 'nelect_adjust' in self.calcgroup_in.keys():
            self.ep1incar.adjust_nelect(self.calcgroup_in['nelect_adjust'],atoms,zvals)
            self.ep2incar.adjust_nelect(self.calcgroup_in['nelect_adjust'],atoms,zvals)
        self.ep1incar.p['ISIF'] = 2 # constant volume
        self.ep2incar.p['ISIF'] = 2 # constant volume
        self.ep1incar.to_file(self.group_path + '/make_ep1_INCAR')
        self.ep2incar.to_file(self.group_path + '/make_ep2_INCAR')


    def set_up_ep_inputs(self):
        ep1_temp_in = whole_input()
        self.bulkcalc_in.clone_to_new_input(ep1_temp_in)
        ep1_temp_in['calc_name'] = self.calcgroup_in['calcgroup_name'] + self.calcgroup_in['append'] + 'ep1'
        ep1_temp_in['poscar'] = poscar(self.group_path + '/make_ep1_POSCAR')
        ep1_temp_in['incar'] = incar(self.group_path + '/make_ep1_INCAR')
        ep1_temp_in.clone_to_new_input(self.ep1calc_in)
        
        ep2_temp_in = whole_input()
        self.bulkcalc_in.clone_to_new_input(ep2_temp_in)
        ep2_temp_in['calc_name'] = self.calcgroup_in['calcgroup_name'] + self.calcgroup_in['append'] + 'ep2'
        ep2_temp_in['poscar'] = poscar(self.group_path + '/make_ep2_POSCAR')
        ep2_temp_in['incar'] = incar(self.group_path + '/make_ep2_INCAR')
        ep2_temp_in.clone_to_new_input(self.ep2calc_in)

    def make_ep2_by_reflecting_ep1(self):
        pass

    def set_up_neb(self):
        neb_temp_in = whole_input()
        self.bulkcalc_in.clone_to_new_input(neb_temp_in)
        self.nebincar = incar()
        self.nebincar.file_to_dictionary(self.group_path + '/make_ep1_INCAR') #TTM 11/22/11 use ep1 INCAR as base INCAR
        self.nebincar.p['IMAGES'] = self.calcgroup_in['num_im']
        if 'spring' in self.calcgroup_in.keys():
            self.nebincar.p['SPRING'] = self.calcgroup_in['spring']
        else:
            self.nebincar.p['SPRING'] = -5
        if 'climb' in self.calcgroup_in.keys():
            self.nebincar.p['LCLIMB'] = self.calcgroup_in['climb'] 
        else:
            self.nebincar.p['LCLIMB'] = 'True'
        self.nebincar.p['IBRION'] = 1 #TTM+1 11/22/11 IBRION and POTIM
        self.nebincar.p['POTIM'] = 0.5 
        self.nebincar.create_data() # TTM 11/17/11 update from dictionary
        self.nebincar.to_file(self.group_path + '/make_neb_INCAR')
        neb_temp_in['incar'] = self.nebincar
        neb_temp_in['calc_name'] = self.calcgroup_in['calcgroup_name'] + self.calcgroup_in['append'] + 'neb' #TTM 11/17/11 added
        if 'walltime' in self.calcgroup_in.keys(): #TTM+1 11/23/11 increase walltime for neb
            neb_temp_in['walltime'] = int(self.calcgroup_in['num_im']) * int(self.calcgroup_in['walltime'])
        neb_temp_in.clone_to_new_input(self.nebcalc_in)

class param: #convention: first option is always the default option: if user presses enter with no input, default selected
    #prompt = "" #TTM 11/16/11 moving initialization to __init__
    #options = None
    #selection = -1
    #response = None
    
    def __init__(self, str):
        self.prompt = ""
        self.selection = -1
        self.response = None
        self.options = []
        if str == None:
            print "improper construction of param: no string provided"
            return
        segs = str.split('|')
        i = 0
        for item in segs:
            if i == 0:
                self.prompt = item
            else:
                self.options.append(item)
            i += 1
            
    def add_option(self, str):
        self.options.append(str)
        
    def print_options(self):
        i = 0
        for item in self.options:
            i += 1
            print str(i) + ". " + item + " ",
        print ' : ',
            
    def output(self):
        print self.prompt
        self.print_options()

    def prompt_user(self):
        if(len(self.options) == 0): #signifies a free response form and not a list number selection
            self.output()
            self.response = raw_input()
            print "Your response was " + self.response
            return
        
        valid = False
        while not valid:
            self.output()
            self.selection = raw_input()

            if(self.selection == ""):
                print "Default selected: " + self.get_option(1)
                self.selection = 1
                return
            
            if not self.is_int(self.selection):
                print "Please input an integer"
                continue
            elif(int(self.selection) >= 1 and int(self.selection) <= len(self.options)):
                valid = True
            else:
                print "There is no such option"

        print "You have chosen " + self.get_option(int(self.selection))
        
    def get_option(self,index):
        return self.options[index-1] #options do not start at zero but rather one to be consistent with their listing
    def get_selection(self): #returns either the numerical selection or the free response
        if(self.selection == -1):
            if(self.response == None):
                return None
            else:
                return self.response
        else:
            return int(self.selection)
    def get_value(self):
        if(self.selection == -1):
            if(self.response == None):
                return None
            else:
                return self.response
        else:
            return self.get_option(self.get_selection())
    def is_int(self, i):
        try:
            int(i)
            return True
        except ValueError:
            return False

class control:

    #control_interface = None #TTM 11/16/11 already in __init__
    
    def __init__(self):
        self.control_interface = interface()
        self.control_interface.retrieve_parameters()
        self.param_dir = os.path.expanduser('~/parameter_defaults') #TTM 11/22/11 add
        
    def setup(self):
        p = self.control_interface.p #create point to the parameters for smaller code
        poscar_temp = "none"
        
        if(p['system_type'].get_value() == 'metal'):
            if(p['crystal_structure'].get_value() == 'FCC'):
                #CREATE POSCAR
                a = float(p['lattice_param'].get_value())
                symbol = p['element'].get_value()
                x = p['poscar_dimensions'].get_value().split()[0]
                y = p['poscar_dimensions'].get_value().split()[1]
                z = p['poscar_dimensions'].get_value().split()[2]       
                poscar_temp = crystal().make_fcc(a,x,y,z,symbol)

        #TTM 11/22/11 unindenting KPOINTS, POTCAR
        #CREATE KPOINTS
        temp_kpoints = kpoints()                      
        temp_kpoints.name = p['element'].get_value()
        temp_kpoints.dimensions = p['kpoints_dimensions'].get_value()
        temp_kpoints.centering = p['kpoints_centering'].get_value()
        temp_kpoints.create_data()

        #CREATE POTCAR
        potcar_path = os.path.join(p['potcar_dir'].get_value(),'POTCAR.Z')
        potcar_temp_path = os.path.join(temp_path,'potcar_' + p['element'].get_value())
        shutil.copy(potcar_path,potcar_temp_path+'.Z')
        unzip = subprocess.Popen('gunzip ' + potcar_temp_path + '.Z',shell=True, stdout = subprocess.PIPE)
        unzip.wait()
        potcar_temp = potcar(potcar_temp_path) #TTM 11/8/11 potcar class #TA can't call potcar or it errors out
        # TTM replace with line below 11/8/11 enmax = util.findEnmax(potcar_temp_path) #put 1.5 times enmax in enmax for incar later
        encut = int(potcar_temp.get_max_enmax() * 1.5) #TA 11/13/11 make int
        os.remove(potcar_temp_path)

        #TTM 11/21/11 commenting out - run is responsible for creating script
        #CREATE SUBMISSION SCRIPT
        #name = p['element'].get_value()
        #temp_input = whole_input() #TTM+4 11/21/11 added to use script fcn
        #for key,value in p.iteritems():
        #    temp_input[key] = value
        #temp_input['run_name'] = name #TTM 11/21/11 added
        #temp_script = script(p)
        #temp_script = submit_script(name,p['queue'].get_value(),p['nodes'].get_value(),p['processors'].get_value(),p['memory'].get_value(),p['walltime'].get_value()) #TTM 11/21/11 changed names to match input file requirements
           
        #create incar for all types
        temp_incar = incar()
        incar_def=""
        if(p['system_type'].get_value() == 'metal'):
            incar_def = 'metal_INCAR'
            temp_incar.file_to_dictionary(os.path.join(self.param_dir, incar_def))
            temp_incar.system = p['element'].get_value()
        elif(p['system_type'].get_value() == 'perovskite'):
            incar_def = 'perovskite_INCAR'
            temp_incar.file_to_dictionary(os.path.join(self.param_dir, incar_def))
        else:
            incar_def = 'metal_INCAR'
        temp_incar.p['ENCUT'] = str(encut) #TTM 11/8/11 change to encut
        temp_incar.p['NPAR'] = str(p['nodes'].get_value())
        temp_incar.create_data()

        #TTM 11/22/11 removing static incar: should be created by calc class
        #static_incar = incar() #should be done by calculation class
        #temp_incar.create_static(static_incar)
        #static_incar.system = temp_incar.system + "_static"
        #static_incar.create_data() #just to make sure system name is put in

        #TTM 11/22/11 unindenting start_input block. This is not really metal specific.
        start_input = whole_input()         #TA 11/12/11 create inputs for runs and start runs
        start_input['calcgroup_name'] = p['element'].get_value()
        start_input['append'] = "_"
        start_input['encomp'] = 0.001
        start_input['entype'] = "T"
        start_input['volcomp'] = 1
        start_input['voltype'] = "P"
        start_input['max_relax'] = 5
        start_input['incar'] = temp_incar 
        start_input['potcar'] = potcar_temp
        start_input['poscar'] = poscar_temp
        start_input['kpoints'] = temp_kpoints
        #start_input['static_incar'] = static_incar    #TTM 11/22/11 removed
        start_input['calcgroup_path'] = ("~/bulks") #TTM 11/21/11 changed to calcgroup_path 11/22/11 changed to just bulks; name to element
        myhost="" #TTM+8 11/23/11 dynamic location lookup
        myloc=""
        myhost = os.getenv('HOSTNAME')
        if ('msae' in myhost):
            myloc = myhost.split('.')[0] 
        elif 'ranger' in myhost:
            myloc = 'ranger'
        else:
            myloc = 'bardeen' #default to bardeen
        start_input['location'] = myloc #TTM 11/23/11 changing to dynamic location
        #if start_input['location'] == 'bardeen': #TTM 11/23/11 commenting out. Never ask for input_hostname; need to get this dynamically.
        #    ipfname = 'input_bardeen'
        #start_input.print_to_file(os.path.expanduser(os.path.join(start_input['calcgroup_path'], ipfname))) #TTM 11/22/11 changed to calcgroup path
        olddir=""
        olddir = os.getcwd()
        os.chdir(start_input['calcgroup_path'])
        if not olddir == "":
            os.chdir(olddir)
        #myp.wait() #TTM 11/22/11 do not wait after all
        #calc_temp = calculation(start_input)
        #calc_temp.start_calc()
        
        myinput=""
        myrun=""
        myproc=""
        myinput = calcgroup_input()
        start_input.clone_to_new_input(myinput)
        myrun = neb_calcgroup(myinput)
        myp = subprocess.Popen(myrun.start_calcgroup())



class interface:
    #p = {} #paramters dictionary #TTM 11/16/11 commenting out
    
    def __init__(self):
        self.p = {} #parameters dictionary
        pass

    def retrieve_parameters(self): #Responsible for getting all of the inputs necessary for the run from the user
        self.p = {}
        p = self.p #simply to reduce ugliness of code
        load_files = None #set of possible parameter sets to load
        param_dir = os.path.expanduser("~/parameter_defaults")
        
        if not os.path.exists(param_dir): # standard save/load dir
            os.mkdir(param_dir)
        
        #initialize all of the prompt paramters
        p["input_mode"] = param("Select an input mode|By File|By Terminal") #prompts should be read from file later
        p["save"] = param("Would you like to save your responses as a default input file?|no|yes") #implement later
        p["save_location"] = param("What file name would you like to give (use .params extension)")
        p["load_location"] = param("What file would you like to load?")
        
        p["system_type"] = param("Select a system type|metal|perovskite(not yet supported)")
        p["element"] = param("What element is this? (Give elemental symbol)")
        p["crystal_structure"] = param("This system should be run in what crystallographic structure?|FCC|BCC|HCP")
        p["lattice_param"] = param("What inital lattice parameter would you like to use?")
        p["poscar_dimensions"] = param("What are the poscar file dimensions (give in form 'x y z')")
        p["constant_kpoints"] = param("Use the same kpoints file throughout?|yes|no(not yet supported)")
        p["kpoints_centering"] = param("Choose a kpoints system|Monkhorst|Gamma")
        p["kpoints_dimensions"] = param("Give the kpoints dimensions")
        #make cross-server, and also query for "optimum queue" (least used at time) as an option
        p["queue"] = param("On what queue shall this be run?|morganshort|morgan1|morgan2|morgan3") #TTM+3 11/21/11 changed name to match input files
        p["nodes"] = param("On how many nodes?|1|2|3|4")
        p["processors"] = param("How many processors per node?")
        p["walltime"] = param("How much wall time?(hours)")
        p["memory"] = param("")
        p["potcar_type"] = param("What type of pseudo-potential?|paw|LDA|GGA")
        p["paw_type"] = param("What type of paw?|PBE|LDA|GGA")
        p["potcar_list"] = param("Select a pseudo-potential to use") #fleshed out in later code
        p["potcar_dir"] = param("") #just used to store as a free response the path to the potcar
        
        #run menu prompts
        p["input_mode"].prompt_user()
        if(p["input_mode"].get_value() == "By File"):
            all_files = os.listdir(param_dir)
            for file in all_files:
                if file.find(".params"): 
                    p["load_location"].options.append(file)
                    
            p["load_location"].prompt_user()
            load_file = os.path.join(param_dir,p["load_location"].get_value())
            self.p = self.load_parameters(load_file)
            
        elif(p["input_mode"].get_value() == "By Terminal"):
            p["system_type"].prompt_user()
            if(p["system_type"].get_selection() == 1):#metallic system
                p["element"].prompt_user()
                p["crystal_structure"].prompt_user()
                p["lattice_param"].prompt_user()
                p["poscar_dimensions"].prompt_user()
                search_dir = potcar_path #potcar_path is defined globally at top
                p["potcar_type"].prompt_user()
                
                if(p["potcar_type"].get_value() == "paw"):
                    search_dir = os.path.join(search_dir,"paw")
                    p["paw_type"].prompt_user()
                    search_dir = os.path.join(search_dir,p["paw_type"].get_value())
                elif(p["potcar_type"].get_value() == "LDA"):
                    search_dir = os.path.join(search_dir,"LDA/" + "potcar.Apr00")
                elif(p["potcar_type"].get_value() == "GGA"):
                    search_dir = os.path.join(search_dir,"GGA")

                p["potcar_list"].options = self.get_potcar_list(p["element"].get_value(),search_dir)
                p["potcar_list"].prompt_user()
                p["potcar_dir"].response = os.path.join(search_dir,p["potcar_list"].get_value())

            p["constant_kpoints"].prompt_user()
            p["kpoints_centering"].prompt_user()
            p["kpoints_dimensions"].prompt_user()
            p["queue"].prompt_user()
            if(p["queue"].get_value() == "morgan3"): #TTM+6 11/21/11 changed names to match input files
                p["memory"].response = "1000"
            else:
                p["memory"].response = "2000"
            p["nodes"].prompt_user()
            p["processors"].prompt_user()
            p["walltime"].prompt_user()
           
            p["save"].prompt_user()
            if(p["save"].get_value() == "yes"):
                p["save_location"].prompt_user()

                self.save_parameters(os.path.join(param_dir,p["save_location"].get_value()))
                print "Parameters saved."
        
    def save_parameters(self, path):
        path = os.path.expanduser(path)
        path = os.path.abspath(path)
        
        writef = open(path,'wb')
        for name,value in self.p.items():
            if value.get_value():
                writef.write(name + "=" + value.get_value() + '\n')
        writef.close()

    def load_parameters(self, path): #TA 11/12/11 load in previously saved interface paramters, returns dictionary
        path = os.path.expanduser(path)
        path = os.path.abspath(path)
        params = {} #load parameters in here
        readf = open(path,'rb')
        all_lines = readf.readlines()
        
        for line in all_lines:
            if(line == ""):
                continue
            key = line.split('=')[0]
            value = line.split('=')[1].strip() #must strip off newline
            params[key] = param("No prompt - this param was loaded") #must be done because params will be accessed as
            params[key].response = value                             #param classes later TA 11/13/11
        readf.close()

        return params
    
    def get_potcar_list(self,element,search_dir): #returns a list of possible potcar directories
        all_dirs = os.listdir(search_dir)
        target_dirs = []

        for dir in all_dirs: #TA 10/20/11 added smart search capabilities
            index = dir.find(element)
            if not index == -1:
                set = True
                next_char = "unchanged"
                try:
                    next_char =  dir[index+(len(element))] #only do if next char exists
                except:
                    next_char = "unchanged" #doesn't exist, so this is the actual element desired
                
                if next_char == "unchanged" or next_char == "_": #TA so that O doesn't retrieve Os...etc.
                    target_dirs.append(dir)

        return target_dirs
    
class vector: #Set of three floats or ints (NOT USED IN ANYTHING YET)
    #x = 0 #TTM 11/16/11 moving initializ. to __init__
    #y = 0
    #z = 0

    def __init__(self):
        self.x = 0
        self.y = 0
        self.z = 0
    def __init__(self,x_i,y_i,z_i):
        self.x = x_i
        self.y = y_i
        self.z = z_i
        
class kpoints(file): #encapsulates a kpoints input file
    #name = "" #name in first line of KPOINTS #TTM 11/16/11 removing initializ.
    #dimensions = "" #add the other shifting line parameters later
    #centering = "" #Monkhorst or Gamma
        
    def __init__(self, file_path=""):
        file.__init__(self)
        self.name = "" #name in first line of KPOINTS
        self.dimensions = "" #add the other shifting line parameters later
        self.centering = "" #Monkhorst or Gamma
        if file_path == "":
            pass
        else:
            if os.path.isfile(file_path):
                self.from_file(file_path)

    def create_data(self): #fill in the file data
        self.data = []
        self.data.append(self.name+'\n')
        self.data.append('0\n')#change later
        self.data.append(self.centering+'\n')
        self.data.append(self.dimensions+'\n')
        self.data.append("0 0 0\n")

    def multiplied_kpoints(self):
        numline=[]
        numsplit=[]
        numline = self.data[3] # fourth line in KPOINTS
        numsplit = numline.split()
        mult=1
        myint=0
        for item in numsplit:
            try:
                myint = int(item)
            except (ValueError, TypeError):
                print 'Error summing kpoints.'
                return
            mult = mult * myint
        return mult


class incar(file):
    #system = ""
    #p = {} #dictionary holding a mapping of param names to param values

    #TTM 11/8/11 allow this to default values in from a file, too?
    def __init__(self, file_path=""):
        file.__init__(self)
        self.system = ""
        self.p = {} #dictionary holding a mapping of param names to param values
        if file_path == "":
            self.p = {} #SET DEFAULT VALUES (these will change in future based on user's needs)
            self.system = "Not Specified"
            self.p['PREC'] = "Accurate"
            self.p['IBRION'] = "2"
            self.p['NSW'] = "191"
            self.p['ISIF'] = "3"
            self.p['ISMEAR'] = "1"
            self.p['SIGMA'] = "0.2"
            self.p['LCHARG'] = "FALSE"
            self.p['LWAVE'] = "FALSE"
            self.p['LREAL'] = "Auto"
        else:   #TTM 11/9/11 added in order to read from file
            if os.path.isfile(file_path):
                self.p = {} #TTM 11/14/11 added: need initialization
                self.system = "" #TTM 11/14/11 added
                self.file_to_dictionary(file_path)
                self.create_data()


    def create_data(self):
        self.data = []
        self.data.append("System is " + self.system + '\n') #TTM 11/9/11 reserve equals sign for actual parameters

        for name,value in self.p.items():
            if not (value == ""): #TTM added condition
                self.data.append(name + " = " + str(value) + '\n') #TTM 11/17/11 string value

    def copy_to(self,dest): #creates a deep copy to ensure not pointer overwriting when static incar is made
        dest.system = self.system
        for name,value in self.p.items():
            dest.p[name] = value
        dest.create_data()

    
    def create_static(self,dest): #TA 11/13/11 create a working version of this function (these defaults are for metals)
        self.copy_to(dest)
        dest.p['IBRION'] = '-1'
        dest.p['NSW'] = '0'
        dest.p['PREC'] = 'Accurate'
        dest.p['ENCUT'] = int(float((self.p['ENCUT'])/(1.2))) #TTM 11/23/11 lower ENCUT so that it is 1.25*max ENMAX
        dest.create_data()

    #TTM 11/14/11 renamed to tetrahedral. Use switch kpoints.multiplied_kpoints() >= 27
    def create_static_tetrahedral(self,dest): #TA 11/13/11 create a working version of this function (these defaults are for metals)
        self.copy_to(dest)
        dest.p['IBRION'] = '-1'
        dest.p['NSW'] = '0'
        dest.p['ISMEAR'] = '-5'  #kpoints dependent; cannot change if irreducible kpts < 4
        dest.p['PREC'] = 'Accurate'
        dest.p['ENCUT'] = int(float((self.p['ENCUT'])/(1.2))) #TTM 11/23/11 lower ENCUT
        dest.create_data()
        
    #TTM+block 11/8/11 created
    def file_to_dictionary(self, from_path):
        self.from_file(from_path)
        line=""
        linesplit=[]
        for line in self.data:
            if line == "":
                pass #empty space
            if not ('=' in line):
                pass # system comment
            if line[0] == "#":
                pass #comments
            linesplit = line.split('=',1) # split in 1 equals sign
            if len(linesplit) > 1:
                self.p[linesplit[0].strip()] = linesplit[1].strip()


    #TTM+block 11/8/11 created
    def use_vacancy_magmom_string(self, whichatom):
        oldstring=""
        oldmagmom=""
        oldmaglist=[]
        newmaglist=[]
        try:
            oldstring = self.p['MAGMOM']
        except KeyError:
            return
        oldmagmom = oldstring.split('#')[0] # get only uncommented parts
        magpiece=""
        piecesplit=[]
        numatoms=""
        moment=""
        totalatoms=0
        found=0
        if '*' in oldmagmom: #then we are using multiplication
            oldmaglist = oldmagmom.split()
            for magpiece in oldmaglist:
                piecesplit = magpiece.split('*')
                numatoms = int(piecesplit[0])
                moment = piecesplit[1]
                totalatoms = totalatoms + numatoms
                if (whichatom <= totalatoms) and (found == 0):
                    newmaglist.append(str(numatoms - 1) +'*'+ moment) # vacancy
                    found = 1
                else:
                    newmaglist.append(magpiece)
        else: #using discrete numbers
            oldmaglist = oldmagmom.split()
            ignoreidx = whichatom - 1 #since matrix index starts at 0
            newmaglist = oldmaglist[0:ignoreidx] + oldmaglist[ignoreidx+1:]
        newstring=""
        for magpiece in newmaglist:
            newstring = newstring + magpiece + ' '
        self.p['MAGMOM'] = newstring.strip()
        self.create_data() #TTM 11/17/11 update
        return True

    def adjust_nelect(self, adjustment, num_atoms_list, zval_list):
        number=0
        zval=0
        lenlist=0
        adjnum=0
        lenlist = len(num_atoms_list)
        if not (lenlist == len(zval_list)):
            print 'Error matching number of atoms to valences.'
            return
        try:
            adjnum = int(adjustment)
        except (ValueError, TypeError):
            print 'Error converting nelect adjustment value.'
            return
        idx=0
        totalzval=0
        while idx < lenlist:
            totalzval = totalzval + num_atoms_list[idx]*zval_list[idx]
            idx = idx + 1
        totalzval = totalzval + adjnum
        self.p['NELECT'] = totalzval
        self.create_data() #TTM 11/17/11 update data from dictionary
        return True

#TTM+block 11/8/11 created
class potcar(file):
    #potcarpath="" #TTM 11/16/11 moving to __init__

    def __init__(self, file_path=""):
        file.__init__(self)
        self.potcarpath = ""
        if file_path == "":
            self.potcarpath = ""
        else:
            if os.path.isfile(file_path):
                self.potcarpath = file_path
                self.from_file(file_path)

    def potcar_exists(self):
        if os.path.isfile(self.potcarpath):
            return True
        return False

    def get_max_enmax(self):
        if not self.potcar_exists():
            return
        searchme=""
        return_string=""
        return_list=[]
        myreturn=""
        mysegment=""
        enmax_list=[]
        max_enmax = 0
        searchme = subprocess.Popen('grep ENMAX ' + self.potcarpath,
                        shell = True, stdout = subprocess.PIPE)
        searchme.wait()
        return_string = searchme.communicate()[0]
        return_list = return_string.split('\n')
        for myreturn in return_list:
            mysegment = self.get_segment(myreturn, 'ENMAX  =', ';')
            if not mysegment == None: #TA 11/13/11 added so that it works now - int() was not converting float strings
                tmp_float = float(mysegment)
                tmp_int = int(tmp_float)
                enmax_list.append(tmp_int)
        if enmax_list == []:
            print 'No ENMAX found in POTCAR.'
            return 0
        max_enmax = max(enmax_list)
        try:
            max_enmax = float(max_enmax)
        except (ValueError, TypeError):
            print 'Error converting max ENMAX from POTCAR into float.'
            return 0
        return max_enmax

    def get_zval_list(self):
        if not self.potcar_exists():
            return
        searchme=""
        return_string=""
        return_list=[]
        myreturn=""
        mysegment=""
        zval_list=[]
        searchme = subprocess.Popen('grep ZVAL ' + self.potcarpath,
                        shell = True, stdout = subprocess.PIPE)
        searchme.wait()
        return_string = searchme.communicate()[0]
        return_list = return_string.split('\n')
        for myreturn in return_list:
            mysegment = self.get_segment(myreturn, 'ZVAL   =', 'mass')
            if not mysegment == None: #TTM 11/17/11 copied Tom's code above TA 11/13/11 added so that it works now - int() was not converting float strings
                tmp_float = float(mysegment)
                tmp_int = int(tmp_float)
                zval_list.append(tmp_int)
        return zval_list

#TTM+block 11/8/11 created
class poscar(file):

    def get_num_atoms_line_number(self):
        numline=""
        numsplit=[]
        # get either line 6 (vasp 4) or line 7 (vasp 5)
        numline = self.get_line_number(6)
        if not numline:
            return None
        if not numline.strip()[0].isdigit():
            return 7  #numline is not line 6; must be line 7
        return 6

    def get_num_atoms_line(self):
        lineno=0
        lineno = self.get_num_atoms_line_number()
        if lineno == None:
            return
        return self.get_line_number(lineno)

    def get_num_atoms_list(self):
        numline = self.get_num_atoms_line()
        if not numline:
            return None
        numsplit=[]
        numsplit = numline.split() # split along spaces; will return either letters or numbers
        listnum=""
        myint=""
        numlist=[]
        for listnum in numsplit:
            try:
                myint = int(listnum)
            except (ValueError, TypeError):
                print 'Error obtaining number of species from CONTCAR in', self.contcar.path
                return None
            numlist.append(myint)
        return numlist

    def get_num_atoms(self):
        numlist=[]
        numlist = self.get_num_atoms_list()
        if not numlist:
            return None
        totnum=0
        mynum=0
        for mynum in numlist:
            totnum = totnum + mynum
        return totnum
 
    #Vmakeendpt, Vlinetospecstr, etc.
    def substitute_vacancy_species_string(self, vac1atom):
        try: #TTM 11/17/11 added so string value accepted
            vac1atom = int(vac1atom)
        except (ValueError, TypeError):
            print vac1atom, 'could not be turned into an integer.'
            return
        atomlist=[]
        addlist=[]
        specline=0
        specline = self.get_num_atoms_line_number()
        atomlist = self.get_num_atoms_list()
        if atomlist == None:
            return
        if len(atomlist) < 1:
            return
        addlist.append(atomlist[0])
        idx=1 # start at second value
        while idx < len(atomlist):
            addlist.append(addlist[idx-1]+atomlist[idx])
            idx = idx + 1
        idx=0
        newstring=""
        foundbin=0
        while idx < len(addlist):
            if (vac1atom <= addlist[idx]) and (foundbin == 0):
                newstring = newstring + ' ' + str(atomlist[idx] - 1)
                foundbin = 1
                # only single vacancy is supported right now
            else:
                newstring = newstring + ' ' + str(atomlist[idx])
            idx = idx + 1
        newstring = newstring.strip()
        newstring = newstring + '\n' #TTM 11/17/11 added carriage return
        self.modify_file_by_line_number(specline, 'R', newstring)
        return True

#TTM+block 11/8/11 created to consolidate input structure
#TTM 11/16/11 changed to dictionary type. (!!!)
class whole_input(dict):
    """
    Dictionary entries include:
    
    run_path
    calc_path
    calcgroup_path
   
    run_name
    calc_name
    calcgroup_name
    append #keeps names separated
 
    incar
    static_incar #this should go away when Tom does the to static incar fcn
    poscar
    potcar
    kpoints
    script
    
    encomp = 0.001
    entype = 'T'
    volcomp = 1
    voltype = 'P'
    max_relax = 5
    
    location = 'bardeen'
    queue = 'router'                   
    nodes = 1                   
    processors = 8
    memory = 2000                   #specify in megabytes
    walltime = 96                 #single integer representing number of hours
    script = ""                  #holds script text to be sent to a .sh file
    execline = '/opt/mpiexec/bin/mpiexec ~/bin/vasp5.2_CNEB' # take whole line, because this is different on ranger

    num_im = 3
    climb = 1 #should this go here?
    
    vac1pos
    vac2pos
    vac2poslist
    nelect_adjust
    """

    def __init__(self):
        dict.__init__(self)
        self.initialized=0
        self.allowed = []

    def __setitem__(self, key, value):
        restricted=""
        keymissing=""
        keymissing = key not in self.allowed
        restricted = not self.allowed == []
        if (self.initialized == 1) and restricted and keymissing:
                raise KeyError("Can only modify these keys:", self.allowed)
        else:
            if 'path' in key:
                dict.__setitem__(self, key, os.path.expanduser(value))
            else:
                dict.__setitem__(self, key, value)
    
    def print_to_file(self, filepath):
        myfile=""
        myfile = open(filepath, 'ab') 
        myfile.write('#' + time.asctime() + '\n')
        for key, value in self.iteritems():
            myfile.write(str(key) + ' = ' + str(value) + '\n')

    #TTM 11/16/11 created for debugging
    def print_to_screen(self): 
        for key, value in self.iteritems():
            print str(key) + ' = ' + str(value)

    def set_values_from_file(self, filepath):
        myfile=""
        filelines=[]
        filepath = os.path.expanduser(filepath) #TTM 11/16/11 added
        if not os.path.isfile(filepath):
            print 'No file found at', filepath
            print 'Could not set inputs.'
            return
        myfile = open(filepath, 'rb')
        filelines = myfile.readlines()
        for line in filelines:
            if line == "":
                pass #empty space
            if line[0] == "#":
                pass #comments
            linesplit = line.split('#') #strip out comments
            line = linesplit[0] # get only uncommented part
            linesplit = line.split('=',1) # split in 1 equals sign
            if len(linesplit) > 1:
                self[linesplit[0].strip()] = linesplit[1].strip(' \'"\n') 
                #strip off single quotes, double quotes, and carriage returns
        self.reconcile_path()
        self.transform_paths_to_file_types()
        self.initialized = 1 #lock

    def reconcile_path(self):
        for key, value in self.iteritems():
            if 'path' in key and (self.allowed == [] or key in self.allowed): #TTM 11/17/11
                self[key] = os.path.expanduser(value)

    #TTM 11/16/11 allow to use pathname in inputfile
    def transform_paths_to_file_types(self):
        filetypes=[]
        filetypes=['incar','kpoints','poscar','potcar','static_incar']
        newval=""
        fullpath=""
        valtype=""
        for key, value in self.iteritems():
            if key in filetypes:
                valtype = value.__class__.__name__
                if valtype == 'str':
                    fullpath = os.path.expanduser(value)
                    if os.path.isfile(fullpath):
                        if key == 'incar':
                            newval = incar(fullpath)
                        elif key == 'static_incar':
                            newval = incar(fullpath)
                        elif key == 'poscar':
                            newval = poscar(fullpath)
                        elif key == 'potcar':
                            newval = potcar(fullpath)
                        elif key == 'kpoints':
                            newval = kpoints(fullpath)
                        self[key] = newval
                    else:
                        print 'ERROR: Attempted to make path from nonexistent file'
                        print 'at', fullpath

    def clone_to_new_input(self, new_input):
        for key, value in self.iteritems():
            new_input[key] = value
        new_input.initialized = 1 #lock

        
class run_input(whole_input):
    def __init__(self):
        whole_input.__init__(self)
        self.allowed=['poscar','incar']
        self.allowed = self.allowed + ['run_name','run_path']
        self.allowed = self.allowed + ['nodes','queue','processors','execline']
        self.allowed = self.allowed + ['walltime','memory','script'] 
    
    def make_name(self):
        if 'run_name' not in self.keys():
            self['run_name'] = os.path.basename(self['run_path']) #TTM 11/16/11 move to run_input
            return
        if self['run_name'] == "":
            self['run_name'] = os.path.basename(self['run_path'])

    #TTM+block 11/16/11 moved to run_input class
    def clone_with_contcar_to_poscar(self, new_input):
        self.clone_to_new_input(new_input)
        new_input['poscar'] = poscar(self['run_path'] + '/CONTCAR') 
    

    #TTM+block 11/9/11 created 11/16/11 moved to within run input class
    def populate_run_input_from_path(self, init_path):
        #this is only for finished runs
        self.initialized = 0
        self['run_path'] = init_path
        self['run_name'] = os.path.basename(init_path)
        self['incar'] = incar(os.path.join(init_path+'/INCAR'))
        self['kpoints'] = kpoints(os.path.join(init_path+'/KPOINTS'))
        self['potcar'] = potcar(os.path.join(init_path+'/POTCAR')) 
        self['poscar'] = poscar(os.path.join(init_path+'/POSCAR'))
        self.initialized = 1 #lock down values again

class calculation_input(whole_input):
    def __init__(self):
        whole_input.__init__(self)
        self.allowed=['calc_name','calc_path','append']

    def ensure_append(self):
        if 'append' not in self.keys():
            self['append'] = "" #this allows string concatenation to occur

class calcgroup_input(whole_input):
    def __init__(self):
        whole_input.__init__(self)
        self.allowed=['calcgroup_name','calcgroup_path']

class crystal: #poscar generator

    def make_fcc(self,a,x,y,z,symbol): #returns final poscar file
        a = str(a)
        x = str(x)
        y = str(y)
        
        while(len(a) < 12):
            a += '0'
            
        filename = os.path.join(temp_path,"primitive_POSCAR"+symbol)

        writef = open(filename,'wb')
        writef.write(symbol + '\n')
        writef.write("1.0000000000\n")
        writef.write(a + " 0.0000000000 0.0000000000\n")
        writef.write("0.0000000000 " + a + " 0.0000000000\n")
        writef.write("0.0000000000 0.0000000000 " + a + '\n')
        writef.write(symbol + '\n')
        writef.write("4\n")
        writef.write("D\n")
        writef.write("0.0000000000 0.0000000000 0.0000000000\n")
        writef.write("0.0000000000 0.5000000000 0.5000000000\n")
        writef.write("0.5000000000 0.0000000000 0.5000000000\n")
        writef.write("0.5000000000 0.5000000000 0.0000000000\n")    
        writef.close()

        writef = open(os.path.join(temp_path,"matrix"),'wb')
        writef.write(x + " 0 0\n")
        writef.write("0 " + y + " 0\n")
        writef.write("0 0 " + z + '\n')
        writef.close()

        cwd = os.getcwd()
        os.chdir(temp_path)
        arg = "convasp -supercell matrix < primitive_POSCAR" + symbol + " > final_poscar" + symbol
        subprocess.Popen(arg,shell=True)

        os.chdir(cwd)
        file_final = os.path.join(temp_path,"final_poscar"+symbol)
        
        return poscar(file_final)
