#!/usr/bin/env python
##
#@package   Velasticmodel
#@ingroup   other
#@ingroup   python
#@brief     From epitaxial strain of cube, get Young's modulus and 
#           Poisson's ratio
#@author    Tam Mayeshiba
#@date      11/18/2012
###

def main(mydir, zerovol, zeroenergy, flaglist="", zerodir=""):

    """
    Epitaxial strain fitting for 2*Youngs modulus/(3*(1-Poissons ratio))
    Assumes equal epitaxial strain.
    Assumes that the unstrained lattice is approximately cubic
    INPUTS:
        mydir <str> = directory
        zerovol <float: deprecated> = zero-strain approx. cubic volume
        zeroenergy <float: deprecated> = zero-strain approx. migration energy
        flaglist <str> = space-delimited list of values to include (designate as 1's) or exclude (designate as 0's)
        zerodir <str> = directory of zero bulk static
    """
    import Vgetenergy
    import Vgetvolume
    import Vgettotmom
    import Platticevec
    import Pgetline
    import math
    import os
    import sys
    import pdb
    import Xgetdirs
    import numpy
    #
    mylistall=[]
    mylist=[]
    edenslist=[]
    poissonlist=[] #using approx of poisson's ratio approx delta z / delta x 
    xstrainlist=[]
    ystrainlist=[]
    zstrainlist=[]
    ymodlist=[]
    maglist=[]
    mylistall = Xgetdirs.main(mydir, 1, 5, "*bulk_static")
    alldir=""
    for alldir in mylistall:
        if not os.path.isfile(alldir + "/OUTCAR"):
            pass
        elif Pgetline.main(alldir, "OUTCAR", "User", "X") == None:
            pass
        else:
            mylist.append(alldir)
    onedir=""
    if mylist=="" or mylist==None or mylist==[]:
        print "Empty list. Exiting."
        return None
    myvol=""
    myenergy=""
    mymag=""
    flag=0
    dlenx=0
    dleny=0
    dlenz=0
    xlen=0
    ylen=0
    zlen=0
    epsx=0
    prhs=0
    plhs=0
    poisson=0
    latticeparams=[]
    zerolattice=0
    mypoisson=0
    zerolatt=Platticevec.main(zerodir, "CONTCAR")
    zerovol = Vgetvolume.main(zerodir)
    zeroenergy = Vgetenergy.main(zerodir)
    for onedir in mylist:
        myvol=Vgetvolume.main(onedir)
        myenergy=Vgetenergy.main(onedir)
        mymag=Vgettotmom.main(onedir)
        edenslist.append((float(myenergy)-float(zeroenergy))/float(myvol))
        if not (mymag == None):
            maglist.append(float(mymag))
        else:
            maglist.append(0)
        latticeparams=Platticevec.main(onedir,"CONTCAR")
        if latticeparams==None:
            print "Error getting lattice parameters for ",onedir
            flag=1
        if flag==1:
            poissonlist.append(0)
            xstrainlist.append(0)
            ystrainlist.append(0)
            zstrainlist.append(0)
            flag=0
        else:
            try:
                zerovol = float(zerovol)
            except (TypeError, ValueError):
                print "Could not convert volume."
                return None
            zerolattice = math.pow(zerovol,0.333333333333)
            xlen = float(latticeparams[0][0])
            ylen = float(latticeparams[1][1])
            zlen = float(latticeparams[2][2])
            dlenx = xlen-zerolatt[0][0]
            dleny = ylen-zerolatt[1][1]
            dlenz = zlen-zerolatt[2][2]
            # as though apply stress in z
            if math.fabs(dlenx) < 0.005:
                poissonlist.append(0)
            else:
                prhs = 1 - ((math.fabs(dlenz)/4)/zerolatt[2][2])
                plhs = 1 + (math.fabs(dlenx)/zerolatt[0][0])
                prhs = math.log(prhs)
                plhs = math.log(plhs)
                mypoisson = -1 * prhs / plhs
                poissonlist.append(mypoisson) #trans delta L = 2*dlenz; and stretching on 2 axes is another factor of 2?
                #poissonlist.append(-1*(dlenz/4)/dlenx) #trans delta L = 2*dlenz; and stretching on 2 axes is another factor of 2?
            #poissonlist.append(0.333333333333) #estimate
            xstrainlist.append(dlenx/zerolatt[0][0])
            ystrainlist.append(dleny/zerolatt[1][1])
            zstrainlist.append(dlenz/zerolatt[2][2])
    yavg=0
    vavg=0
    twoyover31minusv=0
    idx=0
    okays=[]
    idx = 0
    yfit=""
    eokayx=[]
    eokayy=[]
    eokayz=[]
    uokay=[]
    vokay=[]
    if flaglist=="":
        while idx < len(edenslist):
            okays.append(1)
            idx = idx + 1
        idx = 0
    else:
        okays = flaglist.split()
        while idx < len(okays):
            okays[idx] = int(okays[idx])
            idx = idx + 1
        idx = 0
    print "Okays: ",okays
    if not (len(okays) == len(edenslist)):
        print "Error in flag list size."
        return None
    while idx < len(edenslist):
        if not (int(okays[idx])==0):
            uokay.append(edenslist[idx])
            eokayx.append(xstrainlist[idx])
            eokayy.append(ystrainlist[idx])
            eokayz.append(zstrainlist[idx])
            if not (poissonlist[idx] == 0):
                vokay.append(poissonlist[idx])
        idx = idx + 1
    poissonmat = numpy.mat(vokay)
    poisson=0
    poisson = numpy.mean(poissonmat)
    #poisson = 0.3333333333 #TTM DEBUG SUBSTITUTE POISSON
    numer=0
    denom=0
    strsq=0
    idx=0
    ymodlist2=[]
    while idx < len(mylist):
        strsq = math.pow(xstrainlist[idx],2)
        numer = edenslist[idx]*2*(1-math.pow(poisson,2))         
        denom = strsq + strsq + 2*poisson*strsq
        if numer==0 or denom==0:
            ymodlist.append(0)
            ymodlist2.append(0)
        else:
            ymodlist.append(numer/denom)
            ymodlist2.append((edenslist[idx]/strsq) * (1-poisson))
        idx = idx + 1
    print "Dir list: ",mylist
    print "Magmom list: ",maglist
    print "Poisson list: ",poissonlist
    print "Poisson: ",poisson
    print "E dens list: ",edenslist
    print "x strain list: ",xstrainlist
    print "y strain list: ",ystrainlist
    print "z strain list: ",zstrainlist
    print "Okays: ",okays
    yfit = numpy.polyfit(eokayx, uokay, 2)
    ymod = yfit[0]*(1-poisson)
    twoyover31minusv = 2*ymod/(3*(1-poisson))
    print "2D model, strainxx=strainyy: ", yfit
    print "2/3 * (Y/(1-v)): ",str(0.66666666 * yfit[0])
    #3Dstrain U_0 = E/(2(1+v)(1-2v))((1-v)(epsx^2 + epsy^2 + epsz^2) + 2v(epsxepsy + epsyepsz + epsxepsz))
    efflist=[]
    effval=""
    idx = 0
    while idx < len(eokayx):
        effval = ((1-poisson) * (math.pow(eokayx[idx],2) + math.pow(eokayy[idx],2) + math.pow(eokayz[idx],2))) + (2*poisson*((eokayx[idx]*eokayy[idx]) + (eokayy[idx]*eokayz[idx]) + (eokayx[idx]*eokayz[idx])) )
        efflist.append(effval)
        idx = idx + 1
    arrayeff = numpy.array(efflist)[numpy.newaxis].transpose()
    arrayu = numpy.array(uokay)[numpy.newaxis].transpose()
    print "Arrayeff: ",arrayeff
    print "ArrayU: ",arrayu
    yfitlin = numpy.linalg.lstsq(arrayeff, arrayu) #linear fit to coeff E/(2(1+v)(1-2v))
    print "3D model: ",yfitlin
    youngsover = yfitlin[0][0][0]
    print "E/(2(1+v)(1-2v)): ", youngsover
    otheryoung = float(youngsover) * 2*(1+poisson)*(1-(2*poisson))
    otheryoungover = 0.6666666*otheryoung/(1-poisson)
    print "3D: (2/3)(Y/(1-v)): ",otheryoungover
    # linear fit
    linlistx=[]
    idx = 0
    while idx < len(eokayx):
        linlistx.append(math.fabs(eokayx[idx]))
        idx = idx + 1
    arraylinx = numpy.array(linlistx)[numpy.newaxis].transpose()
    arrayu = numpy.array(uokay)[numpy.newaxis].transpose()
    #print "Arraylinx: ",arraylinx
    #print "ArrayU: ",arrayu
    yfitlinlistx = numpy.linalg.lstsq(arraylinx, arrayu) #linear fit
    #print "Yfitlinlistx: ",yfitlinlistx
    simplelin = yfitlinlistx[0][0][0]
    print "Simplelincoeff: ", simplelin 
    simpleover=""
    simpleover = 0.666666666* 100*simplelin / (1-poisson)
    print "(2/3)*100*(Simple)/(1-v)", simpleover
    #return printing
    retstr=""
    retstr = retstr + " :MAG: "
    idx=0
    while idx < len(maglist):
        retstr = retstr + str(maglist[idx]) + " "
        idx = idx + 1
    retstr = retstr + " :strainxx: "
    idx=0
    while idx < len(xstrainlist):
        retstr = retstr + str(xstrainlist[idx]) + " "
        idx = idx + 1
    retstr = retstr + " :strainyy: "
    idx=0
    while idx < len(ystrainlist):
        retstr = retstr + str(ystrainlist[idx]) + " "
        idx = idx + 1
    retstr = retstr + " :strainzz: "
    idx=0
    while idx < len(zstrainlist):
        retstr = retstr + str(zstrainlist[idx]) + " "
        idx = idx + 1
    retstr = retstr + " :Energdens: "
    idx=0
    while idx < len(edenslist):
        retstr = retstr + str(edenslist[idx]) + " "
        idx = idx + 1
    retstr = retstr + " :Okay_is_1: "
    idx=0
    while idx < len(okays):
        retstr = retstr + str(okays[idx]) + " "
        idx = idx + 1
    retstr = retstr + " :Y_in_eV/A^3: " + str(ymod)    
    #retstr = retstr + "Poisson's ratio: " + str(vavg) + " "
    retstr = retstr + ":2Y/3(1-v): " + str(twoyover31minusv) + " "
    retstr = retstr + ":2Y/3(1-v) 3D model: " + str(otheryoungover)
    retstr = retstr +":E/(2(1+v)(1-2v)): " + str(youngsover)
    retstr = retstr + ":Lin fit: " + str(simplelin)
    retstr = retstr + ":Poisson:" + str(poisson)
    return retstr
