#!/usr/bin/env python
##
#@package   Vgetstrain
#@ingroup   other
#@ingroup   python
#@brief     Get strain tensor from activated stress tensor (assume only principal stress) and energy difference
#@author    Tam Mayeshiba
#@date      11/18/2012
###

def main(mydir, initstring, actstring, verbose=0):

    """
    Get strain tensor from activated stress tensor and energy difference between act and init state
    Assumes equal epitaxial strain.
    Assumes that the unstrained lattice is approximately cubic
    INPUTS:
        mydir <str> = directory housing strain series
        initstring <str> = string to match initial directories; include asterices for wildcards
        actstring <str> = string to match activated directories; include asterices for wildcards
        verbose <int> = 0 - do not print extra information (default)
                        1 - print extra information
    """
    import Vgetenergy
    import Vgetvolume
    import Vgetstress
    import Platticevec
    import scipy
    import math
    import os
    import sys
    import pdb
    import Xgetdirs
    import numpy
    from scipy import linalg
    #
    mylist=[]
    mylist = Xgetdirs.main(mydir, 1, 1)
    strdir=""
    actdirs=[]
    initdirs=[]
    for strdir in mylist:
        actlist=[]
        actlist = Xgetdirs.main(strdir, 1, 5, actstring)
        initlist=[]
        initlist = Xgetdirs.main(strdir, 1, 5, initstring)
        if len(actlist) == 1 and len(initlist) == 1:
            actdirs.append(actlist[0])
            initdirs.append(initlist[0])
        else:
            print "Ambiguity finding folders under ", strdir, ". Skipping."
    numpairs = len(actdirs)
    idx = 0
    stressstring=""
    stresslist=[]
    strstring=""
    ediffstr=""
    myvol=""
    volstr=""
    if numpairs == 0:
        print "No pairs found in ", mydir
        return None
    while idx < numpairs:
        stressstring = Vgetstress.main(initdirs[idx],actdirs[idx])
        if stressstring == None:
            print "Error getting stress. Skipping."
        else:
            stresslist = stressstring.split(" ", 1) # energy diff, sigma11, sigma22, sigma33
            ediffstr = ediffstr + str(stresslist[0]) + ";"
            strstring = strstring + str(stresslist[1]) + ";"
            myvol = Vgetvolume.main(actdirs[idx])
            volstr = volstr + str(myvol).strip() + ";"
        idx = idx + 1
    strstring = strstring[:-1] #remove last ";"
    ediffstr = ediffstr[:-1] #remove last ";"
    volstr = volstr[:-1]
    stressmat = scipy.mat("[" + strstring + "]",float)
    ediffmat = scipy.mat("[" + ediffstr + "]",float)
    volmat = scipy.mat("[" + volstr + "]", float)
    # using deltaH = -V * [delta sigma tensor * strain tensor] or equiv:
    #       deltaH = -V * [sigma tensor * delta strain tensor]
    ediffmatovervol=-1*ediffmat/volmat
    myreturn=linalg.lstsq(stressmat, ediffmatovervol)
    eps11=myreturn[0][0][0]
    eps22=myreturn[0][1][0]
    eps33=myreturn[0][2][0]
    resid=myreturn[1][0]
    if verbose == 1:
        print "Initial dirs: ", initdirs
        print "Activated dirs: ", actdirs
        print "Activated stress mat: ", stressmat
        print "Energy differences: ", ediffmat
    return "eps11: " + str(eps11) + " eps22: " + str(eps22) + " eps33: " + str(eps33) + " resid: " + str(resid)

