#!/usr/bin/env python
# 9/30/11 convention: any class taking input will have 
#           classname_input as the input class
#         - class convention: derived class xxx_ancestorname
#         - instance name convention: xxx_[2 char from ancestorname]
import os
import shutil
import subprocess
import util_tom
import time
import math

#Path to temporary files used (ensures cross-user consistency)
temp_path = os.path.expanduser('~/bin/tscripts/temps')

class control_parameters:
    encomp = 0.001
    entype = 'T'
    volcomp = 1
    voltype = 'P'
    maxout = 5

class file:
    data = []

    def __init__(self,file_path):
        self.from_file(file_path)

    def from_file(self,file_path):
        self.data = []
        readf = open(file_path,'rb')
        lines = readf.readlines()
        for line in lines:
            self.data.append(line)
        readf.close()    
    
    def to_file(self,file_path):
        writef = open(file_path,'wb')
        for line in self.data:
            writef.write(line)
        writef.close()

    def to_stdout(self):
        for line in self.data:
            print line,

class input:
    path = ""

    def reconcile_path(self):
        self.path = os.path.expanduser(self.path)
        self.path = os.path.abspath(self.path)

class run_input(input):
    incar = None
    poscar = None
    kpoints = None
    potcar = None
    script = None

#TTM+1 10/3/11 renamed relax_input to calculation_input
class calculation_input(input):
    calc_name = None #TTM 10/3/11 added name so can name runs other than hardcoded 'bulk' hardcoded
    control_param = control_parameters()
    run_in = run_input()
    static_incar = None
    num_im = 1 # TTM 10/3/11 added: number of images, default to 1
    ep1 = None # expect run class # TTM 10/3/11 added
    ep2 = None # expect run class # TTM 10/3/11 added

#TTM+1 10/3/11 renamed relax class to 'calculation' ancestor class, removed update()
class calculation:
    calc_in = None
    run_count = 0
    run = []
#    bulk_completed = False #TTM 10/3/11 removed; specific to relax_calculation
    completed = False
    static_phase = False
    
    def __init__(self, i_input):
        self.calc_in = i_input
        self.calc_in.reconcile_path()
        
        if(os.path.exists(self.calc_in.path)):
            print 'Directory at location ' + self.calc_in.path + ' already exists.'
            return
        
        os.mkdir(self.calc_in.path)        
        self.calc_in.run_in.path = os.path.join(self.calc_in.path, self.calc_in.calc_name + str(self.run_count))) #TTM 10/3/11 use calc_name
        self.run.append(run(self.calc_in.run_in))

    def current_run(self):
        return self.run[self.run_count]
    
    #TTM 10/3/11 make iterate_path() function
    def iterate_path(self):
        self.calc_in.run_in.path = os.path.join(self.calc_in.path, self.calc_in.calc_name + str(self.run_count))


#TTM+1 10/3/11 relax class as inherited calculation class; remove __init__ and current_run functions; all relax_in changed to calc_in
class relax_calculation(calculation):
#    relax_in = None
#    run_count = 0
#    run = []
    bulk_completed = False
#    completed = False
#    static_phase = False
    
#TTM+block 10/3/11 removed
#    def __init__(self, i_input):
#        self.relax_in = i_input
#        self.relax_in.reconcile_path()
#        
#    if(os.path.exists(self.relax_in.path)):
#            print 'Directory at location ' + self.relax_in.path + ' already exists.'
#            return
#        
#        os.mkdir(self.relax_in.path)        
#        self.relax_in.run_in.path = os.path.join(self.relax_in.path, self.relax_in.calc_name + str(self.run_count)) #TTM 10/3/11 use calc_name
#        self.run.append(run(self.relax_in.run_in))

    def update(self):
        self.current_run().update_status()
        if(self.current_run().check_completion()):
            print "Run number " + str(self.run_count) + " of the " + self.calc_in.calc_name + " module completed in "+ str(self.current_run().life_time) + " seconds with an energy of " + str(self.current_run().energy) ## TTM 10/3/11 use calc_name instead of hardcoded 'bulk'
            if(self.run_count == 0):
                self.calc_in.run_in.poscar = file(self.current_run().contcar)
                self.run_count += 1
                self.iterate_path() #TTM 10/3/11 use iterate_path()
                self.run.append(run(self.calc_in.run_in))
            else:

                if not self.bulk_completed:
                    last_energy = self.run[self.run_count-1].energy
                    last_volume = self.run[self.run_count-1].volume
                    current_energy = self.current_run().energy
                    current_volume = self.current_run().volume
                
                    delta_energy = math.fabs(current_energy-last_energy)
                    delta_volume = math.fabs(current_volume-last_volume)
                    energy_percent_change = (delta_energy/last_energy)*100
                    vol_percent_change = (delta_volume/last_volume)*100
                
                    self.bulk_completed = True
                
                    if(self.calc_in.control_param.entype == 'T' or self.calc_in.control_param.entype == 't'):
                        if(delta_energy > self.calc_in.control_param.encomp):
                            self.bulk_completed = False
                    elif(self.calc_in.control_param.entype == 'P' or self.calc_in.control_param.entype == 'p'):
                        if(energy_percent_change > self.calc_in.control_param.encomp):
                            self.bulk_completed = False
                    if(self.calc_in.control_param.voltype == 'T' or self.calc_in.control_param.voltype == 't'):
                        if(delta_volume > self.calc_in.control_param.volcomp):
                            self.bulk_completed = False
                    elif(self.calc_in.control_param.voltype == 'P' or self.calc_in.control_param.voltype == 'p'):
                        if(vol_percent_change > self.calc_in.control_param.volcomp):
                            self.bulk_completed = False
                        
                if not self.bulk_completed:
                    self.calc_in.run_in.poscar = file(self.current_run().contcar)
                    self.run_count += 1
                    self.iterate_path() #TTM 10/3/11 use iterate_path
                    self.run.append(run(self.calc_in.run_in))
                    
                elif not self.completed and not self.static_phase:
                    self.static_phase = True
                    self.calc_in.run_in.poscar = file(self.current_run().contcar)
                    self.run_count +=1
                    self.calc_in.run_in.incar = self.calc_in.static_incar
                    self.iterate_path() #TTM 10/3/11 use iterate_path
                    self.run.append(run(self.calc_in.run_in))
                else:
                        self.completed = True
                                
    def current_run(self):
        return self.run[self.run_count]
    
class neb_calculation(calculation):
    #TTM 10/3/11 remove here; inherited: relax_in = None
    #TTM 10/3/11 remove here; inherited: run_count = 0 
    #TTM 10/3/11 remove here; inherited: run = []
    images_relaxed = False
    local_paths = []
    #TTM 10/3/11 remove here; inherited: completed = False
    #TTM 10/3/11 remove here; inherited: static_phase = False
   
    #TTM+block 10/3/11 override init so run does not start automatically
    def __init__(self, i_input):
        self.calc_in = i_input
        self.calc_in.reconcile_path()
        
        if(os.path.exists(self.calc_in.path)):
            print 'Directory at location ' + self.calc_in.path + ' already exists.'
            return
        
        os.mkdir(self.calc_in.path)        
        self.set_up()

    def set_up(self):
        # copy endpoint CONTCARs
        shutil.copy(self.calc_in.ep1.contcar, self.calc_in.path + '/CONTCARep1')
        shutil.copy(self.calc_in.ep2.contcar, self.calc_in.path + '/CONTCARep2')
        # run_in should take care of KPOINTS, POTCAR, and INCAR (somehow...)
        convintstr=""
        convintstr='nice -n 19 convasp -intpol CONTCARep1 CONTCARep2'
        convintstr = convintstr + ' ' + str(self.calc_in.num_im) + ' n'
        oldwd=""
        oldwd = os.getcwd()
        os.chdir(self.calc_in.path)
        makeme = subprocess.Popen(convintstr, shell=True)
        makeme.wait()
        os.chdir(oldwd)
        # put OSZICARs into endpoint folders
        shutil.copy(self.calc_in.ep1.oszicar, self.calc_in.path + '/00/OSZICAR')
        shutil.copy(self.calc_in.ep2.oszicar, self.calc_in.path + '/0' + str(self.calc_in.num_im + 1) + '/OSZICAR')
        # set run count to 1 for image 1
        self.run_count = 1
        # set up run classes for images
        idx = 1
        while (idx <= self.calc_in.num_im):
            self.calc_in.run_in.path = self.calc_in.path + '/0' + str(self.calc_in.idx)
            run_list.append(image_run(self.calc_in.run_in))  #image run class: set up, but don't run
            idx = idx + 1
        
    def update(self):
        idx = 0
        upperlim = 0
        if self.static_phase == False:
            idx = 1
            while (idx <= self.calc_in.num_im):
                self.run_list[idx - 1].update_status()
                if not (self.run_list[idx - 1].check_completion()):
                    self.images_relaxed = False
                idx = idx + 1
            if self.images_relaxed:
                self.static_phase = True
                idx = 1
                # set up new calc folder for statics and copy over endpoints
                newpath=""
                newpath = self.calc_in.path + '_static'
                if (os.path.exists) newpath:
                    print 'Static path at ', newpath, 'already exists.'
                    return None
                else:
                    os.mkdir(newpath)
                shutil.copytree(self.calc_in.path + '/00', newpath + '/static0'
                shtuil.copytree(self.calc_in.path + '/0' + str(self.calc_in.num_im), newpath + '/static' + str(self.calc_in.num_im)
                self.calc_in.path = newpath
                # print relaxed image results and set up the static images
                self.calc_in.run_in.incar = self.calc_in.static_incar
                while (idx <= self.calc_in.num_im):
                    print "Run " + self.run_list[idx - 1].path + statusline + str(self.current_run().life_time) + " seconds with an energy of " + str(self.current_run().energy) ##
                    self.calc_in.run_in.path = self.calc_in.path + '/static' + str(idx)
                    self.run_list.append(run(self.calc_in.run_in))        
                    idx = idx + 1
        else:
            idx = self.calc_in.num_im + 1
            upperlim = int(2 * self.calc_in.num_im)
            while (idx <= upperlim):
                if (self.current_run().check_completion()):
                    print "Run " + str(self.run_list.path) + statusline + str(self.current_run().life_time) + " seconds with an energy of " + str(self.current_run().energy) ##
                else:
                    self.completed = False

#TTM+block created inherited class image_run, override init not to run automatically
class image_run(run):
    def __init__(self,i_input_files):
        self.status = "N"
        self.run_in = i_input_files
        self.run_in.reconcile_path()
        
        if(os.path.exists(self.run_in.path)):
            print 'Run at location ' + self.run_in.path + ' already exists. Exiting.'
            return
        
        os.mkdir(self.run_in.path)
        self.run_in.incar.to_file(self.run_in.path+'/INCAR')
        self.run_in.poscar.to_file(self.run_in.path+'/POSCAR')
        self.run_in.kpoints.to_file(self.run_in.path+'/KPOINTS')
        self.run_in.potcar.to_file(self.run_in.path+'/POTCAR')
        self.run_in.script.send_to_file(self.run_in.path+'/parallel.sh')

        cwd = os.getcwd()
        os.chdir(self.run_in.path)
    


class run:
    '''
    Encapsulation of a run in VASP - a directory with all of the input files along
    with the submission script. The run object can be queried with regard to the
    state of the run, the time elapsed, and so forth.
    '''
    status = ""     #( N (null), Q (queued), E (errored), R (running), C (complete) )
    run_in = None
    contcar = ""
    outcar = ""
    oszicar = "" #TTM 10/3/11 added
    jobid = -1
    energy = 0
    volume = 0
    submit_time = 0 #time of submission in seconds since the first epoch
    life_time = 0   #time of existence of run in seconds
    
    def __init__(self,i_input_files):
        self.status = "N"
        self.run_in = i_input_files
        self.run_in.reconcile_path()
        
        if(os.path.exists(self.run_in.path)):
            print 'Run at location ' + self.run_in.path + ' already exists. Exiting.'
            return
        
        os.mkdir(self.run_in.path)
        self.run_in.incar.to_file(self.run_in.path+'/INCAR')
        self.run_in.poscar.to_file(self.run_in.path+'/POSCAR')
        self.run_in.kpoints.to_file(self.run_in.path+'/KPOINTS')
        self.run_in.potcar.to_file(self.run_in.path+'/POTCAR')
        self.run_in.script.send_to_file(self.run_in.path+'/parallel.sh')

        cwd = os.getcwd()
        os.chdir(self.run_in.path)
        submitted = subprocess.Popen('qsub ' + 'parallel.sh',shell=True,stdout=subprocess.PIPE)
        submitted.wait()
        self.jobid = util_tom.extractJobid(submitted.communicate()[0])
        os.chdir(cwd)
        self.submit_time = time.time()
        
    def queue_status(self):
        queue = subprocess.Popen('qstat -a ' + str(self.jobid) + ' 2> /dev/null',shell=True, stdout = subprocess.PIPE)
        queue.wait()
        queue_output = queue.communicate()[0]
        
        begin = queue_output.find(str(self.jobid))
        if(begin == -1):
            return None
        
        queue_output = queue_output[begin:]
        end = queue_output.find('\n')
        queue_output = queue_output[:end]
        output = util_tom.compressString(queue_output)
        final_out = output.split(' ')[9]    
        return final_out
                
    def update_status(self):
        qstatus = self.queue_status()
        self.life_time = time.time() - self.submit_time
        self.energy = util_tom.getEnergy(self.run_in.path+'/OSZICAR')
        self.volume = util_tom.getVolume(self.outcar)
        
        if(qstatus == 'R'):
            self.status = 'R'
            self.contcar = self.run_in.path + '/CONTCAR'
            self.outcar = self.run_in.path + '/OUTCAR'
            self.oszicar = self.run_in.path + '/OSZICAR'
            if not (os.path.isfile(self.outcar)) and self.life_time > 60:
                print "Error: Run is running on queue but no OUTCAR file is found in top directory)"
        elif(qstatus == 'Q'):
            self.status = 'Q'
        elif(qstatus == None):
            self.status = 'C'
            
    def check_completion(self):
        if not os.path.isfile(self.outcar):
            return False
        readf = open(self.outcar,'rb')
        for line in readf.readlines():
            if "User time (sec)" in line:
                readf.close()
                return True
            
        readf.close()
        return False
            
    def delete(self):
        termination = subprocess.Popen('qdel ' + str(self.jobid),shell=True,stdout=subprocess.PIPE)
        termination.wait()
        print 'Run number ' + str(self.jobid) + ' terminated.'
        time.sleep(1.3)
        shutil.rmtree(self.run_in.path)

class submit_script:
    '''
    Encapsulation of the submission scripts used in VASP. All of the
    submission script variables (i.e. queue name, number of nodes...) are
    retained by member variables within this class.
    '''
    
    name = ""                    #name of the run
    queue = ""                   #queue on which to run
    nodes = 0                   
    processors = 0
    memory = 0                   #specify in megabytes
    walltime = 0                 #single integer representing number of hours
    script = ""                  #holds script text to be sent to a .sh file
    vaspexec = os.path.expanduser('/opt/mpiexec/bin/mpiexec ~/bin/vasp5.2_CNEB')
    def __init__(self,name,queue,nodes,processors,memory,walltime):
        self.name = name
        self.queue = queue
        self.nodes = nodes
        self.processors = processors
        self.memory = memory
        self.walltime = walltime
        
        self.script += '#!/bin/bash\n'
        self.script += '#PBS -N ' + name + '\n'
        self.script += '#PBS -q ' + queue + '\n'
        self.script += '#PBS -l nodes=' + str(nodes) + ':ppn=' + str(processors) + ',pvmem=' + str(memory) +'mb\n'
        self.script += '#PBS -l walltime=' + str(walltime) + ':00:00\n'
        self.script += '##PBS -j oe\n'
        self.script += '##PBS -k eo\n'

        self.script += '##PBS -o output.$PBS_JOBID\n'
        self.script += '##PBS -e error.$PBS_JOBID\n'

        self.script += 'NN=`cat $PBS_NODEFILE | wc -l`\n'
        self.script += 'echo "Processors received = "$NN\n'
        self.script += 'echo "script running on host `hostname`"\n'

        self.script += 'cd $PBS_O_WORKDIR\n'
        self.script += 'echo "PBS_NODEFILE"\n'
        self.script += 'cat $PBS_NODEFILE\n'
        self.script += self.vaspexec + '\n'
     
    def send_to_file(self,path=""):
        writef = open(path,'wb')
        writef.write(self.script)
