#!/usr/bin/env python
##
#@package   
#@ingroup   
#@ingroup   python
#@brief     fake Run class
#@author    Tam Mayeshiba
#@date      
###
class Run:
    path=""
    outcarpath=""
    incarpath=""
    status="" #(N, Q, E, R, C)
    jobid=-1
    submit_time=0
    script=None
    inputfile=""
    param_sleepsec=5
    
    name = 'testrun'                    #name of the run
    queue = 'default'                   #queue on which to run
    nodes = 1
    processors = 1
    memory = 2000                   #specify in megabytes
    walltime = 2                 #single integer representing number of hours
    script = ""                  #holds script text to be sent to a .sh file
    vaspexec = "Uwrapper.py ZZfakevasprun"

    def __init__(self, i_path):
        import os
        self.status = 'N'
        self.path = os.path.expanduser(i_path)
        self.path = os.path.abspath(self.path)
        self.outcarpath = os.path.join(self.path, 'OUTCAR')
        self.incarpath = os.path.join(self.path, 'INCAR')
        checkinput=""
        checkinput = os.path.join(self.path, 'inputfile')
        inputread=""
        sleepstr=""
        if os.path.isfile(checkinput):
            inputread = open(checkinput, 'rb')
            sleepstr = inputread.readline()
            self.param_sleepsec = int(sleepstr.strip())
            namestr = inputread.readline()
            self.name = namestr.strip()
            inputread.close()
        if os.path.isfile(self.incarpath):
            pass
        else:
            writef=""
            writef = open(self.incarpath, 'wb')
            writef.write(str(self.param_sleepsec) + '\n')
            writef.write(self.name + '\n')
            writef.close()
        return None
    def complete(self):
        import os
        if os.path.isfile(self.outcarpath):
            return True
        return False
    def energy(self):
        import os
        readout=""
        readmat=[]
        if os.path.isfile(self.outcarpath):
            readout = open(self.outcarpath, 'rb')
            readmat = readout.readlines()
            if len(readmat) > 0:
                return readmat[len(readmat)-1] # just return last value
        return None
    def queue_status(self):
        import subprocess
        queuemat=[]
        qline=""
        statmat=[]
        #import util_tom
        #output = self.path + '/' + str(self.jobid) + 'queuefile'
        #writef = open(output,'wb')
        queue = subprocess.Popen('qstat -a ' + str(self.jobid) + ' 2> /dev/null',shell=True, stdout = subprocess.PIPE)
        queue.wait()
        queuemat = queue.communicate()[0]
        if queuemat == None:
            return None
        for qline in queuemat:
            if str(self.jobid) in qline:
                statmat = qline.split()
                return statmat[9]
        return None

    def update_status(self):
        import os
        qstatus=""
        qstatus = self.queue_status()

        if (qstatus == 'R'):
            self.status = 'R'
            if not os.path.isfile(self.outcarpath):
                print "Error: Run is running on queue but no time file is found in top directory"
        elif (qstatus == 'Q'):
            self.status = 'Q'
        elif (qstatus == None):
            self.status = 'C'

    def create_script(self, path):
        import os
        self.script += '#!/bin/bash\n'
        self.script += '#PBS -N ' + self.name + '\n'
        self.script += '#PBS -q ' + self.queue + '\n'
        self.script += '#PBS -l nodes=' + str(self.nodes) + ':ppn=' + str(self.processors) + ',pvmem=' + str(self.memory) +'mb\n'
        self.script += '#PBS -l walltime=' + str(self.walltime) + ':00:00\n'
        self.script += '##PBS -j oe\n'
        self.script += '##PBS -k eo\n'

        self.script += '##PBS -o output.$PBS_JOBID\n'
        self.script += '##PBS -e error.$PBS_JOBID\n'

        self.script += 'NN=`cat $PBS_NODEFILE | wc -l`\n'
        self.script += 'echo "Processors received = "$NN\n'
        self.script += 'echo "script running on host `hostname`"\n'

        self.script += 'cd $PBS_O_WORKDIR\n'
        self.script += 'echo "PBS_NODEFILE"\n'
        self.script += 'cat $PBS_NODEFILE\n'
        self.script += self.vaspexec + '\n' #TTM 9/28/11 use vaspexec param

        writef = open(os.path.join(self.path,'parallel.sh'),'wb')
        writef.write(self.script)
        writef.close()

