#!/usr/bin/env python
##
#@package   add_coord_tom
#@ingroup   vasp
#@ingroup   python
#@brief     Translates direct coordinates in poscar by given vector
#@author    Tom Angsten
#@date      7/2/11 created
###

def main(line="", x=0.0, y=0.0, z=0.0):
    """
    Purpose: Translate direct coordinates of poscar by vector <x,y,z>
        1. original string of coordinates
        2. x component of translation vector (direct coordinates)
        3. y component
        4. z component

    Returns: string containing new coordinates ready to be inserted into the POSCAR
    """

    import os
    import sys

    xi = yi = zi = 0.0
    
    count = 0
    num = [0, 0, 0]
    
    while(len(line) > 0):
        begin = 0
        end = 0
        step = 0

        for char in line:
            if(step == 0):
                if(char == ' '):
                    begin += 1
                    end += 1
                else:
                    step = 1
            if(step == 1):
                if(char != ' '):
                    end += 1
                else:
                    end += 1
                    break
        num[count] = line[begin:end]
        count += 1
        line = line[end:len(line)]

    xi = float(num[0])
    yi = float(num[1])
    zi = float(num[2])

    dx = float(x)
    dy = float(y)
    dz = float(z)

    xf = yf = zf = 0.0

    xf = xi + dx
    yf = yi + dy
    zf = zi + dz

    if(xf >= 1.0):
        xf -= 1.0
    if(yf >= 1.0):
        yf -= 1.0
    if(zf >= 1.0):
        zf -=1.0

    strxf = '%(number)0.10f' % {"number": xf}
    stryf = '%(number)0.10f' % {"number": yf}
    strzf = '%(number)0.10f' % {"number": zf}
    newline = strxf + " " + stryf + " " + strzf + " "

    return newline

