// **************************************************
// BandFuncs
// **************************************************

using namespace std;
#include <string>
#include <iostream>
#include <iomanip>
#include "UsefulFuncs.h"
#include "projdata.h"

// **************************************************
// CalcPathLength
// **************************************************
void CalcPathLength(const projdata & pd, vector<double> & pathl,
		    vector<double> & dpathl)
{
  matrix<double> rlat=RecipLat(pd.lat);
  pathl[0]=0;
  dpathl[0]=0;
  for (int ik=1;ik<pd.nkpts;ik++){
    vector<double> disp(3);
    disp=VVdiff(pd.kpts[ik],pd.kpts[ik-1]);
    dpathl[ik]=dpathl[ik-1]+norm(disp);
    disp=vecD2C(rlat,disp);
    pathl[ik]=pathl[ik-1]+norm(disp);
  }
}// end


// **************************************************
// PrintBands
// **************************************************
void PrintBands(const projdata & pd)
{
  ofstream outf_up("band.up.out");
  outf_up.precision(5);
  outf_up.setf(ios::fixed,ios::floatfield);
  outf_up.setf(ios::left,ios::adjustfield);
  vector<double> pathl(pd.nkpts);
  vector<double> dpathl(pd.nkpts);
  CalcPathLength(pd,pathl,dpathl);
  for (int ik=0;ik<pd.nkpts;ik++){
    outf_up << pathl[ik] << " " << dpathl[ik];
    for (int ib=0;ib<pd.nbands;ib++){
      outf_up << " " << pd.ener_k_b_u[ik][ib];
    }
    outf_up << "          " << ik;
    for (int ic=0;ic<3;ic++){
      outf_up << " " << pd.kpts[ik][ic];
    }
    outf_up<<endl;
  }//kpt
  if (pd.sp){
    ofstream outf_dn("band.dn.out");
    outf_dn.precision(5);
    outf_dn.setf(ios::fixed,ios::floatfield);
    outf_dn.setf(ios::left,ios::adjustfield);
    for (int ik=0;ik<pd.nkpts;ik++){
      outf_dn << pathl[ik] << " " << dpathl[ik];
      for (int ib=0;ib<pd.nbands;ib++){
	outf_dn << " " << pd.ener_k_b_d[ik][ib];
      }
      outf_dn << "          " << ik;
      for (int ic=0;ic<3;ic++){
	outf_dn << " " << pd.kpts[ik][ic];
      }
      outf_dn<<endl;
    }//kpt
  }// if sp
}// end

