// **************************************************
// PrintPDB
// **************************************************
/*  This funtion prints out structural data in a PDB format.
 Note that I don't really know anything about PDB but it seems
 to be an annoyingly column formatted.  Therefore, all the widths
 below must tbe kept as is.  This means that if we have width W
 for variable X, and prec P, then X takes up P+1 for for the decimal
 part and decimal point, and we have only R=W-(P+1)-1=W-P-2 remaining digits
 (the -1 is because if X takes up all of W then you run into 
 the previous field).  So we have the constraints
 Cell vectors: W=9,P=3,R=4 => <=10^5
 Cell angles: W=7,P=2,R=3 => <=10^3 (which always works since angles are 
                                     given as >=0 and <=360)
 Cartesian positions: W=12,8,P=3,R=7,3 => <=10^8,10^4 (>0) 
                                      and <=10^7,10^3 (<0 since you need a
				                       space for the - sign)
 This is all probably fine unless an atom makes it to more negative than
 -999.999.
*/

using namespace std;
#include <cmath>
#include <iomanip>
#include <iostream>
#include <cstdio>
#include <cstdlib>
#include "UsefulFuncs.h"
#include "PrintPDB.h"

void PrintPDB(const structure & str)
{
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  structure sstr=str;

  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  char* title=sstr.GetTitle();
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  int coord_flag=sstr.GetCoordFlag();
  int num_atoms=sstr.GetNumAtoms();
  matrix<double> cpos=sstr.GetCpos();
  vector<string> names=sstr.GetNames();
  vector<string> at_name_vec=sstr.GetAtomNameVec();
  vector<int> at_num_vec;
  at_num_vec=sstr.GetAtomNumVec();

  // Print out data in pdb format
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(3);
  cout << "HEADER    " << title << endl;
  cout << "REMARK   1" << endl;
  cout << "REMARK   PDB Format: Created by convasp from POSCAR file format." << endl;
  cout << "REMARK   1" << endl;
  vector<double> dat(6,0.0);
  dat=Getabc_angles(lat);
  double rad2deg=360/(2*3.1415927);
  cout << "CRYST1" << setw(9) << dat[0] 
       << setw(9) << dat[1] 
       << setw(9) << dat[2];
  cout.precision(2);
  cout << setw(7) << dat[3]*rad2deg 
       << setw(7) << dat[4]*rad2deg 
       << setw(7) << dat[5]*rad2deg;
  // Output space group - Use P1 for now
  cout << setw(11) << "P1" << endl;
  // cout << "              P1" << endl;
  // Coordinates in cartesian
  cout.precision(3);
  for (int i=0;i<num_atoms;i++){
    cout << "ATOM  " << setw(5) << i+1 << setw(4) << names[i].c_str()
   	 << " " << "MOL" << " H" << setw(4) << 0 << " " << "   "
	 << setw(8) << cpos[i][0] 
	 << setw(8) << cpos[i][1] 
	 << setw(8) << cpos[i][2] 
	 << "  1.00" << "  0.00" 
	 << setw(12) << names[i].c_str() << "0" << endl;
  }
  cout << "TER" << endl;
}



