// poscar object. 

using namespace std;
#include <iostream>
#include <sstream>
#include <fstream>
#include <iomanip>
#include "UsefulFuncs.h"
#include "constants.h"
#include "structure.h"
#include "FittingFuncs.h"
#include "assert.h"
#include "matrix.h"
#include "RBFuncs.h"

/*
**************************************************
 Function TotalAtomDist
**************************************************
NOTE: THIS FUNCTION IS OUTDATED BY RBPOSCAR DISP FUNC BELOW.
  This function gets the total distances between 
two structures by taking the sqrt of the sum of 
the squared distances between each atom.  
If path_flag = N/n then the nearest
images are used to measure distance.
*/
double TotalAtomDist(structure str, structure str00, const string & path_flag)
{
  str.ReScale(1);
  str00.ReScale(1);
  matrix<double> cpos=str.GetCpos();
  matrix<double> cpos00=str00.GetCpos();
  matrix<double> lat=str.GetLat();
  int nat=min(cpos.size(),cpos00.size());
  double dtot=0;
  for (int iat=0;iat<nat;iat++){
    vector<double> dp=VVdiff(cpos[iat],cpos00[iat]);
    // If path_flag is n/N then the path is taken between
    // nearest images.  Otherwise path is between the atoms given.
    if (path_flag=="n" || path_flag=="N"){
      vector<double> ddp=vecC2D(lat,dp);
      for (int ic=0;ic<3;ic++){
	ddp[ic]=ddp[ic]-Nint(ddp[ic]);
      }
      dp=vecD2C(lat,ddp);
    }
    dtot=dtot+norm(dp)*norm(dp);
  }
  return sqrt(dtot); 
}


/*
**************************************************
 Function GetRBDir
**************************************************
  This function gets the directories of the rubber
band run.  The directories are assumed to 
go from 00 to 0(nim+1) (or 00 to (nim+1) if nim>8).
*/
vector<string> GetRBDir(const int & nim){
  vector<string> rbdir(nim+2);
  for (int im=0;im<nim+2;im++){
    ostringstream tmp;
    if (im<10) {
      tmp << "0" << im << ends;
    }
    else{
      tmp << im << ends;
    }
    rbdir[im]=tmp.str();
   }


  return rbdir;
}

  
/*
**************************************************
 Function GetRBEner
**************************************************
  This function gets the energy from the directories
of a rubber band run. The energies are looked for 
in the OSZICAR file in the line before the last line.
*/ 
vector<double> GetRBEner(const int & nim)
{
  vector<double> ener((nim+2),0.0);
  system("/bin/rm tmpxx.convasp");


  vector<string> rbdir=GetRBDir(nim);
  for (int im=0;im<nim+2;im++){
    ostringstream cmd;
    cmd << "cd " << rbdir[im].c_str() << ";";  //added c_str() here in v6.0.0


    cmd << " tail -1 OSZICAR | head -1 | awk '{if ($1==\"CG\") {print $4;}} {if ($1==\"DAV:\"){print $3;}} {if ($4==\"E0=\"){print $5;}}' >> ../tmpxx.convasp;" 	<< "cd ..;" << ends; //changed to read in E0 in 6.0.1
    system(cmd.str().c_str());  	


    ifstream INF("tmpxx.convasp");
    INF>>ener[im];
    system("/bin/rm tmpxx.convasp");

  }
  return ener;
}

/*
**************************************************
 Function GetRBStruct
**************************************************
  This function gets the structures from a rubber
band run.  The directories are assumed to go from 00 to 0(nim+1) 
(or 00 to (nim+1) if nim>8).  
*/
vector<structure> GetRBStruct(const int & nim){

  vector<structure> str_vec(nim+2);

  vector<string> rbdir=GetRBDir(nim);

  for (int im=0;im<nim+2;im++){

    ostringstream file;
    if (im==0 || im==(nim+1)){ // First and last files are POSCAR 
	
      file << "./" << rbdir[im].c_str() << "/POSCAR" << ends;  //added .c_str() here in v6.0.0 to fix RBanal

    }

    else{

      file << rbdir[im].c_str() << "/CONTCAR" << ends;		//added .c_str() here in v6.0.0

    }

    ifstream INF(file.str().c_str());			
	
    INF >> str_vec[im];		

    INF.close();

    INF.clear();

  }


  return str_vec;
}

/*
**************************************************
 Function GetRBDistCum
**************************************************
  This function gets the distances between images
from the directories of a rubber band run. The initial distance 
for image i is taken by calculating the sum of the Euclidean 
distances between all atoms from image i to image i-1.
The distances are then summed, so that d[i] is the cumulative
distance to structure i.
*/
vector<double> GetRBDistCum(const vector<structure> & str_vec, 
		  const string & path_flag)
{
  int nim = str_vec.size()-2;
  vector<double> d((nim+2),0.0);
  structure str=str_vec[0];
  structure last_str=str_vec[0];
  structure diffstr;
  matrix<double> cm;

  for (int im=0;im<nim+2;im++){
    str=str_vec[im];
    double dist=0.0;
    RBPoscarDisp(last_str,str,diffstr,dist,cm,path_flag);
    d[im]=dist;
    last_str=str;
  }

  // Turn d into an cumulative distance.
  for (int i=1;i<d.size();i++){
    d[i]=d[i]+d[i-1];
  }
  return d;
}

/*
**************************************************
 Function GetRBDistFromStrI
**************************************************
  This function gets the distances between images
from the directories of a rubber band run and a given 
structure, I (dist[j] = strI - str[j]). 
*/
vector<double> GetRBDistFromStrI(const vector<structure> & str_vec, 
		       const structure & strI,
		       const string & path_flag)
{
  int nim = str_vec.size()-2;
  vector<double> d((nim+2),0.0);
  structure diffstr;
  matrix<double> cm;
  for (int im=0;im<nim+2;im++){
    RBPoscarDisp(str_vec[im],strI,diffstr,d[im],cm,path_flag);
  }
  return d;
}

/*
**************************************************
 Function RBPoscarDisp
**************************************************
  This function gets the displacement between two 
POSCAR files (str2-str1).  
*/
void RBPoscarDisp(const structure & str1in, const structure & str2in,
		  structure & diffstr, double & totdist, matrix<double> & cm,
		  const string & path_flag){
  diffstr=str1in;
  structure str1=str1in;
  structure str2=str2in;
  str1.ReScale(1);
  str2.ReScale(1);
  matrix<double> cpos1=str1.GetCpos();
  matrix<double> cpos2=str2.GetCpos();
  matrix<double> lat1=str1.GetLat();
  matrix<double> lat2=str1.GetLat();
  matrix<double> latdiff(3,3);
  cm = matrix<double> (2);
  int nat=min(cpos1.size(),cpos2.size());
  matrix<double> cposdiff(nat,3);
  totdist=0;
  for (int ic=0;ic<3;ic++){
    latdiff[ic]=VVdiff(lat2[ic],lat1[ic]);
  }
  for (int iat=0;iat<nat;iat++){
    vector<double> dp=VVdiff(cpos2[iat],cpos1[iat]);
    // If path_flag is n/N then the path is taken between
    // nearest images.  Otherwise path is between the atoms given.
    if (path_flag=="n" || path_flag=="N"){
      vector<double> ddp=vecC2D(lat1,dp);
      for (int ic=0;ic<3;ic++){
	ddp[ic]=ddp[ic]-Nint(ddp[ic]);
      }
      dp=vecD2C(lat1,ddp);
    }
    totdist=totdist+norm(dp)*norm(dp);
    cposdiff[iat]=dp;
    // tpx I think
    //    Vout(cposdiff[iat],cout);
    //    cout << "  " << iat << endl;
  }
  totdist=sqrt(totdist);
  diffstr.SetAllAtomPos(cposdiff,1);
  //  diffstr.SetLat(latdiff); // Just keep lat=lat1.
  cm[0]=str1.GetMom1();
  cm[1]=str2.GetMom1();
}

/*
**************************************************
 Function PrintRBPoscarDisp
**************************************************
  This function prints out the displacement info for 
2 POSCAR.
*/
void PrintRBPoscarDisp(const structure & diffstr, double & totdist, 
		  matrix<double> & cm, const string & path_flag, 
		  ostream & outf){
  outf << "*******  BEGIN:  Structure 2 - Structure 1  *******" << endl;
  outf << diffstr;
  outf << "*******  END:  Structure 2 - Structure 1  *******" << endl;
  outf << endl;
  outf << "Center of mass of POSCARs: "<< endl;
  outf << "  POSCAR1  ";
  Vout(cm[0],outf);
  outf << "  POSCAR2  ";
  Vout(cm[1],outf);
  outf << "Total Distance: " << totdist << endl;
  outf << "Method (eE=exact, nN=nearest): " << path_flag << endl;
}

/*
**************************************************
 Function PrintRBAnal
**************************************************
  This function prints out analysis of the rubber band
run.
*/
void PrintRBAnal(const int & nim, const string & path_flag, ostream & outf)

{
  int DEFAULT_NPT_FOR_SPLINE=50;
  cout.setf(ios::left, ios::adjustfield);
  cout.setf(ios::fixed, ios::floatfield);
  int w=10;
  int p=4;

  // Get data
  vector<string> dirvec = GetRBDir(nim);
  vector<double> enervec = GetRBEner(nim); 
	

  vector<structure> strvec = GetRBStruct(nim);
	
  vector<double> dcum(nim+2);
  vector<double> d_from_init(nim+2);
  vector<double> d_from_final(nim+2);


  // Print data
  outf << "********* Analysis of rubber band run *********" << endl;

  // Exact flag
  string path_flag_temp="E";
  dcum=GetRBDistCum(strvec,path_flag_temp);
  d_from_init=GetRBDistFromStrI(strvec,strvec[0],path_flag_temp);
  d_from_final=GetRBDistFromStrI(strvec,strvec[nim+1],path_flag_temp);
  outf << "** Here we calculate with distance method (eE=exact, nN=nearest): " 
       << path_flag_temp << " **" << endl;
  outf << "Dir    Dist(Cum)   Dist(00)    Dist(END)   Ener" <<endl;  
  for (int im=0;im<nim+2;im++){
    outf << setprecision(p) << setw(5) << dirvec[im] 
	 << "  " << setw(w) << dcum[im] 
	 << "  " << setw(w) << d_from_init[im] 
	 << "  " << setw(w) << d_from_final[im] 
	 << "  " << setw(w) << enervec[im] << endl;
  }
  // Nearest flag
  path_flag_temp="N";
  dcum=GetRBDistCum(strvec,path_flag_temp);
  d_from_init=GetRBDistFromStrI(strvec,strvec[0],path_flag_temp);
  d_from_final=GetRBDistFromStrI(strvec,strvec[nim+1],path_flag_temp);
  outf << "** Here we calculate with distance method (eE=exact, nN=nearest): " 
       << path_flag_temp << " **" << endl;
  outf << "Dir    Dist(Cum)   Dist(00)    Dist(END)   Ener" <<endl;  
  for (int im=0;im<nim+2;im++){
    outf << setprecision(p) << setw(5) << dirvec[im] 
	 << "  " << setw(w) << dcum[im] 
	 << "  " << setw(w) << d_from_init[im] 
	 << "  " << setw(w) << d_from_final[im] 
	 << "  " << setw(w) << enervec[im] << endl;
  }

  cout << endl;
  cout << "*******  SPLINE FIT *******" << endl;
  outf << "Distance used in spline (eE=exact, nN=nearest): " 
       << path_flag << endl;  
  cout << endl;
  PrintSpline(dcum,enervec,DEFAULT_NPT_FOR_SPLINE,outf);
}  
