/* Version 6.2.0 
   This program does all sorts of stuff related to vasp files.
   See PrintHelp.cc for more info.
*/

using namespace std;
#include <string>
#include <vector>
#include <math.h>
#include <iostream>
#include <fstream>
#include <iomanip>
/*#include <strstream>*/
#include <sstream>
#include "structure.h"
#include "PrintGulp.h"
#include "PrintFindsym.h"
#include "PrintPlaton.h"
#include "PrintXYZ.h"
#include "PrintMSI.h"
#include "PrintACE.h"
#include "PrintPDB.h"
#include "PrintCIF.h"
#include "PrintData.h"
#include "PrintNData.h"
#include "PrintClat.h"
#include "GetPrim.h"
#include "PrintDist.h"
#include "PrintDisp.h"
#include "PrintAngles.h"
#include "PrintXray.h"
#include "PrintShell.h"
#include "matrix.h"
#include "UsefulFuncs.h"
#include "PrintImages.h"
#include "RBFuncs.h"
#include "FittingFuncs.h"
#include "rtparams.h"
#include "GetSuperCell.h"
#include "RayTraceFuncs.h"
#include "PrintHelp.h"
#include "ProjFuncs.h"
#include "BandFuncs.h"
#include "projdata.h"
#include "pdosdata.h"
#include "SumPDOSFuncs.h"
#include "typedefs.h"
#include "ChgFuncs.h"
#include "RDFFuncs.h"
#include "EwaldFuncs.h"
#include "CellReduceFuncs.h"
#include "CompareStrFuncs.h"
#include "GetKmesh.h"
#include "PrintKmesh.h"
#include "potalign.h"
#include "plotMD.h"

using namespace std;

int main(int argc, char *argv[]) {
  int i,j;
  double TOL=1e-5;

  // ALWAYS
//    cerr << "************************************************************************" << endl;
//    cerr << "*                                                                      *" << endl;
//    cerr << "*   convasp V " << CVERSION << "                                  \t*" << endl;
//    cerr << "*                                                                      *" << endl;
//    cerr << "************************************************************************" << endl;

  // Help
   if (argc==1) {
    cout <<"*************************************************************" << endl;
    cout <<"*                       CONVASP  " << CVERSION <<  "                \t    *" << endl;
    cout <<"************************** convasp usage ********************" << endl;
    PrintUsage(cout);
    exit(1);
   } else if(argc==2 && (strcmp(argv[1],"-h")==0 || strcmp(argv[1],"-help")==0)) {
     cout <<"*********************** convasp help ***********************" << endl;
     PrintHelp(cout);
     exit(1);
     //  To convert to different types of POSCAR files
   } else { // Convert to Cartesian
   if (argc==2 && strcmp(argv[1],"-cart")==0) {
     // Read in input file.
     structure str;
     cin >> str;
     // Set coord_flag to Cartesian
      str.SetCoordFlag(1);
      // Output
      cout << str;
    } else { // Convert to Direct
    if (argc==2 && strcmp(argv[1],"-direct")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      // Set coord_flag to Direct (0)
      str.SetCoordFlag(0);
      // Output
      cout << str;
    } else { // Rescale and output.
    if (argc==3 && strcmp(argv[1],"-scale")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      // Rescale
      str.ReScale(atof(argv[2]));
      cout << str;
    } else { // Add names
    if (argc>=2 && strcmp(argv[1],"-names")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      // Set a name vector
      vector<string> names_vec(argc-2);
      for (i=0;i<(argc-2);i++){
	names_vec[i]=argv[i+2];
      }
      str.SetNames(names_vec);
      cout << str;
    } else { // Add numbered names (numnames)
    if (argc>=2 && strcmp(argv[1],"-numnames")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      // Set a name vector
      vector<int> nt=str.GetNumEachType();
      int nat=str.GetNumAtoms();
      vector<string> names_vec(nat);
      int iat=0;
      for (int it=0;it<nt.size();it++){
	for (int ia=1;ia<(nt[it]+1);ia++){
	  string name;
	  if (it<(argc-2)){
	    name=argv[it+2];
	  }
	  else{
	    name="H";
	  }
	  ostringstream ostmp;
	  ostmp<<ia;
	  string num=ostmp.str();
	  names_vec[iat]=name+num;
	  iat++;
	}
      }
      str.SetAllAtomNames(names_vec);
      cout << str;
    } else { // Map into unit cell
    if (argc==2 && strcmp(argv[1],"-incell")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      str.PutInCell();
      cout << str;
    } else { // Map into most compact version
      if (argc==2 && strcmp(argv[1],"-incompact")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      str.PutInCompact();
      cout << str;
    } else { // Map IN sphere
    if (argc==3 && strcmp(argv[1],"-insphere")==0) {
      structure str;
      cin >> str;
      PrintXYZInSphere(str,atof(argv[2])); 
    } else { // Transform to primitive
    if (argc==2 && strcmp(argv[1],"-prim")==0) {
      // Read in input file.
      structure str,pstr;
      cin >> str;
      pstr=GetPrim(str);
      cout << pstr;
     } else { // linear transform of the cell (lattice params and atoms positions)
    if (argc==3 && strcmp(argv[1],"-ltcell")==0) {
      // Read in input file.
      structure str,ltstr;
      cin >> str;
      ifstream infile(argv[2]);
      matrix<double> lt(3,3);
      for (int i=0;i<3;i++){
	for (int j=0;j<3;j++){
	  infile >> lt[i][j];
        }
      } 
     ltstr=GetLTCell(lt,str);
     cout << ltstr;
   } else { // linear transform of the cell (lattice params and atoms positions) around fixed vector by angle phi
    if (argc==6 && strcmp(argv[1],"-ltcellfv")==0) {
      // Read in input file.
      structure str,ltstr;
      cin >> str;
      vector<double> nvec(3);
      nvec[0]=atof(argv[2]);nvec[1]=atof(argv[3]);nvec[2]=atof(argv[4]);
      double angle=atof(argv[5])/RAD2DEG;
     ltstr=GetLTFVCell(nvec,angle,str);
     cout << ltstr;
   } else { // supercell
    if (argc==3 && strcmp(argv[1],"-supercell")==0) {
      // Read in input file.
      structure str,scstr;
      cin >> str;
      ifstream infile(argv[2]);
      matrix<double> sc(3,3);
      for (int i=0;i<3;i++){
	for (int j=0;j<3;j++){
	  infile >> sc[i][j];
        }
      } 
     scstr=GetSuperCell(sc,str);
     cout << scstr;
     // Check number of atoms is correct
     // Make sure that is the volume ratio is 
     // an integer then the ration of the number
     // of atoms is the same integer.
     double v=GetVol(str.GetScaledLat());
     double vn=GetVol(scstr.GetScaledLat());
     double vr=vn/v;
     int ratio_is_int=1;
     if (abs(vr-Nint(vr))>TOL) ratio_is_int=0;
     if (ratio_is_int){
       int nat=str.GetNumAtoms();
       int natn=scstr.GetNumAtoms();
       double natr=(double)natn/(double)nat;
       if (abs(natr-vr)>TOL){
	cerr << "WARNING: convasp.cc " << endl;	  
	cerr << "WARNING: Wrong number of atoms in supercell. " << endl;
	cerr << "WARNING: Cell volumes: Orig, Supercell, Supercell/Orig " << v << " " << vn << " " << vr << endl;	  
	cerr << "WARNING: Number of atoms: Orig, Supercell, Supercell/Orig " << nat << " " << natn << " " << natr << endl;	  
	cerr << "WARNING: This probably means the code has a bug - sorry! " << endl;
       }
     }
    } else { // Interpolated images
    if (argc==6 && strcmp(argv[1],"-intpol")==0) {
      // Read in input files.
      structure strA,strB;
      ifstream infileA(argv[2]);
      ifstream infileB(argv[3]);
      infileA >> strA;
      infileB >> strB;
      string path_flag(argv[5]);
      PrintImages(strA,strB,atoi(argv[4]),path_flag);
    } else { // Add selective dynamics
    if (argc>=2 && strcmp(argv[1],"-sd")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      // Set a sd vector
      vector<string> sd(argc-2);
      for (i=0;i<(argc-2);i++){
	sd[i]=argv[i+2];
      }
      str.SetISD(1);
      str.SetSD(sd);
      cout << str;
    } else { // remove selective dynamics
    if (argc==2 && strcmp(argv[1],"-nosd")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      str.SetISD(0);
      cout << str;
    } else { // shift origin
    if (argc>=5 && strcmp(argv[1],"-shift")==0) {
      // Get shift vector
      vector<double> shift(3);
      for (int ic=0;ic<3;ic++){shift[ic]=atof(argv[ic+2]);}
      // argv(4)=c cart,d for direct shift
      int flag;
      if (argc==5){
	cerr << "WARNING: 4th argument can be [cCdD]" << endl;
	cerr << "WARNING: Defaulting to [cC]=Cartesian shift" << endl;
      }
      else{
	if (strcmp(argv[5],"c")==0 | strcmp(argv[5],"C")==0){
	  flag=0;
	}
	else{
	  if (strcmp(argv[5],"d")==0 | strcmp(argv[5],"D")==0){
	    flag=1;
	  }
	  else{
	    cerr << "WARNING: 4th argument can be [cCdD]" << endl;
	    cerr << "WARNING: Defaulting to [cC]=Cartesian shift" << endl;
	    flag=0;
	  } //else
	} //else	  
      }//else
      // Read in input file.
      structure str;
      cin >> str;
      str.ShiftPos(shift,flag);
      cout << str;
    } else { // remove copies
    if (argc>=2 && strcmp(argv[1],"-rm_copies")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      str.RemoveCopies();
      cout << str;
    }else{// Set center of mass
    // convasp -setcm cm1 cm2 cm3 POSCAR
    if (argc==6 && strcmp(argv[1],"-setcm")==0) {
      vector<double> newcm(3);
      for (int ic=0;ic<3;ic++){newcm[ic]=atof(argv[ic+2]);}
      structure str;
      ifstream infile(argv[5]);
      infile >> str;
	str.SetMom1(newcm);  //changed in 6.2.0 to allow scale other than 1 in starting POSCAR
      //vector<double> oldcm=str.GetMom1();
      //vector<double> shift=VVdiff(newcm,oldcm);
      //str.ShiftPos(shift,0);
      cout << str;
    }
    else{// Get Niggli reduced cell
    // convasp -niggli < POSCAR
    if (argc==3 && strcmp(argv[1],"-niggli")==0) {
      structure str;
      ifstream infile(argv[2]);
      infile >> str;
      cout << GetNiggliStr(str);
    }

//  To convert to other file formats

  else { // Creat gulp output file.
    if (argc==2 && strcmp(argv[1],"-gulp")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      PrintGulp(str);
  } else { // Creat FINDSYM output file.            // Stefano Curtarolo
    if ((argc==2 || argc==3) && strcmp(argv[1],"-findsym")==0) {
      double tolerance;
      if (argc==2) tolerance=1.0e-5;
      if (argc==3) tolerance=atof(argv[2]);
      // Read in input file.
      structure str;
      cin >> str;
      PrintFindsym(str,tolerance);
  } else { // Creat PLATON output file.             // Stefano Curtarolo
    if ((argc>=2) && strcmp(argv[1],"-platon")==0) {
      bool Platon_EQUAL=FALSE,Platon_EXACT=FALSE;
      double Platon_ang=1.0,Platon_d1=0.25,Platon_d2=0.25,Platon_d3=0.25;
      //     if (argc==2) Platon_ang=1.0e-2;
      // if (argc==3) Platon_ang=atof(argv[2]);
      // Read in input file.
      structure str;
      cin >> str;
      if((argc>=3 && strcmp(argv[2],"EQUAL")==0) || (argc>=4 && strcmp(argv[3],"EQUAL")==0)) Platon_EQUAL=TRUE;
      if((argc>=3 && strcmp(argv[2],"EXACT")==0) || (argc>=4 && strcmp(argv[3],"EXACT")==0)) Platon_EXACT=TRUE;
      if(argc>=5) {
	Platon_ang=atof(argv[argc-4]);
	Platon_d1=atof(argv[argc-3]);
	Platon_d2=atof(argv[argc-2]);
	Platon_d3=atof(argv[argc-1]);
      }
      PrintPlaton(str,Platon_EQUAL,Platon_EXACT,Platon_ang,Platon_d1,Platon_d2,Platon_d3,cout);
      // convasp -platon [EQUAL] [EXACT] [ang d1 d2 d3]
      // CALC ADDSYM (EQUAL) (EXACT) (ang d1 d2 d3) 
      // EQUAL - Search with all atom type treated as equivalent. 
      // EXACT - All atoms should fit for given criteria. 
      // ang - Angle criterium in search for metrical symmetry of the
      //       lattice (default 1.0 degree). 
      // d1 - Distance criterium for coinciding atoms for non-inversion 
      //      (pseudo)symmetry elements (default 0.25 Angstrom). 
      // d2 - Distance criterium for coinciding atoms for (pseudo)
      //      inversion symmetry (default 0.45, 0.25 Angstrom). 
      // d3 - Distance criterium for coinciding atoms for (pseudo)
      //      translation symmetry (default 0.45, 0.25 Angstrom). 
      // SEE: http://www.cryst.chem.uu.nl/platon/pl000401.html
    } else { // Create XYZ output file.
    if (argc==5 && strcmp(argv[1],"-xyz")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      // Set up superlattice vector.
      vector<int> n(3);
      n[0]=atoi(argv[2]);
      n[1]=atoi(argv[3]);
      n[2]=atoi(argv[4]);
      PrintXYZ(str,n);
    } else { // Rewrite POSCAR in WIGNER SEITZ CELL
    if (argc==2 && (!strcmp(argv[1],"-wignerseitz") || !strcmp(argv[1],"-ws"))) {
      structure str;
      cin >> str;
      // Rescale
      str.SetCoordFlag(1);
      str.WignerSeitz();
      cout << str;
    } else { // Create XYZ output file with WIGNER SEITZ
    if (argc==2 && (!strcmp(argv[1],"-xyzwignerseitz") || !strcmp(argv[1],"-xyzws"))) {
      structure str;
      cin >> str;
      PrintXYZws(str);
    } else { // Ouput msi file.
    if (argc==2 && strcmp(argv[1],"-msi")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      PrintMSI(str); 
    } else { // Output ACE file.
    if (argc==2 && strcmp(argv[1],"-ace")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      PrintACE(str); 
    } else { // Output PDB file.
    if (argc==2 && strcmp(argv[1],"-pdb")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      PrintPDB(str); 
    } else { // Output CIF file.
    if (argc==2 && strcmp(argv[1],"-cif")==0) {
      // Read in input file.
      structure str;
      cin >> str;
      PrintCIF(str); 
    }
  // Structural info

  else { // Output data.
    if (argc==2 && strcmp(argv[1],"-data")==0) { 
      // Read in input file.
      structure str;
      cin >> str;
      PrintData(str); 
    } 
  else { // Output data1.
    if (argc==4 && strcmp(argv[1],"-data1")==0) { 
      // Read in input file.
      double rcut=atof(argv[3]);
      ifstream infile1(argv[2]);
      structure str;
      infile1 >> str;
      PrintData1(str,rcut,cout); 
    } 
    else { // Output normalized data.                        // Stefano Curtarolo
    if (argc==2 && strcmp(argv[1],"-ndata")==0) {
      structure str;
      cin >> str;
      PrintNdata(str); 
    } 
  else { // Compare two structures with cmp_str.
    if (argc==5 && strcmp(argv[1],"-cmp_str")==0) { 
      // Read in input file.
      ifstream infile1(argv[2]);
      ifstream infile2(argv[3]);
      double rcut=atof(argv[4]);
      structure str1;
      structure str2;
      infile1 >> str1;
      infile2 >> str2;
      PrintCmpStr(str1,str2,rcut,cout);
    } 
    else { // compare a1 a2 a3 a4 a5 a6 a7 a8 a9 a10 a11 a12 // Stefano Curtarolo
    if (argc==14 && strcmp(argv[1],"-compare")==0) {
      double aa1=atof(argv[2]),     aa2=atof(argv[3]),     aa3=atof(argv[4]),
 	     aa4=atof(argv[5]),     aa5=atof(argv[6]),     aa6=atof(argv[7]),
	     aa7=atof(argv[8]),     aa8=atof(argv[9]),     aa9=atof(argv[10]),
	     aa10=atof(argv[11]),  aa11=atof(argv[12]),   aa12=atof(argv[13]);
      double bb1,bb2,bb3,bb4,bb5,bb6;
      cout << "PERCENTAGES" << "  ";
      cout << (bb1=100*abs(aa1-aa7)/((aa1+aa7)/2.0)) << "  ";
      cout << (bb2=100*abs(aa2-aa8)/((aa2+aa8)/2.0)) << "  ";
      cout << (bb3=100*abs(aa3-aa9)/((aa3+aa9)/2.0)) << "  ";
      cout << (bb4=100*abs(aa4-aa10)/((aa4+aa10)/2.0)) << "  ";
      cout << (bb5=100*abs(aa5-aa11)/((aa5+aa11)/2.0)) << "  ";
      cout << (bb6=100*abs(aa6-aa12)/((aa6+aa12)/2.0)) << "  ";
      cout << endl;
      cout << "MIN " << min(bb1,min(bb2,min(bb3,min(bb4,min(bb5,bb6))))) << endl;
      cout << "MAX " << max(bb1,max(bb2,max(bb3,max(bb4,max(bb5,bb6))))) << endl;      
    } else { // Output lattice in cartesian coords.
    if (argc==8 && strcmp(argv[1],"-clat")==0) {
      PrintClat(argv); 
    } else { // Output distances
    if (argc==3 && strcmp(argv[1],"-dist")==0) {
      structure str;
      cin >> str;
      PrintDist(str,atof(argv[2])); 
    } else { // Output displacements
    if (argc==3 && strcmp(argv[1],"-disp")==0) {
      structure str;
      cin >> str;
      PrintDisp(str,atof(argv[2])); 
    } else { // Output angles
    if (argc==3 && strcmp(argv[1],"-angle")==0) {
      structure str;
      cin >> str;
      PrintAngles(str,atof(argv[2])); 
    } else { // Output xray scatt
    if (argc==3 && strcmp(argv[1],"-xray")==0) {
      structure str;
      cin >> str;
      PrintXray(str,atof(argv[2])); 
    } else { // Output shell
    if (argc==7 && strcmp(argv[1],"-shell")==0) {
      structure str;
      cin >> str;
      PrintShell(str,atoi(argv[2]),atof(argv[3]),atof(argv[4]),argv[5],atoi(argv[6])); 
    } else { // Output radial distribution function 
    // convasp -rdf rmax nbins sigma
    if (argc==5 && strcmp(argv[1],"-rdf")==0) {
      double rmax=atof(argv[2]);
      int nbins=atoi(argv[3]);
      int smooth_width=atoi(argv[4]);
      structure str;
      cin >> str;
      matrix<double> rdf_all;
      GetRDF(str,rmax,nbins,rdf_all);
      matrix<double> rdf_all_sm;
      rdf_all_sm=GetSmoothRDF(rdf_all,smooth_width);
      matrix<double> rdfsh_all;
      matrix<double> rdfsh_loc; // Radial location of rdf shells.
      GetRDFShells(str,rmax,nbins,smooth_width,rdf_all_sm,rdfsh_all,rdfsh_loc);
      PrintRDF(str,rmax,nbins,smooth_width,rdf_all_sm,rdfsh_all,rdfsh_loc,cout);
    } else { // Compare 2 POSCAR with rdf.
    // convasp -rdfcmp rmax nbins sigma nsh POSCARA POSCARB
    if (argc==8 && strcmp(argv[1],"-rdfcmp")==0) {
      double rmax=atof(argv[2]);
      int nbins=atoi(argv[3]);
      int smooth_width=atoi(argv[4]);
      int nsh=atoi(argv[5]);
      ifstream infileA(argv[6]);
      ifstream infileB(argv[7]);
      structure str_A,str_B;
      infileA >> str_A;
      infileB >> str_B;
      // Get rdfs
      matrix<double> rdf_all_A;
      matrix<double> rdf_all_B;
      GetRDF(str_A,rmax,nbins,rdf_all_A);
      GetRDF(str_B,rmax,nbins,rdf_all_B);
      matrix<double> rdf_all_A_sm=GetSmoothRDF(rdf_all_A,smooth_width);
      matrix<double> rdf_all_B_sm=GetSmoothRDF(rdf_all_B,smooth_width);
      // Get shells
      matrix<double> rdfsh_all_A;
      matrix<double> rdfsh_loc_A; // Radial location of rdf shells.
      matrix<double> rdfsh_all_B;
      matrix<double> rdfsh_loc_B; // Radial location of rdf shells.
      GetRDFShells(str_A,rmax,nbins,smooth_width,
		   rdf_all_A_sm,rdfsh_all_A,rdfsh_loc_A);
      GetRDFShells(str_B,rmax,nbins,smooth_width,
		   rdf_all_B_sm,rdfsh_all_B,rdfsh_loc_B);
      vector<int> best_match;
      matrix<double> rms_mat;
      CmpRDFShells(str_A,str_B,rdfsh_all_A,rdfsh_all_B,nsh,
		   best_match,rms_mat);
      PrintRDFCmp(str_A,str_B,rmax,nbins,smooth_width,nsh,
		  rdfsh_all_A,rdfsh_all_B,best_match,rms_mat,cout);
    } else{ 
    // Distance between two POSCAR files.
    // convasp -rbdist POSCAR1 POSCAR2 e|E|n|N
    if (argc==5 && strcmp(argv[1],"-rbdist")==0) {
      structure strA,strB;
      ifstream infileA(argv[2]);
      ifstream infileB(argv[3]);
      infileA >> strA;
      infileB >> strB;
      string path_flag(argv[4]);
      double totdist;
      matrix<double> cm(2,3,999999);
      structure diffstr;
      RBPoscarDisp(strA,strB,diffstr,totdist,cm,path_flag);
      PrintRBPoscarDisp(diffstr,totdist,cm,path_flag,cout);
    }else{  
    // Center of mass (mom1) of a POSCAR file
    // convasp -mom POSCAR1 
    if (argc==3 && strcmp(argv[1],"-mom")==0) {
      structure str;
      ifstream infile(argv[2]);
      infile >> str;
      vector<double> mom1=str.GetMom1();
      cout.setf(ios::left, ios::adjustfield);
      cout.setf(ios::fixed, ios::floatfield);
      cout << "Moment 1: " 
	   << setw(15) << setprecision(10) << mom1[0]
	   << setw(15) << setprecision(10) << mom1[1]
	   << setw(15) << setprecision(10) << mom1[2]
	   << endl;
    }

  // Vasp Post Processing  

  else { // Output ChgInt
    if (argc==3 && strcmp(argv[1],"-chgint")==0) {
      ifstream chgfile(argv[2]);
      structure str;
      vector<int> ngrid(3);
      // Read in charge
      vector<double> chg_tot;
      vector<double> chg_diff;
      ReadChg(str,ngrid,chg_tot,chg_diff,chgfile);
      // Integrate charge
      vector<matrix<double> > rad_chg_int;
      matrix<double> vor_chg_int;
      GetChgInt(rad_chg_int,vor_chg_int,str,ngrid,chg_tot,chg_diff);
      // Print results
      PrintChgInt(rad_chg_int,vor_chg_int); 
    } else { // Analyse Rubber band run
    if (argc==4 && strcmp(argv[1],"-rbanal")==0) {
      // Read in input files.
      int nim=atoi(argv[2]);
      string path_flag(argv[3]);
      PrintRBAnal(nim,path_flag,cout);
    } else { // Cubic spline fit
    if (argc==3 && strcmp(argv[1],"-spline")==0) {
      // Read in input data.
      vector<double> x,y;
      double p,q;
      char dum[500];
      while(cin >> p >> q){
	x.push_back(p);
	y.push_back(q);
	cin.getline(dum,500); // Get anything else on line.
      }
      int npts=atoi(argv[2]);
      PrintSpline(x,y,npts,cout);
    } else { // Create strlist file from OUTCAR and XDATXAR.
    if (argc==4 && strcmp(argv[1],"-make_strlist")==0) {
      ifstream outcar_inf(argv[2]);      
      InFileExistCheck("convasp",argv[2],outcar_inf,cerr);
      ifstream xdatcar_inf(argv[3]);      
      InFileExistCheck("convasp",argv[3],xdatcar_inf,cerr);
      vector<structure> str_vec=GetStrVecFromOUTCAR_XDATCAR(outcar_inf,xdatcar_inf);
      PrintStrVec(str_vec,cout);
    } else { // Create strlist file from joining two strlist files.
    if (argc==4 && strcmp(argv[1],"-join_strlist")==0) {
      ifstream list1_inf(argv[2]);      
      InFileExistCheck("convasp",argv[2],list1_inf,cerr);
      ifstream list2_inf(argv[3]);      
      InFileExistCheck("convasp",argv[3],list2_inf,cerr);
      vector<structure> str_vec_1,str_vec_2,str_vec_tot;
      ReadInStrVec(str_vec_1,list1_inf);
      ReadInStrVec(str_vec_2,list2_inf);
      JoinStrVec(str_vec_1,str_vec_2,str_vec_tot);
      PrintStrVec(str_vec_tot,cout);
    } else { // Create supercell strlist.
    if (argc==4 && strcmp(argv[1],"-supercell_strlist")==0) {
      ifstream infile(argv[2]);
      InFileExistCheck("convasp",argv[2],infile,cerr);
      matrix<double> sc(3,3);
      for (int i=0;i<3;i++){
	for (int j=0;j<3;j++){
	  infile >> sc[i][j];
        }
      } 
      ifstream list_inf(argv[3]);      
      InFileExistCheck("convasp",argv[2],list_inf,cerr);
      vector<structure> str_vec;
      vector<structure> str_vec_sc;
      ReadInStrVec(str_vec,list_inf);
      SuperCellStrVec(str_vec,sc);
      PrintStrVec(str_vec,cout);
    } else { // Create ray traced jpg (mpg) from structure (structure list).
    if (argc==3 && strcmp(argv[1],"-raytrace")==0) {
      RayTraceManager(argv); // Manage ray tracing related activities.
    } else { // Create projected occupations data
    if (argc==3 && strcmp(argv[1],"-pocc")==0) {
      cerr << "# WARNING: THIS REQUIRES AN ALTERED VERSION OF VASP - SEE convasp -h" << endl;
      int only_occ=1; // Use occupied states only in projections.
      projdata proj_dat;
      proj_dat.PROOUTinfile=argv[2];
      ReadInProj(proj_dat);
      CalcNeatProj(proj_dat,only_occ);
      PrintNeatProj(proj_dat);
    } else { // Create projected DOS data
    if (argc==4 && strcmp(argv[1],"-pdos")==0) {
      cerr << "# WARNING: THIS REQUIRES AN ALTERED VERSION OF VASP - SEE convasp -h" << endl;
      int only_occ=0; // Use unoccupied states in PDOS.
      projdata prd;
      pdosdata pdd;
      prd.PROOUTinfile=argv[3];
      pdd.PDOSinfile=argv[2];
      ReadInProj(prd);
      CalcNeatProj(prd,only_occ);
      ReadInPDOSData(prd,pdd);
      CalcPDOS(prd,pdd);
      pdd.PrintPDOS(cout,prd.sp);
      if (pdd.print_params) pdd.PrintParams(cout,prd.LLMnames);
    } else { // Add up pdos data from vasp
    if (argc==4 && strcmp(argv[1],"-sumpdos")==0) {
      //      cerr << "# WARNING: THIS REQUIRES THAT YOU HAVE PDOS IN DOSCAR FILE - THIS IS OBTAINED BY RUNNING WITH LORBIT=2 - SEE convasp -h" << endl;
      ifstream SumPDOSParams_infile(argv[2]);
      InFileExistCheck("convasp.cc",argv[2],SumPDOSParams_infile,cerr);
      ifstream PDOS_infile(argv[3]);
      InFileExistCheck("convasp.cc",argv[3],PDOS_infile,cerr);
      pdosdata pdd;
      matrix<matrix<double> > allpdos;
      ReadSumDOSParams(SumPDOSParams_infile,pdd);
      //      projdata prd;
      //      pdd.PrintParams(cout,prd.LMnames);
      ReadInPDOSData(allpdos,pdd,PDOS_infile);
      SumPDOS(allpdos,pdd);
      PrintSumPDOS(cout,pdd);
    } else { // Calculate bands
    if (argc==3 && strcmp(argv[1],"-bands")==0) {
      projdata prd;
      prd.PROOUTinfile=argv[2];
      ReadInProj(prd);
      structure str;
      cin >> str;
      prd.lat=str.GetScaledLat();
      PrintBands(prd);
    } else { // Output Planar Charge Density
    if (argc==4 && strcmp(argv[1],"-planedens")==0) {
      // Read in charge
      structure str;
      ifstream chgfile(argv[3]);
      vector<int> ngrid(3);
      vector<double> chg_tot;
      vector<double> chg_diff;
      ReadChg(str,ngrid,chg_tot,chg_diff,chgfile);
      // Read in planar density parameters.
      pd_params pdp;
      ifstream pdfile(argv[2]);
      ReadPlaneDensParams(str,pdp,pdfile);
      // Get planar charge density
      vector<double> dens2d_tot;
      vector<double> dens2d_diff;
      GetPlaneDens(pdp,dens2d_tot,dens2d_diff,str,ngrid,chg_tot,chg_diff);
      PrintPlaneDens(pdp,dens2d_tot,dens2d_diff,str); 
    } else { // Difference of two CHGCAR files.
    if (argc==4 && strcmp(argv[1],"-chgdiff")==0) {
      // Read in CHGCAR files
      structure str1,str2;
      ifstream chgfile1(argv[2]);
      ifstream chgfile2(argv[3]);
      vector<int> ngrid1(3),ngrid2(3);
      vector<double> chg_tot1,chg_tot2;
      vector<double> chg_diff1,chg_diff2;
      ReadChg(str1,ngrid1,chg_tot1,chg_diff1,chgfile1);
      ReadChg(str2,ngrid2,chg_tot2,chg_diff2,chgfile2);
      matrix<double> lat1=str1.GetLat();
      matrix<double> lat2=str2.GetLat();
      if (ngrid1[0]!=ngrid2[0] || ngrid1[1]!=ngrid2[1] || ngrid1[2]!=ngrid2[2]){
	cout << endl;
	cout << "WARNING: convasp.cc " << endl;	  
	cout << "WARNING: grids for two CHGCAR's do not match. " << endl;	  
	cout << "WARNING: ngrid1: " << ngrid1[0] << " " <<  ngrid1[1] << " " <<  ngrid1[2] << endl;
	cout << "WARNING: ngrid2: " << ngrid2[0] << " " <<  ngrid2[1] << " " <<  ngrid2[2] << endl;
	cout << "WARNING: This will likely give nonsense!! " << endl;
	cout << endl;
      }
      if (norm(VVdiff(lat1[0],lat2[0]))>TOL 
	  || norm(VVdiff(lat1[1],lat2[1]))>TOL 
	  || norm(VVdiff(lat1[2],lat2[2]))>TOL){
	cout << endl;
	cout << "WARNING: convasp.cc " << endl;	  
	cout << "WARNING: lattice parameters for two CHGCARs do not match. " << endl;	  
	cout << "WARNING: lattice 1: " << endl;;
	lat1.print();
	cout << "WARNING: lattice 2: " << endl;;
	lat2.print();
	cout << "WARNING: This could give nonsense if there is much difference!! " << endl;
	cout << "WARNING: Output will use lattice 1!! " << endl;
	cout << endl;
      }	
      // Get difference
      vector<double> chg_tot_2m1=VVdiff(chg_tot1,chg_tot2);
      vector<double> chg_diff_2m1=VVdiff(chg_diff1,chg_diff2);
      PrintCHGCAR(str1,ngrid1,chg_tot_2m1,chg_diff_2m1);
    } else{ // Ewald sum
    if ((argc==2 || argc==3) && strcmp(argv[1],"-ewald")==0) {
      double eta=-1;
      double SUMTOL=1e-16;
      if (argc==3){
	eta=atof(argv[2]);
      }
      structure str;
      cin >> str;
      str = GetNiggliStr(str);
      double epoint;
      double ereal;
      double erecip;
      double eewald;
      Ewald(str,epoint,ereal,erecip,eewald,eta,SUMTOL);
      PrintEwald(str,epoint,ereal,erecip,eewald,eta,SUMTOL,cout);
    } else{ // Screened Ewald sum
    if ((argc==3) && strcmp(argv[1],"-sewald")==0) {
      double Ks=atof(argv[2]);
      double SUMTOL=1e-16;
      structure str;
      cin >> str;
      str = GetNiggliStr(str);
      double epoint=0;
      double ereal=0;
      double erecip=0;
      double eewald=0;
      ereal=ScreenedESEner(str,Ks,SUMTOL);
      eewald=ereal;
      PrintEwald(str,epoint,ereal,erecip,eewald,Ks,SUMTOL,cout);
    } else{ // kmesh to get kpoint mesh with desired density
    if ((argc==4) && strcmp(argv[1],"-kmesh")==0) {
      double kptdens=atof(argv[2]);
      ifstream infile(argv[3]);
      structure str;
      infile >> str;
      matrix<double> kmesh(4,3,0.0);
      GetKmesh(str,kptdens,kmesh);
      PrintKmesh(kmesh,cout);
    } else{ // potalign, courtesy Brian Puchala
    if ((argc==5) && strcmp(argv[1],"-potalign")==0) {
        string inputfile = argv[2]; //"potalign_input.txt"
        string outcar = argv[3]; // "OUTCAR";
        structure str;
		ifstream infile(argv[4]);		//"CONTCAR"
		infile >> str;
        potalign(inputfile,outcar,str);
    }else{ // plotMD, courtesy Amy Bengston
    if ((argc==2) && strcmp(argv[1],"-plotMD")==0) {
        plotMD();
    }


  // Usage help (this is the default if no flags are recognized)
  else { // Usage help.
    cout << endl;
    cout <<"******************** convasp usage ********************" << endl;
    PrintUsage(cout);
    exit(1);

  } // Usage help

  // Vasp and POSCAR Processing
  } // plotMD
  } // potalign  
  } // kmesh
  } // Sreened Ewald Sum
  } // Ewald Sum
  } // chg difference
  } // plane chg density
  } // bands
  } // dos projection
  } // occupation projection
  } // summing pdos from DOSCAR
  } // raytrace
  } // supercell_strlist
  } // join_strlist
  } // make_list
  } // cubic spline fit
  } // analyse rb run
  } // chgint

  // Structural information
  } // niggli cell
  } // center of mass
  } // rbdist
  } // rdfcmp
  } // rdf
  } // shell
  } // xray
  } // angle
  } // disp
  } // dist
  } // clat
  } // compare           // Stefano Curtarolo
  } // ndata             // Stefano Curtarolo
  } // data
  } // data1
  } // cmp_str

  // To convert to other file formats.
  } // PDB
  } // ACE 
  } // MSI
  } // XYZ 
  } // XYZ WignerSeitz   // Stefano Curtarolo
  } // WignerSeitz       // Stefano Curtarolo
  } // Gulp     
  } // Findsym           // Stefano Curtarolo
  } // Platon            // Stefano Curtarolo
  } // CIF

  // To convert to different types of POSCAR files
  } // Set center of mass
  } // remove copies
  } // shift positions
  } // Remove selective dymamics
  } // Add selective dynamics
  } // Interpolated images
  } // Supercell
  } // Get LTFV cell (linear transform around fixed vector)
  } // Get LT cell (linear transform)
  } // Get primitive
  } // Put in sphere   // Stefano Curtarolo
  } // Put in compact  // Stefano Curtarolo
  } // Put in cell
  } // Names
  } // NumNames
  } // Rescale
  } // Direct
  } // Cart

}

