// structure object header file.

#ifndef structure_comp
#define structure_comp

#include <string>
#include "matrix.h"
#include <vector>
#include "atom.h"
#include "typedefs.h"
#include "ctype.h"

class structure
{
public:
  // Constructors
  structure(); // default
  structure(const structure & b); // copy
  // destructor
  ~structure();
  // Mutators
  void SetTitle(const char* in_title);
  void SetScale(const double in_scale);
  void SetLat(const matrix<double> & in_lat);
  void SetNumEachType(const vector<int> & in_num_each_type);
  void SetTypes(const vector<int> & in_types);
  void SetCoordType(const char* in_coord_type);
  void SetCoordFlag(const int in_coord_flag);
  void SetAllAtomPos(const matrix<double> & in_pos, const int in_coord_flag);
  void ReScale(const double in_scale);
  void WignerSeitz(void);
  void SetNames(vector<string> in_name); // 1 name for each type.
  void SetAllAtomNames(const vector<string> & in_names); // 1 name for each atom.
  void SetNamesWereGiven(vector<int> in_names_were_given);
  void PutInCell();
  void PutInCompact();
  void SetMom1(const vector<double> & mom1_new);
  void SetSD(const vector<string> & in_sd);
  void SetISD(const int & in_isd);
  void AddAtom(const atom & in_atom);
  void RemoveAtom(const int id);
  void ShiftPos(const vector<double> & shift, const int & flag);
  void RemoveCopies();
  void Rotate(const matrix<double> & rm);
  void SetOrigin(const vector<double> & in_origin);
  // Accessors
  char* GetTitle();
  double GetScale() const;
  matrix<double> GetLat() const;
  matrix<double> GetScaledLat() const;
  vector<int> GetNumEachType() const;
  vector<int> GetTypes() const;
  int GetType(const int id) const;
  int GetCoordFlag() const;
  matrix<double> GetCpos() const;
  matrix<double> GetDpos() const;
  vector<string> GetNames() const;
  int GetNumAtoms() const;
  vector<string> GetAtomNameVec() const;
  vector<int> GetAtomNumVec() const;
  vector<int> GetNamesWereGiven() const;
  vector<double> GetDispToAtomDir(const vector<double> & in_dpos, const int at_num) const;
  double GetDistToAtomDir(const vector<double> & in_dpos, const int at_num) const;
  vector<double> GetDispToAtomImageDir(const vector<double> & in_dpos, const int at_num) const;
  double GetDistToAtomImageDir(const vector<double> & in_dpos, const int at_num) const;
  double GetXrayScattFactor(const string & name, const double & lambda) const;
  double GetAtomMass(const int & atnum) const;
  double GetAtomMass(const string & name) const;
  vector<double> GetMom1() const;
  vector<double> GetOrigin() const;
  // Operators
  const structure & operator=(const structure & b);
  friend istream & operator>>(istream &, structure &);
  friend ostream & operator<<(ostream &, const structure &);

 private:
  static const int MAX_TITLE_SIZE=500;
  static const int NUM_ELEMENTS = 110;
  void free();
  void copy(const structure & b);
  int num_types;
  int num_atoms;
  char title[MAX_TITLE_SIZE];
  double scale;
  matrix<double> lat;
  vector<int> num_each_type;
  vector<int> types;
  char coord_type[2];
  int coord_flag; // 0 direct, 1 cart.
  matrix<double> cpos;
  matrix<double> dpos;
  vector<string> names;
  vector<int> names_were_given;
  vector<int> at_num_vec;
  vector<double> xray_scatt_vec;
  vector<double> at_mass_vec;
  vector<string> at_name_vec;
  int isd; // !=0 => Selective dynamics, =0 => no selective dynamics.
  vector<string> sd; // Selective dynamics settings.
  int neg_scale; // 0 for scale>=0, !0 for scale <0.
  vector<double> origin;

  // Mutators
  void CalcAllCartCoords();
  void CalcAllDirectCoords();
};

#endif

