// **************************************************
// PrintDist
// **************************************************
//  This funtion prints out distances

using namespace std;
#include <cmath>
#include <string>
#include <iostream>
#include <fstream>
#include <iomanip>
#include "UsefulFuncs.h"
#include "atom.h"
#include "PrintDist.h"
#include <vector>

void PrintDist(const structure & str, const double cutoff)
{  
  cout.setf(ios::left, ios::adjustfield);
  cout.setf(ios::fixed, ios::floatfield);
  //  cout.setf(ios::fixed,ios::floatfield);
  //  cout.precision(10);
  matrix<atom> neigh_mat;
  GetStrNeighData(str,cutoff,neigh_mat);
  double tol=1e-15;

  // Output header
  cout << "All atoms identified by" << endl;
  cout << "[basis number]  [name]  [n1 n2 n3 (unit cell)]" << endl;
  cout << "Distances are only calculated for the neighbors within " << cutoff <<  endl;

  // Print out distances
  cout << endl;
  cout << "********************" << " DISTANCES " << "********************" << endl;
  for (int ia=0;ia<neigh_mat.size();ia++){
    atom a = neigh_mat[ia][0];

    // Output reference atom info.
    cout << setw(4) << a.GetNum()+1 << " " << setw(4) << a.GetName().c_str();
    vector<double> pos(3);
    if (str.GetCoordFlag()==0) { // direct
      pos=a.GetDpos();
    } else{
      pos=a.GetCpos();
    }
    cout << "   " << setw(9) << setprecision(4) << pos[0] 
	 << setw(9) << setprecision(4) << pos[1] 
	 << setw(9) << setprecision(4) << pos[2] 
	 << endl;

    for (int in=1;in<neigh_mat[ia].size();in++){
      atom an = neigh_mat[ia][in];
	vector<int> uc(3);
	uc=an.GetUnitCell();
	cout << "        ";
	cout << setw(4) << an.GetNum()+1 << " ";
	cout << setw(4) << an.GetName().c_str() << "   ";
	cout << setw(3) << uc[0] << " " << setw(3) << uc[1] << " " << setw(3) << uc[2] << "   ";
	cout << setprecision(4) << AtomDist(a,an);
	cout << endl;
    } // in
  } // ia

} // end routine








