// **************************************************
// PrintGulp
// **************************************************
//  This funtion prints out structural data in a format for
// gulp (file will cause gulp to calculate distances between
// atoms when used as input).

using namespace std;

#include "PrintGulp.h"
#include <iostream>

void PrintGulp(const structure & str)
{
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  structure sstr=str;

  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  int coord_flag=sstr.GetCoordFlag();
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  vector<string> names=sstr.GetNames();

  // Print out data in Gulp format
  cout << "distance" << endl;
  cout << "cutd 5" << endl;
  cout << "vectors" << endl;
  for (int i=0;i<3;i++){
    cout << lat[i][0] << " " << lat[i][1] << " " << lat[i][2] << endl;
  }
  for (int i=0;i<num_each_type.size();i++){
    cout <<  num_each_type[i] << " ";
  }
  cout << endl;
  if (coord_flag==0) cout << "Fractional" << endl;
  if (coord_flag==1) cout << "Cartesian" << endl;
  int num_atoms=cpos.size();
  for (int i=0;i<num_atoms;i++){
    if (coord_flag==0) cout << names[i] << " " << dpos[i][0] << " " 
	  << dpos[i][1] << " " << dpos[i][2] << endl;
    if (coord_flag==1) cout << names[i] << " " << cpos[i][0] << " " 
	  << cpos[i][1] << " " << cpos[i][2] << endl;
  }
}

