// ******************************************************************
// PrintXYZ  PrintXYZws PrintXYZInSphere
// ******************************************************************
//  This funtion prints out structural data in an XYZ format

using namespace std;
#include <iostream>
#include "PrintXYZ.h"

void PrintXYZ(const structure & str, const vector<int> & n)
{
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  structure sstr=str;

  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  char* title=sstr.GetTitle();
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  int coord_flag=sstr.GetCoordFlag();
  int num_atoms=sstr.GetNumAtoms();
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  vector<string> names=sstr.GetNames();

  // Print out data in XYZ format
  cout << num_atoms*n[0]*n[1]*n[2] << endl;
  cout << title << endl;
  for (int i=0;i<n[0];i++){
    for (int j=0;j<n[1];j++){
      for (int k=0;k<n[2];k++){
	for (int iat=0;iat<num_atoms;iat++){
	  cout << names[iat] << " " 
	    << (cpos[iat][0]+i*lat[0][0]+j*lat[1][0]+k*lat[2][0]) << " " 
	    << (cpos[iat][1]+i*lat[0][1]+j*lat[1][1]+k*lat[2][1]) << " " 
	    << (cpos[iat][2]+i*lat[0][2]+j*lat[1][2]+k*lat[2][2]) << " " << endl;
	} // iat
      } // k
    } // j
  } // i
}

// ******************************************************************
//  This funtion moves atoms inside WS cell and prints out structural data in an XYZ format

double _abs3D(const vector<double> & x) { return (double) sqrt(x[0]*x[0]+x[1]*x[1]+x[2]*x[2]); }
double _scl3D(const vector<double> & x,const vector<double> & y) {return (double) x[0]*y[0]+x[1]*y[1]+x[2]*y[2];}

void PrintXYZws(const structure & str) {
  // SC 10Jan2003

  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  structure sstr=str;

  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  char* title=sstr.GetTitle();
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  int coord_flag=sstr.GetCoordFlag();
  int num_atoms=sstr.GetNumAtoms();
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  vector<string> names=sstr.GetNames();
  
  // zoology of vectors
  vector<double> rat(3),xoo(3),yoo(3),zoo(3),xyo(3),xzo(3),yzo(3),xyz(3);
  double axoo,ayoo,azoo,axyo,axzo,ayzo,axyz;
  xoo[0]=lat[0][0];xoo[1]=lat[0][1];xoo[2]=lat[0][2];axoo=_abs3D(xoo);
  yoo[0]=lat[1][0];yoo[1]=lat[1][1];yoo[2]=lat[1][2];ayoo=_abs3D(yoo);
  zoo[0]=lat[2][0];zoo[1]=lat[2][1];zoo[2]=lat[2][2];azoo=_abs3D(zoo);
  xyo[0]=lat[0][0]+lat[1][0];xyo[1]=lat[0][1]+lat[1][1];xyo[2]=lat[0][2]+lat[1][2];axyo=_abs3D(xyo);
  xzo[0]=lat[0][0]+lat[2][0];xzo[1]=lat[0][1]+lat[2][1];xzo[2]=lat[0][2]+lat[2][2];axzo=_abs3D(xzo);
  yzo[0]=lat[1][0]+lat[2][0];yzo[1]=lat[1][1]+lat[2][1];yzo[2]=lat[1][2]+lat[2][2];ayzo=_abs3D(yzo);
  xyz[0]=lat[0][0]+lat[1][0]+lat[2][0];xyz[1]=lat[0][1]+lat[1][1]+lat[2][1];xyz[2]=lat[0][2]+lat[1][2]+lat[2][2];axyz=_abs3D(xyz);
  double projxoo,projyoo,projzoo,projxyo,projxzo,projyzo,projxyz;

  // Print out data in XYZ format
  cout << num_atoms << endl;
  cout << title << endl;
  for (int iat=0;iat<num_atoms;iat++)
    for(int i=-1;i<=1;i++)
      for(int j=-1;j<=1;j++)
	for(int k=-1;k<=1;k++)
	  {
	    rat[0]=(cpos[iat][0]+i*lat[0][0]+j*lat[1][0]+k*lat[2][0]);
	    rat[1]=(cpos[iat][1]+i*lat[0][1]+j*lat[1][1]+k*lat[2][1]);
	    rat[2]=(cpos[iat][2]+i*lat[0][2]+j*lat[1][2]+k*lat[2][2]);
	    projxoo=_scl3D(rat,xoo)/axoo/axoo;
	    projyoo=_scl3D(rat,yoo)/ayoo/ayoo;
	    projzoo=_scl3D(rat,zoo)/azoo/azoo;
	    projxyo=_scl3D(rat,xyo)/axyo/axyo;
	    projxzo=_scl3D(rat,xzo)/axzo/axzo;
	    projyzo=_scl3D(rat,yzo)/ayzo/ayzo;
	    projxyz=_scl3D(rat,xyz)/axyz/axyz;
	    //  if((projxoo>-0.5 && projxoo<=0.5))
	    //  cout << projxoo << " " << projyoo << " " << projzoo << endl;
	    if((projxoo>-0.5 && projxoo<=0.5) &&
	       (projyoo>-0.5 && projyoo<=0.5) &&
	       (projzoo>-0.5 && projzoo<=0.5) &&
	       (projxyo>-0.5 && projxyo<=0.5) &&
	       (projxzo>-0.5 && projxzo<=0.5) &&
	       (projyzo>-0.5 && projyzo<=0.5) &&
	       (projxyz>-0.5 && projxyz<=0.5)) 
	      cout << names[iat] << " " << rat[0] << " " << rat[1] << " " << rat[2] << " " << endl; 
	  }
}


// ******************************************************************
//  This funtion prints out structural data inside a sphere with radius radius 
// and 0,0,0 as origin, in an XYZ format

void PrintXYZInSphere(const structure & str, const double& radius)
{
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  structure sstr=str;
#define _SPHERE_CUT_OFF_ 5

  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  char* title=sstr.GetTitle();
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  int coord_flag=sstr.GetCoordFlag();
  int num_atoms=sstr.GetNumAtoms();
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  vector<string> names=sstr.GetNames();

  int atoms_inside=0;
  vector<double> x(3);
  // Print out data in InSphere format
  for (int i=-_SPHERE_CUT_OFF_;i<_SPHERE_CUT_OFF_;i++){
    for (int j=-_SPHERE_CUT_OFF_;j<_SPHERE_CUT_OFF_;j++){
      for (int k=-_SPHERE_CUT_OFF_;k<_SPHERE_CUT_OFF_;k++){
	for (int iat=0;iat<num_atoms;iat++){
	  x[0]=(cpos[iat][0]+i*lat[0][0]+j*lat[1][0]+k*lat[2][0]);
	  x[1]=(cpos[iat][1]+i*lat[0][1]+j*lat[1][1]+k*lat[2][1]);
	  x[2]=(cpos[iat][2]+i*lat[0][2]+j*lat[1][2]+k*lat[2][2]);
	  if(_abs3D(x)<=radius) atoms_inside++;
	} // iat
      } // k
    } // j
  } // i
  
  cout << atoms_inside << endl;
  cout << title << endl;
  // Print out data in InSphere format
  for (int i=-_SPHERE_CUT_OFF_;i<_SPHERE_CUT_OFF_;i++){
    for (int j=-_SPHERE_CUT_OFF_;j<_SPHERE_CUT_OFF_;j++){
      for (int k=-_SPHERE_CUT_OFF_;k<_SPHERE_CUT_OFF_;k++){
	for (int iat=0;iat<num_atoms;iat++){
	  x[0]=(cpos[iat][0]+i*lat[0][0]+j*lat[1][0]+k*lat[2][0]);
	  x[1]=(cpos[iat][1]+i*lat[0][1]+j*lat[1][1]+k*lat[2][1]);
	  x[2]=(cpos[iat][2]+i*lat[0][2]+j*lat[1][2]+k*lat[2][2]);
	  if(_abs3D(x)<=radius) 
	    cout << names[iat] << " " << x[0] << " " << x[1] << " " << x[2] << " " << endl;
	} // iat
      } // k
    } // j
  } // i
}

