// structure object. 

using namespace std;
#include<string>
#include<iostream>
#include<stddef.h>
#include<assert.h>
#include "atom.h"
#include "UsefulFuncs.h"

void atom::free()
{
}

void atom::copy(const atom & b)
{
  name=b.name;
  num=b.num;
  unit_cell=b.unit_cell;
  cpos=b.cpos;
  dpos=b.dpos;
  corigin=b.corigin;
  type=b.type;
}


// Constructor
atom::atom()
{
  name="NA";
  num=-1;
  unit_cell = vector<int> (3,0);
  cpos = vector<double> (3,-999.0);
  dpos = vector<double> (3,-999.0);
  corigin = vector<double> (3,0.0);
  type=-1;
}

atom::atom(const atom & b) {copy(b);}


// Mutators
void atom::SetName(const string & in_name){name=in_name;}
void atom::SetType(const int in_type){type=in_type;}
void atom::SetNum(const int in_num){num=in_num;}
void atom::SetUnitCell(const vector<int> & in_unit_cell){unit_cell=in_unit_cell; assert (unit_cell.size()==3);}
void atom::SetCpos(const vector<double> & in_cpos){cpos=in_cpos; assert (cpos.size()==3);}
void atom::SetDpos(const vector<double> & in_dpos){dpos=in_dpos; assert (dpos.size()==3);}
void atom::SetCOrigin(const vector<double> & in_corigin){corigin=in_corigin; assert (corigin.size()==3);}

// Accessors
string atom::GetName() const {return name;}
int atom::GetType() const {return type;}
int atom::GetNum() const {return num;}
vector<int> atom::GetUnitCell() const{return unit_cell;}
vector<double> atom::GetCpos() const{return cpos;}
vector<double> atom::GetDpos() const{return dpos;}
vector<double> atom::GetCOrigin() const{return corigin;}
vector<double> atom::GetCDispFromOrigin() const
{
  vector<double> diff(3);
  for (int ic=0;ic<3;ic++){
    diff[ic]=cpos[ic]-corigin[ic];
  }
  return diff;
}
double atom::GetDistFromOrigin() const 
{return norm(GetCDispFromOrigin());}


// Operators
const atom & atom::operator=(const atom & b)
{
  if (this != &b){
    free();
    copy(b);
  }
  return *this;
}

const void atom::Print(ostream & sout){
  sout << "name: " << name << endl;
  sout << "type: " << type << endl;
  sout << "num: " << num << endl;
  sout << "unit cell: " << unit_cell[0] << " " << unit_cell[1] << " " << unit_cell[2] << endl;
  sout << "cpos: " << cpos[0] << " " << cpos[1] << " " << cpos[2] << endl;
  sout << "dpos: " << dpos[0] << " " << dpos[1] << " " << dpos[2] << endl;
  sout << "corigin: " << corigin[0] << " " << corigin[1] << " " << corigin[2] << endl;
}





