// structure object. 

using namespace std;
#include<string>
#include<cmath>
#include<iostream>
#include<stddef.h>
#include "GaussJordan.h"
#include "UsefulFuncs.h"
#include "structure.h"

void structure::free()
{
}

	int V5AtomNames;
	string AtomNameStr;

void structure::copy(const structure & b)
{

  //names copy
  names = vector<string> (b.names.size());
  int names_size=names.size();
  for (int i=0;i<names_size;i++){
    names[i]=b.names[i];
  }

  // All the other stuff not set in the constructor
  num_types=b.num_types;
  num_atoms=b.num_atoms;
  strcpy(title,b.title);
  scale=b.scale;
  lat=b.lat;
  num_each_type=b.num_each_type;
  types=b.types;
  strcpy(coord_type,b.coord_type);
  coord_flag=b.coord_flag;
  cpos=b.cpos;
  dpos=b.dpos;
  // names are above
  names_were_given=b.names_were_given;
  // Stuff set in constructor
  at_name_vec=b.at_name_vec;
  at_num_vec=b.at_num_vec;
  xray_scatt_vec=b.xray_scatt_vec;
  at_mass_vec=b.at_mass_vec;
  sd=b.sd;
  isd=b.isd;
  neg_scale=b.neg_scale;
  origin=b.origin;
}

// Constructors
structure::structure()
{
  num_atoms=0;
  num_types=0;
  strncpy(title,"NO_TITLE_GIVEN",MAX_TITLE_SIZE);
  scale=1;
  lat = matrix<double> (3,3,0.0);
  num_each_type=vector<int> (0);
  types=vector<int> (0);
  strncpy(coord_type,"D",2);
  coord_flag=0;
  cpos=matrix<double> (0);
  dpos=matrix<double> (0);
  names=vector<string> (0);
  names_were_given=vector<int> (0);
  isd=0; // No Selective dynamics.
  sd=vector<string> (0);
  neg_scale=0;
  origin=vector<double> (3,0.0); 
  // List of atomic names and numbers
  at_num_vec=vector<int> (NUM_ELEMENTS,0);
  at_name_vec=vector<string> (NUM_ELEMENTS);
  for (int i=0;i<NUM_ELEMENTS;i++){
    at_num_vec[i]=i+1;
    at_name_vec[i]="NA";
  }
  at_name_vec[0]="H";
  at_name_vec[1]="He";

  at_name_vec[2]="Li";
  at_name_vec[3]="Be";
  at_name_vec[4]="B";
  at_name_vec[5]="C";
  at_name_vec[6]="N";
  at_name_vec[7]="O";
  at_name_vec[8]="F";
  at_name_vec[9]="Ne";

  at_name_vec[10]="Na";
  at_name_vec[11]="Mg";
  at_name_vec[12]="Al";
  at_name_vec[13]="Si";
  at_name_vec[14]="P";
  at_name_vec[15]="S";
  at_name_vec[16]="Cl";
  at_name_vec[17]="Ar";

  at_name_vec[18]="K";
  at_name_vec[19]="Ca";
  at_name_vec[20]="Sc";
  at_name_vec[21]="Ti";
  at_name_vec[22]="V";
  at_name_vec[23]="Cr";
  at_name_vec[24]="Mn";
  at_name_vec[25]="Fe";
  at_name_vec[26]="Co";
  at_name_vec[27]="Ni";
  at_name_vec[28]="Cu";
  at_name_vec[29]="Zn";
  at_name_vec[30]="Ga";
  at_name_vec[31]="Ge";
  at_name_vec[32]="As";
  at_name_vec[33]="Se";
  at_name_vec[34]="Br";
  at_name_vec[35]="Kr";

  at_name_vec[36]="Rb";
  at_name_vec[37]="Sr";
  at_name_vec[38]="Y";
  at_name_vec[39]="Zr";
  at_name_vec[40]="Nb";
  at_name_vec[41]="Mo";
  at_name_vec[42]="Tc";
  at_name_vec[43]="Ru";
  at_name_vec[44]="Rh";
  at_name_vec[45]="Pd";
  at_name_vec[46]="Ag";
  at_name_vec[47]="Cd";
  at_name_vec[48]="In";
  at_name_vec[49]="Sn";
  at_name_vec[50]="Sb";
  at_name_vec[51]="Te";
  at_name_vec[52]="I";
  at_name_vec[53]="Xe";

  at_name_vec[54]="Cs";
  at_name_vec[55]="Ba";
  at_name_vec[56]="La";
  at_name_vec[71]="Hf";
  at_name_vec[72]="Ta";
  at_name_vec[73]="W";
  at_name_vec[74]="Re";
  at_name_vec[75]="Os";
  at_name_vec[76]="Ir";
  at_name_vec[77]="Pt";
  at_name_vec[78]="Au";
  at_name_vec[79]="Hg";
  at_name_vec[80]="Tl";
  at_name_vec[81]="Pb";
  at_name_vec[82]="Bi";
  at_name_vec[83]="Po";
  at_name_vec[84]="At";
  at_name_vec[85]="Rn";

  // I did not put in names for F-valence electron atoms or 
  // atomic numbers > 86.  These default to NA.

// Xray scattering factors
  xray_scatt_vec=vector<double> (NUM_ELEMENTS,0.0);
  for (int i=0;i<NUM_ELEMENTS;i++){
    xray_scatt_vec[i]=(double)i;
  }

  // All indices are the atomic number shifted back by one.

  // All data collected from the NIST online tables: 
  // http://physics.nist.gov/PhysRefData/FFast/html/form.html
  // All data are ideally for f1 values for Cu-alpha (wavelength=1.5418A, E=8.0416keV).
  // These are for E=7.9026keV (Cu-alpha is wavelength=1.5418A, E=8.0416keV). 
  xray_scatt_vec[2]=3.00145E+00; // Li
  xray_scatt_vec[14]=1.53133E+01; // P
  xray_scatt_vec[24]=2.43589E+01; // Mn
  xray_scatt_vec[25]=2.46830E+01; // Fe

  // All data collected from the online tables: 
  // http://www-cxro.lbl.gov/optical_constants/pert_form.html
  // All data are f1 values for Cu-alpha (wavelength=1.5418A, E=8.0416keV).
  xray_scatt_vec[0]=1.000; // H
  xray_scatt_vec[1]=2.000; // He
  xray_scatt_vec[2]=3.001; // Li
  xray_scatt_vec[6]=6.019; // C
  xray_scatt_vec[7]=8.052; // O
  xray_scatt_vec[13]=14.43; // P
  xray_scatt_vec[14]=15.30; // P
  xray_scatt_vec[20]=21.34; // Sc
  xray_scatt_vec[21]=22.24; // Ti 
  xray_scatt_vec[23]=23.84; // Cr
  xray_scatt_vec[24]=24.46; // Mn
  xray_scatt_vec[25]=24.85; // Fe
  xray_scatt_vec[26]=24.59; // Co
  xray_scatt_vec[27]=25.02; // Ni
  xray_scatt_vec[28]=27.03; // Cu
  xray_scatt_vec[29]=28.44; // Zn
  xray_scatt_vec[46]=47.18; // Ag
  xray_scatt_vec[78]=74.99; // Au
  xray_scatt_vec[89]=86.64; // Th
 
// Atomic masses
// All indices are the atomic number shifted back by one.
// All masses are in kilograms
  at_mass_vec=vector<double> (NUM_ELEMENTS,0.0);
  for (int i=0;i<NUM_ELEMENTS;i++){
    at_mass_vec[i]=(double)(2*(i+1))*AMU2KILOGRAM;
  }
  at_mass_vec[0]=1.0079*AMU2KILOGRAM; //H
  at_mass_vec[7]=15.9994*AMU2KILOGRAM; //O
  at_mass_vec[24]=54.93805*AMU2KILOGRAM; //Mn

}
  
// copy
structure::structure(const structure & b) {copy(b);}

// destructor
structure::~structure() {free();}

// Mutators
void structure::SetTitle(const char* in_title) {strncpy(title,in_title,MAX_TITLE_SIZE);}
void structure::SetScale(const double in_scale) {scale=in_scale;}
void structure::SetLat(const matrix<double> & in_lat) {lat=in_lat;}
void structure::SetNumEachType(const vector<int> & in_num_each_type) {num_each_type=in_num_each_type; num_types=num_each_type.size();}
void structure::SetTypes(const vector<int> & in_types) {types=in_types;}
void structure::SetCoordType(const char* in_coord_type) 
{
  strncpy(coord_type,in_coord_type,2);
  if (strcmp(coord_type,"D")==0 || strcmp(coord_type,"d")==0) coord_flag==0;
  if (strcmp(coord_type,"C")==0 || strcmp(coord_type,"c")==0) coord_flag==1;
}
void structure::SetCoordFlag(const int in_coord_flag) 
{
  coord_flag=in_coord_flag;
  if (coord_flag==0) strcpy(coord_type,"D");
  if (coord_flag==1) strcpy(coord_type,"C");
}
void structure::SetAllAtomPos(const matrix<double> & in_pos, const int in_coord_flag) 
{
  assert(in_coord_flag==0 || in_coord_flag==1);
  if (in_coord_flag==0){
    dpos=in_pos; 
    num_atoms=dpos.size(); 
    cpos=matrix<double> (num_atoms,3,0.0);
    CalcAllCartCoords();
  }
  if (in_coord_flag==1){
    cpos=in_pos; 
    num_atoms=cpos.size(); 
    dpos=matrix<double> (num_atoms,3,0.0);
    CalcAllDirectCoords();};
}

void structure::ReScale(double in_scale)
// This resets scale and changes the cell parameters and coordinates
//appropriately.  Keeps volume fixed.
{
  assert(in_scale!=0);
  for (int i=0;i<3;i++){
    for (int j=0;j<3;j++){
      lat[i][j]=lat[i][j]*scale/in_scale;
    }
  }
  for (int i=0;i<num_atoms;i++){
    for (int j=0;j<3;j++){
      cpos[i][j]=cpos[i][j]*scale/in_scale;
    }
  }
  scale=in_scale;
}

double _str_abs3D(const vector<double> & x) { return (double) sqrt(x[0]*x[0]+x[1]*x[1]+x[2]*x[2]); }
double _str_scl3D(const vector<double> & x,const vector<double> & y) {return (double) x[0]*y[0]+x[1]*y[1]+x[2]*y[2];}
void structure::WignerSeitz(void) {
  // zoology of vectors
  vector<double> rat(3),xoo(3),yoo(3),zoo(3),xyo(3),xzo(3),yzo(3),xyz(3);
  double axoo,ayoo,azoo,axyo,axzo,ayzo,axyz;
  xoo[0]=lat[0][0];xoo[1]=lat[0][1];xoo[2]=lat[0][2];axoo=_str_abs3D(xoo);
  yoo[0]=lat[1][0];yoo[1]=lat[1][1];yoo[2]=lat[1][2];ayoo=_str_abs3D(yoo);
  zoo[0]=lat[2][0];zoo[1]=lat[2][1];zoo[2]=lat[2][2];azoo=_str_abs3D(zoo);
  xyo[0]=lat[0][0]+lat[1][0];xyo[1]=lat[0][1]+lat[1][1];xyo[2]=lat[0][2]+lat[1][2];axyo=_str_abs3D(xyo);
  xzo[0]=lat[0][0]+lat[2][0];xzo[1]=lat[0][1]+lat[2][1];xzo[2]=lat[0][2]+lat[2][2];axzo=_str_abs3D(xzo);
  yzo[0]=lat[1][0]+lat[2][0];yzo[1]=lat[1][1]+lat[2][1];yzo[2]=lat[1][2]+lat[2][2];ayzo=_str_abs3D(yzo);
  xyz[0]=lat[0][0]+lat[1][0]+lat[2][0];xyz[1]=lat[0][1]+lat[1][1]+lat[2][1];xyz[2]=lat[0][2]+lat[1][2]+lat[2][2];axyz=_str_abs3D(xyz);
  double projxoo,projyoo,projzoo,projxyo,projxzo,projyzo,projxyz;

  for (int iat=0;iat<num_atoms;iat++) 
    for(int i=-1;i<=1;i++)
      for(int j=-1;j<=1;j++)
	for(int k=-1;k<=1;k++) {
	  rat[0]=(cpos[iat][0]+i*lat[0][0]+j*lat[1][0]+k*lat[2][0]);
	  rat[1]=(cpos[iat][1]+i*lat[0][1]+j*lat[1][1]+k*lat[2][1]);
	  rat[2]=(cpos[iat][2]+i*lat[0][2]+j*lat[1][2]+k*lat[2][2]);
	  projxoo=_str_scl3D(rat,xoo)/axoo/axoo;
	  projyoo=_str_scl3D(rat,yoo)/ayoo/ayoo;
	  projzoo=_str_scl3D(rat,zoo)/azoo/azoo;
	  projxyo=_str_scl3D(rat,xyo)/axyo/axyo;
	  projxzo=_str_scl3D(rat,xzo)/axzo/axzo;
	  projyzo=_str_scl3D(rat,yzo)/ayzo/ayzo;
	  projxyz=_str_scl3D(rat,xyz)/axyz/axyz;
	  if((projxoo>-0.5 && projxoo<=0.5) &&
	     (projyoo>-0.5 && projyoo<=0.5) &&
	     (projzoo>-0.5 && projzoo<=0.5) &&
	     (projxyo>-0.5 && projxyo<=0.5) &&
	     (projxzo>-0.5 && projxzo<=0.5) &&
	     (projyzo>-0.5 && projyzo<=0.5) &&
	     (projxyz>-0.5 && projxyz<=0.5)) {
	    cpos[iat][0]=rat[0];
	    cpos[iat][1]=rat[1];
	    cpos[iat][2]=rat[2];
	    i=10;j=10;k=10;
	  }
	}
}


void structure::SetNames(vector<string> in_names_vec)
{ 
  // Note that in_names_vec has one name for each type, not one name for each atom.
  // Therefore, one must have the number of atoms and types correspond to the names.
  int cnt=-1;
  int in_names_vec_size=in_names_vec.size();
  for (int i=0;i<in_names_vec_size;i++){
    if (i<num_each_type.size()){
      for (int j=0;j<num_each_type[i];j++){
	cnt++;
	names[cnt] = in_names_vec[i];
        if (in_names_vec[i].size()!=0){
          names_were_given[cnt]=1;
        }
        else{
          names_were_given[cnt]=0;
        }
      }
    }
  }
}

void structure::SetAllAtomNames(const vector<string> & in_names){
  names=in_names;
  names_were_given=vector<int> (names.size(),0);
  for (int i=0;i<names.size();i++){
    if (names[i].size()>0) names_were_given[i]=1;
  }
}

void structure::SetNamesWereGiven(const vector<int> in_names_were_given) {names_were_given=in_names_were_given;}

void structure::SetMom1(const vector<double> & mom1_new)
{
  // Get old first moment in cart. coords.
  vector<double> mom1_old=structure::GetMom1();
  // Get change in first moment in cart. coords.
  vector<double> dmom1=VVdiff(mom1_new,mom1_old);
  // Shift scaled cart. positions by dmom1.
  for (int iat=0;iat<num_atoms;iat++){
    cpos[iat]=SVprod(scale,cpos[iat]);
    cpos[iat]=VVsum(cpos[iat],dmom1);
    cpos[iat]=SVprod(1.0/scale,cpos[iat]);
  }
  // Recalculate direct coords.
  CalcAllDirectCoords();
}

void structure::SetSD(const vector<string> & in_sd){
// Note that in_sd has one name for each type, not one name for each atom.
  int cnt=-1;
  int size=in_sd.size();
  for (int i=0;i<size;i++){
    if (i<num_each_type.size()){
      for (int j=0;j<num_each_type[i];j++){
	cnt++;
	sd[cnt] = in_sd[i];
      }
    }
  }
}

void structure::SetISD(const int & in_isd){isd=in_isd;}
  
void structure::AddAtom(const atom & atom)
  // This adds an atom to the structure.
{
  int at_type=atom.GetType();
  vector<double> at_dpos=atom.GetDpos();
  vector<double> at_cpos=atom.GetCpos();
  string at_name=atom.GetName();
  int at_name_given=1;
  if (at_name=="NA") at_name_given=0; // Name still at default value.
  // Figure out where atom goes (index id).
  int id=0;
  if (at_type>=num_types){
    id=num_atoms;
    num_each_type.push_back(1);
    num_types++;
  }
  else{
    for (int i=0;i<=at_type;i++){
      id=id+num_each_type[i];
    }
    num_each_type[at_type]++;
  }
  // Put atom where it goes in structure.
  cpos.insert(id,at_cpos);
  dpos.insert(id,at_dpos);
  names.insert(names.begin()+id,at_name);
  names_were_given.insert(names_were_given.begin()+id,at_name_given);
  num_atoms++;
}

void structure::RemoveAtom(const int id){
  assert(id<num_atoms);
  vector<int>::iterator it_i;
  vector<double>::iterator it_d;
  vector<string>::iterator it_s;
  cpos.erase(id);  
  dpos.erase(id);
  it_s=names.begin()+id;
  names.erase(it_s);
  it_i=names_were_given.begin()+id;
  names_were_given.erase(it_i);
  num_each_type[types[id]]--;
  if (num_each_type[types[id]]==0){
    it_i=num_each_type.begin()+types[id];
    num_each_type.erase(it_i);
  }
  it_i=types.begin()+id;
  types.erase(it_i);
  num_types=num_each_type.size();
  num_atoms=num_atoms-1;
}

void structure::PutInCell()
{
  double TOL=1e-15;
  // For direct coordinates
  for (int i=0;i<num_atoms;i++){
    for (int j=0;j<3;j++){
      dpos[i][j]=dpos[i][j]-int(dpos[i][j]);
      if (dpos[i][j]<-TOL) dpos[i][j]=dpos[i][j]+1;
      if (abs(dpos[i][j])<TOL) dpos[i][j]=0.0;
    }
  } 
  // For Cartesian coordinates (get from direct coords)
  CalcAllCartCoords();
}


// STEFANO TRY TO BRING ATOMS SO THAT THEY HAVE SHORTEST POSSIBLE BONDS !!!

double atom_atom_distance(const vector<double>& A,const vector<double>& B) {
  return sqrt((A[0]-B[0])*(A[0]-B[0])+(A[1]-B[1])*(A[1]-B[1])+(A[2]-B[2])*(A[2]-B[2]));
}

void structure::PutInCompact()
{
  double TOL=1e-15;
  double min_bond,bond;
  // For direct coordinates
  for (int i=0;i<num_atoms;i++){
    for (int j=0;j<3;j++){
      dpos[i][j]=dpos[i][j]-int(dpos[i][j]);
      if (dpos[i][j]<-TOL) dpos[i][j]=dpos[i][j]+1;
      if (abs(dpos[i][j])<TOL) dpos[i][j]=0.0;
    }
  }

  for (int iat=0;iat<dpos.size();iat++){
    for (int ic=0;ic<3;ic++){
      cpos[iat][ic]=0.0;
      for (int jc=0;jc<3;jc++){
	cpos[iat][ic]=cpos[iat][ic]+dpos[iat][jc]*lat[jc][ic];
      }
    }
  }

  vector<double> adref1pos(3),adtstpos(3),adtrgpos(3);
  vector<double> acref1pos(3),actstpos(3),actrgpos(3);
  for (int i=1;i<num_atoms;i++){                         // scan all the atoms to move except the first
    adtrgpos[0]=dpos[i][0];adtrgpos[1]=dpos[i][1];adtrgpos[2]=dpos[i][2];           // default
    actrgpos[0]=cpos[i][0];actrgpos[1]=cpos[i][1];actrgpos[2]=cpos[i][2];           // default
    min_bond=1.0e6;
    for (int ii=0;ii<i;ii++) {                                                      // scan over all the reference atoms 
      adref1pos[0]=dpos[ii][0];adref1pos[1]=dpos[ii][1];adref1pos[2]=dpos[ii][2];   // scan over all the reference atoms 
      for (int ic=0;ic<3;ic++){                                                     // scan over all the reference atoms 
	acref1pos[ic]=0.0;                                                          // scan over all the reference atoms 
	for (int jc=0;jc<3;jc++) acref1pos[ic]=acref1pos[ic]+adref1pos[jc]*lat[jc][ic];// scan over all the reference atoms 
      }                                                                             // scan over all the reference atoms 
 
      for(int i1=-1;i1<=1;i1++)                                                     // roll over first neighbour cells 
	for(int j1=-1;j1<=1;j1++)                                                   // roll over first neighbour cells 
	  for(int k1=-1;k1<=1;k1++) {                                               // roll over first neighbour cells 
	    adtstpos[0]=dpos[i][0]+i1;adtstpos[1]=dpos[i][1]+j1;adtstpos[2]=dpos[i][2]+k1;      // test the atom
	    for (int ic=0;ic<3;ic++){                                                           // test the atom
	      actstpos[ic]=0.0;                                                                 // test the atom                            
	      for (int jc=0;jc<3;jc++)                                                          // test the atom
		actstpos[ic]=actstpos[ic]+adtstpos[jc]*lat[jc][ic];                             // test the atom 
	    }                                                                                   // test the atom                      
	    bond=atom_atom_distance(actstpos,acref1pos);                                        // test the bond distance
	    // if((bond<min_bond) {                  // if it is OK then DO IT !
	    if((bond<1.03*min_bond && abs(i-ii)<10) || (bond<0.98*min_bond) ) {                  // if it is OK then DO IT !
	      min_bond=bond;                                                                    // update
	      adtrgpos[0]=adtstpos[0];adtrgpos[1]=adtstpos[1];adtrgpos[2]=adtstpos[2];          // update
	      actrgpos[0]=actstpos[0];actrgpos[1]=actstpos[1];actrgpos[2]=actstpos[2];          // update
	    }
	  }
    }
    // now the TRG atom has all the information about the bst position of the atom to be moved.
    // than EAT IT !
    dpos[i][0]=adtrgpos[0];dpos[i][1]=adtrgpos[1];dpos[i][2]=adtrgpos[2];
    cpos[i][0]=actrgpos[0];cpos[i][1]=actrgpos[1];cpos[i][2]=actrgpos[2];
  }
  // For Cartesian coordinates (get from direct coords)
  CalcAllCartCoords();
}

void structure::ShiftPos(const vector<double> & shift, const int & flag)
{
  if (flag==0){ // Cartesian shift  
    for (int ia=0;ia<num_atoms;ia++){
      cpos[ia]=VVsum(shift,cpos[ia]);
    }
    CalcAllDirectCoords();
  }
  if (flag==1) { // Direct coords shift  
    for (int ia=0;ia<num_atoms;ia++){
      dpos[ia]=VVsum(shift,dpos[ia]);
    }
    CalcAllCartCoords();
  }
}

void structure::RemoveCopies(){
  double tol=1e-3;
  int remove=0;
  for (int ia1=num_atoms-1;ia1>=0;ia1--){
    for (int ia2=0;ia2<ia1;ia2++){
      if (norm(VVdiff(cpos[ia1],cpos[ia2]))<tol){remove=1;}
    }
    if (remove){
      RemoveAtom(ia1);
    }
    remove=0;
  }
}
void structure::Rotate(const matrix<double> & rm)
  /*  
      Rotation is done around the origin of the structure.
      Define origin q, a point p, and a rotation of p around 
      q (R_q(p)).  Then R_q(p)=R_0(p-q)+q=R_0(p)-R_0(q)+q.
      We can evalutate the R_0 terms by simply mulitplying by 
      a matrix.  Then we must add q to all the final cartesian
      positions.
  */
{
  // Get R_0(p) for all cartesian positions.
  matrix<double> nlat=lat.transpose();
  lat=MMmult(rm,nlat).transpose();
  CalcAllCartCoords();
  //Get R_0(q)
  vector<double> r_orig=MVmult(rm,origin);
  // Assign new cartesian positions
  for (int ia=0;ia<num_atoms;ia++){
    cpos[ia]=VVsum(VVdiff(cpos[ia],r_orig),origin);
  }
  // Get all the direct coords.
  CalcAllDirectCoords();
}


void structure::SetOrigin(const vector<double> & in_origin) {origin=in_origin;}

// Accessors
char* structure::GetTitle(){return title;}
double structure::GetScale() const {return scale;}
matrix<double> structure::GetLat() const {return lat;}
matrix<double> structure::GetScaledLat() const 
{
  matrix<double> tlat(3,3);
  for (int ic=0;ic<3;ic++){
    tlat[ic]=SVprod(scale,lat[ic]);
  }
  return tlat;
}
vector<int> structure::GetNumEachType() const {return num_each_type;}
vector<int> structure::GetTypes() const {return types;}
int structure::GetType(const int id) const {return types[id];}
int structure::GetCoordFlag() const {return coord_flag;}
matrix<double> structure::GetCpos() const {return cpos;}
matrix<double> structure::GetDpos() const {return dpos;}
vector<string> structure::GetNames() const {return names;}
int structure::GetNumAtoms() const {return num_atoms;}
vector<string> structure::GetAtomNameVec() const {return at_name_vec;}
vector<int> structure::GetNamesWereGiven() const {return names_were_given;}
vector<int> structure::GetAtomNumVec() const {return at_num_vec;}
vector<double> structure::GetDispToAtomDir(const vector<double> & in_dpos, const int at_num) const
{
  // returns in_dpos-(dpos of atom at_num) in direct coordinates.  in_dpos should be in direct coords.
  vector<double> disp(3,0.0);
  for (int ic=0;ic<3;ic++){
    disp[ic]=in_dpos[ic]-dpos[at_num][ic];
    //  if (disp[ic]>0.5) disp[ic]=disp[ic]-1;
    //  if (disp[ic]<-0.5) disp[ic]=disp[ic]+1;
  }
  return disp;
}
double structure::GetDistToAtomDir(const vector<double> & in_dpos, const int at_num) const
{
  // returns norm(in_dpos-(pos of atom at_num)).  in_dpos should be in direct coords.
  vector<double> disp(3);
  disp=GetDispToAtomDir(in_dpos,at_num);
  disp=vecD2C(lat,disp);
  return scale*norm(disp);
}
double structure::GetXrayScattFactor(const string & name, const double & lambda) const
{
  // Does not use lambda for now.
  double scatt_fact;
  for (int i=0;i<num_atoms;i++){
    for (int j=0;j<at_name_vec.size();j++){
      if (name==at_name_vec[j]) scatt_fact=xray_scatt_vec[j];
    }
  }
  return scatt_fact;
}

double structure::GetAtomMass(const int & atnum) const
{
  assert (atnum<=at_mass_vec.size());
  return at_mass_vec[atnum-1];
}

double structure::GetAtomMass(const string & name) const
{
  double at_mass=-1;
  for (int i=0;i<num_atoms;i++){
    for (int j=0;j<at_name_vec.size();j++){
      if (name==at_name_vec[j]) at_mass=at_mass_vec[j];
    }
  }
  if (at_mass<0){
    cerr << "ERROR: structure::GetAtomMass" <<endl;
    cerr << "ERROR: cannot find atom with name " << name << endl;
    cerr << "ERROR: exiting" << endl;
    exit(1);
  }
  return at_mass;
}

vector<double> structure::GetMom1() const
{
  // Get's first moment in cartesian coordinates.
  vector<double> mom1(3,0.0);
  for (int iat=0;iat<num_atoms;iat++){
    mom1=VVsum(mom1,cpos[iat]);
  }
  if (num_atoms>0){
    mom1=SVprod(scale/(float)num_atoms,mom1); // Includes rescaling by scale.
  }
  else{
    mom1 = vector<double> (3,0.0);
  } 
  return mom1;
}

vector<double> structure::GetDispToAtomImageDir(const vector<double> & in_dpos, const int at_num) const
{
  // returns {in_dpos}-{dpos of atom at_num} in direct coordinates, where the difference is taken 
  // between the nearest images (braces denote the fact that each set point is really a set of points
  // in different image cells.  in_dpos should be in direct coords.
  vector<double> disp(3,0.0);
  for (int ic=0;ic<3;ic++){
    disp[ic]=in_dpos[ic]-dpos[at_num][ic];
    disp[ic]=disp[ic]-(int)disp[ic];
    if (disp[ic]>0.5) disp[ic]=disp[ic]-1;
    if (disp[ic]<-0.5) disp[ic]=disp[ic]+1;
  }
  return disp;
}
double structure::GetDistToAtomImageDir(const vector<double> & in_dpos, const int at_num) const
{
  // returns {in_dpos}-{dpos of atom at_num} in direct coordinates, where the difference is taken 
  // between the nearest images (braces denote the fact that each set point is really a set of points
  // in different image cells.  in_dpos should be in direct coords.
  vector<double> disp(3);
  disp=GetDispToAtomImageDir(in_dpos,at_num);
  disp=vecD2C(lat,disp);
  return scale*norm(disp);
}
vector<double> structure::GetOrigin() const {return origin;}

// Operators
istream & operator>>(istream & input, structure & in_structure)
{
  int i;
  char temp[200];
  // Title.
  input.getline(in_structure.title,in_structure.MAX_TITLE_SIZE);
  // Scale.
  input >> in_structure.scale;
  // Lattice vectors.
  for (i=0;i<3;i++){
    input >> in_structure.lat[i][0] >> in_structure.lat[i][1] >> in_structure.lat[i][2];
  }
  // If scale < 0 then it should be treated as the volume.
  in_structure.neg_scale=0;
  if (in_structure.scale<0){
    double nvol=-1*in_structure.scale;
    double ovol=GetVol(in_structure.lat);
    in_structure.scale=pow((nvol/ovol),1.0/3.0);
    in_structure.neg_scale=1;
  }
  // Number of atoms of each type and total number of atoms.
  input.getline(temp,200);
//    The following is necessary because if the last lattice parameter has 
//  tabs/spaces after it then the getline just grabs those spaces.  This 
//  second getline will then be called to actually read in the number 
//  of each type of atom.

  string tempns=RemoveWhiteSpaces(string(temp));
  if (string(tempns).size()==0) input.getline(temp,200);
	
	string temp_ns = RemoveSpaces(temp);
	
	V5AtomNames = 0;
	if (isalpha(temp_ns[0])) {
		AtomNameStr = temp;
		V5AtomNames = 1;
		input.getline(temp,200);
	}
	
  in_structure.num_each_type.clear();
  in_structure.types.clear();
  in_structure.num_atoms=0;
  char* token_ptr=0;
  token_ptr=strtok(temp," ");
  while (token_ptr != NULL){
    in_structure.num_each_type.push_back(atoi(token_ptr));
    token_ptr=strtok(NULL," ");
  }
  for (i=0;i<in_structure.num_each_type.size();i++){
    for (int j=0;j<in_structure.num_each_type[i];j++){
      in_structure.types.push_back(i);
    }
    in_structure.num_atoms=in_structure.num_atoms+in_structure.num_each_type[i];
  }
  in_structure.num_types=in_structure.num_each_type.size();

  // Type of coordinates (Direct or Cartesian) - only 1st character matters (D/d or C/c).
  // This line might also be the Selective Dynamics line so you must check for that (S/s).
  string stmp;
  getline(input,stmp);
  // Note that if there are spaces at the beginning of the line we have to remove them.
  string stmp_ns=RemoveSpaces(stmp);
  if (stmp_ns.size()==0) {
	
	
    cout << "ERROR:  Found blank line on line 7." << endl;
    cout << "ERROR:  This line should give coordinate type or selective dynamics." << endl;
    cout << "ERROR:  Exiting" << endl;
    exit(1);
  }
	if (stmp_ns[0]!='S' && stmp_ns[0]!='s' && stmp_ns[0]!='D' && stmp_ns[0]!='d' && stmp_ns[0]!='C' && stmp_ns[0]!='c'){
	getline(input,stmp);
        stmp_ns=RemoveSpaces(stmp);  //6.1.1: Added to be able to handle element names on line 7 in VASP5.2
  }

  if (stmp_ns[0]=='S' || stmp_ns[0]=='s'){
    in_structure.isd=1;
    getline(input,stmp);
    stmp_ns=RemoveSpaces(stmp);
  }
  if (stmp_ns[0]=='D' || stmp_ns[0]=='d'){
    in_structure.coord_type[0]=stmp_ns[0];
    in_structure.coord_flag=0;
  }
  else{
    if (stmp_ns[0]=='C' || stmp_ns[0]=='c'){
      in_structure.coord_type[0]=stmp_ns[0];
      in_structure.coord_flag=1;
    }
    else{
      cout << "ERROR:  Did not find coordinate type D/d or C/c." << endl;
      cout << "ERROR:  Exiting" << endl;
      exit(1);
    }
  }
  in_structure.coord_type[1]='\0';

  // Basis atom positions and names  
  int cnt;
  vector<double> v(3,0.0);
  vector<double> vzero(3,0.0);
  // Set default values: names, sd.
  string def_name="H";
  string def_sd = "TTT";
  int num_atoms=in_structure.num_atoms;
  in_structure.names = vector<string> (num_atoms,def_name);
  in_structure.names_were_given = vector<int> (num_atoms,0);
  in_structure.sd = vector<string> (num_atoms,def_sd);
  in_structure.cpos = matrix<double> (num_atoms,vzero);
  in_structure.dpos = matrix<double> (num_atoms,vzero);
  //  Read in all the atom data.    
    // The number of fields on a line can vary but must mean something, which depends on isd.
    // For isd=0:
    //   If nf=3 then there are only atomic positions.
    //   If nf>=4 then there are atomic positions and a name.
    // For isd!=0:
    //   If nf=3 then there are only atomic positions.
    //   If nf=4 then there are atomic positions and a name.
    //   If nf=6 then there are atomic positions and sd.
    //   If nf>=7 then there are atomic positions and sd and a name.
  for (int iat=0;iat<num_atoms;iat++){
    string stmp;
    getline(input,stmp);
    int nf = GetNumFields(stmp);
    int id=0;
    string name;
    // Read in the atom positions.
    for (int ic=0;ic<3;ic++){
      v[ic]=atof(GetNextVal(stmp,id).c_str());
    }
    if (in_structure.coord_flag==0) {
      in_structure.dpos[iat]=v;
      //      in_structure.CalcAllCartCoords();      
    }
    if (in_structure.coord_flag==1) {
      in_structure.cpos[iat]=v;
      //      in_structure.CalcAllDirectCoords();
    }
    // nf = 4 (plus possible comments).
    // Read in the names.    
    if (nf==4 || (nf>=4 && in_structure.isd==0)){ 
      in_structure.names[iat]=GetNextVal(stmp,id);
      in_structure.names_were_given[iat]=1;
    }
    // nf = 6
    if (in_structure.isd!=0 && nf==6){ 
      string sdt;
      for (int ic=0;ic<3;ic++){
	sdt=sdt+GetNextVal(stmp,id);
      }
      in_structure.sd[iat]=sdt;
    }
    // nf = 7
    if (in_structure.isd!=0 && nf>=7){ 
      string sdt;
      for (int ic=0;ic<3;ic++){
	sdt=sdt+GetNextVal(stmp,id);
      }
      in_structure.sd[iat]=sdt;
      in_structure.names[iat]=GetNextVal(stmp,id);
      in_structure.names_were_given[iat]=1;
    }
  } // iat loop
  // get other type of coordinate than what was read in.
  if (in_structure.coord_flag==0) {
    in_structure.CalcAllCartCoords();      
  }
  if (in_structure.coord_flag==1) {
    in_structure.CalcAllDirectCoords();
  }

  // Check that names and number of each type match
  int warn=0;
  int at_cnt=0;
  for (int it=0;it<in_structure.num_each_type.size();it++){
    if (in_structure.num_each_type[it]>0){
      string old_name=in_structure.names[at_cnt];
      at_cnt++;
      for (int jt=1;jt<in_structure.num_each_type[it];jt++){
	if (!(in_structure.names[at_cnt]==old_name)) warn=1;
	at_cnt++;
      }
    }
  }
  if (warn) {
      cerr << "WARNING:  On input your grouping of types does not match your groups of names.  Continuing." << endl;
  }
      
  return input;
}

ostream & operator<<(ostream & output, const structure & out_structure)
{
  output.setf(ios::fixed,ios::floatfield);
  output.precision(10);
  int i;
  output << out_structure.title << endl;
  if (out_structure.neg_scale==0){
    output << out_structure.scale << endl;
  }
  else{
    double s=out_structure.scale;
    double vol=s*s*s*GetVol(out_structure.lat);
    output << -1*vol << endl;
  }
  for (i=0;i<3;i++){
    output << out_structure.lat[i][0] << " " << out_structure.lat[i][1] << " " << out_structure.lat[i][2] << endl;
  }

	if (V5AtomNames) output << AtomNameStr << endl;

  for (i=0;i<out_structure.num_each_type.size();i++){
    output <<  out_structure.num_each_type[i] << " ";
  }
  output << endl;
  if (out_structure.isd!=0) output << "Selective Dynamics" << endl;
  output << out_structure.coord_type[0] << endl;
  for (i=0;i<out_structure.num_atoms;i++){
    if (out_structure.coord_flag==0) output << out_structure.dpos[i][0] << " " 
      << out_structure.dpos[i][1] << " " << out_structure.dpos[i][2];
    if (out_structure.coord_flag==1) output << out_structure.cpos[i][0] << " " 
      << out_structure.cpos[i][1] << " " << out_structure.cpos[i][2];
    if (out_structure.isd!=0) output << " " << out_structure.sd[i][0] << " " << out_structure.sd[i][1] << " " << out_structure.sd[i][2];
    if (out_structure.names_were_given[i]==1) output << " " << out_structure.names[i];
    output << endl;
  }
}

const structure & structure::operator=(const structure & b)
{
  if (this != &b){
    free();
    copy(b);
  }
  return *this;
}

void structure::CalcAllCartCoords()
{
  vector<double> v(3,0.0);
  for (int iat=0;iat<dpos.size();iat++){
    for (int ic=0;ic<3;ic++){
      cpos[iat][ic]=0.0;
      for (int jc=0;jc<3;jc++){
	cpos[iat][ic]=cpos[iat][ic]+dpos[iat][jc]*lat[jc][ic];
      }
    }
  }
}
void structure::CalcAllDirectCoords()
{
  matrix<double> latT=lat.transpose();
  matrix<double> b=cpos.transpose();
  if (num_atoms>0){
    GaussJordan(latT,b);
    dpos=b.transpose();
  }
  else{
    dpos=matrix<double>(0);
  }
}
    



