using namespace std;
#include "GaussJordan.h"
#include "nr.h"
#include "nrutil.h"


// **************************************************
// Function GaussJordan
// **************************************************
/*
   This function uses Gaussian Jordan elimination to solve A*x=b.  It returns the solution x
  and the inverse of A.  Uses numerical recipes.
*/
  void GaussJordan(matrix<double> & A, matrix<double> & B)
{
  double** a;
  double** b;
  int n = A.size();
  int m = B[0].size();
  a=dmatrix(1,n,1,n);
  b=dmatrix(1,n,1,m);
  for (int i=0;i<n;i++){
    for (int j=0;j<n;j++){
      a[i+1][j+1]=A[i][j];
    }
  }
  for (int i=0;i<n;i++){
    for (int j=0;j<m;j++){
      b[i+1][j+1]=B[i][j];
    }
  }

  // tpx
  // Call NR routine.
  gaussj(a,n,b,m);

  for (int i=0;i<n;i++){
    for (int j=0;j<n;j++){
      A[i][j]=a[i+1][j+1];
    }
  }
  for (int i=0;i<n;i++){
    for (int j=0;j<m;j++){
      B[i][j]=b[i+1][j+1];
    }
  }
  free_dmatrix(a,1,n,1,n);
  free_dmatrix(b,1,n,1,m);
}

