// **************************************************
// Printcif
// **************************************************
/*  This funtion prints out structural data in a cif format.
*/

using namespace std;
#include <cmath>
#include <iomanip>
#include <iostream>
#include <cstdio>
#include <cstdlib>
#include "UsefulFuncs.h"
#include "PrintCIF.h"

void PrintCIF(const structure & str)
{
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  structure sstr=str;

  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  char* title=sstr.GetTitle();
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  int coord_flag=sstr.GetCoordFlag();
  int num_atoms=sstr.GetNumAtoms();
  matrix<double> dpos=str.GetDpos();

  matrix<double> cpos=sstr.GetCpos();
  vector<string> names=sstr.GetNames();
  vector<string> at_name_vec=sstr.GetAtomNameVec();
  vector<int> at_num_vec;
  at_num_vec=sstr.GetAtomNumVec();

  // Print out data in cif format
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  //title
  cout << "data_" << title << endl;
  cout << "_pd_phase_name " << title << endl;
  vector<double> dat(6,0.0);
  dat=Getabc_angles(lat);
  double rad2deg=360/(2*3.1415927);
  cout << "_cell_length_a  " << setw(12) << dat[0] <<endl;
  cout << "_cell_length_b  " << setw(12) << dat[1] <<endl;
  cout << "_cell_length_c  " << setw(12) << dat[2] <<endl;
  cout << "_cell_angle_alpha  " << setw(9) << 1.0*dat[3]*rad2deg <<endl;
  cout << "_cell_angle_beta  " << setw(9) << 1.0*dat[4]*rad2deg <<endl;
  cout << "_cell_angle_gamma  " << setw(9) << 1.0*dat[5]*rad2deg <<endl;
  // Output space group - Use P1 for now
  cout << "_symmetry_space_group_name_H-M  'P1'" << endl;
  cout << "_symmetry_Int_Tables_Number  1" << endl;
  cout << "loop_" << endl;
  cout << "_symmetry_equiv_pos_site_id" << endl;
  cout << "_symmetry_equiv_pos_as_xyz_" << endl;
  cout << "  1  x,y,z" << endl;


  //atom posns header
  cout << "loop_" << endl;
  cout << " _atom_site_label" << endl;
  cout << " _atom_site_occupancy" << endl;
  cout << " _atom_site_fract_x" << endl;
  cout << " _atom_site_fract_y" << endl;
  cout << " _atom_site_fract_z" << endl;
  cout << " _atom_site_thermal_displace_type" << endl;
  cout << " _atom_site_B_iso_or_equiv" << endl;
  cout << " _atom_site_type_symbol" << endl;

  // Coordinates in direct

  cout.precision(10);
  for (int i=0;i<num_atoms;i++){
    cout << names[i].c_str()<< i+1 << " 1.0 " 
   	 << setw(12) << dpos[i][0] <<" "
	 << setw(12) << dpos[i][1] <<" "
	 << setw(12) << dpos[i][2] <<" "
	 << " Biso 1.0 " << names[i].c_str() <<endl;
  }
}



