#ifndef UsefulFuncs_comp
#define UsefulFuncs_comp

#include "atom.h"
#include "structure.h"
#include "matrix.h"
#include "constants.h"
#include <vector>
#include <string>
#include <fstream>
#include <complex>
#include "typedefs.h"

double norm(const vector<double> & v);
vector<double> UnitVec(const vector<double> & v);
double getcos(const vector<double> & a, const vector<double> & b);
//vector<vector<double> > MMmult(const matrix<double> & a, const matrix<double> & b);
vector<vector<double> > MMmult(const vector<vector<double> > & a, const vector<vector<double> > & b);
vector<double> MVmult(const matrix<double> & A, const vector<double> & v);
vector<double> MVmult(const vector<vector<double> > & A, const vector<double> & v);
vector<double> VMmult(const vector<double> & v, const matrix<double> & A);
vector<double> VMmult(const vector<int> & v, const matrix<double> & A);
vector<double> VVcross(const vector<double> & a, const vector<double> & b);
double VVdot(const vector<double> & a, const vector<double> & b);
vector<double> SVprod(const double & s, const vector<double> & b);
vector<double> VVsum(const vector<double> & a, const vector<double> & b);
vector<double> VVsum(const vector<double> & a, const vector<int> & b);
vector<double> VVdiff(const vector<double> & a, const vector<double> & b);
double VVprod(const vector<double> & a, const vector<double> & b);
double VVprod(const vector<double> & a, const vector<int> & b);
double GetVol(const matrix<double> & lat);
double GetSignedVol(const matrix<double> & lat);
vector<double> Getabc_angles(const matrix<double> & lat);
int IsTranslationVector(const structure & str, const vector<double> & tvec);
vector<double> vecC2D(const matrix<double> & lat, const vector<double> & vc);
vector<double> vecD2C(const matrix<double> & lat, const vector<double> & vd);
matrix<double> GetClat(const vector<double> & abc_angles);
int Nint(double x);
vector<double> AtomCDisp(const atom & at1, const atom & at2);
double AtomDist(const atom & at1, const atom & at2);
void GetStrNeighData(const structure & str, const double cutoff, 
		  matrix<atom> & neigh_mat);
void GetGoodShellPoints(matrix<double> & points, const structure & str, 
			const int & ns, const double & rmin, 
			const double & rmax, const string & sname,
			matrix<atom> & neigh_mat);
void GetNeighData(const vector<atom> & atom_vec, const structure & str, 
		  const double & rmin, const double & rmax,
		  matrix<atom> & neigh_mat);
void GetXray(const structure & str, vector<double> & dist,
	     vector<double> & sf, const double & lambda,
	     vector<double> & scatt_fact, vector<double> & mass,
	     vector<double> & twoB_vec);
matrix<double> RecipLat(const matrix<double> & lat);
void Vout(const vector<double> & a, ostream & out);
void Vout(const vector<int> & a, ostream & out);
void Vout(const vector<string> & a, ostream & out);
void Mout(const matrix<double> & m, ostream & out);
void Mout(const vector<vector<double> > & m, ostream & out);
int VVequal(const vector<double> & a, const vector<double> & b);
int VVequal(const vector<int> & a, const vector<int> & b);
structure GetLTCell(const matrix<double> & lt, const structure & str);
structure GetLTFVCell(const vector<double> & nvec, const double angle, const structure & str);
matrix<double> MMmult(const matrix<double> & A, const matrix<double> & B);
structure GetIntpolStr(structure strA, structure strB, const double & f,
		       const string & path_flag, const double & displace, const int & which_atom);
double DebyeWallerFactor(const double & theta, const double & lambda,
			 const double & temp, const double & debye_temp,
			 const double & mass);
string RemoveSpaces(const string & s);
string RemoveWhiteSpaces(const string & s);
int GetNumFields(const string & s);
string GetNextVal(const string & s, int & id);
string PaddedNumString(const int num, const int ndigits);
void InFileExistCheck(const string & routine, const string & filename, 
		      ifstream & file_to_check, ostream & outf);
double GetMag(const complex<double> z);
double Normal(const double & x, const double & mu, const double & sigma);
vector<double> CompAperpB(const vector<double> & a, const vector<double> & b);
vector<double> SmoothFunc(const vector<double> & func, const double & sigma);
int FindIfStringInStream(const string & key, istream & instream);
int Sign(const double & x);
int SignNoZero(const double & x);
void GetUnitCellRep(const vector<double> & in_pos, 
		    vector<double> & p_cell0, 
		    vector<int> & n_cell, 
		    const matrix<double> & lat, 
		    const int & coord_flag);
atom ConvertAtomToLat(const atom & in_at, const matrix<double> & lat);
vector<double> Sort_abc_angles(const vector<double> & abc_angles);
#endif







