// rtparams object. 

using namespace std;
#include <iostream>
#include <iomanip>
#include "projdata.h"


// Constructors
projdata::projdata()
{
  sp=0;
  rspin=2;
  nl_max=4; // for s,p,d,f orbitals
  nlm_max=16; // for s,p,d,f orbitals
  nlmtot_max=20; // for s,p,d,f orbitals + p,d,f,all totals
  nl=nl_max; // Default to max values
  nlm=nlm_max; // Default to max values
  nlmtot=nlmtot_max; // Default to max values
  LMnames = vector<string> (nlm);
  LMnames[0]="S     ";
  LMnames[1]="Py    ";
  LMnames[2]="Pz    ";
  LMnames[3]="Px    ";
  LMnames[4]="Dxy   ";
  LMnames[5]="Dyz   ";
  LMnames[6]="Dz2   ";
  LMnames[7]="Dxz   ";
  LMnames[8]="Dx2-y2";
  LMnames[9]="F1    ";
  LMnames[10]="F2    ";
  LMnames[11]="F3    ";
  LMnames[12]="F4    ";
  LMnames[13]="F5    ";
  LMnames[14]="F6    ";
  LMnames[15]="F7    ";
  Lnames = vector<string> (nl);
  Lnames[0]="S     ";
  Lnames[1]="P     ";
  Lnames[2]="D     ";
  Lnames[3]="F     ";
  LLMnames = vector<string> (nlm+4);
  LLMnames[0]="S     ";
  LLMnames[1]="Py    ";
  LLMnames[2]="Pz    ";
  LLMnames[3]="Px    ";
  LLMnames[4]="Ptot  ";
  LLMnames[5]="Dxy   ";
  LLMnames[6]="Dyz   ";
  LLMnames[7]="Dz2   ";
  LLMnames[8]="Dxz   ";
  LLMnames[9]="Dx2-y2";
  LLMnames[10]="Dtot  ";
  LLMnames[11]="F1    ";
  LLMnames[12]="F2    ";
  LLMnames[13]="F3    ";
  LLMnames[14]="F4    ";
  LLMnames[15]="F5    ";
  LLMnames[16]="F6    ";
  LLMnames[17]="F7    ";
  LLMnames[18]="Ftot  ";
  LLMnames[19]="Tot   ";
}
  
void projdata::Print(ostream & outf)
{

  outf.precision(3);
  outf.setf(ios::fixed,ios::floatfield);
  outf.setf(ios::left,ios::adjustfield);
  
  int ioncnt;
  int w1=5;
  int w2=6;
  string key;


  outf << endl;
  outf << "**************************************************" << endl;
  outf << endl;
  outf << "WARNING:  There is some confusing business about these projections." << endl;
  outf << "WARNING:  To get results that match the vasp summations you may need" << endl;
  outf << "WARNING:  to alter the vasp source code.  See the convasp help by" << endl;
  outf << "WARNING:  typing convasp -h .  Read the section on -pocc." << endl;
  outf << endl;
  outf << "The basic data" << endl;
  outf << "nlm " << nlm << endl;
  outf << "nkpts " << nkpts << endl;
  outf << "nbands " << nbands << endl;
  outf << "nions " << nions << endl;
  outf << "ntypes " << ntypes << endl;
  outf << "type  " << "num of that type " << endl; 
  for (int it=0;it<ntypes;it++){
    outf << it << "         " << num_each_type[it] << endl;
  }
  outf << "Spin polarization (0=no,1=yes) " << sp << endl;
  outf << "rspin (spin degeneracy) " << rspin << endl;
  outf << "Fermi Weights (nbands X nkpts)" << endl;
  outf << "Band  kpt1  kpt2 ..." << endl;
  outf << "Up spin" << endl;
  for (int ib=0;ib<wfermi_u.size();ib++){
    outf << " " << ib;
    for (int ik=0;ik<wfermi_u[ib].size();ik++){
      outf << "   " << wfermi_u[ib][ik];
    }
    outf << endl;
  }
  outf << "Down spin" << endl;
  for (int ib=0;ib<wfermi_d.size();ib++){
    outf << " " << ib;
    for (int ik=0;ik<wfermi_d[ib].size();ik++){
      outf << "   " << wfermi_d[ib][ik];
    }
    outf << endl;
  }

  outf << "kpt values and weights" << endl;
  for (int ik=0;ik<wkpt.size();ik++){
    outf << "  " << ik+1 << "  " 
	 << kpts[ik][0] << "  " 
	 << kpts[ik][1] << "  " 
	 << kpts[ik][2] << "      " 
	 << wkpt[ik] << endl;
  }

  outf << "Band energies" << endl;
  outf << "   " << "kpt   " << "band  " << "energy (up/dn) " << endl;
  for (int ik=0;ik<ener_k_b_u.size();ik++){
    for (int ib=0;ib<ener_k_b_u[ik].size();ib++){
      outf << "   " << setw(5) << ik+1 << " " << setw(5) << ib+1 << " " << setprecision(5) << ener_k_b_u[ik][ib];
      if (sp) outf << "  " << setprecision(5) << ener_k_b_d[ik][ib];
      outf << endl;
    }
  }

  outf.precision(3);

  outf << "Projections" << endl;
  outf << "   " << "type  " << "ion   " << "kpt   " << "band  " << "lm    " << "projection (up/dn)" << endl;
  ioncnt=-1;
  for (int it=0;it<pdat_u.size();it++){
    for (int iat=0;iat<num_each_type[it];iat++){
      ioncnt++;
      for (int ik=0;ik<pdat_u[it].size();ik++){
	for (int ib=0;ib<pdat_u[it][ik][iat].size();ib++){
	  for (int ilm=0;ilm<pdat_u[it][ik][iat][ib].size();ilm++){
	    outf << "   ";
	    outf << setw(w2) << it+1;
	    outf << setw(w2) << ioncnt+1;
	    outf << setw(w2) << ik+1;
	    outf << setw(w2) << ib+1;
	    outf << "   " << LMnames[ilm];
	    outf << "  " << setw(w2) << pdat_u[it][ik][iat][ib][ilm];
	    if (sp) outf << "  " << setw(w2) << pdat_d[it][ik][iat][ib][ilm];
	    outf << endl;
	  }
	}
      }
    }
  }


  // Occupations ION:KPT:BND:LM
  outf << endl;
  outf << "**************************************************" << endl;
  outf << endl;
  outf << "Occupations vs. ION:KPT:BND:LM" << endl;
  outf << endl;
  key="ION:KPT:BND:LM";
  outf << "Up spin"<< endl;
  outf << "   " << "ion   " << "type  " << "kpt   " << "band  ";
  for (int ilm=0;ilm<LLMnames.size();ilm++){
    outf << LLMnames[ilm] << "  ";
  }
  outf << endl;
  ioncnt=-1;
  for (int it=0;it<ntypes;it++){
    for (int iat=0;iat<num_each_type[it];iat++){
      ioncnt++;
      for (int ik=0;ik<nkpts;ik++){
	for (int ib=0;ib<nbands;ib++){
	  outf << "   " << setw(w1) << ioncnt+1;
	  outf << " " << setw(w1) << it+1;
	  outf << " " << setw(w1) << ik+1;
	  outf << " " << setw(w1) << ib+1;
	  outf << " " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][0];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][1];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][2];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][3];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][1];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][4];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][5];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][6];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][7];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][8];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][2];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][9];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][10];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][11];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][12];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][13];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][14];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][15];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][3];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][4];
	  outf << " " << key;
	  outf << endl;
	}
      }
    }
  }
  if (sp){
    outf << "Down spin"<< endl;
    outf << "   " << "ion   " << "type  " << "kpt   " << "band  ";
    for (int ilm=0;ilm<LLMnames.size();ilm++){
      outf << LLMnames[ilm] << "  ";
    }
    outf << endl;
    ioncnt=-1;
    for (int it=0;it<ntypes;it++){
      for (int iat=0;iat<num_each_type[it];iat++){
	ioncnt++;
	for (int ik=0;ik<nkpts;ik++){
	  for (int ib=0;ib<nbands;ib++){
	    outf << "   " << setw(w1) << ioncnt+1;
	    outf << " " << setw(w1) << it+1;
	    outf << " " << setw(w1) << ik+1;
	    outf << " " << setw(w1) << ib+1;
	    outf << " " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][0];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][1];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][2];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][3];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][1];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][4];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][5];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][6];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][7];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][8];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][2];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][9];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][10];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][11];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][12];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][13];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][14];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][15];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][3];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][4];
	  outf << " " << key;
	    outf << endl;
	  }
	}
      }
    }
    outf << "Up-Down spin"<< endl;
    outf << "   " << "ion   " << "type  " << "kpt   " << "band  ";
    for (int ilm=0;ilm<LLMnames.size();ilm++){
      outf << LLMnames[ilm] << "  ";
    }
    outf << endl;
    ioncnt=-1;
    for (int it=0;it<ntypes;it++){
      for (int iat=0;iat<num_each_type[it];iat++){
	ioncnt++;
	for (int ik=0;ik<nkpts;ik++){
	  for (int ib=0;ib<nbands;ib++){
	    outf << "   " << setw(w1) << ioncnt+1;
	    outf << " " << setw(w1) << it+1;
	    outf << " " << setw(w1) << ik+1;
	    outf << " " << setw(w1) << ib+1;
	    outf << " " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][0]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][0];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][1]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][1];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][2]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][2];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][3]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][3];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][1]-occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][1];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][4]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][4];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][5]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][5];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][6]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][6];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][7]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][7];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][8]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][8];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][2]-occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][2];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][9]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][9];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][10]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][10];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][11]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][11];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][12]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][12];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][13]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][13];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][14]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][14];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_lm_u[ioncnt][ik][ib][15]-occ_vs_ion_kpt_bnd_lm_d[ioncnt][ik][ib][15];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][3]-occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][3];
	    outf << "  " << setw(w2) << occ_vs_ion_kpt_bnd_l_u[ioncnt][ik][ib][4]-occ_vs_ion_kpt_bnd_l_d[ioncnt][ik][ib][4];
	    outf << " " << key;
	    outf << endl;
	  }
	}
      }
    }



  }


  // Occupations ION:BND:LM
  outf << endl;
  outf << "**************************************************" << endl;
  outf << endl;
  outf << "Occupations vs. ION:BND:LM" << endl;
  outf << endl;
  key="ION:BND:LM";
  outf << "Up spin"<< endl;
  outf << "   " << "ion   " << "type  " << "band  ";
  for (int ilm=0;ilm<LLMnames.size();ilm++){
    outf << LLMnames[ilm] << "  ";
  }
  outf << endl;
  ioncnt=-1;
  for (int it=0;it<ntypes;it++){
    for (int iat=0;iat<num_each_type[it];iat++){
      ioncnt++;
      for (int ib=0;ib<nbands;ib++){
      outf << "   " << setw(w1) << ioncnt+1;
      outf << " " << setw(w1) << it+1;
      outf << " " << setw(w1) << ib+1;
      outf << " " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][0];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][1];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][2];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][3];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_l_u[ioncnt][ib][1];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][4];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][5];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][6];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][7];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][8];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_l_u[ioncnt][ib][2];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][9];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][10];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][11];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][12];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][13];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][14];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][15];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_l_u[ioncnt][ib][3];
      outf << "  " << setw(w2) << occ_vs_ion_bnd_l_u[ioncnt][ib][4];
      outf << " " << key;
      outf << endl;
      }
    }
  }
  if (sp){
    outf << "Down spin"<< endl;
    outf << "   " << "ion   " << "type  " << "band  ";
    for (int ilm=0;ilm<LLMnames.size();ilm++){
      outf << LLMnames[ilm] << "  ";
    }
    outf << endl;
    ioncnt=-1;
    for (int it=0;it<ntypes;it++){
      for (int iat=0;iat<num_each_type[it];iat++){
	ioncnt++;
	for (int ib=0;ib<nbands;ib++){
	  outf << "   " << setw(w1) << ioncnt+1;
	  outf << " " << setw(w1) << it+1;
	  outf << " " << setw(w1) << ib+1;
	  outf << " " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][0];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][1];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][2];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][3];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_l_d[ioncnt][ib][1];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][4];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][5];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][6];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][7];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][8];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_l_d[ioncnt][ib][2];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][9];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][10];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][11];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][12];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][13];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][14];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_d[ioncnt][ib][15];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_l_d[ioncnt][ib][3];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_l_d[ioncnt][ib][4];
	  outf << " " << key;
	  outf << endl;
	}
      }
    }
    outf << "Up-Down spin"<< endl;
    outf << "   " << "ion   " << "type  " << "band  ";
    for (int ilm=0;ilm<LLMnames.size();ilm++){
      outf << LLMnames[ilm] << "  ";
    }
    outf << endl;
    ioncnt=-1;
    for (int it=0;it<ntypes;it++){
      for (int iat=0;iat<num_each_type[it];iat++){
	ioncnt++;
	for (int ib=0;ib<nbands;ib++){
	  outf << "   " << setw(w1) << ioncnt+1;
	  outf << " " << setw(w1) << it+1;
	  outf << " " << setw(w1) << ib+1;
	  outf << " " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][0]-occ_vs_ion_bnd_lm_d[ioncnt][ib][0];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][1]-occ_vs_ion_bnd_lm_d[ioncnt][ib][1];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][2]-occ_vs_ion_bnd_lm_d[ioncnt][ib][2];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][3]-occ_vs_ion_bnd_lm_d[ioncnt][ib][3];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_l_u[ioncnt][ib][1]-occ_vs_ion_bnd_l_d[ioncnt][ib][1];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][4]-occ_vs_ion_bnd_lm_d[ioncnt][ib][4];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][5]-occ_vs_ion_bnd_lm_d[ioncnt][ib][5];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][6]-occ_vs_ion_bnd_lm_d[ioncnt][ib][6];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][7]-occ_vs_ion_bnd_lm_d[ioncnt][ib][7];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][8]-occ_vs_ion_bnd_lm_d[ioncnt][ib][8];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_l_u[ioncnt][ib][2]-occ_vs_ion_bnd_l_d[ioncnt][ib][2];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][9]-occ_vs_ion_bnd_lm_d[ioncnt][ib][9];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][10]-occ_vs_ion_bnd_lm_d[ioncnt][ib][10];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][11]-occ_vs_ion_bnd_lm_d[ioncnt][ib][11];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][12]-occ_vs_ion_bnd_lm_d[ioncnt][ib][12];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][13]-occ_vs_ion_bnd_lm_d[ioncnt][ib][13];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][14]-occ_vs_ion_bnd_lm_d[ioncnt][ib][14];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_lm_u[ioncnt][ib][15]-occ_vs_ion_bnd_lm_d[ioncnt][ib][15];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_l_u[ioncnt][ib][3]-occ_vs_ion_bnd_l_d[ioncnt][ib][3];
	  outf << "  " << setw(w2) << occ_vs_ion_bnd_l_u[ioncnt][ib][4]-occ_vs_ion_bnd_l_d[ioncnt][ib][4];
	  outf << " " << key;
	  outf << endl;
	}
      }
    }
  }// if sp


  // Occupations ION:KPT:LM
  outf << endl;
  outf << "**************************************************" << endl;
  outf << endl;
  outf << "Occupations vs. ION:KPT:LM" << endl;
  outf << endl;
  key="ION:KPT:LM";
  outf << "Up spin"<< endl;
  outf << "   " << "ion   " << "type  " << "kpt   ";
  for (int ilm=0;ilm<LLMnames.size();ilm++){
    outf << LLMnames[ilm] << "  ";
  }
  outf << endl;
  ioncnt=-1;
  for (int it=0;it<ntypes;it++){
    for (int iat=0;iat<num_each_type[it];iat++){
      ioncnt++;
      for (int ik=0;ik<nkpts;ik++){
      outf << "   " << setw(w1) << ioncnt+1;
      outf << " " << setw(w1) << it+1;
      outf << " " << setw(w1) << ik+1;
      outf << " " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][0];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][1];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][2];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][3];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_l_u[ioncnt][ik][1];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][4];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][5];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][6];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][7];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][8];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_l_u[ioncnt][ik][2];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][9];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][10];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][11];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][12];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][13];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][14];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][15];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_l_u[ioncnt][ik][3];
      outf << "  " << setw(w2) << occ_vs_ion_kpt_l_u[ioncnt][ik][4];
      outf << " " << key;
      outf << endl;
      }
    }
  }
  if (sp){
    outf << "Down spin"<< endl;
    outf << "   " << "ion   " << "type  " << "kpt   ";
    for (int ilm=0;ilm<LLMnames.size();ilm++){
      outf << LLMnames[ilm] << "  ";
    }
    outf << endl;
    ioncnt=-1;
    for (int it=0;it<ntypes;it++){
      for (int iat=0;iat<num_each_type[it];iat++){
	ioncnt++;
	for (int ik=0;ik<nkpts;ik++){
	  outf << "   " << setw(w1) << ioncnt+1;
	  outf << " " << setw(w1) << it+1;
	  outf << " " << setw(w1) << ik+1;
	  outf << " " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][0];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][1];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][2];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][3];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_l_d[ioncnt][ik][1];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][4];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][5];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][6];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][7];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][8];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_l_d[ioncnt][ik][2];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][9];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][10];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][11];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][12];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][13];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][14];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_d[ioncnt][ik][15];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_l_d[ioncnt][ik][3];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_l_d[ioncnt][ik][4];
	  outf << " " << key;
	  outf << endl;
	}
      }
    }
    outf << "Up-Down spin"<< endl;
    outf << "   " << "ion   " << "type  " << "kpt   ";
    for (int ilm=0;ilm<LLMnames.size();ilm++){
      outf << LLMnames[ilm] << "  ";
    }
    outf << endl;
    ioncnt=-1;
    for (int it=0;it<ntypes;it++){
      for (int iat=0;iat<num_each_type[it];iat++){
	ioncnt++;
	for (int ik=0;ik<nkpts;ik++){
	  outf << "   " << setw(5) << ioncnt+1;
	  outf << " " << setw(5) << it+1;
	  outf << " " << setw(5) << ik+1;
	  outf << " " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][0]-occ_vs_ion_kpt_lm_d[ioncnt][ik][0];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][1]-occ_vs_ion_kpt_lm_d[ioncnt][ik][1];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][2]-occ_vs_ion_kpt_lm_d[ioncnt][ik][2];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][3]-occ_vs_ion_kpt_lm_d[ioncnt][ik][3];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_l_u[ioncnt][ik][1]-occ_vs_ion_kpt_l_d[ioncnt][ik][1];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][4]-occ_vs_ion_kpt_lm_d[ioncnt][ik][4];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][5]-occ_vs_ion_kpt_lm_d[ioncnt][ik][5];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][6]-occ_vs_ion_kpt_lm_d[ioncnt][ik][6];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][7]-occ_vs_ion_kpt_lm_d[ioncnt][ik][7];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][8]-occ_vs_ion_kpt_lm_d[ioncnt][ik][8];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_l_u[ioncnt][ik][2]-occ_vs_ion_kpt_l_d[ioncnt][ik][2];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][9]-occ_vs_ion_kpt_lm_d[ioncnt][ik][9];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][10]-occ_vs_ion_kpt_lm_d[ioncnt][ik][10];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][11]-occ_vs_ion_kpt_lm_d[ioncnt][ik][11];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][12]-occ_vs_ion_kpt_lm_d[ioncnt][ik][12];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][13]-occ_vs_ion_kpt_lm_d[ioncnt][ik][13];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][14]-occ_vs_ion_kpt_lm_d[ioncnt][ik][14];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_lm_u[ioncnt][ik][15]-occ_vs_ion_kpt_lm_d[ioncnt][ik][15];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_l_u[ioncnt][ik][3]-occ_vs_ion_kpt_l_d[ioncnt][ik][3];
	  outf << "  " << setw(w2) << occ_vs_ion_kpt_l_u[ioncnt][ik][4]-occ_vs_ion_kpt_l_d[ioncnt][ik][4];
	  outf << " " << key;
	  outf << endl;
	}
      }
    }
  }


  // Occupations ION:LM
  outf << endl;
  outf << "**************************************************" << endl;
  outf << endl;
  outf << "Occupations vs. ION:LM" << endl;
  outf << endl;
  key="ION:LM";
  outf << "Up spin"<< endl;
  outf << "   " << "ion   " << "type  ";
  for (int ilm=0;ilm<LLMnames.size();ilm++){
    outf << LLMnames[ilm] << "  ";
  }
  outf << endl;
  ioncnt=-1;
  for (int it=0;it<ntypes;it++){
    for (int iat=0;iat<num_each_type[it];iat++){
      ioncnt++;
      outf << "   " << setw(w1) << ioncnt+1;
      outf << " " << setw(w1) << it+1; 
      outf << " " << setw(w2) << occ_vs_ion_lm_u[ioncnt][0];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][1];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][2];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][3];
      outf << "  " << setw(w2) << occ_vs_ion_l_u[ioncnt][1];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][4];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][5];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][6];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][7];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][8];
      outf << "  " << setw(w2) << occ_vs_ion_l_u[ioncnt][2];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][9];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][10];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][11];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][12];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][13];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][14];
      outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][15];
      outf << "  " << setw(w2) << occ_vs_ion_l_u[ioncnt][3];
      outf << "  " << setw(w2) << occ_vs_ion_l_u[ioncnt][4];
      outf << " " << key;
     outf << endl;
    }
  }
  if (sp){
    outf << "Down spin"<< endl;
    outf << "   " << "ion   " << "type  ";
    for (int ilm=0;ilm<LLMnames.size();ilm++){
      outf << LLMnames[ilm] << "  ";
    }
    outf << endl;
    ioncnt=-1;
    for (int it=0;it<ntypes;it++){
      for (int iat=0;iat<num_each_type[it];iat++){
	ioncnt++;
	outf << "   " << setw(w1) << ioncnt+1;
	outf << " " << setw(w1) << it+1;
	outf << " " << setw(w2) << occ_vs_ion_lm_d[ioncnt][0];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][1];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][2];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][3];
	outf << "  " << setw(w2) << occ_vs_ion_l_d[ioncnt][1];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][4];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][5];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][6];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][7];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][8];
	outf << "  " << setw(w2) << occ_vs_ion_l_d[ioncnt][2];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][9];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][10];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][11];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][12];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][13];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][14];
	outf << "  " << setw(w2) << occ_vs_ion_lm_d[ioncnt][15];
	outf << "  " << setw(w2) << occ_vs_ion_l_d[ioncnt][3];
	outf << "  " << setw(w2) << occ_vs_ion_l_d[ioncnt][4];
	outf << " " << key;
	outf << endl;
      }
    }
    outf << "Up-Down spin"<< endl;
    outf << "   " << "ion   " << "type  ";
    for (int ilm=0;ilm<LLMnames.size();ilm++){
      outf << LLMnames[ilm] << "  ";
    }
    outf << endl;
    ioncnt=-1;
    for (int it=0;it<ntypes;it++){
      for (int iat=0;iat<num_each_type[it];iat++){
	ioncnt++;
	outf << "   " << setw(5) << ioncnt+1;
	outf << " " << setw(5) << it+1;
	outf << " " << setw(w2) << occ_vs_ion_lm_u[ioncnt][0]-occ_vs_ion_lm_d[ioncnt][0];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][1]-occ_vs_ion_lm_d[ioncnt][1];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][2]-occ_vs_ion_lm_d[ioncnt][2];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][3]-occ_vs_ion_lm_d[ioncnt][3];
	outf << "  " << setw(w2) << occ_vs_ion_l_u[ioncnt][1]-occ_vs_ion_l_d[ioncnt][1];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][4]-occ_vs_ion_lm_d[ioncnt][4];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][5]-occ_vs_ion_lm_d[ioncnt][5];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][6]-occ_vs_ion_lm_d[ioncnt][6];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][7]-occ_vs_ion_lm_d[ioncnt][7];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][8]-occ_vs_ion_lm_d[ioncnt][8];
	outf << "  " << setw(w2) << occ_vs_ion_l_u[ioncnt][2]-occ_vs_ion_l_d[ioncnt][2];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][9]-occ_vs_ion_lm_d[ioncnt][9];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][10]-occ_vs_ion_lm_d[ioncnt][10];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][11]-occ_vs_ion_lm_d[ioncnt][11];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][12]-occ_vs_ion_lm_d[ioncnt][12];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][13]-occ_vs_ion_lm_d[ioncnt][13];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][14]-occ_vs_ion_lm_d[ioncnt][14];
	outf << "  " << setw(w2) << occ_vs_ion_lm_u[ioncnt][15]-occ_vs_ion_lm_d[ioncnt][15];
	outf << "  " << setw(w2) << occ_vs_ion_l_u[ioncnt][3]-occ_vs_ion_l_d[ioncnt][3];
	outf << "  " << setw(w2) << occ_vs_ion_l_u[ioncnt][4]-occ_vs_ion_l_d[ioncnt][4];
	outf << " " << key;
	outf << endl;
      }
    }
  }// sp

}




