/*plotMD
Script designed by Amy Bengston, extracts useful data from VASP MD output,
generates plots using gnuplot
*/

#include <iostream>
#include <fstream>

using namespace std; 
void plotMD();


void plotMD()
{
	//get the data
	system("grep T= OSZICAR > OSZICAR.dat");
        system("grep pressure OUTCAR > PRESSURE.dat");
	
	//generate python plotting script
	ofstream plotScript;
	plotScript.open ("plot.p");
	plotScript << 
"# Gnuplot script for plotting data in file \"OSZICAR.dat\"\n\
# Amy Bengtson 31 January 2011\n\
\n\
#set terminal postscript eps color lw 15 \"Helvetica\" 20\n\
#set out 'plot.eps'\n\
\n\
\n\
set size 1,1\n\
set origin 0,0\n\
set multiplot\n\
\n\
\n\
# TEMPERATURE\n\
set size 0.34,0.34                      \n\
set origin 0,0.67\n\
set   autoscale                        # scale axes automatically\n\
unset log                              # remove any log-scaling\n\
unset label                            # remove any previous labels\n\
set xtic auto                          # set xtics automatically\n\
set ytic auto                          # set ytics automatically\n\
set title \"TEMPERATURE\"\n\
set xlabel \"MD step\"\n\
set ylabel \"Temperature (K)\"\n\
plot    \"OSZICAR.dat\" using 1:3 title 'Column' with lines\n\
\n\
# E\n\
set size 0.34,0.34\n\
set origin 0,0.33\n\
set   autoscale                        # scale axes automatically\n\
unset log                              # remove any log-scaling\n\
unset label                            # remove any previous labels\n\
set xtic auto                          # set xtics automatically\n\
set ytic auto                          # set ytics automatically\n\
set title \"TOTAL FREE ENERGY: E\"\n\
set xlabel \"MD step\"\n\
set ylabel \"E (eV/cell)\"\n\
plot    \"OSZICAR.dat\" using 1:5 title 'Column' with lines\n\
\n\
\n\
\n\
# F\n\
set size 0.34,0.34\n\
set origin 0,0\n\
set   autoscale                        # scale axes automatically\n\
unset log                              # remove any log-scaling\n\
unset label                            # remove any previous labels\n\
set xtic auto                          # set xtics automatically\n\
set ytic auto                          # set ytics automatically\n\
set title \"FREE ENERGY: F\"\n\
set xlabel \"MD step\"\n\
set ylabel \"F\"\n\
plot    \"OSZICAR.dat\" using 1:7 title 'Column' with lines\n\
\n\
# E0\n\
set size 0.34,0.34\n\
set origin 0.33,0.67\n\
set   autoscale                        # scale axes automatically\n\
unset log                              # remove any log-scaling\n\
unset label                            # remove any previous labels\n\
set xtic auto                          # set xtics automatically\n\
set ytic auto                          # set ytics automatically\n\
set title \"POTENTIAL ENERGY: E0\"\n\
set xlabel \"MD step\"\n\
set ylabel \"E0 (eV/cell)\"\n\
plot    \"OSZICAR.dat\" using 1:9 title 'Column' with lines\n\
\n\
# EK\n\
set size 0.34,0.34\n\
set origin 0.33,0.33\n\
set   autoscale                        # scale axes automatically\n\
unset log                              # remove any log-scaling\n\
unset label                            # remove any previous labels\n\
set xtic auto                          # set xtics automatically\n\
set ytic auto                          # set ytics automatically\n\
set title \"KINECTIC ENERGY: EK\"\n\
set xlabel \"MD step\"\n\
set ylabel \"EK (eV/cell)\"\n\
plot    \"OSZICAR.dat\" using 1:11 title 'Column' with lines\n\
\n\
# SP\n\
set size 0.34,0.34\n\
set origin 0.33,0.0\n\
set   autoscale                        # scale axes automatically\n\
unset log                              # remove any log-scaling\n\
unset label                            # remove any previous labels\n\
set xtic auto                          # set xtics automatically\n\
set ytic auto                          # set ytics automatically\n\
set title \"POTENTIAL ENERGY OF NOSE THERMOSTAT: SP\"\n\
set xlabel \"MD step\"\n\
set ylabel \"SP (eV/cell)\"\n\
plot    \"OSZICAR.dat\" using 1:13 title 'Column' with lines\n\
\n\
# SK\n\
set size 0.34,0.34\n\
set origin 0.67,0.67\n\
set   autoscale                        # scale axes automatically\n\
unset log                              # remove any log-scaling\n\
unset label                            # remove any previous labels\n\
set xtic auto                          # set xtics automatically\n\
set ytic auto                          # set ytics automatically\n\
set title \"KINECTIC ENERGY OF NOSE THERMOSTAT: SK\"\n\
set xlabel \"MD step\"\n\
set ylabel \"SK (eV/cell)\"\n\
plot    \"OSZICAR.dat\" using 1:15 title 'Column' with lines\n\
\n\
# PRESSURE\n\
set size 0.34,0.34\n\
set origin 0.67,0.33\n\
set   autoscale                        # scale axes automatically\n\
unset log                              # remove any log-scaling\n\
unset label                            # remove any previous labels\n\
set xtic auto                          # set xtics automatically\n\
set ytic auto                          # set ytics automatically\n\
set title \"PRESSURE\"\n\
set xlabel \"MD step\"\n\
set ylabel \"P (kbar)\"\n\
plot    \"PRESSURE.dat\" using 4 title 'Column' with lines\n\
\n\
\n\
unset multiplot\n\
\n\
\n\
";
	plotScript.close();
	//set permission
	system("chmod +x plot.p");
	//run the script
	system("./plot.p");
	//clean up
	system("rm plot.p");
return 0;
}
 
