// **************************************************
// PrintACE
// **************************************************
//  This funtion prints out structural data in a ace format (for CaRIne).

using namespace std;
#include <iostream>
#include <cmath>
#include "UsefulFuncs.h"
#include "PrintACE.h"

void PrintACE(const structure & str)
{
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  structure sstr=str;

  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  char* title=sstr.GetTitle();
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  int coord_flag=sstr.GetCoordFlag();
  int num_atoms=sstr.GetNumAtoms();
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  vector<string> names=sstr.GetNames();
  vector<string> at_name_vec=sstr.GetAtomNameVec();
  vector<int> at_num_vec;
  at_num_vec=sstr.GetAtomNumVec();

  // Print out data in ace format
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  cout << "CaRIne Crystallography 3.0" << endl;
  cout << "Cell Standard ASCII File=*.ACE" << endl;
  cout << "File Version Number=1" << endl;
  cout << endl;
//    cout << "Colors Refered to: Mendele" << (char) 239 << "ev Table" << endl;
  cout << "Colors Refered to: RGB Components" << endl;
  cout << endl;
  cout << "----------------------" << endl;
  cout << "Cell Definition" << endl;
  cout << "----------------------" << endl;
  cout << endl;
  cout << "Cell Parameters(" << (char) 197 << " and " << (char) 176 << ")"  << endl;
  // Get cell data
  vector<double> data(6,0.0);
  double rad2deg=360/(2*3.1415927);
  data=Getabc_angles(lat);
  cout << "a=" << data[0] << '\t' 
       << "b=" << data[1] << '\t' 
       << "c=" << data[2] << endl;
  cout << "alpha=" << data[3]*rad2deg << '\t'
         << "beta=" << data[4]*rad2deg << '\t'
	 << "gamma=" << data[5]*rad2deg << endl;
  cout << endl;
  cout << "System=not used" << endl;
  cout << "Space Group Number=not used" << endl;
  cout << "Number of positions in Cell=" << num_atoms << endl;
  cout << endl;
  // If cartesian convert to direct
  cout << "Atom" << '\t' << "Oxi." << '\t' << "X" << '\t' << "Y" << '\t' << "Z" << '\t' 
       << "R(A)" << '\t' << "Occ." << endl;
  for (int i=0;i<num_atoms;i++){
    cout << names[i] << '\t' << "0" << '\t' << dpos[i][0] << '\t' << dpos[i][1] << '\t' << dpos[i][2] << '\t' 
         << "1.00" << '\t' << "1.0000" << endl;
  }
}



