// **************************************************
// PrintAngles
// **************************************************
//  This funtion prints out 3 lattice vectors in cartesian coordinates.

using namespace std;
#include <cmath>
#include <string>
#include <iostream>
#include <fstream>
#include <iomanip>
#include "atom.h"
#include <vector>
#include "UsefulFuncs.h"
#include "PrintDist.h"
#include "constants.h"

void PrintAngles(const structure & str, const double cutoff)
{  
  int MAX_NUM_ANGLE=21;
  cout.setf(ios::left, ios::adjustfield);
  cout.setf(ios::fixed, ios::floatfield);
  //  cout.setf(ios::fixed,ios::floatfield);
  //  cout.precision(10);
  matrix<atom> neigh_mat;
  GetStrNeighData(str,cutoff,neigh_mat);
  double tol=1e-15;

  // Output header
  cout << "All atoms identified by" << endl;
  cout << "[basis number]  [name]  [n1 n2 n3 (unit cell)]" << endl;

  // Print out angles
  cout << endl;
  cout << "********************" << " ANGLES " << "********************" << endl;
  cout << "Atoms of increasing indentation are A, B, and C." << endl;
  cout << "The given angles are those made by the lines A to B and B to C (atom B is at the vertex)." << endl;
  cout << "The given angles are always taken 0<=angle<=180." << endl;
  cout << "Angles are only calculated for the first " << MAX_NUM_ANGLE-1 << " neighbors within " << cutoff <<  endl;
  matrix<int> uc(3,3);
  vector<int> pflag(3,0);
  for (int ia1=0;ia1<neigh_mat.size();ia1++){
    pflag[0]=1;
    atom a1=neigh_mat[ia1][0];
    vector<double> a1pos=a1.GetCpos();
    uc[0]=a1.GetUnitCell();
    for (int ia2=0;(ia2<neigh_mat[ia1].size() && ia2<MAX_NUM_ANGLE);ia2++){
      pflag[1]=1;
      atom a2=neigh_mat[ia1][ia2];
      vector<double> a2pos=a2.GetCpos();
      uc[1]=a2.GetUnitCell();
      for (int ia3=0;(ia3<neigh_mat[a2.GetNum()].size() && ia3<MAX_NUM_ANGLE);ia3++){
	pflag[2]=1;
	atom a3=neigh_mat[a2.GetNum()][ia3];      
	vector<double> a3pos=a3.GetCpos();
	uc[2]=a3.GetUnitCell();
	vector<double> a1toa2(3);
	vector<double> a2toa3(3);
	vector<double> a1toa3(3);
	for (int ic=0;ic<3;ic++){
	  a1toa2[ic]=a2pos[ic]-a1pos[ic];
	  a2toa3[ic]=a3pos[ic]-a2pos[ic];
	  a1toa3[ic]=a3pos[ic]-a1pos[ic];
	}
	if (norm(a1toa2)>tol 
	    && norm(a2toa3)>tol 
	    && norm(a1toa3)>tol
	    ){
	  double angle = 180-RAD2DEG*acos(getcos(a1toa2,a2toa3));
	  if (fabs(getcos(a1toa2,a2toa3)-1)<tol) angle=180;
	  if (fabs(getcos(a1toa2,a2toa3)+1)<tol) angle=0;
	  if (pflag[0]){
	    cout << setw(4) << a1.GetNum()+1 
		 << " " << setw(4) << a1.GetName().c_str()
		 << " " << setw(3) << uc[0][0] 
		 << " " << setw(3) << uc[0][1] 
		 << " " << setw(3) << uc[0][2] 
		 << "   ";
	    cout << endl;
	  } // if plag[0]
	  if (pflag[1]){
	    cout << "     " << setw(4) << a2.GetNum()+1 
		 << " " << setw(4) << a2.GetName().c_str()
		 << " " << setw(3) << uc[1][0] 
		 << " " << setw(3) << uc[1][1] 
		 << " " << setw(3) << uc[1][2] 
		 << "   ";
	    cout << endl;
	  } // if plag[1]
	  if (pflag[2]){
	  cout << "          " << setw(4) << a3.GetNum()+1 
	       << " " << setw(4) << a3.GetName().c_str()
	       << " " << setw(3) << uc[2][0] 
	       << " " << setw(3) << uc[2][1] 
	       << " " << setw(3) << uc[2][2] 
	       << "   ";
	  cout << "     "  << setprecision(4) << angle << endl;
	  } // if plag[2]
	  // Set print flags not to print until next looping.
	  for (int i=0;i<3;i++){
	    pflag[i]=0;
	  }
	} // if norms>tol
      } // a3
    } // a2
  } // a1

} // end routine


