// **************************************************
// PrintData
// **************************************************
//  This funtion prints out structural data.

using namespace std;
#include <cmath>
#include <iostream>
#include "UsefulFuncs.h"
#include "PrintData.h"

void PrintData(const structure & str)
{
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(10);
  structure sstr=str;

  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  char* title=sstr.GetTitle();
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  int coord_flag=sstr.GetCoordFlag();
  int num_atoms=sstr.GetNumAtoms();
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  vector<string> names=sstr.GetNames();
  vector<string> at_name_vec=sstr.GetAtomNameVec();
  vector<int> at_num_vec;
  at_num_vec=sstr.GetAtomNumVec();

  // Print out structural data
  cout.setf(ios::fixed,ios::floatfield);
  double vol;
  vol=GetVol(lat);
  cout << "Real space a b c alpha beta gamma: ";
  cout.precision(10);
  double rad2deg=360/(2*3.1415927);
  vector<double> data(6,0.0);
  data=Getabc_angles(lat);
  cout << data[0] << " "
    << data[1] << " "
    << data[2] << " ";
  cout.precision(3);
  cout << data[3]*rad2deg << " "
    << data[4]*rad2deg << " "
    << data[5]*rad2deg << endl;
  cout.precision(4);
  cout << "Real space Volume: " << vol << endl;
  matrix<double> rlat=RecipLat(lat);
  cout << "Reciprocal space lattice:" << endl;
  cout.precision(10);
  for (int i=0;i<3;i++){
    cout <<"  "<<rlat[i][0]<<"  "<<rlat[i][1]<<" "<<rlat[i][2]<<endl;
  }
  cout << "Reciprocal space a b c alpha beta gamma: ";
  data=Getabc_angles(rlat);
  cout << data[0] << " "
    << data[1] << " "
    << data[2] << " ";
  cout.precision(3);
  cout << data[3]*rad2deg << " "
    << data[4]*rad2deg << " "
    << data[5]*rad2deg << endl;
  cout.precision(4);
  double rvol=GetVol(rlat);
  cout << "Reciprocal space Volume: " << rvol << endl;
}



