// **************************************************
// PrintDist
// **************************************************
//  This funtion prints out the xray scattering pattern

using namespace std;
#include <cmath>
#include <string>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <algorithm>
#include "UsefulFuncs.h"
#include "PrintXray.h"
#include "constants.h"

class ids_cmp{
public:
  int operator()(const vector<double> & a, const vector<double> & b)
  {return a[0]>b[0];} // Sorts in increasing order.
};

class hkl_cmp{
 public:
  int operator()(const vector<int> & a, const vector<int> & b)
    {
      int na=a[0]*100+a[1]*10+a[2];
      int nb=b[0]*100+b[1]*10+b[2];
      return na>nb;
      }
};
/*
      int t=1;
      int f=1;
      if (a[0]<b[0]){ 
	return false;
      }
      else{
	if (a[1]<b[1]){ 
	  return false;
	}
	else{
	  if (a[2]<b[2]){ 
	    return false;
	  }
	  else{
	    return true;
	  }
	}
      }
    }
*/

/* 
**************************************************
CorrectionFactor
**************************************************
  This function contains any correction factors that need
to be multiply the structure factors.  Theta should be in rads.
Includes
  - Lorentz-Polarization
*/
double CorrectionFactor(const double & th)
{
  //  double th=theta*TWOPI/360.0; // rad->degrees
  double correction_factor=1;
  // Lorentz-polarization
  double lp;
  double c2t=cos(2.0*th);
  double st=sin(th);
  if (th>0 || th<0){
    lp=(1.0+c2t*c2t)/(st*st);
  }
  else{
    lp=1;
  }
  correction_factor=correction_factor*lp;
  return lp;
}  

void PrintXray(const structure & str, const double & lambda)
{  
  cout.setf(ios::left, ios::adjustfield);
  cout.setf(ios::fixed, ios::floatfield);
  //  cout.setf(ios::fixed,ios::floatfield);
  int PREC_DEFAULT=4;
  cout.precision(PREC_DEFAULT);

  int num_atoms=str.GetNumAtoms();
  vector<string> names=str.GetNames();
  vector<double> dist,sf;
  vector<double> scatt_fact(num_atoms,0.0);
  vector<double> mass(num_atoms,0.0);
  vector<double> twoB_vec(num_atoms,0.0);
  GetXray(str,dist,sf,lambda,scatt_fact,mass,twoB_vec);

  double tol=1.0E-5;
  int w1=4; // int
  int w2=12; // some doubles
  int w3=20; // Integrated intensities
  int tlen=sf.size();
  int len=Nint(pow(tlen,1.0/3.0));
  int kmx=(len-1)/2; // len should be odd.

  // Sort by theta (reverse sort by distance).
  // Define an id pointer to sort.
  vector<double> v(5);
  vector<vector<double> > ids(tlen,v);
  for (int i0=-kmx;i0<=kmx;i0++){
    for (int i1=-kmx;i1<=kmx;i1++){
      for (int i2=-kmx;i2<=kmx;i2++){
        int ii0=i0+kmx;
        int ii1=i1+kmx;
        int ii2=i2+kmx;
        int id=ii2+ii1*len+ii0*len*len;
	ids[id][0]=dist[id];
	ids[id][1]=id;
	ids[id][2]=i0;
	ids[id][3]=i1;
	ids[id][4]=i2;
      } // i2
    } // i1
  } // i0

  sort(ids.begin(),ids.end(),ids_cmp());

// Add corrections to all the amplitudes.
// Get max amplitude for normalizing and percentages.
  double ampmax=1E-8;
  for (int i0=-kmx;i0<=kmx;i0++){
    for (int i1=-kmx;i1<=kmx;i1++){
      for (int i2=-kmx;i2<=kmx;i2++){
        int ii0=i0+kmx;
        int ii1=i1+kmx;
        int ii2=i2+kmx;
        int id1=ii2+ii1*len+ii0*len*len;
        int id=(int) ids[id1][1];
	ii0=(int) ids[id1][2];
	ii1=(int) ids[id1][3];
	ii2=(int) ids[id1][4];
        double theta=0;
	if (dist[id]>0){
          double term=lambda/(2.0*dist[id]);
	  if (term<=1){
            theta=asin(term);
	    if (theta>tol){
	      sf[id]=sf[id]*CorrectionFactor(theta);
	      if (sf[id]>ampmax) {ampmax=sf[id];}
	    } // if theta>tol
          } // if term<=1
        } // if dist>0
      } // i2
    } // i1
  } // i0

// Print out all data.
  cout << "Wavelength (Ang) = " << lambda << endl; 
  cout << "Atom Name  ScattFact   Mass(amu)   B(Ang)(DW=exp(-B*sin(theta)^2/lambda^2))" << endl;
  for (int iat=0;iat<num_atoms;iat++){
    cout <<setw(4)<<iat+1<<" "<<setw(4)<<names[iat]<<" "<<setw(w2)<<scatt_fact[iat]<<setw(w2)<<KILOGRAM2AMU*mass[iat]<<setw(w2)<<1E+20*twoB_vec[iat]/2.0<<endl;
  }
  cout << "******************** All data ********************" << endl; 
  cout << "2*theta      Intensity            h    k    l    dist         keyword " << endl;
  for (int i0=-kmx;i0<=kmx;i0++){
    for (int i1=-kmx;i1<=kmx;i1++){
      for (int i2=-kmx;i2<=kmx;i2++){
        int ii0=i0+kmx;
        int ii1=i1+kmx;
        int ii2=i2+kmx;
        int id1=ii2+ii1*len+ii0*len*len;
        int id=(int) ids[id1][1];
	ii0=(int) ids[id1][2];
	ii1=(int) ids[id1][3];
	ii2=(int) ids[id1][4];
        double theta=0;
	if (dist[id]>0){
          double term=lambda/(2.0*dist[id]);
	  if (term<=1){
            theta=asin(term);
            theta=theta*360.0/TWOPI; // rad->degrees
	    if (theta>tol) cout 
	      <<setw(w2)<<2.0*theta<<" " // angle
	      <<setw(w3)<<sf[id]<<" " // sf
	      <<setw(w1)<<ii0<<" " // h
              <<setw(w1)<<ii1<<" " // k
	      <<setw(w1)<<ii2<<" " // l
              <<setw(w2)<<dist[id]<<" " // dist
              << "SINGLE"
	      << endl;
          } // if term<=1
        } // if dist>0
      } // i2
    } // i1
  } // i0

// Now group everything at the same distance together and only store one entry for each distance.
// Choose hkl such that h is the largest, k the second, and l the third.
// Multiply the sf by the number of degenerate points at that distance.
// Get max integrated intensity for normalizations and percentages.
  double intmax=1E-8;
  double odist=dist[(int)ids[0][1]]; // Initialize odsit to first distance.
  double osf=sf[(int)ids[0][1]]; // Initialize osf to first distance.
  vector<vector<int> > hkl_list;
  vector<int> hkl(3);
  vector<vector<double> > data;
  for (int i0=-kmx;i0<=kmx;i0++){
    for (int i1=-kmx;i1<=kmx;i1++){
      for (int i2=-kmx;i2<=kmx;i2++){
        int ii0=i0+kmx;
        int ii1=i1+kmx;
        int ii2=i2+kmx;
        int id1=ii2+ii1*len+ii0*len*len;
        int id=(int) ids[id1][1];
	ii0=(int) ids[id1][2];
	ii1=(int) ids[id1][3];
	ii2=(int) ids[id1][4];
	double pdist=dist[id]; // Get present distance.
	// Create vector of all the hkl values with the same distance
        if (fabs(pdist-odist)<tol) { // Add present h,k,l to hkl_list.
	  hkl[0]=ii0;		    
	  hkl[1]=ii1;		    
	  hkl[2]=ii2;		    
	  hkl_list.push_back(hkl);
         }
	else{ // Store one hkl, dist, total sf, multiplicity of point in data vector and then reset hkl_list vector to new hkl.
          vector<double> datav(6);
	  // Sort hkl
          sort(hkl_list.begin(),hkl_list.end(),hkl_cmp());
          datav[0]=(double) hkl_list[0][0];  
          datav[1]=(double) hkl_list[0][1];  
          datav[2]=(double) hkl_list[0][2];  
          datav[3]=odist;
          datav[4]=osf*hkl_list.size(); 
          datav[5]=hkl_list.size(); 
	  data.push_back(datav);
	  if (datav[4]>intmax) {intmax=datav[4];}
          vector<int> v(0);
	  hkl_list= vector<vector<int> > (0,v);
	  hkl[0]=ii0;		    
	  hkl[1]=ii1;		    
	  hkl[2]=ii2;		    
	  hkl_list.push_back(hkl);
	}
	odist=pdist;
	osf=sf[id];
      } // i2
    } // i1
  } // i0

// Output grouped data
  cout << "******************** Grouped data ********************" << endl; 
  cout << "2*theta      IntIntensity         %ofMaxInt    h    k    l    dist         mult. correction    keyword " << endl;
  for (int i=0;i<data.size();i++){
    double theta=0;
    if (data[i][3]>0){
      double term=lambda/(2.0*data[i][3]);
      if (term<=1){
        theta=asin(term);
        theta=theta*360.0/TWOPI; // rad->degrees
        if (theta>tol) cout
          <<setw(w2)<<2.0*theta<<" " // angle
          <<setw(w3)<<setprecision(2)<<data[i][4]<<setprecision(PREC_DEFAULT)<<" " // sf
          <<setw(w2)<<setprecision(2)<<100*data[i][4]/intmax<<setprecision(PREC_DEFAULT)<<" " // % max sf
          <<setw(w1)<<(int)data[i][0]<<" " // h
          <<setw(w1)<<(int)data[i][1]<<" " // k
	  <<setw(w1)<<(int)data[i][2]<<" " // l
          <<setw(w2)<<data[i][3]<<" " // dist
          <<setw(5)<<(int)data[i][5]<<" " // mult.
          <<setw(w2)<<CorrectionFactor(theta*TWOPI/360.0)<<" " // correction.
          << " GROUP"
	  << endl;
      } // if term<=1
    } // if dist>0
  } // for

// Output data to plot
  cout << "******************** To Plot data ********************" << endl; 
  cout << "2*theta      Amplitude    keyword " << endl;
  for (int i=0;i<data.size();i++){
    double theta=0;
    if (data[i][3]>0){
      double term=lambda/(2.0*data[i][3]);
      if (term<=1){
        theta=asin(term);
        theta=theta*360.0/TWOPI; // rad->degrees
        if (theta>tol){
          // initial 0.
          cout
          <<setw(w2)<<2.0*theta<<" " // angle
          <<setw(w2)<<"0"<<" " // sf
          << "TOPLOT"
	  << endl;
          // true value of sf/intmax.
          cout
          <<setw(w2)<<2.0*theta<<" " // angle
          <<setw(w2)<<setprecision(2)<<100*data[i][4]/intmax<<setprecision(PREC_DEFAULT)<<" " // sf
          << "TOPLOT"
	  << endl;
          // final 0.
          cout
          <<setw(w2)<<2.0*theta<<" " // angle
          <<setw(w2)<<"0"<<" " // sf
          << "TOPLOT"
	  << endl;
	  // tpx
	} // if theta<tol
      } // if term<=1
    } // if dist>0
  } // for

} // end routine










