#ifndef matrix_compile
#define matrix_compile

#include <vector>
#include <assert.h>
#include "typedefs.h"
#include <complex>

/*using namespace std;*/

template<class T>
class matrix{
public:
// constructors
  matrix();
  matrix(const int m);
  matrix(const int m,const int n);
  matrix(const int m,const int n,const T & inT);
  matrix(const int m,const std::vector<T> & inTvec);
// accessors
  void print() const;
  int size() const;
  matrix<T> transpose() const;
  /*
  vector<vector<T> >::iterator begin();
  vector<vector<T> >::iterator end();
  */
// operator
  std::vector<T> & operator [] (const int index);
  const std::vector<T> & operator [] (const int index) const;
  const matrix<T> &operator=(const matrix<T> &b);
// mutators
  void push_back(const std::vector<T> & inTvec);
  void pop_back();
  void vecvec2mat(const std::vector<std::vector<T> > & inVV);
  void vec2mat(const std::vector<T> & inV);
  void clear();
  void insert(const int & id, const std::vector<T> & inV);
  void erase(const int id);
  void erase_col(const int id);
private:
  std::vector<std::vector<T> > mat;
};



// constructor
template<class T>
matrix<T>::matrix() // default
{}
template<class T>

matrix<T>::matrix(const int m) // specifying rows
{ 
	std::vector<T> v;
  mat=std::vector<std::vector<T> > (m,v);
}

template<class T>
matrix<T>::matrix(const int m, const int n) // specifying rows and columns
{ 
	std::vector<T> v(n);
  mat=std::vector<std::vector<T> > (m,v);
}

template<class T>
matrix<T>::matrix(const int m, const std::vector<T> & inTvec) // specifying rows as vectors.
{ 
  mat=std::vector<std::vector<T> > (m,inTvec);
}

template<class T>
matrix<T>::matrix(const int m, const int n, const T & inT) // specifying rows and columns and initial values
{ 
	std::vector<T> v(n);
  mat=std::vector<std::vector<T> > (m,v);
  for (int i=0;i<mat.size();i++){
    for (int j=0;j<mat[i].size();j++){
      mat[i][j]=inT;
    }
  }
}

// accessors

template<class T>
void matrix<T>::print() const
{ 
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(4);
  for (int i=0;i<mat.size();i++){
      cout << "  ";
    for (int j=0;j<mat[i].size();j++){
      cout << " " << mat[i][j];
    }
    cout << endl;
  }
}

template<class T>
int matrix<T>::size() const
{ 
  return mat.size();
}

template<class T>
matrix<T> matrix<T>::transpose() const{
  matrix<T> tmat;
  int m=mat.size();
  if (m==0) return tmat;
  int n=mat[0].size();
  tmat = matrix<T> (n,m);
  for (int i=0;i<m;i++){
    for (int j=0;j<n;j++){
      tmat[j][i]=mat[i][j];
    }
  }
  return tmat;
}

/*
template<class T>
std::vector<std::vector<T> >::iterator matrix<T>::begin()
{
  return mat.begin();
}

template<class T>
std::vector<std::vector<T> >::iterator matrix<T>::end()
{
  return mat.end();
}
*/

// operator
template<class T>
std::vector<T> & matrix<T>::operator [] (const int index)
{
  assert(index>=0 && index<=mat.size());
  return mat[index];
}

template<class T>
const std::vector<T> & matrix<T>::operator [] (const int index) const
{
  assert(index>=0 && index<=mat.size());
  return mat[index];
}

template<class T>
const matrix<T> & matrix<T>::operator=(const matrix<T> &b)
{
  if (this != &b){
    int m=b.mat.size();
    int n=0;
    mat=std::vector<std::vector<T> > (m);
    for (int i=0;i<m;i++){
      n=b.mat[i].size();
      mat[i]=std::vector<T> (n);
      for (int j=0;j<n;j++){
	mat[i][j]=b.mat[i][j];
      }
    }
  }
  return *this;
}

// mutators
template<class T>
void matrix<T>::push_back(const std::vector<T> & inTvec)
{
  mat.push_back(inTvec);
}
template<class T>
void matrix<T>::pop_back()
{
  mat.pop_back();
}

template<class T>
void matrix<T>::vecvec2mat(const std::vector<std::vector<T> > & inVV)
{
  mat=std::vector<std::vector<T> > (inVV.size());
  for (int i=0;i<mat.size();i++){
    mat[i]=std::vector<T> (inVV[i].size());
    for (int j=0;j<mat[i].size();j++){
      mat[i][j]=inVV[i][j];
    }
  }
}

template<class T>
void matrix<T>::clear()
{
  mat.clear();
}

template<class T>
void matrix<T>::vec2mat(const std::vector<T>  & inV)
{
  mat=std::vector<std::vector<T> > (1);
  mat[0]=std::vector<T> (inV.size());
  for (int j=0;j<mat[0].size();j++){
    mat[0][j]=inV[j];
  }
}

template<class T>
void matrix<T>::insert(const int & id, const std::vector<T> & inV)
{
  mat.insert(mat.begin()+id,inV);
}


template<class T>
void matrix<T>::erase(const int id)
{
	std::vector<std::vector<T> >::iterator p=mat.begin()+id;
  mat.erase(p);
}

template<class T>
void matrix<T>::erase_col(const int id)
{
  for (int i=0;i<mat.size();i++){
	  std::vector<T>::iterator p=mat[i].begin()+id;
    if (id<mat[i].size()) mat[i].erase(p);
  }
}

#endif





