// **************************************************
// PrintPlaton
// **************************************************
// This funtion prints out structural data in a format for PLATON

using namespace std;
#include <iostream>
#include <cstdio>
#include "PrintPlaton.h"
#include "UsefulFuncs.h"

#ifndef PI
#define PI               3.14159265358979323846
#endif

double angle_deg(const double& x1,const double& x2,const double& x3,
		 const double& y1,const double& y2,const double& y3) { 
  return acos((x1*y1+x2*y2+x3*y3)/(sqrt(x1*x1+x2*x2+x3*x3)*sqrt(y1*y1+y2*y2+y3*y3)))*(180/PI);
}

double lenght(const double& x1,const double& x2,const double& x3) {
  return sqrt(x1*x1+x2*x2+x3*x3);
}


void PrintPlaton(const structure & str,bool Platon_EQUAL,bool Platon_EXACT,
		 double Platon_ang,double Platon_d1,double Platon_d2,double Platon_d3, ostream & sout) {
  sout.setf(ios::fixed,ios::floatfield);
  structure sstr=str;
  
  // Set scale to 1 so you don't need to rescale coordinates.
  sstr.ReScale(1.0);
  matrix<double> lat=sstr.GetLat();
  vector<int> num_each_type;
  num_each_type=sstr.GetNumEachType();
  int ntypes=num_each_type.size();
  int coord_flag=sstr.GetCoordFlag();
  matrix<double> cpos=sstr.GetCpos();
  matrix<double> dpos=sstr.GetDpos();
  vector<string> names=sstr.GetNames();
  vector<int> names_were_given=sstr.GetNamesWereGiven();
  vector<string> el_names(5);
  int i,j,k;
  el_names[0]="He";
  el_names[1]="Li";
  el_names[2]="Be";
  el_names[3]="B";
  el_names[4]="C";
  // Print out data in Platon format

  sout << "TITL " << sstr.GetTitle() << endl;
  sout.precision(8);
  /*
  printf("CELL %6.5f %6.5f %6.5f %6.5f %6.5f %6.5f \n",
	 lenght(lat[0][0],lat[0][1],lat[0][2]),
	 lenght(lat[1][0],lat[1][1],lat[1][2]),
	 lenght(lat[2][0],lat[2][1],lat[2][2]),
	 angle_deg(lat[1][0],lat[1][1],lat[1][2],lat[2][0],lat[2][1],lat[2][2]),
	 angle_deg(lat[0][0],lat[0][1],lat[0][2],lat[2][0],lat[2][1],lat[2][2]),
	 angle_deg(lat[0][0],lat[0][1],lat[0][2],lat[1][0],lat[1][1],lat[1][2]));
  */
  sout << "CELL "
       << lenght(lat[0][0],lat[0][1],lat[0][2]) << " "
       << lenght(lat[1][0],lat[1][1],lat[1][2]) << " "
       <<  lenght(lat[2][0],lat[2][1],lat[2][2]) << " "
       << angle_deg(lat[1][0],lat[1][1],lat[1][2],lat[2][0],lat[2][1],lat[2][2]) << " "
       << angle_deg(lat[0][0],lat[0][1],lat[0][2],lat[2][0],lat[2][1],lat[2][2]) << " "
       << angle_deg(lat[0][0],lat[0][1],lat[0][2],lat[1][0],lat[1][1],lat[1][2])  
       << endl;
  // sout << sstr.GetNumAtoms() << endl;
  sout.precision(10); 
  k=0;
  for (i=0;i<ntypes;i++)
    for(j=1;j<=num_each_type[i];j++) {
//      printf("%c%i ",i+66,j); 
      if (names_were_given[k]){
	sout << names[k] << j << " ";
      }
      else{ // Must make up a name
	if (i>(el_names.size()-1)){ // No more default names so make everything W
	  sout << "W" << j << " ";
	}
	else{ // Use default names
	  sout << el_names[i] << j << " ";
	}
      }
      //      printf("%14.10f %14.10f %14.10f \n",dpos[k][0],dpos[k][1],dpos[k][2]);
      sout << dpos[k][0]<< " " << dpos[k][1] << " " << dpos[k][2] << endl;
      k++;
	}
  // sout <<  num_each_type[i] << " ";
  //  int num_atoms=cpos.size();
  // for (i=0;i<num_atoms;i++)
  //  if (coord_flag==0) 
  // sout << " " << dpos[i][0] << " " << dpos[i][1] << " " << dpos[i][2] << endl;
  //  printf("CALC ADDSYM ");
  sout << "CALC ADDSYM " << endl;
  if(Platon_EQUAL) sout << "EQUAL ";
  if(Platon_EXACT) sout << "EXACT ";
  if(Platon_ang!=1.0 || Platon_d1!=0.25 || Platon_d2!=0.25 || Platon_d3!=0.25)
    //    printf("%14.10f %14.10f %14.10f %14.10f ",Platon_ang,Platon_d1,Platon_d2,Platon_d3);  printf("\n");
    sout << Platon_ang << " " << Platon_d1 << " " << Platon_d2 << " " << Platon_d3 << endl;

}

// Stefano Curtarolo MIT 2002
