	using namespace std;
	#include <iostream>
        #include "PrintHelp.h"

	// **************************************************
	// PrintUsage
	// **************************************************
	//  This function prints out the usage documentation.
	void PrintUsage(ostream & outf)
	{
	outf <<" \n\
	convasp -h  \n\
	convasp -cart | -direct | -scale s | -names A B ...| \n\
		-numnames A B ...| \n\
		-incell | -incompact | -insphere radius | \n\
                -prim | -ltcell file | \n\
                -ltcellfv v1 v2 v3 phi | -sd A B ... | \n\
		-nosd | -shift Sx Sy Sz [cCdC]| -rm_copies | -gulp | \n\
		-findsym [tolerance_relative: default 1.0e-5] | \n\
		-platon [EQUAL] [EXACT] [ang d1 d2 d3] | \n\
		-xyz n1 n2 n3 | -msi | -ace | -cif | -pdb | \n\
                -data | -ndata | -dist d | -disp d | -angle d | \n\
                -xray l | -shell ns r1 r2 name dens | \n\
                -supercell file | -rdf rmax nbins sigma  | \n\
		-compare a b c d e f g h k j i l | -ewald [eta] | \n\
                -wignerseitz (or -ws) | -xyzwignerseitz (or -xyzws) \n\
		| -incompact | -sewald eta \n\
		< POSCAR \n\
	convasp -clat a b c alpha beta gamma \n\
	convasp -chgint CHGCAR \n\
	convasp -intpol f1 f2 n x \n\
	convasp -rbanal n x \n\
	convasp -spline npts < file \n\
	convasp -make_strlist OUTCAR XDATCAR \n\
	convasp -join_strlist strlist1 strlist2 \n\
	convasp -supercell_strlist sc.txt strlist \n\
	convasp -raytrace file \n\
	convasp -pocc PROOUT \n\
	convasp -pdos pdos.in PROOUT \n\
        convasp -sumpdos pdos.in DOSCAR \n\
	convasp -bands PROOUT < POSCAR \n\
	convasp -planedens dens2d.in CHGCAR \n\
	convasp -chgdiff CHGCAR1 CHGCAR2 \n\
	convasp -rdfcmp rmax nbins sigma nshmax POSCAR1 POSCAR2 \n\
	convasp -rbdist POSCAR1 POSCAR2 n|N|e|E \n\
	convasp -mom POSCAR \n\
	convasp -setcm cm1 cm2 cm3 POSCAR \n\
	convasp -niggli POSCAR \n\
	convasp -cmp_str POSCAR1 POSCAR2 rcut \n\
	convasp -data1 POSCAR1 rcut \n\
        convasp -kmesh kptdens POSCAR \n\
	" << endl;
	}

	// **************************************************
	// PrintHelp
	// **************************************************
	//  This function prints out all the help documentation.
	void PrintHelp(ostream & outf)
	{  
	  outf << "Version " << CVERSION << " Last updated Aug 6 by Curtarolo" << endl;
	  outf << "People who have helped: \n\
                     Dane Morgan - Most routines.\n\
                     Stefano Curtarolo - Some routines.\n\
                     Marnix Wagemaker - Compiled for C++ V3.2\n\
	             Ceder group at MIT - Extensive testing.  \n\
                  " << endl;
	  outf << "Basic usage" << endl;
	  PrintUsage(outf);
	  outf << endl;
	  outf << "\n\
	Detailed help on all possible functionality\n\
\n\
	In the following I will often refer to a POSCAR file.  This \n\
	is a VASP file whose basic format is as follows\n\
\n\
	    TITLE\n\
	    1.000000000000000\n\
	    2.8619048823981780     .0000000000000000     .0000000000000000\n\
	    .0000000000000000    3.8747900333665160     .0000000000000000\n\
	    .0000000000000000     .0000000000000000    6.0115639071437000\n\
	    2   2   4\n\
	    Direct\n\
	    .2500000000000000   .7500000000000000   .0930470335145194\n\
	    .7500000000000000   .2500000000000000   .9069529664854805\n\
	    .2500000000000000   .7500000000000000   .6375942304936022\n\
	    .7500000000000000   .2500000000000000   .3624057695063981\n\
	    .2500000000000000   .2500000000000000   .1517340127860196\n\
	    .7500000000000000   .7500000000000000   .8482659872139805\n\
	    .2500000000000000   .2500000000000000   .5813623924505660\n\
	    .7500000000000000   .7500000000000000   .4186376075494341\n\
\n\
	The following items can be included in the POSCAR file and \n\
	will be handled correctly by the code: names after the atoms,\n\
	selective dynamics formatting, negative scale to give volume,\n\
	arbitrary spaces on a line (although not extra newlines).\n\
\n\
	convasp -h|help|\n\
	     Gives this help information.\n\
	convasp -cart < POSCAR\n\
	     Outputs to standard out a new POSCAR file with atom \n\
	     positions in cartesian coordinates.\n\
	convasp -direct < POSCAR      \n\
	     Outputs to standard out a new POSCAR file with \n\
	     atom positions in direct (fractional) coordinates.\n\
	convasp -scale s < POSCAR\n\
	     Outputs POSCAR file giving same volume as input but \n\
	     with scale = s.\n\
	convasp -names A1 A2 ... < POSCAR \n\
	     Outputs to standard out a POSCAR file with names \n\
	     A1,A2, ... after the atom positions.  Each name \n\
	     Ai is assigned to all atoms of type i.  If \n\
	     there are too few names then the remaining\n\
	     types are not given names.  This is useful for \n\
	     cases where large numbers of atoms must be named, \n\
	     like when creating an xyz file.\n\
	convasp -numnames A1 A2 ... < POSCAR \n\
	     Same as names except appends an increasing integer to\n\
	     each different atom type.  Starts counting at 1 again\n\
	     for each new atom type.  Any names not given at \n\
	     end are treated as H.\n\
	convasp -incell < POSCAR         \n\
	     Outputs to standard out a POSCAR file with all \n\
	     atoms mapped to their images within the unit cell.\n\
	convasp -incompact < POSCAR         \n\
             Similar to -incell \n\
	     Outputs to standard out a POSCAR file where all the \n\
             atoms are mapped through the unit and neighbours cells \n\
             to minimixe the shortest possible bond with an adjacent atom \n\
             This option is very useful if you run big and complicate \n\
             molecules where atoms exit of the unit cell and you have \n\
             problems understanding where they are because visualization\n\
             packages do not show bonds anymore ... \n\
             Anyway, it is easier to test than to describe. (SC 6 Aug 04)\n\
	convasp -insphere radius < POSCAR         \n\
             Similar to -incell and -incompact \n\
             Outputs in XYZ format (rasmol standard), the \n\
             positions of all the atoms inside a sphere of radius \n\
             \"radius\" centered in the origin. \n\
             If you want another origin, you must shift the atoms first.\n\
             The radius is in carterian format. (SC 6 Aug 04)\n\
	convasp -prim <POSCAR\n\
	     Outputs to standard out a POSCAR file with a \n\
	     primitive unit cell.  In the primitive cell finding \n\
	     function I look for a primitive cell by considering \n\
	     as candidate cell vectors every possible triad of 3 \n\
	     vectors that can be made from the original cell vectors \n\
	     or the basis vectors which translate the lattice onto \n\
	     itself.  I then take the triad with the smallest volume \n\
	     to be the primitive cell.  If there are multiple \n\
	     candidates I take the ones with the largest projections \n\
	     onto the original lattice vectors.  See the rouinte \n\
	     GetPrim.cc for more information.  WARNING:  This is not \n\
	     carefully tested and almost certainly has bugs!!!\n\
	convasp -ltcell file < POSCAR   \n\
	     Outputs to standard out the linear tranform of the input\n\
	     POSCAR file.  This simpy multiplies cell parameters and\n\
	     atom positions by the 3x3 matrix in file.  This can \n\
	     be used to rotate the cell, swap x and y coordinates, etc.\n\
	     This cannot create a supercell (see -suopercell for that).  \n\
	     File should consist of 9 numbers, giving elements \n\
	     a11,a12,a13,a21,a22,a23,a31,a32,a33 of the linear \n\
	     transformation, respectively.  They can be on one or \n\
	     mulitple lines.\n\
	convasp -ltcellfv v1 v2 v3 phi < POSCAR   \n\
	     Rotates the lattice vectors and atoms by angle phi (in degrees)\n\
             around vector (v1,v2,v3).  Outputs to standard out the new\n\
             POSCAR.  This can be used to rotate the cell, swap x and y \n\
             coordinates, etc.\n\
	convasp -supercell file < POSCAR\n\
	     Outputs to standard out a supercell of the input\n\
	     POSCAR file.  This lattice vectors of the supercell are\n\
	     given by multiplying the original cell parameters by\n\
	     by the 3x3 matrix in file.  The supercell need not be integral\n\
	     combinations of the original lattice vectors, although using\n\
	     fraction may cause you to end up with a lattice inequivalent \n\
	     to your original.  This can be used to build big supercells, \n\
	     swap lattice vectors, etc.  This cannot do a rotation (see \n\
	     -ltcell for that).  File should consist of 9 numbers, giving \n\
	     elements a11,a12,a13,a21,a22,a23,a31,a32,a33 of the 3x3 \n\
	     supercell matrix, respectively.  They can be on one or\n\
	     mulitple lines.\n\
	convasp -sd A1 A2 ... < POSCAR     \n\
	     Outputs to standard out a POSCAR with selective dynamics \n\
	     formatting.  Ai gives the selective dynamics setting for \n\
	     atoms of type i and has the form Ai=TTT,TTF,etc..  If there\n\
	     are fewer Ai than atom types then the remaining types are \n\
	     defaulted to TTT.\n\
	convasp -nosd < POSCAR\n\
	     Outputs to standard out a POSCAR without selective dynamics\n\
	     formatting.  Combined with the above -sd option this allows\n\
	     easy movement between POSCAR files with and without selective \n\
	     dynamics formatting.\n\
	convasp -shift Sx Sy Sz [cCdD] < POSCAR\n\
	     Outputs to standard out a POSCAR with all positions shifted by \n\
	     S=(Sx,Sy,Sz).  The shifted is added to the positions.  The shift\n\
	     is assumed to be given in Cartesian coordinates unless you specify\n\
             c or C for Cartesian or d or D for direct at the end.\n\
	convasp -rm_copies < POSCAR\n\
	     Outputs to standard out a POSCAR where only the first appearance of\n\
	     each position has been kept.  Useful if you have multiple copies of\n\
	     atoms at the same positions for some reason.\n\
	convasp -gulp <POSCAR          \n\
	     Outputs to standard out a gulp formatted input file based on \n\
	     the POSCAR input file.  The formatting is for a distance \n\
	     calculation in gulp.  If you want atom names you must put \n\
	     them after each atom position in the POSCAR file (see -names).\n\
	     If any names are missing they are defaulted to H.\n\
	convasp -xyz n1 n2 n3 < POSCAR\n\
	     Outputs to standard out an xyz file based on the POSCAR input\n\
	     file.  This can be used as input for rasmol, xmol, etc..\n\
	     If you want atom names you must put them after each atom \n\
	     position in the POSCAR file (see -names).  If any names are \n\
	     missing they are defaulted to H.\n\
	convasp -msi < POSCAR\n\
	     Outputs to standard out a msi file based on the POSCAR input\n\
	     file.  This can be used as input for cerius.\n\
	     In the functions that output the msi format I do something \n\
	     to put the lattice vectors in a form that cerius2 can read \n\
	     happily.  It involvs making the third lattice vector parallel \n\
	     to Z, the second once in the YZ plane, and the letting the \n\
	     first lattice vector have all 3 components.  It is confusing \n\
	     and probably done in a silly manner, but it seems to work.\n\
	     If you want atom names you must put them after each atom \n\
	     position in the POSCAR file (see -names).  If any names are \n\
	     missing they are defaulted to H.  For an msi file one\n\
	     needs the atomic numbers.  These are coded into the\n\
	     program for most atoms.  If you use atoms with atomic numbers \n\
	     over 86 or f-electron atoms (Lanthanides and Actinides) then \n\
	     you will have to increase the database.  For the list of all \n\
	     coded atomic numbers see the constructor for the structure \n\
	     class in structure.cc.\n\
	convasp -ace < POSCAR           \n\
	     Outputs to standard out a cell standard ASCII (ace) file \n\
	     based on the POSCAR input file.  This can be used as input \n\
	     for CaRIne.\n\
	convasp -cif < POSCAR           \n\
	     Outputs to standard out a Crystallographic Information File \n\
	     (cif) format file based on the POSCAR input file.  \n\
	     This can be used as input for Mercury, and is the preferred  \n\
	     format in Gerd's structure collection.\n\
	convasp -pdb < POSCAR           \n\
	     Outputs to standard out a protein database (PDB) format file \n\
	     based on the POSCAR input file.  This can be used as input\n\
             for many viewing programs.  Note that I don't really know \n\
             anything about PDB but it seems to be an annoyingly column \n\
             formatted.  Therefore, all the widths I use must tbe kept as is.  \n\
             This means that if we have width W for variable X, and prec P, \n\
             then X takes up P+1 for for the decimal part and decimal point, \n\
             and we have only R=W-(P+1)-1=W-P-2 remaining digits (the -1 is \n\
             because if X takes up all of W then you run into the previous \n\
             field).  So we have the constraints \n\
             Cell vectors: W=9,P=3,R=4 => <=10^5\n\
             Cell angles: W=7,P=2,R=3 => <=10^3 (which always works since \n\
                                                 angles are given as >=0 \n\
                                                 and <=360)\n\
             Cartesian positions: W=12,8,P=3,R=7,3 => <=10^8,10^4 (>0) \n\
                                                  and <=10^7,10^3 (<0 since \n\
                                                            you need a space\n\
                                                            for the - sign)\n\
             This is all probably fine unless an atom makes it to more \n\
             negative than -999.999.\n\
	convasp -data < POSCAR\n\
	     Outputs to standard out basic data about the structure in the \n\
	     POSCAR input file.  Output includes volume, \n\
	     a b c alpha beta gamma, reciprocal lattice, reciprocal lattice\n\
	     volume.\n\
	convasp -ndata < POSCAR\n\
	     Outputs to standard output the following normalized data:\n\
	     a1 a2 a3 phi(a2,a3) phi(a3,a1) phi(a1,a2)\n\
	     a1,a2,a3 are NORMALIZED over V^1/3 and phi are in degrees.\n\
	convasp -compare a b c d e f g h k j i l < POSCAR\n\
	     Outputs to standard output the % comparison between\n\
	     a#g b#h c#k d#j e#i f#l in % \n\
	      cout << abs(a1-a7)/((a1+a7)/2.0) << "  ";\n\
	      cout << abs(a2-a8)/((a2+a8)/2.0) << "  ";\n\
	      cout << abs(a3-a9)/((a3+a9)/2.0) << "  ";\n\
	      cout << abs(a4-a10)/((a4+a10)/2.0) << "  ";\n\
	      cout << abs(a5-a11)/((a5+a11)/2.0) << "  ";\n\
	      cout << abs(a6-a12)/((a6+a12)/2.0) << "  ";\n\
	      cout << endl;\n\
	     Useful with -ndata to compare % relaxations:\n\
	     convasp -compare `convasp -ndata < POS1` `convasp -ndata < POS2` \n\
	convasp -ewald [eta] < POSCAR\n\
	     Finds the electrostatic energy of the POSCAR file using the \n\
	     Ewald sum.  Charges must be entered after each atom position.\n\
	     E.g.,   Co 0 0 0 +2.  This is easy to do using the -names \n\
	     option.  Eta is a real space screening parameter.  Setting \n\
	     eta<=0 or leaving it out will cause convasp to choose it \n\
	     automatically (and hopefully optimally).  Eta->0 is no \n\
	     screening and the Ewald recipricol term will be zero.  Eta->inf \n\
	     is total screening and the Ewald real term will be zero.  The \n\
	     Ewald sum itself should always be the same and eta will only \n\
	     affect the efficiency of the calculation. \n\
	convasp -sewald eta < POSCAR\n\
	     Finds the screened electrostatic energy of the POSCAR file using a\n\
	     real space sum.  eta is now the screening length (ie, all \n\
	     coulomb interactions are multiplied by exp(-eta*R).\n\
	     Charges must be entered after each atom position.\n\
	     E.g.,   Co 0 0 0 +2.  \n\
        convasp wignerseitz < POSCAR   [or -ws ]\n\
             Pops out a POSCAR stile file, but moves the images of the atoms in the\n\
             Wigner-Seitz cell. This parameter is usefull to fight against VASP trend\n\
             of moving atoms in different (but traslationally equivalent) positions\n\
             of the unit cell. (SC 10Jan04).\n\
        convasp -xyzwignerseitz < POSCAR   [or -xyzws ]\n\
             Performs \"convasp -xyz 1 1 1\" but moves the images of the atoms in the\n\
             Wigner-Seitz cell. This parameter is usefull to fight against VASP trend\n\
             of moving atoms in different (but traslationally equivalent) positions\n\
             of the unit cell. (SC 10Jan04).\n\
	convasp -clat a b c alpha beta gamma\n\
	     Outputs to standard out the cartesian lattice vectors obtained\n\
	     from the input a b c alpha beta gamma.\n\
	convasp -dist d < POSCAR        \n\
	     Outputs to standard out the distances for each atom to all\n\
	     neighors within a distance d.  Also gives which unit cell the\n\
	     neighbor is in.\n\
	convasp -disp d < POSCAR        \n\
	     Outputs to standard out the displacement from each atom to all\n\
	     neighors within a distance d.  Also gives which unit cell the\n\
	     neighbor is in.\n\
	convasp -angle d < POSCAR   \n\
	     Outputs to standard out the angles for each atom triplet \n\
	     made up of neighors within a distance d of each other.  \n\
	     Actually, since this can be huge, only the first MAX_NUM_ANGLE-1 \n\
	     neighbors within d are used.  MAX_NUM_ANGLE=21 at present and\n\
	     can be set in PrintAngles.cc. \n\
	convasp -xray l < POSCAR\n\
	     Outputs to standard out the powder xray scattering pattern for \n\
	     the structure specified in the POSCAR input file.  The \n\
	     wavelength of the scattering radiation is taken to be l.\n\
	     The xray calculation is almost nothing more that the structure \n\
	     factor, altough it includes approximate treatments of the \n\
	     Debye-Waller terms and the Lorentz-polarization.  In the xray \n\
	     calculations there are a number of terms that require a lot of \n\
	     information to really do right so I approximate them.  First, \n\
	     scattering factors default to the atomic number.  In some cases \n\
	     I coded more accurate values taken from tables for lambda=1.5418.\n\
	     No lambda dependence of the scattering factors is presently \n\
	     included.  For the list of all coded atomic scattering factors\n\
	     see the constructor for the structure class in structure.cc.\n\
	     I include the Lorentz-polarization and an approximate \n\
	     Debye-Waller factor (DWF).  The DWF is found assuming T=300, \n\
	     T_Debye=300, and within the high-temperature Debye aproximation.\n\
	     The atomic mass defaults to twice the atomic number but I \n\
	     have input a few more accurate values in the code.  For the \n\
	     list of all coded atomic masses see the constructor for the \n\
	     structure class in structure.cc.  This simulation should \n\
	     give very accurate peak locations and qualitative relative \n\
	     integrated intensities (peak heights).   I am still not \n\
	     happy with why the peak intensitities do not reproduce other \n\
	     codes better.  The output contains formats with and without\n\
	     peaks at the same 2theta grouped together.  It also contains\n\
	     data ready for plotting.  Each type of data has a keyword on \n\
	     every line so you can grep it out easily.\n\
	convasp -shell ns r1 r2 name dens < POSCAR    \n\
	     Based on the structure in POSCAR, this outputs to standard \n\
	     out all points with ns neighbors in a shell define by inner \n\
	     radius r1 and outer radius r2 (r1<r2).  The shell is \n\
	     restricted to only consider atoms of type name (name=NONE \n\
	     is unrestricted).  The dens is the linear density of \n\
	     candidate points.  So for example, to search for all sites with \n\
	     4 oxygen around them between 1.8 and 2.2 Angstron you could type \n\
		 convasp -shell 4 1.8 2.2 O 20\n\
	     This would construct a 20x20x20 grid of puts in the cell and\n\
	     print out all points that met the shell criteria.  The output\n\
	     gives all the points and their shelss meeting the shell criteria.\n\
	     However, since many points are in the same shell it is \n\
	     convenient to find unique shell environments.  Therefore, I \n\
	     reduce the complete list to keep only one representative point \n\
	     from each unique shell. The shells are distinguished by their\n\
	     centers of mass.  The unique points and their shells are output\n\
	     as all unique points meeting the shell criteria.  The \n\
	     representative point for each shell is the shell center of mass. \n\
	     This routine is great for finding open tetrahedral and octahedral\n\
	     sites where one might put an intercalant.  This routine can\n\
	     take a while to run (usually a 20x20x20 mesh is enough, and might\n\
	     take a couple of minutes for a big cell - start small!).\n\
	convasp -chgint CHGCAR         \n\
	     Outputs to standard out the integrated charge density around \n\
	     every atom.  The total integrated density within the voronoi \n\
	     volume is given for each atom.  The code also calculates \n\
	     the integrated density in a surrounding sphere for each atom.  \n\
	     This is output as a function of radius for radii from 0 to 3 \n\
	     angstrom in steps of 0.1 angstrom.  The total,Up-Dn,Up,Dn \n\
	     charges are all integrated although for non-spin polarized \n\
	     calculations only the first gives new information. The code\n\
	     works on all the recent versions of vasp we have tried but they \n\
	     seem to change the CHGCAR formatting sometimes so it may crash \n\
	     on different versions (at least 4.4.1 and 4.4.5 (including PAW) \n\
	     work).  For a large cell this can take a while (e.g., 80 atoms \n\
	     might take ~20 minutes). \n\
	convasp -intpol file1 file2 nim nearest_image_flag  \n\
	     Creates nim image POSCAR files by interpolating linearly \n\
	     between structures in file1 and file2.  File1 and file2 \n\
	     should both be POSCAR like files with corresponding atoms.  \n\
	     The nearest_image_flag is set to 'e' for exact interpolation \n\
	     and 'n' or 'N' for nearest-image interpolation.  Exact \n\
	     interpolation means that all the positions are taken exactly\n\
	     as they are input.  Nearest-image interpolation means that \n\
	     the interpolation between two corresponding atoms in file1 \n\
	     and file2 is actually done between the atom in file1 and \n\
	     the nearest image of the corresponding atom in file2.  This\n\
	     is useful, e.g., if the positions (in direct coordinates) \n\
	     are 0.01 (file1) and 0.99 (file2).  These are far apart in \n\
	     exact interpolation but very close in nearest-image \n\
	     interpolation.  The POSCAR output files are numbered and \n\
	     output to the present directory.  The code also creates \n\
	     numbered subdirectories and copies the appropriate POSCAR \n\
	     files into those.  This option makes it easy to set up a rubber\n\
	     band calculation in vasp. \n\
	convasp -rbanal nim nearest_image_flag      \n\
	     This is meant to allow you to analyse a rubber band calculation\n\
	     to get energy vs. distance along activation path. The distance\n\
	     is given a cumulative from the 00 to END images (a line integral),\n\
	     and also as the total distance from 00 and END for each image.\n\
	     The first is the activation path, the others are for double checking.\n\
	     The rubber band run must have had nim images.  The distance \n\
	     between two images is defined to be the square root of the \n\
	     sum of all the squared distances between all the atoms in the \n\
	     two images.  The distances depend on the \n\
	     setting of nearest_image_flag (see -intpol).  The code works \n\
	     in two steps.  First it gets energies and distances from the \n\
	     OSZICAR and POSCAR/CONTCAR files, respectively.  This \n\
	     requires that it be run in the directory above all the image \n\
	     directories.  The distances are obtained from POSCAR files \n\
	     for the first and last images, which are not actually calculated\n\
	     so now CONTCAR files exists.  For imtermediate images the\n\
	     POSCAR files are used.  The energies are pulled from the\n\
	     OSZICAR files.  First, these do not exist in the first and \n\
	     last image directories, so you must put them their yourself\n\
	     based on other runs (if you leave them out then no energies\n\
	     will be obtained but you can add the energies later, as described\n\
	     below).  Second, the output format for OSZICAR in a parallel\n\
	     vasp rubber band run is all messed up.  Therefore, the energies\n\
	     are not E0 values (which are not the energy values of the \n\
	     images) but the free energy from the line just before the \n\
	     last E0.  This numbers differ somewhat from E0 values.  This\n\
	     whole output formatting problem changes with version of vasp - \n\
	     I set  it up for version 4.4.1 beta.  It should also work with\n\
	     v 454.   We should change it to use E0 as soon as possible.  \n\
	     Once the distances and energies have been obtained then they \n\
	     are interpolated with a cubic spline curve.  The spline \n\
	     interpolation assumes that the derivatives at the end points \n\
	     are 0.  The E.vs.dist data and the spline interpolation are \n\
	     both output to standard out.  If there is any problem with \n\
	     the energy data (e.g., you need to add end members) then \n\
	     you can fix it and run the spline interpolation again \n\
	     independently (see -spline).  The spline interpolation is \n\
	     done on 50 points by default.  This can be \n\
	     changed by setting DEFAULT_NPT_FOR_SPLINE in RBFuncs.cc.\n\
	convasp -spline npt < file\n\
	     Outputs to standard out a cubic spline interpolation of npt \n\
	     evenly spaced points.  The infput file must be two colums \n\
	     giving X and Y(X).  The derivatives at the endpoints are \n\
	     assumed to be zero.  To change this, change yp1 and ypn \n\
	     in the SetSpline function in the FittingFuncs.cc file.\n\
	convasp -raytrace rtfile\n\
	     Outputs jpeg or mpeg pictures created by ray tracing.  This \n\
	     is a rather elaborate routine.  The basic idea is that it\n\
	     takes as input a list of structures (in POSCAR like format) \n\
	     and converts them into a dat file, inputs that into the ray \n\
	     tracing program tachyon, then takes the tga file output by \n\
	     tachyon and uses convert to make it a jpeg, then if needed \n\
	     takes all the jpegs and puts them together into and mpg using \n\
	     mpeg_encoder.  Therefore the following programs must be \n\
	     installed and in your path: tachyon, convert, mpeg_encoder \n\
	     (only if making a movie).  There is an input file rtfile \n\
	     which contains tokens setting characteristics of the calculation\n\
	     and the ray traced picture.  This input file looks like the \n\
	     following:\n\
\n\
		# These set the input for the ray tracing program tachyon\n\
		CALCTYPE = 0 #0=strlist\n\
		INFILE = strlist # For CALCTYPE=0\n\
		OUTFILE = XXX # All ouput will have this prefix.\n\
		RESX = 600 # X pixels in images.\n\
		RESY = 500 # Y pixels in images.\n\
		ZOOM = 1.5 # Like a zoom lens.\n\
		ASPECTRATIO = 1 # Y height / X height.\n\
		ANTIALIASING = 0 # Something to do with extra accuracy.\n\
		RAYDEPTH = 12 # Number of reflections to keep in ray trace.\n\
		# x,y,z of camera location.  Changes incrementally each frame \n\
		# from initial to final value.  format is ciX cfX ciY cfY ciZ cfZ, \n\
		# where i,f denote initial and final, respectively.\n\
		# Default is displaced along -Y from structure center.\n\
		#CENTER = 1 1 -2 -2 2 4\n\
		#VIEWDIR = 1 0 0 # Direction of camera viewing.\n\
		#UPDIR = 0 0 1 # Up direction for picture.\n\
		BACKGROUND = 0.3 0.1 0.1 # Background lighting (all lights are R G B).\n\
		LIGHT = -20 -20 -20  0.01 1.0 1.0 1.0 # Spherical light source: center(3) radius(1) color(3)\n\
		LIGHT = 20 20 20  0.01 1.0 1.0 1.0 # Spherical light source: center(3) radius(1) color(3)\n\
		ATOMTEXTURE = 1 0.1 0.9 0.0  1.0 # atomtype ambient diffuse specular opacity\n\
		ATOMTEXTURE = 2 0.1 0.9 0.0  1.0 # atomtype ambient diffuse specular opacity\n\
		ATOMTEXTURE = 3 0.1 0.9 0.0  1.0 # atomtype ambient diffuse specular opacity\n\
		ATOMTEXTURE = 4 0.2 0.9 0.3  1.0 # atomtype ambient diffuse specular opacity\n\
		ATOMCOLOR = 1 1.0 0.75 0.3 # atomtype(1) color(3)\n\
		ATOMCOLOR = 2 0.2 0.4 1.0 # atomtype(1) color(3)\n\
		ATOMCOLOR = 3 1.0 1.0 1.0 # atomtype(1) color(3)\n\
		ATOMCOLOR = 4 1.0 0.0 0.0 # atomtype(1) color(3)\n\
		ATOMRAD = 1 0.6 # atomtype rad\n\
		ATOMRAD = 2 0.3 # atomtype rad\n\
		ATOMRAD = 3 1.0 # atomtype rad\n\
		ATOMRAD = 4 0.8 # atomtype rad\n\
		SHADING = mediumshade # fullshade,mediumshade,lowshade,lowestshade  \n\
		# A supercell matrix given as 9 reals:  a11 a12 a13 a21 a22 a23 \n\
		# a31 a32 a33.  Note that using this moves all atoms into images\n\
		# the unit cell.\n\
		# Default = 1 0 0 0 1 0 0 0 1\n\
		SUPERCELL = 1 0 0   0 1 0  0 0 1 \n\
		# Rotation around x,y,z axis through structure_origin \n\
		# (counterclockwise).  Rotation take place incrementally each\n\
		# frame.  Rotation goes from initial to final value.  \n\
		# format is riX ifX riY rfY riZ rfZ, where i,f denote initial \n\
		# and final, respectively.\n\
		# Default = 0 0 0 0 0 0\n\
		ROTATION = 0 0 0 0 -180 180 \n\
		# The zero for the structure coordinates and lattice parameters. \n\
		# Rotations occur around this point.\n\
		# Default = First moment of atom positions.\n\
		STRUCTURE_ORIGIN = 0 0 0 \n\
		PLANE = 1 # 0 for no plane, 1 for plane.\n\
		PLANECENTER = 0 0 -9.21016 # X Y Z for center of plane. \n\
		PLANENORMAL = 0 0 1 # Direction of plane normal.\n\
		PLANECOLOR = 1 1 1 # R G B for plane color.\n\
		PLANETEXTURE = 0.3 0.6 0.8 1.0 # ambient diffuse specular opacity for plane.\n\
\n\
	     Tokens can be in any order, whitespace is ignored, and anything \n\
	     following a # on a line is ignored.  The CENTER, VIEDDIR, and \n\
	     UPDIR depend on the structure and can be a pain to set by hand \n\
	     so they have fairly elaborate defaults that seem to usually work \n\
	     fine.  The other values are not bad places to start.  For \n\
	     more info about these check out the tachyon documentation.\n\
\n\
	     For a single structure the output consists of XXX.dat, XXX.tga,\n\
	     XXX.jpg, XXX.enc, XXX.mpg.  Having all these files is useful \n\
	     if some part of the program did not exectute properly and you \n\
	     need to do steps by hand (tachyon XXX.dat produces XXX.tga, \n\
	     convert XXX.tga XXX.jpg produces XXX.jpg, and mpeg_encode XXX.enc\n\
	     produces XXX.mpg).  For more than one structure the XXX.dat \n\
	     and XXX.tga files are erased to avoid clutter.  The XXX.enc file \n\
	     is used for the mpeg_encode programs and is totally set in \n\
	     convasp and the user has not control over it from the rtfile.  \n\
	     If you want to alter it after it is output go ahead but I know \n\
	     almost nothing about how it works.  To make your own mpg from \n\
	     the jpg type: \n\
		mpeg_encode XXX.enc \n\
	     \n\
	     Right now I only know this works on africa.  The tga, jpg files \n\
	     can be viewed with xv and the mpg with mpeg_play. 	\n\
	convasp -make_strlist OUTCAR XDATCAR\n\
	     Outputs to standard out a sequence of structures in POSCAR format.\n\
	     The strlist file created contains a list of structures, each \n\
	     formatted exactly like a POSCAR file, with an empty line after \n\
	     all but the last one.  The file must have no extra lines at the end\n\
	     or all routines that read it will crash.  The lattice parameters \n\
	     are pulled out from the OUTCAR and the positions from XDATCAR.  \n\
	     If OUTCAR has too few lattice parameters then copies of the last \n\
	     set are used for the remaining XDATCAR.\n\
	convasp -supercell_strlist sc.txt strlist\n\
	     Outputs to standard out a sequence of structures in POSCAR format.\n\
	     The structures are supercells formed using the supercell matrix\n\
	     in sc.txt based on the structures in strlist.  For more information\n\
	     on sc.txt and supercells see -supercell.  For more information\n\
	     on a strlist see -make_strlist.\n\
	convasp -join_strlist strlist1 strlist2\n\
	     Outputs to standard out a sequence of structures in POSCAR format.\n\
	     The structures consist of those in strlist1 with the atoms from \n\
	     strlist2 added in.  The positions are taken from strlist2, \n\
	     transformed into Cartesian coordinates, and then inserted into\n\
	     strlist1.  All inserted atoms are added as new types.  If one\n\
	     file has more structures than the other then the shorter file\n\
	     is padded with copies of the last structure.  For more information\n\
	     on a strlist see -make_strlist.\n\
	convasp -pocc PROOUT\n\
	     Outputs occupations calculated from projections onto spherical \n\
	     harmonics for each ion for many combinations of L, M, bands, \n\
	     and kpoints.  This works with version 445 and has not been tested \n\
	     on any other version (and probably won't work).  To make this work\n\
	     you need to do the following\n\
		- Use slightly altered version of vasp (see below)\n\
		- set LORBIT=2 in INCAR (see below)\n\
		- Set ISYM=0 in INCAR (see below)\n\
		- Set RWIGS in INCAR (see below)\n\
	     Here is why you need to do these things (feel free to skip this).  \n\
	     The -pocc option reads the PROOUT file, which is produced by \n\
	     running vasp with LORBIT=2 in the INCAR file.  There is an \n\
	     annoying subtle point here. I am not sure I totally get this \n\
	     but here is my best understanding.  The projection onto the \n\
	     spherical harmonics actually uses some atomic like radial \n\
	     functions (bessel functions).  This means that for each \n\
	     spherical harmonic there are multiple states, corresponding \n\
	     to different atomic energy levels and radial functions.  \n\
	     Write the projection of band n, at kpt k, onto spherical harmonic \n\
	     with angular quantum numbers l,m and energy level e as Pnklme.  \n\
	     As compiled, the vasp code outputs projections Pnklm, summing \n\
	     over the e parameter.  This makes each projection a sum of \n\
	     complex numbers, allowing some cancellations.  However, for \n\
	     occupations, like those output in PROCAR (LORBIT=1) and OUTCAR, \n\
	     the summations over e are done with the squares of the Pnklme.  \n\
	     This makes sense, since you want to add up probabilities, \n\
	     not amplitudes, to get an occupation.  Unfortunately, from \n\
	     the output Pnklm one cannot reconstruct the Pnklme, so the \n\
	     ouptut in PROOUT is not enough to reproduce the occupations.\n\
	     Therefore, I suggest the following.  Recompile vasp with following\n\
	     modifications to sphpro.F\n\
	     change line 252\n\
			WRITE(IUP,'(9F12.6)') CSUM_PHASE\n\
	     to\n\
			write(IUP,'(9F12.6)') CSUM_ABS\n\
	     CSUM_ABS is a complex variable but the imaginary part is zero.  It\n\
	     is the squared amplitude for each projection.  Note that this is \n\
	     a probability.  You use it directly (do not square it) to get \n\
	     occupations.  Note that the augmentations are added to this real \n\
	     number, and are therefore probablities, not amplitudes.  \n\
	     This makes sense when you look at the code in sphpro.F that \n\
	     calculates the augmentation portion.  The augmentations can be \n\
	     <0, which I assume corresponds to reducing the probability of \n\
	     finding electrons.  This make me a little uncomfortable but \n\
	     I guess it is OK.  At this point the convasp code assumes \n\
	     the above modification and uses the magnitude of the projections \n\
	     to calculate all occupations (not the magnitude squared).  \n\
	       Also, there is another subtle point with the kpoints.  If \n\
	     you use symmetry certain sets of symmetry equivalent kpoints \n\
	     (a star) are represented by a single irreducible kpoint.  We \n\
	     are used to ignoring this and simply weighting things associated \n\
	     with the irreducible kpoint appropriately to account for the \n\
	     whole star.  However, the projections onto different orbitals \n\
	     are not the same for all the points in a star.  For example, \n\
	     the kpoints (0.1,0,0),(0,0.1,0),(0,0,0.1) may all be in the \n\
	     same star in fcc, but states associated with them will project \n\
	     differently onto Px orbitals.  Therefore, if you use irreducible\n\
	     kpoints you will get the wrong projections (it seems like you\n\
	     do get the right totals for S,P, and D, bit I am not sure that \n\
	     is always the case).  To be safe, don't use any symmetry - i.e.,\n\
	     set ISYM=0 in the INCAR file.\n\
	       You must set RWIGS so that the code knows the radius of\n\
	     the spheres onto which it projects. \n\
	       To make sure all is in order you can look at the total \n\
	     occupations for each ion in OUTCAR.  These should match \n\
	     the Occupations vs. ION:LM values in the output of convasp \n\
	     (the last lines in the output).  A more detailed check is \n\
	     to run vasp with LORBIT=1 and compare the PROCAR file with \n\
	     the output of convasp.  I am not sure that this will all \n\
	     work the same way with PAW potentials.  \n\
	convasp -pdos pdos.in PROOUT\n\
	     Writes the projected DOS for any desired combination of atoms,\n\
	     kpoints, bands, and lm values.  The output consists of 6 \n\
	     columns, spin up, down, up-down, and the cumulative DOS for \n\
	     each of those.  Only up spin data is given for non \n\
	     spin-polarized calculations.  The PDOS should look like\n\
	     an equivalent vasp output but will not be identical since\n\
	     vasp uses a different smearing method to get the PDOS. \n\
	     However, I think these PDOS are basically correct, at least\n\
	     qualitatively.  To make this work you must\n\
		- Use slightly altered version of vasp\n\
		- set LORBIT=2 in INCAR\n\
		- Set ISYM=0 in INCAR \n\
		- Set RWIGS in INCAR (see below)\n\
	     The details explaining all this are given above in the -pocc \n\
	     section.  The input file has the following format\n\
\n\
		# Input for convasp -pdos.\n\
\n\
		# These values you supply once.\n\
		EMIN = -20.01477264 # default: 0.5eV below lowest energy.\n\
		EMAX = 6.90250559 # default: 0.5ev above highest energy.\n\
		NBINS = 300 # default: 300 \n\
		SMOOTH_SIGMA = 0.1 # Gaussian smoothing of the DOS. \n\
				   # default: 1 bin width.\n\
		PRINT_PARAMS = 1 # 0=prints only data (easy to plot).\n\
				 # 1=prints all the input parameters.\n\
				 # default: 0\n\
\n\
		# You can have as many cases as you want.\n\
		# They are all added together.\n\
\n\
		# case 1\n\
		ATOMS = 1 # default: all atoms.\n\
		KPOINTS = # default: all kpoints.  \n\
		BANDS = # default: all bands.\n\
		LMVALUES = # default: all s,p,d,f.\n\
\n\
		# case 1\n\
		ATOMS = 2 # default: all atoms.\n\
		KPOINTS = 1 2 3 # default: all kpoints.  \n\
		BANDS = 1 # default: all bands.\n\
		LMVALUES = 3 # default: all s,p,d,f.\n\
\n\
	     All # denote comment lines.  Each case is started when the token\n\
	     ATOMS is used.  Following an ATOMS token, all KPOINTS, BANDS, \n\
	     LMVALUES tokens will apply to the atoms denoted in the preceeding \n\
	     ATOMS token until the next ATOM token.  The  KPOINTS, BANDS, \n\
	     LMVALUES tokens can be left out in which case their default values \n\
	     will be used.  You can have any number of cases.  The above \n\
	     example will calculate a PDOS with projections onto atom 1\n\
	     for all kpoints, bands, and s,p,d, and f (case 1) added to the\n\
	     projection onto atom 2 for kpoints 1-3, band 1, and the Pz orbital.\n\
	     I believe you can create any desired projections with this input\n\
	     file.  The LMVALUES use the following correspondence between \n\
	     numerical input and orbitals projected.\n\
\n\
	     Input number: 1 2  3  4  5    6   7   8   9   10     11   \n\
	     Orbitals:     S Py Pz Px Ptot Dxy Dyz Dz2 Dxz Dx2-y2 Dtot\n\
	     Input number: 12 13 14 15 16 17 18 19   20\n\
	     Orbitals:     F1 F2 F3 F4 F5 F6 F7 Ftot Tot\n\
	 \n\
	     I have not done the work to figure out which of the standard\n\
	     f-orbital functions correspond to F1-F7.  \n\
	     \n\
	     This is only tested for version 4.4.5.\n\
	convasp -sumpdos pdos.in PROOUT\n\
             Works for vasp.46x\n\
             This allows you to sum up projected DOS together for convenient\n\
             plotting.  It only works when you have run with the following \n\
             INCAR file settings.  LORBIT=1 or 2 and set RWIGS, or \n\
             LORBIT = 11 or 12 and no RWIGS (only works when using PAW PP).\n\
             If you are running parallel you must set NPAR=1.  The\n\
             input file is similar to -pdos above but somewhat simpler\n\
             so I give a full example here.\n\
\n\
		# Input for convasp -sumpdos.\n\
\n\
		# These values you supply once.\n\
                SPIN = 1 # 1 = non-spin polarized, 2 = spin polarized\n\
                         # Default 1\n\
                EFERMI = -999 # This will be subracted from the energies.\n\
                              # Set to 0 to subtract nothing, -999 to use\n\
                              # the E_Fermi in the DOSCAR.  Default -999.\n\
                NLM = 9 # number of orbitals, 9 (spd:1+3+5) or 16 (spdf:1+3+5+7)\n\
                        # Default 9\n\
		PRINT_PARAMS = 0 # 0=prints only data (easy to plot).\n\
				 # 1=prints all the input parameters.\n\
				 # default: 0\n\
\n\
		# You can have as many cases as you want.\n\
		# They are all added together.\n\
\n\
		# case 1: t2g on atom 1\n\
		ATOMS = 1 # default: no atoms\n\
		LMVALUES = 5 6 8 # default: no lm\n\
		# case 2: eg on atoms 2 and 3\n\
		ATOMS = 2 3 # default: no atoms\n\
		LMVALUES = 7 9 # default: no lm\n\
\n\
	     All # denote comment lines and can be put anywhere.  \n\
             Each case is started when the token ATOMS is used.  \n\
             Following an ATOMS token, all LMVALUES tokens \n\
             will apply to the atoms denoted in the preceeding \n\
	     ATOMS token until the next ATOM token. You can have any number \n\
             of cases, and the results for all cases are added together.  \n\
             WARNING: make sure to use SPIN to set if it is a spin poloarized\n\
             calculation, make sure to use EFERMI to your desired reference,\n\
             and make sure to set NLM to 9 (spd) or 16 (spdf).\n\
             The lm values correspond to orbitals by the following scheme.\n\
	     Input number: 1 2  3  4  5   6   7   8   9     \n\
	     Orbitals:     S Py Pz Px Dxy Dyz Dz2 Dxz Dx2-y2\n\
	     Input number: 10 11 12 13 14 15 16 \n\
	     Orbitals:     F1 F2 F3 F4 F5 F6 F7 \n\
  	convasp -bands PROOUT < POSCAR\n\
	     Outputs the up and down bands to band.up.out and band.dn.out.\n\
	     Format is\n\
	      k-space path length (true, fractional), bands (up or down), nkpt, kx, ky, kz\n\
	     Uses data in PROOUT file.  Uses POSCAR to get lattice to calculate\n\
	     true recipricol distances.  Must run vasp with LORBIT=2 to get\n\
	     the PROOUT file.\n\
	convasp -planedens dens2d.in CHGCAR\n\
	     This finds the charge density in a plane.  The input file dens2d.in \n\
	     has the form\n\
		D # Coordinates for following points (Direct/Cartesian)\n\
		scale # Scale factor - edges of plane get mult. by this (but not origin).\n\
		x y z # origin point\n\
		x y z # X axis\n\
		x y z # Y axis\n\
		Nx Ny # Number of X and Y grid points\n\
		Middle # Location for origin (Middle/Corner).\n\
		Ortho # Whether to use Y orthogonal to X (Ortho/Strict).\n\
	     The output consists of 4 files, dens.[tot/diff/up/dn].out.  Each \n\
	     has the same format, consisting of rows of density values, each \n\
	     row corresponding to a value along the X axis and each column\n\
	     to a value along the Y axis.  The format can be read directly \n\
	     into Excel and easily into MatLab.  This routine uses the same \n\
	     routine as -chgint to read in the CHGCAR and should work\n\
	     for the same versions of vasp (see -chgint for more information).\n\
	convasp -chgdiff CHGCAR1 CHGCAR2\n\
	     This takes the difference between two CHGCAR files, outputting\n\
	     CHGCAR1-CHGCAR2.  The output is in the form of a vasp445 \n\
	     CHGCAR.  The output file is CHGCAR.diff.out.  \n\
	     This routine uses the same routine as -chgint to read in the \n\
	     CHGCAR and should work for the same versions of vasp (see \n\
	     -chgint for more information).\n\
	convasp -rdf rmax nbins sigma < POSCAR\n\
	     Get the radial distribution function (rdf) for a POSCAR file\n\
	     rmax: The radius out to which the rdf is calculated.\n\
	     nbins:  The number of bins for the rdf.\n\
	     sigma:  The sigma of the gaussian used to smear the rdf.\n\
	     This also finds the nearest-nieghbor shells by looking at \n\
	     where the rdf has minima.  The output gives the rdf and nn shells\n\
	     for each atom and for each type of neighbor, including a sum\n\
	     of all neighbor types.  If you want to treat all atoms as the \n\
	     same (e.g., to look at the parent lattice) just set one\n\
	     type in the POSCAR file.  You may need to play with sigma\n\
	     and the nbins to get atoms grouped together that you want in \n\
	     one shell but not to group atoms you want in different shells.\n\
	     One apporach is to choose enough bins to make sure to distinguish\n\
	     every shell of interest and the increase sigma until you group\n\
	     desired atoms together.  sigma=0 does no smearing.\n\
	convasp -rdfcmp rmax nbins sigma nshmax POSCAR1 POSCAR2\n\
	     Uses the radial distribution functions (rdf) for POSCAR 1 and 2 \n\
	     to assess how close the structures are.  For each atom the\n\
	     rdf and radial shell function (rsf - this gives the coordination\n\
	     numbers for each shell) are found for each type.  \n\
	     The shells are found by looking at the derivatives of the rdf.\n\
	     Then the atoms of the same types are compared and the rms \n\
	     errors in the rsf for POSCAR1 and POSCAR2 are computed.  Then \n\
	     we step through the atoms of POSCAR1, matching them up \n\
	     with the atoms of POSCAR2, based on the minimum rms.  When a \n\
	     POSCAR2 atom is matched it not considered for later matches.\n\
	     The total rms based on these best matches is found by averaging \n\
	     all the individual atom RMS's.  The total RMS and the \n\
	     rsf for the best matched atoms are output.\n\
	     rmax: The radius out to which the rdf is calculated.\n\
	     nbins:  The number of bins for the rdf.\n\
	     sigma:  The sigma of the gaussian used to smear the density.\n\
	     nshmax:  The farthest possible shell used for computing RMS \n\
		      with the rsf (if <nshmax shells are found then fewer\n\
		      will be used).\n\
	     I think this works but large changes in shape give rsf that \n\
	     are too different to compare exactly and the RMS!=0.  However,\n\
	     looking at the output can make it clear how the shells are\n\
	     related.  At this point the 2 structure must have the same\n\
	     number of atoms of each type.  If you want to compare 2 POSCARs\n\
	     where one is a multiple of the other in terms of atoms of \n\
	     each type you can create supercell (see -supercell) to make this\n\
	     function usable.\n\
	convasp -platon [EQUAL] [EXACT] [ang d1 d2 d3] < POSCAR | platonSG\n\
	     This finds the space group.  convasp is creating an output\n\
	     file, which is piped into a script platonSG, which uses the \n\
	     program platon.\n\
	     Wraps input file for Platon ADDSYM package:\n\
	     CALC ADDSYM (EQUAL) (EXACT) (ang d1 d2 d3) \n\
	     where: \n\
	     EQUAL - Search with all atom type treated as equivalent. \n\
	     EXACT - All atoms should fit for given criteria. \n\
	     ang - Angle criterium in search for metrical symmetry of the\n\
		   lattice (default 1.0 degree). \n\
	     d1 - Distance criterium for coinciding atoms for non-inversion \n\
		  (pseudo)symmetry elements (default 0.25 Angstrom). \n\
	     d2 - Distance criterium for coinciding atoms for (pseudo)\n\
		  inversion symmetry (default 0.45, 0.25 Angstrom). \n\
	     d3 - Distance criterium for coinciding atoms for (pseudo)\n\
		  translation symmetry (default 0.45, 0.25 Angstrom). \n\
	     To get the space group, type\n\
	         convasp -platon < POSCAR | platonSG\n\
	     To just get see the output file from convasp, type\n\
		convasp -platon < POSCAR\n\
	     To check for errors and see output from platon, type\n\
		convasp -platon < POSCAR | platon -o\n\
  	     Note that the added flags above do not seem to work.  To change tolerance\n\
             create output file from convasp, and then add the four tolerances\n\
             after CALC ADDSYM (on the same line).  Then pipe this file to platonSG. \n\
	     Convasp will use your atom labels if they are there.  If you \n\
	     give no atom labels it will use defaults for each atom type,\n\
	     (these are He,Li,Be,B,C, and then W for all remaining atom types).\n\
	     WARNING: If you have more than 6 atom types the W default will give the wrong space group.\n\
	     WARNING: The equal flag may not work.\n\
	     WARNING: Atoms labeled with H do not get read by default in platon.  Do not use H labels.\n\
	     SEE: http://www.cryst.chem.uu.nl/platon/pl000401.html\n\
	     ASK: Stefano or Eric or Dane\n\
	convasp -rbdist POSCAR1 POSCAR2 n|N|e|E\n\
	     Gets the distance between two POSCAR files.  Uses lattice params\n\
	     of POSCAR1.  For use of n|N|e|E see -intpol.\n\
	convasp -mom POSCAR \n\
	     Gets the mass moments of the POSCAR file.  Gets only the first\n\
	     (center of mass) at this point.\n\
	convasp -setcm cm1 cm2 cm3 POSCAR\n\
	     Sets to center of mass to (cm1,cm2,cm3) by shifting all the \n\
	     atom positions.  Use -mom to check the shift was correct.\n\
	convasp -niggli POSCAR\n\
	     Converts the unit cell to the standardized Niggli form.  The\n\
	     form is unique (up to some signs, I think).  The transformation\n\
	     makes use of only the lattice vectors and does not depend on the\n\
	     basis atoms.  This will work on any cell, but it treats the given\n\
	     cell as primitive, and it will not reduce the cell to primitive \n\
	     if it is not primitive already.  At present the algorithm seems to \n\
	     hang if I force more than about 6 digits of accuracy so be aware that\n\
	     small errors might be introduced (these can break symmetry!).\n\
	convasp -cmp_str POSCAR1 POSCAR2 rcut\n\
	     This compares the characteristics of two POSCAR files and is\n\
	     useful for determining if the files are the same.  Characteristics\n\
	     compared include number of atoms, number of types, total volume, \n\
	     volume per atom, lattice parameters, number of neighbors of each\n\
	     pair type out to rcut, differences in bond lengths for neighbors\n\
	     of each pair type out to cutoff, and space groups.  For rcut<0 the\n\
	     cutoff is set to 4*(Wigner-Seitz radius of each atom) (this is the \n\
	     radius such that num atom spheres occupies the whole volume).\n\
	convasp -data1 POSCAR1 rcut\n\
	     This is basically like cmp_str except that it works on 1 str.  Slightly\n\
	     different data is given: No space groups, All bond lengths.\n\
	convasp -kmesh kptdens POSCAR\n\
             This funtion prints a set of Monkhorst-Pack kpoint mesh values \n\
             (divisions along each reciprical lattice parameter)\n\
             based on the desired kpt density (kptdens) and the \n\
             lattice parameters.  The values assure the most even \n\
             distribution of kpts along the lattice params consistent \n\
             with the kpt density.  Returns 4 kmesh vectors of length 3\n\
               1. Rounded to Nint\n\
               2. Exact\n\
               3. Rounded down\n\
               4. Rounded up\n\
\n\
	     ******************** End of convasp help ******************** \n\
	" << endl;
	} // end routine





