// **************************************************
// PrintKmesh
// **************************************************
//  This funtion prints a set of Monkhurst-Pack kpoint mesh values 
// (divisions along each reciprical lattice parameter)
// based on the desired kpt density and the lattice parameters.  The
// values assure the most even distribution of kpts along the lattice 
// params consistent with the kpt density.  Returns 4 kmesh vectors of length
// 3
//    1. Rounded to Nint
//    2. Exact
//    3. Rounded down
//    4. Rounded up

using namespace std;
#include <iostream>
#include <vector>
#include "matrix.h"
#include "UsefulFuncs.h"
#include <string>

void PrintKmesh(const matrix<double> & kmesh, ostream & sout)
{
  cout.setf(ios::fixed,ios::floatfield);
  cout.precision(3);
  vector<string> s(4);
  s[0]="RoundNint:";
  s[1]="Exact:    ";
  s[2]="RoundDown:";
  s[3]="RoundUp:  ";
  for (int i=0;i<4;i++){
    sout << s[i];
    for (int j=0;j<3;j++){
      sout << "  " << kmesh[i][j];
    }
    sout << endl;
  }
}





