import os
import sys
import numpy as np
"""Hocking, Methods and Applications of Linear Models: Regression and the Analysis of Variance, 2nd Edition (2003), Ch. 2
"""
forbesdata=dict() #key = case number, value = [pressure, boiling pt]
forbesdata["legend"] = ["pressure","boiling point"]
forbesdata[1]=   [20.79,   194.5]
forbesdata[2]=   [20.79,   194.3]
forbesdata[3]=   [22.40,   197.9]
forbesdata[4]=   [22.67,   198.4]
forbesdata[5]=   [23.15,   199.4]
forbesdata[6]=   [23.35,   199.9]
forbesdata[7]=   [23.89,   200.9]
forbesdata[8]=   [23.99,   201.1]
forbesdata[9]=   [24.02,   201.4]
forbesdata[10]=  [24.01,   201.3]
forbesdata[11]=  [25.14,   203.6]
forbesdata[12]=  [26.57,   204.6]
forbesdata[13]=  [28.49,   209.5]
forbesdata[14]=  [27.76,   208.6]
forbesdata[15]=  [29.04,   210.7]
forbesdata[16]=  [29.88,   211.9]
forbesdata[17]=  [30.66,   212.2]
forbeskeys = forbesdata.keys()
forbeskeys.sort()
forbeskeys.remove("legend")

pressure_full=np.zeros(len(forbeskeys), float)
pressure_reduced=np.zeros(len(forbeskeys)-1, float)
bp_full=np.zeros(len(forbeskeys), float)
bp_reduced=np.zeros(len(forbeskeys)-1, float)
for kct in range(0, len(forbeskeys)):
    mykey = forbeskeys[kct]
    pressure_full[kct] = forbesdata[mykey][0]
    bp_full[kct] = forbesdata[mykey][1]
    if mykey < 12:
        pressure_reduced[kct]=forbesdata[mykey][0]
        bp_reduced[kct]=forbesdata[mykey][1]
    elif mykey > 12:
        pressure_reduced[kct-1]=forbesdata[mykey][0]
        bp_reduced[kct-1]=forbesdata[mykey][1]
print pressure_full
print bp_full
print pressure_reduced
print bp_reduced
pb_strength = np.array([66.30,
64.84,
64.36,
69.70,
66.26,
72.06,
73.23,
71.40,
68.85,
75.78,
72.57,
76.64,
78.87,
77.37,
75.94,
78.82,
77.13,
77.09],float)
pb_temp = np.array([40,
40,40,45,45,45,50,50,50,
55,55,55,60,60,60,65,65,65],float)
def least_squares_regression(xvec, yvec, order=1, zerointercept=False, verbose=1):
    """Least-squares regression from Hocking, 
        Methods and Applications of Linear Models: 
        Regression and the Analysis of Variance, 2nd Edition,
        John Wiley & Sons, Inc., 2003, Ch. 2
        Args:
            xvec <numpy array of float>: no-error inputs
            yvec <numpy array of float>: observed outputs
            order <int>: order of fit 
                            1 - linear (default)
                            2 - quadratic
                            3 - cubic
                            (no other values supported)
            zerointercept <bool>: zero the intercept
            verbose <int>: verbosity of output 
                            1 - verbose (default)
                            0 - concise
    """
    print "---------------------"
    jvec = np.ones(len(xvec),float)
    if order == 1:
        if zerointercept:
            xmat = xvec.transpose()
        else:
            xmat = np.vstack([jvec,xvec]).transpose()
    elif order == 2:
        if zerointercept:
            xmat = np.vstack([xvec, xvec*xvec]).transpose()
        else:
            xmat = np.vstack([jvec,xvec,xvec*xvec]).transpose()
    elif order == 3:
        if zerointercept:
            xmat = np.vstack([xvec, xvec*xvec, np.power(xvec, 3)]).transpose()
        else:
            xmat = np.vstack([jvec,xvec,xvec*xvec, np.power(xvec, 3)]).transpose()
    xtrans = xmat.transpose()
    invxtx = np.linalg.inv(np.dot(xtrans, xmat))
    betamat = np.dot(np.dot(invxtx,xmat.transpose()),yvec.transpose())
    print "Beta matrix: ", betamat
    hatmat = np.dot(np.dot(xmat, invxtx),xtrans)
    yhat = np.dot(hatmat, yvec)
    rvec = yvec - yhat
    rss = np.dot(rvec.transpose(),rvec)
    iminush = np.eye(hatmat.shape[0]) - hatmat
    rank = np.rank(iminush)
    print "Rank: ", rank
    s_squared = rss/(len(yhat)-float(rank))
    print "S-squared: ", s_squared
    print "Sqrt s-squared: ", np.sqrt(s_squared)
    stderrsq = np.diag(invxtx)*s_squared
    stderr = np.sqrt(stderrsq)
    print "Std errors in coefficients: ", stderr
    print "------------------"
    
print "Begin least-squares regressions"
print "Forbes Full set, linear:"
least_squares_regression(bp_full, pressure_full, 1)
print "Forbes Reduced set, linear:"
least_squares_regression(bp_reduced, pressure_reduced, 1)
print "Forbes Reduced set, quadratic:"
least_squares_regression(bp_reduced, pressure_reduced, 2)
print "Particle board, linear:"
least_squares_regression(pb_temp, pb_strength,1)
print "Particle board, quadratic:"
least_squares_regression(pb_temp, pb_strength, 2)
print "Particle board, quadratic, zero intercept:"
least_squares_regression(pb_temp, pb_strength, 2, True)
