#!/usr/bin/env python
##
#@package   hvfcalc_tom
#@ingroup   vasp
#@ingroup   python
#@brief     Calculates the vacancy formation energy from the bulk and endpoint 
#           directories
#@author    Tom Angsten
#@date      7/9/11 created
###

import os
import sys
import util_tom

def main(bulkdir="",vacdir="",numatoms=108):
    """
    Purpose: Calculates Enthalpy of Vacancy Formation
    inputs:  
        [$1] path for bulk
        [$2] path for vac
        [$3] number of atoms in cell (default = 108)
    """
    
    bulkdir = os.path.abspath(bulkdir)
    vacdir = os.path.abspath(vacdir)

    if (os.path.exists(bulkdir) == False):
        print 'Bulk directory does not exist'
        sys.exit(1)
    elif (os.path.exists(vacdir) == False):
        print 'Vacancy directory does not exist'
        sys.exit(1)

    bulk_oszicar = bulkdir + '/OSZICAR'
    vac_oszicar = vacdir + '/OSZICAR'

    bulk_energy = util_tom.getEnergy(bulk_oszicar)
    vac_energy = util_tom.getEnergy(vac_oszicar)
    normalized_energy = ( bulk_energy/numatoms )
    return ( (vac_energy+normalized_energy)-bulk_energy )
