#!/usr/bin/env python
##
#@package   Prunindir 
#@ingroup   vasp
#@ingroup   python
#@brief     loops over a directory and runs submission scripts
#@author    Tam Mayeshiba
#@date      11/22/10 created
#@date      3/22/11 make validation optional 
#@date      8/19/11 find submission script with Psubfile
#@date      8/19/11 find directory list with Xgetdirs
#@date      8/19/11 validate top directory with Xvaliddirectory
#@date      8/19/11 validate askfirst response with Xvalidletter
###
def main(topdir, askfirst=1):
    """
    Purpose: loops over a directory and runs submission scripts
        inputs:  
            [$1] directory in which to loop
                 example: /home/tam/starts/
            [$2] 1 - ask before running (default)
                 0 - run without asking
        returns:
            none
        outputs: outputs to screen
        note: only goes one level deep
    """
#   Modules:
    import os
    import sys
    import shutil
    import subprocess
    import Xvaliddirectory
    import Xvalidinteger
    import Xgetdirs
    import Psubfile
    import Xvalidletter
#   Validation
#   #   askfirst
    askfirst = Xvalidinteger.main(askfirst, 0, 1)
    if (askfirst == None):
        print '(Exiting.)'
        return None
#   inputs other than the top directories handled by Pcompare
#   #   validate first top directory path
    topdir = Xvaliddirectory.main(topdir)
    if (topdir == None):
        print '(Exiting.)'
        return None
#   #   get list of folders within directory, sorted alphabetically
    dirlist=[]
    dirlist = Xgetdirs.main(topdir, 1, 1)
    if (dirlist == None):
        print 'Error getting directories under', topdir
        return None
    if len(dirlist) == 0:
        print 'No directories found under', topdir
        return None
#   submit run for each directory
    onedir=""
    okaytorun=""
    subrun=""
    subfile=""
    curdir = os.getcwd()
    runme=""
    for onedir in dirlist:
        if (askfirst == 0):
            okaytorun = 'Y'
        else:
            okaytorun = raw_input('Run for ' + onedir + ' (Y/N)?')
        runme = Xvalidletter.main(okaytorun,['y'],1)
        if (runme == 'y'):
            subfile = ""
            subfile = Psubfile.main(onedir)
            if (subfile == None):
                print 'Could not find submission file in', onedir
            else:
                os.chdir(onedir)
                print "cwd: ",os.getcwd()
                print "subfile: ",subfile
                subfile = 'qsub ' + subfile
                subrun = subprocess.Popen(subfile, shell=True)
                subrun.wait()
        else:
            print 'Not run.'
            continue
    os.chdir(curdir)
    print 'Done.'  
#   prompts
def prompt():
    inplist=[]
    dirloop = raw_input('Directory over which to loop: ')
    inplist.append(dirloop)
    return inplist
