#!/usr/bin/env python
##
#@package   Uwrapper
#@ingroup   other
#@ingroup   python
#@brief     universal wrapper for python scripts with arguments
#@note      most of my scripts can be run using Uwrapper
#@author    Tam Mayeshiba
#@date      12/17/10 created 
#@date      3/29/11 added support for more parameters
###
def mydoc():
    """
    Purpose: universal wrapper
        inputs:  
            [$1] script name
                    script name WITHOUT .py (finding path tree should
                    be in your bash profile)
                    for a specific function in the script, use script.fcn
                example: Prunindir.prompt
            [$2...] arguments (according to function)
            example:
        outputs:
        notes:
            takes '/home/tam/bin/tscripts' as base directory right now
            assumes that all calls are Scriptname.main(arg,arg,arg)
            integer-appearing inputs are always converted to integers
            'None' is converted to None

    """
#   Modules:
import os
import sys
import imp
#
scriptdir='/home/tam/bin/tscripts' # this should be changed to be general
sysarglen=0
scriptname=""
scriptbase=""
funname=""
namesplit=[]
namesplitlen=0
sysarglen = len(sys.argv)
if (sysarglen < 2):
    print mydoc.__doc__
    print '(Exiting.)'
    sys.exit(1)
#   get script name
scriptname = sys.argv[1]
namesplit = scriptname.split('.')
namesplitlen = len(namesplit)
if len(namesplit)>0:
    scriptbase = namesplit[0]
if len(namesplit)>1:
    funname = namesplit[1]
if (funname == ""):
    funname = 'main' # default to main
#   load script
foundfile=""
loaded=""
try:
    foundfile = imp.find_module(scriptbase)
except (ImportError):
    print 'Module', scriptbase, 'could not be found.'
    print '(Exiting.)'
    sys.exit(1)
try:
    loaded = imp.load_source(scriptbase, foundfile[1], foundfile[0])
except (ImportError, TypeError):
    print 'Module', scriptbase, 'could not be loaded.'
    print '(Exiting.)'
    foundfile[0].close() # must close the opened file
    sys.exit(1)
foundfile[0].close() # must close the opened file
#   prompt for argument list if specified
rawarglist=[]
arglist=[]
argct=0
argitem=""
arglen=0
if (sysarglen > 2) and (sys.argv[2] == 'prompt'):
    if (hasattr(loaded,'prompt')):
        rawarglist = loaded.prompt()
    else:
        print scriptbase, 'has no prompt function.'
        print '(Exiting.)'
        sys.exit(1)
#   if prompt is not returned, get arguments from sys.argv (all strings)
if len(rawarglist) == 0:
    for argct in range(sysarglen):
        if (argct == 0):
            pass # call to Uwrapper
        elif (argct == 1):
            pass # script name
        else:
            rawarglist.append(sys.argv[argct])
# convert arguments (integers to integers, "None" to None)
arglen = len(rawarglist)
for argct in range(arglen):
    argitem = rawarglist[argct]
    #print argitem
    #print type(argitem)
    if (argitem == "None"):
        argitem = None
    else:
        try:
            argitem = int(argitem)
        except (TypeError, ValueError):
            pass
    arglist.append(argitem)
arglen = len(arglist)
#   debug
#for argitem in arglist:
#    print argitem
#sys.exit(1)#debug
#   check for function name
if (hasattr(loaded, funname) == False):
    print funname, 'in', scriptbase, 'does not exist.'
    print 'Trying main function.'
    funname = 'main'
if (hasattr(loaded, funname) == False):
    print funname, 'in', scriptbase, 'does not exist.'
    print 'Please check the tags available in your module.'
    print '(Exiting.)'
    sys.exit(1)
#   run script
returnval=""
flag=""
loadfun=""
loadfun = getattr(loaded, funname)
#print 'arglen: ', arglen #debug
#for argitem in arglist:
#    print 'argitem: ', argitem #debug
if (arglen == 0):
    try:
        returnval = loadfun()
    except (TypeError):
        flag = 'T'
elif (arglen == 1):
    try:
        returnval = loadfun(arglist[0])
    except (TypeError):
        flag = 'T'
elif (arglen == 2):
    try:
        returnval = loadfun(arglist[0], arglist[1])
    except (TypeError):
        flag = 'T'
elif (arglen == 3):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2])
    except (TypeError):
        flag = 'T'
elif (arglen == 4):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3])
    except (TypeError):
        flag = 'T'
elif (arglen == 5):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4])
    except (TypeError):
        flag = 'T'
elif (arglen == 6):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5])
    except (TypeError):
        flag = 'T'
elif (arglen == 7):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6])
    except (TypeError):
        flag = 'T'
elif (arglen == 8):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6], arglist[7])
    except (TypeError):
        flag = 'T'
elif (arglen == 9):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6], arglist[7], arglist[8])
    except (TypeError):
        flag = 'T'
elif (arglen == 10):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6], arglist[7], arglist[8],
                                arglist[9])
    except (TypeError):
        flag = 'T'
elif (arglen == 11):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6], arglist[7], arglist[8],
                                arglist[9], arglist[10])
    except (TypeError):
        flag = 'T'
elif (arglen == 12):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6], arglist[7], arglist[8],
                                arglist[9], arglist[10], arglist[11])
    except (TypeError):
        flag = 'T'
elif (arglen == 13):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6], arglist[7], arglist[8],
                                arglist[9], arglist[10], arglist[11],
                                arglist[12])
    except (TypeError):
        flag = 'T'
elif (arglen == 14):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6], arglist[7], arglist[8],
                                arglist[9], arglist[10], arglist[11],
                                arglist[12], arglist[13])
    except (TypeError):
        flag = 'T'
elif (arglen == 15):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6], arglist[7], arglist[8],
                                arglist[9], arglist[10], arglist[11],
                                arglist[12], arglist[13], arglist[14])
    except (TypeError):
        flag = 'T'
elif (arglen == 16):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6], arglist[7], arglist[8],
                                arglist[9], arglist[10], arglist[11],
                                arglist[12], arglist[13], arglist[14],
                                arglist[15])
    except (TypeError):
        flag = 'T'
elif (arglen == 17):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6], arglist[7], arglist[8],
                                arglist[9], arglist[10], arglist[11],
                                arglist[12], arglist[13], arglist[14],
                                arglist[15], arglist[16])
    except (TypeError):
        flag = 'T'
elif (arglen == 18):
    try:
        returnval = loadfun(arglist[0], arglist[1], arglist[2],
                                arglist[3], arglist[4], arglist[5],
                                arglist[6], arglist[7], arglist[8],
                                arglist[9], arglist[10], arglist[11],
                                arglist[12], arglist[13], arglist[14],
                                arglist[15], arglist[16], arglist[17])
    except (TypeError):
        flag = 'T'
else:
    print 'Only up to 18 arguments supported.'
    print '(Exiting.)'
    sys.exit(1)
if (flag == 'T'):
    print ''
    print 'Wrong number or wrong type of arguments supplied,'
    print 'or there has been an error calling the program.'
    print ''
    print scriptbase + '.' + funname
    print loadfun.__doc__
    print '(Exiting.)'
    sys.exit(1)
print 'Return:', returnval
