#!/usr/bin/env python
##
#@package   Xvaliddirectory
#@ingroup   other
#@ingroup   python
#@brief     checks to see that a directory exists, is not a file,
#           and is not blank; returns the directory
#@author    Tam Mayeshiba
#@date      11/30/2010 
#@date      5/24/11 major change: prepend home directory if can't find directory
###
def main(checkdir, nonorm=0):
    """
    Purpose:
        inputs:  
            [$1] directory to check, as a string
                example: '/home/tam/NaClfiles'
            [$2] 0 - norm the path (default)
                    turns // into / (unless it's the leading term)
                    removes / from end of path
                 1 - don't normalize the path
        outputs:
            directory
            None - directory failed some check.
            
    """
#   Modules:
    import os
    import sys
    import Xvalidinteger
#   
#   #  validate normpath
    nonorm = Xvalidinteger.main(nonorm, 0, 1)
    if (nonorm == None):
        return None
#   #  validate directory
    try:
        checkdir = str(checkdir)
    except (ValueError, TypeError):
        print checkdir, 'should be a directory like "//home/tam/mydir"'
        print '(Exiting.)'
        return None
    if (checkdir == "") or (checkdir == '/') or (checkdir == '~'):
        print 'Directory is not well-defined. High risk of overwriting.' 
        print 'If this is intentional, please try again'
        print 'using //home/<username>. Or, please create a new directory'
        print 'and move files there.'
        print '(Exiting.)'
        return None
#   #   expand tildes
    if checkdir[0:2] == '~/':
        checkdir = os.path.expanduser(checkdir)
#   #   expand dots (this can be worked with 'os.path.realpath' in python 2.2)
    current=""
    if checkdir[0:2] == './':
        current = os.getcwd()
        checkdir = current + checkdir[1:]
    if checkdir[0:3] == '../':
        current = os.getcwd()
        checkdir = os.path.dirname(current) + checkdir[2:]
#   #   tack on HOME or current directory
    if not (checkdir[0:1] == '/'):
        current = os.getcwd()
        checkdir = current + '/' + checkdir
    if (os.path.exists(checkdir)) == False:
        print checkdir, 'is not a valid path. Adding HOME.'
        checkdir = os.environ["HOME"] + '/' + checkdir
        if (os.path.exists(checkdir)) == False:
            print checkdir, 'is not a valid path.'
            print '(Exiting.)'
            return None
    if (os.path.isfile(checkdir)):
        print checkdir, 'is a file, not a directory.'
        print '(Exiting.)'
        return None
#   #   passed all checks
    if (nonorm == 1):
        return checkdir
    else:
        return os.path.normpath(checkdir) # normpath may break in Windows
