#!/usr/bin/env python
##
#@package   taccneb 
#@ingroup   vasp
#@ingroup   python
#@brief     restart NEB according to Ming-Jie's criteria for TACC/Lonestar
#@author    Tam Mayeshiba
#@date      3/2/12
###
def main(starthere="",output='.*[.][o].*',accstr='\"reached required\"',
            cleanstr='\"Cleaning up\"'):
    """
    Purpose: restart NEB for TACC/lonestar
    inputs:
        directory which houses image folders: example my_neb
            OR directory housing a bunch of directories
        matching string for output file (default: '.*[.][o].*')
        matching string for finished neb (default: 'reached required')
        matching string for finished submission (default: 'Cleaning up')
    output:
    """
    import os
    import sys
    import time
    import re
    import subprocess
    import shutil
    import taccneb
    #import pdb
# defaults
    toplist=[]
    litem=""
    fpath=""
    match=""
    topdirs=[]
    if os.path.isdir(starthere):
        starthere = os.path.abspath(starthere)
        toplist=os.listdir(starthere)
        for litem in toplist:
            fpath = os.path.join(starthere, litem)
            if os.path.isdir(fpath):
                match = re.match('[0-9]?[0-9]',litem)
                if match == None: #make sure these aren't just image folders
                    topdirs.append(fpath)
        if len(topdirs) == 0:
            taccneb.checkone(starthere, output, accstr, cleanstr)
        else:
            litem=""
            for litem in topdirs:
                print '----------------',litem, '----------------'
                taccneb.checkone(litem, output, accstr, cleanstr)
        
def checkone(mydir="",output='.*[.][o].*',accstr='\"reached required\"',
            cleanstr='\"Cleaning up\"'):
    """
    Purpose: restart NEB for TACC/lonestar
    inputs:
        top directory (which houses image folders): example my_neb
        matching string for output file (default: '.*[.][o].*')
        matching string for finished neb (default: 'reached required')
        matching string for finished submission (default: 'Cleaning up')
    output:
    """
    import os
    import sys
    import time
    import re
    import subprocess
    import shutil
#import pdb
# defaults
#pdb.set_trace()
    mydir = os.path.abspath(mydir)
    if not os.path.exists(mydir):
        print 'Directory', mydir, 'could not be found. Exiting.'
        return None

    fname=""
    listdir=[]
    files=[]
    grepproc=""
    grepstr=""
    fullpath=""
    acc=""
    clean=""
    dirlist=[]
    match=""

    listdir = os.listdir(mydir)
    for fname in listdir:
        fullpath = os.path.join(mydir, fname)
        if os.path.isfile(fullpath):
            match = re.match(output, fname)
            if not match == None:
                files.append(fullpath)
        elif os.path.isdir(fullpath):
            match = re.match('[0-9]?[0-9]',fname)
            if not match == None:
                dirlist.append(fullpath)
    files.sort()
    dirlist.sort()

    if len(files) == 0:
        print 'No output files matching', output, 'found. Exiting.'
        return None

    lastfile = os.path.join(mydir,files[len(files)-1])
    print 'Using output file: ',lastfile
    grepstr = 'grep ' + accstr + ' ' + lastfile
    grepproc = subprocess.Popen(grepstr, shell=True, stdout = subprocess.PIPE)
    grepproc.wait()
    acc = grepproc.communicate()[0]
    
    grepstr = 'grep ' + cleanstr + ' ' + lastfile
    grepproc = subprocess.Popen(grepstr, shell=True, stdout = subprocess.PIPE)
    grepproc.wait()
    clean = grepproc.communicate()[0]

    if clean == None or clean == '':
        print 'Job does not appear to be complete. Exiting.'
        return None

    imdir=""
    dirct=0
    oldpath=""
    newpath=""
    i=1
    contcar=""
    if acc == None or acc == '':
        print 'No accuracy tags in output file.'
        print 'Archiving old outputs.'
        print str(len(dirlist)-2), 'image(s) detected.'    
        while dirct < len(dirlist)-2:
            dirct = dirct + 1
            imdir = dirlist[dirct]
            for item in ['OUTCAR','POSCAR','CONTCAR','OSZICAR']:
                oldpath = os.path.join(imdir,item)
                if os.path.isfile(oldpath):
                    newpath = os.path.join(imdir,item + '1')
                    i=1
                    while i<50 and os.path.isfile(newpath):
                        i = i+1
                        newpath = os.path.join(imdir, item + str(i))
                    shutil.copy(oldpath, newpath)
                    print '...', item, '...'
            contcar = os.path.join(imdir,'CONTCAR')
            if int(os.path.getsize(contcar)) == 0:
                pass
            else:
                shutil.copy(contcar,os.path.join(imdir,'POSCAR'))
                print 'Copied', contcar, 'to POSCAR.'
    else:
        print 'Job appears to be complete.'
    return None

import taccneb
import os
import sys
mdir = os.getcwd()
moutput = '.*[.][o].*'
maccstr = '\"reached required\"'
mcleanstr = '\"Cleaning up\"'

if len(sys.argv) > 1:
    mdir = sys.argv[1]
if len(sys.argv) > 2:
    moutput = sys.argv[2]
if len(sys.argv) > 3:
    maccstr = sys.argv[3]
if len(sys.argv) > 4:
    mcleanstr = sys.argv[4]


taccneb.main(mdir, moutput, maccstr, mcleanstr)
