#!/usr/bin/env python
##
#@package   translate_poscar_tom
#@ingroup   vasp
#@ingroup   python
#@brief     Translates the direct coordinates of the poscar by a given nearest-neighbor vector (used with synthep2_tom)
#@author    Tom Angsten
#@date      7/2/11 created
###

def main(poscar="", x=0.0, y=0.1666666667, z=0.1666666667, mapkey="~/bin/map.file"):
    """
    Purpose: Translate direct coordinates of poscar by vector <x,y,z>
        1. path of POSCAR file
        2. x component of translation vector (direct coordinates)
        3. y component
        4. z component
        5. atom position mapping file
    """
#comment
    import os
    import sys
    import shutil
    import add_coord_tom

    x = float(x)
    y = float(y)
    z = float(z)
    
    poscar = os.path.abspath(poscar)
    mapkey = os.path.expanduser(mapkey)
    
    readf = open(poscar,'rb')
    writef = open(poscar + '.work','wb')
    
    slot = 0
    for line in readf:
        if(slot == 0):
            writef.write(line)
            if(line.find('D') != -1):
                slot = 1
        elif(slot == 1):
            if(len(line) < 5):
                slot = 0
                writef.write(line)
                continue
            newline = add_coord_tom.main(line,x,y,z)
            writef.write(newline + '\n')
    readf.close()
    writef.close()


    transf = open(poscar + '.work','rb')
    mapfile = open(mapkey,'rb')
    wfin = open(poscar + '.final','wb')
    
    translisting = transf.readlines()
    maplisting = mapfile.readlines()

    pos = 1
    while(pos < 9):
        wfin.write(translisting[pos-1])
        pos += 1

    
    while(pos < 116):
        mapcount = 0
        while(mapcount < len(maplisting)):
            index0 = maplisting[mapcount].split(":")[0]
            index1 = maplisting[mapcount].split(":")[1]
            
            index1 = index1[:len(index1)-1]
            if(index1 == str(pos)):
                wfin.write(translisting[int(index0)-1])
            mapcount += 1
        pos += 1

    while(pos < len(translisting)+1):
        wfin.write(translisting[pos-1])
        pos += 1

    os.remove(poscar + '.work')
    shutil.move(poscar + '.final', poscar)
